# package BuildForge::API
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other intellectual property
# rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API
package BuildForge::API;

use BuildForge::Connection::Encoded;
use BuildForge::NameMap;
use BuildForge::API::Project;
use BuildForge::API::Build;
use BuildForge::API::Step;
use BuildForge::API::Class;
use BuildForge::API::Environ;
use BuildForge::API::Access;
use BuildForge::API::Server;
use BuildForge::API::User;
use BuildForge::API::Schedule;
use BuildForge::API::TagVar;
use BuildForge::API::Register;
use BuildForge::API::Adaptor;
use BuildForge::API::Collector;
use BuildForge::API::Selector;
use BuildForge::API::Engine;
use BuildForge::API::Auth;

require Exporter;

use strict;

our $VERSION = '7.0.2.0.0.0005';

sub DESTROY {
	my($self) = @_;
	$self->{'_map'}->TearDown() if(defined($self->{'_map'}));
	$self->{'_map'} = undef;
	$self->{'_connection'}->TearDown() if(defined($self->{'_connection'}));
	$self->{'_connection'} = undef;
}

# Instantiation
sub new {
	my ($class,$Params) = @_;
	
	$Params->{'port'} = 80 unless ($Params->{'port'});
	$Params->{'base'} = "/api/api.php" unless ($Params->{'base'});
	
	my $self={
		'user'=>$Params->{'login'},
		'pass'=>$Params->{'password'},
		'domain'=>$Params->{'domain'},
		'host'=>$Params->{'host'},
		'port'=>$Params->{'port'},
		'base'=>$Params->{'base'},
		'_error'=>0,
		'_info' =>"",
		'_connection' => undef,
		'_map' => undef,
	};
	
	bless $self, $class || ref($class);
	
	$self->{'_error'}=700 unless($Params->{'login'});
	$self->{'_error'}=701 unless($Params->{'password'});
	$self->{'_error'}=702 unless($Params->{'host'});
	$self->{'_error'}=703 unless($Params->{'port'});
	$self->{'_error'}=704 unless($Params->{'base'});
	
	# Authenticate us with our provided credentials.
 	unless($self->{'_error'}) {
		$self->{'_error'}=$self->Authenticate();
		$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	}
	# Load the name translation map for each type of request & response.
	unless($self->{'_error'}) {
		$self->{'_error'}=$self->LoadNameMap();
		$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	}
	
	return $self;
}

sub LogOut {
	my($self) = @_;
	$self->{'_connection'}->LogOut() if(ref($self->{'_connection'}) eq "BuildForge::Connection::Encoded");
	$self->{'_map'}->TearDown() if(defined($self->{'_map'}));
	$self->{'_map'} = undef;
	$self->{'_connection'}->TearDown() if(defined($self->{'_connection'}));
	$self->{'_connection'} = undef;
}

sub GetLastError {
	my ($self) = @_;
	return $self->{'_error'};
}

sub GetLastMessage {
	my ($self) = @_;
	return $self->{'_info'};
}

sub LoadNameMap {
	my ($self) = @_;
	$self->{'_map'} = new BuildForge::NameMap();
	return $self->{'_map'}->GetLastError();
}

sub Authenticate {
	my ($self) = @_;

	# Make a connection to the Server
	$self->{'_connection'} = new BuildForge::Connection::Encoded($self);
	return $self->{'_connection'}->GetLastError();
}

# High level request to get the list of Projects, basically a table row query.
sub ProjectList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_projects",
					'Fields' => "*",
					'Clause' => "",
				},
				TableTrans=>"bf_projects"
			});
}

sub AuthList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_serverauth",
					'Fields' => "bf_authid",
					'Clause' => "",
				},
				TableTrans=>"bf_serverauth"
			});
}

sub SelectorList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_selector",
					'Fields' => "distinct bf_selector_id",
					'Clause' => "",
				},
				TableTrans=>"bf_selector"
			});
}

sub CollectorList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_collector",
					'Fields' => "distinct bf_coll_id",
					'Clause' => "",
				},
				TableTrans=>"bf_collector"
			});
}

# High level request to get the list of schedules, basicly a table row query.
sub ScheduleList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_cron",
					'Fields' => "*",
					'Clause' => "",
				},
				TableTrans=>"bf_cron"
			});
}

# High level request to get the list of classes, basically a table row query.
sub ClassList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_classes",
					'Fields' => "*",
					'Clause' => "",
				},
				TableTrans=>"bf_classes"
			});
}

# High level request to get the list of servers, basically a table row query.
sub ServerList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_servers",
					'Fields' => "*",
					'Clause' => ""
				},
				TableTrans=>"bf_servers",
			});
}

# High level request to get the list of env groups, basically a table row query.
sub EnvList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_env",
					'Fields' => "bf_bid,bf_eid,bf_level,bf_param",
					'Clause' => "WHERE bf_eid=0"
				},
				TableTrans=>"bf_env",
			});
}

# High level request to get the list of access groups, basically a table row query.
sub GroupList { AccessList(@_); }
sub AccessList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_levels",
					'Fields' => "distinct bf_name",
					'Clause' => ""
				},
				TableTrans=>"bf_levels",
			});
}

# High level request to get the list of users, basically a table row query.
sub UserList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_users",
					'Fields' => "*",
					'Clause' => ""
				},
				TableTrans=>"bf_users",
			});
}
	
# High level request to get the list of adaptors, basically a table row query.
sub AdaptorList {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_interfaces",
					'Fields' => "DISTINCT bf_link,bf_mode",
					'Clause' => "ORDER BY bf_mode,bf_link"
				},
				TableTrans=>"bf_interfaces",
			});
}

# High level request to get the list of adaptors, basically a table row query.
sub AdaptorTemplates {
	my($self) = @_;
	return $self->_GenericListing({
				RequestParams=>{
					'Command' => "APIQueryRecords",
					'Table' => "bf_interfacetemplates",
					'Fields' => "DISTINCT bf_name,bf_mode",
					'Clause' => "ORDER BY bf_mode,bf_name"
				},
				TableTrans=>"bf_interfacetemplates",
			});
}

# _GenericListing
#
# Task	: Get a generic list of information from the server...
# Params:
# 	RequestParams=>{}
# 	TableTrans=>$
# Return:
# 	Array ref of Objects
#
sub _GenericListing {
	my($self,$Params) = @_;
	# Create a request
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	# Add some specifics
	$self->{'_connection'}->AddRequestContent($Request,$Params->{'RequestParams'});
	# Process it
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	# Check for errors/info
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	# Parse results into some nicely formatted manner
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_map'}->GetTableTrans($Params->{'TableTrans'})});
	# Now gen the objects for each of these...
	return $QueryResults;
}

# Always Return Nicely
1;

__END__

=head1 I<NAME>
	
BuildForge::API

=head1 I<CONTENTS>
	
lib::BuildForge::API
lib::BuildForge::NameMap
lib::BuildForge::Connection::Encoded
lib::BuildForge::API::Access
lib::BuildForge::API::Adaptor
lib::BuildForge::API::Build
lib::BuildForge::API::Class
lib::BuildForge::API::Environ
lib::BuildForge::API::Group
lib::BuildForge::API::Interface
lib::BuildForge::API::Project
lib::BuildForge::API::Register
lib::BuildForge::API::Schedule
lib::BuildForge::API::Server
lib::BuildForge::API::Step
lib::BuildForge::API::TagVar
BuildForge::API::User

=head1 I<SYNOPSIS>

	use BuildForge::API
	my $API = new BuildForge::API({
		login=>"root",
		password=>"root",
		host=>"bfhost.company.com",
		port=>80,
		base=>"/api/api.php"});

=head1 I<DESCRIPTION>

    Creates a connection to the BuildForge API to allow interaction with the
console. It is recommended that only one API object exist in your code.

=head1 I<METHODS>

=head2 new({login,password,host,[port]})

Creates a new connection to the BuildForge API.

=over 8

=item I<login>

The username to loginto the BuildForge Console (required).

=item I<password>

The corresponding password for the user (required).

=item I<host>

The hostname of machine running the BuildForge UI (required).

=item I<port>

The http port the Buildforge UI is running on.

=item I<base>

The location for the api.php file via HTTP (for example, if you have
the console set up in http://www.yourdomain.com/BuildForge/, this should 
be set to "/BuildForge/api/api.php").  By default, this is set to 
the standard install location for BuildForge, "/api/api.php".

=back

=head2 LogOut()

Logs the current API session out of the BuildForge console. 

=head2 ProjectList()

Returns an array ref of hash refs containing basic Project information 
(Project ID, Project Title, and Access Level).

=head2 AuthList()

Returns an array ref of hash refs containing names that can be used
for server authentication authids.
(Auth Id).

=head2 CollectorList()

Returns an array ref of hash refs containing names that can be used
to gather server configurations.
(Collector Id).

=head2 SelectorList()

Returns an array ref of hash refs containing names that can be used
by Projects and Steps to select a server configuration.
(Selector Id).

=head2 ServerList()

Returns an array ref of hash refs containing basic Server information
(Server Name, Server Host, and Access Level).

=head2 ScheduleList()

Returns an array ref of hash refs containing Schedule Information.

=head2 ClassList()

Returns an array ref of hash refs containing Class Information.

=head2 EnvList()

Returns an array ref of hash refs containing Environment Group Information.

=head2 AccessList()

Returns	an array ref of hash refs containing Access Group Information.

=head2 GroupList() (deprecated)

Returns	an array ref of hash refs containing Access Group Information.  This
method is deprecated; use AccessList() instead.

=head2 UserList()

Returns	an array ref of hash refs containing User Information.

=head2 AdaptorList()

Returns	an array ref of hash refs containing Adaptor Information.

=head2 GetLastError()

Returns	a scalar value identifying the last error code encountered by the API object.

=head2 LogOut()

Logs out the current session.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
