# package BuildForge::API::Access
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Access
package BuildForge::API::Access;

use BuildForge::API::Interface;
use BuildForge::NameMap;

use strict;

use vars qw (@ISA);

# Some Class Statics
{
	my $default={};
	sub get_default {$default}
	sub clear_default {$default={}}	
}

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	$self->{'access_group_id'} = 0;
	$self->{'_public'}->{'control_id'} = 0;
	# For performance reasons, we're only going to set what we get passed in. 
	# A sync operation will update us from the db.
	map {$self->{$_} = $Params->{$_}} keys(%{$Params});
	if($self->{'access_group_id'} < 1) {
		$self->SetDefaultLevel();
	}
	return $self;
}

sub SetDefaultLevel {
	my ($self) = @_;
	my $Class = ref($self);
	my $Default = $Class->get_default();
	my $SetId = $self->{'access_group_id'};
	if($SetId < 1) {
		$SetId = "_default";
	}
	if(exists($Default->{$SetId})) {
		map { $self->{$_} = $Default->{$SetId}->{$_} } keys(%{$Default->{$SetId}});
		return;
	}
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $WhereClause = "where bf_level = $self->{'access_group_id'} " if($self->{'access_group_id'});
	$self->{'_connection'}->AddRequestContent($Request,{
		'Command'	=> "APIQueryRecords",
		'Table'	=> "bf_access",
		'Fields'	=> "bf_level",
		'Clause'	=> "${WhereClause}order by bf_level desc",
		'Limit'	=> 1,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_access")});
	# Now update our object for each key...
	map {$self->{$_} = ${$QueryResults}[0]->{$_}} keys(%{${$QueryResults}[0]});
	$self->{'control_id'} = $self->{'access_group_id'} unless($self->{'control_id'});
	$Default->{$SetId}=${$QueryResults}[0];
}

sub GetGroupID {
	my($self,$GroupInfo) = @_;

	$self->{'_error'} = 1301 unless($GroupInfo eq "");

	my $Request=$self->{'_connection'}->NewDefaultRequest();
	$self->{'_connection'}->AddRequestContent($Request,{
                  'Command' => "APIQueryRecords",
                  'Table' => "bf_levels",
                  'Fields' => "bf_level",
                  'Clause' => "where bf_name='$GroupInfo'",
				  'limit' => 1
          });

	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_levels")});
	my $id = ${$QueryResults}[0]->{'control_id'};

	return($id);

}	

sub CreateGroup {
	my($self,$GroupInfo) = @_;
	$self->{'_error'} = 1301 unless($GroupInfo->{'name'});
	return undef if($self->{'_error'});
	$GroupInfo->{'default'} = 0 unless($GroupInfo->{'default'});
	$GroupInfo->{'dn'} = "" unless($GroupInfo->{'dn'});
	$GroupInfo->{'access_group_id'} = $self->{'access_group_id'} unless($GroupInfo->{'access_group_id'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddAccess");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $GroupInfo->{$Var};
	}
	$RequestContent->{'Command'} = "AddAccess";
	my $QueryResults = $self->ProcessQuery($RequestContent,"bf_access");
	return $self->{'_error'};
}

sub DeleteGroup {
	my($self,$GroupInfo) = @_;
	$self->{'_error'} = 1302 unless($GroupInfo->{'name'} || $GroupInfo->{'control_id'});
	return undef if($self->{'_error'});
	my $Command = "DeleteAccessById";
	$Command="DeleteAccessByName" if($GroupInfo->{'name'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams($Command);
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $GroupInfo->{$Var};
	}
	$RequestContent->{'Command'} = $Command;
	my $QueryResults = $self->ProcessQuery($RequestContent,"bf_access");
	return $self->{'_error'};
}

sub AddUser {
	my($self,$User) = @_;
	$self->{'_error'} = 1303 unless($self->{'control_id'});
	$self->{'_error'} = 1304 unless($User->{'user_id'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddAccessMember");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "user_id") {
			$RequestContent->{$Param} = $User->{$Var};
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "AddAccessMember";
	$self->ProcessQuery($RequestContent,"bf_levels");
	return $self->{'_error'};
}

sub DeleteUser {
	my($self,$User) = @_;
	$self->{'_error'} = 1303 unless($self->{'control_id'});
	$self->{'_error'} = 1304 unless($User->{'user_id'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteAccessMember");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "user_id") {
			$RequestContent->{$Param} = $User->{$Var};
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "DeleteAccessMember";
	$self->ProcessQuery($RequestContent,"bf_levels");
	return $self->{'_error'};
}

sub AddGroup {
	my($self,$Group) = @_;
	$self->{'_error'} = 1303 unless($self->{'control_id'});
	$self->{'_error'} = 1305 unless($Group->{'control_id'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddAccessGroup");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "group_id") {
			$RequestContent->{$Param} = $Group->{'control_id'};
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "AddAccessGroup";
	$self->ProcessQuery($RequestContent,"bf_levels");
	return $self->{'_error'};
}

sub DeleteGroup {
	my($self,$Group) = @_;
	$self->{'_error'} = 1303 unless($self->{'control_id'});
	$self->{'_error'} = 1305 unless($Group->{'control_id'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteAccessGroup");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "group_id") {
			$RequestContent->{$Param} = $Group->{'control_id'};
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "DeleteAccessGroup";
	$self->ProcessQuery($RequestContent,"bf_levels");
	return $self->{'_error'};
}

sub PermissionList {
	my($self) = @_;
	return $self->ProcessForArray({
					'Command' => "APIQueryRecords",
					'Table' => "bf_perms",
					'Fields' => "bf_tag,bf_description",
					'Clause' => "WHERE bf_description != '' ORDER by bf_tag"
				},
				"bf_ptable");
}

sub AddPermission {
	my($self,$Permission) = @_;
	$self->{'_error'} = 1303 unless($self->{'control_id'});
	$self->{'_error'} = 1306 unless($Permission->{'name'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddPermMember");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "perm") {
			$RequestContent->{$Param} = $Permission->{'name'};
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "AddPermMember";
	$self->ProcessQuery($RequestContent,"bf_ptable");
	return $self->{'_error'};
}

sub DeletePermission {
	my($self,$Permission) = @_;
	$self->{'_error'} = 1303 unless($self->{'control_id'});
	$self->{'_error'} = 1306 unless($Permission->{'name'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeletePermMember");
	while(my($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "perm") {
			$RequestContent->{$Param} = $Permission->{'name'};
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "DeletePermMember";
	$self->ProcessQuery($RequestContent,"bf_ptable");
	return $self->{'_error'};
}

sub MakeDefault {
	my($self) = @_;
	$self->{'_error'} = 1303 unless($self->{'control_id'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("SetPermDefault");
	while(my($Param,$Var) = each %{$RequestContent}) {
			$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "SetPermDefault";
	$self->ProcessQuery($RequestContent,"bf_levels");
	return $self->{'_error'};
}

sub ClearDefault {
	my($self) = @_;
	$self->{'_error'} = 1303 unless($self->{'control_id'});
	return undef if($self->{'_error'});
	my $RequestContent = $self->{'_namemap'}->FunctionParams("UnsetPermDefault");
	while(my($Param,$Var) = each %{$RequestContent}) {
			$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "UnsetPermDefault";
	$self->ProcessQuery($RequestContent,"bf_levels");
	return $self->{'_error'};
}

# Always Return Nicely
1;


__END__

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been 
instantiated. It is used to create and modify BuildForge Access groups for use
in email notification and access rights.

=head1 I<METHODS>

=head2 new({AccessParamsHashRef})

Create an object to create or modify access levels/groups.

=over 8

=item I<access_group_id>

The id for the access group.

=item I<name>

The name for the access group.

=back

=head2 CreateGroup($NewGroupParamsHashref)

Creates a new access group.

=over 8

=item I<default>

Allows this access level to be a default for new users.

=item I<dn>

The dn to relate this access group to relate this access to an ldap group.

=item I<access_group_id>

The access level that is required to modify this new access level.

=back

=head2 DeleteGroup($DeleteGroupParamsHashref)

Allows you to delete an access group

=over 8

=item I<name>

The name of the access group to delete

=item I<control_id>

The id of the access group to delete

=back

=head2 AddUser($AddUserParamsHashref)

=over 8

=item I<control_id>

The id of the access group of which to add the user.

=item I<user_id>

The id of the user to add to the access group.

=back

=head2 DeleteUser($UserParamsHashref)

=over 8

=item I<control_id>

The id of the access group to remove from the user.

=item I<user_id>

The id of the user from which to remove the access group.

=back

=head2 AddGroup($GroupParamsHashref)

=over 8

=item I<control_id>

The id of the access group in which to add to this group

=back

=head2 DeleteGroup($GroupParamsHashref)

=over 8

=item I<control_id>

The id of the access group in which to remove from this group.

=back

=head2 PermissionList()

Returns a list of hashrefs indicating all available permissions and a short 
description.

=head2 AddPermission($PermissionParamsHashref)

Add a permission to this access group

=over 8

=item I<name>

The name of the permission to add to this access group.

=back

=head2 DeletePermission($PermissionParamsHashref)

=over 8

=item I<name>

The name of the permission to remove from this access group.

=back

=head2 MakeDefault

Sets this access group as a default for newly created users.

=head2 ClearDefault

Removes this access group from being a default for new users.

=head2 GetLastError()

Returns	the last error code encountered by the object.

=head2 GetLastMessage()

Returns an last message string encountered by the object.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
