# package BuildForge::API::Auth
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other intellectual property
# rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Auth
package BuildForge::API::Auth;

use BuildForge::Connection::Encoded;
use BuildForge::API::Interface;
use BuildForge::NameMap;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	$self->{'authid'} = "";
	$self->{'username'} = "";
	$self->{'password'} = "";
	$self->{'access_group_id'} = new BuildForge::API::Access();
	
	while(my($variable,$value) = each %{$Params}) {
		$self->{$variable} = $value;
	}
	return $self;
}

sub Sync {
	my($self) = @_;
	if(!$self->{'authid'}) {
		$self->{'_error'}=2201;
	}
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("SyncAuthBlob");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "SyncAuthBlob";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_serverauth");
	return $self->GetLastError();
	return $self;
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 2201 unless($self->{'authid'});
	
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteAuthBlob");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteAuthBlob";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_serverauth");
	return $self->GetLastError();
}

sub Create {
	my ($self) = @_;
	$self->{'_error'} = 2201 unless($self->{'authid'});
	
	return undef if($self->{'_error'});
	
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("CreateAuthBlob");
	
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "CreateAuthBlob";
	
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_serverauth");
	return $self->GetLastError();
}

sub Update {
	my ($self) = @_;
	$self->{'_error'} = 2201 unless($self->{'authid'});
	
	return undef if($self->{'_error'});
	
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("UpdateAuthBlob");
	
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "UpdateAuthBlob";
	
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_serverauth");
	return $self->GetLastError();
}
# Always Return Nicely
1;


__END__

=head1 I<NAME>

BuildForge::API::Auth

=head1 I<SYNOPSIS>

	my $Auth = new BuildForge::API::Auth();
	$Auth->{'authid'} = "My Auth Block";
	$Auth->{'username'} = "myuser";
	$Auth->{'password'} = "myuserspass";
	$Auth->Create();

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been
instantiated. It is used to create new authentication blobs for use with
BuildForge servers.

=head1 I<METHODS>

=head2 new({AuthParamsHashRef})

Create an object to access existing auth information or to add a new server
authetication blob to the BuildForge console. 

=over 8

=item I<authid>

The name for the server authentication

=item I<username>

ID of the user to use with this authentication

=item I<password>

The corresponding password to use with this authentication

=back

=head2 Create()

Create a new server authentication blob. Requires 'authid' to be set.

=head2 Update()

Updates a server authentication blob. Requires 'authid' to be set.

=head2 Delete()

Deletes the server authentication blob. Requires the 'authid' to be set.

=head2 Sync()

Syncs the server authentication object with information from the console
database. Requires the 'authid' to be set.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
