# package BuildForge::API::Build
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Build
package BuildForge::API::Build;

use BuildForge::Connection::Encoded;
use BuildForge::NameMap;
use BuildForge::API::Interface;
use BuildForge::API::Server;
use BuildForge::API::Access;
use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);

	$self->{'server'} = new BuildForge::API::Server();
	$self->{'access_group_id'} = new BuildForge::API::Access();

	while(my($variable,$value) = each %{$Params}) {
		$self->{$variable} = $value;
	}
	return $self;
}

sub Details {
	my($self) = @_;
	return $self->Sync();
}

sub Sync {
	my ($self) = @_;

	if(!$self->{'build_id'} ) {
		$self->{'_error'}=751;
		return $self;
	}

	my $Clause = "WHERE bf_bid=$self->{'build_id'}";

	my $QueryResults = $self->ProcessForHash({
			Command => "APIQueryRecords",
			Table => "bf_builds",
			Fields => "*",
			Clause => $Clause,
		},"bf_builds");
			
	if(!defined($QueryResults->{'_record'})) {
		$self->{'_error'} = 752;
		$self->{'_info'} = "Did not find an matching build for build id '$self->{'build_id'}'.";
		return $self->{'_error'};
	}	
		
	return $self;
}

sub Waiting {
	my ($self) = @_;

	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $Clause = "WHERE bf_state='W' AND bf_process='B'";
	$self->{'_connection'}->AddRequestContent($Request,{
		Command => "APIQueryRecords",
		Table => "bf_builds",
		Fields => "*",
		Clause => $Clause,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_builds")});
	return $QueryResults;
}

sub Running {
	my ($self) = @_;

	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $Clause = "WHERE bf_state='R' AND bf_process='B'";
	$self->{'_connection'}->AddRequestContent($Request,{
		Command => "APIQueryRecords",
		Table => "bf_builds",
		Fields => "*",
		Clause => $Clause,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_builds")});
	return $QueryResults;
}

sub Complete {
	my ($self) = @_;

	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $Clause = "WHERE bf_state='C' AND bf_process='B'";
	$self->{'_connection'}->AddRequestContent($Request,{
		Command => "APIQueryRecords",
		Table => "bf_builds",
		Fields => "*",
		Clause => $Clause,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_builds")});
	return $QueryResults;
}

sub Locked {
	my ($self) = @_;

	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $Clause = "WHERE bf_state='L' AND bf_process='B'";
	$self->{'_connection'}->AddRequestContent($Request,{
		Command => "APIQueryRecords",
		Table => "bf_builds",
		Fields => "*",
		Clause => $Clause,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_builds")});
	return $QueryResults;
}

sub Purge {
	my ($self) = @_;

	if(!$self->{'build_id'} ) {
		$self->{'_error'}=753;
		return $self->{'_error'};
	}

	$self->{'_error'} = 0;
	$self->Details();
	if ($self->{'_error'}) {
		return $self;
	}

	my $Request=$self->{'_connection'}->NewDefaultRequest();

	my $RequestContent = $self->{'_namemap'}->FunctionParams("PurgeBuild");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param}=$self->{$Var};
	}
	$RequestContent->{'Command'}="PurgeBuild";
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
}

sub Cancel {
	my ($self) = @_;

	if(!$self->{'build_id'} ) {
		$self->{'_error'}=756;
		return $self->{'_error'};
	}

	$self->{'_error'} = 0;
	$self->Details();
	if ($self->{'_error'}) {
		return $self;
	}

	my $Request=$self->{'_connection'}->NewDefaultRequest();

	my $RequestContent = $self->{'_namemap'}->FunctionParams("CancelRunning");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param}=$self->{$Var};
	}
	$RequestContent->{'Command'}="CancelRunning";
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
}

sub Lock {
	my ($self) = @_;

	if(!$self->{'build_id'} ) {
		$self->{'_error'}=754;
		return $self->{'_error'};
	}

	$self->{'_error'} = 0;
	$self->Details();
	if ($self->{'_error'}) {
		return $self;
	}

	my $Request=$self->{'_connection'}->NewDefaultRequest();

	my $RequestContent = $self->{'_namemap'}->FunctionParams("LockBuild");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param}=$self->{$Var};
	}
	$RequestContent->{'Command'}="LockBuild";
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
}

sub UnLock {
	my ($self) = @_;

	if(!$self->{'build_id'} ) {
		$self->{'_error'}=755;
		return $self->{'_error'};
	}

	$self->{'_error'} = 0;
	$self->Details();
	if ($self->{'_error'}) {
		return $self;
	}

	my $Request=$self->{'_connection'}->NewDefaultRequest();

	my $RequestContent = $self->{'_namemap'}->FunctionParams("UnlockBuild");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param}=$self->{$Var};
	}
	$RequestContent->{'Command'}="UnlockBuild";
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
}

sub ResultList {
	my ($self) = @_;

	if(!$self->{'build_id'} ) {
		$self->{'_error'}=757;
		return $self->{'_error'};
	}

	$self->{'_error'} = 0;
	$self->Details();
	if ($self->{'_error'}) {
		return $self;
	}

	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $Clause = "WHERE bf_bid=" . $self->{'build_id'} . " ORDER BY bf_sid";
	$self->{'_connection'}->AddRequestContent($Request,{
		Command => "APIQueryRecords",
		Table => "bf_results",
		Fields => "*",
		Clause => $Clause,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_results")});
	return $QueryResults;
}

sub ResultLog {
	my ($self, $step_result) = @_;

	if (!$self->{'build_id'} ) {
		$self->{'_error'} = 758;
		return undef;
	}

	# If a step result is specified, it must match this build.
	use Data::Dumper;
	print Data::Dumper->Dump([$step_result])."\n";
	if (ref($step_result)
			&& $step_result->{'build_id'} == $self->{'build_id'}
			&& length $step_result->{'step_id'}) {
		$step_result = $step_result->{'log_unique_id'};
	} else {
		$self->{'_error'} = 759;
		return undef;
	}

	$self->{'_error'} = 0;
	$self->Details();
	if ($self->{'_error'}) {
		return $self;
	}

	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $Clause = "WHERE bf_uid='" . $step_result."'";
#	if (ref($step_result) eq 'HASH' && length $step_result->{'step_id'}) {
#		$Clause .= " AND bf_sid=" . $step_result->{'step_id'}
#	}
	$Clause .= " ORDER BY bf_lid";
	$self->{'_connection'}->AddRequestContent($Request,{
		Command => "APIQueryRecords",
		Table => "bf_logs",
		Fields => "*",
		Clause => $Clause,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_logs")});
	return $QueryResults;
}

# Always Return Nicely
1;


__END__


=head1 I<NAME>

BuildForge::API::Build

=head1 I<SYNOPSIS>

	my $Build = new BuildForge::API::Build
	my $build_list = $Build->Waiting();
	foreach my $bld (@$build_list) {
		print $bld->{'build_id'} . "--" . $bld->{'tag'} . "\n";
	}

=head1 I<DESCRIPTION>

	This is available to use after a BuildForge::API object has been
instantiated. A Build object can be used to access the various build queues,
and to perform actions on running and historic builds. 

=head1 I<METHODS>

=head2 new({BuildParamsHashRef})

Create an object to access and modify builds and build queues.

=over 8

=item I<build_id>

The build id to associate this object with

=back

=head2 Waiting()

Returns an array ref of hash refs of builds in the waiting queue. 

=head2 Running()

Returns an array ref of hash refs of builds in the running queue.

=head2 Complete()

Returns an array ref of hash refs of builds in the completed queue.

=head2 Purge()

Queues the selected build to be purged. The Build object should be
instantiated, or set, with a build_id.

=head2 Cancel()

Cancels the build. The Build object should be instantiated, or set, with a
build_id.  If the build is currently Waiting, the cancellation will not take
effect until the build is Running.

=head2 Lock()

Locks the build. The Build object should be instantiated, or set, with a
build_id.

=head2 UnLock()

Unlocks the build. The Build object should be instantiated, or set, with a
build_id.

=head2 Locked()

Returns an array ref of hash refs of locked builds.

=head2 Sync()

Syncs the object with information from the console database. The Build object
should be instantiated, or set, with a build_id.

=head2 ResultList()

Returns the step results for the build.  Note that the step results from
pass and fail chains are not included.  The Build object should be
instantiated, or set, with a build_id.

=head2 ResultLog({StepResult})

Returns the message log for the specified step result, or for the entire
build if no valid step result is given.  Note that the logs from pass
and fail chains are not included.  The step result, if specified, is a
single entry from the array whose reference is returned by ResultList().
The Build object should be instantiated, or set, with a build_id.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
