# package BuildForge::API::Class
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Class
package BuildForge::API::Class;

use BuildForge::NameMap;
use BuildForge::API::Interface;
use BuildForge::Connection::Encoded;
use BuildForge::API::Access;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);

# Some class vars.
{
	my $default={};
	sub get_default {$default}
	sub clear_default {$default={}}	
}
# Instantiation
sub new {
	my ($class,$Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	
	$self->{'name'} = "";
	$self->{'access_group_id'} = new BuildForge::API::Access();
	$self->{'keep_builds'} = 0; # unlimited;
	$self->{'keep_type'} = "AnyBuild"; # AnyBuild/Only Failed/Only Passed/KeepPass
	$self->{'keep_days'} = 0; # number of days
	$self->{'purge_type'} = "E"; # E=Everything/B=Console Data/D=Logs & Files/L=Logs/F=Files;
	$self->{'purge_chain'} = 0; # PID
	$self->{'delchanges'} = "N"; # No options.
	$self->{'entry_chain'} = 0; # PID
	$self->{'exit_chain'} = 0; # PID
	
	map { $self->{$_} = $Params->{$_} } keys(%{$Params});
	
	# Get a default class...
	$self->SetClass($Params) unless($self->{'name'});
	
	return $self;
}

sub STORE {
	my($self,$key,$value) = @_;
	if(exists($self->{'_public'}->{$key})) {
		if($key eq "keep_type") {
			if($value =~ /Any/i) {
				$self->{'_public'}->{$key} = "AnyBuild";
			}
			elsif($value =~ /Failed/i) {
				$self->{'_public'}->{$key} = "Only Failed";
			}
			elsif($value =~ /Passed/i) {
				$self->{'_public'}->{$key} = "Only Passed";
			}
			elsif($value =~ /(Keep|Pass|One|1)/i) {
				$self->{'_public'}->{$key} = "KeepPass";
			}
			else {
				$self->{'_error'} = 840;
				$self->{'_info'} = "Unrecognized value [$value] for $key";
			}
			return;
		}
		elsif($key eq "purge_type") {
			if($value =~ /Everything/i) {
				$self->{'_public'}->{$key} = "E";
			}
			elsif($value =~ /(Console|Data)/i) {
				$self->{'_public'}->{$key} = "B";
			}
			elsif($value =~ /(\&|AND)/i) {
				$self->{'_public'}->{$key} = "D";
			}
			elsif($value =~ /Logs/i) {
				$self->{'_public'}->{$key} = "L";
			}
			elsif($value =~ /Files/i) {
				$self->{'_public'}->{$key} = "F";
			}
			else {
				$self->{'_public'}->{$key} = $value;
			}
			return;
		}
		elsif($key eq "purge_chain") {
			if(ref($value) eq "BuildForge::API::Project") {
				if($value->{'project_id'} > 0) {
					$self->{'_public'}->{$key} = $value->{'project_id'};
				}
				else {
					$self->{'_error'} = 842;
					$self->{'_info'} = "Attempted to set $key with an undefined project object.";
				}
			}
			elsif(ref(\$value) eq "SCALAR") {
				$value =~ /(\d*)/;
				if($1 > -1) {
					$self->{'_public'}->{$key} = $1;
				}
				else {
					$self->{'_error'} = 843;
					$self->{'_info'} = "Unrecognized value [$1] for $key";
				}
			}
			else {
				$self->{'_error'} = 844;
				$self->{'_info'} = "Unrecognized value [$value] for $key";
			}
			return;
		}
		elsif($key eq "entry_chain") {
			if(ref($value) eq "BuildForge::API::Project") {
				if($value->{'project_id'} > 0) {
					$self->{'_public'}->{$key} = $value->{'project_id'};
				}
				else {
					$self->{'_error'} = 842;
					$self->{'_info'} = "Attempted to set $key with an undefined project object.";
				}
			}
			elsif(ref(\$value) eq "SCALAR") {
				$value =~ /(\d*)/;
				if($1 > -1) {
					$self->{'_public'}->{$key} = $1;
				}
				else {
					$self->{'_error'} = 843;
					$self->{'_info'} = "Unrecognized value [$1] for $key";
				}
			}
			else {
				$self->{'_error'} = 844;
				$self->{'_info'} = "Unrecognized value [$value] for $key";
			}
			return;
		}
		elsif($key eq "exit_chain") {
			if(ref($value) eq "BuildForge::API::Project") {
				if($value->{'project_id'} > 0) {
					$self->{'_public'}->{$key} = $value->{'project_id'};
				}
				else {
					$self->{'_error'} = 842;
					$self->{'_info'} = "Attempted to set $key with an undefined project object.";
				}
			}
			elsif(ref(\$value) eq "SCALAR") {
				$value =~ /(\d*)/;
				if($1 > -1) {
					$self->{'_public'}->{$key} = $1;
				}
				else {
					$self->{'_error'} = 843;
					$self->{'_info'} = "Unrecognized value [$1] for $key";
				}
			}
			else {
				$self->{'_error'} = 844;
				$self->{'_info'} = "Unrecognized value [$value] for $key";
			}
			return;
		}
		
	}
	$self->SUPER::STORE($key,$value);
}

sub FETCH {
	my($self,$key) = @_;
	if(exists($self->{'_public'}->{$key})) {
		if($key eq "keep_type") {
			my $value = $self->{'_public'}->{$key};
			if($value =~ /AnyBuild/i) {
				return "Any Build";
			}
			elsif($value =~ /(KeepPass)/i) {
				return "Keep 1 Pass";
			}
			return $value;
		}
		elsif($key eq "purge_type") {
			my $value = $self->{'_public'}->{$key};
			if($value eq "E") {
				return "Everything";
			}
			elsif($value eq "B") {
				return "Console Data";
			}
			elsif($value eq "D") {
				return "Files \& Logs";
			}
			elsif($value eq "L") {
				return "Logs Only";
			}
			elsif($value eq "F") {
				return "Files Only";
			}
			return $value;
		}
	}
	$self->SUPER::FETCH($key);
}

sub Sync {
	my($self) = @_;
	if(!$self->{'name'}) {
		$self->{'_error'}=847;
		return $self->{'_error'};
	}
	my $WhereClause = "WHERE bf_name='$self->{'name'}'";
	my $QueryResults = $self->ProcessForHash({
			Command => "APIQueryRecords",
			Table => "bf_classes",
			Fields => "*",
			Clause => $WhereClause,
			Limit => 1,
		},"bf_classes");
	if(!defined($QueryResults->{'_record'})) {
		$self->{'_error'} = 848;
		$self->{'_info'} = "Did not find a matching class for '$self->{'name'}'.";
		return $self->{'_error'};
	}	
	return $self;
}

sub Create {
	my ($self) = @_;
	$self->{'_error'} = 845 unless($self->{'name'});
	
	return undef if($self->{'_error'});

	# Invalidate Cache
	$self->InvalidateCache();
	
	my $Request = $self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddClass");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "access_group_id") { 
			$RequestContent->{$Param} = $self->{$Var};
		}
		else {
			$RequestContent->{$Param} = $self->{'_public'}->{$Var};
		}
	}
	$RequestContent->{'Command'} = "AddClass";
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	$self->SetClass($self) unless($self->{'_error'});
	return $self;
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 846 unless($self->{'name'});
	
	return $self->{'_error'} if($self->{'_error'});
	# Invalidate Cache...
	$self->InvalidateCache();
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteClass");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteClass";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_classes");
	return $self->GetLastError();
}

sub InvalidateCache {
	my($self) = @_;
	ref($self)->clear_default();
}	

sub SetClass {
	my ($self,$Params) = @_;
	my $Class = ref($self);
	my $Default = $Class->get_default();
	if(!defined($Params->{'name'})) {
		$Params->{'name'}="_default";
	}
	if(exists($Default->{$Params->{'name'}})) {
		map { $self->{$_} = $Default->{$Params->{'name'}}->{$_} }keys(%{$Default->{$Params->{'name'}}});
		return;
	}
		
	
	my $WhereClause;
	$WhereClause = "where bf_name='$Params->{'name'}' " if($Params->{'name'} ne "_default");
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	$self->{'_connection'}->AddRequestContent($Request,{
		'Command'	=> "APIQueryRecords",
		'Table'	=> "bf_classes",
		'Fields'	=> "*",
		'Clause'	=> $WhereClause,
		'Limit'	=> 1,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_classes")});
	# Now update our object for each key...
	
	map {$self->{$_} = ${$QueryResults}[0]->{$_}} keys(%{${$QueryResults}[0]});
	$Default->{$Params->{'name'}} = ${$QueryResults}[0];
}

# Always Return Nicely
1;

__END__

=head1 I<NAME>

BuildForge::API::Class

=head1 I<SYNOPSIS>

	my $Class = new BuildForge::API::Class();
	$Class->{'name'} = "My New Class";
	$Class->Create();

=head1 I<DESCRIPTION>

    This class allows you to create, access, and delete BuildForge project
classes. 

=head1 I<METHODS>

=head2 new({ClassParamsHashRef})

Create an object to access existing class information, or to add a new class
to the BuildForge console.

=over 8

=item I<name>

The name for the class

=item I<access_group_id>

ID of the access level for this class

=item I<keep_builds>

Number of builds to keep

=item I<keep_type>

Type (AnyBuild|Only Failed|Only Passed|Keep 1 Pass)

=item I<keep_days>

Number of days to keep builds around

=item I<purge_type>

(Everything|Console Data|Logs & Files|Logs|Files)

=item I<purge_chain>

Project to call on purge (Object or project_id)

=item I<entry_chain>

Project to call on Class Change (Object or project_id)

=item I<exit_chain>

Project to call on Class Change (Object or project_id)

=back

=head2 Create()

Creates a new project Class. This method requires the variable 'name' be set
within the class.

=head2 Delete()

Deletes the Class. Requires the class 'name' to be set.

=head2 Sync()

Syncs the object data with information from the console database.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
