# package BuildForge::API::Engine
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Engine
package BuildForge::API::Engine;

use BuildForge::API::Interface;
use BuildForge::NameMap;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	map {$self->{$_} = $Params->{$_}} keys(%{$Params});
	return $self;
}

sub Pause {
	my ($self) = @_;
	return $self->{'_error'} if($self->{'_error'});
	my $RequestContent = {};
	$RequestContent->{'Command'} = "SetEnginePauseState";
	$RequestContent->{'State'} = "Y";
	$self->ProcessQuery($RequestContent);
	return $self->GetLastError();
}

sub Resume {
	my ($self) = @_;
	return $self->{'_error'} if($self->{'_error'});
	my $RequestContent = {};
	$RequestContent->{'Command'} = "SetEnginePauseState";
	$RequestContent->{'State'} = "N";
	$self->ProcessQuery($RequestContent);
	return $self->GetLastError();
}

# Always Return Nicely
1;

__END__

=head1 I<NAME>

BuildForge::API::Engine

=head1 I<SYNOPSIS>

	my $Engine = new BuildForge::API::Engine();
	$Engine->Pause();
	$Engine->Resume();

=head1 I<DESCRIPTION>

    This class allows for very basic pause/resume control of the buildforge
engine.

=head1 I<METHODS>

=head2 new()

Creates the object to allow control over the buildforge engine.

=head2 Pause()

Pauses the BuildForge engine, currently running builds will continue 
to completion however no newly queued builds will be allowed to start.

=head2 Resume()

Resumes the BuildForge engine.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
