# package BuildForge::API::Environ
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Environ
package BuildForge::API::Environ;

use BuildForge::API::Interface;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	
	$self->{'group_id'} = 0;
	$self->{'vars'} = [];
	$self->{'access_group_id'} = new BuildForge::API::Access();
	$self->{'name'} = "";
	
	map {$self->{$_} = $Params->{$_} unless($Params->{$_} =~/^_/) } keys(%{$Params});
	$self->SetEnvironment($Params) if(defined($Params));
	return $self;
}

sub SetEnvironment {
	my ($self,$Params,$Cond) = @_;
	my $WhereClause;
	if($Params->{'group_id'}) {
		$WhereClause = "WHERE bf_bid = $Params->{'group_id'} AND bf_eid=0 ";
	}
	elsif($Params->{'name'}) {
		$WhereClause = "WHERE bf_param = '$Params->{'name'}' AND bf_eid=0 ";
	}
	else {
		$self->{'group_id'} = 0;
		return 0;
	}
	my $Request = $self->{'_connection'}->NewDefaultRequest();
	$self->{'_connection'}->AddRequestContent($Request,{
		'Command'	=> "APIQueryRecords",
		'Table'	=> "bf_env",
		'Fields'	=> "*",
		'Clause'	=> $WhereClause,
		'Limit'	=> 1,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError() unless($Cond->{'nomsg'});
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage() unless($Cond->{'nomsg'});
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_env")});
	map {$self->{$_} = ${$QueryResults}[0]->{$_} unless($_ =~/^_/) } keys(%{${$QueryResults}[0]});
}

sub Sync {
	my($self,$Cond) = @_;
	$self->SetEnvironment($self,$Cond);
	return $self->{'_error'}  if($self->{'_error'});
	if($self->{'group_id'} == 0) {
		$self->{'_error'} = 876;
		$self->{'_info'} = "Cannot sync on a non-existant environment group";
		return $self->GetLastError();
	}
	$self->{'vars'} = $self->ProcessForArray({
			Command => "APIQueryRecords",
			Table => "bf_env",
			Fields => "*",
			Clause => "WHERE bf_bid = $self->{'group_id'} AND bf_eid > 0 ORDER BY bf_eid",
		},"bf_env",$Cond);

	return $self->GetLastError();
}
	

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 871 unless($self->{'group_id'} || $self->{'name'});
	return $self->{'_error'} if($self->{'_error'});
	$self->SetEnvironment($self);
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteEnvGroup");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteEnvGroup";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_env");
	return $self->GetLastError();
}	

sub Create {
	my ($self,$From) = @_;
	$self->{'_error'} = 870 unless($self->{'name'});
	return undef if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddEnvGroup");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "AddEnvGroup";
	$RequestContent->{'Copy'} = $From->{'group_id'} || 0;
	
	my $Results = $self->{'_connection'}->ProcessNewRequest($RequestContent);
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_env")});
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	map {$self->{$_} = $QueryResults->{$_} unless($_=~/^_/)} keys(%{$QueryResults});
	$self->Sync() unless($self->{'_error'});
	return $self;
}

sub DeleteVariable {
	my($self,$Variable) = @_;
	$self->{'_error'} = 874 unless($self->{'name'});
	$self->{'_error'} = 875 unless($Variable->{'name'});
	return undef if($self->{'_error'});
	$self->Sync();
	return $self->{'_error'} if($self->{'_error'});

	foreach my $var (@{$self->{'vars'}}) {
		if($var->{'name'} eq $Variable->{'name'}) {
			$var->{'group_id'} = $self->{'group_id'};
			my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteEnvGroupValue");
			while(my($Param,$Var) = each %{$RequestContent}) {
				$RequestContent->{$Param} = $var->{$Var};
			}
			$RequestContent->{'Command'} = "DeleteEnvGroupValue";
			my $QueryResults = $self->ProcessForHash($RequestContent,"bf_env");
			return $self->GetLastError();
		}
	}
}

sub VariableList {
	my($self) = @_;
	$self->Sync();
	return $self->{'vars'};
}

sub AddVariable {
	my($self,$Variable) = @_;
	
	$self->{'_error'} = 872 unless($self->{'name'});
	$self->{'_error'} = 873 unless($Variable->{'name'});
	return undef if($self->{'_error'});
	$self->Sync();
	return $self->{'_error'} if($self->{'_error'});
	$Variable->{'group_id'} = $self->{'group_id'};
	$Variable->{'value'} = "" unless($Variable->{'value'});
	$Variable->{'action'} = "S" unless($Variable->{'action'});
	$Variable->{'on_project'} = "N" unless($Variable->{'on_project'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddEnvGroupValue");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $Variable->{$Var};
	}
	$RequestContent->{'Command'} = "AddEnvGroupValue";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_env");
	$self->Sync({'nomsg' => 1}) unless($self->{'_error'});
	return $self->GetLastError();
}
	
# Always Return Nicely
1;

__END__

=head1 I<NAME>

BuildForge::API::Environ

=head1 I<SYNOPSIS>

	my $Environ = new BuildForge::API::Environ();
	$Environ->{'name'} = "New Env Group";
	$Environ->Create();

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been
instantiated. It is used to create, delete and modify environment groups
and variables.

=head1 I<METHODS>

=head2 new({EnvironParamsHashRef})

Create an object to access existing environment information, or to add a new
environment group to the BuildForge console. 

=over 8

=item I<name>

The title of this env group

=item I<access_group_id>

The controlled access to this group

=back

=head2 Create(From)

Creates a new environment group. Requires the 'name' object property to be set.

=over 8

=item I<From>

A BuildForge::API::Environ object.

=back

=head2 Delete()

Deletes the environment group. Requires either the 'group_id' or 'name' to be
set.

=head2 AddVariable({VariableParamsHashRef})

Adds a new variable to the environment group. Requires the object 'name' to be
set, and the 'name' key to be set in the VariableParamsHashRef. 

=over 8

=item I<name>

The name of the variable you with to create

=item I<value>

The initial value for the variable

=item I<action>

(S/N/A/P/C/U/H)

=over 16

=item	:  S = Assign As

=item	:  N = Assign If Not Assigned

=item	:  A = Append Existing

=item	:  P = Prepend Existing

=item	:  C = Clear Existing

=item	:  U = Delete Variable

=item	:  H = Assign Hidden

=back

=item I<on_project >

(N/R/O/S/C)

=over 16

=item	:  N = Normal 

=item	:  R = Required

=item	:  O = Read-Only

=item	:  S = Suppress Display

=item	:  C = Must Change

=back

=back

=head2 DeleteVariable({VariableParamsHashRef})

Deletes a variable from the environment group. Requires the objects 'name'
parameter to be set, and the 'name' key of the VariableParamsHashRef to be set.

=over 8

=item I<name>

The name of the variable you with to delete

=back

=head2 VariableList()

Returns a list ref of hash refs containing information about the variables, 
values and settings of each entry in the Environment Group. Requires the 
objects 'group_id' or 'name' to be set.

=head2 Sync()

Syncs the object with information from the console database. Requires the 
objects 'group_id' or 'name' to be set.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
