# package BuildForge::API::Interface
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other intellectual property
# rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Interface
# This package is used for:
# Standard Inherited package implementing common routines.
package BuildForge::API::Interface;

use BuildForge::Connection::Encoded;
use BuildForge::NameMap;
use BuildForge::API::Environ;
use BuildForge::API::Class;
use BuildForge::API::Access;
use BuildForge::API::Server;

use strict;

# Instantiation
sub new {
	my ($class,$Params) = @_;
	my $self = {};
	my $selfhash={
		'_error'	=> 0,
		'_info'	=> 0,
		'_namemap'	=> undef,
		'_connection'	=> new BuildForge::Connection::Encoded(),
		'_public'	=> {},
		'_private'	=> {
			'_namemap'	=>{},
			'_timestamp' => 0,
			},
	};
	
	my $NameMap = new BuildForge::NameMap();
	
	$selfhash->{'_public'} = $NameMap->PublicNames($class);
	
	tie %{$self},$class,$selfhash;
	$self->{'_namemap'} = $NameMap;
	
	return bless $self,$class || ref($class);
}

sub TIEHASH {
	my ($class,$selfhash) = @_;
	bless $selfhash,$class;
}

sub FETCH {
	my ($self,$key) = @_;
	if(exists($self->{'_public'}->{$key})) {
		if($key eq "environment_id" && ref($self) ne "BuildForge::API::Environ") {
			return $self->{'_public'}->{'environment_id'}->{'group_id'};
		}
		elsif($key eq "class" && ref($self) ne "BuildForge::API::Class") {
			return $self->{'_public'}->{'class'}->{'name'};
		}
		elsif($key eq "access_group_id" && ref($self) ne "BuildForge::API::Access") {
			if (ref($self) eq "BuildForge::API::Step") {
				if ($self->{'set_default_level'} == 1) {
					return 0;
				}	
			}	
			return $self->{'_public'}->{'access_group_id'}->{'access_group_id'};
		}
		elsif(( ($key eq "server") || ($key eq "default_server") ) && ref($self) ne "BuildForge::API::Server") {
			return $self->{'_public'}->{'server'}->{'name'};
		}
		return $self->{'_public'}->{$key};
	}
	elsif(exists($self->{'_private'}->{$key})) {
		return $self->{'_private'}->{$key};
	}
	return $self->{$key};
}

sub STORE {
	my ($self,$key,$val) = @_;
	if(exists($self->{'_public'}->{$key})) {
		if($key eq "environment_id" && ref($self) ne "BuildForge::API::Environ") {
			if(ref($val) ne "BuildForge::API::Environ") {
				if(ref(\$val) eq "SCALAR") {
					if($val =~ /^\d+$/) {
						$val = new BuildForge::API::Environ({'group_id'=>$val});
					}
					else {
						$val = new BuildForge::API::Environ({'name'=>$val});
					}
				}
				else {
					$self->{'_error'} = 752;
					$self->{'_info'} = "Set of environment_id requires a name, environment id, or BuildForge::API::Environ object";
					return;
				}
			}
		}
		elsif($key eq "class" && ref($self) ne "BuildForge::API::Class") {
			if(ref($val) ne "BuildForge::API::Class") {
				if(ref(\$val) eq "SCALAR") {
					$val = new BuildForge::API::Class({'name'=>$val});
				}
				else {
					$self->{'_error'} = 753;
					$self->{'_info'} = "Set of class requires a name, or BuildForge::API::Class object";
					return;
				}
			}
		}
		elsif($key eq "access_group_id" && ref($self) ne "BuildForge::API::Access") {
			if(ref($val) ne "BuildForge::API::Access") {
				if(ref(\$val) eq "SCALAR") {
					$val = new BuildForge::API::Access({'access_group_id'=>$val});
				}
				else {
					$self->{'_error'} = 754;
					$self->{'_info'} = "Set of access_group_id requires an access group id, or BuildForge::API::Access object";
					return;
				}
			}
		}
		elsif(( ($key eq "server") || ($key eq "default_server") ) && ref($self) ne "BuildForge::API::Server") {
			if(ref($val) ne "BuildForge::API::Server") {
				if(ref(\$val) eq "SCALAR") {
					$val = new BuildForge::API::Server({'name'=>$val});
				}
				else {
					$self->{'_error'} = 755;
					$self->{'_info'} = "Set of server requires a name or BuildForge::API::Server object";
				}
			}
		}
		$self->{'_public'}->{$key} = $val;
	}
	elsif(exists($self->{'_private'}->{$key})) {
		$self->{'_private'}->{$key} = $val;
	}
	else {
		$self->{$key} = $val;
	}
}

sub EXISTS {
	my ($self,$key) = @_;
	if(exists($self->{'_public'}->{$key})) {
		return 1;
	}
	elsif(exists($self->{'_private'}->{$key})) {
		return 1;
	}
	elsif(exists($self->{$key})) {
		return 1;
	}
	return 0;
}

sub DELETE {
	my ($self,$key) = @_;
	if(exists($self->{'_public'}->{$key})) {
		$self->{'_public'}->{$key} = ();
		return 1;
	}
	return(delete($self->{$key}));
}

sub CLEAR {
	my ($self) = @_;
	print ref($self)."->CLEAR not overridden in child class\n";
}

sub FIRSTKEY {
	my ($self) = @_;
	keys %{$self->{'_public'}};
	my $first_key = each %{$self->{'_public'}};
	return undef unless defined($first_key);
	return $first_key;
}

sub NEXTKEY {
	my ($self,$prevkey) = @_;
	my $next_key = each %{$self->{'_public'}};
	return undef unless defined $next_key;
	return $next_key;
}

sub GetLastError {
	my ($self) = @_;
	return $self->{'_error'};
}

sub GetLastMessage {
	my ($self) = @_;
	return $self->{'_info'};
}

sub ProcessForHash {
	my($self,$RequestContent,$NameMap,$Cond) = @_;
	my $Results = $self->{'_connection'}->ProcessNewRequest($RequestContent);
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans($NameMap)});
	$self->{'_error'} = $self->{'_connection'}->GetLastError() unless($Cond->{'nomsg'});
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage() unless($Cond->{'nomsg'});
	map {$self->{$_} = $QueryResults->{$_} unless($_=~/^_/)} keys(%{$QueryResults});
	return $QueryResults;
}

sub ProcessForArray {
	my($self,$RequestContent,$NameMap,$Cond) = @_;
	my $Results = $self->{'_connection'}->ProcessNewRequest($RequestContent);
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans($NameMap)});
	$self->{'_error'} = $self->{'_connection'}->GetLastError() unless($Cond->{'nomsg'});
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage() unless($Cond->{'nomsg'});
	return $QueryResults;
}

sub ProcessQuery {
	my($self,$RequestContent,$NameMap,$Cond) = @_;
	my $Results = $self->{'_connection'}->ProcessNewRequest($RequestContent);
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans($NameMap)});
	$self->{'_error'} = $self->{'_connection'}->GetLastError() unless($Cond->{'nomsg'});
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage() unless($Cond->{'nomsg'});
	return $QueryResults;
}

# Always Return Nicely
1;
