# package BuildForge::API::Project
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Project
package BuildForge::API::Project;

use BuildForge::Connection::Encoded;
use BuildForge::API::Environ;
use BuildForge::API::Class;
use BuildForge::API::Access;
use BuildForge::NameMap;
use BuildForge::API::Interface;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self = $class->SUPER::new(@_);

	$self->{'tag_format'} = "BUILD_\$BF_B";
	$self->{'selector'} = "";
	$self->{'sticky'} = 0;
	$self->{'environment_id'} = new BuildForge::API::Environ();
	$self->{'pass_notify_group'} = "0";
	$self->{'fail_notify_group'} = "0";
	$self->{'tagsync'} = 0;
	$self->{'access_group_id'} = new BuildForge::API::Access();
	$self->{'class'} = new BuildForge::API::Class();
	$self->{'maxthread'} = 0;
	$self->{'start_notify_group'} = "0";
	$self->{'exclusive'} = 0;
	$self->{'pass_chain'} = 0;
	$self->{'fail_chain'} = 0;
	$self->{'geo_id'} = "";
	
	while(my($variable,$value) = each %{$Params}) {
		$self->{$variable} = $value;
	}
	$self->Sync() if($Params->{'project_id'} > 0);
	return $self;
}

sub STORE {
	my($self,$key,$value) = @_;
	if($key eq "server") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key 'server' is no longer used, please use 'selector' instead.";
		return;
	}
	$self->SUPER::STORE($key,$value);
}

sub FETCH {
	my($self,$key) = @_;
	if($key eq "server") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key 'server' is no longer used, please use 'selector' instead.";
		return;
	}
	$self->SUPER::FETCH($key);
}

sub GetEnvironment {
	my($self) = @_;
	return $self->{'_public'}->{'environment_id'};
}

# Details	: Deprecated.
sub Details {
	my($self) = @_;
	return $self->Sync();
}

sub Sync {
	my ($self) = @_;
	if(!$self->{'title'} && !$self->{'project_id'}) {
		$self->{'_error'}=751;
		return $self;
	}

	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $Clause = "WHERE bf_pid=$self->{'project_id'}";
	if(!$self->{'project_id'}) {
		$Clause = "WHERE bf_description='$self->{'title'}'";
	}
	$self->{'_connection'}->AddRequestContent($Request,{
		Command => "APIQueryRecords",
		Table => "bf_projects",
		Fields => "*",
		Clause => $Clause,
		Limit => 1,
	});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_projects")});
	# Now update our object for each key...
	my $Return = [];
	while (my ($Field,$Value) = each %{$QueryResults}) {
		$self->{$Field}=$Value unless($Field=~/^_/);
	}
	if(!defined($QueryResults->{'_record'})) {
		$self->{'_error'} = 752;
		return $self->{'_error'};
	}	
	return $self;
}

sub Clobber {
	my ($self) = @_;

	$self->Sync();
	my $Request=$self->{'_connection'}->NewDefaultRequest();

	my $RequestContent = $self->{'_namemap'}->FunctionParams("ClobberProject");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param}=$self->{$Var};
	}
	$RequestContent->{'Command'}="ClobberProject";

	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
}

sub Fire {
	my ($self,$EnvHash,$Class) = @_;

	$self->{'_error'} = 0;
	$self->Details();
	return($self) if ($self->{'_error'});

	my $Request=$self->{'_connection'}->NewDefaultRequest();

	my $RequestContent = $self->{'_namemap'}->FunctionParams("FireBuild");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param}=$self->{$Var};
	}
	$RequestContent->{'Command'}="FireBuild";
	$RequestContent->{'APIEnvAdd'}=$self->ProcessEnvHash($EnvHash) if($EnvHash);
	$RequestContent->{'APIClass'} = $Class if($Class);

	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_projects")});
	$self->{'_info'} = $QueryResults->{'_info'};
	my $bid = $self->{'_info'};
	$bid =~ s/^.*: (\d+)$/$1/;
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	return($bid);
}

sub ProcessEnvHash {
	my($self,$EnvRef) = @_;
	my $EnvString = "";
	my $EnvObj = $self->GetEnvironment();
	foreach my $EnvItem (@{$EnvObj->VariableList()}) {
		if(exists($EnvRef->{$EnvItem->{'name'}})) {
			$EnvString.="$EnvItem->{'action'}:$EnvItem->{'name'}:$EnvRef->{$EnvItem->{'name'}}\n";
			delete $EnvRef->{$EnvItem->{'name'}};
		}
	}
	map { $EnvString.="S:$_:$EnvRef->{$_}\n" } keys(%{$EnvRef});
	return $EnvString;
}

sub Create {
	my ($self,$From) = @_;
	$self->{'_error'} = 750 unless($self->{'title'});
	
	return undef if($self->{'_error'});
	
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	# Need:
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddProject");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		if ($From && $From->{'project_id'} && $Var ne 'title')  {
			$self->{$Var} = $From->{$Var};
		}
		if ($Var eq "sticky") {
			$RequestContent->{$Param} = ($self->{$Var} ? "on" : "off");
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "AddProject";
	$RequestContent->{'Copy'} = $From->{'project_id'} || 0;
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);

	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	if($self->{'_error'} = $self->{'_connection'}->GetLastError()) {
		$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	} else {	
		my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_projects")});
		$self->Sync();
		$self->{'_info'}=$QueryResults->{'_info'};
	}

	return $self; 
}

sub AddStep {
	my($self,$Step) = @_;
	$self->Details();
	$Step->{'project_id'} = $self->{'project_id'};
	$self->{'_error'} = $Step->Create();
	$self->{'_info'} = $Step->GetLastMessage();
}

sub DeleteStep {
	my($self,$Step) = @_;
	$self->Details();
	$Step->{'project_id'} = $self->{'project_id'};
	$self->{'_error'} = $Step->Delete();
	$self->{'_info'} = $Step->GetLastMessage();
	return $self->GetLastError();
}

sub UpdateStep {
	my($self,$Step,$Params) = @_;
	$self->Details();
	$Step->{'project_id'} = $self->{'project_id'};
	$self->{'_error'} = $Step->Update($Params);
	$self->{'_info'} = $Step->GetLastMessage();
	return $self->GetLastError();
}

sub AddTagVar {
	my($self,$TagVar) = @_;
	$self->Details();
	$TagVar->{'project_id'} = $self->{'project_id'};
	$self->{'_error'} = $TagVar->Create();
	$self->{'_info'} = $TagVar->GetLastMessage();
}

sub DeleteTagVar {
	my($self,$TagVar) = @_;
	$self->Details();
	$TagVar->{'project_id'} = $self->{'project_id'};
	$self->{'_error'} = $TagVar->Delete();
	$self->{'_info'} = $TagVar->GetLastMessage();
}

sub AddRegister {
	my($self,$Register) = @_;
	$self->Sync();
	$Register->{'project_id'} = $self->{'project_id'};
	$self->{'_error'} = $Register->Create();
	$self->{'_info'} = $Register->GetLastMessage();
}

sub DeleteRegister {
	my($self,$Register) = @_;
	$self->Sync();
	$Register->{'project_id'} = $self->{'project_id'};
	$self->{'_error'} = $Register->Delete();
	$self->{'_info'} = $Register->GetLastMessage();
}

sub TagVarList {
	my($self) = @_;
	$self->Sync();
	return $self->ProcessForArray({
					'Command' => "APIQueryRecords",
					'Table' => "bf_tags",
					'Fields' => "*",
					'Clause' => "where bf_pid=$self->{'project_id'}"
				},
				"bf_tags");
}

sub StepList {
	my($self) = @_;
	$self->Sync();
	return $self->ProcessForArray({
					'Command' => "APIQueryRecords",
					'Table' => "bf_steps",
					'Fields' => "*",
					'Clause' => "where bf_pid=$self->{'project_id'} order by bf_sid"
				},
				"bf_steps");
}

sub RegisterList {
	my($self) = @_;
	$self->Sync();
	return $self->ProcessForArray({
					'Command' => "APIQueryRecords",
					'Table' => "bf_data",
					'Fields' => "*",
					'Clause' => "where bf_uid=$self->{'project_id'} AND bf_bid=0"
				},
				"bf_data");
}	

# Always Return Nicely
1;


__END__

=head1 I<NAME>

BuildForge::API::Project

=head1 I<SYNOPSIS>

	my $Project = new BuildForge::API::Project();
	$Project->{'title'} = "My New Project";
	$Project->Create();

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been 
instantiated. It is used to create new BuildForge Projects, and to access 
existing project information.

=head1 I<METHODS>

=head2 new({ProjectParamsHashRef})

Create an object to access existing project information or to add a new project
to the BuildForge console.

=over 8

=item I<title>

The name for the project

=item I<tag_format>

Tag format for the Project

=item I<selector>

Default Selector name for the project

=item I<sticky>

Lock project to the server selected by the selector (0 or 1)

=item I<environment_id>

ID of the environment group to use (integer)

=item I<pass_notify_group>

Group name to notify upon project success.

=item I<fail_notify_group>

Group name to notify upon project failure.

=item I<tagsync>

Project ID to sync tag variables with.

=item I<access_group_id>

ID of the access level for this project

=item I<class>

The class for the project.

=item I<maxthread>

Maximum number of threads for this project (integer)

=item I<exclusive>

Max number of running instances of this project.

=item I<project_id>

ID used to uniquely identify the project inside BuildForge

=back

=head2 Sync()

Populates the Project object with data from the BF Console. Requires either 
Project 'title' or 'project_id' to be set.

=head2 GetEnvironment()

Returns a BuildForge::API::Environ object representing the Project Environment.
Requires either Project 'title' or 'project_id' to be set.

=head2 Clobber()

Clobbers the project from the management console. The project, all steps, all 
builds, and all log data are removed. Use with caution. Requires either Project
'title' or 'project_id' to be set.

=head2 Fire({EnvHashRef},{ClassName})

Starts a build of the Project. Requires either Project 'title' or 'project_id' 
to be set. Returns the internal Build ID of the started build.

=over 8

=item I<EnvHashRef>

A Hash ref of environment name/value pairs to be set after the 
project env is set. If any of the names exist within the Project Env, they will 
adopt the same action. For example, if you Append to the PATH within the 
Project env, and you pass Fire() with a hashref that contains PATH information,
both PATH variables will be appended to the projects running path.

=item I<ClassName>

The name of the class to run this with.

=back

=head2 Create(From)

Create a new Project inside BuildForge. Requires Project 'title' to be set. The
optional From parameter allows you to use an existing Project as a template. 
All settings except the project title will then be taken from the source project.

=over 8

=item I<From>

A BuildForge::API::Project Object to use as a template for this new 
project.

=back

=head2 AddStep(Step)

Adds a step to an existing Project. Requires either Project 'title'
or 'project_id' to be set.

=over 8

=item I<Step>

BuildForge::API::Step object (required).

=back

=head2 UpdateStep(Step,MoveToHashRef)

Updates a Project Step, and optionally allows you to change its location.

=over 8

=item I<Step>

BuildForge::API::Step object (required).

=item I<MoveToHashRef>

A hashref containing one key:
{MoveTo => Step Object or location(1 based)}

=back

=head2 DeleteStep(Step)

Deletes a step from a Project. Requires either Project 'title' or 'project_id'
to be set.

=over 8

=item I<Step>

BuildForge::API::Step object (required).

=back

=head2 AddTagVar(TagVar)

Adds a tag variable to project. Requires either Project 'title' or 'project_id'
to be set.

=over 8

=item I<TagVar>

A BuildForge::API::TagVar object (required).

=back

=head2 DeleteTagVar(TagVar)

Deletes a tag variable from a project. Requires either Project 'title' or 
'project_id' to be set.

=over 8

=item I<TagVar>

A BuildForge::API::TagVar object (required).

=back

=head2 GetLastError()

Returns	the last error code encountered by the object.

=head2 GetLastMessage()

Returns an last message string encountered by the object.

=head2 StepList()

Returns an array ref of hash refs containing step details. Requires either 
Project 'title' or 'project_id' to be set.

=head2 AddRegister(Register)

Add a Register entry to the Project. Requires either Project 'title' or 
'project_id' to be set.

=over 8

=item I<Register>

A BuildForge::API::Register object (required).

=back

=head2 DeleteRegister(Register)

Deletes a register from the Project. Requires either Project 'title' or 
'project_id' to be set.

=over 8

=item I<Register>

A BuildForge::API::Register object (required).

=back

=head2 RegisterList()

Returns an array ref of hash refs containing register details. Requires either 
Project 'title' or 'project_id' to be set.

=head2 TagVarList()

Returns an array ref of hash refs containing tag variable details. Requires 
either Project 'title' or 'project_id' to be set.

=head2 Sync()

Syncs the object with information from the console database. Requires either 
Project 'title' or 'project_id' to be set.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
