# package BuildForge::API::Register
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other intellectual property
# rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Register
package BuildForge::API::Register;

use BuildForge::API::Interface;
use BuildForge::NameMap;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	$self->{'register'} = "";
	$self->{'data'} = "";
	map {$self->{$_} = $Params->{$_}} keys(%{$Params});
	return $self;
}

sub Create {
	my($self) = @_;
	$self->{'_error'} = 930 unless($self->{'project_id'});
	$self->{'_error'} = 931 unless($self->{'register'});

	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddProjectReg");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "AddProjectReg";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_data");
	return $self->GetLastError();
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 932 unless($self->{'project_id'});
	$self->{'_error'} = 933 unless($self->{'register'});

	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DelProjectReg");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DelProjectReg";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_data");
	return $self->GetLastError();
}


# Always Return Nicely
1;

__END__

=head1 I<NAME>

BuildForge::API::Register

=head1 I<SYNOPSIS>

	my $Register = new BuildForge::API::Register();
	$Register->{'name'} = "ProjectReg";
	$Register->{'data'} = "My Register Data";
	$Project->AddRegister($Register);

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been
instantiated. It is used to create and delete register variables defined for 
projects inside the BuildForge console.

=head1 I<METHODS>

=head2 new({RegisterParamsHashRef})

Creates a new object to add or remove registers from a BuildForge project.
The objects 'register' property must be set prior to passing the object to a 
BuildForge::API::Project object.

=over 8

=item I<register>

The name of this register

=item I<data>

The data held in this register

=back

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
