# package BuildForge::API::Server
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Server
package BuildForge::API::Server;

use BuildForge::Connection::Encoded;
use BuildForge::API::Environ;
use BuildForge::API::Interface;
use BuildForge::NameMap;
use BuildForge::API::Access;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	$self->{'name'} = "";
	$self->{'hostname'} = "";
	$self->{'access_group_id'} = new BuildForge::API::Access();
	# 4.0 $self->{'pool_name'} = "";
	$self->{'root_path'} = "/";
	# 4.0 Sellector $self->{'load_percent'} = 40;
	# 4.0 Collector $self->{'job_limit'} = 3;
	$self->{'transfer_mode'} = "none";
	$self->{'environment_id'} = new BuildForge::API::Environ();
	# 4.0 Authid $self->{'agent_username'} = "";
	# 4.0 AUthid $self->{'agent_password'} = "";
	$self->{'active'} = 1;
	# 4.0 new:
	$self->{'authid'} = "";
	$self->{'collector'} = "";
	$self->{'server_id'} = 0;
	
	while(my($variable,$value) = each %{$Params}) {
		$self->{$variable} = $value;
	}
	return $self;
}

sub STORE {
	my($self,$key,$value) = @_;
	if($key eq "agent_username" || $key eq "agent_password") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key '$key' is no longer used, please use 'authid' instead.";
		return;
	}
	elsif($key eq "pool_name" || $key eq "load_percent" || $key eq "job_limit") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key '$key' is no longer used, please use a 'collector' property instead.";
		return;
	}
	$self->SUPER::STORE($key,$value);
}

sub FETCH {
	my($self,$key) = @_;
	if($key eq "agent_username" || $key eq "agent_password") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key '$key' is no longer used, please use 'authid' instead.";
		return;
	}
	elsif($key eq "pool_name" || $key eq "load_percent" || $key eq "job_limit") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key '$key' is no longer used, please use a 'collector' property instead.";
		return;
	}
	$self->SUPER::FETCH($key);
}

sub Sync {
	my($self) = @_;
	if(!$self->{'name'}) {
		$self->{'_error'}=815;
		return $self->{'_error'};
	}
	my $WhereClause = "WHERE bf_name='$self->{'name'}'";
	my $QueryResults = $self->ProcessForHash({
			Command => "APIQueryRecords",
			Table => "bf_servers",
			Fields => "*",
			Clause => $WhereClause,
			Limit => 1,
		},"bf_servers");
	if(!defined($QueryResults->{'_record'})) {
		$self->{'_error'} = 816;
		$self->{'_info'} = "No matching server found.";
		return $self->{'_error'};
	}
	return $self;
}

sub IncrementJobCount {
	my($self) = @_;
	if(!$self->{'name'}) {
		$self->{'_error'}=817;
		return $self->{'_error'};
	}
	$self->ProcessForHash({
		Command => "ChangeJobCount",
		Type => "Increment",
		Server => $self->{'name'},
	},"bf_servers");
	return $self->GetLastError();	
}

sub DecrementJobCount {
	my($self) = @_;
	if(!$self->{'name'}) {
		$self->{'_error'}=818;
		return $self->{'_error'};
	}
	$self->ProcessForHash({
		Command=>"ChangeJobCount",
		Type=>"Decreas",
		Server=>$self->{'name'},
	},"bf_servers");
	return $self->GetLastError();	
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 812 unless($self->{'name'});
	
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteServer");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteServer";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_servers");
	return $self->GetLastError();
}

sub Enable {
	my($self) = @_;
	$self->{'_error'} = 813 unless($self->{'name'});
	
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("EnableServer");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "ActivateServer";
	$RequestContent->{'Action'} = "Enable";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_servers");
	return $self->GetLastError();
}

sub Disable {
	my($self) = @_;
	$self->{'_error'} = 814 unless($self->{'name'});
	
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DisableServer");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "ActivateServer";
	$RequestContent->{'Action'} = "Disable";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_servers");
	return $self->GetLastError();
}

sub Create {
	my ($self) = @_;
	$self->{'_error'} = 810 unless($self->{'name'});
	$self->{'_error'} = 811 unless($self->{'hostname'});
	
	return undef if($self->{'_error'});
	
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddServer");
	
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "AddServer";
	
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_servers")});
	$self->{'_info'}= $self->{'_connection'}->GetLastMessage();
	return $self;		
}

sub Manifest {
	my($self) = @_;
	
	$self->{'_error'} = 812 unless($self->{'name'});
	return undef if($self->{'_error'});
	
	my $WhereClause = "WHERE bf_name='$self->{'name'}'";
	my $Results = $self->{'_connection'}->ProcessNewRequest({
			Command => "APIQueryRecords",
			Table => "bf_manifest",
			Fields => "*",
			Clause => $WhereClause,
			Limit => 0,
		});
	my $QueryResults = $self->{'_connection'}->ParseResultsAsArrayHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_manifest")});
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	my $ResHash = {};
	foreach my $ValueSet (@{$QueryResults}) {
		$ResHash->{$ValueSet->{'name'}} = $ValueSet->{'value'};
	}
	return $ResHash;
}
# Always Return Nicely
1;


__END__

=head1 I<NAME>

BuildForge::API::Server

=head1 I<SYNOPSIS>

	my $Server = new BuildForge::API::Server();
	$Server->{'name'} = "My Build Server";
	$Server->{'hostname'} = "build.buildforge.com";
	$Server->{'authid'} = "MyAuthId";
	$Server->{'collector'} = "MyCollector";
	$Server->Create();

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been
instantiated. It is used to create new BuildForge servers, and to access
existing server definitions.

=head1 I<METHODS>

=head2 new({ServerParamsHashRef})

Create an object to access existing server information or to add a new server
to the BuildForge console. 

=over 8

=item I<name>

The name for the server

=item I<hostname>

The host or address for the server

=item I<access_group_id>

ID of the access level for this server

=item I<root_path>

The root directory for this servers step activity

=item I<load_percent>

Load balancing number (1 to 100)

=item I<transfer_mode>

Allowed file access ("none","put","get","all")

=item I<environment_id>

ID of the environment group to use (integer)

=item I<authid>

The Authentication combo-name to use with this server to log onto the host

=item I<collector>

The collector name that identifies what information to gather about this server to use for steps server selection

=back

=head2 Create()

Create a new BuildForge Server. Requires server 'name', and 'hostname', 'authid', and 'collector' to be set.

=head2 Delete()

Deletes the server. Requires the server 'name' to be set.

=head2 Enable()

Enables the server. Requires the server 'name' to be set.

=head2 Disable()

Disables the server. Requires the server 'name' to be set.

=head2 IncrementJobCount()

Increases the Job count for a server. Requires the server 'name' to be set.

=head2 DecrementJobCount()

Decreases the Job count for a server - does not decrease below 0. Requires the
server 'name' to be set.

=head2 Manifest()

Returns a hash-ref containing name/value pairs that are in the servers current manifest.

=head2 Sync()

Syncs the server object with information from the console database. Requires
the server 'name' to be set.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
