# package BuildForge::API::Step
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::Step
package BuildForge::API::Step;

use BuildForge::Connection::Encoded;
use BuildForge::API::Environ;
use BuildForge::API::Class;
use BuildForge::API::Access;
use BuildForge::NameMap;
use BuildForge::API::Interface;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	
	$self->{'_public'}->{'pass_wait'}=0;
	$self->{'_public'}->{'fail_wait'}=0;
	$self->{'_public'}->{'pass_notify_changers'}=0;
	$self->{'_public'}->{'fail_notify_changers'}=0;
	$self->{'step_uid'} = "";
	$self->{'environment_id'} = new BuildForge::API::Environ();
	$self->{'filter_id'} = 0;
	$self->{'inline_chain_id'} = 0;
	$self->{'dir'} = "/";
	$self->{'pass_notify_group'} = "0";
	$self->{'fail_notify_group'} = "0";
	$self->{'onfail'} = 0;
	$self->{'selector'} = "";
	$self->{'threading'} = "N";
	$self->{'absolute'} = 0;
	$self->{'timeout'} = 300;
	$self->{'inline'} = 0;
	$self->{'access_group_id'} = new BuildForge::API::Access();
	$self->{'broadcast'} = 0;
	$self->{'pass_chain_id'} = 0;
	$self->{'fail_chain_id'} = 0;
	$self->{'set_default_level'} = 0;
	
	while(my($variable,$value) = each %{$Params}) {
		$self->{$variable} = $value;
	}
	return $self;
}

sub Details {
}

sub STORE {
	my($self,$key,$value) = @_;
	if(exists($self->{'_public'}->{$key})) {
		if($key eq "broadcast") {
			$self->{'_public'}->{$key} = ($value eq "Y") ? 1 : 0;
			return;
		}
		if($key eq "onfail") {
			$self->{'_public'}->{$key} = ($value eq "H" || $value == 1) ? 1 : 0;
			return;
		}
	}
	if($key eq "server") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key 'server' is no longer used, please use 'selector' instead.";
		return;
	}
	$self->SUPER::STORE($key,$value);
}

sub FETCH {
	my($self,$key) = @_;
	if($key eq "server") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key 'server' is no longer used, please use 'selector' instead.";
		return;
	}
	$self->SUPER::FETCH($key);
}


sub Sync {
	my($self) = @_;
	if(!$self->{'step_id'} && !$self->{'title'} && !$self->{'step_uid'}) {
		$self->{'_error'} = 795;
		$self->{'info'} = "Can't sync step without a uid, an id or description";
		return $self->{'_error'};
	}
	if(!$self->{'project_id'}) {
		$self->{'_error'} = 796;
		$self->{'info'} = "Can't sync step without an associated project";
		return $self->{'_error'};
	}
	my $Clause = "WHERE bf_pid=$self->{'project_id'}";
	if($self->{'title'}) {
		$Clause .= " AND bf_description='$self->{'title'}'";
	}
	elsif($self->{'step_id'}) {
		$Clause .= " AND bf_sid=$self->{'step_id'}";
	}
	elsif($self->{'step_uid'}) {
		$Clause .= " AND bf_uid=$self->{'step_uid'}";
	}	
	else {
		$self->{'_error'} = 797;
		$self->{'info'} = "Impossible condition met";
		return $self->{'_error'};
	}
	my $QueryResults = $self->ProcessForHash({
			Command=>"APIQueryRecords",
			Table=>"bf_steps",
			Fields=>"*",
			Clause=>$Clause,
			Limit => 1,
		},"bf_steps");
	map { $self->{$_} = $QueryResults->{$_} unless($_=~/^_/)} keys(%{$QueryResults});

	if(!defined($QueryResults->{'_record'})) {
		$self->{'_error'} = 798;
		$self->{'_info'} = "Did not find any matching steps.";
	}	
	return $self->GetLastError();
}

sub Create {
	my ($self) = @_;
	$self->{'_error'} = 790 unless($self->{'title'});
	$self->{'_error'} = 791 unless($self->{'project_id'});
	$self->{'_error'} = 792 unless($self->{'command'});
			
	return $self->{'_error'} if($self->{'_error'});
	
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddProjectStep");
	# Add some missing things...
	$RequestContent->{'absolute'}="absolute";
	$RequestContent->{'Level'}="access_group_id";
	$RequestContent->{'EID'}="environment_id";
	$RequestContent->{'DID'}="filter_id";
	
	while(my ($Param,$Var) = each %{$RequestContent}) {
		# Checkboxes first
		if($Var eq "pass_wait") {
			$RequestContent->{$Param}=($self->{'pass_wait'} ? "on" : "off");
		}
		elsif($Var eq "onfail") {
			$RequestContent->{$Param}=($self->{'onfail'} ? "on" : "off");
		}
		elsif($Var eq "broadcast") {
			$RequestContent->{$Param}=($self->{'broadcast'} ? "on" : "off");
		}
		elsif($Var eq "pass_notify_changers") {
			$RequestContent->{$Param}=($self->{'pass_notify_changers'} ? "on" : "off");
		}
		elsif($Var eq "fail_notify_changers") {
			$RequestContent->{$Param}=($self->{'fail_notify_changers'} ? "on" : "off");
		}
		elsif($Var eq "pass_wait") {
			$RequestContent->{$Param}=($self->{'pass_wait'} ? "on" : "off");
		}
		elsif($Var eq "fail_wait") {
			$RequestContent->{$Param}=($self->{'fail_wait'} ? "on" : "off");
		}
		elsif($Var eq "absolute") {
			$RequestContent->{$Param}=($self->{'absolute'} ? "on" : "off");
		}
		elsif($Var eq "inline") {
			$RequestContent->{$Param} = ($self->{'inline'} ? "Y" : "N");
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}

	$RequestContent->{'Command'} = "AddProjectStep";

	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);

	my $Results = $self->{'_connection'}->ProcessRequest($Request);

	if ($self->{'_error'} = $self->{'_connection'}->GetLastError()) {
		$self->{'_info'} = $self->{'_connection'}->GetLastMessage();
	} else {	
		my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_steps")});
		#	$self->Details();
		$self->{'_info'}=$QueryResults->{'_info'};
	}	
	return $self->{'_error'};		
}

sub Update {
	my($self,$Params) = @_;
	$self->{'_error'} = 795 unless($self->{'title'});
	$self->{'_error'} = 796 unless($self->{'project_id'});
	$self->{'_error'} = 797 unless($self->{'command'});
	return $self->{'_error'} if($self->{'_error'});
	
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("UpdateProjectStep");
	
	while(my ($Param,$Var) = each %{$RequestContent}) {
		# Checkboxes first
		if($Var eq "pass_wait") {
			$RequestContent->{$Param}=($self->{'pass_wait'} ? "on" : "off");
		}
		elsif($Var eq "onfail") {
			$RequestContent->{$Param}=($self->{'onfail'} ? "on" : "off");
		}
		elsif($Var eq "broadcast") {
			$RequestContent->{$Param}=($self->{'broadcast'} ? "on" : "off");
		}
		elsif($Var eq "pass_notify_changers") {
			$RequestContent->{$Param}=($self->{'pass_notify_changers'} ? "on" : "off");
		}
		elsif($Var eq "fail_notify_changers") {
			$RequestContent->{$Param}=($self->{'fail_notify_changers'} ? "on" : "off");
		}
		elsif($Var eq "pass_wait") {
			$RequestContent->{$Param}=($self->{'pass_wait'} ? "on" : "off");
		}
		elsif($Var eq "fail_wait") {
			$RequestContent->{$Param}=($self->{'fail_wait'} ? "on" : "off");
		}
		elsif($Var eq "absolute") {
			$RequestContent->{$Param}=($self->{'absolute'} ? "on" : "off");
		}
		elsif($Var eq "inline") {
			$RequestContent->{$Param} = ($self->{'inline'} ? "Y" : "N");
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	# Handle the move to stuff here...
	my $MoveTo = $Params->{'MoveTo'};
	if(ref($MoveTo) eq "BuildForge::API::Step") {
		$RequestContent->{'Location'} = $MoveTo->{'step_id'};
	}
	elsif(defined($MoveTo)) {
		$RequestContent->{'Location'} = $MoveTo;
	}
	else {
		$RequestContent->{'Location'} = $self->{'step_id'};
	}
	$RequestContent->{'Command'} = "UpdateProjectStep";
	
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_steps")});
	$self->{'_info'}=$QueryResults->{'_info'};
	return $self->GetLastError();		
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 793 unless($self->{'title'});
	$self->{'_error'} = 794 unless($self->{'project_id'});
	return $self->{'_error'} if($self->{'_error'});
	$self->Sync();
	return $self->{'_error'} if($self->{'_error'});

	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteProjectStep");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteProjectStep";
	$self->ProcessForHash($RequestContent,"bf_steps");
	return $self->GetLastError();
}

sub Enable {
	my($self) = @_;
	$self->{'_error'} = 799 unless($self->{'title'});
	
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("EnableStep");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "ActivateProjectStep";
	$RequestContent->{'Mode'} = "On";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_steps");
	return $self->GetLastError();
}

sub Disable {
	my($self) = @_;
	$self->{'_error'} = 800 unless($self->{'title'});
	
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DisableStep");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "ActivateProjectStep";
	$RequestContent->{'Mode'} = "Off";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_steps");
	return $self->GetLastError();
}
# Always Return Nicely
1;


__END__


=head1 I<NAME>

BuildForge::API::Step

=head1 I<SYNOPSIS>

	my $Step = new BuildForge::API::Step();
	$Step->{'title'} = "My Build Step";
	$Step->{'command'} = "make mysteptarget";
	$Project->AddStep($Step);

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been 
instantiated. Typically Step objects are used in conjunction with a Project
object. The BuildForge API allows for creation, modification, and deletion of
Project Steps.

=head1 I<METHODS>

=head2 new({StepParamsHashRef})

Create an object to access and modify existing step information, or to create
new steps.

=over 8

=item I<title>

The name of the step.

=item I<command>

The command to execute for this step.

=item I<environment_id>

ID of the environment group to use (integer)

=item I<filter_id>

ID of the filter set to use

=item I<inline_chain_id>

Project ID of the project to inline chain

=item I<dir>

Sub-path to run the command in.

=item I<pass_notify_group>

Group name to notify upon step pass

=item I<fail_notify_group>

Group name to notify upon step failure

=item I<onfail>

Continue on fail status, if filter_id is not set (0 or 1)

=item I<selector>

Name of the selector to use with this step.

=item I<threading>

Thread this step ('Y' or 'N' or 'J')

=item I<absolute>

This step should not create/use project dir (0 or 1)

=item I<timeout>

Step timeout (string)

=item I<access_group_id>

ID of the access level for this step

=item I<broadcast>

Broadcast step to pool (0 or 1)

=item I<pass_chain_id>

Project ID to chain on pass

=item I<fail_chain_id>

Project ID to chain on fail

=item I<pass_wait>

Wait for Chained Pass Project to complete (0 or 1)

=item I<fail_wait>

Wait for Chained Fail Project to complete (0 or 1)

=item I<pass_notify_changers>

Notify changeset info users on pass (0 or 1)

=item I<fail_notify_changers>

Notify changeset info users on fail (0 or 1)

=back

=head2 GetLastError()

Returns the last error code encountered by the object.

=head2 GetLastMessage()

Returns the last message string encountered by the object.

=head2 Sync()

Populates the object with information from the console database. The step
object should be instantiated with one of the hash refs returned from the
StepList method of a Project object.

=head2 Enable()

Enables the step.

=head2 Disable()

Disables the step.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
