# package BuildForge::API::TagVar
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::TagVar
package BuildForge::API::TagVar;

use BuildForge::Connection::Encoded;
use BuildForge::NameMap;
use BuildForge::API::Interface;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
	
	my $self= $class->SUPER::new(@_);
	
	$self->{'name'} = "";
	$self->{'value'} = 1;
	$self->{'auto_increment'} = "Y";
	$self->{'project_id'} = 0;
	$self->{'tag_id'} = 0;
	$self->{'width'} = 0;
	
	while(my($variable,$value) = each %{$Params}) {
		$self->{$variable} = $value;
	}
	return $self;
}

sub Sync {
	my($self) = @_;
	$self->{'_error'} = 912 unless($self->{'project_id'});
	$self->{'_error'} = 913 unless($self->{'name'} || $self->{'tag_id'});
	return undef if($self->{'_error'});
	
	my $WhereClause = "";
	if($self->{'tag_id'}) {
		$WhereClause = "WHERE bf_tid=$self->{'tag_id'} AND bf_pid=$self->{'project_id'}";
	}
	else {
		$WhereClause = "WHERE bf_name='$self->{'name'}' AND bf_pid=$self->{'project_id'}";
	}
	my $QueryResults = $self->ProcessForHash({
			Command => "APIQueryRecords",
			Table => "bf_tags",
			Fields => "*",
			Clause => $WhereClause,
			Limit => 1,
		},"bf_tags");

	if(!defined($QueryResults->{'_record'})) {
		$self->{'_error'} = 914;
		return $self->{'_error'};
	}	
		
	return $self;
}

sub Delete {
	my($self) = @_;
	$self->Sync();
	$self->{'_error'} = 916 unless($self->{'project_id'});
	$self->{'_error'} = 915 unless($self->{'tag_id'});
	
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteProjectTagVar");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteProjectTagVar";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_tags");
	return $self->GetLastError();
}

sub Create {
	my($self) = @_;
	$self->{'_error'} = 910 unless($self->{'name'});
	$self->{'_error'} = 911 unless($self->{'project_id'});
	
	return $self->{'_error'} if($self->{'_error'});
	
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddProjectTagVar");
	while(my($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "AddProjectTagVar";
	my $QueryResults = $self->ProcessForHash($RequestContent,"bf_tags");
	return $self->GetLastError();
}

# Always Return Nicely;
1;

__END__

=head1 I<NAME>

BuildForge::API::TagVar

=head1 I<SYNOPSIS>

	my $TagVar = new BuildForge::API::TagVar();
	$TagVar->{'name'} = "Patch";
	$TagVar->{'auto_increment'} = "N";
	$Project->AddTagVar($TagVar);

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been
instantiated. It is used to create and delete tag variables defined for 
projects inside the BuildForge console. The initial value of a variable is '1'.

=head1 I<METHODS>

=head2 new({TagVarParamsHashRef})

Create an ojbect to access tag variables from a BuildForge project.

=over 8

=item I<name>

The name for this tag variable

=item I<project_id>

The id of the associated project

=item I<value>

The initial value for this variable

=item I<auto_increment>

(Y/N) Should variable increase each build

=back

=head2 Sync()

Syncs the object with information from the console database. Requires the 
objects 'project_id' variable to be set, along with either the objects 'tag_id'
or 'name' property. 

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
