# package BuildForge::API::User
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other intellectual property
# rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::API::User
package BuildForge::API::User;

use BuildForge::Connection::Encoded;
use BuildForge::API::Interface;
use BuildForge::NameMap;
use BuildForge::API::Access;

use strict;

use vars qw (@ISA);

@ISA = qw(BuildForge::API::Interface);
# Instantiation
sub new {
	my $class = shift;
	my ($Params) = @_;
 	
	my $self= $class->SUPER::new(@_);
	$self->{'user_id'} = 0;
	$self->{'name'} = "";
	$self->{'access_group_id'} = new BuildForge::API::Access();
	$self->{'email_address'} = "";
	$self->{'login'} = "";
	$self->{'password'} = "";
	$self->{'build_limit'} = 0;
	$self->{'password_expires'} = 1;
	$self->{'password_expired'} = 0;
	$self->{'time_zone'} = "EDT";
	$self->{'date_format'} = "%d %b %y %H:%M";
	$self->{'priority_login'} = 0;
	$self->{'show_quick_start_menu'} = 0;
	$self->{'lang'} = "en_US";
	$self->{'allowed_access'} = [1,2,3];	
	
	while(my($variable,$value) = each %{$Params}) {
		$self->{$variable} = $value;
	}
	return $self;
}

sub STORE {
	my($self,$key,$value) = @_;
	if($key eq "theme") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key 'theme' is no longer used in this version of the Application.";
		return;
	}
	$self->SUPER::STORE($key,$value);
}

sub FETCH {
	my($self,$key) = @_;
	if($key eq "theme") {
		$self->{'_error'} = 999;
		$self->{'_info'} = "Object key 'theme' is no longer used in this version of the Application.";
		return;
	}
	$self->SUPER::FETCH($key);
}

sub Sync {
	my ($self) = @_;
	if(!$self->{'user_id'} && !$self->{'login'} && !$self->{'name'} && !$self->{'email_address'}) {
		$self->{'_error'} = 833;
		return $self;
	}
	my $Clause = "WHERE bf_name='$self->{'name'}'";
	if($self->{'user_id'}>0) {
		$Clause = "WHERE bf_userid=$self->{'user_id'}";
	}
	elsif($self->{'login'}) {
		$Clause = "WHERE bf_login='$self->{'login'}'";
	}
	elsif($self->{'email_address'}) {
		$Clause = "WHERE bf_email='$self->{'email_address'}'";
	}

	my $QueryResults = $self->ProcessForHash({
			Command => "APIQueryRecords",
			Table => "bf_users",
			Fields => "*",
			Clause => $Clause,
			Limit => 1,
		},"bf_users");

	if(!defined($QueryResults->{'_record'})) {
		$self->{'_error'} = 835;
		$self->{'_info'} = "Cannot find any matching user.";
		return $self->{'_error'};
	}	
	return $self;
}

sub Delete {
	my($self) = @_;
	$self->{'_error'} = 833 unless($self->{'name'});
	$self->{'_error'} = 834 unless($self->{'user_id'}>1);
	
	return undef if($self->{'_error'});
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("DeleteUser");
	while(my ($Param,$Var) = each %{$RequestContent}) {
		$RequestContent->{$Param} = $self->{$Var};
	}
	$RequestContent->{'Command'} = "DeleteUser";
	
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_users")});
	$self->{'_info'}= $self->{'_connection'}->GetLastMessage();
	return $self;
}

sub Create {
	my($self) = @_;
	$self->{'_error'} = 830 unless($self->{'name'});
	$self->{'_error'} = 831 unless($self->{'login'});
	$self->{'_error'} = 832 unless($self->{'password'});
	
	return undef if($self->{'_error'});
	
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	my $RequestContent = $self->{'_namemap'}->FunctionParams("AddUser");
	$RequestContent->{'confirm'} = "password";
	$RequestContent->{'Access[]'} = "allowed_access";
	
	while(my ($Param,$Var) = each %{$RequestContent}) {
		if($Var eq "password_expires") {
			$RequestContent->{$Param}=($self->{$Var} ? "on" : "off");
		}
		elsif($Var eq "priority_login") {
			$RequestContent->{$Param}=($self->{$Var} ? "on" : "off");
		}
		else {
			$RequestContent->{$Param} = $self->{$Var};
		}
	}
	$RequestContent->{'Command'} = "AddUser";
	
	$self->{'_connection'}->AddRequestContent($Request,$RequestContent);
	
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results,'NameMap'=>$self->{'_namemap'}->GetTableTrans("bf_users")});
	$self->{'_info'}= $self->{'_connection'}->GetLastMessage();
	return $self;		
}

# Always Return Nicely
1;

__END__

=head1 I<NAME>

BuildForge::API::User

=head1 I<SYNOPSIS>

	my $User = new BuildForge::API::User();
	$User->{'name'} = "The API User";
	$User->{'login'} = "apiuser";
	$User->{'password'} = "apiuser";
	$User->Create();

=head1 I<DESCRIPTION>

    This is available to use after a BuildForge::API object has been
instantiated. It is used to create and delete users defined in the BuildForge
console. 

=head1 I<METHODS>

=head2 new({UserParamsHashRef})

Create an object to access existing user information, or to add a new user
to the BuildForge console.

=over 8

=item I<name>

The long name for the user

=item I<login>

The login name for the user

=item I<password>

The password for the user

=item I<build_limit>

Maximum number of builds aloowed to run per day

=item I<email_address>

The email address for the user

=item I<time_zone>

Time zone code (EDT,EST,CDT,... etc.)

=item I<date_format>

Date display format (e.g., "%d %b %y %H:%M")

=item I<password_expires>

User password will expire

=item I<priority_login>

User can kick other users off system for license

=item I<allowed_access>

Access group ids for the user (array reference)

=back

=head2 Create()

Create a new BuildForge user. Requires the objects 'name', 'login', and 
'password' to be set.

=head2 Sync()

Syncs the user object with the corresponding user data from the database. The 
user object must have any of 'name', 'login', 'user_id' or 'email_address' for
this method to succeed.

=head2 Delete()

Deletes a user account, will not allow you to delete userid 1 (root). Requires
'name' or 'userid' variables to be set.

=head1 I<AUTHOR>

BuildForge, Inc.

=cut
