# package BuildForge::NameMap
#
# Copyright 2003-2007, IBM, Inc. All rights reserved.
# IBM owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and IBM.

# BuildForge::NameMap
package BuildForge::NameMap;

use Exporter;
use BuildForge::Connection::Encoded;

use strict;

use vars qw(@ISA @EXPORT);

@ISA = qw(Exporter);

@EXPORT = qw(Table);
# Make this a singleton for now...
{
	my $object_exists=0;
	my $object_ref=undef;
	sub exists {$object_exists}
	sub create_static {$object_exists++}
	sub destroy_static {$object_exists--}
	sub get_static {$object_ref}
	sub set_static {$object_ref=$_[1]}
	sub Table {$object_ref->{'_mapout'}->{$_[1]}};
}

sub DESTROY {
	my ($self) = @_;
	my $class = ref($self);
	if($class->exists()) {
		$class->set_static(undef);
		$class->destroy_static();
	}
}
	
sub new {
	my($class,$Params) = @_;
	if($ENV{'BF_API_DEBUG'}) {
		my($Pack,$File,$Line) = caller();
		print "NameMap Reference Requested from $Pack($Line:$File)\n";
	}
	if($class->exists()) {
		return $class->get_static();
	}

	my $self = {
		'_connection'=>new BuildForge::Connection::Encoded(),
		'_mapin'=>undef,
		'_mapout'=>undef,
		'_error'=>undef,
		'_class_translation' => {
			'BuildForge::API::Project'	=> 'bf_projects',
			'BuildForge::API::Environ'	=> 'bf_env',
			'BuildForge::API::Class'	=> 'bf_classes',
			'BuildForge::API::Access'	=> 'bf_access',
			'BuildForge::API::Step'		=> 'bf_steps',
			'BuildForge::API::Server'	=> 'bf_servers',
			'BuildForge::API::User'		=> 'bf_users',
			'BuildForge::API::Build'	=> 'bf_builds',
			'BuildForge::API::TagVar'	=> 'bf_tags',
			'BuildForge::API::Schedule'	=> 'bf_cron',
			'BuildForge::API::Register'	=> 'bf_data',
			'BuildForge::API::Adaptor'	=> 'bf_interfaces',
			'BuildForge::API::Collector' => 'bf_collector',
			'BuildForge::API::Selector' => 'bf_selector',
			'BuildForge::API::Auth'		=> 'bf_serverauth',
		},
	};
	
	bless $self,$class || ref($class);
	
	$self->GetMappings();

	$class->create_static();
	$class->set_static($self);
	
	return $self;
}

sub TearDown {
	my ($self) = @_;
	my $class = ref($self);
	$self->{'_connection'} = undef;
	if($class->exists()) {
		$class->set_static(undef);
		$class->destroy_static();
	}
}


sub GetLastError {
	my ($self) = @_;
	return $self->{'_error'};
}

sub GetTableTrans {
	my($self,$Table) = @_;
	return $self->{'_mapout'}->{$Table}
}

sub PublicNames {
	my($self,$class) = @_;
	my $TableName = $self->{'_class_translation'}->{$class};
	my $RetHash = {};
	while(my ($Col,$Name) = each %{$self->{'_mapout'}->{$TableName}}) {
		$RetHash->{$Name}="";
	}
	return $RetHash;
}

sub FunctionParams {
	my($self,$Function) = @_;
	my $RetHash = {};
	while(my ($Var,$Param) = each %{$self->{'_mapin'}->{$Function}}) {
		$RetHash->{$Param}=$Var;
	}
	return $RetHash;
}

sub GetMappings {		
	my ($self) = @_;
	my $Request=$self->{'_connection'}->NewDefaultRequest();
	$self->{'_connection'}->AddRequestContent($Request,{'Command' => "APIGetNameMapOut"});
	my $Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	my $QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results});
	# Now update our object for each key...
	while (my ($Field,$Value) = each %{$QueryResults}) {
		my ($Table,$Column) = split(/:/,$Field);
		$self->{'_mapout'}->{$Table}->{$Column}=$Value;
	}
	return if($self->{'_error'});

	$Request=$self->{'_connection'}->NewDefaultRequest();
	$self->{'_connection'}->AddRequestContent($Request,{'Command' => "APIGetNameMapIn"});
	$Results = $self->{'_connection'}->ProcessRequest($Request);
	$self->{'_error'} = $self->{'_connection'}->GetLastError();
	$QueryResults = $self->{'_connection'}->ParseResultsAsHash({'Results'=>$Results});
	# Now update our object for each key...
	while (my ($Field,$Param) = each %{$QueryResults}) {
		my ($Method,$Name) = split(/:/,$Field);
		$self->{'_mapin'}->{$Method}->{$Name}=$Param;
	}
}

1;
