<?php

//
// Copyright ©2003–2006, BuildForge, Inc.  All rights reserved.
// BuildForge, Inc. owns the copyright and other intellectual property
// rights in this software.  Duplication or use of the Software is not
// permitted except as expressly provided in a written agreement between
// your company and BuildForge, Inc.
//

$fails = 0;

//================================================================
// UTILITY FUNCTIONS
//================================================================

function check_include($filename, $testclass, $nofail = false) {
	global $fails;

	echo "  Checking for PHP include $filename ... ";
	@include($filename);
	if (class_exists($testclass)) {
		echo "found\n";
		return true;
	}

	echo "MISSING\n";
	return failed($nofail);
}

function check_extension($extension, $nofail = false) {
	global $fails;

	echo "  Checking for PHP extension $extension ... ";
	if (extension_loaded($extension)) {
		echo "built-in\n";
		return true;
	}

	if (strtoupper( substr(PHP_OS, 0, 3) ) === 'WIN') {
		@dl('php_' . $extension . '.dll');
	} else {
		@dl($extension . '.so');
	}
	if (extension_loaded($extension)) {
		echo "dynamic\n";
		return true;
	}

	echo "MISSING\n";
	return failed($nofail);
}

function failed($nofail) {
	global $fails;
	if ($nofail) {
		if ($nofail !== true) {
			echo "\n\tWARNING: $nofail\n";
		}
	} else {
		$fails++;
	}
	return false;
}

function check_db($db, $driver, $alternate = false) {
	echo "\nChecking for database '$db' support in PHP...\n";
	if (check_extension($driver,true)) { return true; }
	if ($alternate && check_extension($alternate,true)) { return true; }
	$fails++;
	return false;
}

//================================================================
// CHECK SEQUENCE
//================================================================

$drivers = array(
		'db2' => array('IBM DB2', 'ibm_db2'),
		'mysql' => array('MySQL', 'mysqli'),
		'sybase' => array('ODBC (for Sybase SQLServer)', 'odbc'),
		'mssql' => array('ODBC (for Microsoft SQLServer)', 'odbc'),
		'oracle' => array('Oracle', 'oci8')
);
$driver = is_null($argv[1]) ? 'db2' : $argv[1];
$modinfo = $drivers[$driver];

if (is_null($modinfo)) {
	echo "\n\tERROR: This script does not recognize database type '$driver'\n";
	$fails++;
} else {
	check_db($modinfo[0], $modinfo[1]);
}

echo "\nChecking for other things that we need...\n";
check_include('PEAR.php', 'PEAR');
check_include('DB/common.php', 'DB_common');
check_extension('ldap', 'This is required for logins to work properly.');
check_extension('mbstring', 'Your console will be slow without this.');
check_extension('shmop', 'Your console will be slow without this.');
check_extension('zip', 'This is required by the Reflector feature.');
echo "\n";

//================================================================
// REPORT RESULTS
//================================================================

if ($fails) {
	print "FAIL $fails\n";
	exit(1);
}
exit(0);
?>
