#!/usr/bin/perl
#
# Copyright ©2003–2007, BuildForge, Inc.  All rights reserved.
# BuildForge, Inc. owns the copyright and other intellectual property
# rights in this software.  Duplication or use of the Software is not
# permitted except as expressly provided in a written agreement between
# your company and BuildForge, Inc.
#

$|++;

use strict;

use CPAN;
use Getopt::Long;

my ($withperl,$PERL,$PHP,$BFDIR,$usecpan,$dbtype,$reexecute);
my ($SKIPDBD,$DBUSER,$DBPASS,$DBHOST,$DBNAME);
my @ARGS=@ARGV;
GetOptions(	"with-perl=s" => \$withperl,
						"with-php=s" => \$PHP,
						"use-CPAN" => \$usecpan,
						"with-dbtype=s" => \$dbtype,
						"with-bfdir=s" => \$BFDIR,
						"skip-DBD" => \$SKIPDBD,
						"reexecute" => \$reexecute
);
if(! $dbtype) {usage();}

my $ERRORS = 0;
my @install_these;
my $cpan_install="n";
my $module;
my $dbdir;
my $execdir=`pwd`;
chomp $execdir;


if($dbtype=~/,/)
{
	($dbtype,$dbdir)=split(/,/,$dbtype,2);
	if($dbtype eq 'oracle')
	{
		$ENV{ORACLE_HOME}=$dbdir unless $ENV{ORACLE_HOME};
		$ENV{LD_LIBRARY_PATH}="$ENV{LD_LIBRARY_PATH}:$dbdir";
		$ENV{TNS_ADMIN}=$dbdir unless $ENV{TNS_ADMIN};
	}
}


# validate correct perl binary 
$PERL=check_perl_bin();


$BFDIR=get_bf_inst_dir($BFDIR);

if($BFDIR =~ /\./)
{
  $BFDIR=$execdir;
}
cleandir("$BFDIR/ui/templates_c");

if($PERL ne "/usr/bin/perl")
{
	frob_magicline($PERL,$BFDIR);
}

# check for needed perl modules;
perl_modules();

# build BuildForge::Key
build_module("$BFDIR/Platform/BuildForge/Key");
# build BFFlex
build_module("$BFDIR/license");

drop_buildforge_conf();

frob_rc_file("$BFDIR/rc/buildforge");

print "\nChecking PHP module and database compliance... \n\n";
check_php_bin();

#check the modules
drop_php_checklist();
my @php=php_checklist($dbtype);
clean_php_checklist();

my ($state,$driver);

print "\n";

my $phpmissing=0;
foreach (@php)
{
	if(/^php:(\w+):(\w+)$/)
	{
		($state,$driver)=($2,$1);
		if($state=~/MISSING/)
		{
			$phpmissing=1;
		}
		print "PHP $driver driver is $state\n";
	}
}

if($phpmissing)
{
	print "\nMissing PHP drivers need to be added to the php.ini as \n";
	print "extensions and the extensions need to be copied into the \n";
	print "php extension_dir specified in the php.ini file.\n";
	print "These extensions can be build from the php source by adding\n";
	print "a configure parameter to add shared DB drivers\n";
	print "(look at the output of ./configure --help in the php sourcedir\n";
	print "for a list of parameters)\n";
	print "\nThe php source can be downloaded from http://php.net/releases\n";
	print "If you are running a unix that has a package manager, check for\n";
	print "packages containing the php module for your database\n\n";
}


print"\n\n";
exit 0;

#================================================================
# UTILITY FUNCTIONS
#================================================================

sub cleandir
{
	unlink(glob("$_[0]/* $_[0]/.*"));
}

sub version_split {
	my ($ver) = @_;
	$ver =~ y/_//d;
	$ver =~ /^\d[\d\.]*\d$/ and return split(/\./, $ver);
	return undef;
}

sub version_test {
	my ($ver1, $args) = @_;
	my ($oper, $ver2) = split(/ /, $args, 2);

	my @ver1 = version_split($ver1);
	return defined(@ver1) ? 1 : 0 if $oper =~ /def(ined)?/;
	return !defined(@ver1) ? 1 : 0 if $oper =~ /undef(ined)?/;

	my @ver2 = version_split($ver2);
	return undef unless defined(@ver1) && defined(@ver2);

	my $cmp = 0;
	while (@ver1 || @ver2) {
		my $p1 = shift @ver1;
		my $p2 = shift @ver2;
		$cmp--, last if $p1 < $p2;
		$cmp++, last if $p1 > $p2;
	}

	return $cmp == 0 ? 1 : 0 if $oper eq '==';
	return $cmp != 0 ? 1 : 0 if $oper eq '!=';
	return $cmp <  0 ? 1 : 0 if $oper eq '<';
	return $cmp <= 0 ? 1 : 0 if $oper eq '<=';
	return $cmp >  0 ? 1 : 0 if $oper eq '>';
	return $cmp >= 0 ? 1 : 0 if $oper eq '>=';

	warn "Invalid oper given to check_version: '$oper'\n";
	return undef;
}

sub check {
	my ($data, $tests) = @_;
	$tests = [ 'defined' ] unless ref($tests) eq 'ARRAY';

	my @errors = ();
	my $fails = 0;
	while (@$tests) {
		my $arg = shift @$tests;
		my $optional = undef;
		my $flag = 'FAIL:';
		if (substr($arg,0,1) eq '?') {
			$optional = 1;
			$flag = 'WARN:';
			$arg = substr($arg, 1);
		}
		if( ! defined $data ) { 
			# undef $data means the module was missing
			$fails++ unless $optional;
		}

		next if version_test($data, $arg);
		push @errors, "$flag Expected $arg\n";
		$fails++ unless $optional;
		next;
	}

	foreach my $error (@errors) { print "\t$error\n"; }
	$ERRORS += $fails;
	return !$fails;
}


#================================================================
# UNIX FUNCTIONS
#================================================================
sub unix_user { return getpwnam($_[0]); }
sub unix_group { return getgrnam($_[0]); }


#================================================================
# PERL FUNCTIONS
#================================================================
sub perl_version {
	my ($checks) = @_;
	print "Perl executing from ... $^X\n";
	print "Checking Perl version ... ";
	my $ver = sprintf("%vd", $^V);
	print "$ver\n";
	return check($ver, $checks);
}

sub perl_module {
	my ($module, $checks) = @_;
	print "Checking Perl module $module ... ";
	eval "use $module;";
	if ($@) {
		print "MISSING\n";
		push(@install_these,$module);
		return check(undef, $checks);
	}
	my $ver = eval "\$${module}::VERSION";
	if ($@) {
		print "found\n";
		push(@install_these,$module);
		return check('', $checks);
	}
	print "$ver\n";
	my $rv= check($ver, $checks);
	if(!$rv)
	{
		push(@install_these,$module);
	}
	return $rv;
}


#================================================================
# PHP FUNCTIONS
#================================================================

sub php_version {
	my ($checks) =@_;

	print "Checking PHP version ... ";
	my ($ver, $maj, $min, $rev);

PHP_VERSION:
	if (open (PHP, "$PHP --version 2>&1 |")) {
		$ver = '';
		while (my $line = <PHP>) {
			if ($line =~ /^PHP ((\d+)\.(\d+)\.(\d+))/) {
				($ver, $maj, $min, $rev) = ($1, $2, $3, $4);
				1 while $line = <PHP>;
			}
		}
		close PHP;
	}

	if (!defined($ver)) {
		print "MISSING\n";
	} elsif (!length($ver)) {
		print "UNKNOWN VERSION\n";
	} #tweaked elsif to allow for 5.1.2-x (still supported) or 5.2.1-x
		elsif  ($maj != 5  || # unsup if major is not v5
						($min != 1 || $rev < 2) && # unsup if minor is 1 and rev is < 2
						($min != 2 || $rev < 1)) { # unsup if minor is 2 and rev is < 1
		print "UNSUPPORTED VERSION: $ver\n";
	} else {
		print "$ver\n";
	}
	return check($ver, $checks);
}

sub php_checklist {
	my $dbtype=shift();
	print "Launching PHP checks ... ";
	if (!open (PHP, "$PHP -q /tmp/checklist.$$.php $dbtype 2>/dev/null |")) {
		print "FAILED\n";
		$ERRORS++;
		return undef;
	}
	print "launched\n";

	my @php=<PHP>;

	close PHP;

	return @php;
}

#===========================================================

sub check_perl_bin
{
	my $PERL=$;
	my $perl_yn;
	if($withperl) 
	{ 
		if($withperl ne $)
		{
			reexec($withperl);
		}
		return $PERL; 
	}
	if($PERL=~/^\//)
	{
		print "The current perl binary is $PERL.  Is this the same perl that\n";
		print "Build Forge will be using?  ";
		$perl_yn=<>;
	}
	else # in the internally indicated perl doesnt start with a '/'
	{		 # then we need to ask the user which perl we should be using
		$perl_yn="n";
	}

	if($perl_yn =~ /^\s*n/i)
	{
		print "\nWhich perl will Build Forge be using? (absolute path please) ";
		$PERL=<>;
		chomp($PERL);
		reexec($PERL);
	}
	print "\n\n";
	return $PERL;
}

sub reexec
{
	my $withperl=shift();
	if(!-x $withperl)
	{
		die "The perl specified ($withperl) is not executable, please doublecheck\n";
	}
	if($reexecute)
	{
		print "I re-executed myself with $withperl, but ended up running $ (probably due \n";
		print "to a symlink somewhere). This shouldn't cause any problems, but make note in case\n";
		print "you have perl module problems later.\n\n\n";
		return 0;
	}	
	print "I'm re-executing myself with that perl now.  Wait one...\n\n";
	exec($withperl,$0,@ARGS,"--with-perl=$withperl","--reexecute");
}

# kind of pointless to seperate this one out, but did it anyway
sub get_bf_inst_dir
{
	my $BFDIR=shift(@_);
	while(not -d $BFDIR)
	{
		print "Where did you unpack the buildforge tar file?  ";
		$BFDIR=<>;
		chomp $BFDIR;
		if(not -d $BFDIR)
		{
			print "\tThe directory \'$BFDIR\' does not exist.\n";
		}
		elsif(not -w "$BFDIR/rc/buildforge")
		{
			print "\tWARNING: $BFDIR/rc/buildforge is not writable by this user.\n";
			print "\tyou may have to re-run the install utility to modify this script\n";
			print "(press enter to continue or CTRL-C to exit)\n\n";
			my $junk=<>;
		}
	}
	return $BFDIR;
}


sub frob_magicline
{
	my ($PERL,$BFDIR)=@_;

	my $p=$PERL;
	chdir "$BFDIR";
	$PERL=~s/\//\\\//g;
	system($p.' -pi -e \'s/\/usr\/bin\/perl/'.$PERL.'/\' '.join(' ',glob("*/*.pl */*/*.pl")));
	chdir $execdir;
}

sub perl_modules
{
	print "\n\n Checking perl modules (ALL are required):\n\n";

	perl_version([ '>= 5.8.0', '< 5.9.0' ]);
	perl_module('Exporter');
	perl_module('LWP::UserAgent');
	perl_module('HTTP::Request');
	perl_module('Config::IniFiles');
	perl_module('Data::Dumper');
	perl_module('Data::UUID');
	perl_module('DBI', [ '>= 1.49' ]);
	perl_module('File::Basename');
	perl_module('File::Path');
	perl_module('File::Temp');
	perl_module('Getopt::Std');
	perl_module('HTML::Template', [ '>= 2.8' ]);
	perl_module('IO::Select');
	perl_module('IO::Socket');
	perl_module('IO::File');
	perl_module('IPC::Open2');
	perl_module('MIME::Lite');
	perl_module('POSIX');
	perl_module('Set::Crontab');
	perl_module('Sys::Hostname');
	perl_module('Text::Balanced');
	perl_module('Time::Local');
	perl_module('Time::Piece');
	perl_module('XML::Writer', ['>= 0.600']);
	perl_module('XML::Writer::String');
	perl_module('XML::XPath', [ '>= 1.13' ]);
	if(not $SKIPDBD)
	{
		if($dbtype eq "db2")
		{
			print "DB2: ";
			perl_module('DBD::DB2');
		} 
		elsif($dbtype eq "mysql")
		{
			print "MySQL: ";
			perl_module('DBD::mysql');
		} 
		elsif($dbtype eq "oracle")
		{
			print "Oracle: ";
			my $rc=perl_module('DBD::Oracle');
			if ($rc != 1)
			{
				print "\tthis may be due to an incorrect setting of \$ORACLE_HOME\n";
				print "\t\$ORACLE_HOME is currently set to $ENV{ORACLE_HOME}\n";
				print "\tThis environment variable will have to be set to the correct location\n";
				print "\tfor perl to properly load the DBD::Oracle module and the engine to function.\n";
				print "\tIf you know that DBD::Oracle is installed, add the --skip-DBD flag\n";
				print "\tto the bfinstall.pl execution\n\n";
			}
		} 
		elsif($dbtype eq "odbc")
		{
			print "ODBC (Sybase and MSSQL): ";
			perl_module('DBD::ODBC');
		}
	}
	
	if($#install_these >=0)
	{
		print "The following perl modules are missing or need to be updated.\n  ";
		print join("\n  ",@install_these);
		if($usecpan)
		{
			$cpan_install="y";
			print "Using CPAN to install these modules\n";
		}
		else
		{
			print "\nWould you like me to try to do that now using the perl CPAN module\n";
			print "installation service? ";
			$cpan_install=<>;
		}
		if($cpan_install=~/^\s*y/i)
		{
			print "If this is the first time you've used CPAN, it will ask you\n";
			print "some configuration questions\n";
			print "\nYou should answer yes to any pre-requisites.  If you are unsure\n";
			print "about any of these questions, you should contact your System Admin.\n";
			my $cpanobj;
			foreach $module (@install_these)
			{
				$cpanobj=CPAN::Shell->expand('Module',$module);
				if($module=~/DBD/)
				{
					#force install
					$cpanobj->force('install');
					$cpanobj->install;
				}
				elsif($module=~/XML::XPath/)
				{
					#call a shell to install these because the CPAN programming interface 
					#doesn't like to follow pre-reqs for them for some silly reason.
					system("$ -MCPAN -e 'install $module'");
				}
				elsif($module=~/(LWP|HTTP::Request)/)
				{
					# these are both LWP modules
					system("echo force install Bundle::LWP|$^X -MCPAN -e 'shell'");
				}
				else
				{
					$cpanobj->install;
				}
			}
		}
		else
		{
			print "You can use CPAN to install these modules all yourself with the\n";
			print "following commandlines, or you can download them from\n";
			print	"http://cpan.org/ and install them manually\n\n";
			foreach $module (@install_these)
			{
				print "\t$ -MCPAN -e 'install $module'\n";
			}
			print "\nOnce those modules are installed, please re-run this installer.\n";
			exit 1;
		}
	}
}

sub check_php_bin
{
	if(! $PHP)
	{
		$PHP=`which php 2> /dev/null`;
		if($?!=0) #php not found
		{
			$PHP=0;
		}
		chomp($PHP);
		
		my $php_yn;
		if(not $PHP =~ /php/)
		{
			print "php was not found.";
			$php_yn="no";
		}
		else
		{
			print "Is the php being used for the web server $PHP? ";
			$php_yn=<>;
		}
		if($php_yn =~ /^\s*n/i)
		{
			print "\nWhich php is being used for the web server? ";
			$PHP=<>;
			chomp($PHP);
		}
	}
	php_version([ '>= 5.0.3' ]);
}


sub usage
{
	print "\nusage: $0 --with-dbtype=<DBTYPE>[,DBlibdir]\n";
	print "\t[--with-perl=<path to perl>][--with-php=<path to php>]\n";
	print "\t[--use-CPAN][--with-bfdir=<Build Forge InstallDir>]\n";
	print "\t[--skip-DBD]\n";
	print "\n\tDBTYPE=mysql|db2|oracle|odbc\n\n\n";
	print "example:\n\t $0 --with-dbtype=oracle,/usr/local/orahome\n\n";
	print "Note: the flags --with-perl, --with-php, --use-CPAN, and\n"; 
	print "--with-bfdir are all intended to automate installation and make it\n";
	print "less interactive they are entirely optional and if omitted the\n";
	print "installer will ask questions to get the information it needs.\n";
	print "\nHowever, dbtype must be specified on the commandline if you wish\n";
	print "to check your configuration\n";

	exit 1;
}

sub drop_buildforge_conf
{
	my $conffile="$BFDIR/Platform/buildforge.conf";
	if( -f $conffile )
	{
		print "buildforge.conf exists and looks like this:\n";
		open (F,$conffile) or die "couldn't open $conffile  $!\n";
		print <F>;
		close(F);
		return 0;
	}
	if(! $DBHOST)
	{
		print "What host is housing the DB used for buildforge? ";
		$DBHOST=<>;
		chomp $DBHOST;
	}
	if(! $DBNAME)
	{
		print "What is the name of the database for buildforge? ";
		$DBNAME=<>;
		chomp $DBNAME;
	}
	if(! $DBUSER)
	{
		print "What is the name of the database user for buildforge? ";
		$DBUSER=<>;
		chomp $DBUSER;
	}
	if(! $DBPASS)
	{
		print "What is the password of the database user for buildforge? ";
		$DBPASS=<>;
		chomp $DBPASS;
	}
	my $db=(split(/\W+/,$dbtype,1))[0];
	open (F,">$conffile") or die "couldn't create $conffile  $!\n";
	print F "db_hostname $DBHOST\n";
	print F "db_database $DBNAME\n";
	print F "db_username $DBUSER\n";
	print F "db_password $DBPASS\n";
	print F "db_type odbc\n";
	print F "db_provider $db\n";
	print F "gdd none\n";
	close(F);
}

sub frob_rc_file
{
	my $file=shift();
	my $bfdir=$BFDIR;
	my $p=$PERL;
	#remove double slashes
	$file=~s/\/\//\//;
	# add backslash quotes
	$PERL=~s/\//\\\//g;
	$bfdir=~s/\/\//\//;
	$bfdir=~s/\//\\\//g;
	system("$p -pi.orig -e's/^INSTALLDIR=.*/INSTALLDIR=$bfdir/; s/^PERL=.*/PERL=$PERL/' $file");
}


sub clean_php_checklist
{
	unlink("/tmp/checklist.$$.php");
}

sub build_module
{
	my $dir=shift();
	#chdir("$BFDIR/Platform/BuildForge/Key");
	chdir($dir);
	my $output=`$PERL Makefile.PL`;
	$output.=`make && make install`;

}

sub drop_php_checklist
{
	open(TMPFILE,">/tmp/checklist.$$.php") or die "couldn't create file /tmp/checklist.$$.php  $!\n";
print TMPFILE q|

<?php

//
// Copyright ©2003–2006, BuildForge, Inc.  All rights reserved.
// BuildForge, Inc. owns the copyright and other intellectual property
// rights in this software.  Duplication or use of the Software is not
// permitted except as expressly provided in a written agreement between
// your company and BuildForge, Inc.
//

$fails = 0;

//================================================================
// UTILITY FUNCTIONS
//================================================================

function check_include($filename, $testclass, $nofail = false) {
	global $fails;

	echo "  Checking for PHP include $filename ... ";
	@include($filename);
	if (class_exists($testclass)) {
		echo "found\n";
		return true;
	}

	return failed($nofail);
}

function check_extension($extension, $nofail = false) {
	global $fails;

	if (extension_loaded($extension)) {
		echo "built-in\n";
		return true;
	}

	if (strtoupper( substr(PHP_OS, 0, 3) ) === 'WIN') {
		@dl('php_' . $extension . '.dll');
	} else {
		@dl($extension . '.so');
	}
	if (extension_loaded($extension)) {
		echo "dynamic\n";
		return true;
	}

	return failed($nofail);
}

function failed($nofail) {
	global $fails;
	if ($nofail) {
		if ($nofail != true) {
			echo "\n\tWARNING: $nofail\n";
		}
	} else {
		$fails++;
	}
	return false;
}

function check_db($db, $driver, $alternate = false) {
	$found="MISSING";
	if (check_extension($driver,true) or
			($alternate && check_extension($alternate,true)) )
	{
		$found="INSTALLED";
	}
	echo "php:$driver:$found\n";
	return false;
}

//================================================================
// CHECK SEQUENCE
//================================================================

print_r($argv);

if($argv[1] == "db2")
{
	check_db('IBM DB2', 'ibm_db2') ;
}
elseif($argv[1] == "mysql")
{
	check_db('MySQL', 'mysql') ;
	check_db('MySQL', 'mysqli') ;
}
elseif($argv[1] == "odbc")
{
	check_db('ODBC (Sybase or Microsoft SQLServer)', 'odbc') ;
}
elseif($argv[1] == "oracle")
{
	check_db('Oracle', 'oci8');
}
else
{
	echo "\n\tERROR: No database drivers were selected!\n";
	echo "\t(you may have a cgi-enabled version of php installed)\n";
}

echo "\nChecking for other things that we need...\n";
check_include('PEAR.php', 'PEAR');
check_include('DB/common.php', 'DB_common');
check_extension('ldap', 'This is required for logins to work properly');
check_extension('zip', 'This is required by the Reflector feature.');
echo "\n";

//================================================================
// REPORT RESULTS
//================================================================

if ($fails) {
	print "FAIL $fails\n";
	exit(1);
}
exit(0);
?>
|; # right
close(TMPFILE);
}
