#!/usr/bin/perl

#
# Copyright (c)2003-2006, BuildForge, Inc.  All rights reserved.
# BuildForge, Inc. owns the copyright and other intellectual property
# rights in this software.  Duplication or use of the Software is not
# permitted except as expressly provided in a written agreement between
# your company and BuildForge, Inc.
#


use ExtUtils::MakeMaker;
use FindBin;
use Config;

# Global Defs
my $FlexVersion = "v7.0f";

my $Plat = $^O;
print "Plat: [$Plat]\n";
print "Base: [$FindBin::Bin]\n";
my $FLEXLIB = 'bfflexs$(OBJ_EXT)';
$FLEXLIB .= '-$(OSNAME)' if $Config{'obj_ext'} eq '.o';

# If we have the C source file, then we can also supply an explicit
# rule for building the key library from the source...
sub MY::postamble {
	return undef unless -f 'bfflexs.c';
	return "\n"
		. "\n"
		. "$FLEXLIB: bfflexs.c\n"
		. "\t\$(CCCMD) \$(CCCDLFLAGS) \"-I\$(PERL_INC)\" \$(PASTHRU_DEFINE) \$(DEFINE) -o \$@ bfflexs.c\n"
		. "\n";
}



$BuildConfig = {
	'Base'		=> $FindBin::Bin,
	'LFlags'	=> "",
	'Object'	=> "$FLEXLIB BFFlex.o",
	'Inc'		=> "-I. \"-I$FindBin::Bin/FLEXlm/$FlexVersion/machind\"",
	'Libs'		=> ["-lm"],
	'Blib'		=> "blib-$^O",
	};
$BuildConfig->{'Base'} =~ s/\/$//;
$BuildConfig->{'Inc'} =~ s/\/$//;

if($Plat =~ /solaris/i) {
	print "Configuring for Solaris...\n";
	$BuildConfig->{'Libs'} = ["-lm -L$BuildConfig->{'Base'}/FLEXlm/$FlexVersion/sun4_u5 -llmgr"];
	$BuildConfig->{'Object'} .= " $BuildConfig->{'Base'}/FLEXlm/$FlexVersion/sun4_u5/lm_new.o"
}
elsif($Plat =~ /Win32/i) {
	print "Configuring for Win32...\n";
	$BuildConfig->{'Base'} =~ s/\//\\/g;
	$BuildConfig->{'LFlags'} = "-dll -nologo -nodefaultlib -debug -opt:ref,icf -LIBPATH:\"$BuildConfig->{'Base'}\\FLEXlm\\$FlexVersion\\i86_n3\"";
	$BuildConfig->{'Object'} = "bfflexs.obj BFFlex.obj lmgr_md.lib lm_new_md.obj comctl32.lib";
	$BuildConfig->{'Inc'} =~ s/\//\\/g;
	$BuildConfig->{'Libs'}	= [""];
}
elsif($Plat =~ /linux/i) {
	print "Configuring for Linux...\n";
	$BuildConfig->{'Libs'} = ["-lm -L$BuildConfig->{'Base'}/FLEXlm/$FlexVersion/i86_r6 -llmgr"];
	$BuildConfig->{'Object'} .= " $BuildConfig->{'Base'}/FLEXlm/$FlexVersion/i86_r6/lm_new.o"
}
elsif($Plat =~ /aix/i) {
	print "Configuring for AIX...\n";
	$BuildConfig->{'Libs'} = ["-lm -L$BuildConfig->{'Base'}/FLEXlm/$FlexVersion/rs6000_u3 -llmgr"];
	$BuildConfig->{'Object'} .= " $BuildConfig->{'Base'}/FLEXlm/$FlexVersion/rs6000_u3/lm_new.o"
}
map {print "[$_] => [$BuildConfig->{$_}]\n"} keys(%{$BuildConfig});
WriteMakefile(
	'NAME'		=> 'BuildForge::BFFlex',
    'VERSION_FROM' => 'BFFlex.pm',
	'LIBS'		=> $BuildConfig->{'Libs'},
	'LDDLFLAGS'		=> $BuildConfig->{'LFlags'},
	'OBJECT'	=> $BuildConfig->{'Object'},
	'INC'		=> $BuildConfig->{'Inc'},
	'INST_ARCHLIB'  => $BuildConfig->{'Blib'},
	'INST_LIB'		=> $BuildConfig->{'Blib'},
	'INST_BIN'		=> $BuildConfig->{'Blib'},
);

