
#
# Copyright (c)2003-2007, BuildForge, Inc.  All rights reserved.
# BuildForge, Inc. owns the copyright and other intellectual property
# rights in this software.  Duplication or use of the Software is not
# permitted except as expressly provided in a written agreement between
# your company and BuildForge, Inc.
#


package BuildForge::BFFlex;

use 5.006;
use strict;
use warnings;
use Carp;

require Exporter;
require DynaLoader;

our @ISA = qw(Exporter DynaLoader);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use BuildForge::BFFlex ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	Connect
	firstKey
	nextKey
	keyUserCount
	getError
	getInfo
	checkoutALicense
	heartBeats
	tearDown
	checkLoad
	checkText
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	Connect
	firstKey
	nextKey
	keyUserCount
	getError
	getInfo
	checkoutALicense
	heartBeats
	tearDown
	checkLoad
	checkText
);
our $VERSION = '7.0.1';

bootstrap BuildForge::BFFlex $VERSION;

1;
__END__

=head1 I<NAME>

BuildForge::BFFlex

=head1 I<SYNOPSIS>

	use BuildForge::BFFlex;
	print "checkLoad:".BuildForge::BFFlex::checkLoad()."\n";
	print "checkText:".BuildForge::BFFlex::checkText("Hello")."\n";
	Error() if(BuildForge::BFFlex::Connect("rfuller"));
	print "Connected\n";
	Error() if(BuildForge::BFFlex::checkoutALicense("BuildForgeStd"));
	print "Got a License:";
	print BuildForge::BFFlex::getInfo()."\n";
	my $Key = "";
	$Key = BuildForge::BFFlex::firstKey("BuildForgeStd");
	print "Key = $Key\n";
	Error() if(!$Key);
	for($n=0;$n<10;$n++) {
		print "Heartbeat:".BuildForge::BFFlex::heartBeats()."\n";
	sleep(20);
	}
	BuildForge::BFFlex::tearDown();
	exit 0;	
	
	sub Error {
		print "ERROR:".BuildForge::BFFlex::getError()."\n";
		exit 0;
	}


=head1 I<DESCRIPTION>

	This class interfaces with the Rational License Administration utility. It allows you to checkout licenses, check the total number of licenses available for a particular license, amongst other things.

	Note This class should not be instantiated, all methods are class methods.

=head1 AUTHOR

BuildForge, E<lt>support@buildforge.comE<gt>


=head1 I<METHODS>

=head2 Connect(<server string>)

Creates a connection to the license server so that licenses may be checked out.
Returns non-zero on failure.

=over 8

=item I<server string>

The name and port(optional) of the license server. (@hostname, port@hostname)

=back

=head2 firstKey(<license type>)

Returns the license key associated with the ckecked-out License type. If no license of the type is checked out it will return an empty string.

=over 8

=item I<license type>

The name of the checked out feature license.

=back

=head2 keyUserCount(<license type>)

Returns the number of licenses of that type that may be checked out. At least one of the licenses of the type must be checked out in order to query the license. If no license has been checked out, then the return is 0.

=over 8

=item I<license type>

The name of the checked out feature license.

=back

=head2 getError()

Returns a string containing the last error encountered by the licensing system.

=head2 getInfo()

Returns a string containing the last status message posted by the licensing system.

=head2 checkoutALicense(<license type>)

Attempts to checkout a license for a feature, returns non-zero is the checkout was not successful.

=over 8

=item I<license type>

The name of the checked out feature license.

=back

=head2 heartBeats()

Sends a heartbeat to the license server for each checked out license. The return value if non-zero identifies that the license server could not be contacted. If the license server goes down, and comes back up, the licenses will be automatically re-checked out. This method should be called at regular intervals, however intervals less than 20 seconds apart are redundant.

=head2 tearDown()

Checks in all checked out licenses and closes the connection to the license server. This must be called when your application is exiting.

=head1 I<AUTHOR>

IBM - Rational - Build Forge

=cut
