
package BuildForge::Ex::Throwable;

use strict;
use overload '""' => \&toString,
		'bool' => sub { 1 },
		'0+' => sub { 1 };

use IO::Handle;
use UNIVERSAL qw(isa);

my $stderr = new IO::Handle();
$stderr->fdopen(fileno(STDERR), "w");

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $cause = undef;
	$cause = pop if @_ && isa($_[$#_], 'BuildForge::Ex::Throwable');

	my $self = {
		'args' => @_ ? [@_] : [],
		'cause' => $cause,
		'trace' => []
	};
	bless $self, $class;

	$self->fillInStackTrace();
	return $self;
}
sub getClass { ref($_[0]) }



sub fillInStackTrace {
	my ($self) = @_;
	my $trace = $self->{'trace'} = [];
	my $depth = 0;
	my @frame;
	while (scalar( @frame = caller($depth++) )) {
		last unless $frame[0] =~ /^BuildForge::Ex:/;
	}
	while (scalar( @frame = caller($depth++) )) {
		push @$trace, [@frame];
	}
}
sub getCause { $_[0]->{'cause'} }
sub getLocalizedMessage { $_[0]->getMessage; }
sub getMessage {
	my ($self) = @_;
	my $args = $self->{'args'};
	!defined($args) and return undef;
	return join('|', map { defined($_) ? $_ : '' } @$args);
}
sub getStackTrace { $_[0]->{'trace'} }
sub getStackTraceText {
	my ($self) = @_;
	my $buf = '';
	for (;;) {
		my $class = ref($self);
		my $msg = $self->getLocalizedMessage;
		my $trace = $self->{'trace'};
		my $cause = $self->getCause;
		$buf .= $class;
		$buf .= ': ' . $msg if defined $msg;
		$buf .= "\n";
		my $last = @$trace - 1;
		for (my $level=0; $level<=$last; $level++) {
			my ($package, $filename, $line, $subroutine, $hasargs, $wantarray,
					$evaltext, $is_require, $hints, $bitmask) = @{ $trace->[$level] };
			# Because of how strack trace info is returned in Perl, it makes
			# more sense to use the next frame's subroutine name...
			$subroutine = $level == $last ? "top level" : $trace->[$level+1]->[3];
			if (defined($evaltext)) {
				$evaltext =~ s#\n#\n\t\t#g;
				$subroutine = "(eval){\n\t\t$evaltext\n\t}" if length $evaltext;
			}
			$buf .= "\tat $subroutine($filename:$line)\n";
		}
		last unless defined $cause;
		$buf .= "Caused by: ";
		$self = $cause;
	}
	return $buf;
}
sub initCause {
	my ($self, $cause) = @_;
	defined($cause) && !isa($cause, 'BuildForge::Ex::Exception')
		and $cause = new BuildForge::Ex::Exception("$cause");
	$self->{'cause'} = $cause;
}
sub printStackTrace {
	my ($self, $fh) = @_;
	$fh = $stderr unless defined $fh;
	$fh->print($self->getStackTraceText);
	return undef;
}
sub toString {
	my ($self) = @_;
	return $self->getStackTraceText if $ENV{'BFDEBUG_EX_DETAIL'};
	my $sb = ref($self);
	my $msg = $self->getLocalizedMessage();
	defined($msg) and $sb = $sb . ': ' . $msg;
	$sb .= "\n";
	my $cause = $self->getCause();
	defined($cause) and $sb = $sb . "Caused by: " . $cause->toString();
	return $sb;
}
sub msg { $_[0]->{'msg'} }
sub throw { die $_[0]->toString(); }

1;

