
package BuildForge::Services::DBO;

use strict;
use overload '""' => \&toString;

use BuildForge::Ex;
use Data::Dumper;
use UNIVERSAL qw(can isa);

# TODO Environment
# TODO EnvironmentEntry
# TODO Filter
# TODO FilterPattern

my @DBO_LIST = qw(
	AccessGroup
	Adaptor
	AdaptorLink
	BOM
	BOMManifest
	Build
	BuildClass
	Collector
	CollectorProperty
	Cron
	Engine
	Environment
	EnvironmentEntry
	Filter
	FilterPattern
	LDAP
	Log
	Manifest
	ManifestProperty
	Message
	Note
	Project
	Register
	Result
	Selector
	SelectorProperty
	Semaphore
	Server
	ServerAuth
	Step
	SysConfig
	TagVar
	Template
	TimeZone
	User
);
my @ABSTRACT_FUNCTIONS = qw(
	getTypeKey
	fromArray
	toArray
);

foreach my $class (@DBO_LIST) {
	my $pkg = 'BuildForge::Services::DBO::' . $class;
	eval "require $pkg";
	die $@ if $@;

	# Sanity check each of the DBO classes
	foreach (@ABSTRACT_FUNCTIONS) {
		$pkg->can($_) or die "$pkg: Must implement abstract function $_\n";
	}
}

sub getConnection { $_[0]->{'_conn'}; }

sub checkConnection {
	my ($self, $conn) = @_;
	my $ref = ref($conn);
	if ($ref ne 'BuildForge::Services::Connection') {
		my @c = caller(1);
		die BuildForge::Ex::IllegalArgumentException->new("\$conn isa '$ref' - $c[3] ($c[1]:$c[2])");
	}
}

# Verifies that the function that called us was in fac
sub internal {
	my $caller = caller(1);
	$caller =~ /^BuildForge::Services::DBO::/
		or die BuildForge::Ex::IllegalAccessError->new(join(':', caller));
}


sub badEnum { BuildForge::Ex::APIException->badIndex($_[1], $_[2]); }
sub badIndex { BuildForge::Ex::APIException->badIndex($_[1]); }
sub dead { BuildForge::Ex::APIException->dead($_[1]); }
sub invalid {
	my ($proto, $key, $field) = @_;
	if (!defined($field)) {
		$field = $key;
		$key = $proto->getTypeKey();
	}
	BuildForge::Ex::APIException->invalid($key, $field);
}
sub live { BuildForge::Ex::APIException->live($_[1]); }
sub malformed {
	my ($self, $expected, $got) = @_;
	BuildForge::Ex::APIException->badDBO(
			$self->getTypeKey,
			$expected,
			$got);
}

sub missing { BuildForge::Ex::APIException->missing($_[1]); }

sub wrap {
	my ($self, $dbo) = @_;
	if (isa($dbo, 'HASH')) {
		foreach (keys %$dbo) {
			$self->{$_} = $dbo->{$_} unless m/^_/;
		};
	}
	return $self;
}

sub new {
	my ($proto, $prototype, $conn, $dbo) = @_;
	ref($conn) ne 'BuildForge::Services::Connection' and die BuildForge::Ex::IllegalArgumentException->new('$conn');
	ref($prototype) ne 'HASH' and die BuildForge::Ex::IllegalArgumentException->new('$prototype');
	my $ref = ref($proto);
	my $self;
	if ($ref) {
		$self = bless {}, $ref;
		$dbo = $proto;
	} else {
		$self = bless {}, $proto;
	}
	$self->{'_conn'} = $conn;
	!defined($dbo) || !isa($dbo, 'HASH') and $dbo = $prototype;
	for (keys %$prototype) { $self->{$_} = $dbo->{$_}; }
	return $self;
}
sub clone { return $_[0]->new($_[0]->{'_conn'}, {}); }

sub toString { Data::Dumper->Dump([ $_[0] ]); }

sub fromArrayImpl {
	my ($self, $data, $keys) = @_;
	my $exp = @$keys;
	my $got = @$data;

	$exp == $got or die $self->malformed($exp, $got);
	for (my $i=0; $i<$exp; $i++) {
		$self->{ $keys->[$i] } = $data->[$i];
	}
	return $self;
}

1;

