
package BuildForge::Services::DBO::Engine;

use strict;
use constant FLAG_RESUME => 'R';

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(version alive uptime master tick license reqMaster
	ping mResponse sResponse zone bias population);
my $prototype = {
	'version' => "",
	'alive' => ' ',
	'uptime' => 0,
	'master' => ' ',
	'tick' => 0,
	'license' => ' ',
	'reqMaster' => ' ',
	'ping' => "",
	'mResponse' => 0,
	'sResponse' => 0,
	'zone' => "",
	'bias' => "",
	'population' => 0,
};
sub getTypeKey { 'Engine' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'uptime'});
	Integer(\$self->{'tick'});
	Integer(\$self->{'mResponse'});
	Integer(\$self->{'sResponse'});
	Integer(\$self->{'population'});
	return $self;
}


sub pause {
	my ($proto, $conn) = @_;
	$conn->request(COMMAND_ENGINE_PAUSE);
	$conn->call();
}

sub resume {
	my ($proto, $conn) = @_;
	$conn->request(COMMAND_ENGINE_PAUSE);
	$conn->writeEntryString(KEY_FLAGS, FLAG_RESUME);
	$conn->call();
}


# wrap is inherited

sub _checkId($) { !defined($_[0]) || !length($_[0]) and die __PACKAGE__->invalid('ID'); }


1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Engine


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Pausing / resuming the engine.
	BuildForge::Services::DBO::Engine->pause($conn);
	BuildForge::Services::DBO::Engine->resume($conn);

	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Engine allows you to pause and resume the operation of the BuildForge engine.


=head1 METHODS


=head2 BuildForge::Services::DBO::Engine->pause(connection)

Pauses the operation of the BuildForge engine.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head2 BuildForge::Services::DBO::Engine->resume(connection)

Resumes the operation of the BuildForge engine.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

