
package BuildForge::Services::DBO::ManifestProperty;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(serverName propertyName propertyValue propertyCategory);
my $prototype = {
	_parent             => undef,
	serverName          => undef,
	propertyName        => undef,
	propertyValue       => undef,
	propertyCategory    => undef,
};
sub getTypeKey { 'ManifestProp' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }


sub new {
	my $proto = shift;
	my ($newParent) = splice(@_, 1, 1);
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'_parent'} = $newParent;
	Enum(ENUM_MANIFESTPROPERTY_CATEGORY, \$self->{'propertyCategory'});
	return $self;
}


sub getParent              { $_[0]->{'_parent'}; }

sub getServerName          { $_[0]->{'serverName'}; }
sub getPropertyName        { $_[0]->{'propertyName'}; }
sub getPropertyValue       { $_[0]->{'propertyValue'}; }
sub getPropertyCategory    { $_[0]->{'propertyCategory'}; }

# setters are not used



1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::ManifestProperty


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting manifest properties
	$servers = BuildForge::Services::DBO::Server->findAll($conn);
	$manifest = $servers->[0]->getManifest();
	$properties = $manifest->getProperties();
	$property = $manifest->getProperty('BF_JOBS');
	
	# Getter functions
	$manifest = $property->getParent();
	$serverName = $property->getServerName();
	$name = $manifest->getPropertyName();
	$value = $manifest->getPropertyValue();
	$category = $manifest->getPropertyCategory();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

ManifestProperty allows you to access the individual properties in the manifest of a 
specific BuildForge server.  See BuildForge::Services::DBO::Manifest->getProperty() or
getProperties() to see how to get the ManifestProperty objects.


=head1 METHODS


=head2 getParent()

Returns the parent BuildForge::Services::DBO::Manifest object for this manifest property.


=head2 getServerName()

Returns the server name of the server that this manifest property is connected to.


=head2 getPropertyName()

Returns the name of the manifest property.


=head2 getPropertyValue()

Returns the value of the manifest property.


=head2 getPropertyCategory()

Returns the category of the manifest property.  This may be 'NORMAL' for built-in
properties, 'AUTO' for properties set by the user, and 'USER' for run command properties.
See BuildForge::Services::DBO::CollectorProperty for information on how the manifest
properties are created.


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

