
package BuildForge::Services::DBO::Message;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);
use overload "" => \&toString;

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(timestamp userId processId severity type messageKey args indirectionMask);
my $prototype = {
	'timestamp' => 0,
	'userId' => UID_SYSTEM,
	'processId' => 0,
	'severity' => ENUM_MESSAGE_SEVERITY->INFO,
	'type' => '',
	'messageKey' => '',
	'indirectionMask' => 0,
};
sub getTypeKey { 'Message' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	$self->{'args'} = [];
	Integer(\$self->{'timestamp'});
	Integer(\$self->{'userId'});
	Integer(\$self->{'processId'});
	Enum(ENUM_MESSAGE_SEVERITY, \$self->{'severity'});
	Integer(\$self->{'indirectionMask'});
	return $self;
}

sub dbo {
	my $proto = shift;
	my $self = { %$prototype };
	my ($severity, $messageKey, $args, $mask) = @_;
	$self->{'severity'} = $severity;
	$self->{'messageKey'} = defined($messageKey) ? $messageKey : '(null)';
	$self->{'args'} = isa($args, 'ARRAY') ? $args : [];
	$self->{'timestamp'} = time;
	$self->{'processId'} = $$;
	$self->{'indirectionMask'} = $mask;
	return bless $self;
}

sub translate {
	my ($self, $locale, $tzone) = @_;
	my $conn = $self->{'_conn'};
	!defined($conn) and $conn = BuildForge::Services::Connection->getDefault();
	if (ref($conn) eq 'BuildForge::Services::Connection') {
		my $msg = undef;
		$msg = eval { $conn->translate($self, $locale, $tzone) };
		return $msg if !$@ && defined($msg) && length($msg);
	}
	return $self->toString();
}

# wrap is inherited

sub getArgs                    { $_[0]->{'args'}; }
sub getIndirectionMask         { $_[0]->{'indirectionMask'}; }
sub getProcessId               { $_[0]->{'processId'}; }
sub getMessageKey              { $_[0]->{'messageKey'}; }
sub getSeverity                { $_[0]->{'severity'}; }
sub getTimestamp               { $_[0]->{'timestamp'}; }
sub getType                    { $_[0]->{'type'}; }
sub getUserId                  { $_[0]->{'userId'}; }

# No setters -- no need to change a message once it has been created

sub toString {
	my ($self) = @_;
	my $args = $self->{'args'};
	isa($args, 'ARRAY') or $args = [];
	return 'Message['
		. 'timestamp=' . $self->{'timestamp'} . ', '
		. 'userId=' . $self->{'userId'} . ', '
		. 'processId=' . $self->{'processId'} . ', '
		. 'severity=' . $self->{'severity'} . ', '
		. 'type=' . $self->{'type'} . ', '
		. 'messageKey=' . $self->{'messageKey'} . ', '
		. 'args={' . join('|', map { defined($_) ? $_ : '(null)' } @$args) . '}, '
		. 'indirectionMask=' . $self->{'indirectionMask'} . ']';
}

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Message


=head1 SYNOPSIS


=head1 DESCRIPTION

Message is the (currently not enabled) interface to the BuildForge console System Messages.


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

