
package BuildForge::Services::DBO::Result;

use strict;
use constant FLAG_GET_COMMAND => 'C';
use constant FLAG_GET_ENV     => 'E';

use BuildForge::Ex;
use BuildForge::Services::Constants;
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info (DataDBO)
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(logId buildId uid projectId originalStepId resultStepId
	startTimestamp duration result description directory caller flag
	environmentGroupId filterSetId chainId commandText passNotify
	failNotify onFail resource server threaded absolute 
	timeout level broadcast environmentText owner passChainId
	failChainId floating finish passWait failWait chainedBuildId);
my $prototype = {
	'logId' => '',
	'buildId' => 0,
	'uid' => undef,
	'projectid' => 0,
	'originalStepId' => 0,
	'resultStepid' => 0,
	'startTimestamp' => 0,
	'duration' => 0,
	'result' => ENUM_RESULT_STEPRESULT->NONE,
	'description' => '',
	'directory' => '',
	'caller' => 0,
	'flag' => ENUM_RESULT_STEPFLAG->RUN,
	'environmentGroupId' => 0,
	'filterSetId' => 0,
	'chainId' => 0,
	'commandText' => undef,
	'passNotify' => 0,
	'failNotify' => 0,
	'onFail' => ENUM_STEP_ONFAIL->HALT,
	'resource' => '',
	'server' => '',
	'threaded' => ENUM_STEP_THREADED->NO,
	'absolute' => 0,
	'timeout' => 300,
	'level' => 0,
	'broadcast' => 0,
	'environmentText' => undef,
	'owner' => 0,
	'passChainId' => 0,
	'failChainId' => 0,
	'floating' => ENUM_RESULT_FLOATING->YES,
	'finish' => 0,
	'passWait' => 0,
	'failWait' => 0,
	'chainedBuildId' => 0,
};
sub getTypeKey { 'Result' }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'buildId'});
	Integer(\$self->{'projectId'});
	Integer(\$self->{'originalStepdId'});
	Integer(\$self->{'resultStepdId'});
	Integer(\$self->{'startTimestamp'});
	Integer(\$self->{'duration'});
	Enum(ENUM_RESULT_STEPRESULT, \$self->{'result'});
	Integer(\$self->{'caller'});
	Enum(ENUM_RESULT_STEPFLAG, \$self->{'flag'});
	Integer(\$self->{'environmentGroupId'});
	Integer(\$self->{'filterSetId'});
	Integer(\$self->{'chainId'});
	Integer(\$self->{'passNotify'});
	Integer(\$self->{'failNotify'});
	Enum(ENUM_STEP_ONFAIL, \$self->{'onFail'});
	Enum(ENUM_STEP_THREADED, \$self->{'threaded'});
	Boolean(\$self->{'absolute'});
	Integer(\$self->{'timeout'});
	Integer(\$self->{'level'});
	Boolean(\$self->{'broadcast'});
	Integer(\$self->{'owner'});
	Integer(\$self->{'passChainId'});
	Integer(\$self->{'failChainId'});
	Enum(ENUM_RESULT_FLOATING, \$self->{'floating'});
	Integer(\$self->{'finish'});
	Integer(\$self->{'passWait'});
	Integer(\$self->{'failWait'});
	Integer(\$self->{'chainedBuildId'});
	return $self;
}


sub findByBuild {
	my ($proto, $conn, $buildId, $getCmd, $getEnv) = @_;
	$proto->checkConnection($conn);
	!defined($getCmd) and $getCmd = 1;
	!defined($getEnv) and $getEnv = 0;
	$conn->request(COMMAND_RESULT_FIND_BID);
	$conn->writeEntryString(KEY_BUILD_ID, $buildId);
	my $sb = '';
	$getCmd and $sb .= FLAG_GET_COMMAND;
	$getEnv and $sb .= FLAG_GET_ENV;
	length($sb) and $conn->writeEntryString(KEY_FLAGS, $sb);
	my $resp = $conn->call();

	my $list = $resp->{+KEY_RESULT_LIST};
	!isa($list, 'ARRAY') and return undef;
	my @results = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@results;
}


sub getAbsolute           { $_[0]->{'absolute'}; }
sub getBroadcast          { $_[0]->{'broadcast'}; }
sub getBuildId            { $_[0]->{'buildId'}; }
sub getCaller             { $_[0]->{'caller'}; }
sub getChainedBuildId     { $_[0]->{'chainedBuildId'}; }
sub getChainId            { $_[0]->{'chainId'}; }
sub getCommandText        { $_[0]->{'_commandText'}; }
sub getDescription        { $_[0]->{'description'}; }
sub getDirectory          { $_[0]->{'directory'}; }
sub getDuration           { $_[0]->{'duration'}; }
sub getEnvironmentGroupId { $_[0]->{'environmentGroupId'}; }
sub getEnvironmentText    { $_[0]->{'_environmentText'}; }
sub getFailChainId        { $_[0]->{'failChainId'}; }
sub getFailNotify         { $_[0]->{'failNotify'}; }
sub getFailWait           { $_[0]->{'failWait'}; }
sub getFilterSetId        { $_[0]->{'filterSetId'}; }
sub getFinish             { $_[0]->{'finish'}; }
sub getFlag               { $_[0]->{'flag'}; }
sub getFloating           { $_[0]->{'floating'}; }
sub getLevel              { $_[0]->{'level'}; }
sub getLogs {
	my ($self) = @_;
	BuildForge::Services::DBO::Log->findByResult($self->{'_conn'}, $self->getBuildId(), $self->getUid());
}
sub getOnFail             { $_[0]->{'onFail'}; }
sub getOriginalStepId     { $_[0]->{'originalStepId'}; }
sub getOwner              { $_[0]->{'owner'}; }
sub getPassChainId        { $_[0]->{'passChainId'}; }
sub getPassNotify         { $_[0]->{'passNotify'}; }
sub getPassWait           { $_[0]->{'passWait'}; }
sub getProjectId          { $_[0]->{'projectId'}; }
sub getResource           { $_[0]->{'resource'}; }
sub getResult             { $_[0]->{'result'}; }
sub getResultStepId       { $_[0]->{'resultStepId'}; }
sub getServer             { $_[0]->{'server'}; }
sub getStartTimestamp     { $_[0]->{'startTimestamp'}; }
sub getThreaded           { $_[0]->{'canBG'}; }
sub getTimeout            { $_[0]->{'timeout'}; }
sub getUid                { $_[0]->{'uid'}; }

# TODO No setters, for now...

# toString is inherited


1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Result


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting build results from an existing build
	$project = new BuildForge::Services::DBO::Project($conn);
	$project->setName('Test Project');
	$project->create();
	$build = BuildForge::Services::DBO::Build->fire($conn, $project->getProjectId());
	$results = $build->getResults();
	$result = $results->[0];
	$results2 = BuildForge::Services::DBO::Build->findByBuild($conn, $build->getBuildId(), 
			$alsoGetCommand, $alsoGetEnvironmentBlock)

	# Getter functions
	$buildId = $result->getBuildId();
	$stepId = $result->getUid();
	$resultStepId = $result->getResultStepId();
	$wasAbsolute = $result->getAbsolute();
	$wasBroadcast = $result->getBroadcast();
	$callingBuild = $result->getCaller();
	$buildKickedOffByResult = $result->getChainedBuildId();
	$inlinedProjectId = $result->getChainId();
	$stepCommand = $result->getCommandText();
	$stepDescription = $result->getDescription();
	$stepDirectory = $result->getDirectory();
	$durationInSeconds = $result->getDuration();
	$environmentId = $result->getEnvironmentGroupId();
	$environmentBlock = $result->getEnvironmentText();
	$projectIdToRunOnStepFailure = $result->getFailChainId();
	$accessGroupIdToNotifyOnStepFailure = $result->getFailNotify();
	$failWait = $result->getFailWait();
	$filterSetId = $result->getFilterSetId();
	$finishTimestamp = $result->getFinish();
	$skippedOrCancelled = $result->getFlag();
	$serverExplicitlySet = $result->getFloating();
	$accessGroupId = $result->getLevel();
	$continueOnFail = $result->getOnFail();
	$stepId = $result->getOriginalStepId();
	$userId = $result->getOwner();
	$projectIdToRunOnStepPass = $result->getPassChainId();
	$accessGroupIdToNotifyOnStepPass = $result->getPassNotify();
	$passWait = $result->getPassWait();
	$projectId = $result->getProjectId();
	$selectorId = $result->getResource();
	$resultCode = $result->getResult();
	$serverId = $result->getServer();
	$startTimestamp = $result->getStartTimestamp();
	$wasThreaded = $result->getThreaded();
	$stepTimeout = $result->getTimeout();
	$logs = $result->getLogs();
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Result allows you to access the individual step results from an existing build.
Many fields are duplicated directly from the BuildForge::Services::DBO::Step object.
Check there for more details if necessary.


=head1 METHODS


=head2 BuildForge::Services::DBO::Build->findByBuild(connection, buildId, getCommand, getEnvironment)

Returns a reference to an array of BuildForge::Services::DBO::Build objects for
the individual step results of the desired build.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item buildId

The build ID (from $build->getBuildId()) of the build from which to get step results.

=item getCommand

If this evaluates to true, the step commands will be part of the
returned objects and $result->getCommandText() will return them.  Otherwise, 
$result->getCommandText() returns the empty string.

=item getEnvironment

If this evaluates to true, the step environment text will be part of the
returned objects and $result->getEnvironmentText() will return it.  Otherwise, 
$result->getEnvironmentText() returns the empty string.

=back


=head2 $result->getBuildId()

Returns the build ID of the build that this step result set belongs to.


=head2 $result->getUid()

Returns the ID of this step result set.


=head2 $result->getResultStepId()

Returns the step number as executed.  That is, if this was the first step started, this
will return 1, if it was the second step, it returns 2, etc.  


=head2 $result->getAbsolute()

Returns 1 if the step was set to absolute path, 0 if it was relative.


=head2 $result->getBroadcast()

Returns 1 if the step was a broadcast step, 0 otherwise.


=head2 $result->getCaller()

If this step was inlined by another step, and the inlining step was threaded, this will return
the step ID (as in $result->getResultStepId()) of the inlining step.  Otherwise, 0.


=head2 $result->getChainedBuildId()

If a build was chained by this step, either from a pass or fail chain, this will return
the build ID of the build that was chained.  Otherwise, it returns 0.


=head2 $result->getChainId()

Returns the project ID of the project that got inlined from this step or 0 if no
inlining occurred.


=head2 $result->getCommandText()

Returns the step command if this result was found with BuildForge::Services::DBO::Build->findByBuild and 
getCommand was set or the empty string otherwise.


=head2 $result->getDescription()

Returns the step name.


=head2 $result->getDirectory()

Returns the directory the step executed in relative to the build directory (if this was a 
relative step) or to the base server directory (if this was an absolute step).


=head2 $result->getDuration()

Returns the time in seconds the step took to execute.


=head2 $result->getEnvironmentGroupId()

Returns the environment group ID of the environment the step had applied to it
over the project environment.


=head2 $result->getEnvironmentText()

Returns the step environment if this result was found with BuildForge::Services::DBO::Build->findByBuild and 
getEnvironment was set or the empty string otherwise.


=head2 $result->getFailChainId()

Returns the project ID if the step was set to chain a project on failure or 0 
otherwise.  This will be set to the project ID whether the step passed or failed.


=head2 $result->getFailNotify()

Returns the access group ID if the step was set to notify an access group on failure
or 0 otherwise.  This will be set to the access group ID whether the step passed or failed.


=head2 $result->getFailWait()

Returns 1 if the step was set to wait for the chained build to complete on failure and 0 otherwise.


=head2 $result->getFilterSetId()

Returns the filter set ID if the step had one set or 0 otherwise.


=head2 $result->getFinish()

Returns the timestamp at which the step finished processing.


=head2 $result->getFlag()

This will return 'RUN' if the step is running / ran normally, 'SKIP' if the step
was skipped, 'CANCEL' if the step was cancelled, or 'PAUSE' if the step is being paused.


=head2 $result->getFloating()

This will return 'FIXED' if the step's selector was explicitly set, 'NO' if the step is not
sticky, 'YES' if the step was sticky but a server hasn't been picked for it, or 'STUCK'
if the step was sticky and a server has been picked for it.


=head2 $result->getLevel()

Returns the access group ID of the access group that owns the step.  Users not in the
access group cannot view the step results.


=head2 $result->getLogs()

Returns a reference to an array of BuildForge::Services::DBO::Log objects, one for each
line of the step output.


=head2 $result->getOnFail()

Returns 1 if the step was set to continue on failure or 0 otherwise.


=head2 $result->getOriginalStepId()

Returns the original step number within its project.


=head2 $result->getOwner()

Returns the step ID (from $result->getResultSetId()) of the step that inlined this one
if this was an inlined step and 0 otherwise.


=head2 $result->getPassChainId()

Returns the project ID if the step was set to chain a project on success or 0 
otherwise.  This will be set to the project ID whether the step passed or failed.


=head2 $result->getPassNotify()

Returns the access group ID if the step was set to notify an access group on success
or 0 otherwise.  This will be set to the access group ID whether the step passed or failed.


=head2 $result->getPassWait()

Returns 1 if the step was set to wait for the chained build to complete on success and 0 otherwise.


=head2 $result->getProjectId()

Returns the project ID of the project the step originally belonged to.


=head2 $result->getResource()

Returns the selector on which the step has run.


=head2 $result->getResult()

This may return any of:

=over 8

=item NONE

If the step has not yet started processing.

=item  RUNNING

If the step is currently running.

=item  PASSED

If the step passed successfully.

=item  STOPPED

If the step has been paused.

=item  SKIPPED

If the step has been skipped.

=item  FAILWARN

If the step failed but has continue on failure set.

=item  FILTWARN

If the step has a filter ID and the filter set passed with warnings.

=item  FAILED

If the step failed.

=back


=head2 $result->getServer()

Returns the name of the server on which the step has run.


=head2 $result->getStartTimestamp()

Returns the timestamp for the time the step started executing.


=head2 $result->getThreaded()

Returns 'NO' if the step is unthreaded, 'YES' if the step is threaded,
or 'JOIN' if the step is a join step.


=head2 $result->getTimeout()

Returns the timeout time in seconds for the original step.


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

