
package BuildForge::Services::DBO::Template;

use strict;

use BuildForge::Services::Constants;
use BuildForge::Services::TextUtils ':all';
use BuildForge::Types ':all';
use UNIVERSAL qw(isa);

# DBO Info
our @ISA = qw(BuildForge::Services::DBO);
my @keys = qw(projectId stepId name description from subject bodyText locale
		liveName liveProjectId liveStepId liveLocale);
my $prototype = {
	'projectId' => 0,
	'stepId' => 0,
	'name' => '',
	'description' => '',
	'from' => '',
	'subject' => '',
	'bodyText' => '',
	'locale' => 'en_US',
	'liveName' => undef,
	'liveProjectId' => 0,
	'liveStepId' => 0,
	'liveLocale' => undef,
};
sub getTypeKey { 'Template'; }
sub fromArray { return $_[0]->fromArrayImpl($_[1], \@keys); }
sub toArray { return [ @{$_[0]}{@keys} ]; }

sub new {
	my $proto = shift;
	my $self = $proto->SUPER::new($prototype, @_);
	Integer(\$self->{'projectId'});
	Integer(\$self->{'liveProjectId'});
	Integer(\$self->{'stepId'});
	Integer(\$self->{'liveStepId'});
	return $self;
}

sub findById {
	my ($proto, $conn, $projectId, $stepId, $templateName, $locale) = @_;
	$proto->checkConnection($conn);
	!defined($projectId) || $projectId < 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	!defined($stepId) || $stepId < 0 and die BuildForge::Services::DBO::Step->invalid('ID');
	!defined($templateName) || !length($templateName) and die $proto->invalid('Name');
	!defined($locale) || !length($locale) and die $proto->invalid('Locale');
	$conn->request(COMMAND_TEMPLATE_FIND_ID);
	$conn->writeEntryString(KEY_TEMPLATE_PROJECT_ID, $projectId);
	$conn->writeEntryString(KEY_TEMPLATE_STEP_ID, $stepId);
	$conn->writeEntryString(KEY_TEMPLATE_NAME, $templateName);
	$conn->writeEntryString(KEY_TEMPLATE_LOCALE, $locale);
	my $dbo = $conn->call()->{+KEY_TEMPLATE_DBO};
	return defined($dbo) ? __PACKAGE__->new($conn)->fromArray($dbo) : undef;
}

sub findByProjectId {
	my ($proto, $conn, $projectId) = @_;
	$proto->checkConnection($conn);
	!defined($projectId) || $projectId < 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	$conn->request(COMMAND_TEMPLATE_FIND_PID);
	$conn->writeEntryString(KEY_TEMPLATE_PROJECT_ID, $projectId);
	return _getList($conn, $conn->call());
}

sub create {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $projectId = $self->{'projectId'};
	my $stepId = $self->{'stepId'};
	my $name = $self->{'name'};
	my $locale = $self->{'locale'};
	!defined($projectId) || $projectId < 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	!defined($stepId) || $stepId < 0 and die BuildForge::Services::DBO::Step->invalid('ID');
	!defined($name) || !length($name) and die $self->invalid('Name');
	!defined($locale) || !length($locale) and die $self->invalid('Locale');
	#$self->{'liveProjectId'} = $projectId;
	#$self->{'liveStepId'} = $stepId;
	#$self->{'liveName'} = $name;
	#$self->{'liveLocale'} = $locale;
	$conn->request(COMMAND_TEMPLATE_ADD);
	$conn->writeEntryDBO(KEY_TEMPLATE_DBO, $self);
	my $dbo = $conn->call()->{+KEY_TEMPLATE_DBO};
	defined($dbo) or die $self->missing(KEY_TEMPLATE_DBO);
	return $self->fromArray($dbo);
}

sub update {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	my $projectId = $self->{'projectId'};
	my $stepId = $self->{'stepId'};
	my $name = $self->{'name'};
	my $locale = $self->{'locale'};
	!defined($projectId) || $projectId < 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	!defined($stepId) || $stepId < 0 and die BuildForge::Services::DBO::Step->invalid('ID');
	!defined($name) || !length($name) and die $self->invalid('Name');
	!defined($locale) || !length($locale) and die $self->invalid('Locale');
	$conn->request(COMMAND_TEMPLATE_UPD);
	$conn->writeEntryDBO(KEY_TEMPLATE_DBO, $self);
	my $dbo = $conn->call()->{+KEY_TEMPLATE_DBO};
	defined($dbo) or die $self->missing(KEY_TEMPLATE_DBO);
	return $self->fromArray($dbo);
}

sub delete {
	my ($self) = @_;
	my $conn = $self->{'_conn'};
	$self->deleteById($conn, $self->{'liveProjectId'}, $self->{'liveStepId'}, $self->{'liveName'},
			$self->{'liveLocale'});
	$self->{'projectId'} = $self->{'liveProjectId'};
	$self->{'stepId'} = $self->{'liveStepId'};
	$self->{'name'} = $self->{'liveName'};
	$self->{'locale'} = $self->{'liveLocale'};
	$self->{'liveName'} = '';
	$self->{'liveLocale'} = '';
	$self->{'liveProjectId'} = 0;
	$self->{'liveStepId'} = 0;
}

sub deleteById {
	my ($proto, $conn, $projectId, $stepId, $name, $locale) = @_;
	$proto->checkConnection($conn);
	!defined($projectId) || $projectId < 0 and die BuildForge::Services::DBO::Project->invalid('ID');
	!defined($stepId) || $stepId < 0 and die BuildForge::Services::DBO::Step->invalid('ID');
	!defined($name) || !length($name) and die $proto->invalid('Name');
	$conn->request(COMMAND_TEMPLATE_DEL);
	$conn->writeEntryString(KEY_TEMPLATE_PROJECT_ID, $projectId);
	$conn->writeEntryString(KEY_TEMPLATE_STEP_ID, $stepId);
	$conn->writeEntryString(KEY_TEMPLATE_NAME, $name);
	$conn->writeEntryString(KEY_TEMPLATE_LOCALE, $locale);
	$conn->call();
}

# wrap is inherited

sub _getList($$) {
	my ($conn, $resp) = @_;
	my $list = $resp->{+KEY_TEMPLATE_LIST};
	!defined($list) || !isa($list, 'ARRAY') and return [];
	my @temps = map { __PACKAGE__->new($conn)->fromArray($_) } @$list;
	return \@temps;
}



sub getBodyText		{ $_[0]->{'bodyText'}; }
sub getDescription	{ $_[0]->{'description'}; }
sub getFrom			{ $_[0]->{'from'}; }
sub getName			{ defined($_[0]->{'name'}) ? $_[0]->{'name'} : $_[0]->{'liveName'}; }
sub getProjectId	{ defined($_[0]->{'projectId'}) ? $_[0]->{'projectId'} : $_[0]->{'liveProjectId'}; }
sub getStepId		{ defined($_[0]->{'stepId'}) ? $_[0]->{'stepId'} : $_[0]->{'liveStepId'}; }
sub getSubject		{ $_[0]->{'subject'}; }
sub getLocale		{ defined($_[0]->{'locale'}) ? $_[0]->{'locale'} : $_[0]->{'liveLocale'}; }


sub setBodyText		{ $_[0]->{'bodyText'}		= $_[1]; }
sub setDescription	{ $_[0]->{'description'}	= $_[1]; }
sub setFrom			{ $_[0]->{'from'}			= $_[1]; }
sub setName			{ $_[0]->{'name'}			= $_[1]; }
sub setProjectId	{ $_[0]->{'projectId'}		= $_[1]; }
sub setStepId		{ $_[0]->{'stepId'}			= $_[1]; }
sub setSubject		{ $_[0]->{'subject'}		= $_[1]; }
sub setLocale		{ $_[0]->{'locale'}			= $_[1]; }


# toString is inherited

1;


__END__

=pod

=head1 NAME

BuildForge::Services::DBO::Template


=head1 SYNOPSIS

	use BuildForge::Services;

	$conn = new BuildForge::Services::Connection($hostname);
	$token = $conn->authUser($user, $pass);
	
	# Getting existing templates
	$allProjectTemplates = BuildForge::Services::DBO::Template->findByProjectId($conn, $projectId);
	$oldtemplate = BuildForge::Services::DBO::Template->findById($conn, $projectId, $stepId, 
			$templateName);

	# Getter / setter functions
	$projectId = $oldtemplate->getProjectId();
	$stepId = $oldtemplate->getStepId();
	$templateName = $oldtemplate->getName();
	$description = $oldtemplate->getDescription();
	$from = $oldtemplate->getFrom();
	$subject = $oldtemplate->getSubject();
	$body = $oldtemplate->getBodyText();
	$locale = $oldtemplate->getLocale();

	$oldtemplate->setProjectId($projectId);
	$oldtemplate->setStepId($stepId);
	$oldtemplate->setName('build_break');
	$oldtemplate->setDescription('New Template Description');
	$oldtemplate->setFrom('user@company.com');
	$oldtemplate->setSubject('Subject of email message');
	$oldtemplate->setBodyText('Body of email message');
	$oldtemplate->setLocale('en_US');
	$oldtemplate->update();
	
	# Template creation, updating, deletion
	$newtemplate = new BuildForge::Services::DBO::Template($conn);
	$newtemplate->setProjectId($projectId);
	$newtemplate->setStepId($stepId);
	$newtemplate->setName('build_pass');
	$newtemplate->setLocale('es_ES');
	$newtemplate->create();

	$newtemplate->setDescription('Another Template Description');
	$newtemplate->update();
	
	$newtemplate->delete();
	BuildForge::Services::DBO::Template->deleteById($conn, $projectId, $stepId, $templateName);
		
	$conn->logout;
	$conn->close;


=head1 DESCRIPTION

Template allows you to create, access, and delete BuildForge notification templates, used to
send emails to users upon encountering various alert events. 


=head1 METHODS


=head2 new BuildForge::Services::DBO::Template(connection, {TemplateParamsHashRef})

Returns a new instance of a Template object.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item TemplateParamsHashRef

An optional reference to a hash containing any or all of the following fields.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the project that the template is connected to.
A value of 0 indicates that this is one of the base templates which get used in the absence
of a specific project template.  Base templates should not be created or deleted, only
edited.

=item stepId

The step number (from $step->getStepOrdinal()) of the step that the template is used for
for step-specific templates.  A value of 0 (the default) indicates that this template is
used for all steps and for project-specific purposes.

=item name

The key for the template type.  This should be identical to one of the keys for the base templates.

=item locale

The code for the locale of the template.  Defaults to 'en_US', the English language locale.
A template for a given locale will be sent
to users from that locale when the appropriate event is kicked off.  For example, if a build
is set to send an email on success to a given access group, it'll send the 'en_US' template to
the english speakers, the 'es_ES' template to the spanish speakers, etc.

=item description

The description of the template.

=item from

The "From" field of the outgoing email message.

=item subject

The "Subject" field of the outgoing email message.  This may have a number of template-specific
variables included.  For more details of the variables, see the BuildForge online help topic 
Setting Up Notification->Notification Templates->Special Notification Template Variables.

=item bodyText

The body of the outgoing email message.  This may have a number of template-specific
variables included.  For more details of the variables, see the BuildForge online help topic 
Setting Up Notification->Notification Templates->Special Notification Template Variables.

=back

=back


=head2 BuildForge::Services::DBO::Template->findByProjectId(connection, projectId)

Returns a reference to an array of BuildForge::Services::DBO::Template objects corresponding
to all templates in the database for the given project.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID of the project that the templates relate to (from $project->getProjectId()), or 0
to get all of the base templates.

=back


=head2 BuildForge::Services::DBO::Template->findById(connection, projectId, stepNumber, templateKey)

Returns the BuildForge::Services::DBO::Template object corresponding to the given
database template, or undef if no such object exists.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID of the project that the template relates to (from $project->getProjectId()), or 0
to get one of the base templates.

=item stepNumber

The step number of the step that the project relates to (from $step->getStepOrdinal()), or 0 for
the all-step or project-specific templates.

=item templateKey

The key for the template (from $template->getName()).

=back


=head2 $template->create()

Creates the template record within the database.  Base templates (those with project ID of
0) cannot be created.


=head2 $template->update()

Updates the template record within the database.


=head2 $template->delete()

Deletes the template record from the database.  Base templates (those with project ID of
0) cannot be deleted.


=head2 BuildForge::Services::DBO::Template->deleteById(connection, projectId, stepNumber, templateKey)

Deletes the template record from the database with the given project, step, and ID.
Base templates (those with project ID of 0) cannot be deleted.

=over 8

=item connection

A connected BuildForge::Services::Connection object.

=item projectId

The project ID of the project that the template relates to (from $project->getProjectId()).

=item stepNumber

The step number of the step that the project relates to (from $step->getStepOrdinal()), or 0 for
the all-step or project-specific templates.

=item templateKey

The key for the template (from $template->getName()).

=back


=head2 $template->getName()

Returns the template's key.


=head2 $template->getProjectId()

Returns the project ID for the project the template is connected to, or 0 for the base templates.


=head2 $template->getStepId()

Returns the step number for the step the template is connected to, or 0 for the all-step or
project-specific templates.


=head2 $template->getDescription()

Returns the template's description.


=head2 $template->getFrom()

Returns the "From" field of the emails the template sends out.


=head2 $template->getSubject()

Returns the "Subject" field of the emails the template sends out.
This may have a number of template-specific
variables included.  For more details of the variables, see the BuildForge online help topic 
Setting Up Notification->Notification Templates->Special Notification Template Variables.


=head2 $template->getBodyText()

Returns the body of the emails the template sends out.
This may have a number of template-specific
variables included.  For more details of the variables, see the BuildForge online help topic 
Setting Up Notification->Notification Templates->Special Notification Template Variables.


=head2 $template->getLocale()

Returns the code for the locale of the template.  A template for a given locale will be sent
to users from that locale when the appropriate event is kicked off.  For example, if a build
is set to send an email on success to a given access group, it'll send the 'en_US' template to
the english speakers, the 'es_ES' template to the spanish speakers, etc.


=head2 $template->setName(templateKey)

Sets the template's key.
$template->update() must be run before changes are replicated in the database.

=over 8

=item templateKey

The new key for the template.  This should be identical to one of the keys of the base
templates.

=back


=head2 $template->setProjectId(projectId)

Sets the project that the template is connected to.
$template->update() must be run before changes are replicated in the database.

=over 8

=item projectId

The project ID (from $project->getProjectId()) of the project that this template will get connected to.

=back


=head2 $template->setStepId(stepNumber)

Sets the step that the template is connected to.
$template->update() must be run before changes are replicated in the database.

=over 8

=item stepNumber

The step number (from $step->getStepOrdinal()) that the template is connected to
or 0 for templates connected to all project steps or used for project-specific 
purposes.

=back


=head2 $template->setDescription(description)

Sets the description of the template.
$template->update() must be run before changes are replicated in the database.

=over 8

=item description

The new template description.

=back


=head2 $template->setFrom(from)

Sets the "From" field for the emails sent with this template.
$template->update() must be run before changes are replicated in the database.

=over 8

=item from

The email address this message is listed as coming from.

=back


=head2 $template->setSubject(subject)

Sets the "Subject" field for the emails sent with this template.
$template->update() must be run before changes are replicated in the database.

=over 8

=item subject

The new email subject line.  
This may have a number of template-specific
variables included.  For more details of the variables, see the BuildForge online help topic 
Setting Up Notification->Notification Templates->Special Notification Template Variables.

=back


=head2 $template->setBodyText(body)

Sets the message body for the emails sent with this template.  
$template->update() must be run before changes are replicated in the database.

=over 8

=item body

The new email body.  
This may have a number of template-specific
variables included.  For more details of the variables, see the BuildForge online help topic 
Setting Up Notification->Notification Templates->Special Notification Template Variables.

=back


=head2 $template->setLocale(locale)

Sets the template's locale.
A template for a given locale will be sent
to users from that locale when the appropriate event is kicked off.  For example, if a build
is set to send an email on success to a given access group, it'll send the 'en_US' template to
the english speakers, the 'es_ES' template to the spanish speakers, etc.
$template->update() must be run before changes are replicated in the database.

=over 8

=item locale

The key for the locale to set.

=back


=head1 COPYRIGHT

Copyright (c)2006-2007 International Business Machines, Inc.  All rights reserved.

=cut

