# Copyright 2003-2007, BuildForge, Inc. All rights reserved.
# BuildForge, Inc. owns the copyright and other
# intellectual property rights in this software.
#
# Duplication or use of the Software is not permitted
# except as expressly provided in a written agreement 
# between your company and BuildForge, Inc.

# package BuildForge::Utilities::Crypt
#
# BuildForge::Utilities::Crypt
# This package is used for crypt and decrypt functions.
# 
#

package BuildForge::Services::TextUtils;

use strict;
use bytes;
use Exporter;
use Encode qw(encode_utf8 decode_utf8);

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(bf_encrypt);
our %EXPORT_TAGS = ('all' => \@EXPORT_OK);

my $hex = 
	"000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f" .
	"202122232425262728292a2b2c2d2e2f303132333435363738393a3b3c3d3e3f" .
	"404142434445464748494a4b4c4d4e4f505152535455565758595a5b5c5d5e5f" .
	"606162636465666768696a6b6c6d6e6f707172737475767778797a7b7c7d7e7f" .
	"808182838485868788898a8b8c8d8e8f909192939495969798999a9b9c9d9e9f" .
	"a0a1a2a3a4a5a6a7a8a9aaabacadaeafb0b1b2b3b4b5b6b7b8b9babbbcbdbebf" .
	"c0c1c2c3c4c5c6c7c8c9cacbcccdcecfd0d1d2d3d4d5d6d7d8d9dadbdcdddedf" .
	"e0e1e2e3e4e5e6e7e8e9eaebecedeeeff0f1f2f3f4f5f6f7f8f9fafbfcfdfeff";

# Class methods only.
sub bf_encrypt {
	my ($pt) = @_;
	defined($pt) or return undef;
	my @ptb = unpack("C*", encode_utf8($pt));
	@ptb > 116 and return undef;
	my @key;
	my $ct = '';
	for (my $i=0; $i<8; $i++) { $ct .= substr($hex, ($key[$i] = int(rand(0xFF)) + 1) << 1, 2); }
	$ct .= '00';
	$ct .= substr($hex, (255 - @ptb) << 1, 2);
	for (my $i=0; $i<@ptb; $i++) { $ct .= substr($hex, ($ptb[$i] ^ $key[$i & 7]) << 1, 2); }
	for (my $i=@ptb; $i<16; $i++) { $ct .= substr($hex, int(rand(0x100)) << 1, 2); }
	return $ct;
}

# Always Return Nicely
1;

###TRUNCATE###

sub bf_decrypt {
	my ($ct) = @_;
	defined($ct) or return undef;
	$ct = pack("H*", $ct);
	length($ct) < 25 and return undef;
	my @ctb = unpack("C*", $ct);

	my $len = 255 - $ctb[8];
	if ($len != 255) {
		# Old style
		@ctb < 9 + $len and return undef;

		# Decrypt message
		my $ptb = '';
		for (my $i=0; $i<$len; $i++) {
			my $value = $ctb[$i+9];
			$value < 0x20 || $value >= 0x7F and return undef;
			my $offset = $value + 0x011D - $ctb[$i & 7];
			$ptb .= chr($offset % 0x5F + 0x20);
		}
		return $ptb;
	}

	# New style
	$len = 255 - $ctb[9];
	@ctb < 10 + $len and return undef;

	# Decrypt message
	my @ptb = ();
	for (my $i=0; $i<$len; $i++) {
		push(@ptb, $ctb[$i + 10] ^ $ctb[$i & 7]);
	}
	my $pt = pack("C*", @ptb);
	return decode_utf8($pt);
}
push @EXPORT_OK, 'bf_decrypt';

# Always Return Nicely
1;

