<?php


/** DB2 Debug Information
 * In order to get a snapshot of DB2 information run the following command in DB2 CLP;
 * 	"update dbm cfg using dft_mon_bufpool on dft_mon_lock on dft_mon_sort on dft_mon_table on dft_mon_uow on  mon_heap_sz 512"
 * Now restart the database 
 *  db2stop force
 *  db2start 
 *  
 * Now use one of the following to get your information;
 *  "get snapshot for dbm" -  Database manager snapshot ~ memory usage and what not
 *  "get snapshot for database on build" - Information specific to build
 *  "get snapshot for bufferpools on build" - buffer pool information
 *  "get snapshot for dynamic sql on build" - guess.. 
 *  "get snapshot for applications on build" - application specific stuff
 *  "get snapshot for locks on build" - Locks/connections applications   
 *  
 * To take db2 out of debug mode, do another stop and start as above.
 *  "update dbm cfg using dft_mon_bufpool off dft_mon_lock off dft_mon_sort off dft_mon_table off dft_mon_uow off  mon_heap_sz 512"
 * Now restart the database          
 */ 

class DB_db2 extends DB_common {
	protected $db2_options = array('db2_attr_case' => DB2_CASE_LOWER);
	protected $statements = array();

	protected function runQuery($query, $params = null) {
		$result = false;
		if (!is_null($params) && is_array($params)) {
			$md5key = md5($query);
			if (!isset($this->statements[$md5key])) {
				$this->statements[$md5key] = db2_prepare($this->handle, $query);
				if (!$this->statements[$md5key]) {
					return new DB_error(db2_stmt_errormsg());
				}
			}
			$result = db2_execute($this->statements[$md5key], $params);
			if ($result) {
				return $this->statements[$md5key];
			} else {
				return new DB_error(db2_stmt_errormsg($this->statements[$md5key]));
			}
		} else {
			$result = db2_exec($this->handle, $query);
			return $result;
		}
	}
	
	public function query($query, $params = null) {
		$result = $this->runQuery($query, $params);
		if ($result === true) {
			return true;
		} else if ($result === false) {
			return false;
		} else if (DB::isError($result)) {
			return $result;
		} else {
			$real_result = new DB_result_db2($result, $this->options);
			$real_result->setHandle($this->handle);
			$real_result->setParameters($params);
			$real_result->setQuery($query);
			return $real_result;
		}
	}
	
	public function getRow($query, $params = null, $fetchmode = null) {
		$result = $this->runQuery($query, $params);
		if ($result === true) {
			return true;
		} else if ($result === false) {
			return false;
		} else if (DB::isError($result)) {
			return new DB_error($query);
		} else {
			$db_result = new DB_result_db2($result, $this->options);
			return $db_result->fetchRow($fetchmode);
		}
	}
	
	public function getCol($query, $index = 0, $params = null) {
		$result = $this->runQuery($query, $params);
		if ($result === true) {
			return true;
		} else if ($result === false) {
			return false;
		} else if (DB::isError($result)) {
			return new DB_error($query);
		} else {
			$column = array();
			while ($row = db2_fetch_array($result)) {
				if (!array_key_exists($index, $row)) {
					$index = 0;
				}
				$column[] = $row[$index];
			}
			return $column;
		}
	}
	
	public function getOne($query, $index = 0, $params = null) {
		$result = $this->runQuery($query, $params);
		if ($result === true) {
			return true;
		} else if ($result === false) {
			return false;
		} else if (DB::isError($result)) {
			return new DB_error($query);
		} else if ($row = db2_fetch_array($result)) {
			if (!$index || !isset($row[$index])) {
				$index = 0;
			}
			return $row[$index];
		} else {
			return false;
		}
	}
	
	public function getSpecialQuery($token) {
		switch ($token) {
			case 'tables':
				return 'SELECT table_name FROM SYSIBM.tables_s WHERE is_insertable_into = \'YES\' and table_schema = CURRENT_SCHEMA';
			default:
				exit($token);
		}
	}
	
	public function quoteIdentifier($ident) {
		return str_replace('"', '":"', $ident);
	}
	
	public function quoteSmart($in, $type = null) {
	    if ($type != null) {
	    	$type = strtolower($type);
		    if ($type == 'string' || strpos($type, 'char') !== false) {
			    return '\'' . ($in) . '\'';
		    } elseif (strpos($type, 'int') !== false) {
		    	return (int) $in;
		    } else {
			    return $in;
		    }
	    }

		if (is_int($in) || is_double($in)) {
			return $in;
		} elseif (is_bool($in)) {
			return $in ? 1 : 0;
		} elseif (is_null($in)) {
			return 'NULL';
		} else {
			return '\'' . $this->escapeSimple($in) . '\'';
		}
    }

	public function escapeSimple($string) {
		return str_replace('\'', '\'\'', $string);
	}
	
	public function tableInfo($param) {
		$info = array();
		if (is_string($param)) {
			$query = 'SELECT * FROM ' . $this->quoteIdentifier($param);
			if ($result = db2_exec($this->handle, $query)) {
				$db2_result = new DB_result_db2($result, $this->options);
				$info = $db2_result->tableInfo();
			}
		} else if ($param instanceof DB_result_db2) {
			$temp_info = $param->tableInfo();
			foreach ($temp_info as $k => $v) {
				$info[$k] = array();
				foreach ($v as $kk => $vv) {
					$info[$k][$kk] = $vv;
				}
			}
		}
		return $info;
	}
	
	protected function connect() {
		if (!function_exists('db2_connect')) {
			throw new BFDBException(translate('database extension not found', 'ibm_db2'));
		}

		$port = (isset($this->dsn_parsed['tcp_port']) ? (int)$this->dsn_parsed['tcp_port'] : 50000);
		$dsn = 'DRIVER={IBM DB2 ODBC DRIVER};DATABASE=' . $this->dsn_parsed['database']
			. ';HOSTNAME=' . $this->dsn_parsed['hostname']
			. ';PORT=' . $port
			. ';PROTOCOL=TCPIP;UID=' . $this->dsn_parsed['username']
			. ';PWD=' . $this->dsn_parsed['password'] . ';';
		
		if (array_key_exists('persistent', $this->options) && $this->options['persistent'] === true) {
			$this->handle = db2_pconnect($dsn, '', '');
		} else {
			$this->handle = db2_connect($dsn, '', '');
		}
		if ($this->handle) {
			db2_set_option($this->handle, $this->db2_options, 1);
			return true;
		} else {
			$this->handle = db2_connect($this->dsn_parsed['database'], $this->dsn_parsed['username'], 
					$this->dsn_parsed['password']);
			if ($this->handle)  {
				@trigger_error(translate('2259'), E_WARNING);
				db2_set_option($this->handle, $this->db2_options, 1);
				return true;
			}  else {
				throw new BFDBException(db2_conn_errormsg());
			}
		}
	}

	public function close() {
		// Close all open statements
		foreach ($this->statements as $stmt) {
			db2_free_stmt($stmt);
		}
		return db2_close($this->handle);
	}
}

class DB_result_db2 extends DB_result {
	protected $num_rows;
	protected $fetched;

	public function fetchRow($fetch_mode = false, $rownum = null) {
		//  This clumsily replicates rownum functionality by resending the request every time after the
		//  first that rownum is called set to 0.  If we ever actually need "row #5" we are out of luck.
		if ($rownum === 0)  {
			if ($this->fetched)  {
				if (isset($this->params)) {
					db2_execute($this->result, $this->params);
				} else {
					db2_free_result($this->result);
					$this->result = db2_exec($this->handle, $this->query);
				}
			} else {
				$this->fetched = true;
			}
		}
		return $this->fetchNext($fetch_mode, $rownum);
	}
	
	public function fetchNext($fetch_mode = false, $rownum = null) {
		if (!$fetch_mode) {
			$fetch_mode = $this->options['mode'];
		}

		switch ($fetch_mode) {
			case DB_FETCHMODE_ARRAY:
				return (false && !is_null($rownum)) ? db2_fetch_array($this->result, ++$rownum) : 
						db2_fetch_array($this->result);
			case DB_FETCHMODE_ASSOC:
				return (false && !is_null($rownum)) ? db2_fetch_assoc($this->result, ++$rownum) : 
						db2_fetch_assoc($this->result);
			case DB_FETCHMODE_OBJECT:
				return (false && !is_null($rownum)) ? db2_fetch_object($this->result, ++$rownum) : 
						db2_fetch_object($this->result);
		}
	}
	
	public function numRows() {
		return db2_num_rows($this->result);
	}
	
	public function free() {
		db2_free_result($this->result);
	}
	
	public function tableInfo() {
		$info = array();
		
		$field_count = db2_num_fields($this->result);
		for ($i = 0; $i < $field_count; $i++) {
			$info[] = array(
							'name' => strtolower(db2_field_name($this->result, $i)),
							'type' => db2_field_type($this->result, $i),
							'len' => db2_field_width($this->result, $i)
							);
		}
		
		return $info;
	}

	public function __construct($res, $opts)  {
		$this->fetched = false;
		parent::__construct($res, $opts);
	}
}

?>
