<?php

$GLOBALS['MYSQLI_DATA_TYPES'] = array(
						   MYSQLI_TYPE_DECIMAL => 'decimal',
						   #MYSQLI_TYPE_NEWDECIMAL => 'decimal',
						   #MYSQLI_TYPE_BIT => 'tinyint',
						   MYSQLI_TYPE_TINY => 'tinyint',
						   MYSQLI_TYPE_SHORT => 'int',
						   MYSQLI_TYPE_LONG => 'int',
						   MYSQLI_TYPE_FLOAT => 'float',
						   MYSQLI_TYPE_DOUBLE => 'double',
						   MYSQLI_TYPE_NULL => 'null',
						   MYSQLI_TYPE_TIMESTAMP => 'timestamp',
						   MYSQLI_TYPE_LONGLONG => 'bigint',
						   MYSQLI_TYPE_INT24 => 'mediumint',
						   MYSQLI_TYPE_DATE => 'date',
						   MYSQLI_TYPE_TIME => 'time',
						   MYSQLI_TYPE_DATETIME => 'datetime',
						   MYSQLI_TYPE_YEAR => 'year',
						   MYSQLI_TYPE_NEWDATE => 'date',
						   MYSQLI_TYPE_ENUM => 'enum',
						   MYSQLI_TYPE_SET => 'set',
						   MYSQLI_TYPE_TINY_BLOB => 'tinyblob',
						   MYSQLI_TYPE_MEDIUM_BLOB => 'mediumblob',
						   MYSQLI_TYPE_LONG_BLOB => 'longblob',
						   MYSQLI_TYPE_BLOB => 'blob',
						   MYSQLI_TYPE_VAR_STRING => 'varchar',
						   MYSQLI_TYPE_STRING => 'char',
						   MYSQLI_TYPE_GEOMETRY => 'geometry',
						   );

class DB_mysqli extends DB_common {
	public function query($query, $params = null) {
		//file_put_contents('/tmp/mysql_debugging', $query . "\n", FILE_APPEND);
		if ($result = $this->handle->query($query)) {
			if ($result === true) {
				return true;
			} else if ($result === false) {
				return false;
			} else {
				return new DB_result_mysqli($result, $this->options);
			}
		} else {
			return new DB_error($this->handle->error);
		}
	}
	
	public function getRow($query, $params = null, $fetchmode = null) {
		if ($result = $this->handle->query($query)) {
			$db_result = new DB_result_mysqli($result, $this->options);
			$row = $db_result->fetchRow($fetchmode);
			$db_result->free();
			return $row;
		}
		return false;
	}
	
	public function getCol($query, $index = 0) {
		$column = array();
		if ($result = $this->handle->query($query)) {
			while ($row = $result->fetch_row()) {
				if (!array_key_exists($index, $row)) {
					$index = 0;
				}
				$column[] = $row[$index];
			}
			$result->free();
		}
		return $column;
	}
	
	public function getOne($query, $index = 0) {
		if ($result = $this->handle->query($query)) {
			if ($row = $result->fetch_row()) {
				if (!array_key_exists($index, $row)) {
					$index = 0;
				}
				$result->free();
				return $row[$index];
			}
			$result->free();
		}
		return false;
	}
	
	public function getSpecialQuery($token) {
		switch ($token) {
			case 'tables':
				return 'SHOW TABLES';
			default:
				exit($token);
		}
	}
	
	public function quoteIdentifier($ident) {
		return '`' . $this->handle->real_escape_string($ident) . '`';
	}
	
	public function quoteSmart($in, $type = null) {
	    if ($type != null) {
		    if ($type == 'text' || $type == 'blob' || strpos($type, 'char') !== false) {
			    return '\'' . $this->handle->real_escape_string($in) . '\'';
		    } elseif ($type == 'int') {
		    	return (int) $in;
		    } else {
			    return $in;
		    }
	    }

        if (is_int($in) || is_double($in)) {
            return $in;
        } elseif (is_bool($in)) {
            return $in ? 1 : 0;
        } elseif (is_null($in)) {
            return 'NULL';
        } else {
            return '\'' . $this->handle->real_escape_string($in) . '\'';
        }
    }
	
	public function tableInfo($param) {
		global $MYSQLI_DATA_TYPES;
		
		$info = array();
		if (is_string($param)) {
			$query = 'SELECT * FROM ' . $this->quoteIdentifier($param) . ' LIMIT 0';
			if ($result = $this->handle->query($query)) {
				$temp_info = $result->fetch_fields();
				foreach ($temp_info as $temp) {
					$info[] = array(
									'name' => $temp->name,
									'type' => $MYSQLI_DATA_TYPES[$temp->type],
									'len' => $temp->length
									);
				}
			}
		} else if ($param instanceof DB_result_mysqli) {
			$temp_info = $param->tableInfo();
			foreach ($temp_info as $k => $v) {
				$info[$k] = array();
				foreach ($v as $kk => $vv) {
					$info[$k][$kk] = $vv;
				}
			}
		}
		return $info;
	}
	
	protected function connect() {
		if (!function_exists('mysqli_connect')) {
			throw new BFDBException(translate('database extension not found', 'mysqli'));
		}

		$port = (isset($this->dsn_parsed['tcp_port']) ? (int)$this->dsn_parsed['tcp_port'] : 3306);
		$this->handle = new mysqli($this->dsn_parsed['hostname'], $this->dsn_parsed['username'], $this->dsn_parsed['password'], null, $port);
		$connection_error = mysqli_connect_error();
		if (empty($connection_error)) {
			if ($this->handle->select_db($this->dsn_parsed['database'])) {
				$this->handle->set_charset($GLOBALS['GLOBAL_DB_CHARSET']);
				return true;
			} else {
				throw new BFDBException($this->handle->error);
			}
		} else {
			throw new BFDBException(mysqli_connect_error());
		}
	}

	public function close() {
		return $this->handle->close();
	}
}

class DB_result_mysqli extends DB_result {
	public function fetchRow($fetch_mode = false, $rownum = null) {
		if (isset($rownum)) {
			$this->result->data_seek($rownum);
		}
		return $this->fetchNext($fetch_mode);
	}
	
	/**
	 * Like fetchRow, but without the rownum
	 */
	public function fetchNext($fetch_mode = false) {
		if (!$fetch_mode) {
			$fetch_mode = $this->options['mode'];
		}
		switch ($fetch_mode) {
			case DB_FETCHMODE_ARRAY:
				return $this->result->fetch_array();
			case DB_FETCHMODE_ASSOC:
				return $this->result->fetch_assoc();
			case DB_FETCHMODE_OBJECT:
				return $this->result->fetch_object();
		}
	}
	
	public function numRows() {
		return $this->result->num_rows;
	}
	
	public function free() {
		$this->result->free_result();
	}
	
	public function tableInfo() {
		global $MYSQLI_DATA_TYPES;
		
		$info = array();
		$temp_info = $this->result->fetch_fields();
		
		foreach ($temp_info as $temp) {
			$info[] = array(
							'name' => $temp->name,
							'type' => $MYSQLI_DATA_TYPES[$temp->type],
							'len' => $temp->length
							);
		}
		
		return $info;
	}
}

?>
