<?php

$GLOBALS['OCI_TYPES'] = array(
		'NUMBER' => SQLT_INT,
		'INT' => SQLT_INT,
		'TINYINT' => SQLT_INT,
		'CHAR' => SQLT_CHR,
		'BLOB' => SQLT_BLOB,
		'VARCHAR2' => SQLT_CHR,
		'VARCHAR' => SQLT_CHR
	);

class DB_oci8 extends DB_common {
	protected $statements = array();
	
	/**
	 * $params MUST get passed as an associative array
	 * so that the keys match the string names in the
	 * text of the query itself.
	 */
	public function runQuery($query, $params = null) {
		// Create the statement reference if it doesn't already exist
		$md5key = md5($query);
		if (!isset($this->statements[$md5key])) {
			$this->statements[$md5key] = oci_parse($this->handle, $query);
			if (!$this->statements[$md5key]) {
				$error = oci_error($this->handle);
				return new DB_error($error['message']);
			}
		}
		$result = false;

		if (!is_null($params) && is_array($params)) {
			// Optionally prefetch the paginated amount
			if (isset($params['prefetch'])) {
				oci_set_prefetch($this->statements[$md5key], $params['prefetch']);
				unset($params['prefetch']);
			}

			foreach ($params as $name => $value) {
				$params[$name][1] = strtoupper($params[$name][1]);
				if (!$params[$name][0] && $params[$name][0] !== 0 && $params[$name][0] !== '0' && 
						($params[$name][1] == 'VARCHAR2' || $params[$name][1] == 'CHAR' ||
						$params[$name][1] == 'VARCHAR'))  {
					$params[$name][0] = ' ';
				}
				oci_bind_by_name(
						$this->statements[$md5key],
						$name,
						$params[$name][0],
						-1,
						$GLOBALS['OCI_TYPES'][$params[$name][1]]
					);
			}
			$result = oci_execute($this->statements[$md5key]);
			if ($result) {
				return $this->statements[$md5key];
			} else {
				$error = oci_error($this->handle);
				print_r(new Exception($error['message']));
				return false;
			}
		} else {
			if (oci_execute($this->statements[$md5key])) {
				return $this->statements[$md5key];
			} else {
				return new DB_error($query);
			}
		}
	}
	
	public function query($query, $params = null) {
		$result = $this->runQuery($query, $params);
		if ($result === true) {
			return true;
		} else if ($result === false) {
			return false;
		} else if (DB::isError($result)) {
			return $result;
		} else {
			$real_result = new DB_result_oci($this, $result, $this->options);
			$real_result->setHandle($this->handle);
			$real_result->setParameters($params);
			$real_result->setQuery($query);
			return $real_result;
		}
	}
	
	public function getRow($query, $params = null, $fetchmode = null) {
		$result = $this->runQuery($query, $params);
		if ($result === true || $result === false) {
			return $result;
		} else if (DB::isError($result)) {
			return new DB_error($query);
		} else {
			$db_result = new DB_result_oci($this, $result, $this->options);
			return $db_result->fetchRow($fetchmode);
		}
	}
	
	public function getCol($query, $index = 0, $params = null) {
		$result = $this->runQuery($query, $params);
		if ($result === true || $result === false) {
			return $result;
		} else if (DB::isError($result)) {
			return new DB_error($query);
		} else {
			$column = array();
			while ($row = oci_fetch_array($result)) {
				if (!array_key_exists($index, $row)) {
					$index = 0;
				}
				$column[] = ($row[$index] == ' ' ? '' : $row[$index]);
			}
			return $column;
		}
	}
	
	public function getOne($query, $index = 0, $params = null) {
		$result = $this->runQuery($query, $params);
		if ($result === true || $result === false) {
			return $result;
		} else if (DB::isError($result)) {
			return new DB_error($query);
		} else if ($row = oci_fetch_array($result)) {
			if (!$index || !isset($row[$index])) {
				$index = 0;
			}
			return ($row[$index] == ' ' ? '' : $row[$index]);
		} else {
			return false;
		}
	}
	
	public function getSpecialQuery($token) {
		switch ($token) {
			case 'tables':
				return 'SELECT table_name FROM user_tables';
			default:
				exit($token);
		}
	}
	
	public function quoteIdentifier($ident) {
		return '"' . strtoupper(str_replace('"', '"""', $ident)) . '"';
	}
	
	public function quoteSmart($in, $type = null) {
	    if ($type != null) {
	    	$type = strtolower($type);
		    if ($type == 'string' || strpos($type, 'char') !== false) {
			    return '\'' . (($in || $in === 0 || $in === '0') ? $in : ' ') . '\'';
		    } elseif (strpos($type, 'int') !== false || $type == 'number') {
		    	return (int) $in;
		    } else {
			    return $in;
		    }
	    }

		if (is_int($in))  {
			return (int) $in;
		}  elseif (is_double($in)) {
			return $in;
		} elseif (is_bool($in)) {
			return $in ? 1 : 0;
		} elseif (is_null($in)) {
			return ' ';
		} else {
			$ret = $this->escapeSimple($in);
			return '\'' . (($ret || $ret === 0 || $ret === '0') ? $ret : ' ') . '\'';
		}
    }

	public function escapeSimple($string) {
		return str_replace('\'', '\'\'', $string);
	}
	
	public function tableInfo($param) {
		$info = array();
		if (is_string($param)) {
			// I can't limit the query, but I can make sure it returns no rows.
			$query = 'SELECT * FROM ' . $this->quoteIdentifier($param) . ' WHERE 0 = 1';
			$result = oci_parse($this->handle, $query);
			if (oci_execute($result)) {
				$oci_result = new DB_result_oci($this, $result, $this->options);
				$info = $oci_result->tableInfo();
			}
		} else if ($param instanceof DB_result_oci) {
			$temp_info = $param->tableInfo();
			foreach ($temp_info as $k => $v) {
				$info[$k] = array();
				foreach ($v as $kk => $vv) {
					$info[$k][$kk] = $vv;
				}
			}
		}
		return $info;
	}
	
	protected function connect() {
		if (!function_exists('oci_connect')) {
			throw new BFDBException(translate('database extension not found', 'oci'));
		}

		if (array_key_exists('persistent', $this->options) && $this->options['persistent'] === true) {
			$this->handle = oci_pconnect($this->dsn_parsed['username'], $this->dsn_parsed['password'], $this->dsn_parsed['database']);
		} else {
			$this->handle = oci_connect($this->dsn_parsed['username'], $this->dsn_parsed['password'], $this->dsn_parsed['database']);
		}
		if ($this->handle) {
			return true;
		} else {
			$err = oci_error();
			throw new BFDBException($err['message']);
		}
	}

	public function close() {
		// Close all open statements
		foreach ($this->statements as $stmt) {
			oci_free_statement($stmt);
		}
		oci_close($this->handle);
	}
}

//  We're going to have to do all kinds of fun postprocessing in order to allow this Happy Fun database
//  to work properly.
class DB_result_oci extends DB_result {
	protected $num_rows;
	protected $fetched;
	protected $db;

	public function fetchRow($fetch_mode = false, $rownum = null) {
		//  This clumsily replicates rownum functionality by resending the request every time after the
		//  first that rownum is called set to 0.  If we ever actually need "row #5" we are out of luck.
		if ($rownum === 0)  {
			if ($this->fetched && $this->result)  {
				// Only bind parameters again if using prepared statements
				if (isset($this->params)) {
					if (isset($this->params['prefetch'])) {
						oci_set_prefetch($this->result, $this->params['prefetch']);
						unset($this->params['prefetch']);
					}
					foreach ($this->params as $name => $value) {
						if (!$this->params[$name][0] && ($this->params[$name][1] == 'VARCHAR2' || 
								$this->params[$name][1] == 'CHAR' || $this->params[$name][1] == 'VARCHAR'))  {
							$this->params[$name][0] = ' ';
						}
						oci_bind_by_name($this->result, $name, $this->params[$name][0], -1, 
								$GLOBALS['OCI_TYPES'][$this->params[$name][1]]);
					}
				}
				oci_execute($this->result);
			}  else {
				$this->fetched = true;
			}
		}
		return $this->fetchNext($fetch_mode);
	}
	
	public function fetchNext($fetch_mode = false) {
		$result = null;
		if (!$fetch_mode) {
			$fetch_mode = $this->options['mode'];
		}

		switch ($fetch_mode) {
			case DB_FETCHMODE_ARRAY:
				$result = oci_fetch_row($this->result);
				if ($result)  {
					foreach ($result as $k => $v)  { $result[$k] = ($v == ' ' ? '' : $v); }
				}
				break;
			case DB_FETCHMODE_ASSOC:
				// Smell that? That smells like good, old fashioned hack.
				// You see, Oracle returns field names in UPPERCASE.  Handy, no?
				$oldresult = oci_fetch_array($this->result, OCI_ASSOC+OCI_RETURN_NULLS);
				$result = array();
				if ($oldresult)  {
					foreach ($oldresult as $k => $v)  { $result[strtolower($k)] = ($v == ' ' ? '' : $v); }
				}
				break;
			case DB_FETCHMODE_OBJECT:
				$result = oci_fetch_object($this->result);
				if ($result)  {
					foreach (get_object_vars($result) as $k=>$v)  {
						$k = strtolower($k);
						$result->$k = ($v == ' ' ? '' : $v);
					}
				}
				break;
		}
		return $result;
	}
	
	public function numRows() {
		if (!isset($this->num_rows))  {
			if (!preg_match('/^select/i', $this->query))  {
				$this->num_rows = oci_num_rows($this->result);
			}  else {
				if ($this->params)  {
					$result = $this->db->getOne('SELECT count(*) FROM (' . $this->query . ')',
							0, $this->params);
					if ($result && !DB::isError($result)) {
						$this->num_rows = $result;
					}
				}  else {
					$result = oci_parse($this->handle, 'SELECT count(*) FROM (' . $this->query . ')');
					$row = null;
					if ($result && oci_execute($result) && $row = oci_fetch_array($result)) {
						$this->num_rows = $row[0];
					}
				}
			}
		}
		return $this->num_rows;
	}
	
	public function free() {
		oci_free_statement($this->result);
	}
	
	public function tableInfo() {
		$info = array();
		
		$field_count = oci_num_fields($this->result);
		for ($i = 1; $i <= $field_count; $i++) {
			$info[] = array(
							'name' => strtolower(oci_field_name($this->result, $i)),
							'type' => oci_field_type($this->result, $i),
							'len' => oci_field_size($this->result, $i)
							);
		}
		
		return $info;
	}
	
	public function __construct($db, $res, $opts)  {
		$this->db = $db;
		$this->fetched = false;
		parent::__construct($res, $opts);
	}
}

?>
