<?php

class DB_odbc extends DB_common {
	protected $sql_server_version = false;
	
	public function realQuery($query, $params = null)  {
		if ($params)  {
			if (isset($params['rowcount']) && $params['rowcount'])  {
				@odbc_exec($this->handle, 'set rowcount ' . $params['rowcount']);
				if (isset($params['select2']) && $params['select2'])  {
					@odbc_exec($this->handle, $query);
					$result = @odbc_exec($this->handle, $params['select2']);
					@odbc_exec($this->handle, $this->params['drop']);
				}  else {
					$result = @odbc_exec($this->handle, $query);
				}
				@odbc_exec($this->handle, 'set rowcount 0');
				return $result;
			}
		}  else {
			return @odbc_exec($this->handle, $query);
		}
	}
	
	public function query($query, $params = null) {
		if ($result = $this->realQuery($query, $params)) {
			if ($result === true) {
				return true;
			} else if ($result === false) {
				return false;
			} else {
				$real_result = new DB_result_odbc($this, $result, $this->options);
				$real_result->setQuery($query);
				return $real_result;
			}
		} else {
			return new DB_Error(odbc_errormsg($this->handle));
		}
	}
	
	public function getRow($query, $params = null, $fetchmode = null) {
		if ($result = $this->realQuery($query, $params)) {
			$db_result = new DB_result_odbc($this, $result, $this->options);
			$ret = $db_result->fetchRow($fetchmode);
			$db_result->free();
			return $ret;
		}
		return false;
	}
	
	public function getCol($query, $index = 0) {
		$column = array();
		if ($result = $this->realQuery($query)) {
			$row = array();
			while ($cols = odbc_fetch_into($result, $row, 0)) {
				if (!array_key_exists($index, $row)) {
					$index = 0;
				}
				$column[] = ($row[$index] == ' ' ? '' : $row[$index]);
			}
			odbc_free_result($result);
		}
		return $column;
	}
	
	public function getOne($query, $index = 0) {
		if ($result = $this->realQuery($query)) {
			$row = array();
			$cols = odbc_fetch_into($result, $row);
			if ($cols !== false) {
				if (!isset($row[$index])) {
					$index = 0;
				}
				odbc_free_result($result);
				return ($row[$index] == ' ' ? '' : $row[$index]);
			}
		}
		return false;
	}
	
	public function getSpecialQuery($token) {
		global $controller;
		switch ($token) {
			case 'tables':
				if ($controller->getDBType()  == 'sqlserver')  {
					return 'SELECT table_name FROM information_schema.tables';
				}  elseif ($controller->getDBType() == 'sybase')  {
					return 'select name from sysobjects where type=\'U\'';
				}
			default:
				exit($token);
		}
	}
	
	public function quoteIdentifier($ident) {
		return str_replace('"', '":"', $ident);
	}
	
	public function quoteSmart($in, $type = null) {
	    if ($type != null) {
	    	$type = strtolower($type);
		    if ($type == 'string' || strpos($type, 'char') !== false) {
			    return '\'' . $this->escapeSimple($in) . '\'';
		    } elseif (strpos($type, 'int') !== false) {
		    	return (int) $in;
		    } else {
			    return $in;
		    }
	    }

		if (is_int($in) || is_double($in)) {
			return $in;
		} elseif (is_bool($in)) {
			return $in ? 1 : 0;
		} elseif (is_null($in)) {
			return 'NULL';
		} else {
			return '\'' . $this->escapeSimple($in) . '\'';
		}
    }

	public function escapeSimple($string) {
		return str_replace('\'', '\'\'', $string);
	}
	
	public function tableInfo($param) {
		$info = array();
		if (is_string($param)) {
			$query = 'SELECT * FROM ' . $this->quoteIdentifier($param) . ' WHERE 0 = 1';
			if ($result = $this->realQuery($query)) {
				$odbc_result = new DB_result_odbc($this, $result, $this->options);
				$info = $odbc_result->tableInfo();
			}
		} else if ($param instanceof DB_result_odbc) {
			$temp_info = $param->tableInfo();
			foreach ($temp_info as $k => $v) {
				$info[$k] = array();
				foreach ($v as $kk => $vv) {
					$info[$k][$kk] = $vv;
				}
			}
		}
		return $info;
	}
	
	protected function connect() {
		if (!function_exists('odbc_connect')) {
			throw new BFDBException(translate('database extension not found', 'odbc'));
		}
		if (array_key_exists('persistent', $this->options) && $this->options['persistent'] === true) {
			$this->handle = odbc_pconnect($this->dsn_parsed['database'], $this->dsn_parsed['username'], $this->dsn_parsed['password']);
		} else {
			$this->handle = odbc_connect($this->dsn_parsed['database'], $this->dsn_parsed['username'], $this->dsn_parsed['password']);
		}
		if ($this->handle) {
			return true;
		} else {
			throw new BFDBException(odbc_errormsg());
		}
	}

	public function close() {
		odbc_close($this->handle);
	}

	public function getType() {
		return $this->dsn_parsed['php_type'];
	}

	public function getSQLServerVersion()  {
		global $controller;
		if ($this->sql_server_version === false)  {
			if ($controller->getDBType()  != 'sqlserver')  {
				$this->sql_server_version = 0;
			}  else {
				$this->sql_server_version = preg_match('/SQL Server 2005/i', 
						$this->getOne('SELECT @@VERSION')) ? 2005 : 2000;
			}
		}
		return $this->sql_server_version;
	}
}

class DB_result_odbc extends DB_result {
	protected $num_rows;
	protected $db;
	
	public function fetchRow($fetch_mode = false, $rownum = null) {
		if (!$fetch_mode) {
			$fetch_mode = $this->options['mode'];
		}

		if (!is_null($rownum) && !$rownum)  {$rownum = 1;}
		switch ($fetch_mode) {
			case DB_FETCHMODE_ARRAY:
				// Smell that? That smells like good, old fashioned hack.
				$rows = array();
				if (!is_null($rownum)) { 
					odbc_fetch_into($this->result, $rows, $rownum);
				}  else {
					odbc_fetch_into($this->result, $rows);
				}
				if ($rows)  {
					foreach ($rows as $k => $v)  { 
						$rows[$k] = ($v == ' ' ? '' : $v); 
					}
				}
				return $rows;
			case DB_FETCHMODE_ASSOC:
				if (!is_null($rownum)) { 
				   $rows = odbc_fetch_array($this->result, $rownum);
				}  else {
				   $rows = odbc_fetch_array($this->result);
				}
				if ($rows)  {
					if (isset($rows['__temp_key']))  {
						unset($rows['__temp_key']);
					}
					if (isset($rows['__rownum']))  {
						unset($rows['__rownum']);
					}
					foreach ($rows as $k => $v)  { 
						$rows[$k] = ($v == ' ' ? '' : $v); 
					}
				}
				return $rows;
			case DB_FETCHMODE_OBJECT:
				$result = (!is_null($rownum)) ? odbc_fetch_object($this->result, $rownum) : 
						odbc_fetch_object($this->result);
				if ($result)  {
					if (isset($result->__temp_key))  {
						unset($result->__temp_key);
					}
					if (isset($result->__rownum))  {
						unset($result->__rownum);
					}
					foreach (get_object_vars($result) as $k=>$v)  {
						$result->$k = ($v == ' ' ? '' : $v);
					}
				}
				return $result;
		}
	}
	
	public function fetchNext($fetch_mode = false, $rownum = null) {
		return $this->fetchRow($fetch_mode, $rownum);
	}
	
	/**
	 * There IS a function which ostensibly returns the number of result rows.  However, in SQL server,
	 * half the time it doesn't work.  This being the case... another query.
	 */
	public function numRows() {
		if (!isset($this->num_rows))  {
			$result = $this->db->getOne('SELECT count(*) FROM (' . $this->query . ') origquery');
			$this->num_rows = $result;
		}
		return $this->num_rows;
	}
	
	public function free() {
		odbc_free_result($this->result);
	}
	
	public function tableInfo() {
		$info = array();

		$field_count = odbc_num_fields($this->result);
		for ($i = 1; $i <= $field_count; $i++) {
			if (odbc_field_name($this->result, $i) == '__rownum' || 
					odbc_field_name($this->result, $i) == '__temp_key')  {
				continue;
			}
			$info[] = array(
							'name' => odbc_field_name($this->result, $i),
							'type' => odbc_field_type($this->result, $i),
							'len' => odbc_field_len($this->result, $i)
							);
		}
		
		return $info;
	}

	public function __construct($db, $res, $opts)  {
		$this->db = $db;
		parent::__construct($res, $opts);
	}
}

?>
