<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFExceptions
 * 
 * The BFExceptions provide a way of tracking errors as they come up,
 * for the users to report, but mainly for us to debug... Each object
 * or main piece of functionality can extend the BFException and throw
 * them as needed.
 * This file also contains the code setting a global exception
 * handler for the entire app.
 * @package Console
 * @subpackage Core
 */
if (!defined('BF_ERR_LOGGING')) define('BF_ERR_LOGGING', true);
if (!defined('BF_ERR_DISPLAY')) define('BF_ERR_DISPLAY', true);

set_exception_handler('bf_catch_exception');

function bf_catch_exception($e) {
	if (BF_ERR_LOGGING && !($e instanceof BFDBException)) {
		BFExceptionHandler::logError($e);
	}
	if (BF_ERR_DISPLAY) {
		try {
			$GLOBALS['controller']->exitWithError($e);
		} catch(Exception $err) {
			BFExceptionHandler::showError($err);
		}
	}
}

/**
 * BFException
 * The Zeus of all Exceptions to get used in the user interface.
 * In other words, it will have all Exceptions used here as its
 * children and rule over the default behaviors.
 */
class BFException extends Exception {
	public function toString() {
		return get_class($this) . '[' . $this->getCode() . ']: ' . $this->getMessage() . "\n" . 
				translate('trace') . ': ' . print_r($this->getTrace(), 1);
	}
	public function __construct($msg, $code = 0, $log = false) {
		parent::__construct($msg, $code);
		if ($log && !($this instanceof BFDBException))  {
			BFExceptionHandler::logError($this, $GLOBALS['controller']->getDB());
		}
	}
}

/**
 * BFDBException
 * Since DB_error doesn''t actually extend Exception...
 */
class BFDBException extends BFException {
	public function __construct($e, $code = 0) {
		Exception::__construct($e, $code);
		$this->message = $e;
	}
}

/**
 * BFAccessException
 * This Exception retrieves the description for a given access key
 * (if possible) and throws an Exception with a message based on it.
 */
class BFAccessException extends BFException {
	public function __construct($key) {
		$this->message = translate('2154', $key);
		try {
			$GLOBALS['controller']->require_module('BFPermission');
			$perm = new BFPermission();
			if ($perm->select($perm->primary_key, $key)) {
				$this->message = translate('2154', $perm->get('bf_description'));
			}
		} catch (Exception $e) {
		}
		parent::__construct($this->message);
	}
}

/**
 * BFExceptionHandler
 * The BFExceptionHandler class exists to pull out error reporting
 * and logging from the controller and have the ability to extend
 * and/or override the behaviors.
 */
class BFExceptionHandler {
	public static function logError(BFException $e, $db = null, $severity = 'B', $pid = null) {
		return;
		global $user, $controller;
		$controllerset = $controller instanceof CentralController;
		if (!($db instanceof DB_common)) {
			if ($controllerset) {
				$controller->exitWithError($e);
			} else {
				self::printDumbErrorPage($e->getMessage());
			}
			return;
		}
		
		$uid = null;
		if (isset($user)) $uid = $user->get('bf_userid') ? $uid : null;
		$sql = 'INSERT INTO ' . $db->quoteIdentifier('bf_messages') . ' VALUES (?, ?, ?, ?, ?)';
		$params = array(
					$db->quoteSmart(time()),
					$db->quoteSmart($uid),
					$db->quoteSmart($pid),
					$db->quoteSmart($severity),
					$db->quoteSmart($e->toString())
				);
		$err = $db->query($sql, $params);
		if (DB::isError($err)) {
			if ($controllerset) {
				$controller->exitWithError($e);
			} else {
				self::printDumbErrorPage($e->getMessage());
			}
		}
	}
	
	/**
	 * printDumbErrorPage
	 *
	 * When all else fails, call this with a reason why.
	 */
	public static function printDumbErrorPage($msg) {
		$errorpage = @file_get_contents($GLOBALS['GLOBAL_SETTINGS']['site']['errorpage']);
		if (!$errorpage) $errorpage = BF_LANG_ERR_ERRORPAGE;
		printf($errorpage, BF_WEBROOT, $msg, translate('2282'));
		exit();
	}
	
	public static function showError($e) {
		echo(translate('2155', $e->getMessage(), $e->getFile(), $e->getLine()));
	}
}

?>
