<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * config
 * This acts (more or less) like an about:config a la Mozilla for the web app.
 * Lots of defaults, options, strings, regular expressions and just about
 * anything else that would otherwise get implemented as an inline constant
 * should live in here. We don''t have to tell anyone about it as far as users
 * go, but it should make for a useful tool down the road instead of having to
 * hunt for embedded strings...
 * @package Console
 * @subpackage Core
 */

// Just in case...
$GLOBAL_VERSION = array();
# $GLOBAL_VERSION['CURRENT']			= BF_CONSOLE_VERSION;
$GLOBAL_VERSION['CURRENT']			= BF_SCHEMA_VERSION;

// PEAR::DB options - DO NOT ADD NON-DB OPTIONS HERE, IT WILL BREAK
$GLOBAL_DB_OPTIONS = array();
$GLOBAL_DB_OPTIONS['portability']	= 63;
$GLOBAL_DB_OPTIONS['debug']			= 2;
$GLOBAL_DB_OPTIONS['persistent']	= true;

// The number of tries to connect to the database before giving up and going home
$GLOBAL_DB_ATTEMPTS					= 11;
$GLOBAL_DB_CHARSET					= 'UTF8';

/**
 * This specifies the PHP database driver abstraction layer to use:
	BFDB.php // PEAR::DB replacement
	DB.php // PEAR::DB
 */
$GLOBAL_DB_ENGINE					= 'BFDB.php';

// Templates for each request type as an array that can have more values appended to it
$GLOBAL_TEMPLATES                   = array();
$GLOBAL_TEMPLATES[BF_REQUEST_XHTML] = 'xhtml.tpl';
$GLOBAL_TEMPLATES[BF_REQUEST_XML]   = 'xml.tpl';
$GLOBAL_TEMPLATES[BF_REQUEST_RSS2]	= 'rss2.tpl';
$GLOBAL_TEMPLATES[BF_REQUEST_RAW]   = 'raw.tpl';

// HTTP headers, call also get more values appended to it
$GLOBAL_HEADERS                     = array();
$GLOBAL_HEADERS[BF_REQUEST_XHTML]   = 'Content-type: text/html; charset: UTF-8';
$GLOBAL_HEADERS[BF_REQUEST_XML]     = 'Content-type: text/xml; charset: UTF-8';
$GLOBAL_HEADERS[BF_REQUEST_RSS2]	= 'Content-type: application/rss+xml; charset: UTF-8';
$GLOBAL_HEADERS[BF_REQUEST_RAW]     = 'Content-type: text/plain; charset: UTF-8';

$GLOBAL_XMLHEADER					= '<?xml version="1.0" encoding="UTF-8"?>'."\n";

// Global site settings for use by jTE when an XHTML request
$GLOBAL_SETTINGS                            = array();
$GLOBAL_SETTINGS['site']                    = array();
$GLOBAL_SETTINGS['site']['loginredirect']   = 'fullcontrol/';
$GLOBAL_SETTINGS['site']['errorpage']       = BF_ROOT . '/errorpage.html';
$GLOBAL_SETTINGS['site']['errortemplate']   = 'error.tpl';
$GLOBAL_SETTINGS['site']['navstate']        = 1;
$GLOBAL_SETTINGS['site']['actionkey']       = 'action';

// Default page refresh settings
$GLOBAL_SETTINGS['site']['refresh']     = array();
$GLOBAL_SETTINGS['site']['refresh'][0] = 'None';
$GLOBAL_SETTINGS['site']['refresh'][5] = '5 sec';
$GLOBAL_SETTINGS['site']['refresh'][10] = '10 sec';
$GLOBAL_SETTINGS['site']['refresh'][15] = '15 sec';
$GLOBAL_SETTINGS['site']['refresh'][30] = '30 sec';
$GLOBAL_SETTINGS['site']['refresh'][45] = '45 sec';
$GLOBAL_SETTINGS['site']['refresh'][60] = '60 sec';

// Extend the max execution time
$GLOBAL_SETTINGS['site']['max_execution_time'] = 3600;

// Default reporting settings
$GLOBAL_SETTINGS['reporting']           = array();
$GLOBAL_SETTINGS['reporting']['count']  = 15;
$GLOBAL_SETTINGS['reporting']['offset'] = 0;

$GLOBAL_SETTINGS['formatting']['date'] = 'h:i a m/d/y';

// Default database settings
$GLOBAL_DATABASE             = array();
$GLOBAL_DATABASE['type']     = 'local';
$GLOBAL_DATABASE['provider'] = 'db2';
$GLOBAL_DATABASE['hostname'] = 'localhost';
$GLOBAL_DATABASE['username'] = 'build';
$GLOBAL_DATABASE['password'] = 'build';
$GLOBAL_DATABASE['database'] = 'build';
$GLOBAL_DATABASE['instance'] = 'build';
$GLOBAL_DATABASE['tcp_port'] = '50000';
$GLOBAL_DATABASE['valid_types'] = array('generic','geo','env','dn','message','vars','command','body','data','group','interface','text','step');

// Supported Database Types
$GLOBAL_DATABASE_TYPES				= array();
$GLOBAL_DATABASE_TYPES['db2']		= 'db2';
$GLOBAL_DATABASE_TYPES['mysql']		= 'mysql';
$GLOBAL_DATABASE_TYPES['oracle']	= 'oracle';
$GLOBAL_DATABASE_TYPES['postgres']	= 'postgres';
$GLOBAL_DATABASE_TYPES['sqlserver']	= 'sql server';
$GLOBAL_DATABASE_TYPES['sybase']	= 'sybase';

// Configuration files in the order of priority
$GLOBAL_CONFIGS   = array();
$GLOBAL_CONFIGS[] = BF_ROOT . '/buildforge.conf';
$GLOBAL_CONFIGS[] = BF_ROOT . '/../buildforge.conf';
$GLOBAL_CONFIGS[] = BF_ROOT . '/../engine/buildforge.conf';
$GLOBAL_CONFIGS[] = BF_ROOT . '/../Platform/buildforge.conf';

// Master configuration files in the order of priority
$GLOBAL_MASTERS   = array();
$GLOBAL_MASTERS[] = BF_ROOT . '/master.conf';
$GLOBAL_MASTERS[] = BF_ROOT . '/../master.conf';
$GLOBAL_MASTERS[] = BF_ROOT . '/../engine/master.conf';
$GLOBAL_MASTERS[] = BF_ROOT . '/../Platform/master.conf';

// Cookie settings
$GLOBAL_COOKIE          = array();
$GLOBAL_COOKIE['salt']  = 'something creative';
$GLOBAL_COOKIE['name']  = 'bfsession';

// Default locale string
$LANG_DEFAULT			= 'en_US';

$TIME_FORMATS = array(
	'de_DE' => array(
		'date' => array(
			'short' => '%d.%m.%y',
			'medium' => '%d.%m.%Y',
			'long' => '%e %B %Y',
			'full' => '%A, %e %B %Y',
		),
		'time' => array(
			'short' => '%H:%M',
			'medium' => '%H:%M:%S',
			'long' => '%H:%M:%S %Z',
			'full' => '%I:%M Uhr %Z',
		),
		'datetime' => '{date} {time}'
	),
	'en_US' => array(
		'date' => array(
			'short' => '%m/%d/%y',
			'medium' => '%b %e %y',
			'long' => '%B %e %Y',
			'full' => '%A %B %e %Y',
		),
		'time' => array(
			'short' => '%I:%M%p',
			'medium' => '%I:%M:%S%p',
			'long' => '%I:%M:%S%p %Z',
			'full' => '%I:%M:%S%p %Z',
		),
		'datetime' => '{date} {time}'
	),
	'fr_FR' => array(
		'date' => array(
			'short' => '%d/%m/%y',
			'medium' => '%e %b %y',
			'long' => '%e %B %Y',
			'full' => '%A %e %B %Y',
		),
		'time' => array(
			'short' => '%H:%M',
			'medium' => '%H:%M:%S',
			'long' => '%H:%M:%S %Z',
			'full' => '%H h %M %Z',
		),
		'datetime' => '{date} {time}'
	),
	'it_IT' => array(
		'date' => array(
			'short' => '%d/%m/%y',
			'medium' => '%d/%b/%y',
			'long' => '%e %B %Y',
			'full' => '%A %e %B %Y',
		),
		'time' => array(
			'short' => '%H:%M',
			'medium' => '%H:%M:%S',
			'long' => '%H:%M:%S %Z',
			'full' => '%H:%M:%S %Z',
		),
		'datetime' => '{date} {time}'
	),
	'ja_JP' => array(
		'date' => array(
			'short' => '%y/%m/%d',
			'medium' => '%Y/%m/%d',
			'long' => '%Y年 %B %e日',
			'full' => '%Y年 %B %e日 %A',
		),
		'time' => array(
			'short' => '%p %I:%M',
			'medium' => '%p %I:%M:%S',
			'long' => '%p %I:%M:%S %Z',
			'full' => '%p %I時 %M分 %S秒 %Z',
		),
		'datetime' => '{date} {time}'
	),
	'ko_KR' => array(
		'date' => array(
			'short' => '%y-%m-%d',
			'medium' => '%Y-%m-%d',
			'long' => '%Y년 %B %e일 %a',
			'full' => '%Y년 %B %e일 %A',
		),
		'time' => array(
			'short' => '%p %H시 %M분',
			'medium' => '%p %H시 %M분',
			'long' => '%p %H시 %M분 %S초',
			'full' => '%p %H시 %M분 %S초  %Z',
		),
		'datetime' => '{date} {time}'
	),
	'pt_BR' => array(
		'date' => array(
			'short' => '%d/%m/%y',
			'medium' => '%d/%m/%Y',
			'long' => '%e de %B de %Y',
			'full' => '%A, %e de %B de %Y',
		),
		'time' => array(
			'short' => '%H:%M',
			'medium' => '%H:%M:%S',
			'long' => '%Ih %Mmin %Ss %Z',
			'full' => '%Hh %Mmin %Ss %Z',
		),
		'datetime' => '{date} {time}'
	),
	'es_ES' => array(
		'date' => array(
			'short' => '%d/%m/%y',
			'medium' => '%d/%m/%Y',
			'long' => '%e de %B de %Y',
			'full' => '%A, %e de %B de %Y',
		),
		'time' => array(
			'short' => '%H:%M',
			'medium' => '%H:%M:%S',
			'long' => '%Ih %Mmin %Ss %Z',
			'full' => '%Hh %Mmin %Ss %Z',
		),
		'datetime' => '{date} {time}'
	),
	'zh_CN' => array(
		'date' => array(
			'short' => '%y/%m/%d',
			'medium' => '%y年 %b月 %e日',
			'long' => '%Y年 %B %e日',
			'full' => '%A, %Y年 %B %d',
		),
		'time' => array(
			'short' => '%H:%M',
			'medium' => '%H:%M:%S',
			'long' => '%H:%M:%S %Z',
			'full' => '%H:%M:%S %Z',
		),
		'datetime' => '{date} {time}'
	),
	'zh_TW' => array(
		'date' => array(
			'short' => '%y/%m/%d',
			'medium' => '%Y/%m/%d',
			'long' => '%Y年 %B %e日',
			'full' => '%Y年 %B %e日 %A',
		),
		'time' => array(
			'short' => '%p %I:%M',
			'medium' => '%p %I:%M:%S',
			'long' => '%p%H時 %M分 %S秒',
			'full' => '%p%H時 %M分 %S秒 %Z',
		),
		'datetime' => '{date} {time}'
	)
);

$NONSTD_TIME_FORMATS = array(
	'de_DE' => array(
		'month/year' => '%B %Y'
	),
	'en_US' => array(
		'month/year' => '%B %Y'
	),
	'fr_FR' => array(
		'month/year' => '%B %Y'
	),
	'it_IT' => array(
		'month/year' => '%B %Y'
	),
	'ja_JP' => array(
		'month/year' => '%Y年 %B'
	),
	'ko_KR' => array(
		'month/year' => '%Y년 %B'
	),
	'pt_BR' => array(
		'month/year' => '%B de %Y'
	),
	'es_ES' => array(
		'month/year' => '%B de %Y'
	),
	'zh_CN' => array(
		'month/year' => '%Y年 %B'
	),
	'zh_TW' => array(
		'month/year' => '%Y年 %B'
	)
);

$LANG_LOCALES = array(
	'de' => 'de_DE',
	'en' => 'en_US',
	'es' => 'es_ES',
	'fr' => 'fr_FR',
	'it' => 'it_IT',
	'ja' => 'ja_JP',
	'ko' => 'ko_KR',
	'pt' => 'pt_BR',
	'zh' => 'zh_TW'
);

$LANGUAGES = array(
	'german' => 'Deutsch',
	'english us' => 'English US',
	'spanish' => 'Español',
	'french' => 'Français',
	'italian' => 'Italiano',
	'japanese' => '日本語',
	'chinese simplified' => '简体中文',
	'korean' => '한국어',
	'portuguese' => 'Português',
	'traditional chinese' => '繁體中文'
);

// Default pselect truncation length
$PSELECT_TRUNCATION		= 25;

$GLOBAL_LOGGING                     = array();
$GLOBAL_LOGGING['SQL']              = array();
$GLOBAL_LOGGING['SQL']['FAIL']      = true;
$GLOBAL_LOGGING['SQL']['OKAY']      = true;
$GLOBAL_LOGGING['INCLUDE']          = array();
$GLOBAL_LOGGING['INCLUDE']['FAIL']  = true;
$GLOBAL_LOGGING['INCLUDE']['OKAY']  = true;
$GLOBAL_LOGGING['REQUIRE']          = array();
$GLOBAL_LOGGING['REQUIRE']['FAIL']  = true;
$GLOBAL_LOGGING['REQUIRE']['OKAY']  = true;

$strings = array(); // added to via language packs

$GLOBAL_IMPORT_SCRIPT = array();
$GLOBAL_IMPORT_SCRIPT['windows'] = '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'bfimport.exe';
$GLOBAL_IMPORT_SCRIPT['unix'] = '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'Platform' . DIRECTORY_SEPARATOR . 'bfimport.pl';
$GLOBAL_IMPORT_SCRIPT['other'] = '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'engine' . DIRECTORY_SEPARATOR . 'bfimport.pl';

// UTF-8 Character Conversion Map
$GLOBAL_CONVMAP = array(0x0, 0x2FFFF, 0, 0xFFFF);

// The tables we expect to find.
$GLOBAL_DB_TABLES = array(
	'bf_access',
	'bf_accesscache',
	'bf_bom',
	'bf_bom_manifests',
	'bf_builds',
	'bf_checkpoints',
	'bf_classes',
	'bf_collector',
	'bf_cron',
	'bf_data',
	'bf_dates',
	'bf_engines',
	'bf_env',
	'bf_filterevents',
	'bf_filters',
	'bf_geo',
	'bf_interfacecategories',
	'bf_interfacedata',
	'bf_interfacenotices',
	'bf_interfacerelations',
	'bf_interfaces',
	'bf_interfacesections',
	'bf_interfacetemplates',
	'bf_ldap',
	'bf_levels',
	'bf_logbuckets',
	'bf_logs',
	'bf_mailq',
	'bf_manifest',
	'bf_messages',
	'bf_modules',
	'bf_notes',
	'bf_notifications',
	'bf_perms',
	'bf_plugins',
	'bf_preferences',
	'bf_projects',
	'bf_ptable',
	'bf_results',
	'bf_scopes',
	'bf_selector',
	'bf_semaphores',
	'bf_serverauth',
	'bf_servers',
	'bf_sessions',
	'bf_stepcmd',
	'bf_steps',
	'bf_store',
	'bf_strings',
	'bf_sysconfig',
	'bf_tags',
	'bf_templates',
	'bf_tzones',
	'bf_uid',
	'bf_uifilters',
	'bf_users'
);

$GDD_LOCAL_TABLES = array(
	'bf_logs',
	'bf_messages',
	'bf_results',
	'bf_stepcmd',
	'bf_bom',
	'bf_bom_manifests',
	'bf_interfacecategories',
	'bf_interfacenotices',
	'bf_interfacesections',
	'bf_interfacedata',
	'bf_interfacerelations',
	'bf_filterevents',
	'bf_notifications',
	'bf_engines',
	'bf_interfacedatags',
);

?>
