<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * functions
 * Leftovers from older versions of the product. These should and will get
 * re-implemented in more OO-friendly ways, but for now they live here.
 *
 * Also added: some functions that don't make sense to keep in objects.
 * Just little code blocks that would get duplicated everywhere...
 *
 * @package Console
 * @subpackage Core
 */

function bf_encrypt($pt) {
	$len = strlen($pt);
	if ($len >= 122) return null;
	$ct = '';
	for ($i=0; $i<8; $i++) $ct .= chr(rand(0x01, 0xFF));
	$ct .= chr(0) . chr(255 - $len);
	for ($i=0; $i<$len; $i++) $ct .= chr(ord($pt[$i]) ^ ord($ct[$i & 7]));
	for ($i=$len; $i<16; $i++) $ct .= chr(rand(0x00, 0xFF));
	$ct = unpack("H*hex", $ct);
	return $ct['hex'];
}

function bf_decrypt($ct) {
	if (is_null($ct)) return null;
	$ct = pack("H*", $ct);
	$ctlen = strlen($ct);
	if ($ctlen < 25) return null;
	$len = 255 - ord($ct[8]);
	if ($len != 255) {
		if ($ctlen < 9 + $len) return null;
		$pt = '';
		for ($i=0; $i<$len; $i++) {
			$value = ord($ct[$i+9]);
			if ($value < 0x20 || $value >= 0x7F) return null;
			$offset = $value + 0x011D - ord($ct[$i & 7]);
			$pt .= chr($offset % 0x5F + 0x20);
		}
		return $pt;
	}
	$len = 255 - ord($ct[9]);
	if ($ctlen < 10 + $len) return null;
	$pt = '';
	for ($i=0; $i<$len; $i++) {
		$pt .= chr(ord($ct[$i + 10]) ^ ord($ct[$i & 7]));
	}
	return $pt;
}

/**
 * One way or another, get a locale string
 */
function bf_get_language() {
	global $user, $LANG_DEFAULT, $LANG_LOCALES, $controller;

	$lang = null;

	// First, try the logged-in user
	if (isset($user)) {
		// Saved preference?
		$templang = $user->get('bf_lang');
		if (file_exists(BF_ROOT . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR . $templang)) {
			return $templang;
		}
	}
	
	// Browser?
	if (empty($lang)) {
		$lang_header = $controller->getRequestHeader('Accept-Language');
		$lang_split = preg_split('/[.;,=.0-9]/', $lang_header, -1, PREG_SPLIT_NO_EMPTY);
		foreach ($lang_split as $option) {
			// Force correct formatting
			$option = str_replace('-', '_', $option);
			// Prefix if the full string fails to match
			$prefix = $option;
			// Force the string to the expected (and used) format;
			if (strlen($option) == 5) {
				$option[3] = strtoupper($option[3]);
				$option[4] = strtoupper($option[4]);
				$prefix = $option[0] . $option[1];
			}
			$langpath = BF_ROOT . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR . $option;
			if (is_dir($langpath)
					&& file_exists($langpath . DIRECTORY_SEPARATOR . 'language.php')) {
				return $option;
			}
			if (isset($LANG_LOCALES[$prefix])) {
				$langpath = BF_ROOT . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR . $LANG_LOCALES[$prefix];
				if (is_dir($langpath)
						&& file_exists($langpath . DIRECTORY_SEPARATOR . 'language.php')) {
					return $LANG_LOCALES[$prefix];
				}
			}
		}
	}
	
	// Fine, just use the default
	return $LANG_DEFAULT;
}

/**
 * For any bf_string.bf_key and language (or get the users' lang if not specified),
 * look up the string in the cached file
 */
function bf_translate_string($key = '', $lang = null) {
	global $controller, $strings;
	if (!$key)  {
		return false;
	}
	
	if (!isset($lang)) {
		$lang = bf_get_language();
	}
	
	if (!isset($strings[$lang])) {
		bf_load_strings_pack($lang);
	}
	
	if (isset($strings[$lang][$key])) {
		return $strings[$lang][$key];
	} else {
		return $key;
	}
}

/**
 * Load in the particular language pack
 */
function bf_load_strings_pack($lang) {
	global $controller;
	
	$file = BF_ROOT . DIRECTORY_SEPARATOR . 'templates_c' . DIRECTORY_SEPARATOR . 'strings.' . $lang . '.lang';
	
	if (file_exists($file) && is_readable($file)) {
		include $file;
	} else {
		try {
			bf_update_strings();
			include $file;
		} catch(Exception $e) {
			$controller->addError($e->getMessage());
			$controller->addError(translate('language x not found', $lang));
		}
	}
}

/**
 * Force a refresh from the bf_strings table
 */
function bf_update_strings($db = null) {
	$qb = BFQueryBuilder::select($db);
	$qb->addColumn('bf_key', 'bf_strings', 'key');
	$qb->addColumn('bf_message', 'bf_strings', 'message');
	$qb->addColumn('bf_lang', 'bf_strings', 'lang');
	$qb->orderBy('ASC', 'bf_strings', 'bf_lang');
	$qb->orderBy('ASC', 'bf_strings', 'bf_key');
	if ($res = $qb->getResult()) {
		$lang = null;
		$strings = array();
		while ($row = $res->fetchRow()) {
			// If next language, write to cache
			if ($lang != $row->lang) {
				if (isset($lang)) {
					$output = BF_ROOT . DIRECTORY_SEPARATOR . 'templates_c' . DIRECTORY_SEPARATOR . 'strings.' . $lang . '.lang';
					file_put_contents(
							$output,
							'<'.'?php global $strings; $strings[\'' . $lang . '\'] = '
								. var_export($strings, 1)
								. '; ?'.'>',
							LOCK_EX
						);
					unset($strings);
					$strings = array();
					// Next language please!
				}
				$lang = $row->lang;
			}
			$strings[$row->key] = $row->message;
		}
		// Need it here as well to write the very last language
		if (isset($lang)) {
			$output = BF_ROOT . DIRECTORY_SEPARATOR . 'templates_c' . DIRECTORY_SEPARATOR . 'strings.' . $lang . '.lang';
			file_put_contents(
					$output,
					'<'.'?php global $strings; $strings[\'' . $lang . '\'] = '
						. var_export($strings, 1)
						. '; ?'.'>',
					LOCK_EX
				);
			unset($strings);
			$strings = array();
		}
	} else {
		throw new BFException();
	}
}

function bf_get_geo_location($geo_id = '') {
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_location', 'bf_geo', 'location');
	$qb->where(array('=', 'bf_geo', 'bf_geo_id', $geo_id));
	return $qb->getOne();
}

function bf_get_geo($geo_id = null) {
	if ($geo_id !== null) {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_location', 'bf_geo', 'location');
		$qb->where(array('=', 'bf_geo', 'bf_geo_id', $geo_id));
		return $qb->getOne();
	} else {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_geo_id', 'bf_geo', 'geo_id');
		$qb->addColumn('bf_location', 'bf_geo', 'location');
		$qb->orderBy('ASC', 'bf_geo', 'bf_location');
		$res = $qb->getResult();
		$geos = array();
		while ($geo = $res->fetchRow()) {
			$geos[$geo->geo_id] = $geo->location;
		}
		return $geos;
	}
}

/**
 * No nested level structure, just give me a list of level names by the level.
 */
function bf_get_levels($userid = null, $returnJustNumbers = false, $controlled = false) {
	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
	$levels = array();

	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_level', 'bf_levels', 'num');
	$qb->addColumn('bf_name', 'bf_levels', 'nom');

	// If specified, restrict to a userid
	if (!is_null($userid) && !is_array($userid) && (int)$userid !== 1) {
		if ($controlled) {
			$qb->where(array(
						array('=', 'bf_levels', 'bf_lvl', 'bf_accesscache', 'bf_level'),
						array('=', 'bf_accesscache', 'bf_userid', $userid)
						));
		}
		else {
			$qb->where(array(
						 array('=', 'bf_levels', 'bf_level', 'bf_accesscache', 'bf_level'),
						 array('=', 'bf_accesscache', 'bf_userid', $userid)
						 ));
		}
	}

	$qb->orderBy('ASC', 'bf_levels', 'bf_name');
	$res = $qb->getResult();

	if (DB::isError($res)) {
		throw new BFDBException($res->getMessage());
	}

	while ($level = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
		$levels[intval($level['num'])] = ($returnJustNumbers)?$level['num']:$level['nom'];
	}
	return $levels;
}

/**
 * Return the name given a level ID.
 */
function bf_get_level_name($level = 0)  {
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_name', 'bf_levels');
	$qb->where(array('=', 'bf_levels', 'bf_level', $level));
	$res = $qb->getOne();
	return DB::isError($res) ? '' : $res;
}

// Why are we reinventing the wheel here?!
function bf_get_dateformats() {
	global $user, $TIME_FORMATS, $LANG_DEFAULT;

	$formats = array();
	$locale = $user->get('bf_lang');
	
	if (!isset($TIME_FORMATS[$locale])) {
		$locale = $LANG_DEFAULT;
	}

	foreach ($TIME_FORMATS[$locale]['date'] as $name => $date) {
		$temp_format = str_replace('{time}', $TIME_FORMATS[$locale]['time'][$name], $TIME_FORMATS[$locale]['datetime']);
		$temp_format = str_replace('{date}', $TIME_FORMATS[$locale]['date'][$name], $temp_format);
		// use the date format function below instead of rehashing it.
		$formats[$name] = bf_format_date(1041453000,$temp_format);
	}

	return $formats;
}

function bf_get_timezones() {
	if (!array_key_exists('bf_get_timezones', $GLOBALS)) {
		$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_zone', 'bf_tzones', 'zone');
		$qb->addColumn('bf_message', 'bf_strings', 'desc');
		$qb->where(array(
			array('=', 'bf_tzones', 'bf_desc', 'bf_strings', 'bf_key'),
			array('=', 'bf_strings', 'bf_lang', $GLOBALS['user']->get('bf_lang'))
		));
		$qb->orderBy('ASC', 'bf_tzones', 'bf_desc');
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}

		$GLOBALS['bf_get_timezones'] = array();
		while ($zone = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$GLOBALS['bf_get_timezones'][$zone['zone']] = $zone['desc'];
		}
	}

	return $GLOBALS['bf_get_timezones'];
}

function bf_get_collectors() {
	if (!array_key_exists('bf_get_collectors', $GLOBALS)) {
		$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_coll_id', 'bf_collector', 'id', 'DISTINCT');
		$qb->addColumn('bf_coll_id', 'bf_collector', 'name');
		$qb->orderBy('ASC', 'bf_collector', 'bf_coll_id');
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}

		$GLOBALS['bf_get_collectors'] = array();
		while ($collector = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$GLOBALS['bf_get_collectors'][$collector['id']] = $collector['name'];
		}
	}

	return $GLOBALS['bf_get_collectors'];
}

function bf_get_collector_props() {
	if (!array_key_exists('bf_get_collector_props', $GLOBALS)) {
		$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_prop', 'bf_collector', 'id', 'DISTINCT');
		$qb->addColumn('bf_prop', 'bf_collector', 'name');
		$qb->orderBy('ASC', 'bf_collector', 'bf_prop');
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}

		$GLOBALS['bf_get_collector_props'] = array();
		while ($collector = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$GLOBALS['bf_get_collector_props'][$collector['id']] = $collector['name'];
		}
	}

	return $GLOBALS['bf_get_collector_props'];
}


/** get_env_tree
 *  BFCron uses this still..
 */
function bf_get_env_tree($bid) {
	if ($bid && !array_key_exists('bf_get_env_tree', $GLOBALS)) {
		$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid',		'bf_env', 'bid');
		$qb->addColumn('bf_param',		'bf_env', 'param');
		$qb->addColumn('bf_value',		'bf_env', 'value');
		$qb->addColumn('bf_mode',		'bf_env', 'mode');
		$qb->addColumn('bf_eid',		'bf_env', 'eid');
		$qb->addColumn('bf_level',		'bf_env', 'level');
		$qb->addColumn('bf_action',		'bf_env', 'action');

		$qb->where(array('=', 'bf_env', 'bf_bid', $bid));
		$qb->orderBy('ASC', 'bf_env', 'bf_eid');
		
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFException($res->getMessage());
		}

		$GLOBALS['bf_get_env_tree'] = array();
		$counter = 0;
		while ($env = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$env['uniqueid'] = $UID = substr(substr(uniqid(mktime(),4),16),0,7) . $counter++;
			$GLOBALS['bf_get_env_tree'][] = $env;
		}
	}
	return $GLOBALS['bf_get_env_tree'];
}

/** get_env_by_name
 *  Returns the BID of $name environment
 */
function bf_get_env_by_name($name) {
	if (!array_key_exists('bf_get_env_by_name', $GLOBALS)) {
		$GLOBALS['bf_get_env_by_name'] = array();
	}
	if (!array_key_exists($name, $GLOBALS['bf_get_env_by_name'])) {
		$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid',	'bf_env',   'bid');
		$qb->where(array(
			array('=', 'bf_env', 'bf_param', $name),
			array('=', 'bf_env', 'bf_eid', '0')
		));
		$GLOBALS['bf_get_env_by_name'][$name] = $qb->getOne();
	}
	return $GLOBALS['bf_get_env_by_name'][$name];
}

/** get_envs
 *  Returns an array of all environment names
 */
function bf_get_envs() {
	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_param',	'bf_env',   'name');
	$qb->where(array(
		array('=',	'bf_env',	'bf_eid',	0)
	));
	return $qb->getCol();
}


/** get_selectors
 *  Returns an array of all selector names
 */
function bf_get_selectors() {
	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_selector_id',	'bf_selector',   'name');
	$qb->where(array(
		array('=',	'bf_selector',	'bf_prop',	'')
	));
	return $qb->getCol();
}


/** get_steps
 *  Returns an array of all steps in project $PID (or a single step name, if $SID is set)
 */
function bf_get_steps($PID = 0, $SID = 0) {
	if (!$PID) {
		return array();
	}
	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_description',	'bf_steps',   'name');
	$qb->addColumn('bf_sid',			'bf_steps',   'sid');
	if ($SID)  {
		$qb->where(array(
			array('=',	'bf_steps',	'bf_pid',	intval($PID)),
			array('=',	'bf_steps',	'bf_sid',	intval($SID))
		));
		$res = $qb->getOne();
		return DB::isError($res) ? '' : $res;
	}  else {
		$qb->where(array('=',	'bf_steps',	'bf_pid',	intval($PID)));
		$res = $qb->getResult();
		$projects = array();
		while ($project = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$projects[$project['sid']] = $project['name'];
		}
		return $projects;
	}
}

/** get_project_names
 *  Returns an array of all step names in project $PID
 */
function bf_get_step_names($PID = '') {
	if ($PID != '') {
		$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description',	'bf_steps',   'name');
		$qb->addColumn('bf_uid',			'bf_steps',   'uid');
		$qb->addColumn('bf_sid',			'bf_steps',   'sid');
		$qb->where(array(
			array('=',	'bf_steps',	'bf_pid',	$PID)
		));
		$qb->orderBy('ASC', 'bf_steps', 'bf_sid');
		$res = $qb->getResult();
		$projects = array();
		while ($project = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$projects[$project['uid']] = $project['name'];
		}
		return $projects;
	}
}


/**
 * Just a flat function to list the available interface templates
 */
function bf_get_interfacetemplates($mode) {
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_name', 'bf_interfacetemplates', 'id', 'DISTINCT');
	$qb->where(array('=', 'bf_interfacetemplates', 'bf_type', 'interface'));
	if ($mode) {
		$qb->where(array('=', 'bf_interfacetemplates', 'bf_mode', $mode));
	}
	$qb->orderBy('ASC', 'id');
	$res = $qb->getResult();

	if (DB::isError($res)) {
		throw new BFDBException($res->getMessage());
	}

	$interfacetemplates = array();
	while ($uifilter = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
		$interfacetemplates[$uifilter['id']] = $uifilter['id'];
	}

	return $interfacetemplates;
}


/**
 * Another flat function to get some specific interface template.
 */
function bf_get_interfacetemplate($id = null) {
	if (!$id)  {
		return false;
	}
	
	$template = new BFInterfaceTemplate();
			
	// Only continue if we find an interface template by this name
	$fields = array($template->primary_key, 'bf_type');
	$values = array($id, 'interface');
	if ($template->select($fields, $values)) {
		return($template->get('bf_text'));
	} else {
		return false;
	}
}


/**
 * Another flat function to get some specific interface template name.
 */
function bf_get_interface_template_name($id = null) {
	if (!$id)  {
		return false;
	}

	$template = new BFInterface();

	// Only continue if we find an interface by this name
	$fields = array('bf_link');
	$values = array($id);
	if ($template->select($fields, $values)) {
		return($template->get('bf_template'));
	} else {
		return false;
	}
}


/**
 * Just a flat function to list the interfaces.
 */
function bf_get_interfaces($mode) {
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_link', 'bf_interfaces', 'id');
	$qb->addColumn('bf_template', 'bf_interfaces', 'template');

	if ($mode)  {
		// Only list interfaces in the given mode
		$qb->where(array('=', 'bf_interfaces', 'bf_mode', $mode));
	}
	$qb->orderBy('ASC', 'id');
	$res = $qb->getResult();
	$return = array();
	if (!DB::isError($res))  {
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$return[$row['id']] = $row['template'];
		}
	}
	return $return;
}


/**
 * Given the ID of an interface or interface template, returns an array of all entry points to the 
 * interface / interface template.
 */
function bf_get_entry_points($id) {
	if (!$id)  {
		return false;
	}
	
	$GLOBALS['controller']->require_module('BFInterface');
	$interface = new BFInterface();
	$interface->select($interface->primary_key, $id);
	$template = $interface->get('bf_interface');
	preg_match_all('/<interface\s+name="([^"]+)"[^>]*>/', $template, $matches);
	$entry_points = array();
	foreach ($matches[1] as $match) {
		$entry_points[] = $match;
	}
	
	if ($entry_points)  {
		return $entry_points;
	}
	
	//  Okay, no results from the interfaces, so we'll try the interface templates.	
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_text', 'bf_interfacetemplates', 'id');
	$qb->where(array(
			array('=', 'bf_interfacetemplates', 'bf_name', $id), 
			array('=', 'bf_interfacetemplates', 'bf_type', 'entrypoint'), 
			));
	$qb->orderBy('ASC', 'id');
	$res = $qb->getCol();
	
	return DB::isError($res) ? array() : $res;
}

/**
 * Alias to the UTF-8-safe function.
 */
function hexentities($str) {
	return escapeUtf8($str);
}


/** List Functionality
 * I need a way to push stuff onto a stack within the templates and then check against that stack
 * This is ugly but affords me that functionality
 * Currently the stack may be any var you choose..
 */
function bf_tpl_list_clear($params,$jte) {
	if (array_key_exists('list', $params)) {
		$jte->templateList[$params['list']] = '';
	}
}
function bf_tpl_list_push($params, $jte) {
	if (array_key_exists('list', $params) && array_key_exists('value', $params)) {
		$jte->templateList[$params['list']][] = $params['value'];
//		print_r($jte->templateList[$params['list']]);
	}
}
function bf_tpl_list_in($params, $jte) {
	if (array_key_exists('list', $params) && array_key_exists('assign', $params)) {
		if (!is_array($jte->templateList[$params['list']])) {
			$jte->templateList[$params['list']] = array();
		}
		if (in_array($params['value'], $jte->templateList[$params['list']])) {
			$jte->assign($params['assign'], TRUE);
		} else {
			$jte->assign($params['assign'], FALSE);
		}
	}
}
function bf_tpl_list_dump($params, $jte) {
	print "HERE";
	print_r($jte->templateList);
	die;
}

/** bf_get_storage
 *  A system for pulling data from the storage
 */
function bf_get_storage($id = null) {
	if (!is_string($id) || $id == '') {
		return false;
	}
	$storageObject = new BFStore();
	$s = $storageObject->getStorage($id);
	return $s['TEXT'];
}

function bf_get_serverauths($userid) {
	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_authid', 'bf_serverauth', 'id', 'DISTINCT');
	$qb->addColumn('bf_authid', 'bf_serverauth', 'name');

	// If specified, restrict to a userid
	if (!is_null($userid) && !is_array($userid) && (int)$userid !== 1) {
		$qb->where(array(
				array('=', 'bf_serverauth', 'bf_level', 'bf_accesscache', 'bf_level'),
				array('=', 'bf_accesscache', 'bf_userid', $userid)
		));
	}

	$qb->orderBy('ASC', 'bf_serverauth', 'bf_authid');
	$res = $qb->getResult();

	if (DB::isError($res)) {
		throw new BFDBException($res->getMessage());
	}

	$serverauths = array();
	while ($serverauth = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
		$serverauths[$serverauth['id']] = $serverauth['name'];
	}

	return $serverauths;
}

function bf_get_users($id = false) {
	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_name',	'bf_users', 'username');
	$qb->addColumn('bf_userid',	'bf_users', 'id');
	$qb->orderBy('ASC', 'bf_users', 'bf_name');
	if ($id)  {
		$qb->where(array('=', 'bf_users', 'bf_userid', $id));
		$res = $qb->getOne();
	}  else {
		$res = $qb->getResult();
	}

	if (DB::isError($res)) {
		throw new BFDBException($res->getMessage());
	}

	if ($id)  {
		return $res;
	}  else {
		$users = array();
		while ($user = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$users[$user['id']] = $user['username'];
		}
	
		return $users;
	}
}

function bf_get_uifilters($action) {
    global $user;
    $GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

    $qb = BFQueryBuilder::select();
    $qb->addColumn('bf_id', 'bf_uifilters', 'id');
    $qb->addColumn('bf_filter', 'bf_uifilters', 'name');
    $qb->where(array(
        array('=', 'bf_uifilters', 'bf_action', $action),
        array('=', 'bf_uifilters', 'bf_id', $user->get('bf_userid'))
    ));
    $qb->orderBy('ASC', 'bf_uifilters', 'bf_filter');

    $res = $qb->getResult();

    if (DB::isError($res)) { throw new BFDBException($res->getMessage()); }

    $uifilters = array();
    while ($uifilter = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $uifilters[] = $uifilter['name'];
    }

    return $uifilters;
}

function bf_save_uifilter($filter, $action) {
    global $user;
		$filter=trim($filter);
    if (empty($filter) || empty($action)) return false;
    if ($GLOBALS['controller']->require_module('BFUIFilter')) {
        $uifilter = new BFUIFilter();
        // Don't care if it selected or not, just save it
        $uifilter->select(array('bf_id', 'bf_action', 'bf_filter'), array($user->get('bf_userid'),$action, $filter));
        $uifilter->set('bf_id', $user->get('bf_userid'));
        $uifilter->set('bf_action', $action);
        $uifilter->set('bf_filter', $filter);
        return $uifilter->save();
    }

    return false;
}

function bf_delete_uifilter($filter, $action) {
    global $user;
    $qb = BFQueryBuilder::delete();
    $qb->where(array(
        array('=', 'bf_uifilters', 'bf_action', $action),
        array('=', 'bf_uifilters', 'bf_filter', $filter),
        array('=', 'bf_uifilters', 'bf_id', $user->get('bf_userid'))
    ));
    $res = $qb->getResult();
}


function bf_format_date($stamp = null, $userFormat = null, $locale = null) {
	global $TIME_FORMATS, $controller, $user;

	$stamp = preg_replace("/\(|\)/","",$stamp);
	$userFormat = preg_replace("/\(|\)/","",$userFormat);
	$locale = preg_replace("/\(|\)/","",$stamp);


	if (empty($stamp)) {
		$stamp = 1041453000;
	} else {
		$stamp = (int)$stamp;
	}
	$user_locale = $user->get('bf_lang');

	// Adjust for the user's timezone - we're assuming that all dates are GMT-specific and then adding / 
	// subtracting to get the local timezone.
	if ($stamp != 1041453000) {
		$stamp += $controller->getTimeOffset($stamp, $user);
	}

	if (!isset($locale) || !isset($TIME_FORMATS[$locale])) {
		$locale = $user_locale;
	}
	$defaultFormat = 'long';

	if (empty($userFormat)) {
		if (isset($user)) {
			$userFormat = $user->get('bf_datefmt');
			$userFormat = (!empty($userFormat)) ? $userFormat : $defaultFormat;
		} else {
			$userFormat = $defaultFormat;
		}
	}

	// Old or new?
	if (isset($TIME_FORMATS[$locale]['date'][$userFormat])) {
		$temp_format = $TIME_FORMATS[$locale]['date'][$userFormat];
		$temp_format = str_replace('{time}', $TIME_FORMATS[$locale]['time'][$userFormat], $TIME_FORMATS[$locale]['datetime']);
		$userFormat = str_replace('{date}', $TIME_FORMATS[$locale]['date'][$userFormat], $temp_format);
	}

	// Switch locales temporarily to get the format for the specifiied locale
	if ($user_locale != $locale) {
		setlocale(LC_TIME, $locale . '.UTF-8');
	}

	// First, let's make sure that characters without preceding % are escaped to prevent gmdate() from
	// turning them EVIL.
	$dateFormat = preg_replace('/(?<!%)([a-zA-Z])/', '\\\\$1', $userFormat);

	// Since PHP5 has issues with UTF-8 on just about every level we have to use GMDate to cobble together a time/date format
	//  this means we must convert the strftime codes, to php date codes.  Yay us.
	$sft2gmd_src = array('%a','%A','%b','%B','%d','%e','%h','%H','%I','%j','%m','%M','%p','%r','%S','%U','%V','%W','%w','%y','%Y','%z','%Z', '%D');
	$sft2gmd_dest= array( 'D', 'l', 'M', 'F', 'd', 'j', 'M', 'H', 'h', 'z', 'm', 'i', 'a', 'A', 's', 'N', 'W', 'W', 'w', 'y', 'Y', 'e', 'T', 'm/d/y');

	// Make a new date format from the old one
	$dateFormat = str_replace($sft2gmd_src, $sft2gmd_dest, $dateFormat);

	// Create the date stamp based on gmstrftime since we are using GM times
	$date = gmdate($dateFormat, $stamp);

	// Translate Month Names
	$isModified = 0;
	foreach (array('January','February','March','April','May','June','July','August','September','October','November','December',
				   'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec') as $k=>$v) {
		$date = str_replace($v, translate($v), $date, $isModified);
		//RATLC01270943
		if($isModified) {
			break;
		}
	}

	// Translate Day Names
	$isModified = 0;
	foreach (array('Monday','Tuesday','Wednesday','Thursday','Friday','Saturday','Sunday',
				   'Mon','Tue','Wed','Thu','Fri','Sat','Sun') as $k=>$v) {
		$date = str_replace($v, translate($v), $date, $isModified);
		//RATLC01270943
		if($isModified) {
			break;
		}
	}
	
	// Translate Everything Else - Put everything else in this array
	foreach (array('am','pm') as $k=>$v) {
		$date = str_replace($v, translate($v), $date);
	}

	// Switch locale back
	if ($user_locale != $locale) {
		setlocale(LC_TIME, $user_locale . '.UTF-8');
	}

	// Set the timezone properly.
	if (isset($user)) {
		$tZone = $user->get('bf_tzone');
		$tZone = (!empty($tZone)) ? $tZone : 'GMT';
	} else {
		$tZone = 'GMT';
	}
	//GMT is the default TZ in the date string when calling the gmtime() function.
	$date = str_replace('GMT', $tZone, $date);

	return $date;
}

function bf_seconds_to_time($seconds = '0') {
	$seconds = intval($seconds);
	$minutes = 0;
	$hours = 0;

	$minutes = floor($seconds/60);
	$hours = floor($minutes/60);

	$seconds = $seconds - (60*$minutes);
	$seconds = ($seconds < 10)?'0'.$seconds:$seconds;

	$minutes = $minutes - (60*$hours);
	$minutes = ($minutes < 10)?'0'.$minutes:$minutes;

	return $hours.":".$minutes.":".$seconds;
}

/**
 * Return associated string or "err: " + param if undefined
 */
function translate() {
	global $language, $LANGUAGES;
	if (!isset($language)) {
		// Hope we always have the English translation.
		// It'll make a lot more sense than seeing "err: 2161"
		include_once BF_ROOT . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR . bf_get_language() . 
				DIRECTORY_SEPARATOR . 'language.php';
	}
	$function_args = func_get_args();
	$key = strtolower($function_args[0]);

	if (isset($LANGUAGES[$key])) {
		return $LANGUAGES[$key];
	} else {
		return (isset($key) && isset($language[$key])) ?
			preg_replace('/\%([0-9]+)/e', '$function_args[$1]', $language[$key]) :
			"err: $key";
	}
}

/**
 * Exports a previously created navigation array as xml to be parsed by javascript.
 */
function bf_output_nav_xml($navarr)  {
	$results = '';
	if ($navarr && is_array($navarr))  {
		foreach ($navarr as $k => $v)  {
			if ($v)  {
				$results .= '<navitem';
				foreach ($v as $key => $value)  {
					if ($key != 'subnav')  {
						$results .= ' ' . $key . '="';
						$results .= escapeUtf8($value) . '"';
					}
				}
				if (array_key_exists('subnav', $v))  {
					$results .= '>' . bf_output_nav_xml($v['subnav']) . '</navitem>';
				}  else {
					$results .= '/>';
				}
			}
		}
	}
	return $results;
}

/**
 * PHP equivalent of the Javascript NavReplacer::replaceWith - exports a previously created navigation
 * array as xhtml lower lefthand navigation to be included in the page source.
 */
function bf_output_nav_xhtml_item($nav)  {
	$results = '';
	$id = substr(uniqid(mktime(),4),16);
	if ($nav && is_array($nav))  {
		$results .= '<div class="nav_' . (array_key_exists('subnav', $nav) ? 'parent' : 'child') . '">';
		$results .= '<a class="nav_link' . (array_key_exists('subnav', $nav) ? '' : ' status1 type2') . '"';
		$results .= ' id="nav_' . $id . '"';

		$image = '';
		if (array_key_exists('image', $nav))  {
			$image = $nav['image'];
		}  elseif (array_key_exists('id', $nav))  {
			$image = '../images/';
			switch ($nav['mod'])  {
				case 'projects':  
					switch ($nav['action'])  {
						case 'adaptors.view':
						case 'adaptorlinks.view':
							$image .= 'icons/adaptor.gif'; break;
						case 'classes.view':
							$image .= 'icons/class.gif'; break;
						case 'templates.view':
							$image .= 'icons/library_link.gif'; break;
						case 'steps.view':
							$image .= 'icons/step.gif'; break;
						default:
							$image .= 'nav_icon_projects.gif'; break;
					}
					break;
				case 'library':  
					$image .= ($nav['action'] == 'steps.view') ? 
							'icons/step.gif' : 'nav_icon_library.gif';
					break;
				case 'projectruns':  $image .= 'nav_icon_project_builds.gif'; break;
				case 'schedules':  $image .= 'icons/schedule.gif'; break;
				case 'environments':  $image .= 'nav_icon_environments.gif'; break;
				case 'servers':  
					switch ($nav['action'])  {
						case 'selectors.view':
							$image .= 'icons/selector.gif'; break;
						case 'collector.view':
							$image .= 'icons/collector.gif'; break;
						case 'serverauth.view':
							$image .= 'icons/user.gif'; break;
						default:
							$image .= 'nav_icon_servers.gif'; break;
					}
					break;
				case 'administration':
					switch ($nav['action'])  {
						case 'groups.view':
							$image .= 'icons/access_group.gif'; break;
						case 'permissions.view':
							$image .= 'icons/permission.gif'; break;
						case 'ldap.view':
						case 'system.view':
							$image .= 'icons/system.gif'; break;
						case 'messages.view':
							$image .= 'icons/audit.gif'; break;
						default:
							$image .= 'icons/user.gif'; break;
					}
					break;
			}
		}  else {
			$image = '../images/icons/open_folder.gif';
		}
		
		if ($image) {
			$results .= ' style="background-image: url(' . $image . ');';
			if ($image == '../images/icons/step_threaded.gif' || $image == '../images/icons/step_joined.gif')  {
				$results .= 'padding-left:73px;';
			}
			$results .= '"';
		}
	
		$href = '';
		if (array_key_exists('href', $nav))  {
			$href = $_SERVER['PHP_SELF'] . '?';
		}  else if (array_key_exists('mod', $nav))  {
			$href = $_SERVER['PHP_SELF'] . '?mod=' . $nav['mod'];

			if (isset($nav['id']) && isset($nav['action']) && isset($nav['mod']) && $nav['action'] == 'environment.view') {
				$nav['action'] = 'variable.view';
			}

			$href .= array_key_exists('action', $nav) ? ('&amp;action=' . $nav['action']) : '';
			$href .= array_key_exists('id', $nav) ? ('&amp;' . $nav['id']) : '';
			$href .= array_key_exists('subid', $nav) ? ('&amp;' . $nav['subid']) : '';
		}
		$results .= ' href="' . $href . '"';
		if (array_key_exists('onclick', $nav))  {
			$results .= ' onclick="' . $nav['onclick'] . '"';
		}  elseif (array_key_exists('load', $nav))  {
			$results .= ' onclick="return !page_details.loadDetails(\'' . $nav['load'] . '\');"';
		}  elseif (array_key_exists('subnav', $nav))  {
			$results .= ' onclick="toggleDiv(\'' . $id . '\');return false;"';
		}
		$results .= '>' . $nav['text'] . '</a>';
		
		if (array_key_exists('subnav', $nav))  {
			$results .= '<div class="nav_children" id="subnav_' . $id . '">';
			foreach ($nav['subnav'] as $child)  {
				$results .= bf_output_nav_xhtml_item($child);
			}
			$results .= '</div>';
		}
		$results .= '</div>';
	}
	return $results;
}

/**
 * Wrapper for bf_output_nav_xhtml_item .
 */
function bf_output_nav_xhtml($navarr)  {
	$results = '';
	if ($navarr && is_array($navarr))  {
		foreach ($navarr as $k => $v)  {
			$results .= bf_output_nav_xhtml_item($v);
		}
	}
	return $results;
}

/**
 * Overly complicated function to allow projects and steps to be searched for some sort of clause.
 * Returns a navigation array which is split up into projects and libraries.
 *		$where - where clause to filter projects with
 *		$stepwhere - where clause to filter steps with.  null if you don't want steps to be included in the
 *				search.  '' if you want it to use the $where clause for both projects and steps.
 *		$orand - 'AND' if both $where and $stepwhere need to be true.  'OR' if either clause can be true.
 *				'NOJOIN' to make it not left join the two tables together (and do a straight select from
 *				both tables).
 */
function bf_get_nav_projects_and_steps($where, $stepwhere = '', $orand = 'OR')  {
	$usesteps = ($stepwhere === null) ? 0 : ($orand == 'NOJOIN' ? 2 : 1);
	if (!$where[1])  {  $where[1] = 'bf_projects';  }
	if ($usesteps == 1)  {
		$stepwhere = $stepwhere ? $stepwhere : $where;
		if (!$stepwhere[1] || $stepwhere[1] == 'bf_projects')  {  $stepwhere[1] = 'bf_steps';  }
	}
	if (is_array($where) && !is_array($where[0]) && $where[0] == 'IN')  {
		$where = array($where);
	}
	if ($stepwhere && is_array($stepwhere) && !is_array($stepwhere[0]) && $stepwhere[0] == 'IN')  {
		$stepwhere = array($stepwhere);
	}

	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');

	$qb = BFQueryBuilder::select();

	$result = array();

	// Special case.  We're selecting all projects or steps with a certain attribute and arranging them
	// particularly.
	$qb->addColumn('bf_pid', 'bf_projects', 'pid');
	$qb->addColumn('bf_description', 'bf_projects', 'pdesc');
	$qb->addColumn('bf_platform', 'bf_projects', 'selector');
	if ($usesteps == 1)  {
		$qb->addColumn('bf_sid', 'bf_steps', 'sid');
		$qb->addColumn('bf_description', 'bf_steps', 'sdesc');
		$qb->addColumn('bf_canbg', 'bf_steps', 'canbg');
		$qb->join(array('LEFT', 'bf_steps', array(
				array('=', 'bf_projects', 'bf_pid', 'bf_steps', 'bf_pid'),
				$stepwhere
				)));
		if ($orand == 'AND')  {
			$qb->where(array($where, $stepwhere));
		}  else {
			$qb->where(array($where, 'OR', $stepwhere));
		}
		$qb->orderBy('ASC', 'bf_projects', 'bf_description');
		$qb->orderBy('ASC', 'bf_steps', 'bf_sid');
	}  elseif ($usesteps == 2)  {
		$qb->addColumn('bf_pid', 'bf_steps', 'spid');
		$qb->addColumn('bf_canbg', 'bf_steps', 'canbg');
		$qb->where($where);
		if ($stepwhere)  {
			$qb->where($stepwhere);
		}
		$qb->orderBy('ASC', 'bf_projects', 'bf_description');
	}  else {
		$qb->where($where);
		$qb->orderBy('ASC', 'bf_projects', 'bf_description');
	}
	$res = $qb->getResult();

	if (!DB::isError($res)) {
		$curnav = array('projects' => array(), 'library' => array());
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC))  {
			$which = $row['selector'] ? 'projects' : 'library';
			if (!array_key_exists($row['pdesc'], $curnav[$which]))  {
				$curnav[$which][$row['pdesc']] = array(
						'mod' => $which,
						'action' => 'projects.view',
						'id' => 'bf_id=' . $row['pid'],
						'text' => $row['pdesc']
				);
			}
			if (array_key_exists('sid', $row) && $row['sid'])  {
				if (!array_key_exists('subnav', $curnav[$which][$row['pdesc']]))  {
					$curnav[$which][$row['pdesc']]['subnav'] = array();
				}
				$curnav[$which][$row['pdesc']]['subnav'][$row['sid']] = array(
						'mod' => $which,
						'action' => 'steps.view',
						'id' => 'bf_id=' . $row['pid'] . '&step=' . $row['sid'],
						'image' => ($row['canbg'] == 'Y' ? '../images/icons/step_threaded.gif' : ($row['canbg'] == 'J' ? 
							'../images/icons/step_joined.gif' : '../images/icons/step.gif')),
						'text' => $row['sdesc']
				);
			}
		}
		if ($curnav['projects'])  {
			$result['projects'] = array(
					'mod' => 'projects',
					'text' => translate('projects'),
					'subnav' => $curnav['projects']
			);
		}
		if ($curnav['library'])  {
			$result['library'] = array(
					'mod' => 'library',
					'text' => translate('libraries'),
					'subnav' => $curnav['library']
			);
		}
	}
	return $result;
}

/**
 * Searches for all permissions the given access group(s) have, and returns a navigation array.
 * 		$array - either the id of an access groups or an array of a number of ids of access groups to search.
 */
function bf_get_nav_perms($access)  {
	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
	if (!$access)  {
		return array();
	}

	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_tag', 'bf_perms', 'id', 'DISTINCT');
	$qb->addColumn('bf_message', 'bf_strings', 'desc');
	$qb->addColumn('bf_groupid', 'bf_accesscache', 'level');
	$qb->join(array('LEFT', 'bf_accesscache', array('=', 'bf_ptable', 'bf_level', 'bf_accesscache', 'bf_groupid')));
	$comp = is_array($access) ? 'IN' : '=';
	$qb->where(array(
			array('=', 'bf_ptable', 'bf_tag', 'bf_perms', 'bf_tag'),
			'AND',
			'(',
			array($comp, 'bf_accesscache', 'bf_level', $access),
			'OR',
			array($comp, 'bf_ptable', 'bf_level', $access),
			')',
			array('=', 'bf_perms', 'bf_description', 'bf_strings', 'bf_key'),
			array('=', 'bf_strings', 'bf_lang', $GLOBALS['user']->get('bf_lang'))
	));
	$qb->orderBy('ASC', 'bf_strings', 'bf_message');

	$result = array();
	
	$res = $qb->getResult();
	if (!DB::isError($res)) {
		$curnav = array();
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC))  {
			$curnav[$row['desc']] = array(
					'mod' => 'administration',
					'action' => 'permissions.view',
					'id' => 'bf_tag=' . $row['id'],
					'text' => $row['desc']
			);
		}
		if ($curnav)  {
			$result = array(
				'mod' => 'administration',
				'action' => 'permissions.view',
				'text' => translate('permissions'),
				'subnav' => $curnav
			);
		}
	}
	return $result;
	
}


/**
 * Basically a convenience function.  Finds all $type objects that match the given $where clause,
 * and returns them as a nav-type array for later passing to bf_output_nav_xml.
 * Yes, it's a large switch statement.  Sue me.
 */
function bf_get_nav_objects($type, $where)  {
	$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
	if (is_array($where) && array_key_exists(0, $where) && $where[0] == 'IN')  {
		$where = array($where);
	}

	$qb = BFQueryBuilder::select();
	$result = array();
	$mod = '';
	$action = '';
	$idname = 'bf_id';
	switch($type)  {
		case 'adaptors':
			$qb->addColumn('bf_link', 'bf_interfaces', 'id');
			$qb->addColumn('bf_link', 'bf_interfaces', 'desc');
			$qb->orderBy('ASC', 'bf_interfaces', 'bf_link');
			$mod = 'projects';
			$action = 'adaptors.view';
			$idname = 'bf_link';
			break;
		case 'adaptor links':
			$qb->addColumn('bf_sid', 'bf_scopes', 'id');
			$qb->addColumn('bf_link', 'bf_scopes', 'desc');
			$qb->orderBy('ASC', 'bf_scopes', 'bf_link');
			$mod = 'projects';
			$action = 'adaptorlinks.view';
			$idname = 'bf_sid';
			break;
		case 'classes':
			$qb->addColumn('bf_name', 'bf_classes', 'id');
			$qb->addColumn('bf_name', 'bf_classes', 'desc');
			$qb->orderBy('ASC', 'bf_classes', 'bf_name');
			$mod = 'projects';
			$action = 'classes.view';
			$idname = 'bf_name';
			break;
		case 'templates':
			$qb->addColumn('bf_name', 'bf_templates', 'id');
			$qb->addColumn('bf_description', 'bf_templates', 'desc');
			$qb->orderBy('ASC', 'bf_templates', 'bf_description');
			$mod = 'projects';
			$action = 'templates.view';
			$idname = 'bf_name';
			break;
			
		case 'log filters':
			$qb->addColumn('bf_did', 'bf_filters', 'id');
			$qb->addColumn('bf_pattern', 'bf_filters', 'desc');
			$qb->orderBy('ASC', 'bf_filters', 'bf_pattern');
			$mod = 'projects';
			$action = 'filters.view';
			$idname = 'bf_did';
			break;

		case 'schedules':
			$qb->addColumn('bf_cid', 'bf_cron', 'id');
			$qb->addColumn('bf_description', 'bf_cron', 'desc');
			$qb->orderBy('ASC', 'bf_cron', 'bf_description');
			$mod = 'schedules';
			$action = 'schedule.view';
			$idname = 'bf_cid';
			break;

		case 'environments':
			$qb->addColumn('bf_bid', 'bf_env', 'id');
			$qb->addColumn('bf_param', 'bf_env', 'desc');
			$qb->orderBy('ASC', 'bf_env', 'bf_param');
			$mod = 'environments';
			$action = 'environment.view';
			$idname = 'bf_bid';
			break;

		case 'servers':
			$qb->addColumn('bf_id', 'bf_servers', 'id');
			$qb->addColumn('bf_name', 'bf_servers', 'desc');
			$qb->orderBy('ASC', 'bf_servers', 'bf_name');
			$mod = 'servers';
			$action = 'servers.view';
			break;
		case 'collectors':
			$qb->addColumn('bf_coll_id', 'bf_collector', 'id');
			$qb->addColumn('bf_coll_id', 'bf_collector', 'desc');
			$qb->orderBy('ASC', 'bf_collector', 'bf_coll_id');
			$mod = 'servers';
			$action = 'collectors.view';
			$idname = 'bf_coll_id';
			break;
		case 'selectors':
			$qb->addColumn('bf_selector_id', 'bf_selector', 'id');
			$qb->addColumn('bf_selector_id', 'bf_selector', 'desc');
			$qb->orderBy('ASC', 'bf_selector', 'bf_selector_id');
			$mod = 'servers';
			$action = 'selectors.view';
			$idname = 'bf_selector_id';
			break;
		case 'server auth':
			$qb->addColumn('bf_authid', 'bf_serverauth', 'id');
			$qb->addColumn('bf_authid', 'bf_serverauth', 'desc');
			$qb->orderBy('ASC', 'bf_serverauth', 'bf_authid');
			$mod = 'servers';
			$action = 'serverauth.view';
			$idname = 'bf_authid';
			break;

		case 'access groups':
			$qb->addColumn('bf_level', 'bf_levels', 'id');
			$qb->addColumn('bf_name', 'bf_levels', 'desc');
			$qb->orderBy('ASC', 'bf_levels', 'bf_name');
			$mod = 'administration';
			$action = 'groups.view';
			$idname = 'bf_level';
			break;
		case 'users':
			$qb->addColumn('bf_userid', 'bf_users', 'id');
			$qb->addColumn('bf_name', 'bf_users', 'desc');
			$qb->orderBy('ASC', 'bf_users', 'bf_name');
			$mod = 'administration';
			$action = 'users.view';
			$idname = 'bf_userid';
			break;
	}
	$qb->where($where);
	
	$res = $qb->getResult();
	if (!DB::isError($res)) {
		$curnav = array();
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC))  {
			$curnav[$row['desc']] = array(
					'mod' => $mod,
					'action' => $action,
					'id' => $idname . '=' . $row['id'],
					'text' => $row['desc']
			);
		}
		if ($curnav)  {
			$result = array(
				'mod' => $mod,
				'action' => $action,
				'text' => translate($type),
				'subnav' => $curnav
			);
		}
	}
	return $result;
}


/**
 * Because I need to know the default action for any given mod in a number of places, I'll just codify them
 * here.
 */
function bf_get_default_action($mod)  {
	switch($mod)  {
		case 'administration':  return('users.view');
		case 'environments':  return('environment.view');
		case 'jobs':  return('view');
		case 'library':  return('library.view');
		case 'projects':  return('projects.view');
		case 'schedules':  return('schedule.view');
		case 'servers':  return('servers.view');
	}
}

/** EscapeUTF8
 *  This is the new and improved UTF escape system.  Instead of escaping the entire thing thus creating a 1 to 6 scale up in all output,
 * we only escape that stuff we need to escape.  In this case all CTRL characters (<0x20).. And we also now just convert those pesky html entities
 * to their html entity instead of escaping them.  So &,",',< and >.  We loose the value of MBString where people have it installed, but;
 * A) we don't ship with it installed.
 * B) users with long lists of things would STILL get the 1 to 6 expansion of their data and any savings we would get from MBString is lost in the
 *    noise of data transfer.
 */
function escapeUtf8($str) {
	$ret = '';
	if (is_null($str) || ($str == '' && $str !== 0)) return $ret;
	if (is_numeric($str) || is_bool($str)) return (string)$str;

	$str = htmlspecialchars($str);
	$len = strlen($str);
	for($i=0;$i<$len;$i++) {
		$ret .= (ord($str[$i]) < 0x20)?"&#".ord($str[$i]).";":$str[$i];
	}
	return $ret;
}


$GLOBALS['LEN_MAP'] = array(
							1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
							1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
							1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
							1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
							1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
							1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
							1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
							1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,
							0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,
							0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,
							0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,
							0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,
							2, 2, 2, 2,  2, 2, 2, 2,  2, 2, 2, 2,  2, 2, 2, 2,
							2, 2, 2, 2,  2, 2, 2, 2,  2, 2, 2, 2,  2, 2, 2, 2,
							3, 3, 3, 3,  3, 3, 3, 3,  3, 3, 3, 3,  3, 3, 3, 3,
							4, 4, 4, 4,  4, 4, 4, 4, -1,-1,-1,-1, -1,-1,-1,-1,
							);
$GLOBALS['MB_INSTALLED'] = 0;
function OLDescapeUtf8($str) {
	global $GLOBAL_CONVMAP, $LEN_MAP, $MB_INSTALLED;
	// Use mb_encode_numericentity if available - much faster...
	if ($MB_INSTALLED === 1) {
		return str_replace('&#0;', '', mb_encode_numericentity($str, $GLOBALS['GLOBAL_CONVMAP'], 'UTF-8'));
	} else if ($MB_INSTALLED === 0) {
		$MB_INSTALLED = function_exists('mb_encode_numericentity') ? 1 : -1;
		return escapeUtf8($str);
	}
	$INVALID = 0x0000FFFD;
	$ret = '';
	$u8 = 0;
	$u8_end = strlen($str);
	$pre = '';
	
	while ($u8 < $u8_end) {
		$code = ord($str[$u8]);
		$len = $LEN_MAP[$code];
		switch ($len) {
			case 1:
				$u8++;
				break;
			case 2:
				if (++$u8 >= $u8_end || ($c1 = ord($str[$u8])) < 0x80 || $c1 >= 0xC0) { $code = $INVALID; break; }
				$code = (($code & 0x1F) << 6) | ($c1 & 0x3F);
				if ($code < 0x80) $code = $INVALID;
					$u8++;
				break;
			case 3:
				if (++$u8 >= $u8_end || ($c1 = ord($str[$u8])) < 0x80 || $c1 >= 0xC0) { $code = $INVALID; break; }
				if (++$u8 >= $u8_end || ($c2 = ord($str[$u8])) < 0x80 || $c2 >= 0xC0) { $code = $INVALID; break; }
				$code = (($code & 0x0F) << 12) | (($c1 & 0x3F) << 6) | ($c2 & 0x3F);
				if ($code < 0x800 || ($code >= 0xD800 && $code < 0xE000)) $code = $INVALID;
					$u8++;
				break;
			case 4:
				if (++$u8 >= $u8_end || ($c1 = ord($str[$u8])) < 0x80 || $c1 >= 0xC0) { $code = $INVALID; break; }
				if (++$u8 >= $u8_end || ($c2 = ord($str[$u8])) < 0x80 || $c2 >= 0xC0) { $code = $INVALID; break; }
				if (++$u8 >= $u8_end || ($c3 = ord($str[$u8])) < 0x80 || $c3 >= 0xC0) { $code = $INVALID; break; }
				$code = (($code & 0x07) << 18) | (($c1 & 0x3F) << 12) | (($c2 & 0x3F) << 6) | ($c3 & 0x3F);
				if ($code < 0x10000 || $code >= 0x110000) $code = $INVALID;
					$u8++;
				break;
			default:
				$code = $INVALID;
				$u8++;
		}
		$ret .= $code ? "&#x" . dechex($code) . ';' : '';
	}
	return $ret;
}

$map_len_map = array(1 => 1, 2 => 1, 3 => 1, 4 => 1);
function truncate($string, $trunc = 80) {
	global $LEN_MAP, $map_len_map;

	// Need at least a string to truncate
	if (count($this->function_args) == 1) {
		$halfway = ceil($trunc / 2);
		if (isset($string[$trunc])) {
			// First byte?
			$code = ord($string[$halfway]);
			while (!isset($map_len_map[$LEN_MAP[$code]])) {
				$halfway--;
				$code = ord($string[$halfway]);
			}
			$newstring = substr($string, 0, $halfway);
			$newstring .= '…';
			// First byte?
			$length = strlen($string);
			$code = ord($string[$length - $halfway]);
			while (!isset($map_len_map[$LEN_MAP[$code]])) {
				$halfway++;
				$code = ord($string[$length - $halfway]);
			}
			$newstring .= substr($string, $length - $halfway);
			return $newstring;
		}
		return $string;
	}
}

/**
 * Multi-byte-safe substring, based on the bytes rather than characters
 */
function bf_substr($string, $start, $end = null) {
	global $LEN_MAP, $map_len_map;
	
	// Trim the bad from the front
	$code = ord($string[$start]);
	while (!isset($map_len_map[$LEN_MAP[$code]])) {
		$code = ord($string[++$start]);
	}
	
	// Trim the bad from the end
	if (isset($end) && isset($string[$end])) {
		$code = ord($string[$end]);
		while (!isset($map_len_map[$LEN_MAP[$code]])) {
			$code = ord($string[--$end]);
		}
	}
	
	$string = substr($start, $end);
}


/**
 * Since it's used in multiple places and it can get annoyingly complicated, this contains the logic
 * for whether or not a given project can be started by the current user.  Currently, it can't be started if:
 *   - The project isn't active.  (Currently not an accessible option, but we'll check for it anyhow.)
 *   - The user doesn't have access to the project.  (Shouldn't come up here, but we'll still check.)
 *   - The user doesn't have access to the project's environment.
 *   - The project has at least one active step where either
 *     a) the step is accessible to the user or 
 *     b) the user has ExecStepPerm permission
 *   - The project's current number of builds is equal to or exceeds the project's maximum concurrent 
 *     number of builds.
 *   - The user's daily total number of builds is equal to or exceeds the user's maximum daily number 
 *     of builds.
 * Things this function does NOT check, as they can be set from the start page:
 *   - If the project has a selector set
 *   - If the must change environment variables have been changed and the required ones have been set
 *   - If the user has access to the selector
 *   - If the user has access to the class
 * Currently, this returns '' if the user can start the project or the language file descriptor for what the
 * error is if the user can't.
 */
function bf_can_start($pid)  {
	global $user;
	
	
	if (!$GLOBALS['controller']->userHasAccessTo('RunBuild'))  {
		return '2039';
	}
	
	$qb = BFQueryBuilder::select();
	
	$qb->addColumn('bf_active', 'bf_projects', 'project_active');
	$qb->addColumn('bf_level', 'bf_projects', 'project_level');
	$qb->addColumn('bf_exclusive', 'bf_projects', 'project_runlimit');
	$qb->addColumn('bf_level', 'bf_env', 'env_level');
	$qb->join(array('LEFT', 'bf_env', array(
			array('=', 'bf_projects', 'bf_eid', 'bf_env', 'bf_bid'),
			array('=', 'bf_env', 'bf_eid', 0)
			)));
	$qb->where(array('=', 'bf_projects', 'bf_pid', $pid));

	$res = $qb->getRow();

	$uid = intval($user->get('bf_userid'));
	$access = bf_get_levels($uid);

	if (DB::isError($res))  {
		return 'database error';
	}  elseif  ($res['project_active'] == 'N')  {
		return '2248';  // Project is inactive
	}  elseif (!isset($access[$res['project_level']])) {
		return '2260';  // User doesn't have access to the project
	}  elseif ($res['env_level'] && !isset($access[$res['env_level']]))  { 
		return '2261';  // User doesn't have access to the projects environment.
	}


	if ($user->get('bf_maxbuilds'))  {
		$qb = BFQueryBuilder::select();
		
		$month = date('n');
		$day = date('j');
		$year = date('Y');
		$qb->addColumn('bf_bid', 'bf_builds', 'bf_bid', 'COUNT');
		$qb->where(array(
				array('=', 'bf_builds', 'bf_userid', $uid),
				array('>=', 'bf_builds', 'bf_fired', mktime(0, 0, 0, $month, $day, $year)),
				array('<=', 'bf_builds', 'bf_fired', mktime(23, 59, 59, $month, $day, $year))
				));
		$numuserbuilds = $qb->getOne();

		if (DB::isError($numuserbuilds))  {
			return 'database error';
		}  elseif ($numuserbuilds >= $user->get('bf_maxbuilds'))  {
			return '2034';  // The user has started more builds today than they can run.
		}
	}


	if ($res['project_runlimit'])  {
		$qb = BFQueryBuilder::select();
		
		$qb->addColumn('bf_bid', 'bf_builds', 'bf_bid', 'COUNT');
		$qb->where(array(
				array('=', 'bf_builds', 'bf_pid', $pid),
				array('=', 'bf_builds', 'bf_process', 'P'),
				array('IN', 'bf_builds', 'bf_state', array('W', 'F', 'R'))
				));
		$numbuilds = $qb->getOne();

		if (DB::isError($numbuilds))  {
			return 'database error';
		}  elseif ($numbuilds >= $res['project_runlimit'])  {
			return '2249';  // The project has been kicked off more than their concurrent limit.
		}
	}


	if ($GLOBALS['controller']->userHasAccessTo('ExecStepPerm'))  {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_sid', 'bf_steps', 'bf_sid', 'COUNT');
		$qb->where(array(
			array('=', 'bf_steps', 'bf_pid', $pid),
			array('=', 'bf_steps', 'bf_active', 'Y')
		));
		$res = $qb->getOne();
		if (DB::isError($res))  {
			return 'database error';
		}  elseif (!((int) $res))  {
			return '2032';  // There are no active steps in the project
		}  else {
			return '';
		}
	}  else {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_level', 'bf_steps', 'bf_level');
		$qb->where(array(
			array('=', 'bf_steps', 'bf_pid', $pid),
			array('=', 'bf_steps', 'bf_active', 'Y')
		));
		$res = $qb->getCol();
		if (DB::isError($res))  {
			return 'database error';
		}  else {
			foreach ($res as $k=>$v)  {
				if (!$k || isset($access[$k]))  {
					return '';
				}
			}
			return '2032';  // There are no active steps in the project that the user has access to.
		}
	}
}


/**
 * Since it's used in multiple places and it can get annoyingly complicated, this contains the logic
 * for whether or not a given project can be quickstarted by the current user.  Currently, it can't be 
 * quickstarted if:
 *   - It can't be started (obvious, but we still need to check for it).
 *   - The user doesn't have access to the project's selector or class.
 *   - It has (or, theoretically, includes) an environment which contains must-change variables or 
 *     required variables that are blank.
 * Currently, this returns '' if the user can quickstart the project or the language file descriptor 
 * for what the error is if the user can't.
 */
function bf_can_quickstart($pid)  {
	$canstart = bf_can_start($pid);
	if ($canstart)  {
		return $canstart;
	}

	global $user;
	
	$qb = BFQueryBuilder::select();
	
	$qb->addColumn('bf_pid', 'bf_projects', 'pid');
	$qb->addColumn('bf_level', 'bf_selector', 'selector_level');
	$qb->addColumn('bf_level', 'bf_classes', 'class_level');
	$qb->join(array('INNER', 'bf_classes', array('=', 'bf_projects', 'bf_class', 'bf_classes', 'bf_name')));
	$qb->join(array('LEFT', 'bf_selector', array(
			array('=', 'bf_projects', 'bf_platform', 'bf_selector', 'bf_selector_id'),
			array('=', 'bf_selector', 'bf_prop', ''),
			)));
	$qb->where(array('=', 'bf_projects', 'bf_pid', $pid));
	$res = $qb->getRow();
	$uid = intval($user->get('bf_userid'));
	$access = bf_get_levels($uid);

	if (DB::isError($res))  {
		return 'database error';
	}  elseif (!isset($res['selector_level']) || $res['selector_level'] == '')  {
		return '2031';  // The project has no selector.
	}  elseif (!isset($access[$res['selector_level']])) {
		return '2262';	// No access to the selector
	}  elseif (!isset($access[$res['class_level']]))  {
		return '2263';  // No access to the class
	}

	$qb = BFQueryBuilder::select();
	
	$qb->addColumn('bf_bid', 'bf_env', 'bid');
	$qb->where(array(
	array('=', 'bf_projects', 'bf_pid', $pid),
	array('=', 'bf_projects', 'bf_eid', 'bf_env', 'bf_bid')));
	
	$state='';
	$res = $qb->getResult();
	$checkDepth = ($GLOBALS['controller']->getSysconfig('quickstart_depth_check') || 
		($GLOBALS['controller']->getSysconfig('quickstart_depth_check') == 0)) ?
			$GLOBALS['controller']->getSysconfig('quickstart_depth_check') : 5;

	while ($bid = $res->fetchRow()) {
		# last paramater on the next line is the recursion limit for checking
		# included environments.  This shouldn't be to large
		$state = isEnvIncluded($bid->bid, $checkDepth);
	}

	return $state;
}

/* Lv1	Lv2	Lv3
 * env = 
 *		.include = 'env2'
 *			.include = 'env3'
 *				var1 = ''   <-----this is a must change.
 *			somevar = 'somevalue'
 *		.include = 'someotherenv'
 * 			var2 = 'value1'
 *			var3 = 'value3'
 *		var1 = 'value1'
 *
 * So we basically search the top most environment for the project first for must change or required
 * values first. (in the example above, the top most env is denoted as Lv1, which has 2 .include variables
 * and regular variable called var1).  if no must change or required are found, then we check all the 
 * .include variables in the top most environment next.  The .included environments are denoted as Lv2 above.
 * There is only one depth level 3 environment here (env3, denoted as Lv3).  These environments are checked
 * for must change vars only after all Level2 (Lv2) environments are checked. A total of $max_iter .include
 * environments are checked, starting in level 2 (Lv2) and going through to Level N until a max of 5 .include
 * environments are searched, or we run out of .include environments to check, whichever comes first.
 */
function isEnvIncluded($bid, $max_iter) {
	if($return = isEnvRequired($bid)) {
		return $return;
	}
	if($max_iter == 0) {
		return '';
	}

	$envIds=array();
	$done = 0;	
	$qbName = BFQueryBuilder::select();
	$qbName->addColumn('bf_value', 'bf_env', 'includename');
	$qbName->where(array(
			array('=', 'bf_env', 'bf_bid', $bid),
			array('=', 'bf_env', 'bf_param', '.include')
			));
	$resName = $qbName->getResult();
	$idx = 0;

	while (!$done) {
			while (($nameRow = $resName->fetchRow())&& !$done) {
				$qbId = BFQueryBuilder::select();
				$qbId->addColumn('bf_bid', 'bf_env', 'includeid');
				$qbId->where(array(
					array('=', 'bf_env', 'bf_eid', '0'),
					array('=', 'bf_env', 'bf_value', ''),
					array('=', 'bf_env', 'bf_param', $nameRow->{'includename'})
					));
				$resId = $qbId->getResult();
				$idRow = $resId->fetchRow();
				$idToCheck = $idRow->{'includeid'};
				array_push($envIds, $idToCheck);
				
				if($return = isEnvRequired($idToCheck)) {
					return $return;
				}
				if(count($envIds) >= $max_iter) {
						$done = 1;
				}
			}
			
			$arrSize = count($envIds);
			if($arrSize < $max_iter && $idx < $arrSize) {
				$bid = $envIds[$idx];
				$qbName = BFQueryBuilder::select();
				$qbName->addColumn('bf_value', 'bf_env', 'includename');
				$qbName->where(array(
						array('=', 'bf_env', 'bf_bid', $bid),
						array('=', 'bf_env', 'bf_param', '.include')
						));
				$resName = $qbName->getResult();
				$idx++;
			} else {
				$done = 1;
			}
	}

	return '';
}

// This function does the heavy lifting to actually determine if there is
// a Required or Must Change variable.
function isEnvRequired($bid) {
	$qb = BFQueryBuilder::select();
	
	$qb->addColumn('bf_mode', 'bf_env', 'env_count', 'COUNT');
	$qb->where(array(
		array('=', 'bf_env', 'bf_bid', $bid),
		'AND',
		'(',
		array('=', 'bf_env', 'bf_mode', 'C'),
		'OR',
		'(',
			array('=', 'bf_env', 'bf_mode', 'R'),
			array('=', 'bf_env', 'bf_value', ''),
		')',
		')'
	));

	$res = $qb->getOne();

	if (DB::isError($res))  {
		return 'database error';
	}  elseif($res > 0)  {
		return '2250';  // The project has 'must change' variables or 'required' variables that are empty.
	}

	return '';
}

/** Convert 2 json
 * yeah it does just what you think it does.. 
 * It uses json_encode if you have 5.2, otherwise it rolls through and uses
 * a home grown system which is exceedingly simple.
 * At this time, it only does strings and arrays.  Feel free to add others
 */
function convert2json ($input) {
	if (function_exists('json_encode')) {
		return json_encode($input);
	} else {
		if (is_array($input)) { return array2json($input); }
		if (is_string($input)) { return string2json($input); }
	}
}

function array2json ($arr) {
	$json = '[';
	if (is_array($arr)) {
		foreach($arr as $k=>$v) {
			$json .= (($s1)?',':'').'"'.$k.'":'.convert2json($v);
			$s1 = 1;
		}
	}
	$json .= ']';
	return $json;
}

function string2json($str) {
	if (is_string($str)) {
		$json = '"' . $str . '"';
	}
	return $json;
}

/**
 * This marks a value in the bf_sysconfig table to dirty, so that the services layer
 * will reload the permissions for it.  This fixes the issue I was seeing, but may
 * introduce other craziness because as the intern, I have no clue what I am doing.
 * when it comes to php and the UI.
 */
function bf_mark_access_cashe_dirty() {
	//Make up a new timestamp
	$time = localtime(time(),true);
	$access_cache_dirty = sprintf("API%02d%02d%02d%02d%02d%d",$time['tm_mon']+1,$time['tm_mday'],
			$time['tm_hour'],$time['tm_min'],$time['tm_sec'],rand()); 

	$qb = BFQueryBuilder::update();
	$qb->setColumn('bf_value', 'bf_sysconfig', $access_cache_dirty);
	$qb->where(array('=', 'bf_sysconfig', 'bf_param', 'access_cache_dirty'));
	$res = $qb->getResult();
	if (DB::isError($res)) {
		throw new BFDBException($res->getMessage());
	}
}

?>
