<?php
/**
 * BFCalendar
 * 
 * @package Console
 * @subpackage Schedules
 */
class BFCalendar {
	protected $today;			// Actual now time
	protected $calendar;		// THE array
	protected $events;		  // events[day] = array({event},{event})
	protected $allevents;		  // Unfiltered listing of events
	protected $filterBy;		// An array of event id's that we want to filter by, or show all
	protected $months;		  // Number of months to show
	protected $perColumn;	   // The number of months to show per column (3.8 only)
	protected $curToday;		// This is the "today" we are working with - used for multiple months
	protected $projects;		// projects[pid] = description;

	/**
	 * Actually return the current calendar in all its glory
	 */
	public function getCalendar() {
		return $this->calendar;
	}

	/**
	 * Actually return the current day
	 */
	public function getToday() {
		return $this->today;
	}

	/**
	 * Actually return the current events of the object
	 */
	public function getEvents() {
		return $this->events;
	}

	/**
	 * Returns all events
	 */
	public function getAllEvents() {
		return $this->allevents;
	}

	/**
	 * Actually return the number of months to view
	 */
	public function getMonthCount() {
		return count($this->calendar);
	}

	/**
	 * Actually return the number of events to view
	 */
	public function getEventsCount() {
		return count($this->events);
	}

	/*
	* Return an array of projects: pid => description
	 */
	public function getProjects() {
		return $this->projects;
	}

	/**
	 * Return an array of projects, regardless of whether or not they
	 * exist in the current calendar: pid => description
	 */
	public function getAllProjects() {
		global $user;
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid', 'bf_cron', 'pid');
		$qb->addColumn('bf_description', 'bf_projects', 'pdescription');
		$qb->join(array('LEFT', 'bf_projects', array('=', 'bf_projects', 'bf_pid', 'bf_cron', 'bf_pid')));
		// Add constraints
		$uid = (int)$user->get('bf_userid');
		if ($uid !== 1) {
			$qb->where(array(array('IN', 'bf_projects', 'bf_level', array_keys(bf_get_levels($uid)))));
		}

		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}

		$projects = array();
		while($cron = $res->fetchRow()) {
			$projects[$cron->pid] = $cron->pdescription;
		}

		// We humans like things this way
		natsort($projects);
		return $projects;
	}


	/**
	 * For those of us too lazy to pass each and every event
	 */
	public function loadAllEvents($filter = '', $sortby = '', $sortorder = '') {
		global $user;

		$rep = new BFReport();
		$rep->addColumn('bf_description', 'bf_cron', 'pname');
		$rep->addColumn('bf_description', 'bf_projects', 'pdescription');
		$rep->addColumn('bf_class', 'bf_cron', 'class');
		$rep->addColumn('bf_name', 'bf_users', 'owner');
		$rep->addColumn('bf_param', 'bf_env', 'environment');
		$rep->addColumn('bf_selector', 'bf_cron', 'selector');
		$rep->addColumn('bf_nextrun', 'bf_cron', 'nextrun');
		$rep->addColumn('bf_cid', 'bf_cron', 'cid');
		$rep->addColumn('bf_pid', 'bf_cron', 'pid');
		$rep->addColumn('bf_eid', 'bf_cron', 'eid');
		$rep->addColumn('bf_minute', 'bf_cron', 'minute');
		$rep->addColumn('bf_hour', 'bf_cron', 'hour');
		$rep->addColumn('bf_dom', 'bf_cron', 'day');
		$rep->addColumn('bf_month', 'bf_cron', 'month');
		$rep->addColumn('bf_dow', 'bf_cron', 'dayofweek');
		$rep->addColumn('bf_active', 'bf_cron', 'active');
		$rep->addColumn('bf_owner', 'bf_cron', 'ownerid');
		$rep->join(array('LEFT', 'bf_projects', array('=', 'bf_projects', 'bf_pid', 'bf_cron', 'bf_pid')));
		$rep->join(array('LEFT', 'bf_users', array('=', 'bf_users', 'bf_userid', 'bf_cron', 'bf_owner')));
		$rep->join(array('LEFT', 'bf_env', array(array('=', 'bf_env', 'bf_bid', 'bf_cron', 'bf_eid'), 
				array('=', 'bf_env', 'bf_eid', 0))));

		if ($filter)  {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter($filter);

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'description', array(array('LIKE', 'bf_cron', 'bf_description')));
			$userDefinedFilter->setKey( 'project', array(array('LIKE', 'bf_projects', 'bf_description')));
			$userDefinedFilter->setKey( 'class', array(array('LIKE', 'bf_cron', 'bf_class')));
			$userDefinedFilter->setKey( 'owner', array(array('LIKE', 'bf_users', 'bf_name')));
			$userDefinedFilter->setKey( 'environment', array(array('LIKE', 'bf_env', 'bf_param')));
			$userDefinedFilter->setKey( 'selector', array(array('LIKE', 'bf_cron', 'bf_selector')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_cron', 'bf_description'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_description'),
											'OR',
											array('LIKE', 'bf_cron', 'bf_class'),
											'OR',
											array('LIKE', 'bf_users', 'bf_name'),
											'OR',
											array('LIKE', 'bf_env', 'bf_param'),
											'OR',
											array('LIKE', 'bf_cron', 'bf_selector'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sortorder = $sortorder == 'desc' ? 'DESC' : 'ASC';
		$possible = array(
						  'description' => array('bf_cron', 'bf_description'),
						  'project' => array('bf_projects', 'bf_description'),
						  'class' => array('bf_cron', 'bf_class'),
						  'owner' => array('bf_users', 'bf_name'),
						  'environment' => array('bf_env', 'bf_param'),
						  'selector' => array('bf_cron', 'bf_selector'),
						  'nextrun' => array('bf_cron', 'bf_nextrun')
						  );
		if (isset($possible[$sortby])) {
			$rep->orderBy($sortorder, $possible[$sortby][0], $possible[$sortby][1]);
		} else {
			$rep->orderBy('ASC', 'bf_cron', 'bf_description');
		}
		

		if (!$rep->compile() || !$rep->valid()) {
			throw new BFDBException($res->getMessage());
		}

		$res = $rep->getReportResults();

		$this->projects = array();
		$res_count = $res->getCount();
		for ($i = 0; $i < $res_count; $i++)  {
			$cron = $res->next();
			if (!isset($for_project) || $for_project === '' || $cron['pid'] == $for_project)  {
				$this->setEvent($cron['cid'], $cron['pid'], $cron['pname'], $cron['minute'], $cron['hour'], 
						$cron['day'], $cron['month'], $cron['dayofweek'], $cron['ownerid'], $cron['active'], 
						$cron['nextrun']);
			}
			if (!array_key_exists($cron['pid'], $this->projects)) {
				$this->projects[$cron['pid']] = $cron['pdescription'];
			}
		}
		// We humans like things this way
		natsort($this->projects);
		$res->rewind();
		$this->allevents = $res;		
	}

	/**
	 * Gives method by which to set events
	 */
	public function setEvent($cid, $pid, $pname, $minute, $hour, $day, $month, $dayofweek, $owner, $active, 
			$nextrun) {
		$this->events[] = array ( 'cid'			=> $cid,
								  'pid'			=> $pid,
								  'pname'		=> $pname,
								  'minute'		=> $minute,
								  'hour'		=> $hour,
								  'day'			=> $day,
								  'month'		=> $month,
								  'dayofweek'	=> $dayofweek,
								  'owner'		=> $owner,
								  'active'		=> $active,
								  'active_current' => $active,
								  'nextrun'		=> $nextrun
								  );
	}

	// Allows you to set the start date
	public function setDate($date) {
		if (!empty($date)) {
			$this->today = strtotime($date);
		}
	}

	// Allows you to set the number of months
	public function setMonths($numMonths) {
		if(!empty($numMonths)) {
			$this->months = intval($numMonths);
		}
	}

	/**
	 * Set which projects we want to see - NULL = show all projects
	 * or an array of exactly which project id's we should show
	 */
	public function setFilter($by) {
		if (!empty($by)) {
			$this->filterBy = (!is_array($by))?explode(',',$by):$by;
		}
	}

	/** Creates a vast array of calendicular goodness
	 * Figures out what the beginning day in our "today" month is and uses that to create the weeks
	 */
	public function generateCalendar() {
		global $NONSTD_TIME_FORMATS, $user;
		
		// strtotime + months == buggy.  Go on, try "date('r', strtotime(date('r') . '+0month'))",
		// just to see what you get.  PHP.  Bah.
		$curYear	= date('Y', strtotime(date('r',$this->today)));
		$curMonth   = date('m', strtotime(date('r',$this->today)));		

		for ($t = 0; $t < $this->months; $t++, $curMonth++) {
			if ($curMonth > 12)  {
				$curYear++;
				$curMonth = 1;
			}
			$this->curToday = mktime(0,0,0, $curMonth, 1, $curYear);
			$this->calendar[bf_format_date($this->curToday + 86400, $NONSTD_TIME_FORMATS[$user->get('bf_lang')]['month/year'])] = 
					$this->makeWeeks($curYear, $curMonth);
		}
	}

	public function makeWeeks($curYear, $curMonth) {
		global $user;
		
		$firstDate  = mktime(0, 0, 5, $curMonth, 1, $curYear);
		$lastDate   = mktime(0, 0, 5, $curMonth, date('t', $firstDate), $curYear);
		$startDOW   = $user->getPreference('schedule_start_dow');
		$startDOW   = $startDOW ? $startDOW : 0;
		$startDay	= date('w', $firstDate) - $startDOW;
		if ($startDay < 0)  {
			$startDay += 7;
		}

		$limit = (ceil((date('t', $firstDate) + $startDay) / 7) * 7) * 86400;
		$startDate  = $firstDate - $startDay * 86400;
		$actualdate = date('Y-m-j', $this->today);

		$weeks = array();
		$weekNumber = -1;
		// Working with seconds here...
		for ($t = 0; $t < $limit; $t+=86400) {
			$today = $startDate + $t;
			$curMonth   = date('m', $today);
			$curDay  = date('j', $today);
			$curDOW  = date('w', $today);

			$curEvents  = $this->getEventsMacro($curMonth, $curDay, $curDOW);

			$calcDOY        = date('z', $today);
			$calcMonth      = date('m', $today);
			$name           = date('D', $today);
			$date           = "$curYear-$curMonth-$curDay";
			
			if (date('z', $today + 86400) != $calcDOY)  {
				if ($curDOW == $startDOW)  {
					$weekNumber++;
				}
				$weeks[$weekNumber][$calcDOY] = array(
							'NAME'  => $name,
							'NUM'   => $curDay,
							'WKND'  => (in_array($name, array('Sun', 'Sat'))) ? true : false,
							'MONTH' => (date('m', $this->curToday) == $calcMonth ) ? true : false,
							'NUMTASKS' => count($curEvents),
							'TASKS' => $curEvents,
							'DATE' => "$curYear-$curMonth-$curDay",
							'CURRENT' => ($date == $actualdate)
						);
			}  else {
				$limit += 86400;
			}
		}
		return $weeks;
	}

	/**
	 * Fixes Ranges, Steps, etc.. to get the delimited values
	 */
	// Converts ranges and steps to a comma delimited sets
	//  Ex:			1-5 = 1,2,3,4,5
	//  Ex:			*/2 = 0,2,4,6,8
	//  Ex:		 1-10/2 = 2,4,6,8,10
	//  Ex:  1-3,6-18/2,23 = 1,2,3,6,8,10,12,14,16,18,23
	//  Ex:	  *,!6-18/2 = Every day but NOT 6,8,10,12,14,16,18
	//  Ex:		 *,!*/4 = Every day but NOT 4,8,12,16,20,24,28
	//  Ex:	 16,<10,>20 = The 16th, all days up to 10 and from 20 on
	//
	public function getDelimitedSets($inSet,$maxRange) {
		$inArray = explode(',',$inSet);
		$outArray = array();
		$notArray = array();

		foreach($inArray as $k=>$v) {
			$NOT = false;
			if (strpos($v,'!')!==false) {
				$NOT = true;
				$v = preg_replace('|^!|','',$v);
			}
			// Convert * to numbers
			if ($v=='*') {
				$v = '0';
				for($t=1;$t<=$maxRange;$v.=','.$t,$t++);
			}
			// Fix Steps
			$v = $this->fixSteps($v,$maxRange);
			// Fix Ranges
			$v = $this->fixRanges($v,$maxRange);

			if ($NOT) {
				$notArray[] = $v;
			} else {
				$outArray[] = $v;
			}
		}
		// Combine the oddly shaped set of arrays into a string
		$outSet = implode(',',$outArray);
		$notSet = implode(',',$notArray);
		// Now reexplode those strings back into a uniform array and diff it out
		$finalOutArray = array_diff(explode(',',$outSet),explode(',',$notSet));
		// Recombine back to string and return - ugly but it works and my brain hurts.
		return implode(',',$finalOutArray);
	}

	public function fixSteps($value,$maxRange) {
		$minRange = 0;
		if (strpos($value,'/')!==false) {
			preg_match('/(.*)\/(\d+)/',$value,$m);
			if (strpos($m[1],'-')!==false) {
				preg_match ('/(\d+)-(\d+)/',$value,$ma);
				$minRange = $ma[1];
				$maxRange = $ma[2];
			}
			$value = '';
			for($t=$minRange;$t<=$maxRange;$t=$t+$m[2]) {
				$value .= (($value!='')?',':'').$t;
			}
		}
		return $value;
	}

	/**
	 * Convert a range of numbers to a comma delimited set of values
	 * Ex: 1-5 = 1,2,3,4,5
	 * Ex:  <5 = 1,2,3,4,5
	 * Ex: >25 = 25,26,27,28,29,30,31 {days}
	 */
	public function fixRanges($value,$maxRange) {
		// if its a #-# range then convert it
		if (strpos($value,'-')!==false) {
			preg_match ('/(\d+)-(\d+)/',$value,$m);
			for($value=$m[1],$t=($m[1]+1);$t<($m[2]+1);$value.=','.$t,$t++);

			// if its a <# range then convert it
		} elseif (strpos($value,'<')!==false) {
			preg_match('/^<(\d+)/',$value,$m);
			for($value='0',$t=1;$t<=$m[1];$value.=','.$t,$t++);

			// if its a ># range then convert it
		} elseif (strpos($value,'>')!==false) {
			preg_match('/^>(\d+)/',$value,$m);
			for($value=$m[1],$t=$m[1];$t<=$maxRange;$value.=','.$t,$t++);
		}
		return $value;
	}


	/**
	 * Returns the day's events by month and day
	 */
	public function getEventsMacro($month, $day, $dow) {
		$curEvents = array();

		foreach ($this->events as $k=>&$v) {

			if ($v['active_current'] == 'N')  {
				continue;
			}

			if ($v['active_current'] == 'O')  {
				if ($v['nextrun'] >= mktime(0, 0, 0, $month, $day) && 
						$v['nextrun'] < mktime(23, 59, 59, $month, $day))  {
					$curEvents[] = $v;
				}
				continue;
			}
			
			$passMonth  = false;
			$passDay	= false;
			$passDOW	= false;

			// Create list of valid - should be 2 each
			$testMonth  = array('*', $month);
			$testDay	= array('*', $day);
			$testDOW	= array('*', $dow);

			// Fix Ranges and Steps
			$eventMonth = $this->getDelimitedSets($v['month'],'12');
			$eventDays  = $this->getDelimitedSets($v['day'],'31');
			$eventDOW   = $this->getDelimitedSets($v['dayofweek'],'7');

			// All Months,Days,DOW in THIS event
			$eventMonth = explode(',', $eventMonth);	// This events Months
			$eventDays  = explode(',', $eventDays);	 // This events Days
			$eventDOW   = explode(',', str_replace('7','0',$eventDOW)); // This events DOW (7 = 0 sunday)

			// Set passMonth to true if we have a valid month
			foreach($eventMonth as $t) {
				if (in_array($t, $testMonth)) {
					$passMonth = true;
				}
			}

			// Set passDay to true if we have a valid day
			foreach($eventDays as $t) {
				if (in_array($t, $testDay)) {
					$passDay = true;
				}
			}

			// Set passDOW to true if we have a valid day of week
			foreach($eventDOW as $t) {
				if (in_array($t, $testDOW)) {
					$passDOW = true;
				}
			}

			// Set all are true set the event - apply filter
			if ($passMonth && (($v['day'] == '*' || $v['dayofweek'] == '*') ? $passDay && $passDOW : 
					$passDay || $passDOW)) {
				if (!empty($this->filterBy)) {
					if (in_array($v['pid'],$this->filterBy)) {
						$curEvents[] = $v;
					}
				} else {
					$curEvents[] = $v;
				}
			}
			

		}
		return (!empty($curEvents))?$curEvents:NULL;
	}

	public function __construct() {
		$this->today = strtotime('now');
		$this->calendar = array();
		$this->events = array();
		$this->filterBy = array();
		$this->months = 1;
		$this->perColumn = 3;
	}
}

?>
