<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFClass
 * A database object providing access to CRUDing the classes stored in the
 * database.
 * @package Console
 * @subpackage Classes
 */


$GLOBALS['controller']->require_module('BFDBO');

class BFClass extends BFDBO {
	public $table 		= 'bf_classes';
	public $primary_key = 'bf_name';
	public $descriptor_key = 'bf_name';
	protected $required_fields = array('bf_name', 'bf_level');
	public $stored_values = array('bf_name' => null, 'bf_level' => null);
	public $field_defaults = array('bf_purge_days' => '0', 'bf_archive_server' => '0', 'bf_delchanges' => 'N');
	public $type_name = 'Class';
	public $field_descriptions = array(
			'bf_name' => 'name',
			'bf_level' => 'access',
			'bf_archive_days' => 'count',
			'bf_archive_server' => 'which',
			'bf_purge_days' => 'days',
			'bf_precious' => 'delete files',
			'bf_pid' => 'start on purge',
			'bf_delchanges' => '',
			'bf_npid' => 'start on entry',
			'bf_xpid' => 'start on exit'
			);

	/** Overriden to check for an existing interface with that name
	 */
	public function save() {
		if ($this->updated['bf_name'] && (!array_key_exists('bf_name', $this->stored_values) || $this->stored_values['bf_name'] != $this->fields['bf_name'])) {

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_name', $this->table, 'bf_name', 'COUNT');
			$qb->where(array(
				array('=', $this->table, 'bf_name', $this->fields['bf_name']),
				array('!=', $this->table, 'bf_name', $this->stored_values['bf_name'])
			));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('class')));
			}
		}

		return parent::save();
	}

	/**
	 * Override for access check
	 */
	public function insert() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('AddClass')) {
			return parent::insert();
		} else {
			throw new BFAccessException('AddClass');
		}
	}

	/**
	 * Override for access check
	 */
	public function update() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('UpdateClass') || 
				((int)$GLOBALS['user']->get('bf_userid') !== 1 && 
				!array_key_exists((int)$this->stored_values['bf_level'], 
				bf_get_levels($GLOBALS['user']->get('bf_userid'))))) {
			throw new BFAccessException('UpdateClass');
		} else {
			if (!$this->hasRequiredFields()) {
				// Catch this and handle it sanely per-object
				throw new BFException(translate('2181'));
			}
			$qb = BFQueryBuilder::update();
	
			// No sense in sending empty queries. Just return true to keep everyone happy
			$updated = false;
			foreach ($this->tablefields as $field) {
				if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
					if (!$updated) {
						$updated = true;
					}
					// If a bfstored value, need to save that first to get the ID
					if (array_key_exists($field, $this->bfstores)) {
						$this->fields[$field] = $this->bfstore->setStorage($this->bfstores[$field]['TEXT'], $this->bfstores[$field]['UID']);
					}
					$qb->setColumn($field, $this->table, $this->fields[$field]);
				}
			}
	
			if ($updated) {
				$qb->where(array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]));
				$res = $qb->getResult();
				if (DB::isError($res)) {
					throw new BFDBException($res->getMessage());
				} else if (array_key_exists($this->primary_key, $this->updated) && $this->updated[$this->primary_key]) {
					if ($this->killZombies())  {
						$this->logChange('Updated');
					}						
				} else {
					$this->logChange('Updated');
					return true;
				}
			} else {
				return true;
			}
		}
	}
	
	/**
	 * Need to update all tables referencing this one, since we don't actually
	 * have a primary key (just a freakin' name) to use.
	 */
	protected function killZombies() {
		$tables = array('bf_projects', 'bf_builds', 'bf_cron');
		foreach ($tables as $table) {
			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_class', $table, $this->fields[$this->primary_key]);
			$qb->where(array('=', $table, 'bf_class', $this->stored_values[$this->primary_key]));
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}
		}
		
		return true;
	}
	
	/**
	 * loadFields() checks your access as well, if necessary
	 */
	public function loadFields() {
		global $user;
		parent::loadFields();
		
		// If not root, you need to have access
		$uid = (int)$user->get('bf_userid');
		if ($uid !== 1) {
			$l = array_keys(bf_get_levels($uid));
			if (is_array($l) && count($l) > 0) {
				$this->querybuilder->where(array(array('IN', $this->table, 'bf_level', $l)));
			} else {
				$this->querybuilder->where(array('=', $this->table, 'bf_level', 0));
			}
											 
		}
	}

	/**
	 * Override for access check
	 */
	public function delete() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('DeleteClass')) {
			return parent::delete();
		} else {
			throw new BFAccessException('DeleteClass');
		}
	}


	/**
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		$tables = array('bf_projects', 'bf_builds', 'bf_cron');
		$count = 0;
		foreach ($tables as $table) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_class', $table, 'bf_class', 'COUNT');
			$qb->where(array('=', $table, 'bf_class', $this->fields[$this->primary_key]));
			$res = $qb->getOne();
			$count += ($res && !DB::isError($res)) ? intval($res) : 0;
		}
		return $count > 0;
	}

}

?>
