<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFCollector
 * A database object providing access to CRUDing the collectors stored in the
 * database.
 * @package Console
 * @subpackage Collectors
 */


$GLOBALS['controller']->require_module('BFDBO');

class BFCollector extends BFDBO {
	public $table		   = 'bf_collector';
	public $primary_key	 = 'bf_coll_id';
	public $descriptor_key = 'bf_coll_id';
	public $required_fields = array('bf_coll_id');
	public $stored_values = array('bf_coll_id' => null, 'bf_seq' => null, 'bf_prop' => null, 'bf_value' => null, 'bf_dirty' => null);
	public $field_descriptions = array(
			'bf_coll_id' => 'name',
			'bf_seq' => 'sequence number',
			'bf_prop' => 'property',
			'bf_value' => 'collector',
			'bf_user' => '',
			'bf_regex' => 'regular expression',
			'bf_dirty' => ''
			);
	
	public function set($field, $action) {
		if ($field == 'bf_seq') {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_seq', $this->table, 'bf_seq', 'MAX');
			$qb->where(array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]));
			$max = intval($qb->getOne(false)) + 1;
			if (empty($action) || $action > $max)  {
				$action = $max;
			}
		}
		
		// Any change needs updating done by the engine
		$this->violate();

		parent::set($field, $action);
	}
	
	// I wanna feel dirty
	public function violate() {
		if ($this->fields['bf_dirty'] != 'Y') {
			parent::set('bf_dirty', 'Y');
		}
	}

	/**
	 * Whee...validation.
	 */
	public function save() {
		if (array_key_exists('bf_seq', $this->fields) && !empty($this->fields['bf_seq'])) {
			// Test the format
			if (!$this->manageSEQs()) {
				throw new BFException(translate('2182'));
			}
		}
		if ($this->fields['bf_seq'] > 0 || $this->fields['bf_coll_id'] == $this->stored_values['bf_coll_id']) {
			// Access check
			if (!$GLOBALS['controller']->userHasAccessTo('EditCollector')) {
				throw new BFAccessException('EditCollector');
			}
			if ($this->fields['bf_prop'] != $this->stored_values['bf_prop']) {
				// Test the format
				if (preg_match('/^BF_/', $this->fields['bf_prop'])) {
					throw new BFException(translate('2117'));
				}
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_prop', 'bf_collector', 'props', 'COUNT');
				$qb->where(array(
								 array('=', 'bf_collector', 'bf_coll_id', $this->fields['bf_coll_id']),
								 array('=', 'bf_collector', 'bf_prop', $this->fields['bf_prop']),
								 array('=', 'bf_collector', 'bf_value', $this->fields['bf_value']),
								 array('!=', 'bf_collector', 'bf_coll_id', $this->stored_values['bf_coll_id']),
								 array('!=', 'bf_collector', 'bf_seq', $this->stored_values['bf_seq'])
								 ));
				$count = $qb->getOne();
				if (DB::isError($count)) {
					throw new BFDBException($count->getMessage());
				} else if (intval($count) > 0) {
					throw new BFException(translate('2183'));
				}
			}
			$this->notifyServers();
			// Update the entire collector with the dirty flag
			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_dirty', $this->table, 'Y');
			$qb->where(array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]));
			$qb->getResult();
		} else {
			// Access check
			if ($this->inserted && !$GLOBALS['controller']->userHasAccessTo('EditCollector')) {
				throw new BFAccessException('EditCollector');
			} else if (!$this->inserted && !$GLOBALS['controller']->userHasAccessTo('AddCollector')) {
				throw new BFAccessException('AddCollector');
			}

			// Test for duplicate collector names
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_prop', 'bf_collector', 'props', 'COUNT');
			$qb->where(array(
							 array('=', 'bf_collector', 'bf_prop', ''),
							 array('=', 'bf_collector', 'bf_coll_id', $this->fields['bf_coll_id']),
							 array('!=', 'bf_collector', 'bf_coll_id', $this->stored_values['bf_coll_id'])
						));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('collector')));
			}
		}
		// bf_seq can't = '' ~ Int actions can't be null
		$this->fields['bf_seq'] = ($this->fields['bf_seq'] == '')?'0':$this->fields['bf_seq'];
		if ($this->inserted)  {
			$type = 'Updated';
		}  else {
			$type = 'Created';
		}
		if (parent::save() && $this->refreshManifests()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	* No matter what, if the seq changes or an filter variable gets
	 * created, we need the seqs to stay in line.
	 */
	protected function manageSEQs() {
		// Only change SEQs if inserting or updating this SEQ
		if ($this->inserted && !$this->updated['bf_seq']) {
			return true;
		}
		
		// No matter what, we'll need the seqs for all filter vars in this filter.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_seq', $this->table, 'bf_seq');
		$qb->where(array(
						 array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						 array('!=', $this->table, 'bf_seq', 0)
						 ));
		$qb->orderBy('ASC', 'bf_seq');
		
		// Please don't break...
		$seqs = $qb->getCol(0, false);

		if (DB::isError($seqs)) {
			throw new BFDBException($seqs->getMessage());
		}
		// Doesn't matter if this edits this object in the mad dash, about to save anyway
		if ($this->inserted || in_array($this->fields['bf_seq'], $seqs)) {
			$new_seq = intval($this->fields['bf_seq']);
			$old_seq = ($this->inserted) ? intval($this->stored_values['bf_seq']) : $seqs[count($seqs) - 1] + 1;
			$dir = ($new_seq < $old_seq) ? -1 : 1;
			// Okay...
			// If the SEQ gets increased, count from the old SEQ + 1 to the new SEQ, subtracting 1 as we go.
			// If the SEQ gets decreased, count from the old SEQ - 1 to the new SEQ, adding 1 as we go.
			for ($i = $old_seq + $dir; ($i >= $new_seq && $dir == -1) || ($i <= $new_seq && $dir == 1) && in_array($i, $seqs); $i += $dir) {
				$update = BFQueryBuilder::update();
				$update->setColumn('bf_seq', $this->table, $i - $dir);
				$update->where(array(
									 array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
									 array('=', $this->table, 'bf_seq', $i)
									 ));
				$upres = $update->getResult();
				if (DB::isError($upres)) {
					return false;
				}
			}
			// Fix if necessary
			if ($dir == 1 && $this->inserted && !in_array($this->fields['bf_seq'], $seqs)) {
				$this->fields['bf_seq']--;
			}
		}
		return true;
	}

	protected function cleanSEQs() {
		// No matter what, we'll need the seqs for all filter vars in this filter.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_seq', $this->table, 'bf_seq');
		$qb->where(array(
						 array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						 array('!=', $this->table, 'bf_seq', 0)
						 ));
		$qb->orderBy('ASC', 'bf_seq');
		
		// Please don't break...
		$seqs = $qb->getCol(0, false);
		if (DB::isError($seqs)) {
			throw new BFDBException($seqs->getMessage());
		}
		// TODO: Use an increment expression rather than a ton of updates
		$this_seq = intval($this->fields['bf_seq']);
		foreach ($seqs as $seq) {
			$seq = intval($seq);
			if ($seq > $this_seq) {
				$update = BFQueryBuilder::update();
				$update->setColumn('bf_seq', $this->table, ($seq - 1));
				$update->where(array(
									 array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
									 array('=', $this->table, 'bf_seq', $seq)
									 ));
				$upres = $update->getResult();
				if (DB::isError($upres)) {
					return false;
				}
			}
		}
		
		return true;
	}

	public function update() {

		$qb = BFQueryBuilder::update();
		$updated = false;
		
		foreach ($this->tablefields as $field) {
			if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
				if (!$updated) {
					$updated = true;
				}
				$qb->setColumn($field, $this->table, $this->fields[$field]);
			}
		}
		
		// No updates? Eh, it worked.
		if (!$updated) {
			return true;
		}
	   
		// Update the collector as a whole
		if ($this->fields['bf_coll_id'] != $this->stored_values['bf_coll_id']) {
			// Dirty, dirty collector!  Unless we've already set it, in which case, never mind.
			if (!$this->updated['bf_dirty'])  {
				$qb->setColumn('bf_dirty', $this->table, 'Y');
			}
			$this->updateServerReferences();
			$qb->where(array(
						 array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key])
					 ));
		// Or update a single variable in the collector
		} else {
			$qb->where(array(
							 array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]),
							 array('=', $this->table, 'bf_seq', $this->stored_values['bf_seq']),
							 array('=', $this->table, 'bf_prop', $this->stored_values['bf_prop'])
					 ));
		}
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			$GLOBALS['controller']->addError('Failed to update the Collector: ' . $res->getMessage());
		} else {
			$this->logChange('Updated');
			return true;
		}
	}

	protected function updateServerReferences() {
		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_coll_id', 'bf_servers', $this->fields['bf_coll_id']);
		$qb->where(array('=', 'bf_servers', 'bf_coll_id', $this->stored_values['bf_coll_id']));
		$qb->getResult();
	}

	public function refreshManifests() {
		$this->controller->require_module('BFServer');
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_id', 'bf_servers', 'id');
		$qb->where(array('=', 'bf_servers', 'bf_coll_id', $this->fields[$this->primary_key]));
		$servers = $qb->getCol();
		
		if (DB::isError($servers)) {
			throw new BFDBException($servers->getMessage());
		} else {
			foreach ($servers as $sid) {
				$server = new BFServer();
				$server->select($server->primary_key, $sid);
				$server->deleteManifest();
				$server->populateManifest();
			}
			return true;
		}
	}


	/**
	 * Overridden in order to copy all the collector variables as well as the collector itself on a 
	 * top-level copy and copying to a given location on a low-level copy.
	 */
	public function copy()  {
		if ($this->fields['bf_prop'])  {
			$this->inserted = false;
			return $this->save();
		}  else {
			//  We need to select all the selector variables from the database and re-insert them with the new ID.
			$oldid = $this->fields['bf_coll_id'];
			parent::copy();
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_prop', 'bf_collector');
			$qb->addColumn('bf_seq', 'bf_collector');
			$qb->where(array(
							array('!=', 'bf_collector', 'bf_seq', 0),
							array('=', 'bf_collector', 'bf_coll_id', $oldid)
						));
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			} else {
				$new_id = $this->get($this->primary_key);
				while ($var = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
					$new_var = new BFCollector();
					$new_var->select(array('bf_coll_id', 'bf_prop', 'bf_seq'), 
							array($oldid, $var['bf_prop'], $var['bf_seq']));
					$new_var->set('bf_coll_id', $new_id);
					$new_var->copy();
				}
				return true;
			}
			return false;
		}
	}



	public function delete() {
		if ($this->stored_values['bf_seq'] == '0') {
			return $this->deleteCollector();
		} else {
			return $this->deleteCollectorVariable();
		}
	}

	/**
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_coll_id', 'bf_servers', 'bf_coll_id', 'COUNT');
		$qb->where(array('=', 'bf_servers', 'bf_coll_id', $this->fields[$this->primary_key]));
		if (intval($qb->getOne()) > 0)  {
			return true;
		}
		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_coll_id', 'bf_collector', 'bf_coll_id', 'COUNT');
		$qb->where(array(
				array('=', 'bf_collector', 'bf_prop', '.include'),
				array('=', 'bf_collector', 'bf_value', $this->fields[$this->primary_key]),
				array('!=', 'bf_collector', 'bf_coll_id', $this->fields[$this->primary_key])
				));
		if (intval($qb->getOne()) > 0)  {
			return true;
		}
		return false;
	}
	
	public function deleteCollector() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteCollector')) {
			throw new BFAccessException('DeleteCollector');
		}
		if ($this->referenced())  {
			throw new BFException(translate('2048'));
		}
		
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key])
					));
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($count->getMessage());
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}

	public function deleteCollectorVariable() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('EditCollector')) {
			throw new BFAccessException('EditCollector');
		}
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						 array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						 array('=', $this->table, 'bf_seq', $this->stored_values['bf_seq'])
					));
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else if (!$this->cleanSEQs()) {
			throw new BFException(translate('2185'));
		} else {
			// Any change needs updating done by the engine
			$parent = new BFCollector();
			$parent->selectByField('bf_coll_id', $this->fields['bf_coll_id'], '=');
			$parent->violate();
			$parent->save();
			$this->notifyServers();

			$this->logChange('Deleted');
			return true;
		}
	}

	protected function notifyServers() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_name', 'bf_servers', 'bf_name');
		$qb->where(array('=', 'bf_servers', 'bf_coll_id', $this->fields[$this->primary_key]));
		$servers = $qb->getCol();
		if (count($servers) > 0) {
			// Cheating.
			$up = BFQueryBuilder::update();
			$up->setColumn('bf_value', 'bf_manifest', 0);
			$up->where(array(
							 array('IN', 'bf_manifest', 'bf_name', $servers),
							 array('=', 'bf_manifest', 'bf_prop', '_AGE')
							 ));
			$up->getResult();
		}
	}
	
	/**
	 * Overridden to deal with the collector / collector property dichotomy.
	 */
	public function logChange($type)  {
		if ($this->log_changes && ($type != 'Updated' || 
				$this->controller->getSysconfig('log_all_changes') != 'Y'))  {
			if ($this->fields['bf_prop'])  {
				$this->controller->logAudit('CollectorProp' . $type, 
						array($this->fields['bf_prop'], $this->fields['bf_coll_id']));
			}  else {
				$this->controller->logAudit('Collector' . $type, array($this->fields['bf_coll_id']));
			}
		}
	}

	/**
	 * A BFDBO-centered logging function used to log before-and-after state on changed fields.  
	 */
	public function logChangedFields($field, $before, $after)  {
		$fieldname = isset($this->field_descriptions[$field]) ? 
				($this->field_descriptions[$field] ? translate($this->field_descriptions[$field]) : '') : 
				$field;
		if ($this->log_changes && $this->inserted && $fieldname && 
				$this->controller->getSysconfig('log_all_changes') == 'Y')  {
			if ($this->fields['bf_prop'])  {
				$this->controller->logAudit('CollectorPropUpdatedField', array($fieldname,
						$this->fields['bf_prop'], $this->fields['bf_coll_id'], $before, $after));
			}  else {
				$this->controller->logAudit('CollectorUpdatedField', array($fieldname, 
						$this->fields['bf_coll_id'], $before, $after));
			}
		}
	}

}

?>
