<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFCron
 * A database object providing access to CRUDing the crons stored in the
 * database.
 * @package Console
 * @subpackage Schedules
 */

$GLOBALS['controller']->require_module('BFDBO');

class BFCron extends BFDBO {
	public $table 		= 'bf_cron';
	public $primary_key = 'bf_cid';
	public $descriptor_key = 'bf_description';
	public $stored_values = array('bf_active' => null, 'bf_description' => null);
	protected $bfstores = array('bf_env' => null);
	protected $env_tree;
	protected $field_defaults = array(
									'bf_description'	=> '',
									'bf_pid'			=> '0',
									'bf_owner'			=> '0',
									'bf_eid'			=> '0',
									'bf_env'			=> '',
									'bf_active'			=> 'Y',
									'bf_minute'			=> '',
									'bf_hour'			=> '',
									'bf_dom'			=> '',
									'bf_month'			=> '',
									'bf_dow'			=> '',
									'bf_fired'			=> '0',
									'bf_nextrun'		=> '0',
									'bf_selector'		=> '',
									'bf_class'			=> ''
									);
	public $type_name = 'Cron';
	public $field_descriptions = array(
			'bf_cid' => '',
			'bf_description' => 'description',
			'bf_pid' => 'project',
			'bf_owner' => 'owner',
			'bf_eid' => 'environment',
			'bf_env' => 'environment',
			'bf_active' => 'mode',
			'bf_minute' => 'minutes',
			'bf_hour' => 'hours',
			'bf_dom' => 'dates',
			'bf_month' => 'months',
			'bf_dow' => 'days',
			'bf_fired' => '',
			'bf_nextrun' => '',
			'bf_selector' => 'selector',
			'bf_class' => 'class'
			);

	/**
	 * Overridden for access check on disable/enable
	 */
	public function set($key, $val) {
		if ($this->inserted && $key == 'bf_active' && $val != $this->stored_values[$key]) {
			// Access check
			if (!$this->controller->userHasAccessTo('ActivateSched')) {
				throw new BFAccessException('ActivateSched');
			}
		}
		return parent::set($key, $val);
	}

	/**
	 * Overwrite to get and set a whole new environment block.
	 */
	public function save() {
		global $user;
		// Test for duplicate selector names
		if ($this->fields['bf_description'] != $this->stored_values['bf_description']) {
			// Access check
			$qb = BFQueryBuilder::select();
			$qb->where(array(
				array('=', 'bf_cron', 'bf_description', $this->fields['bf_description']),
				array('!=', 'bf_cron', 'bf_description', $this->stored_values['bf_description'])
			));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('schedule')));
			}
		}

		// Has permission to set schedule owner?
		$uid = (int)$user->get('bf_userid');
		if ($uid !== 1 && (int)$this->fields['bf_owner'] !== 0 && (int)$this->fields['bf_owner'] !== $uid) {
			throw new BFAccessException('EditSchedule');
		}
		
		// Is it a class purge schedule with no class?
		if (!$this->fields['bf_pid'] && !$this->fields['bf_class'])  {
			throw new BFException(translate('2256', translate('class purge schedule'), translate('class')));
		}

		// Only when selecting a new environment altogether
		if (array_key_exists('bf_eid', $this->updated) && $this->updated['bf_eid'] && 
				$this->fields['bf_pid'] != 0)  {
			if ($this->fields['bf_eid'])  {
				$env_tree = bf_get_env_tree($this->fields['bf_eid']);
				$this->newEnvBlock($env_tree);
			}  else {
				$this->newEnvBlock();
			}
		}
		parent::set('bf_nextrun', '0');
		if ($this->inserted)  {
			$type = 'Updated';
		}  else {
			$type = 'Created';
		}
		return parent::save();
	}
	
	public function insert()  {
		if (!$GLOBALS['controller']->userHasAccessTo('AddSchedule')) {
			throw new BFAccessException('AddSchedule');
		}
		
		return parent::insert();		
	}
	
	public function update()  {
		if (!$GLOBALS['controller']->userHasAccessTo('EditSchedule')) {
			throw new BFAccessException('EditSchedule');
		}
		
		return parent::update();		
	}

	public function delete()  {
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteSchedule')) {
			throw new BFAccessException('DeleteSchedule');
		}
		
		if (parent::delete())  {
			return true;
		}
		return false;
	}

	public function setEnvironment($envs) {
		// Step through the new ENVS and pass them
		foreach($envs as $k=>$v) {
			list($fake, $bid, $sid, $action, $key) = split("_", $k, 5);
			$this->env_tree[] = ((isset($action))?$action:"S") . ":" . $key . ":" . $v;
		}

		$this->newEnvBlock($this->env_tree);
	}

	protected function newEnvBlock($tree = '') {
		// Initialize environment block
		$environment  = '';

		if ($tree)  {
			/**
			 * Append the passed environment tree to the environment block.
			 */
			foreach ($tree as $env)  {
				if (is_array($env))  {
					if ($env['eid'])  {
						$environment .= $env['action'] . ':' . $env['param'] . ':' . $env['value'] . "\n";
					}
				}  else {
					$environment .= $env . "\n";
				}
			}
		}
		$this->set('bf_env', $environment);
	}

	/**
	 * parseTag($ProjectID)
	 * This function does the job of parsing out the tag information
	 */
	private function parseTag($PID) {
		// An array of each tag and final value to be returned
		$returnTags = array();

		// Define Built in tags
		// Get tag_d_format - Set S:BF_D:...
		$e = BFQueryBuilder::select();
		$e->addColumn('bf_value', 'bf_sysconfig', 'tag_d_format');
		$e->where(array('=', 'bf_sysconfig', 'bf_param', 'tag_d_format'));
		$builtInTag['BF_D'] = date($e->getOne());

		// Get tag_t_format - Set S:BF_T:...
		$e = BFQueryBuilder::select();
		$e->addColumn('bf_value', 'bf_sysconfig', 'tag_t_format');
		$e->where(array('=', 'bf_sysconfig', 'bf_param', 'tag_t_format'));
		$builtInTag['BF_T'] = date(str_replace('h', 'H', str_replace('m', 'i', $e->getOne())));

		// Set Julian Date
		$builtInTag['BF_J'] = date('z');

		// Set Week Day
		$builtInTag['BF_W'] = date('w');

		// Get our project information
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_platform',	'bf_projects','resource');
		$qb->addColumn('bf_tag',		'bf_projects','tag');
		$qb->addColumn('bf_level',		'bf_projects','level');
		$qb->addColumn('bf_eid',		'bf_projects','env');
		$qb->addColumn('bf_class',		'bf_projects','bfclass');
		$qb->where(
				   array('=', 'bf_projects', 'bf_pid',	$PID)
				   );
		$res = $qb->getResult();
		$ourProject = $res->fetchRow();

		// Actually start doing stuff with the tag
		$workingTag = $ourProject->tag;
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tid',	'bf_tags','id');
		$qb->addColumn('bf_name',	'bf_tags','name');
		$qb->addColumn('bf_value',	'bf_tags','value');
		$qb->addColumn('bf_parent',	'bf_tags','padding');
		$qb->addColumn('bf_autoinc','bf_tags','autoinc');
		$qb->where(
				   array('=', 'bf_tags', 'bf_pid', $PID)
				   );
		$res = $qb->getResult();

		// Define the tags and values and do any refinement here necessary
		$projectTags = array();
		while($row = $res->fetchRow()) {
			if (is_object($row)) {
				$projectTags[$row->name] = $row;
			}
		}
		// Reorder the tags so they are longest to shortest - so $BOB will parse before $BO
		uksort($projectTags, array($this, 'tagLength'));

		// Parse out the various built in tags - So $BF_D will parse out before $B
		foreach($builtInTag as $k=>$v) {
			$returnTags[$k] = $v;
			$workingTag = preg_replace("/\\\$".$k."/", $v, $workingTag);
		}

		// TODO - TAG SYNC doesnt work yet - so fix that..

		// Parse out the users project tags - Giggity
		foreach($projectTags as $ourTag) {
			// Add Padding
			$paddedValue = $ourTag->value;
			if ($ourTag->padding != '0') {
				for($t = strlen($paddedValue); $t < $ourTag->padding; $t++ , $paddedValue = '0'.$paddedValue);
			}
			// Increment Tag if needed
			if ($ourTag->autoinc == 'Y') {
				$qb = BFQueryBuilder::update(null, true);
				$qb->setColumn('bf_value', 'bf_tags', ++$ourTag->value);
				$qb->where(
						   array (
								  array('=', 'bf_tags',  'bf_pid', $PID),
								  array('=', 'bf_tags',  'bf_tid', $ourTag->id)
								  )
						   );
				if (DB::isError($qb->getResult())) { return false; }
			}
			$returnTags[$ourTag->name] = $paddedValue;
			$workingTag = preg_replace("/\\\${$ourTag->name}/", $paddedValue, $workingTag);
		}

		// Handle any %{ISO} date/time stuff in the tag
		$workingTag = strftime($workingTag, time());

		return array($workingTag, $returnTags);
	}
	// sort function used only by parseTag
	private function tagLength($a, $b) {
		if (strlen($a) == strlen($b)) {
			return 0;
		}
		return (strlen($a) > strlen($b))?-1:1;
	}

	private function parseEnv($PID) {
		// get our projects environment
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_eid', 'bf_projects', 'env');
		$qb->where(array('=', 'bf_projects', 'bf_pid', $PID));
		$EID = $qb->getOne();
		return $this->getEnvironment($EID);
	}

	private function getEnvironment($EID) {
		$env = array();
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_param', 'bf_env', 'key');
		$qb->addColumn('bf_value', 'bf_env', 'value');
		$qb->addColumn('bf_action','bf_env', 'action');
		$qb->where(array(
						 array('=', 'bf_env', 'bf_bid', $EID),
						 array('!=','bf_env', 'bf_eid', '0'),
						 ));
		$res = $qb->getResult();
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			// Add the environment - force to default if |'d input
			$work = preg_split("/\|/",$row['value']);
			$env[] = array(
						   "key"		=> $row['key'],
						   "value"		=> $work[0],
						   "action"	=> $row['action'],
						   );
		}
		return $env;
	}

	/**
		* loadFields() checks your access as well, if necessary
	 */
	public function loadFields() {
		global $user;
		parent::loadFields();

		// If not root, you need to have access
		$uid = (int)$user->get('bf_userid');
		if ($uid !== 1) {
			$this->querybuilder->where(array(
					'(',
					array('=', $this->table, 'bf_owner', $uid),
					'OR',
					array('=', $this->table, 'bf_owner', 0),
					')'
			));
		}
	}
}

?>
