<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 *  This file is the base non-dynamic SQL lookup utilities used to Key-off
 *  SQL in a similiar fashion as internationalized text but used for databases
 *  instead of languages.  
 * 
 *  Queries that go in this file are the default queries and are overriden by
 *  the files in sql_{db_type}.php that get loaded through BFDBO.
 */

abstract class CommonSQL {	
	private $db_type;
	public $statements = array();
	public $sql = array();
	
	/**
	 *  Abstract Constructor.
	 */
	function __construct() {
	 	$this->_init();
	}
	
	/**
	 * abstract method to populate $this->sql array with database type specific Keys/SQL
	 *  to override the default ones in this class.
	 */
	public abstract function initSQL();
	
	/**
	 *   _init - place all the default keys/queries here.  This should be as generic a 
	 *    sql statement as possible to minimize the number of overrides. 
	 */
	public function _init() {
		/** KEEP THIS LIST IN ORDER **/
		$this->statements = array(
			'ADAPTER_LIST_BY_PROJECT'	=> "SELECT * FROM bf_scopes WHERE bf_pid=? and bf_active='Y'",
			'BUILD_COUNT'	=> "SELECT bf_result,count(*) AS bf_count,bf_level FROM bf_builds WHERE bf_pid=? GROUP BY bf_result,bf_level",
			'BUILD_DETAILS'	=> "SELECT b.*,p.bf_description FROM bf_builds b LEFT JOIN bf_projects p on b.bf_pid=p.bf_pid WHERE b.bf_bid=?",
			'BUILD_DURATION'	=> "SELECT bf_duration,bf_level FROM bf_builds WHERE bf_pid=?",
			'BUILD_DURATION_AVERAGE'	=> "SELECT avg(bf_duration) AS bf_average FROM bf_builds WHERE bf_pid=?",
			'BUILD_DURATION_PASS'	=> "SELECT bf_duration,bf_level FROM bf_builds WHERE bf_pid=? AND bf_result='P' ORDER BY bf_bid DESC",
			'BUILD_LIST'	=> "SELECT bf_tag,bf_duration,bf_level FROM bf_builds WHERE bf_pid=? AND bf_result='P' ORDER BY bf_bid DESC",
			'BUILD_REPORT'	=> "SELECT  bf_projects.bf_description,bf_projects.bf_pid,bf_projects.bf_level,bf_builds.bf_result, COUNT(bf_builds.bf_result) AS bf_count FROM bf_builds INNER JOIN bf_projects ON bf_builds.bf_pid=bf_projects.bf_pid AND bf_start>=? AND bf_start<=? GROUP BY bf_projects.bf_description,bf_projects.bf_pid,bf_projects.bf_level,bf_builds.bf_result",
			'BUILD_RESULT_INFO'	=> "SELECT bf_start,bf_duration,bf_result,bf_level FROM bf_builds WHERE bf_pid=? ORDER BY bf_bid DESC",
			'BUILD_SERVER_UTILIZATION'	=> "SELECT bf_duration,bf_server,bf_level FROM bf_results WHERE bf_bid=?",
			'BUILD_STEP_DISTINCT_SERVER'	=> "SELECT DISTINCT bf_server FROM bf_results WHERE bf_spid=? AND bf_uid=? ORDER BY bf_server",
			'BUILD_STEP_INFO_PASS_ONLY'	=> "SELECT bf_duration,bf_server,bf_level FROM bf_results where bf_uid=? AND bf_spid=? AND bf_result='P' ORDER BY bf_sid DESC",
			'BUILD_STEP_INFO_WITH_FAIL'	=> "SELECT bf_duration,bf_server,bf_level FROM bf_results where bf_uid=? AND bf_spid=? ORDER BY bf_sid DESC",
			'BUILD_STEP_RESULTS'		=> "SELECT bf_duration,bf_server,bf_sid,bf_description,bf_level FROM bf_results WHERE bf_bid=? ORDER BY bf_sid DESC",
			'BUILD_STEP_SERVER_UTILIZATION'	=> "SELECT bf_result,bf_duration FROM bf_results WHERE bf_spid=? AND bf_uid=? and bf_server=? ORDER BY bf_start DESC",
			'MANIFESTS'			=> "SELECT m.*,bf_servers.bf_level FROM bf_manifest m JOIN bf_servers on m.bf_name=bf_servers.bf_name ORDER by m.bf_name,m.bf_prop",
			'MD5_BUILD_SEARCH'	=> "SELECT bf_bid,bf_path FROM bf_bom WHERE bf_data=? AND bf_type='F'",
			'PROJECT_BUILDS_PERFORMANCE_PASS_ONLY'	=> "SELECT bf_bid,bf_tag,bf_duration,bf_result,bf_level FROM bf_builds WHERE bf_pid=? AND bf_result='P' ORDER BY bf_start DESC",
			'PROJECT_BUILDS_PERFORMANCE_WITH_FAIL'	=> "SELECT bf_bid,bf_tag,bf_duration,bf_result,bf_level FROM bf_builds WHERE bf_pid=? ORDER BY bf_start DESC",
			'PROJECTS_BY_DESCRIPTION'	=> "SELECT * FROM bf_projects ORDER BY bf_description",
			'PROJECT_LAST_RUN'	=> "SELECT bf_start FROM bf_builds WHERE bf_pid=? ORDER BY bf_bid DESC LIMIT 1",
			'PROJECT_LIST'		=> "SELECT bf_pid,bf_level,bf_description FROM bf_projects ORDER BY bf_description",
			'PROJECT_NAME'		=> "SELECT bf_description FROM bf_projects WHERE bf_pid=?",
			'PROJECT_STEP_INFO'	=> "SELECT bf_uid,bf_description,bf_level FROM bf_steps WHERE bf_pid=?",
			'PROJECT_STEP_LIST'	=> "SELECT b.* FROM bf_steps b WHERE bf_pid=? ORDER BY bf_sid",
			'SERVER_LIST'	=> "SELECT bf_name,bf_level FROM bf_servers",
			'SERVERS_REPORT'	=> "SELECT DISTINCT bf_projects.bf_description AS project, bf_projects.bf_level,bf_projects.bf_pid,bf_builds.bf_resource AS pserver FROM (bf_projects INNER JOIN bf_builds ON bf_builds.bf_pid=bf_projects.bf_pid) ORDER by bf_projects.bf_pid",
			'SERVERS_REPORT_STEPS_SERVER'	=> "SELECT DISTINCT bf_steps.bf_description,bf_steps.bf_sid,bf_steps.bf_uid,bf_results.bf_server,bf_servers.bf_id FROM bf_results JOIN bf_steps ON bf_results.bf_spid=? AND bf_results.bf_uid=bf_steps.bf_uid JOIN bf_servers on bf_servers.bf_name=bf_results.bf_server ORDER BY bf_steps.bf_sid",
			'SERVERS_REPORT_STEPS_VALID'	=> "SELECT bf_sid FROM bf_steps WHERE bf_uid=? and bf_pid=?",
			'USER_ACCESS_LEVELS'	=> "SELECT bf_level from bf_accesscache where bf_userid=?",
			'USER_ZONE'				=> "SELECT bf_zone,bf_gmtoffset,bf_dstart_pos,bf_dstart_day,bf_dstart_mon,bf_dend_pos,bf_dend_day,bf_dend_mon,bf_dst_offset FROM bf_tzones WHERE bf_zone=?",
			'UTILIZATION_REPORT'	=> "SELECT DISTINCT bf_server,SUM(bf_duration) as bf_time,COUNT(*) as bf_stepcount,bf_servers.bf_level,bf_servers.bf_id FROM bf_results INNER JOIN bf_servers ON bf_servers.bf_name=bf_results.bf_server WHERE bf_results.bf_start>=? AND bf_results.bf_start<=? GROUP BY bf_results.bf_server,bf_servers.bf_id,bf_servers.bf_level",
			'UTILIZATION_REPORT_SELECTORS'	=> "SELECT DISTINCT bf_resource,SUM(bf_duration) as bf_time,count(*) as bf_buildcount,bf_selector.bf_level FROM bf_builds INNER JOIN bf_selector on bf_selector.bf_selector_id=bf_builds.bf_resource AND bf_builds.bf_start>=? AND bf_builds.bf_start<=? GROUP BY bf_resource,bf_selector.bf_level",
			'TEST_KEY' 		=> "SELECT * FROM bf_sysconfig WHERE bf_param=?",
			'SYSCONFIG_VALUE' => 'SELECT bf_value from bf_sysconfig where bf_param=?', 
			'FIRE_BUILD_BUILD_ENTRY' => "INSERT INTO bf_builds VALUES(?,?, 0,?,'B','','N',?,?,?,?,'',0,0,?,?,0,?,?,'',?,?,0,?)",
			'CLOBBER_PROJECT_BUILD_ENTRY' => "INSERT INTO bf_builds VALUES(?,?,0,'W','N','N','N','','',?,0,'',0,0,'',?,0,'','','',0,0,0,'')",
			'CBUILD_UPDATE_BID_SYSCONF' => "UPDATE bf_sysconfig SET bf_value=? WHERE bf_param='build_id' AND bf_value=?",
			'CREFLECT_UPDATE_FIRE_BUILD' => "UPDATE bf_builds SET bf_state = 'W', bf_size=1 WHERE bf_bid = ?", 
			'BF_JOBS_COUNT_DEC' => "UPDATE bf_manifest SET bf_value=bf_value-1 WHERE bf_name=? AND bf_prop='BF_JOBS' AND bf_value > 0",
			'BF_JOBS_COUNT_INC' => "UPDATE bf_manifest SET bf_value=bf_value+1 WHERE bf_name=? AND bf_prop='BF_JOBS'",
			'PLUGIN_LIST_STEP_BUILDS' => "SELECT bf_geo_id from bf_builds b WHERE b.bf_bid=?",
			'PLUGIN_LIST_STEP_DETAILS' => "SELECT bf_lid, bf_type, bf_stamp, bf_message FROM bf_logs WHERE bf_uid=? ORDER BY bf_lid",	
			'PLUGIN_LIST_BUILDS' => "SELECT b.*, u.bf_name as bf_username FROM bf_builds b, bf_users u WHERE b.bf_process='B' AND b.bf_pid=? AND b.bf_userid=u.bf_userid",
			'PLUGIN_LIST_PROJECTS' => "SELECT * FROM bf_projects WHERE bf_platform != ''",
			'PLUGIN_LIST_STEPS' => "SELECT * FROM bf_steps WHERE bf_pid=? AND bf_active='Y' ORDER BY bf_sid",
			'PLUGIN_LIST_PROJECT_ENV' => "SELECT e.bf_eid, e.bf_level, e.bf_param, e.bf_value, e.bf_action, e.bf_mode FROM bf_env e, bf_projects p WHERE p.bf_eid=e.bf_bid AND e.bf_mode != 'S' AND e.bf_action != 'H' AND p.bf_pid=?",
			'GEO_LOOKUP_BY_ID' => "SELECT * from bf_geo where bf_geo_id=?",
			'PLUGIN_LIST_STEP_RESULTS' => "SELECT bf_bid, bf_sid, bf_description, bf_result, bf_lid FROM bf_results r WHERE r.bf_bid=? ORDER BY r.bf_sid",
		''
		);
		
		$this->initSQL();
		$this->mergeSQL();
	} 
	
	/** 
	 * getSQL -  used to retrieve the SQL statement.
	 *     $key  -  The Lookup Key for the particular SQL needed.
	 */
	public function getSQL ($key) {
		if (array_key_exists($key, $this->statements)) {
			return $this->statements[$key];
		}
		// assume this is sql
		return $key;
	}
	
	/** 
	 * mergeSQL - used to merge the SQL arrays of this file and the 
	 *  database specific inheriter into one lookup array.
	 */
	private function mergeSQL() {
		$this->statements = array_merge($this->statements, $this->sql);
	}
}


?>
