<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFEnv
 * A database object providing access to CRUDing the environments stored in the
 * database.
 *
 * Since this table overrides the bf_eid field and uses it, paired with bf_bid,
 * we need to override a whole bunch of methods to make sure that the updates to
 * a record only affect ONE record.
 *
 * @package Console
 * @subpackage Environments
 */


$GLOBALS['controller']->require_module('BFDBO');

class BFEnv extends BFDBO {
	public $table	   = 'bf_env';
	// Tricky, since not a true primary key. Thanks for overloading the bf_eid field, guys...
	public $primary_key = 'bf_bid';
	public $descriptor_key = 'bf_param';
	protected $required_fields = array('bf_bid', 'bf_param');
	public $stored_values = array('bf_eid' => null, 'bf_param' => null, 'bf_value' => null);
	protected $field_defaults = array('bf_level' => 0);
	public $envname = '';
	public $field_descriptions = array(
			'bf_bid' => '',
			'bf_eid' => 'sequence number',
			'bf_level' => 'access',
			'bf_param' => 'name',
			'bf_value' => 'value',
			'bf_action' => 'action',
			'bf_mode' => 'on project'
			);

	public function getProjectReferences() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn(BF_FIELD_PREFIX . 'description', BF_TABLE_PREFIX . 'projects', BF_FIELD_PREFIX . 'description');
		$qb->where(array('=', BF_TABLE_PREFIX . 'projects', BF_FIELD_PREFIX . 'eid', $this->fields[BF_FIELD_PREFIX . 'bid']));
		$res = $qb->getCol();
		
		return (DB::isError($res)) ? false : $res;
	}
	
	public function getStepReferences() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn(BF_FIELD_PREFIX . 'description', BF_TABLE_PREFIX . 'steps', BF_FIELD_PREFIX . 'description');
		$qb->where(array('=', BF_TABLE_PREFIX . 'steps', BF_FIELD_PREFIX . 'eid', $this->fields[BF_FIELD_PREFIX . 'bid']));
		$res = $qb->getCol();
		
		return (DB::isError($res)) ? false : $res;
	}
	
	
	/**
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		// Since we live in the wonderful world of defaulting to 0 and also using it to choose none
		if (!$this->inserted) {
			return false;
		}
		$tables = array('bf_servers', 'bf_projects', 'bf_steps', 'bf_cron', 'bf_scopes');
		$count = 0;
		foreach ($tables as $table) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_eid', $table, 'bf_eid', 'COUNT');
			$qb->where(array('=', $table, 'bf_eid', $this->fields[$this->primary_key]));
			$count += intval($qb->getOne());
		}
		
		$envname = $this->getEnvGroupName();

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_value', 'bf_env', 'bf_value', 'COUNT');
		$qb->where(array(
			array('!=', 'bf_env', 'bf_eid', $this->fields[$this->primary_key]),		
			array('=', 'bf_env', 'bf_param', '.include'),
			array('=', 'bf_env', 'bf_value', $envname)
		));
		$count += intval($qb->getOne());
		
		return $count > 0;
	}


	public function set($field, $value) {
		if ($field == 'bf_eid') {
			if (empty($value) && $this->fields['bf_bid']) {
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_eid', 'bf_env', 'bf_eid', 'MAX');
				$qb->where(array('=', 'bf_env', 'bf_bid', $this->fields['bf_bid']));
				$res = $qb->getOne();
				$value = intval($res) + 1;
			}
		}
		
		parent::set($field, $value);
	}

	/**
	 * loadFields() sets up a couple of added fields from other tables,
	 * as well as a couple of automatic joins to go with.
	 */
	public function loadFields() {
		parent::loadFields();
		$this->fields['bf_level']   = null;

		$this->querybuilder->addColumn('bf_name', 'bf_levels', 'bf_lname');
		$this->querybuilder->where(array(
						'('
						,array('=', 'bf_env', 'bf_level', 'bf_levels', 'bf_level')
						,'OR'
						,array('=', 'bf_env', 'bf_level', 0)
						,')'
					));
	}

	/**
	 * Thanks to over-riding fields, I need to implement a catch-all for
	 * eid change and make sure they happen smoothly.
	 */
	public function save() {
		global $_REQUEST;

		if (array_key_exists('bf_eid', $this->fields) && !empty($this->fields['bf_eid'])) {
			// Access check
			if (!$this->inserted && !$GLOBALS['controller']->userHasAccessTo('AddEnvGroupValue')) {
				throw new BFAccessException('AddEnvGroupValue');
			} else if ($this->inserted && $this->fields['bf_eid'] != $this->stored_values['bf_eid'])  {
				if (!$GLOBALS['controller']->userHasAccessTo('MoveEnvValue')) {
					throw new BFAccessException('MoveEnvValue');
				}
			} else if ($this->inserted && !$GLOBALS['controller']->userHasAccessTo('EditEnvValue')) {
				throw new BFAccessException('EditEnvValue');
			}
			// Test the format
			if (preg_match('/^BF_/', $this->fields['bf_param'])) {
				throw new BFException(translate('2116'));
			// Actually try to manage the EIDs for the given BID
			} else if (!$this->manageEIDs()) {
				throw new BFException(translate('2190'));
			}
		} else if (strtolower($this->fields['bf_param']) != strtolower($this->stored_values['bf_param'])) {
			// Access check
			if (!$this->inserted && !$GLOBALS['controller']->userHasAccessTo('AddEnvGroup')) {
				throw new BFAccessException('AddEnvGroup');
			} else if ($this->inserted && !$GLOBALS['controller']->userHasAccessTo('ModifyEnvGroup')) {
				throw new BFAccessException('ModifyEnvGroup');
			}
	
			// Test for duplicate environment names
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_param', 'bf_env', 'params', 'COUNT');
			$qb->where(array(
							array('=', 'bf_env', 'bf_eid', 0),
							array('=', 'bf_env', 'bf_param', $this->fields['bf_param'])
						));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('environment')));
			}		
		}
		// bf_eid can't = '' ~ Int values can't be null
		$this->fields['bf_eid'] = (!$this->fields['bf_eid'])?'0':$this->fields['bf_eid'];
		if (parent::save())  {
			return true;
		}
		return false;
			
	}

	/**
	 * No matter what, if the eid changes or an environment variable gets
	 * created, we need the eids to stay in line.
	 */
	protected function manageEIDs() {
		// Only change EIDs if inserting or updating this EID
		if ($this->inserted && !$this->updated['bf_eid']) {
			return true;
		}

		// No matter what, we'll need the eids for all environment vars in this environment.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_eid', 'bf_env', 'bf_eid');
		$qb->where(array(
						array('=', 'bf_env', 'bf_bid', $this->fields['bf_bid']),
						array('!=', 'bf_env', 'bf_eid', 0)
					));
		$qb->orderBy('ASC', 'bf_eid');

		// Please don't break...
		$eids = $qb->getCol(0, false);
		if (DB::isError($eids)) {
			throw new BFDBException($eids->getMessage());
		}

		// Never let bf_eid be greater than 1+ count(eids). (new below/move to bottom)
		// if eids == 0 (empty envgroup) set eid = 1;
		// Did this stuff ever work? - KWP
		$MaxEid = count($eids);
		$MaxEid = $MaxEid ? $MaxEid : 1 ;
		if ($this->fields['bf_eid'] > $MaxEid + 1) {
			$this->fields['bf_eid'] = $MaxEid;
		}

		// If moving this variable, move it aside and THEN into place to avoid stepping on constraints
		if ($this->inserted) {
			$update = BFQueryBuilder::update();
			$update->setColumn('bf_eid', 'bf_env', count($eids) + 1);
			$update->where(array(
						array('=', 'bf_env', 'bf_bid', $this->fields['bf_bid']),
						array('=', 'bf_env', 'bf_eid', $this->stored_values['bf_eid'])
					));
			$upres = $update->getResult();
			if (DB::isError($upres)) {
				throw new BFDBException($upres->getMessage());
			}
		}

		// Doesn't matter if this edits this object in the mad dash, about to save anyway
		if ($this->inserted || in_array($this->fields['bf_eid'], $eids)) {
			$new_eid = intval($this->fields['bf_eid']);
			$old_eid = ($this->inserted) ? intval($this->stored_values['bf_eid']) : $eids[count($eids) - 1] + 1;
			$dir = ($new_eid < $old_eid) ? -1 : 1;
			// Okay...
			// If the EID gets increased, count from the old EID + 1 to the new EID, subtracting 1 as we go.
			// If the EID gets decreased, count from the old EID - 1 to the new EID, adding 1 as we go.
			for ($i = $old_eid + $dir; ($i >= $new_eid && $dir == -1) || ($i <= $new_eid && $dir == 1) && in_array($i, $eids); $i += $dir) {
				$update = BFQueryBuilder::update();
				$update->setColumn('bf_eid', 'bf_env', $i - $dir);
				$update->where(array(
							array('=', 'bf_env', 'bf_bid', $this->fields['bf_bid']),
							array('=', 'bf_env', 'bf_eid', $i)
						));
				$upres = $update->getResult();
				if (DB::isError($upres)) {
					return false;
				}
			}
			// Fix if necessary
			if ($dir == 1 && $this->inserted && !in_array($this->fields['bf_eid'], $eids)) {
				$this->fields['bf_eid']--;
			}
		}
		$this->stored_values['bf_eid'] = count($eids) + 1;
		return true;
	}

	protected function cleanEIDs() {
		// No matter what, we'll need the eids for all environment vars in this environment.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_eid', 'bf_env', 'bf_eid');
		$qb->where(array(
						 array('=', 'bf_env', 'bf_bid', $this->fields['bf_bid']),
						 array('!=', 'bf_env', 'bf_eid', 0)
						 ));
		$qb->orderBy('ASC', 'bf_eid');

		// Please don't break...
		$eids = $qb->getCol();
		if (DB::isError($eids)) {
			throw new BFDBException($eids->getMessage());
		}

		// TODO: Use an increment expression rather than a ton of updates
		$this_eid = intval($this->fields['bf_eid']);
		foreach ($eids as $eid) {
			$eid = intval($eid);
			if ($eid > $this_eid) {
				$update = BFQueryBuilder::update();
				$update->setColumn('bf_eid', 'bf_env', ($eid - 1));
				$update->where(array(
									 array('=', 'bf_env', 'bf_bid', $this->fields['bf_bid']),
									 array('=', 'bf_env', 'bf_eid', $eid)
									 ));
				$upres = $update->getResult();
				if (DB::isError($upres)) {
					return false;
				}
			}
		}

		return true;
	}

	public function update() {
		$qb = BFQueryBuilder::update();

		foreach ($this->tablefields as $field) {
			if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
				$qb->setColumn($field, $this->table, $this->fields[$field]);
			}
		}

		$qb->where(array(
						 array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						 array('=', $this->table, 'bf_eid', $this->stored_values['bf_eid']),
						 array('=', $this->table, 'bf_param', $this->stored_values['bf_param']),
						 array('=', $this->table, 'bf_value', $this->stored_values['bf_value'])
					));
		$res = $qb->getResult();
		if (!DB::isError($res))  {
			$this->logChange('Updated');
			return true;
		}
		return false;
	}

	public function delete() {
		if ($this->stored_values['bf_eid'] == '0') {
			return $this->deleteEnvironment();
		} else {
			return $this->deleteEnvironmentVariable();
		}
	}
	
	/**
	 * Overridden in order to copy all the environment variables as well as the environment itself on a top-level copy.
	 */
	public function copy() {
		if ($this->fields['bf_eid'] != '0')  {
			$this->inserted = false;
			return $this->save();
		}  else {
			//  We need to select all the selector variables from the database and re-insert them with the new ID.
			$oldid = $this->fields['bf_bid'];
			parent::copy();
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_eid', 'bf_env');
			$qb->addColumn('bf_param', 'bf_env');
			$qb->addColumn('bf_value', 'bf_env');
			$qb->addColumn('bf_action', 'bf_env');
			$qb->addColumn('bf_mode', 'bf_env');
			$qb->where(array(
							array('!=', 'bf_env', 'bf_eid', '0'),
							array('=', 'bf_env', 'bf_bid', $oldid)
						));
			$qb->orderBy('ASC', 'bf_eid');
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			} else {
				$new_id = $this->get($this->primary_key);
				while ($var = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
					$new_var = new BFEnv();
					$new_var->set('bf_bid', $new_id);
					$new_var->set('bf_eid', $var['bf_eid']);
					$new_var->set('bf_level', 0);
					$new_var->set('bf_param', $var['bf_param']);
					$new_var->set('bf_value', $var['bf_value']);
					$new_var->set('bf_action', $var['bf_action']);
					$new_var->set('bf_mode', $var['bf_mode']);
					$new_var->save();
				}
				return true;
			}
			return false;
		}
	}

	protected function deleteEnvironment() {
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteEnvGroup')) {
			throw new BFAccessException('DeleteEnvGroup');
		}
		// Only delete if not referenced anywhere
		if ($this->referenced()) {
			throw new BFException(translate('2201', translate('environment')));
		}
		
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key])
					));
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}

	protected function deleteEnvironmentVariable() {
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteEnvGroupValue')) {
			throw new BFAccessException('DeleteEnvGroupValue');
		}
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						array('=', $this->table, 'bf_eid', $this->stored_values['bf_eid']),
						array('=', $this->table, 'bf_param', $this->stored_values['bf_param']),
						array('=', $this->table, 'bf_value', $this->stored_values['bf_value'])
					));
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else if (!$this->cleanEIDs()) {
			throw new BFException(translate('2190'));
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}

	/**
	 * Override to store identifying fields for re-organizing. Stupid field overloading...
	 */
	public function fetch() {
		parent::fetch();

		foreach ($this->stored_values as $key => $val) {
			$this->stored_values[$key] = $this->fields[$key];
		}
	}


	/**
	 * Because I'm sick and tired of not having it, here's a function to get the name of this environment
	 * group, whether this is a variable or a group.  Never say I don't do anything nice for you.
	 */
	 public function getEnvGroupName()  {
	 	if (!$this->envname)  {
		 	if (!$this->get('bf_eid'))  {
		 		$this->envname = $this->get('bf_param');
		 	}  else {
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_param', 'bf_env', 'name');
				$qb->where(array(
								array('=', 'bf_env', 'bf_eid', 0),
								array('=', 'bf_env', 'bf_bid', $this->fields['bf_bid'])
							));
				$this->envname = $qb->getOne();
				if (DB::isError($this->envname))  {
					$this->envname = '';
				}
			}
	 	}
	 	return $this->envname;
	 }

	/**
	 * Overridden to deal with the environment / environment variable dichotomy.
	 */
	public function logChange($type)  {
		if ($this->log_changes && ($type != 'Updated' || 
				$this->controller->getSysconfig('log_all_changes') != 'Y'))  {
			if ($this->get('bf_eid'))  {
				$this->controller->logAudit('EnvVar' . $type, 
						array($this->fields['bf_param'], $this->getEnvGroupName()));
			}  else {
				$this->controller->logAudit('Env' . $type, array($this->get('bf_param')));
			}
		}
	}

	/**
	 * A BFDBO-centered logging function used to log before-and-after state on changed fields.  
	 */
	public function logChangedFields($field, $before, $after)  {
		$fieldname = isset($this->field_descriptions[$field]) ? 
				($this->field_descriptions[$field] ? translate($this->field_descriptions[$field]) : '') : 
				$field;
		if ($this->log_changes && $this->inserted && $fieldname &&
				$this->controller->getSysconfig('log_all_changes') == 'Y')  {
			if ($this->get('bf_eid'))  {
				$this->controller->logAudit('EnvVarUpdatedField', array($fieldname,
						$this->fields['bf_param'], $this->getEnvGroupName(), $before, $after));
			}  else {
				$this->controller->logAudit('EnvUpdatedField', array($fieldname,
						$this->get('bf_param'), $before, $after));
			}
		}
	}
}

?>
