<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFFilter
 * A database object providing access to CRUDing the filters stored in the
 * database.
 *
 * Since this table overrides the bf_fid field and uses it, paired with bf_did,
 * we need to override a whole bunch of methods to make sure that the updates to
 * a record only affect ONE record.
 *
 * @package Console
 * @subpackage LogFilters
 */


$GLOBALS['controller']->require_module('BFDBO');

class BFFilter extends BFDBO {
	public $table	   = 'bf_filters';
	// Tricky, since not a true primary key. Thanks for overloading the bf_fid field, guys...
	public $primary_key = 'bf_did';
	public $descriptor_key = 'bf_pattern';
	protected $required_fields = array('bf_did', 'bf_pattern');
	public $stored_values = array('bf_fid' => null, 'bf_pattern' => null, 'bf_action' => null);
	protected $field_defaults = array('bf_level' => 0);
	public $filtername;
	public $field_descriptions = array(
			'bf_did' => '',
			'bf_fid' => 'sequence number',
			'bf_pattern' => 'name',
			'bf_action' => 'action',
			'bf_level' => 'access'
			);

	public function set($field, $action) {
		if ($field == 'bf_fid') {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_fid', 'bf_filters', 'bf_fid', 'MAX');
			$qb->where(array('=', 'bf_filters', 'bf_did', $this->fields['bf_did']));
			$max = intval($qb->getOne(false)) + 1;
			if (empty($action) || $action > $max)  {
				$action = $max;
			}
		}

		parent::set($field, $action);
	}

	/**
	 * Thanks to over-riding fields, I need to implement a catch-all for
	 * fid change and make sure they happen smoothly.
	 */
	public function save() {
		global $_REQUEST;
		if (array_key_exists('bf_fid', $this->fields) && !empty($this->fields['bf_fid'])) {
			// Test the format
			if (!$this->manageFIDs()) {
				throw new BFException(translate('2193'));
			}
		// Test for duplicate filter names
		} else if (strtolower($this->fields['bf_pattern']) != strtolower($this->stored_values['bf_pattern'])) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_pattern', 'bf_filters', 'patterns', 'COUNT');
			$qb->where(array(
							array('=', 'bf_filters', 'bf_fid', 0),
							array('=', 'bf_filters', 'bf_pattern', $this->fields['bf_pattern'])
						));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('log filter')));
			}
		}
		// bf_fid can't = '' ~ Int actions can't be null
		$this->fields['bf_fid'] = ($this->fields['bf_fid'] == '')?'0':$this->fields['bf_fid'];
		
		$type = $this->inserted ? 'Updated' : 'Created';
		if (parent::save())  {
			return true;
		}
		
		return false;
	}

	/**
	 * No matter what, if the fid changes or an filter variable gets
	 * created, we need the fids to stay in line.
	 */
	protected function manageFIDs() {
		// Only change FIDs if inserting or updating this FID
		if ($this->inserted && !$this->updated['bf_fid']) {
			return true;
		}

		// No matter what, we'll need the fids for all filter vars in this filter.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_fid', 'bf_filters', 'bf_fid');
		$qb->where(array(
						array('=', 'bf_filters', 'bf_did', $this->fields['bf_did']),
						array('!=', 'bf_filters', 'bf_fid', 0)
					));
		$qb->orderBy('ASC', 'bf_fid');

		// Please don't break...
		$fids = $qb->getCol(0, false);
		if (DB::isError($fids)) {
			throw new BFDBException($fids->getMessage());
		}

		// Never let bf_fid be greater than 1+ count(fids). (new below/move to bottom)
		// if fids == 0 (empty envgroup) set eid = 1;
		// Did this stuff ever work? - KWP
		$MaxFid = count($fids);
		$MaxFid = $MaxFid ? $MaxFid : 1 ;
		if ($this->fields['bf_fid'] > $MaxFid + 1) {
			$this->fields['bf_fid'] = $MaxFid;
		}

		// If moving this variable, move it aside and THEN into place to avoid stepping on constraints
		if ($this->inserted) {
			$update = BFQueryBuilder::update();
			$update->setColumn('bf_fid', 'bf_filters', count($fids) + 1);
			$update->where(array(
					array('=', 'bf_filters', 'bf_did', $this->fields['bf_did']),
					array('=', 'bf_filters', 'bf_fid', $this->stored_values['bf_fid'])
				));
			$upres = $update->getResult();
			if (DB::isError($upres)) {
				throw new BFDBException($upres->getMessage());
			}
		}

		// Doesn't matter if this edits this object in the mad dash, about to save anyway
		if ($this->inserted || in_array($this->fields['bf_fid'], $fids)) {
			$new_fid = intval($this->fields['bf_fid']);
			$old_fid = ($this->inserted) ? intval($this->stored_values['bf_fid']) : $fids[count($fids) - 1] + 1;
			$dir = ($new_fid < $old_fid) ? -1 : 1;
			// Okay...
			// If the EID gets increased, count from the old EID + 1 to the new EID, subtracting 1 as we go.
			// If the EID gets decreased, count from the old EID - 1 to the new EID, adding 1 as we go.
			for ($i = $old_fid + $dir; ($i >= $new_fid && $dir == -1) || ($i <= $new_fid && $dir == 1) && in_array($i, $fids); $i += $dir) {
				$update = BFQueryBuilder::update();
				$update->setColumn('bf_fid', 'bf_filters', $i - $dir);
				$update->where(array(
							array('=', 'bf_filters', 'bf_did', $this->fields['bf_did']),
							array('=', 'bf_filters', 'bf_fid', $i)
						));
				$upres = $update->getResult();
				if (DB::isError($upres)) {
					return false;
				}
			}
			// Fix if necessary
			if ($dir == 1 && $this->inserted && !in_array($this->fields['bf_fid'], $fids)) {
				$this->fields['bf_fid']--;
			}
		}
		$this->stored_values['bf_fid'] = count($fids) + 1;
		return true;
	}

	protected function cleanFIDs() {
		// No matter what, we'll need the fids for all filter vars in this filter.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_fid', 'bf_filters', 'bf_fid');
		$qb->where(array(
						 array('=', 'bf_filters', 'bf_did', $this->fields['bf_did']),
						 array('!=', 'bf_filters', 'bf_fid', 0)
						 ));
		$qb->orderBy('ASC', 'bf_fid');

		// Please don't break...
		$fids = $qb->getCol();
		if (DB::isError($fids)) {
			throw new BFDBException($fids->getMessage());
		}

		// TODO: Use an increment expression rather than a ton of updates
		$this_fid = intval($this->fields['bf_fid']);
		foreach ($fids as $fid) {
			$fid = intval($fid);
			if ($fid > $this_fid) {
				$update = BFQueryBuilder::update();
				$update->setColumn('bf_fid', 'bf_filters', ($fid - 1));
				$update->where(array(
									 array('=', 'bf_filters', 'bf_did', $this->fields['bf_did']),
									 array('=', 'bf_filters', 'bf_fid', $fid)
									 ));
				$upres = $update->getResult();
				if (DB::isError($upres)) {
					return false;
				}
			}
		}

		return true;
	}

	public function insert()  {
		if ($this->fields['bf_fid'] == '0' && !$GLOBALS['controller']->userHasAccessTo('AddFilterSet')) {
			throw new BFAccessException('AddFilterSet');
		}  elseif ($this->fields['bf_fid'] != '0' && !$GLOBALS['controller']->userHasAccessTo('AddFilterSetEntry')) {
			throw new BFAccessException('AddFilterSetEntry');
		}
		
		return parent::insert();
	}

	public function update() {
		if ($this->stored_values['bf_fid'] != '0' && !$GLOBALS['controller']->userHasAccessTo('EditFilterSetEntry')) {
			throw new BFAccessException('EditFilterSetEntry');
		}

		$qb = BFQueryBuilder::update();
		$updated = false;

		foreach ($this->tablefields as $field) {
			if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
				$qb->setColumn($field, $this->table, $this->fields[$field]);
				$updated = true;
			}
		}
		if (!$updated)  {
			return true;
		}

		$qb->where(array(
						 array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						 array('=', $this->table, 'bf_fid', $this->stored_values['bf_fid']),
						 array('=', $this->table, 'bf_pattern', $this->stored_values['bf_pattern']),
						 array('=', $this->table, 'bf_action', $this->stored_values['bf_action'])
					));
		$res = $qb->getResult();
		if (!DB::isError($res))  {
			$this->logChange('Updated');
			return true;
		}

		return false;
	}

	public function delete() {
		if ($this->stored_values['bf_fid'] == '0') {
			return $this->deleteFilter();
		} else {
			return $this->deleteFilterPattern();
		}
	}

	protected function deleteFilter() {
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteFilterSet')) {
			throw new BFAccessException('DeleteFilterSet');
		} elseif ($this->referenced() > 0) {
			throw new BFException(translate(2054));
		}

		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key])
					));
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}


	/**
	 * Overridden in order to copy all the filter patterns as well as the filter itself on a top-level copy.
	 */
	public function copy($direction = null)  {
		if ($this->stored_values['bf_fid'] != '0')  {
			if ($direction)  {
				$newfid = $this->fields['bf_fid'] + $direction;
				// Inserting upstream goes one too far
				if ($direction < 0) {
					$newfid++;
				}
				// FIDs must stay > 0
				if ($newfid < 1)  {
					$newfid = 1;
				}
				$this->set('bf_fid', $newfid);
			}  else {
				$this->set('bf_fid', '');
			}
			$this->inserted = false;
			return $this->save();
		}  else {
			//  We need to select all the filter patterns from the database and re-insert them with the new ID.
			$oldid = $this->fields['bf_did'];
			parent::copy();
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_fid', 'bf_filters');
			$qb->addColumn('bf_pattern', 'bf_filters');
			$qb->addColumn('bf_action', 'bf_filters');
			$qb->addColumn('bf_level', 'bf_filters');
			$qb->where(array(
							array('!=', 'bf_filters', 'bf_fid', 0),
							array('=', 'bf_filters', 'bf_did', $oldid)
						));
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			} else {
				$new_did = $this->get($this->primary_key);
				while ($var = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
					$new_var = new BFFilter();
					$new_var->set($this->primary_key, $new_did);
					$new_var->set('bf_fid', $var['bf_fid']);
					$new_var->set('bf_pattern', $var['bf_pattern']);
					$new_var->set('bf_action', $var['bf_action']);
					$new_var->set('bf_level', $var['bf_level']);
					$new_var->save();
				}
				return true;
			}
			return false;
		}
	}


	/**
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_did', 'bf_steps', 'bf_did', 'COUNT');
		$qb->where(array('=', 'bf_steps', 'bf_did', $this->fields[$this->primary_key]));
		return intval($qb->getOne()) > 0;
	}


	protected function deleteFilterPattern() {
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteFilterSetEntry')) {
			throw new BFAccessException('DeleteFilterSetEntry');
		}

		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						array('=', $this->table, 'bf_fid', $this->stored_values['bf_fid']),
						array('=', $this->table, 'bf_pattern', $this->stored_values['bf_pattern']),
						array('=', $this->table, 'bf_action', $this->stored_values['bf_action'])
					));
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else if (!$this->cleanFIDs()) {
			throw new BFException(translate('2193'));
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}


	/**
	 * Because I'm sick and tired of not having it, here's a function to get the name of this filter
	 * group, whether this is a pattern or a group.  Never say I don't do anything nice for you.
	 */
	 public function getFilterGroupName()  {
	 	if (!$this->filtername)  {
		 	if ($this->fields['bf_fid'] == '0')  {
		 		$this->filtername = $this->get('bf_pattern');
		 	}  else {
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_pattern', 'bf_filters', 'name');
				$qb->where(array(
								array('=', 'bf_filters', 'bf_fid', 0),
								array('=', 'bf_filters', 'bf_did', $this->fields['bf_did'])
							));
				$this->filtername = $qb->getOne();
				if (DB::isError($this->filtername))  {
					$this->filtername = '';
				}
			}
	 	}
	 	return $this->filtername;
	 }

	/**
	 * Overridden to deal with the filter / pattern dichotomy.
	 */
	public function logChange($type)  {
		if ($this->log_changes && ($type != 'Updated' || 
				$this->controller->getSysconfig('log_all_changes') != 'Y'))  {
			if ($this->fields['bf_fid'] != '0')  {
				$this->controller->logAudit('FilterPattern' . $type, 
						array($this->fields['bf_pattern'], $this->getFilterGroupName()));
			}  else {
				$this->controller->logAudit('Filter' . $type, array($this->fields['bf_pattern']));
			}
		}
	}

	/**
	 * A BFDBO-centered logging function used to log before-and-after state on changed fields.  
	 */
	public function logChangedFields($field, $before, $after)  {
		$fieldname = isset($this->field_descriptions[$field]) ? 
				($this->field_descriptions[$field] ? translate($this->field_descriptions[$field]) : '') : 
				$field;
		if ($this->log_changes && $this->inserted && $fieldname &&
				$this->controller->getSysconfig('log_all_changes') == 'Y')  {
			if ($this->fields['bf_fid'] != '0')  {
				$this->controller->logAudit('FilterPatternUpdatedField', array($fieldname,
						$this->fields['bf_pattern'], $this->getFilterGroupName(), $before, $after));
			}  else {
				$this->controller->logAudit('FilterUpdatedField', array($fieldname,
						$this->fields['bf_pattern'], $before, $after));
			}
		}
	}
}

?>
