<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFGeo
 * A database object providing access to CRUDing the small cars stored in the
 * database.
 * @package Console
 * @subpackage Core
 */


class BFGeo extends BFDBO {
	public $table = 'bf_geo';
	public $primary_key = 'bf_geo_id';
	public $primary_key_type_override = 'generic';
	public $required_fields = array('bf_location', 'bf_hostname', 'bf_database', 'bf_db', 'bf_active');
	public $field_defaults = array('bf_active' => 'Y', 'bf_failover' => null);
	public $type_name = 'Console';
	protected $dsn;
	protected $type;
	public $field_descriptions = array(
			'bf_geo_id' => '',
			'bf_location' => 'location',
			'bf_hostname' => 'host',
			'bf_database' => 'database',
			'bf_username' => 'username',
			'bf_port' => 'port',
			'bf_password' => '',
			'bf_db' => 'db',
			'bf_active' => 'active',
			'bf_failover' => 'failover geo'
			);

	public function set($field, $value) {
		if ($field == 'bf_active' || $field == 'bf_port') {
			return parent::set($field, $value);
		} else if ($field == 'bf_failover') {
			if ($value == $this->fields[$this->primary_key]) {
				return false;
			} else {
				return parent::set($field, $value);
			}
		} else {
			return false;
		}
	}
	
	/**
	 * Override to check permissions
	 */
	public function save() {
		if ($this->controller->userHasAccessTo('SaveGeo')) {
			return parent::save();
		} else {
			throw new BFAccessException('SaveGeo');
		}
	}
	
	/**
	 * Override to check permissions
	 */
	public function delete() {
		if ($this->controller->userHasAccessTo('DeleteGeo')) {
			if ($this->projectReferences() > 0) {
				throw new BFException(translate('no delete geo project references'));
			} else if ($this->buildReferences() > 0) {
				throw new BFException(translate('no delete geo build references'));
			} else if ($this->geoReferences() > 0) {
				throw new BFException(translate('no delete geo failover references'));
			}
			return parent::delete();
		} else {
			throw new BFAccessException('DeleteGeo');
		}
	}

	/**
	 * If it has everything it needs to make a full DSN, it makes one with the info out of the
	 * configuration file. If not, it makes one with the system defaults.
	 */
	public function reconstructDSN() {
		global $GLOBAL_DATABASE, $GLOBAL_DB_CHARSET;
		$provider = null;
		$hostname = null;
		$this->fields['bf_db'] = strtolower($this->fields['bf_db']);

		if ($this->fields['bf_db'] == 'local') {
			$this->type = $provider = 'db2';
			$hostname = 'localhost';
		} else {
			$this->fields['bf_db'] = strtolower($this->fields['bf_db']);
			$hostname = $this->fields['bf_hostname'];
			switch ($this->fields['bf_db']) {
				case 'local':
					$this->type = $provider = 'db2';
					break;
				case 'mysql':
					$this->type = $provider = 'mysqli';
					break;
				case 'pgsql':
				case 'postgres':
					$this->type = $provider = 'pgsql';
					break;
				case 'sybase':
				case 'sqlserver':
					$provider = 'odbc';
					$this->type = $this->fields['bf_db'];
					break;
				case 'oracle':
					$this->type = $provider = 'oci8';
					break;
				default:
					$provider = $this->fields['bf_db'];
					$this->type = $provider;
					break;
			}
		}
		$this->dsn = array(
			'php_type' => $this->type,
			'provider' => $provider,
			'username' => $this->fields['bf_username'],
			'password' => $this->fields['bf_password'],
			'hostname' => $this->fields['bf_hostname'],
			'tcp_port' => (isset($this->fields['db_tcp_port']) ? $this->fields['bf_port'] : null),
			'database' => $this->fields['bf_database']
		);
	}

	/**
	 * Using the current field values, return a DSN string
	 */
	public function getDSN() {
		if (!isset($this->dsn)) {
			$this->reconstructDSN();
		}
		return $this->dsn;
	}

	public function test() {
		try {
			$connection = $this->controller->getDB($this->fields[$this->primary_key]);
			return true;
		} catch (Exception $e) {
			return $e->getMessage();
		}
	}
	
	/**
	 * Get the number of projects referencing this location
	 */
	public function projectReferences() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid', 'bf_projects', 'projects', 'COUNT');
		$qb->where(array('=', 'bf_projects', 'bf_geo_id', $this->fields['bf_geo_id']));
		return (int)$qb->getOne();
	}

	/**
	 * Get the number of builds run on this location
	 */
	public function buildReferences() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_builds', 'builds', 'COUNT');
		$qb->where(array('=', 'bf_builds', 'bf_geo_id', $this->fields['bf_geo_id']));
		return (int)$qb->getOne();
	}

	/**
	 * Get the number of geos failing over to this location
	 */
	public function geoReferences() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_geo_id', 'bf_geo', 'geos', 'COUNT');
		$qb->where(array('=', 'bf_geo', 'bf_failover', $this->fields['bf_geo_id']));
		return (int)$qb->getOne();
	}

	static public function getDSNFromGeoID($geo_id) {
		$geo = new BFGeo();
		$geo->select($geo->primary_key, $geo_id);
		$dsn = $geo->getDSN();
		return $dsn;
	}
}

?>
