<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFInterface.php
 * 
 * @package Console
 * @subpackage Adaptors
 */

/**
 * BFInterface
 */
class BFInterface extends BFDBO {
	public $table = 'bf_interfaces';
	public $primary_key = 'bf_link';
	public $descriptor_key = 'bf_link';
	public $bfstores = array('bf_interface' => null);
	public $required_fields = array('bf_link', 'bf_mode', 'bf_lid', 'bf_level');
	public $field_defaults = array('bf_lid' => '1');
	public $stored_values = array('bf_link' => null);
	public $type_name = 'Interface';
	public $field_descriptions = array(
			'bf_link' => 'name',
			'bf_mode' => 'type',
			'bf_interface' => 'interface',
			'bf_template' => 'template',
			'bf_lid' => '',
			'bf_level' => 'access'
			);

	/**
	 * Overridden for access check on disable/enable
	 */
	public function set($key, $val) {
		if ($this->inserted && $key == 'bf_active') {
			// Access check
			if (!$this->controller->userHasAccessTo('ToggleLink')) {
				throw new BFAccessException('ToggleLink');
			}
		}
		return parent::set($key, $val);
	}

	/**
	 * Overriden to check for an existing interface with that name
	 */
	public function save() {
		if ($this->updated['bf_link'] && $this->stored_values['bf_link'] != $this->fields['bf_link']) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_link', $this->table, 'bf_link', 'COUNT');
			$qb->where(array(
				array('=', $this->table, 'bf_link', $this->fields['bf_link']),
				array('!=', $this->table, 'bf_link', $this->stored_values['bf_link']),
			));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
				return false;
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('adaptor')));
				return false;
			}

			//  Also check if there are any interface templates with the same name.
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_name', 'bf_interfacetemplates', 'bf_name', 'COUNT');
			$qb->where(array(
					array('=', 'bf_interfacetemplates', 'bf_name', $this->fields['bf_link']),					
					array('=', 'bf_interfacetemplates', 'bf_type', 'interface')
					));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
				return false;
			} else if (intval($count) > 0) {
				throw new BFException(translate('2236', translate('adaptor')));
				return false;
			}
			
			// Then, rename all Adaptor Links currently pointing at this Adaptor but only if we are re-naming, not for a copy
			if($this->inserted){
				$qb = BFQueryBuilder::update();
				$qb->setColumn('bf_link', 'bf_scopes', $this->fields['bf_link']);
				$qb->where(array('=', 'bf_scopes', 'bf_link', $this->stored_values['bf_link']));
				$qb->getResult(); 
			}
		}
		
		$type = $this->inserted ? 'InterfaceUpdated' : 'InterfaceCreated';

		if (parent::save())  {
			return true;
		}
		return false;
	}

	/**
	 * Override insert to fetch the content from bf_interfacetemplates
	 * if necessary, of course.
	 */
	public function insert() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('AddInterface')) {
			throw new BFAccessException('AddInterface');
		}
		
		// Only bother with this if using a template
		if (!empty($this->fields['bf_template'])) {
			$template = bf_get_interfacetemplate($this->fields['bf_template']);
			if (!$template)  {
				$this->controller->addError(translate('2197'));
			}  else {
				$this->set('bf_interface', $template);
			}
		}
		
		// Pass everything off to the parent object
		return parent::insert();
	}
	
	public function update() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('EditInterface')) {
			throw new BFAccessException('EditInterface');
		}
		
		if (!$this->hasRequiredFields()) {
			// Catch this and handle it sanely per-object
			throw new BFException(translate('2181'));
		}
		$qb = BFQueryBuilder::update();
		
		foreach ($this->tablefields as $field) {
			if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
				// If a bfstored value, need to save that first to get the ID
				if (array_key_exists($field, $this->bfstores)) {
					$this->fields[$field] = $this->bfstore->setStorage($this->bfstores[$field]['TEXT'], $this->bfstores[$field]['UID']);
				}
				$qb->setColumn($field, $this->table, $this->fields[$field]);
			}
		}
		
		$qb->where(array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]));
		
		$res = $qb->getResult();
		if (!DB::isError($res))  {
			$this->logChange('Updated');
			return true;
		}
		return false;
	}
	
	public function delete() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteInterface')) {
			throw new BFAccessException('DeleteInterface');
		}
		
		$refs = $this->referenced();
		if ($refs)  {
			throw new BFDBException(translate('2062', $refs));
		}
		
		$qb = BFQueryBuilder::delete();
		$qb->where(array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]));
		
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}

	/**
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_link', 'bf_scopes', 'bf_link', 'COUNT');
		$qb->where(array('=', 'bf_scopes', 'bf_link', $this->fields[$this->primary_key]));
		return intval($qb->getOne());
	}

}

class BFInterfaceTemplate extends BFDBO {
	public $table = 'bf_interfacetemplates';
	public $primary_key = 'bf_name';
	public $bfstores = array('bf_text' => null);
}



?>
