<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFJobs
 *
 * @package Console
 * @subpackage Core
 */

define('TABLE_LOCK', true);

$GLOBALS['controller']->require_module('BFDBO');
$GLOBALS['controller']->require_module('BFUID');
$GLOBALS['controller']->require_module('BFStart');

class BFJobs extends BFDBO {
	public $table		= 'bf_builds';
	public $primary_key = 'bf_bid';


	/** Purge Jobs
	 *	This function purges all selected jobs by creating a new record in the build database flagged as purge.
	 *	Required: $Jobs = array of build IDs
	 *	User must have access to purge builds and the build must be completed.	 	 
	 */
	public function purgeJobs($jobs) {
		global $user;
		
		if (!$GLOBALS['controller']->userHasAccessTo('PurgeBuild')) {
			throw new BFAccessException('PurgeBuild');
		}

		$purges = array();
		
		/** Only projects that are completed or archived may be purged */
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_builds');
		$qb->where(array(
				array('IN', 'bf_builds', 'bf_bid', $jobs),
				'AND',
				'(',
				array('=', 'bf_builds', 'bf_state', 'C'),
				'OR',
				array('=', 'bf_builds', 'bf_state', 'A'),
				')'
				));
		$purges = $qb->getCol();

		/** Walk through purges and make it happen */		
		foreach ($purges as $k=>$PurgeID) {
			
			// Get the build id for this build
			$qb2 = BFQueryBuilder::select();
			$qb2->addColumn('bf_value',   'bf_sysconfig',	'value');
			$qb2->where(array (
				array('=', 'bf_sysconfig',  'bf_param', 'build_id')
			));
			$thisBuildID = $qb2->getOne(false);
			$nextBuildID = $thisBuildID + 1; // Probably need to run some check here to make sure that id hasn't been used or isn't in use.

			// Set the next build id to be used
			$qb3 = BFQueryBuilder::update(null, TABLE_LOCK);
			$qb3->setColumn('bf_value',  'bf_sysconfig',  $nextBuildID);
			$qb3->where(array(
				array('=', 'bf_sysconfig', 'bf_param', 'build_id'),
				array('=', 'bf_sysconfig', 'bf_value', $thisBuildID)
			));
			$res = $qb3->getResult();

			if (DB::isError($res)) {
				$this->controller->addError(translate(2165));
				return false;
			}

			// Get the tag of the build to be purged (for logging)
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tag', 'bf_builds', 'tag');
			$qb->where(array(
				array('=', 'bf_builds', 'bf_bid', $PurgeID),
			));
			$tag = $qb->getOne();

// CHAD
			// Get the geo_id of the build to be purged (so that a secondary engine can purge if need be)
			$qb4 = BFQueryBuilder::select();
			$qb4->addColumn('bf_geo_id', 'bf_builds', 'geo_id');
			$qb4->where(array(
				array('=', 'bf_builds', 'bf_bid', $PurgeID),
			));
			$thisGeo = $qb4->getOne();
// CHAD

			$qb = BFQueryBuilder::insert(null, TABLE_LOCK);
			$qb->setColumn('bf_bid',		'bf_builds',	$thisBuildID);				//Use THIS buildid as pulled from bf_sysconfig
			$qb->setColumn('bf_pid',		'bf_builds',	$PurgeID);					//Purge ID
			$qb->setColumn('bf_start',		'bf_builds',	'0');						//When run starts, for us set to zero
			$qb->setColumn('bf_state',		'bf_builds',	'W');						//State of build, for us 'W'aiting
			$qb->setColumn('bf_process',	'bf_builds',	'P');						//Type of build, for us 'P'urge
			$qb->setColumn('bf_prg',		'bf_builds',	'N');						//No longer used, 'N'
			$qb->setColumn('bf_can',		'bf_builds',	'N');						//Cancelled, 'N'
			$qb->setColumn('bf_resource',	'bf_builds',	'');						//Server
			$qb->setColumn('bf_tag',		'bf_builds',	'');						//Tag parse-see bf_tags table
			$qb->setColumn('bf_userid',		'bf_builds',	$user->get('bf_userid'));	//User ID of user that started build
			$qb->setColumn('bf_level',		'bf_builds',	'0');						//Project Level
			$qb->setColumn('bf_result',		'bf_builds',	'');						//Result,''
			$qb->setColumn('bf_duration',	'bf_builds',	'0');						//Time taken to execute,'0'
			$qb->setColumn('bf_size',		'bf_builds',	'0');						//Overloaded,Reflector setting,'0'
			$qb->setColumn('bf_stage',		'bf_builds',	'');						//Stage of build,''
			$qb->setColumn('bf_fired',		'bf_builds',	date('U'));					//UTC of when person clicked "RUN"
			$qb->setColumn('bf_jobid',		'bf_builds',	'0');						//No longer used,'0'
			$qb->setColumn('bf_env',		'bf_builds',	'');						//Project Environment
			$qb->setColumn('bf_class',		'bf_builds',	'');						//Project Class
			$qb->setColumn('bf_engineid',	'bf_builds',	'');
			$qb->setColumn('bf_geo_id',		'bf_builds',	$thisGeo);
			$res = $qb->getResult();
			
			if (DB::isError($res)) {			
				$this->controller->addError(translate(2166));
				return false;
			}
			// Logging.
			$this->controller->logAudit('BuildPurgeQueued', array($tag));
		}			

		return true;	
	}


	/** Lock Jobs
	 * This will lock a job 
	 * $jobs is an array of all jobs to be locked	 	
	 */	
	public function lockJobs($jobs) {
		if (!$GLOBALS['controller']->userHasAccessTo('LockBuild')) {
			throw new BFAccessException('LockBuild');
		}

		/** Only projects that are completed may be locked */
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_builds');
		$qb->where(array(
						array('=', 'bf_builds', 'bf_state', 'C'),
						array('IN', 'bf_builds', 'bf_bid', $jobs)
					));
		$ourRuns = $qb->getCol();
			
		if (!empty($ourRuns)) {
			$restarts = array();
			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_state',	'bf_builds',	'L');
			$qb->where(array('IN', 'bf_builds', 'bf_bid', $ourRuns));
			if (DB::isError($qb->getResult())) { return false; }

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tag', 'bf_builds', 'tag');
			$qb->addColumn('bf_description', 'bf_projects', 'projectname');

			$qb->where(array(
				array('IN', 'bf_builds', 'bf_bid', $ourRuns),
				array('=', 'bf_builds', 'bf_pid', 'bf_projects', 'bf_pid')
			));
			$results = $qb->getResult();
			if (!DB::isError($results))  {
				while($row = $results->fetchRow())  {
					$this->controller->logAudit('BuildLocked', array($row->projectname, $row->tag));
				}
			}
			return true;
		}
		return false;
	}


	/** Unlock Jobs
	 * This will unlock an array of jobs
	 * $Jobs is an array of jobs to be unlocked
	 */	 	 	
	public function unlockJobs($jobs) {
		if (!$GLOBALS['controller']->userHasAccessTo('UnlockBuild')) {
			throw new BFAccessException('UnlockBuild');
		}

		$ourRuns = array();
		/** Only projects that are locked may be unlocked */
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_builds');
		$qb->where(array(
						array('=', 'bf_builds', 'bf_state', 'L'),
						array('IN', 'bf_builds', 'bf_bid', $jobs)
					));
		$ourRuns = $qb->getCol();
		
		if (!empty($ourRuns)) {
			$restarts = array();
			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_state',	'bf_builds',	'C');
			$qb->where(array('IN', 'bf_builds', 'bf_bid', $ourRuns));
			if (DB::isError($qb->getResult())) { return false; }

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tag', 'bf_builds', 'tag');
			$qb->addColumn('bf_description', 'bf_projects', 'projectname');

			$qb->where(array(
				array('IN', 'bf_builds', 'bf_bid', $ourRuns),
				array('=', 'bf_builds', 'bf_pid', 'bf_projects', 'bf_pid')
			));
			$results = $qb->getResult();
			if (!DB::isError($results))  {
				while($row = $results->fetchRow())  {
					$this->controller->logAudit('BuildUnlocked', array($row->projectname, $row->tag));
				}
			}
			return true;
		}
		return false;
	}

	/** Pause Step
	 * This function will pause a job at a specific step
	 */
	public function pauseStep($bid, $lid) {
		if (!$GLOBALS['controller']->userHasAccessTo('PauseStep')) {
			throw new BFAccessException('PauseStep');
		}

		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_flag', 'bf_results', 'P');
		$qb->where(array(
			array('=', 'bf_results', 'bf_bid', $bid),
			array('=', 'bf_results', 'bf_lid', $lid)
		));
		if (DB::isError($qb->getResult())) { return false; }

		// Logging.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tag', 'bf_builds', 'tag');
		$qb->where(array(
			array('=', 'bf_builds', 'bf_bid', $bid),
		));
		$tag = $qb->getOne();

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description', 'bf_results');
		$qb->where(array(
			array('=', 'bf_results', 'bf_bid', $bid),
			array('=', 'bf_results', 'bf_lid', $lid)
		));
		$step = $qb->getOne();
		$this->controller->logAudit('BuildStepPaused', array($step, $tag));
		return true;
	}

	/** Unpause Step
	 * This function will unpause a job at a specific step
	 */
	public function unpauseStep($bid, $lid) {
		if (!$GLOBALS['controller']->userHasAccessTo('RunStep')) {
			throw new BFAccessException('RunStep');
		}

		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_flag', 'bf_results', 'R');
		$qb->where(array(
			array('=', 'bf_results', 'bf_bid', $bid),
			array('=', 'bf_results', 'bf_lid', $lid)
		));
		if (DB::isError($qb->getResult())) { return false; }

		// Logging.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tag', 'bf_builds', 'tag');
		$qb->where(array(
			array('=', 'bf_builds', 'bf_bid', $bid),
		));
		$tag = $qb->getOne();

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description', 'bf_results');
		$qb->where(array(
			array('=', 'bf_results', 'bf_bid', $bid),
			array('=', 'bf_results', 'bf_lid', $lid)
		));
		$step = $qb->getOne();
		$this->controller->logAudit('BuildStepUnpaused', array($step, $tag));
		return true;
	}

	/** Cancel Purge
	 *  This function will cancel a purge
	 */
	public function cancelPurge($bid) {
		if (!$GLOBALS['controller']->userHasAccessTo('PurgeBuild')) {
			throw new BFAccessException('PurgeBuild');
		}

		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_can', 'bf_builds', 'Y');
		$qb->where(array(
			array('=', 'bf_builds', 'bf_bid', $bid),
		));
		if (DB::isError($qb->getResult())) { return false; }

		// Logging.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tag', 'bf_builds', 'tag');
		$qb->where(array(
			array('=', 'bf_builds', 'bf_bid', $bid),
		));
		$tag = $qb->getOne();
		$this->controller->logAudit('BuildPurgeCancelled', array($tag));
		return true;
	}
	
	
	/** Cancel Jobs
	 * This function will cancel all jobs in the array  
	 * 	 if the job is still in a fetal state, it just deletes the build record,
	 *   otherwise it sets the can flag
	 */
	public function cancelJobs($jobs) {		
		foreach($jobs as $key => $value) {
			// Is this job currently in the waiting state and not started?
			$qb = BFQueryBuilder::select();
			$qb->where(array('=', 'bf_builds', 'bf_bid', $value));
			$res = $qb->getResult();
			
			if (!DB::isError($res)) {
				$result = $res->fetchRow();
				if ($result->bf_state == 'W' && $result->bf_start == '0') {
					if ($this->controller->userHasAccessTo('DeleteBuildWaiting')) {
						$qb = BFQueryBuilder::delete();
						$qb->where(array('=', 'bf_builds', 'bf_bid', $value));
						if (DB::isError($qb->getResult())) {
							$this->controller->addError(translate(2167));
							return false;
						}						
					} else {
						throw new BFAccessException('DeleteBuildWaiting');
						return false;
					}
				} else if ($result->bf_state != 'C') {
					if ($this->controller->userHasAccessTo('CancelRunning')) {
						$qb = BFQueryBuilder::update();
						$qb->setColumn('bf_can',	'bf_builds',	'Y');
						$qb->where(array('=', 'bf_builds', 'bf_bid', $value));
						if (DB::isError($qb->getResult())) { 
							$this->controller->addError(translate(2168));
							return false;
						}
					} else {
						throw new BFAccessException('CancelRunning');
						return false;
					}
				}			
			}
		}
		
		return true;			
	}	 	 	
	
	
	
	public function changeClass($job, $newClassName) {
		global $user;
		
		// Does the user have access to do this?
		if (!$GLOBALS['controller']->userHasAccessTo('ChangeBuildClass')) {
			throw new BFAccessException('ChangeBuildClass');
		}
		
		/** Does the new class exist?  If so get the entry point project.
		 * Note: that even if there is no entry project, its still set as zero.
		 *  a result of nothing from in this query means our class name doesn't exist */		 		 		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_npid', 'bf_classes', 'entry');
		$qb->addColumn('bf_description', 'bf_projects', 'project');
		$qb->join(array('LEFT', 'bf_projects', array('=', 'bf_classes', 'bf_npid', 'bf_projects', 'bf_pid')));
		$qb->where(array('=', 'bf_classes', 'bf_name', $newClassName));
		$newClass = $qb->getRow(DB_FETCHMODE_ASSOC);
		if (!$newClass) {
			$this->controller->addError(translate(2169,$newClassName));
			return false;
		}
	
		// Does the builds class differ from newClassName?	
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_class', 'bf_builds');
		$qb->where(array('=', 'bf_builds', 'bf_bid', $job));
		$oldClassName = $qb->getOne();
		if ($oldClassName == $newClassName) {
			$this->controller->addError(translate(2170));
			return false;
		}	

		/** Get exit point for our old class - If the old class doesn't have an exit project
		 * it should still return a result of zero.  If it doesn't then our old class doesn't
		 * exist - but at this point we don't care so don't fail if the exit point returns null */		 		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_xpid', 'bf_classes', 'exit');
		$qb->addColumn('bf_description', 'bf_projects', 'project');
		$qb->join(array('LEFT', 'bf_projects', array('=', 'bf_classes', 'bf_xpid', 'bf_projects', 'bf_pid')));
		$qb->where(array('=', 'bf_classes', 'bf_name', $oldClassName));
		$oldClass = $qb->getRow(DB_FETCHMODE_ASSOC);

		/** A change class could fire off 0, 1 or 2 jobs	
		 *  Determine the users build limit, then determine the number of jobs
		 *	that will be fired off due to this action, then get the number of 
		 *	current builds the user has. 
		 *	If current build count + new job count > limit ... fail */		 
		$limit = $user->get('limit');
				
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_builds', 'bf_bid', 'COUNT');
		$qb->where(array(array('=', 'bf_builds', 'bf_userid', intval($user->get('bf_userid')))));
		$jobCount = (int)$qb->getOne();
						 	
		$jobCount = (!isset($newClass['entry']) || $newClass['entry'] === '')?$jobCount:$jobCount++;
		$jobCount = (!isset($oldClass['exit']) || $oldClass['exit'] === '')?$jobCount:$jobCount++;
		
		if ($limit > 0 && $jobCount >= $limit) {
			$this->controller->addError(translate(2171));
			return false;		
		} 
		
		/** Yeah we made it!   Assuming nothing has gone wrong above,
		 * its time to actually change the class  --  this is the point
		 * of no return -- Change the class. */		 		
		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_class', 'bf_builds', $newClassName);
		$qb->where(array(
			array('=', 'bf_builds', 'bf_class', $oldClassName),
			array('=', 'bf_builds', 'bf_bid', $job)
			));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			$this->controller->addError(translate(2172));
			return false;
		}

		// Gonna need this to run the two new projects
		$startObj = new BFStart();

		/** Run Entry Point
		 * Entry point project should be run first to avoid possible race conditions with files etc. 
		 * however the database picks them up and runs them in whatever order it wants so.. so be it. 
		 * 
		 * Send this job ID and the ID of the entry/exit point project - null/"" project will get
		 * kicked by the bfstart object so no need to check. */
		$notes = new BFNotes();
		$notes->noteType = 'ProjectRun';
		if (isset($newClass['entry']) && $newClass['entry'])  {
			$startObj->ClassChangeStart($job, $newClass['entry']);
			$notes->saveNote(array('note_text' => translate('2254', $newClassName, $newClass['project'])), $job);
		}
		// End: Edit/Save/Delete Notes
		if (isset($oldClass['exit']) && $oldClass['exit'])  {
			$startObj->ClassChangeStart($job, $oldClass['exit']);
			$notes->saveNote(array('note_text' => translate('2253', $oldClassName, $oldClass['project'])), $job);
		}

		return true;
	}
	
	
	
	
}

?>
