<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFLDAP
 * A database object providing access to CRUDing the LDAP records stored in the
 * database.
 * @package Console
 * @subpackage LDAP
 */


$GLOBALS['controller']->require_module('BFDBO');

class BFLDAP extends BFDBO {
	public $table		   = 'bf_ldap';
	public $primary_key	 = 'bf_domain';
	public $descriptor_key = 'bf_domain';
	public $required_fields = array('host');
	public $stored_values = array('bf_domain' => null);
	public $primary_key_type = 'VARCHAR';
	public $type_name = 'LDAP';
	public $field_descriptions = array(
			'bf_domain' => 'name',
			'access_groups' => 'map access groups',
			'admin_cn' => 'admin cn',
			'admin_password' => '',
			'displayname' => 'display name',
			'distinguishedname' => 'distinguished name',
			'groupname' => 'group name',
			'host' => 'host',
			'mailname' => 'mail name',
			'memberof' => 'authorized group dn',
			'search_base' => 'search base',
			'unique_identifier' => 'unique identifier',
			'user_bind' => 'bind user account',
			'group_search_base' => 'groups search base',
			'group_unique_identifier' => 'groups unique identifier',
			'protocol' => 'protocol'
			);

	/**
	 * Fetch the database object's record from the BFQueryBuilder result
	 * Necessary to override so that it loads the data from ALL matching
	 * records, rather than an actual table
	 */
	public function fetch() {
		if (!is_array($this->fields)) {
			$this->loadFields();
		}
		$res = $this->querybuilder->getResult();

		if (DB::isError($res)) {
			throw new BFDBException($this->querybuilder->getMessage());
		} else {
			$col = $res->fetchRow(DB_FETCHMODE_ASSOC);
			if (!DB::isError($col) && $col) {
				$this->updated	= array();	// Keeps track of which fields get modified
				$this->inserted = true;
				$firstfield = true;
				do  {
					// Only need to set this once...
					if ($firstfield) {
						$this->fields[$this->primary_key] = $col[BF_FIELD_PREFIX . 'domain'];
						$this->stored_values[$this->primary_key] = $col[BF_FIELD_PREFIX . 'domain'];
						$this->updated[$this->primary_key] = false;
						$firstfield = false;
					}
					$this->fields[$col[BF_FIELD_PREFIX . 'param']] = $col[BF_FIELD_PREFIX . 'value'];
					$this->updated[$col[BF_FIELD_PREFIX . 'param']] = false;
				}  while ($col = $res->fetchRow(DB_FETCHMODE_ASSOC));
				return true;
			}
		}
		
		// If you made it here, something went wrong...
		return false;
	}
	
	public function set($field, $value) {
		if ($field == 'protocol') {
			return parent::set($field, ($value == 'ldaps' ? $value : 'ldap'));
		} else {
			return parent::set($field, $value);
		}
	}
	
	/**
	 * Need to override loadFields, since this object has fake fields.
	 * "Fake" meaning that the fields actually come from records in the table
	 * rather than actual fields in the table. Can we re-design the schema yet?
	 */
	protected function loadFields() {
		$this->fields = array(
							  'bf_domain' => null,
							  'access_groups' => null,
							  'admin_cn' => null,
							  'admin_password' => null,
							  'displayname' => null,
							  'distinguishedname' => null,
							  'groupname' => null,
							  'host' => null,
							  'mailname' => null,
							  'memberof' => null,
							  'search_base' => null,
							  'unique_identifier' => null,
							  'user_bind' => null,
							  'group_search_base' => null,
							  'group_unique_identifier' => null,
							  'protocol' => null
							  );
		$this->updated = array(
							  'bf_domain' => null,
							  'access_groups' => null,
							  'admin_cn' => null,
							  'admin_password' => null,
							  'displayname' => null,
							  'distinguishedname' => null,
							  'groupname' => null,
							  'host' => null,
							  'mailname' => null,
							  'memberof' => null,
							  'search_base' => null,
							  'unique_identifier' => null,
							  'user_bind' => null,
							  'group_search_base' => null,
							  'group_unique_identifier' => null,
							  'protocol' => null
							  );
		$this->tablefields = array(
								   'access_groups',
								   'admin_cn',
								   'admin_password',
								   'displayname',
								   'distinguishedname',
								   'groupname',
								   'host',
								   'mailname',
								   'memberof',
								   'search_base',
								   'unique_identifier',
								   'user_bind',
								   'group_search_base',
								   'group_unique_identifier',
								   'protocol'
								   );
		return true;
	}
	
	/**
	 * Override to test for existing domains
	 */
	public function save() {
		// Got one like that already?
		if ($this->updated[$this->primary_key] && $this->fields[$this->primary_key] != $this->stored_values[$this->primary_key]) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn($this->primary_key, $this->table, $this->primary_key, 'COUNT');
			$qb->where(array(
				array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
				array('!=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key])
			));
			$qb->groupBy($this->table, $this->primary_key);
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2198'));
			}
		}
		$type = $this->inserted ? 'LDAPUpdated' : 'LDAPCreated';
		if (parent::save())  {
			return true;
		}
		return false;
	}
	
	/**
	 * Need to override insert as well to handle the fake fields.
	 * So instead of inserting a single record with a PK, we insert
	 * a record for each field with the LDAP domain.
	 */
	public function insert() {
		if (!$this->hasRequiredFields()) {
			// Catch this and handle it sanely per-object
			throw new BFException(translate('2181'));
		}
		
		// Loop through each of the TABLE fields. If we loop through all
		// $this->fields' members, we'd insert a record for the PK as well.
		foreach ($this->tablefields as $field) {
			$qb = BFQueryBuilder::insert();
			
			$qb->setColumn($this->primary_key, $this->table, $this->fields[$this->primary_key]);
			$qb->setColumn('bf_param', $this->table, $field);
			$qb->setColumn('bf_value', $this->table, $this->fields[$field]);
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}
		}
		
		// If you've made it this far...woo-hoo!
		$this->logChange('Created');
		return true;
	}
	
	/**
	 * Need to override update as well to handle the fake fields.
	 * So instead of updating a single record with a PK, we update
	 * a record for each field with the LDAP domain (if modified).
	 */
	public function update() {
		if (!$this->hasRequiredFields()) {
			// Catch this and handle it sanely per-object
			throw new BFException(translate('2181'));
		}
		
		// Loop through each of the TABLE fields. If we loop through all
		// $this->fields' members, we'd update a non-record for the PK as well.
		foreach ($this->tablefields as $field) {
			if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
				$qb = BFQueryBuilder::update();
				$qb->setColumn('bf_value', $this->table, $this->fields[$field]);
				$qb->where(array(
								 array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]),
								 array('=', $this->table, 'bf_param', $field)
								 ));
				$res = $qb->getResult();
				if (DB::isError($res)) {
					throw new BFDBException($res->getMessage());
				}
			}
		}
		
		// Update the domain itself?
		if ($this->updated[$this->primary_key]) {
			$qb = BFQueryBuilder::update();
			$qb->setColumn($this->primary_key, $this->table, $this->fields[$this->primary_key]);
			$qb->where(array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]));
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}
		}
		
		// If you've made it this far...woo-hoo!
		$this->logChange('Updated');
		return true;
	}

	
	/**
	 * Bind attempt
	 */
	public function test() {
		if (extension_loaded('ldap')) {

			$ldap_connection = @ldap_connect($this->get('protocol') . '://' . $this->get('host'));
			if (!$ldap_connection && !($ldap_connection = ldap_connect($this->get('host')))) {
				$this->controller->addError(translate('2200') . ' - ' . @ldap_error($ldap_connection));
				return false;
			}
			
			@ldap_set_option($ldap_connection, LDAP_OPT_PROTOCOL_VERSION, 3);
			$r = @ldap_bind($ldap_connection, $this->get('admin_cn'), bf_decrypt($this->get('admin_password')));
			if ($r) {
				return true;
			} else {
				throw new BFException(translate('2200') . ' - ' . ldap_error($ldap_connection));
			}
		} else {
			throw new BFException(translate('2199'));
		}
	}
	
	/**
	 * Get the information for a given user
	 * @param string username
	 * @return array
	 */
	public function getUserInfo($username) {
		global $controller;

		// Attempt to connect to the LDAP server
		$ldap_connection_string = $this->get('protocol') . '://' . $this->get('host');
		$ldap_connection = ldap_connect($ldap_connection_string);
		if (!$ldap_connection && !($ldap_connection = ldap_connect($this->get('host')))) {
			$controller->addError(translate('2200'));
			return false;
		}
		// Otherwise, attempt to bind based on the credentials supplied
		ldap_set_option($ldap_connection, LDAP_OPT_PROTOCOL_VERSION, 3);

		if(!($r = @ldap_bind($ldap_connection, $this->get('admin_cn'), bf_decrypt($this->get('admin_password'))))) {
			// This means something bad happened during the bind process
			$controller->addError(translate('2267'));
		};
		if ($r) {
			$ldap_dname = $this->get('host');
			// sub in the login string and go find the user account record
			$ldap_search_filter = str_replace("%",$username,$this->get('unique_identifier'));
			$ldap_query_fields = array(
									$ldap_dname,
									$this->get('displayname'),
									$this->get('mailname'),
									$this->get('groupname')
								);
			
			//escape parenthesis in search filter...
			if(strpos($ldap_search_filter,'(') || strpos($ldap_search_filter, ')'))
			{
				$ldap_search_filter = str_replace('(', '\(',$ldap_search_filter);
				$ldap_search_filter = str_replace(')', '\)',$ldap_search_filter);
			}
			$sr = ldap_search($ldap_connection, $this->get('search_base'), $ldap_search_filter, $ldap_query_fields);
			if ($sr) {
				// there must be only one...
				$records = ldap_count_entries($ldap_connection,$sr);
				if ($records == 1) {
					// read the record and use the DN field to re-bind or crypt and compare to the ldap entry to validate
	   				$ldap_info = ldap_get_entries($ldap_connection, $sr);

					// re-bind is ok or it crypt'd ok, valid user login.
					$results['fullname'] = $ldap_info[0][strtolower($this->get('displayname'))][0];
					$results['fullname'] = str_replace('\'', '&#39;', $results['fullname']);
					$results['email'] = (isset($ldap_info[0][$this->get('mailname')]))?$ldap_info[0][strtolower($this->get('mailname'))][0]:'';

					return $results;
				} elseif ($records == 0) {
					$controller->addError(translate('2232'));
				} else {
					$controller->addError(translate('2233'));
				}
			} else {
				$controller->addError(translate('2234'));
			}
		} else {
			$controller->addError(translate('2235'));
		}

		return false;
	}
}

?>
