<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFLevel
 * A database object providing access to CRUDing the selectors stored in the
 * database.
 * @package Console
 * @subpackage AccessGroups
 */


$GLOBALS['controller']->require_module('BFDBO');

class BFLevel extends BFDBO {
	public $table		   = 'bf_levels';
	public $primary_key	 = 'bf_level';
	public $descriptor_key = 'bf_name';
	public $required_fields = array('bf_name', 'bf_lvl');
	public $stored_values = array('bf_name' => null);
	protected $bfstores = array('bf_dn' => null);
	protected $field_defaults = array('bf_dn' => null, 'bf_def' => 'N');
	public $members;		//  $members are users who belong to the level
	public $nonmembers;
	public $children;		//  $children are levels who belong to the level
	public $nonchildren;
	public $parents;
	protected $stored_members = array();
	protected $stored_children = array();
	protected $members_added = array();
	protected $members_removed = array();
	protected $children_added = array();
	protected $children_removed = array();
	public $type_name = 'Access';
	public $field_descriptions = array(
			'bf_level' => '',
			'bf_name' => 'name',
			'bf_def' => 'default',
			'bf_dn' => 'ldap group dns',
			'bf_lvl' => 'owner'
			);

	/**
	 * Override save() to keep from duplicating access group names
	 */
	public function save() {
		// Only check if changed or inserting
		if ($this->updated['bf_name']) {
			// Get the number of groups with the current name
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_name', 'bf_levels', 'namecount', 'COUNT');
			$qb->where(array(
				array('=', 'bf_levels', 'bf_name', $this->fields['bf_name']),
				array('!=', 'bf_levels', 'bf_name', $this->stored_values['bf_name'])
			));
			$count = $qb->getOne();

			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2268'));
			}
		}
		return parent::save();
	}

	/**
	 * This updates bf_accesscache as well as bf_access.  Access checks are performed here.
	 */
	public function update() {
		$changes = false;
		if (count($this->members_removed) > 0) {
			if ($GLOBALS['controller']->userHasAccessTo('DeleteAccessMember')) {
				$changes = true;
				$this->clearMembers();
				$this->members_removed = array();
			} else {
				throw new BFAccessException('DeleteAccessMember');
			}
		}
		if (count($this->members_added) > 0) {
			if ($GLOBALS['controller']->userHasAccessTo('AddAccessMember')) {
				$changes = true;
				$this->insertMembers();
				$this->members_added = array();
			} else {
				throw new BFAccessException('AddAccessMember');
			}
		}
		if (count($this->children_removed) > 0) {
			if ($GLOBALS['controller']->userHasAccessTo('AddAccess')) {
				$changes = true;
				$this->clearChildren();
				$this->children_removed = array();
			} else {
				throw new BFAccessException('AddAccess');
			}
		}
		if (count($this->children_added) > 0) {
			if ($GLOBALS['controller']->userHasAccessTo('AddAccess')) {
				$changes = true;
				$this->insertChildren();
				$this->children_added = array();
			} else {
				throw new BFAccessException('AddAccess');
			}
		}
		
		// Changes? Rebuild the cache.
		if ($changes) {
			$s = explode(" ", microtime());	$m = explode(".", $s[0]);
			$uidLock = (substr($s[1], 4, 4).substr($m[1], 0, 4));
			
			// Now define the lockObject either nicely or through brute force.			
			$lockObject = $this->setUIDLock('generic', $uidLock);			
			if (!is_object($lockObject)) {
				$lockObject = $this->forceUIDLock('generic', $uidLock);
				if (!is_object($lockObject)) {
					throw new BFException(translate('2186'));
					die;
				}
			}
			$this->resetUIDLock($this->primary_key_type_override, $uidLock);
	
			// Now after all that mess, update the access cache.
			bf_rebuild_accesscache();
			bf_rebuild_paccesscache();
			bf_mark_access_cashe_dirty();
			return true;
		}
		if ($this->updated['bf_def'] && !$GLOBALS['controller']->userHasAccessTo('ToggleDefGroup'))  {
			throw new BFAccessException('ToggleDefGroup');
		}
		if (!$GLOBALS['controller']->userHasAccessTo('EditLevel')) {
			foreach(array_keys($this->fields) as $field)  {
				if ($field != 'bf_def' && $this->updated[$field])  {
					throw new BFAccessException('EditLevel');
				}
			}
		}  else {
			return parent::update();
		}
	}


	/**
	 * Override for access check
	 */
	public function insert() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('AddLevel')) {
			return parent::insert();
		} else {
			throw new BFAccessException('AddLevel');
		}
	}
	
	/**
	 * This deletes from bf_accesscache as well as bf_access
	 */
	public function delete() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('DeleteAccess')) {
			if (!$this->referenced()) {
				// Delete permission records
				$qb = BFQueryBuilder::delete();
				$qb->where(array('=', 'bf_ptable', 'bf_level', $this->fields['bf_level']));
				$res = $qb->getResult();
				// Delete access records
				$qb = BFQueryBuilder::delete();
				$qb->where(array(
								 array('=', 'bf_access', 'bf_groupid', $this->fields['bf_level']),
								 'OR',
								 array('=', 'bf_access', 'bf_level', $this->fields['bf_level'])
								 ));
				$res = $qb->getResult();
				// Delete access cache records
				$qb = BFQueryBuilder::delete();
				$qb->where(array(
								 array('=', 'bf_accesscache', 'bf_groupid', $this->fields['bf_level']),
								 'OR',
								 array('=', 'bf_accesscache', 'bf_level', $this->fields['bf_level'])
								 ));
				$res = $qb->getResult();
				
				return parent::delete();
			} else {
				throw new BFException(translate('2201', translate('access group')));
			}
		} else {
			throw new BFAccessException('DeleteAccess');
		}
	}
	
	/**
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		$tables = array('bf_servers', 'bf_projects', 'bf_steps', 'bf_filters', 'bf_classes', 'bf_env', 'bf_selector','bf_serverauth');
		$count = 0;
		foreach ($tables as $table) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_level', $table, 'bf_level', 'COUNT');
			$qb->where(array('=', $table, 'bf_level', $this->fields[$this->primary_key]));
			$count += intval($qb->getOne());
		}
		
		//Check if we own any other access group....
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_lvl', 'BF_LEVELS', 'bf_lvl', 'COUNT');
		$qb->where(array('=', 'BF_LEVELS', 'bf_lvl', $this->fields[$this->primary_key]));
		$count += intval($qb->getOne());
		
		return $count > 0;
	}
	
	protected function clearMembers() {
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						 array('=', 'bf_access', 'bf_level', $this->fields['bf_level']),
						 array('IN', 'bf_access', 'bf_userid', $this->members_removed)
						 ));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}
	}
	
	protected function insertMembers() {
		foreach ($this->members_added as $userid) {
			$qb = BFQueryBuilder::insert();
			$qb->setColumn('bf_level', 'bf_access', $this->fields['bf_level']);
			$qb->setColumn('bf_userid', 'bf_access', $userid);
			$qb->setColumn('bf_groupid', 'bf_access', 0);
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}
		}
	}
	
	protected function clearChildren() {
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						 array('=', 'bf_access', 'bf_groupid', $this->fields['bf_level']),
						 array('IN', 'bf_access', 'bf_level', $this->children_removed)
						 ));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}
	}
	
	protected function insertChildren() {
		foreach ($this->children_added as $child) {
			$qb = BFQueryBuilder::insert();
			$qb->setColumn('bf_level', 'bf_access', $child);
			$qb->setColumn('bf_userid', 'bf_access', 0);
			$qb->setColumn('bf_groupid', 'bf_access', $this->fields['bf_level']);
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}
		}
	}

	public function addMember($member) {
		return $this->addMembers(array($member));
	}
	
	public function addMembers($members) {
		if (!is_array($this->members)) {
			$this->loadMembers();
		}

		// Access check
		if ($this->controller->userHasAccessTo('UpdateAccessMember')) {
			if ($this->controller->userHasAccessTo('AddAccessMember')) {
				if (is_array($members)) {
					foreach ($members as $userid) {
						if (!array_key_exists($userid, $this->members) && 
								array_key_exists($userid, $this->nonmembers)) {
							$this->members[$userid] = $this->nonmembers[$userid];
							$this->members_added[] = $userid;
							unset($this->nonmembers[$userid]);
						}
					}
					return true;
				}
			} else {
				throw new BFAccessException('AddAccessMember');
			}
		} else {
			throw new BFAccessException('UpdateAccessMember');
		}

		return false;
	}

	public function deleteMember($member) {
		return $this->removeMembers(array($member));
	}
	
	public function deleteMembers($members) {
		return $this->removeMembers($members);
	}

	public function removeMembers($members) {
		if (!$this->members)  {
			$this->loadMembers();
		}

		// Access check
		if ($this->controller->userHasAccessTo('UpdateAccessMember')) {
			if ($this->controller->userHasAccessTo('DeleteAccessMember')) {
				if (is_array($members)) {
					foreach ($members as $userid) {
						if (array_key_exists($userid, $this->members)) {
							$this->nonmembers[$userid] = $this->members[$userid];
							$this->members_removed[] = $userid;
							unset($this->members[$userid]);
						}
					}
					return true;
				}

				return false;
			} else {
				throw new BFAccessException('DeleteAccessMember');
			}
		} else {
			throw new BFAccessException('UpdateAccessMember');
		}
	}
	
	public function addChildren($children) {
		if (!is_array($this->children)) {
			$this->children = array();
			$this->nonchildren = array();
		}
		
		if (is_array($children)) {
			foreach ($children as $parent) {
				if (!array_key_exists($parent, $this->children) && array_key_exists($parent, $this->nonchildren)) {
					$this->children[$parent] = $this->nonchildren[$parent];
					$this->children_added[] = $parent;
					unset($this->nonchildren[$parent]);
				}
			}
			return true;
		}
		
		return false;
	}
	
	public function deleteChildren($children) {
		return $this->removeChildren($children);
	}
	
	public function removeChildren($children) {
		if (is_array($children)) {
			foreach ($children as $parent) {
				if (array_key_exists($parent, $this->children)) {
					$this->nonchildren[$parent] = $this->children[$parent];
					$this->children_removed[] = $parent;
					unset($this->children[$parent]);
				}
			}
			return true;
		}
		
		return false;
	}
	
	public function getMembers() {
		if (!isset($this->members)) {
			$this->loadMembers();
		}
		
		return $this->members;
	}
	
	public function getNonMembers() {
		if (!isset($this->nonmembers)) {
			$this->loadMembers();
		}
		
		return $this->nonmembers;
	}
	
	public function getChildren() {
		if (!isset($this->children)) {
			$this->loadChildren();
		}
		
		return $this->children;
	}
	
	public function getNonChildren() {
		if (!isset($this->nonchildren)) {
			$this->loadChildren();
		}
		
		return $this->nonchildren;
	}
	
	protected function loadMembers() {
		$this->members = array();
		$this->nonmembers = array();
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_userid', 'bf_users', 'id');
		$qb->addColumn('bf_name', 'bf_users', 'name');
		$qb->addColumn('bf_level', 'bf_access', 'level');
		
		// No reason to even display root as an option when it comes to members
		$qb->where(array('!=', 'bf_users', 'bf_userid', 1));
		
		if (!empty($this->fields['bf_level'])) {
			$qb->join(array('LEFT', 'bf_access', array(
													   array('=', 'bf_users', 'bf_userid', 'bf_access', 'bf_userid'),
													   array('=', 'bf_access', 'bf_level', $this->fields['bf_level'])
													   )));
		} else {
			$qb->join(array('LEFT', 'bf_access', array('=', 'bf_users', 'bf_userid', 'bf_access', 'bf_userid')));
		}
		$qb->orderBy('ASC', 'bf_users', 'bf_name');
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			$this->controller->addError($res->getMessage());
			return false;
		}
		
		while ($user = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			if (!$user['level']) {
				$this->nonmembers[$user['id']] = $user['name'];
			} else {

				$this->members[$user['id']] = $user['name'];
			}
		}
	}
	
	protected function loadChildren() {
		$this->children = array();
		$this->nonchildren = array();
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_level', 'bf_levels', 'id');
		$qb->addColumn('bf_name', 'bf_levels', 'name');
		$qb->addColumn('bf_level', 'bf_access', 'level');
		if (!empty($this->fields['bf_level'])) {
			$qb->join(array('LEFT', 'bf_access', array(
													   array('=', 'bf_levels', 'bf_level', 'bf_access', 'bf_level'),
													   array('=', 'bf_access', 'bf_groupid', $this->fields['bf_level'])
													   )));
		} else {
			$qb->join(array('LEFT', 'bf_access', array('=', 'bf_levels', 'bf_level', 'bf_access', 'bf_level')));
		}
		$qb->where(array('!=', 'bf_levels', 'bf_level', $this->fields['bf_level']));
		$qb->orderBy('ASC', 'bf_levels', 'bf_name');
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			$this->controller->addError($res->getMessage());
			return false;
		}
		
		while ($level = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			if (!$level['level']) {
				$this->nonchildren[$level['id']] = $level['name'];
			} else {

				$this->children[$level['id']] = $level['name'];
			}
		}
	}
	
	protected function loadParents() {
		$this->parents = array();
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_level', 'bf_levels', 'id');
		$qb->addColumn('bf_name', 'bf_levels', 'name');
		if (!empty($this->fields['bf_level'])) {
			$qb->where(array(
							 array('=', 'bf_levels', 'bf_level', 'bf_access', 'bf_groupid'),
							 array('=', 'bf_access', 'bf_level', $this->fields['bf_level']),
							 array('!=', 'bf_access', 'bf_groupid', $this->fields['bf_level'])
							 ));
		} else {
			$qb->where(array('=', 'bf_levels', 'bf_level', 'bf_access', 'bf_level'));
		}
		$qb->orderBy('ASC', 'bf_levels', 'bf_name');
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			$this->controller->addError($res->getMessage());
			return false;
		}
		
		while ($level = $res->fetchRow()) {
			$this->parents[$level->id] = $level->name;
		}
	}
	
	/**
	 * Override to load in the members and children
	 */
	public function select($fields, $values, $comp = '=') {
		if (parent::select($fields, $values, $comp)) {
			$this->loadMembers();
			$this->loadChildren();
			$this->loadParents();
			return true;
		} else {
			return false;
		}
	}
}

/**
 * TODO: do this for real
 */
function bf_rebuild_accesscache() {
	global $controller;
	
	$level_userids = array();
	$groupid_levels = array();
	
	// Clear the current cache
	$del = BFQueryBuilder::delete();
	$del->where(array('=', 'bf_accesscache', 'bf_level', 'bf_accesscache', 'bf_level'));
	$del->getResult();
	
	// Get the mapping of the groups to parents/users
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_level', 'bf_access', 'bf_level');
	$qb->addColumn('bf_userid', 'bf_access', 'bf_userid');
	$qb->addColumn('bf_groupid', 'bf_access', 'bf_groupid');
	$result = $qb->getResult();
	while ($row = $result->fetchRow()) {
		if ($row->bf_userid == '0') {
			if (!array_key_exists($row->bf_groupid, $groupid_levels)) {
				$groupid_levels[$row->bf_groupid] = array($row->bf_level);
			} else {
				$groupid_levels[$row->bf_groupid][] = $row->bf_level;
			}
		} else {
			if (!array_key_exists($row->bf_level, $level_userids)) {
				$level_userids[$row->bf_level] = array($row->bf_userid);
			} else {
				$level_userids[$row->bf_level][] = $row->bf_userid;
			}
		}
	}
	
	$flattened_groupid_levels = array();
	$flattened_level_userids = array();
	
	// Flatten it out
	foreach ($groupid_levels as $level => $children) {
		$flattened_groupid_levels[$level] = array();
		$passed = array();
		bf_flatten_access($level, $groupid_levels, $flattened_groupid_levels[$level], $passed);
	}
	
	// Holy crap, this will take forever!
	foreach ($level_userids as $level => $members) {
		foreach ($members as $member) {
			$insert = BFQueryBuilder::insert();
			$insert->setColumn('bf_level', 'bf_accesscache', $level);
			$insert->setColumn('bf_userid', 'bf_accesscache', $member);
			$insert->setColumn('bf_groupid', 'bf_accesscache', 0);
			$insert->getResult();
			// Now add to each of the parent's parents...
			// Remember that when dealing with group inheritance,
			// the parent/child status of bf_level gets reversed!
			foreach ($flattened_groupid_levels as $inherited_level => $children) {
				if (in_array($level, $children)) {
					$insert = BFQueryBuilder::insert();
					$insert->setColumn('bf_level', 'bf_accesscache', $inherited_level);
					$insert->setColumn('bf_userid', 'bf_accesscache', $member);
					$insert->setColumn('bf_groupid', 'bf_accesscache', 0);
					$insert->getResult();
				}
			}
		}
	}
	// Remember that when dealing with group inheritance,
	// the parent/child status of bf_level gets reversed!
	foreach ($flattened_groupid_levels as $level => $children) {
		foreach ($children as $child) {
			$insert = BFQueryBuilder::insert();
			$insert->setColumn('bf_level', 'bf_accesscache', $child);
			$insert->setColumn('bf_userid', 'bf_accesscache', 0);
			$insert->setColumn('bf_groupid', 'bf_accesscache', $level);
			$insert->getResult();
		}
	}
}

/**
 * Recursion. w00t.
 */
function bf_flatten_access($id, $access, &$flattened, &$passed) {
	if (array_key_exists($id, $access) && !in_array($id, $passed)) {
		foreach ($access[$id] as $child) {
			if (!in_array($child, $flattened)) {
				$flattened[] = $child;
				bf_flatten_access($child, $access, $flattened, $passed);
			}
		}
		$passed[] = $id;
	}
}

/**
 * bf_rebuild_paccesscache()
 * 
 * Rebuild the access cache stored in shmop. Since this gets stored per-user,
 * get all user IDs and loop through them, attempting to kill each cache of permissions.
 * The shmop entry will get rebuilt next time it gets asked for. No sense in loading everything
 * into memory prematurely...
 */
function bf_rebuild_paccesscache() {
	global $controller;
	$querybuilder = BFQueryBuilder::select();
	$querybuilder->addColumn('bf_userid', 'bf_users', 'bf_userid');
	$users = $querybuilder->getCol();
	
	if ($users) {
		foreach ($users as $userid) {
			$controller->delPCache('permissions.'.$userid);
		}
	}
}

?>
