<?php /* (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the copyright and other intellectual property rights in this software.
	Duplication or use of the Software is not permitted except as expressly provided in a written agreement between your company and BuildForge, Inc. */

if (!defined('TABLE_LOCK')) { define('TABLE_LOCK', true); }

$GLOBALS['controller']->require_module('BFStore');

class BFNotes extends BFDBO {
	var $currentNote = '';

	/** Note Type
	 * Defines the note type we are working with for this instance of bfNotes;
	 * Example:
	 *  ProjectStep
	 *  ServerNote
	 *  JobNote
	 *  etc...
	 */	 	 	 	 	 	 	
	var $noteType = '';	

	/** registerNotesList
	 * Registers our notes list with the JTE instance passed
	 * if we have a "current note" then set it up as the single.
	 */
	public function registerNotesList($JTE, $REGNAME, $PID, $ID='') {
		$listing = $this->getNotesList($PID, $ID);
//print_r($listing);die;	
		$JTE->assign($REGNAME, $listing);
		$JTE->assign($REGNAME.'count', count($listing));
		$JTE->assign($REGNAME.'id', $ID);
		$JTE->assign($REGNAME.'pid', $PID);

		if (!empty($this->currentNote)) {
			$note = $this->getNote($this->currentNote);
			$JTE->assign($REGNAME.'single', $note);
		}
	}


	/** getNotesList
	 * Returns a report of notes
	 * PID :: Project or build ID
	 * ID :: Whatever other ID needed, most likely Step ID or 0 for no id
	 */
	public function getNotesList($PID, $ID) {

		// Get users
		$users = bf_get_users();

		// get Notes
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid',		'bf_notes', 'pid');
		$qb->addColumn('bf_id',			'bf_notes', 'id');
		$qb->addColumn('bf_created',	'bf_notes', 'date');
		$qb->addColumn('bf_userid',		'bf_notes', 'bf_user');
		$qb->addColumn('bf_data',		'bf_notes', 'note');
		$qb->addColumn('bf_type',		'bf_notes', 'type');

		$qb->where(
			array(
				array('=',	'bf_notes', 'bf_type',	$this->noteType),
				array('=',	'bf_notes', 'bf_pid',	$PID),
				array('=',	'bf_notes', 'bf_id',	$ID)
				)
			);

		$qb->orderBy( 'DESC', 'bf_created' );
		$res = $qb->getResult();

		if (DB::isError($res)) {
			$this->controller->addError(translate('2219'));
			return false;
		}

		$results = array();
		while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$results[] = array(
				'noteid'	=> $row['note'], // delete note
				'editid'	=> $row['note'], // edit note
				'date'		=> $row['date'],
				'user'		=> ((isset($users[$row['bf_user']]))?$users[$row['bf_user']]:'----'),
				'note'		=> bf_get_storage($row['note']),
				// don't display below
				'id'		=> $row['id'],
				'pid'		=> $row['pid'],
				'userid'	=> $row['bf_user'],
				'type'		=> $row['type'],
			);
		}

		return $results;
	}

	/** getNote
	 * Returns a single note based on note id
	 * REQ :: Form request with note id in it.
	 */
	public function getNote($NID) {

		if (!isset($NID) || empty($NID)) {
			$this->controller->addError(translate('2220'));
			return false;
		}

		// Get users
		$users = bf_get_users();

		// get Notes
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid',		'bf_notes', 'pid');
		$qb->addColumn('bf_id',			'bf_notes', 'id');
		$qb->addColumn('bf_created',	'bf_notes', 'date');
		$qb->addColumn('bf_userid',		'bf_notes', 'bf_user');
		$qb->addColumn('bf_data',		'bf_notes', 'note');
		$qb->addColumn('bf_type',		'bf_notes', 'type');

		$qb->where(
			array(
				array('=',	'bf_notes', 'bf_data',	$NID),
				)
			);

		$res = $qb->getResult();

		if (DB::isError($res)) {
			$this->controller->addError(translate('2221'));
			return false;
		}

		$results = array();
		while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$results[] = array(
				'note_response_noteid'	=> $row['note'], // delete note
				'note_response_date'	=> $row['date'],
				'note_response_user'	=> ((isset($users[$row['bf_user']]))?$users[$row['bf_user']]:'----'),
				'note_response_note'	=> bf_get_storage($row['note']),
				'note_response_id'		=> $row['id'],
				'note_response_pid'		=> $row['pid'],
				'note_response_userid'	=> $row['bf_user'],
				'note_response_type'	=> $row['type'],
			);
		}

		return $results;
	}

	/** saveNote
	 * Returns true or false
	 * REQ :: The request with the various form bits
	 * PID :: Project or build ID
	 * ID :: Whatever ID needed, most likely step ID or 0 for No ID
	 *
	 * Text, User and Date are pulled from $this->request for now.
	 * Form names for now are 'note_text', 'note_user_id', 'note_date'
	 */
	public function saveNote($REQ, $PID, $ID = '') {
		global $user;

		// Pull information from $this->request
		$noteText = $REQ['note_text'];
		$noteUser = $user->get('bf_userid');
		$noteDate = strtotime('now');
		$NID = (array_key_exists('note_id', $REQ))?$REQ['note_id']:'';

		// Store the note text in storage if NID is set this will be an edit, else it will insert a new one
		$notesStorage = new BFStore();
		$storeID = $notesStorage->setStorage($noteText, $NID);

		// Submit to the notes table if a NEW note, else don't
		if ($NID == '') {
		
			$qb = BFQueryBuilder::insert(null, TABLE_LOCK);
			$qb->setColumn('bf_created',	'bf_notes',	 $noteDate);
			$qb->setColumn('bf_userid',		'bf_notes',	 $noteUser);
			$qb->setColumn('bf_type',		'bf_notes',	 $this->noteType);
			$qb->setColumn('bf_pid',		'bf_notes',	 $PID);
			$qb->setColumn('bf_id',			'bf_notes',	 $ID);
			$qb->setColumn('bf_data',		'bf_notes',	 $storeID);

			$res = $qb->getResult();

			if (DB::isError($res)) {
				$this->controller->addError(translate('2222'));
				$notesStorage->removeStorage($storeID);
				return false;
			}
		}
		return true;
	}


	/** saveNote
	 * Returns true or false
	 * REQ :: The request with the various form bits
	 * PID :: Project or build ID
	 * ID :: Whatever ID needed, most likely step ID or 0 for No ID
	 *
	 * note id is pulled from $this->request for now.
	 * $NID of ALL means kill all of them.	 
	 */
	public function deleteNote($REQ, $PID, $ID) {
		if (is_array($REQ)) {
			// Get Note ID
			$NID = (is_array($REQ) && array_key_exists('delete_note_id', $REQ))?$REQ['delete_note_id']:'';
			if (!isset($NID) || empty($NID)) {
				$this->controller->addError(translate('2220'));
				return false;
			}
			// Delete a specific note by NoteID
			$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
			$qb->where(array(
				array('=', 'bf_notes', 'bf_type',	$this->noteType),
				array('=', 'bf_notes', 'bf_pid',	$PID),
				array('=', 'bf_notes', 'bf_id',	  	$ID),
				array('=', 'bf_notes', 'bf_data',	$NID)
			));
			
			$res = $qb->getResult();
			if (DB::isError($res)) {
				$this->controller->addError(translate('2223'));
				return false;
			}
	
			$notesStorage = new BFStore();
			$res = $notesStorage->removeStorage($NID);
			if (! $res) {
				$this->controller->addError(translate('2224', $NID));
				return false;
			}
	
		} else {
			if (strtolower($REQ) == 'all') {	
				// Get all the Note Ids First
				$qb = BFQueryBuilder::select();
				$qb->where(array(
					array('=', 'bf_notes', 'bf_type',	$this->noteType),
					array('=', 'bf_notes', 'bf_pid',	$PID),
					array('=', 'bf_notes', 'bf_id',	  	$ID)
				));		
				$NIDS = $qb->getCol();						
				
				// Delete ALL notes to this parent::child
				$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
				$qb->where(array(
					array('=', 'bf_notes', 'bf_type',	$this->noteType),
					array('=', 'bf_notes', 'bf_pid',	$PID),
					array('=', 'bf_notes', 'bf_id',	  	$ID)
				));		
				
				$res = $qb->getResult();
				if (DB::isError($res)) {
					$this->controller->addError(translate('2223'));
					return false;
				}
				
				foreach($NIDS as $v) {
					$notesStorage = new BFStore();
					$res = $notesStorage->removeStorage($v);
					if (! $res) {
						$this->controller->addError(translate('2224', $v));
					}
				}
			}			
		}

		return true;
	}

}

?>
