<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFPermission
 * 
 * Not particularly useful without the BFPermissionManager, as the manager caches configuration
 * settings as they get used.
 * 
 * @package Console
 * @subpackage Core
 */
class BFPermission extends BFDBO {
	public $table =			'bf_perms';
	public $primary_key =	'bf_tag';
	public $members;
	public $nonmembers;
	protected $stored_members = array();
	protected $members_added = array();
	protected $members_removed = array();

	/**
	 * Override BFDBO::get() in order to return translated
	 * strings for the descriptions
	 */
	public function get($field) {
		if ($field == 'bf_description') {
			return bf_translate_string($this->fields['bf_description']);
		} else {
			return parent::get($field);
		}
	}

	/**
	 * This really only updates the bf_ptable
	 */
	public function update() {
		$changed=false;
		if (count($this->members_removed) > 0) {
			$this->clearMembers();
			$this->members_removed = array();
			$changed=true;
		}
		if (count($this->members_added) > 0) {
			$this->insertMembers();
			$this->members_added = array();
			$changed=true;
		}
		
		if ($changed=true) {
			bf_mark_access_cashe_dirty();
		}
		return true;
	}

	/**
	 * No inserting allowed!
	 */
	public function insert() {
		return false;
	}

	/**
	 * No deleting allowed!
	 */
	public function delete() {
		return false;
	}
	
	protected function clearMembers() {
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						 array('=', 'bf_ptable', 'bf_tag', $this->fields['bf_tag']),
						 array('IN', 'bf_ptable', 'bf_level', $this->members_removed)
						 ));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}
		foreach ($this->members_removed as $level)  {
			$this->controller->logAudit('PermissionRemoved', array(bf_get_level_name($level),
					$this->fields['bf_tag']));
		}
	}
	
	protected function insertMembers() {
		foreach ($this->members_added as $level) {
			$qb = BFQueryBuilder::insert();
			$qb->setColumn('bf_tag', 'bf_ptable', $this->fields['bf_tag']);
			$qb->setColumn('bf_level', 'bf_ptable', $level);
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}

			$this->controller->logAudit('PermissionAdded', array(bf_get_level_name($level), 
					$this->fields['bf_tag']));
		}
	}
	
	public function addMembers($members) {
		// Access check
		if ($this->controller->userHasAccessTo('UpdatePermMember')) {
			if (is_array($members)) {
				foreach ($members as $level) {
					if (!array_key_exists($level, $this->members) && 
							array_key_exists($level, $this->nonmembers)) {
						$this->members[$level] = $this->nonmembers[$level];
						$this->members_added[] = $level;
						unset($this->nonmembers[$level]);
					}
				}
				return true;
			}

			return false;
		} else {
			throw new BFAccessException('UpdatePermMember');
		}
	}

	public function deleteMembers($members) {
		return $this->removeMembers($members);
	}

	public function removeMembers($members) {
		// Access check
		if ($this->controller->userHasAccessTo('UpdatePermMember')) {
			if (is_array($members)) {
				foreach ($members as $level) {
					if (array_key_exists($level, $this->members)) {
						$this->nonmembers[$level] = $this->members[$level];
						$this->members_removed[] = $level;
						unset($this->members[$level]);
					}
				}
				return true;
			}

			return false;
		} else {
			throw new BFAccessException('UpdatePermMember');
		}
	}
	
	public function getMembers() {
		if (!isset($this->members)) {
			$this->loadMembers();
		}
		
		return $this->members;
	}
	
	public function getNonMembers() {
		if (!isset($this->nonmembers)) {
			$this->loadMembers();
		}
		
		return $this->nonmembers;
	}
	
	protected function loadMembers() {
		$this->members = array();
		$this->nonmembers = array();
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_level', 'bf_levels', 'id');
		$qb->addColumn('bf_name', 'bf_levels', 'name');
		$qb->addColumn('bf_tag', 'bf_ptable', 'level');
		if (!empty($this->fields['bf_tag'])) {
			$qb->join(array('LEFT', 'bf_ptable', array(
													   array('=', 'bf_levels', 'bf_level', 'bf_ptable', 'bf_level'),
													   array('=', 'bf_ptable', 'bf_tag', $this->fields['bf_tag'])
													   )));
		} else {
			$qb->join(array('LEFT', 'bf_ptable', array('=', 'bf_levels', 'bf_level', 'bf_ptable', 'bf_level')));
		}
		$qb->orderBy('ASC', 'bf_levels', 'bf_name');
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			$this->controller->addError($res->getMessage());
			return false;
		}
		
		while ($group = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			if ($group['level'] == '') {
				$this->nonmembers[$group['id']] = $group['name'];
			} else {
				$this->members[$group['id']] = $group['name'];
			}
		}
	}

	/**
	 * A method defined only because someone never bothered to setup the schema correctly.
	 * As such, we need to translate the group names from keys
	 */
	protected function fixGroup() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tag', 'bf_perms', 'bf_tag');
		$qb->where(array(
						array('=', 'bf_perms', 'bf_description', ''),
						array('=', 'bf_perms', 'bf_group', $this->fields['bf_group'])
						));
		$this->fields['bf_group'] = $qb->getOne();
	}
	
	/**
	 * Override to load in the members and parents
	 */
	public function select($field, $value, $comp = '=') {
		if (parent::select($field, $value, $comp)) {
			$this->fixGroup();
			$this->loadMembers();
			return true;
		} else {
			return false;
		}
	}
}

?>
