<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFProject
 * A database object providing access to CRUDing the projects stored in the
 * database.
 * @package Console
 * @subpackage Projects
 */

if (!defined('TABLE_LOCK')) { define('TABLE_LOCK', true); }

$GLOBALS['controller']->require_module('BFDBO');
$GLOBALS['controller']->require_module('BFStore');
$GLOBALS['controller']->require_module('BFUID');
$GLOBALS['controller']->require_module('BFNotes');

class BFProject extends BFDBO {

	public $table		= 'bf_projects';
	public $primary_key = 'bf_pid';
	public $descriptor_key = 'bf_description';
	protected $required_fields = array('bf_description');
	protected $field_defaults = array(
									'bf_count' => '0',
									'bf_eid' => '0',
									'bf_tagsync' => '0',
									'bf_pcid' => '0',
									'bf_fcid' => '0',
									'bf_active' => 'Y',
									'bf_increment' => '0',
									'bf_geo_id' => ''
									);
	public $type_name = 'Project';
	public $field_descriptions = array(
			'bf_pid' => '',
			'bf_level' => 'access',
			'bf_description' => 'name',
			'bf_class' => 'class',
			'bf_platform' => 'selector',
			'bf_tag' => 'tag format',
			'bf_increment' => '',
			'bf_count' => '',
			'bf_eid' => 'environment',
			'bf_active' => 'active',
			'bf_snotify' => 'start notify',
			'bf_pnotify' => 'pass notify',
			'bf_fnotify' => 'fail notify',
			'bf_maxthread' => 'max threads',
			'bf_exclusive' => 'run limit',
			'bf_tagsync' => 'tag sync',
			'bf_sticky' => 'sticky',
			'bf_pcid' => 'pass chain',
			'bf_fcid' => 'fail chain',
			'bf_geo_id' => 'geo'
			);

	public function set($field, $value) {
		switch ($field) {
			case 'bf_description':
				// Multple spaces must be one space, and trim string
				$value = preg_replace('/\s+/', ' ', trim($value));
		}
		parent::set($field, $value);
	}

	public function save() {		

		// If the name is already in use, return false
		$qb = BFQueryBuilder::select();		
		$qb->addColumn('bf_pid', 'bf_projects', 'id');
		$qb->addColumn('bf_platform', 'bf_projects', 'selector');
		$qb->where(array(
			array('=', 'bf_projects', 'bf_description', $this->fields['bf_description']),
			array('!=', 'bf_projects', 'bf_pid', $this->fields['bf_pid'])	
		));
		$project = $qb->getRow(DB_FETCHMODE_ASSOC);

		if (!DB::isError($project) && isset($project['id']) && $project['id']) {
		 	$this->controller->addError(translate('2195', $project['selector'] ? translate('project') :
		 			translate('library')));
		 	return false;			
		}
		
		if ($this->fields['bf_tag'] == '') {
			$this->controller->addError(translate(2241));
			return false;
		}
		
		return parent::save();
	}

	/**
	 * Override for permission check
	 */
	public function insert() {
		if ($this->controller->userHasAccessTo('AddProject')) {
			if (parent::insert()) {
				// Insert the new tag record
				$tqb = BFQueryBuilder::insert(null, TABLE_LOCK);
				$tqb->setColumn('bf_pid',		'bf_tags',	 $this->fields['bf_pid']);
				$tqb->setColumn('bf_tid',		'bf_tags',	 '1');
				$tqb->setColumn('bf_name',		'bf_tags',	 'B');
				$tqb->setColumn('bf_value',		'bf_tags',	 '1');
				$tqb->setColumn('bf_parent',	'bf_tags',	 '0');
				$tqb->setColumn('bf_autoinc',	'bf_tags',	 'Y');

				$tagres = $tqb->getResult();

				if (!$tagres || DB::isError($tagres)) { 
					return true;
				}

			}
		} else {
			throw new BFAccessException('AddProject');
		}

		return false;
	}

	/**
	 * Override for permission check
	 */
	public function update() {
		if ($this->controller->userHasAccessTo('UpdateProject')) {
			if (parent::update()) {
				return true;
			}
		} else {
			throw new BFAccessException('UpdateProject');
		}
		return false;
	}

	/**
	 * Overridden in order to copy the project steps.
	 */
	public function copy()  {
		$oldid = $this->fields['bf_pid'];
		parent::copy();

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_uid', 'bf_steps');
		$qb->where(array('=', 'bf_steps', 'bf_pid', $oldid));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$new_pid = $this->get('bf_pid');
			while ($step = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$new_step = new BFStep();
				$new_step->select('bf_uid', $step['bf_uid']);
				$new_step->set('bf_pid', $new_pid);
				$new_step->copy(false);
			}
		}

		return true;
	}

	/** deleteStep($projectID, $stepID)
	 * Deletes $StepID from $ProjectID
	 * Deletes all associated notes
	 * Deletes the command from storage
	 */
	public function deleteStep($PID, $SUID) {
		if (empty($PID) || empty($SUID)) { return false; }
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteProjectStep')) {
			throw new BFAccessException('DeleteProjectStep');
		}
		
		$storage = new BFStore();

		// Get all NOTES and delete them
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_data',   'bf_notes', 'nid');
		$qb->where(array(
			array('=', 'bf_notes', 'bf_id', 	$SUID),
			array('=', 'bf_notes', 'bf_pid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }
		
		// Now delete those buggers
		$this->notes = new BFNotes();
		$this->notes->noteType = 'ProjectStep';
		while($row = $res->fetchRow()) {
			$this->notes->deleteNote(array('delete_note_id'=>$row->nid), $PID, $SUID);
		}

		// Get the Command value
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_command', 'bf_steps', 'cmd');
		$qb->where(array(
			array('=', 'bf_steps', 'bf_uid', $SUID),
			array('=', 'bf_steps', 'bf_pid', $PID)
		));
		$CMD = $qb->getOne();
		// Delete the command from storage
		$storage->removeStorage($CMD);

		// Get the step name (for logging)
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description', 'bf_steps', 'name');
		$qb->where(array(
			array('=', 'bf_steps', 'bf_pid', $PID),
			array('=', 'bf_steps', 'bf_uid', $SUID)
		));
		$stepName = $qb->getOne();
		
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_steps', 'bf_uid',	 $SUID),
			array('=', 'bf_steps', 'bf_pid',	 $PID)
		));
		$res = $qb->getResult();

		if (DB::isError($res)) { return false; }

		//  Log the info.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description', 'bf_projects', 'name');
		$qb->where(array(
			array('=', 'bf_projects', 'bf_pid', $PID)
		));
		$projectName = $qb->getOne();

		$this->controller->logAudit('StepDeleted', array($stepName, $projectName));

		return '1';
	}

	public function deleteProject($PID) {	
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteProject')) {
			throw new BFAccessException('DeleteProject');
		}
		if ($this->referenced())  {
			throw new BFException(translate('2201', translate('project')));
		}

		// For each step in the project
		$qb = BFQueryBuilder::select();
		$qb->where(array(
			array('=', 'bf_steps', 'bf_pid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }
		while ($stepRow = $res->fetchRow()) {
			$UID = $stepRow->bf_uid;
			$this->deleteStep($PID, $UID);
		}

		// Delete the tags associated to this project
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_tags', 'bf_pid',	$PID)
		));		
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }


		// Delete the registers associated with this project
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_data', 'bf_bid',	0),
			array('=', 'bf_data', 'bf_uid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }


		// Delete the notification templates associated with this project
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_templates', 'bf_pid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }


		// Delete the schedules associated with this project
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_cron', 'bf_pid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }


		// Delete the adaptor links associated with this project
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_scopes', 'bf_pid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }


		// Set all steps that refer to the project (inlining, chaining) to refer to no project
		$qb = BFQueryBuilder::update(null, TABLE_LOCK);
		$qb->setColumn('bf_cid', 'bf_steps', 0);
		$qb->where(array(
			array('=', 'bf_steps', 'bf_cid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }

		$qb = BFQueryBuilder::update(null, TABLE_LOCK);
		$qb->setColumn('bf_pcid', 'bf_steps', 0);
		$qb->where(array(
			array('=', 'bf_steps', 'bf_pcid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }

		$qb = BFQueryBuilder::update(null, TABLE_LOCK);
		$qb->setColumn('bf_fcid', 'bf_steps', 0);
		$qb->where(array(
			array('=', 'bf_steps', 'bf_fcid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }


		// Set all projects that refer to the project (tag syncing) to refer to no project
		$qb = BFQueryBuilder::update(null, TABLE_LOCK);
		$qb->setColumn('bf_tagsync', 'bf_projects', 0);
		$qb->where(array(
			array('=', 'bf_projects', 'bf_tagsync',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }


		// Now delete the project
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_projects', 'bf_pid',	$PID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }

		$this->logChange('Deleted');
		
		return true;
	}


	/**
	 * Returns true if a) a class references the project, b) a project references the project, or 
	 * c) a step references the project.
	 * 
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_name', 'bf_classes', 'bf_name', 'COUNT');
		$qb->where(array(
			array('=', 'bf_classes', 'bf_pid', $this->fields[$this->primary_key]),
			'OR',
			array('=', 'bf_classes', 'bf_npid', $this->fields[$this->primary_key]),
			'OR',
			array('=', 'bf_classes', 'bf_xpid', $this->fields[$this->primary_key])
		));
		if (intval($qb->getOne()) > 0)  {
			return true;
		}
		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid', 'bf_projects', 'bf_pid', 'COUNT');
		$qb->where(array(
			'(',
			array('=', 'bf_projects', 'bf_pcid', $this->fields[$this->primary_key]),
			'OR',
			array('=', 'bf_projects', 'bf_fcid', $this->fields[$this->primary_key]),
			')',
			array('!=', 'bf_projects', 'bf_pid', $this->fields[$this->primary_key])			
		));		
		if (intval($qb->getOne()) > 0)  {
			return true;
		}

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_uid', 'bf_steps', 'bf_uid', 'COUNT');
		$qb->where(array(
			'(',
			array('=', 'bf_steps', 'bf_cid', $this->fields[$this->primary_key]),
			'OR',
			array('=', 'bf_steps', 'bf_pcid', $this->fields[$this->primary_key]),
			'OR',
			array('=', 'bf_steps', 'bf_fcid', $this->fields[$this->primary_key]),
			')',
			array('!=', 'bf_steps', 'bf_pid', $this->fields[$this->primary_key])			
		));		
		return intval($qb->getOne()) > 0;
	}


	/** getStepCount($PID)
	 * Returns the number of steps in PID
	 */
	public function getStepCount($PID) {
		if (empty($PID)) { return false; }

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_sid', 'bf_steps', 'max', 'MAX');
		$qb->where(array(
			array('=', 'bf_steps', 'bf_pid',	$PID),
		));
		$lastStep = $qb->getOne();

		return $lastStep;
	}


	/** editTag
	 *  Saves the tag information to the bf_tags database
	 */
	public function editTag($PID, $post) {
		if (empty($PID)) { return false; }
		
		$TID = (array_key_exists('tag_edited', $post))?$post['tag_edited']:'';
		if ($TID == '' && !$this->controller->userHasAccessTo('AddProjectTagVar')) {
			throw new BFAccessException('AddProjectTagVar');
		}
		if ($TID != '' && !$this->controller->userHasAccessTo('EditProjectTagVar')) {
			throw new BFAccessException('EditProjectTagVar');
		}

		// Test value
		if (!is_numeric($post['bf_value'])) {
			$this->controller->addError(translate(2245));
			return false;
		}

		// Test Name
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_name', 'bf_tags');
		$qb->where(array(
			array('=', 'bf_tags',  'bf_pid', $PID)
		));
		$tagNames = $qb->getCol();

		// Check for BF_ tag names
		if (preg_match('/^BF_/',$post['bf_name'])) {
			$this->controller->addError(translate(2116));
			return false;
		}
		
		// Check for unique tag name on a per project basis
		if (in_array($post['bf_name'], $tagNames) && $TID == '') {
			$this->controller->addError(translate(2246));
			return false;
		}
		
		// Check for the inclusion of a colon
		if (strpos($post['bf_name'], ':') !== false) {
			$this->controller->addError(translate(2247));
			return false;
		}		

		// Creating a new tag
		if ($TID == '') {
			// One Up for the TID
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tid',	'bf_tags',  'tid', 'MAX');
			$qb->where(array(
				array('=', 'bf_tags',  'bf_pid', $PID)
			));
	
			
			$TID = $qb->getOne();

			$qb = BFQueryBuilder::insert(null, TABLE_LOCK);
			$qb->setColumn('bf_pid',	'bf_tags',	 $PID);
			$qb->setColumn('bf_tid',	'bf_tags',	 ++$TID);

			foreach($post as $k=>$v) {
				if ($k != 'tag_edited') {
					$qb->setColumn($k,	'bf_tags', $v);
				}
			}
			
			$type = 'TagCreated';
		}
		// Editing a tag
		else {
			$qb = BFQueryBuilder::update(null, TABLE_LOCK);
			foreach($post as $k=>$v) {
				if ($k != 'tag_edited') {
					$qb->setColumn($k,	'bf_tags', $v);
				}
			}
			$qb->where(array(
				array('=', 'bf_tags', 'bf_pid', $PID),
				array('=', 'bf_tags', 'bf_tid', $TID)
			));

			$type = 'TagUpdated';
		}

		// if they attempt to create a tag that is BF_ something.. denied.
		if (preg_match("/^BF_/i", $post['bf_name'], $matches) == false) {
		
			$res = $qb->getResult();
			if (DB::isError($res))  {
				return false;
			}

			//  Log the info.
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_description', 'bf_projects', 'name');
			$qb->where(array(
				array('=', 'bf_projects', 'bf_pid', $PID)
			));
			$projectName = $qb->getOne();
			
			$this->controller->logAudit($type, array($post['bf_name'], $projectName));

			return true;
		}
		return false;
	}

	/** Delete Tag
	 *  Deletes a tag from a project
	 */
	public function deleteTag($PID, $post) {
		if (empty($PID)) { return false; }
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteProjectTagVar')) {
			throw new BFAccessException('DeleteProjectTagVar');
		}

		//  Get tag name (for logging).
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_name', 'bf_tags', 'name');
		$qb->where(array(
			array('=', 'bf_tags', 'bf_tid', $post['tid']),
			array('=', 'bf_tags', 'bf_pid', $PID)
		));
		$tagName = $qb->getOne();
		
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(
			array(
				array('=', 'bf_tags', 'bf_tid', $post['tid']),
				array('=', 'bf_tags', 'bf_pid', $PID)
				)
			);

		$res = $qb->getResult();

		if (DB::isError($res))  {
			return false;
		}

		//  Log the info.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description', 'bf_projects', 'name');
		$qb->where(array(
			array('=', 'bf_projects', 'bf_pid', $PID)
		));
		$projectName = $qb->getOne();

		$this->controller->logAudit('TagDeleted', array($tagName, $projectName));

		return true;
	}

	/** hasBuilds()
	 * Returns boolean if there are builds for PID
	 */
	public function hasBuilds() {
		if (!isset($this->request['bf_id'])) { return false; }

		$PID = $this->request['bf_id'];

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid', 'bf_builds', 'builds', 'COUNT');
		$qb->where(array(
			array('=', 'bf_builds', 'bf_pid',	$PID),
		));
		$count = (int)$qb->getOne();

		return ($count > 0);
	}

	/** hasSteps()
	 * Returns an array of each PID with a boolean of whether or not it has any steps
	 */
	public function hasSteps() {
		if (empty($this->fields['bf_pid'])) {
			$results = array();
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_pid', 'bf_projects', 'pid');
			$PIDS = $qb->getCol();
		} else {
			$PIDS = array($this->fields['bf_pid']);
		}

		if (!empty($PIDS)) {
			foreach($PIDS as $pid) {
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_uid', 'bf_steps', 'step', 'COUNT');
				$qb->where(array(
					array('=', 'bf_steps', 'bf_pid', $pid)
				));
				$results[$pid] = ((int)$qb->getOne() > 0);
			}
		}
		return $results;
	}


	/** validateSelectors
	 *  Steps though the projects and makes sure the selector they have
	 * set is actually a valid selector, else it clears it --
	 * which means it would become a library at that point.
	 */
	public function validateSelectors() {
		// get list of selectors
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_selector_id', 'bf_selector', 'name', 'DISTINCT');
		$validSelectors = $qb->getCol();

		if (is_array($validSelectors)) {
			// get list of projects
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_platform', 'bf_projects',  'Selector', 'DISTINCT');
			$qb->where(array(array(array('!=', 'bf_projects',  'bf_platform', ''))));
			$projectSelectors = $qb->getCol();

			if(is_array($projectSelectors)) {
				foreach($projectSelectors as $selector) {
					if (! in_array($selector, $validSelectors)) {
						// These selectors aren't valid -- replace them
						$qb = BFQueryBuilder::update(null, TABLE_LOCK);
						$qb->setColumn('bf_platform', 'bf_projects', '');
						$qb->where(
							array(
								array('=', 'bf_projects', 'bf_platform', $selector)
								)
							);
						$res = $qb->getResult();
					}
				}
			}
		}
	}

	/** editRegister
	 *  Saves to the datbase register information using the name as key
	 */
	public function editRegister($PID, $post) {
		if (empty($PID)) { return false; }

		$regEdited = (array_key_exists('reg_edited', $post))?$post['reg_edited']:'';
		if (empty($regEdited) && !$this->controller->userHasAccessTo('AddProjectReg')) {
			throw new BFAccessException('AddProjectReg');
		}
		if (!empty($regEdited) && !$this->controller->userHasAccessTo('EditProjectReg')) {
			throw new BFAccessException('EditProjectReg');
		}

		$regName = $post['bf_name'];
		$regCont = $post['bf_value'];

		// Test Name
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_register', 'bf_data');
		$qb->where(array(
			array('=', 'bf_data',  'bf_uid', $PID)
		));
		$regNames = $qb->getCol();

		if (in_array(strtoupper($regName), $regNames) && empty($regEdited)) {
			$this->controller->addError(translate(2281));
			return false;
		}

		$storage = new BFStore();
		$storeID = $storage->setStorage($regCont, $regEdited);

		if (empty($regEdited)) {
				$qb = BFQueryBuilder::insert(null, TABLE_LOCK);
				$qb->setColumn('bf_bid',		'bf_data',	'0');
				$qb->setColumn('bf_uid',		'bf_data',	$PID);
				$qb->setColumn('bf_register',	'bf_data',	strtoupper($regName));
				$qb->setColumn('bf_stamp',		'bf_data',	date('U',time()));
				$qb->setColumn('bf_data',		'bf_data',	$storeID);
				$type = 'RegCreated';
		} else {
				$qb = BFQueryBuilder::update(null, TABLE_LOCK);
				$qb->setColumn('bf_register',	'bf_data',	strtoupper($regName));
				$qb->setColumn('bf_stamp',		'bf_data',	date('U',time()));
				$qb->where(array(
					array('=', 'bf_data', 'bf_data', $regEdited)
				));
				$type = 'RegUpdated';
		}
		$res = $qb->getResult();
		if (DB::isError($res)) {
			$this->controller->addError(translate(2280));
			return false;
		}

		//  Log the info.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description', 'bf_projects', 'name');
		$qb->where(array(
			array('=', 'bf_projects', 'bf_pid', $PID)
		));
		$projectName = $qb->getOne();

		$this->controller->logAudit($type, array(strtoupper($regName), $projectName));

		return true;
	}

	/** deleteRegister
	 *	Deletes a register with UID of $ and removes the store
	 */
	public function deleteRegister($PID, $post) {
		if (empty($post) || !isset($post['RID'])) { return false; }
		
		if (!$GLOBALS['controller']->userHasAccessTo('DelProjectReg')) {
			throw new BFAccessException('DelProjectReg');
		}
		
		$dataID = $post['RID'];

		//  Get the register name (for logging).
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_register', 'bf_data', 'name');
		$qb->where(array(
			array('=', 'bf_data', 'bf_data', $dataID)
		));
		$regName = $qb->getOne();
		
		// Delete from bf_data
		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_data',	'bf_data',  $dataID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			$this->controller->addError("Error: Nothing to delete?");
			return false;
		}
		// Delete from storage
		$storage = new BFStore();
		$storage->removeStorage($dataID);

		//  Log the info.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description', 'bf_projects', 'name');
		$qb->where(array(
			array('=', 'bf_projects', 'bf_pid', $PID)
		));
		$projectName = $qb->getOne();

		// Audit trace
		$this->controller->logAudit('RegDeleted', array($regName, $projectName));
		
		return true;
	}

}

?>
