<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/


 /**
  * BFQueryCompiler_db2 a database specific extension of BFQueryCompiler
  *
  * @package Console
  * @subpackage Core
  */

// Set TRUE to have files written to temp directory for each query type.
define('DATABASE_DEBUG', false);

$GLOBALS['controller']->require_file('includes/BFColumnTypeCache.php');

class BFQueryCompiler_sybase extends BFQueryCompiler {
	protected $quotedTypes;

	/**
	 * compile
	 *
	 * The compile functionality is database type specific because SQL isn't
	 * standardized between the different database types.
	 * As such, this function will be used to actualy build the SQL.
	 */
	public function compile() {
		switch ($this->querytype) {
			case BFQUERYBUILDER_INSERT:
				return $this->compileInsert();
			case BFQUERYBUILDER_SELECT:
				return $this->compileSelect();
			case BFQUERYBUILDER_UPDATE:
				return $this->compileUpdate();
			case BFQUERYBUILDER_DELETE:
				return $this->compileDelete();
		}
	}

	/**
	 * Compile INSERT queries
	 *
	 * You'd better do this from a database object (since it uses all
	 * fields) or know exactly what the query will do. I don't care
	 * if fields can accept a null value from not getting set, but
	 * the database may or may not throw up on you.
	 */
	protected function compileInsert() {
		$QUERY =	'INSERT INTO %s (%s) VALUES (%s)';
		$TABLE =	null;
		$FIELDS =   '';
		$VALUES =   '';

		// Build fields to insert to
		if (!empty($this->presql['setfields'])) {
			foreach($this->presql['setfields'] as $k=>$v) {

				if (is_null($TABLE)) {
					$TABLE = $v[0];
				}
				$FIELDS .= (($k=='0') ? ' ' : ', ') . $v[1];

				$columnType = $this->columnInfo->getColumnType($v[0], $v[1]);
				$VALUES .= (($k=='0') ? ' ' : ', ')
								. $this->db->quoteSmart($v[2], $columnType);
			}

			// Decide whether we need to insert into one table or multiple tables
			if (count($FIELDS) == 1) {
				$this->sql = sprintf($QUERY, $TABLE, $FIELDS, $VALUES);
			} else {
				// TODO
			}

			if (DATABASE_DEBUG) {
				$out = $this->sql . "\n";
				file_put_contents("c:/temp/sybase_insert.log", $out, FILE_APPEND);
			}

			return true;
		}
		// Not about to insert only blank fields
		return false;
	}

	/**
	 * Compile SELECT queries
	 */
	protected function compileSelect() {
		// Filter out the tables used in JOINs. We need to do this first,
		// since it can't exist in two places...
		if (array_key_exists('join', $this->presql)) {
			$this->filterTables();
		}

		$SELECT = '';
		// Build fields to select from
		if (!empty($this->presql['fields'])) {
			foreach($this->presql['fields'] as $k=>$v) {
				$prepender = ''; $appender = '';
				if (!empty($v[3])) {
					// Handle different options if there are any, otherwise drive on!
					switch($v[3]){
						case('MAX'): // Get the macimum value from the selected results
							$prepender = 'MAX(';
							$appender = ')';
							break;
						case('SUBSTRING'): // Selects a specified chunk of string
							$prepender = 'SUBSTRING(';
							$appender = ','.$this->db->quoteSmart($v[4][0]) . ((isset($v[4][1]))?','.$this->db->quoteSmart($v[4][1]):'') . ')';
							break;
						case('COUNT'): // The total number of a certain field within results
							$prepender = 'COUNT(';
							$appender = ')';
							break;
						case('DISTINCT'): // The total number of a certain field within results
							$prepender = 'DISTINCT ';
							$appender = '';
							break;
						default:
							$prepender = '';
							$appender = '';
							break;
					}
				}

				if (!empty($v[1])) {
					$SELECT .= (($k=='0')?' ':', ').$prepender
												.$v[0].'.'.$v[1].$appender
												.((!empty($v[2]))?' AS '."'".$v[2]."'":'');
				}
			}
		} else {
			$SELECT .= ' * ';
		}

		// Build from
		$FROM = ' FROM ';
		foreach($this->presql['tables'] as $k=>$v) {
			if (array_key_exists('join', $this->presql) && is_array($this->presql['join'])) {
				if (in_array($v, $this->presql['join'])) {
					continue;
				}
			}
			$FROM .= (($k=='0')?' ':', ').$v;
		}
		$FROM .= ' ';

		// Build joins
		$JOIN = '';
		if (array_key_exists('join', $this->presql) && is_array($this->presql['join'])) {
			$JOIN = $this->parseJoins($this->presql['join']);
		}

		// Build where
		$WHERE = "";
		if (array_key_exists('where', $this->presql) && is_array($this->presql['where'])) {
			foreach ($this->presql['where'] as $where_seg) {
				 if ($WHERE == '') {
					$WHERE  = ' WHERE ';
				} else {
					$WHERE .= ' AND ';
				}
				$WHERE .= $this->parseWhere($where_seg);
			}
		}

		// Build filter (second part to where)
		$FILTER = (!empty($this->presql['filter'])) ? 
				(!$WHERE ? ' WHERE ' : ' AND ') . $this->parseWhere($this->presql['filter']) : '';

		// Group by
		$GROUP = '';
		if (array_key_exists('groupby', $this->presql) && is_array($this->presql['groupby'])) {
			$GROUP  = ' GROUP BY ';
			$GROUP .= $this->presql['groupby'][0].
					  ((isset($this->presql['groupby'][1]))?'.'.$this->presql['groupby'][1]:'');
		}

		// Order by
		$ORDER = '';
		if (array_key_exists('orderby', $this->presql) && is_array($this->presql['orderby'])) {
			$ORDER  = ' ORDER BY ';
			foreach($this->presql['orderby'] as $k=>$v) {
				if ($k > 0) {
					$ORDER .= ',';
				}
				$ORDER .= $v[1].
					  ((isset($v[2]))?'.'.$v[2]:'').' '.
					  ((strtoupper($v[0])=='DESC')?'DESC':'ASC');
			}
		}

		// Sybase, in its infinite wisdom, does not have any way of performing pagination in one
		// query.  Ergo, we will construct the queries here and the DB object will execute them in
		// sequence.
		$this->sql	   = 'SELECT '.$SELECT.$FROM.$JOIN.$WHERE.$FILTER.$GROUP.$ORDER;
		if (isset($this->presql['limit']) && is_array($this->presql['limit']) && 
				($this->presql['limit']['count'] || $this->presql['limit']['offset']))  {
			// Hack it up to get the DB object to do all the dirty work.
    		$this->parameters['rowcount'] = $this->presql['limit']['count'] + 
    				$this->presql['limit']['offset'];

        	if ($this->presql['limit']['offset'])  {
				$s = explode(" ", microtime());
				$m = explode(".", $s[0]);
        		$temptable = '#bf_' . (substr($s[1], 4, 4).substr($m[1], 0, 4));
				$this->sql = 'SELECT ' . $SELECT . ', __temp_key = identity(10) INTO ' . 
						$temptable . $FROM . $JOIN . $WHERE . $FILTER . $GROUP . $ORDER;
				$this->parameters['select2'] = 'SELECT * FROM ' . $temptable . ' WHERE __temp_key between ' .
						$this->presql['limit']['offset'] . ' AND ' . ($this->parameters['rowcount'] - 1) . 
						' ORDER BY __temp_key';
				$this->parameters['drop'] = 'DROP TABLE ' . $temptable;
        	}
		}


		// Removed Distinct for Count
		$COUNTBY = (!empty($this->presql['countby']))?'DISTINCT '
			.$this->presql['countby'][0].".".$this->presql['countby'][1]:"*";

		$this->sql_count = 'SELECT count( '.$COUNTBY.' )'.$FROM.$JOIN.$WHERE.$FILTER.$GROUP;

		if (DATABASE_DEBUG) {
			$out = $this->sql . "\n";
			file_put_contents("c:/temp/sybase_select.log", $out, FILE_APPEND);
		}

		return true;
	}

	/**
	 * Compile UPDATE queries
	 */
	protected function compileUpdate() {
		// Build table list
		$TABLES = " ";
		foreach($this->presql['tables'] as $k=>$v) {
			$TABLES .= (($k=='0')?' ':', ').$v;
		}


		$UPDATE = '';
		// Build fields to update
		if (!empty($this->presql['setfields'])) {
			foreach($this->presql['setfields'] as $k=>$v) {
				$escaped;
				// Get Column type.
				$columnType = $this->columnInfo->getColumnType($v[0], $v[1]);
				if (is_null($v[2])) {
					$escaped = 'NULL';
				} else {
					$escaped = $this->db->quoteSmart($v[2], $columnType);
				}
				$UPDATE .= (($k=='0')?' ':', ')
											.$v[0] . '.'
											.$v[1] . ' = '
											.$escaped;
			}
		} else {
			return false;
		}

		// Build where
		$WHERE = '';
		if (array_key_exists('where', $this->presql) && is_array($this->presql['where'])) {
			foreach ($this->presql['where'] as $where_seg) {
				 if ($WHERE == '') {
					$WHERE  = ' WHERE ';
				} else {
					$WHERE .= ' AND ';
				}
				$WHERE .= $this->parseWhere($where_seg);
			}
		}

		// Build filter (second part to where) - why not? It could happen.
		$FILTER = (!empty($this->presql['filter']))?' AND '.$this->parseWhere($this->presql['filter']):'';

		$this->sql	   = 'UPDATE ' . $TABLES . ' SET ' . $UPDATE . $WHERE . $FILTER;

		if (DATABASE_DEBUG) {
			$out = $this->sql . "\n";
			file_put_contents("c:/temp/sybase_update.log", $out, FILE_APPEND);
		}

		return true;
	}

	/**
	 * Compile DELETE queries
	 *
	 * DON'T FORGET YOUR WHERE CLAUSE OR WE'LL ALL DIE HORRIBLY
	 */
	protected function compileDelete() {
		// Build table list
		$TABLES = " ";
		foreach($this->presql['tables'] as $k=>$v) {
			$TABLES .= (($k=='0')?' ':', ').$v;
		}
		$TABLES .= " ";

		// Build where
		$WHERE = "";
		if (array_key_exists('where', $this->presql) && is_array($this->presql['where'])) {
			foreach ($this->presql['where'] as $where_seg) {
				 if ($WHERE == "") {
					$WHERE  = " WHERE ";
				} else {
					$WHERE .= " AND ";
				}
				$WHERE .= $this->parseWhere($where_seg);
			}
		}

		$this->sql	   = 'DELETE FROM ' . $TABLES . $WHERE;

		if (DATABASE_DEBUG) {
			$out = $this->sql . "\n";
			file_put_contents("c:/temp/sybase_delete.log", $out, FILE_APPEND);
		}

		return true;
	}

	protected function filterTables() {
		foreach ($this->presql['join'] as $join_arr) {
			if (is_array($join_arr) && is_string($join_arr[1]) && $k = array_search($join_arr[1], $this->presql['tables'])) {
				unset($this->presql['tables'][$k]);
			}
		}
	}

	/**
	 * May get split up by types of joins later on. Or not.
	 */
	protected function parseJoins($joins) {
		$JOIN = '';
		foreach ($this->presql['join'] as $join_arr) {
			// Joins need a type, a table and a where clause
			if (is_array($join_arr)
					&& is_string($join_arr[0])
					&& is_string($join_arr[1])
					&& is_array($join_arr[2])) {
				$temp_join = '';
				$type = strtoupper($join_arr[0]);
				if ($type == 'STRAIGHT') {
					$temp_join  .= ' JOIN ' . $join_arr[1];
				} else if ($type == 'INNER') {
					$temp_join  .= ' INNER JOIN ' . $join_arr[1];
				} else if ($type == 'LEFT') {
					$temp_join  .= ' LEFT JOIN ' . $join_arr[1];
				} else {
					continue;
				}

				$temp_join .= ' ON ' . $this->parseWhere($join_arr[2]);
				$JOIN .= $temp_join;
			}
		}

		return $JOIN;
	}

	// Does the job of recursively parsing the WHERE clause
	// ALSO allows yo uto use in conditionals in the select portion of the SQL statement.. *gleee*
	private function parseWhere($whereArray) {
		$sigils = array ('=','!=','<>','<','>','<=','>=','IN', 'LIKE', 'LIKE BINARY', 'IS NULL', 'IS NOT NULL');

		$results = '';
		foreach($whereArray as $k=>$v) {
			if (is_array($v)) {
				if (in_array($v[0], $sigils)) {
					if ($v[0] == 'LIKE BINARY') {
						$v[0] = 'LIKE';
					}

					// WHERE foo in ()  fails badly - so stop it here
					if ($v[0] == 'IN' && empty($v[3])) {
						$v[0] = "=";
						$v[3] = "";
					}

					//if (strtoupper($v[0]) != 'IN') {
						// If this isn't the first item, and the next item after this one is an array, or if the next one isn't set
						// or if the previous one wasn't an array and is set
						if ($k!='0' && !in_array(trim(substr($results, -3, 3)),array('AND','OR','('))) {
							$results .= ' AND';
						}

						// If the query is a LIKE query add % around the value
						$wrap = (strtoupper($v[0] == 'LIKE'))?'%':'';

						// Handle the query based on the number of elements in the array
						switch (count($v)) {
							case(2):
								if ($v[0] == 'IS NULL' || $v[0] == 'IS NOT NULL') {
									$results .= ' '.$v[1].' '.$v[0];
								}
								break;
							case(3):
								if ($v[0] == 'IS NULL' || $v[0] == 'IS NOT NULL') {
									$results .= ' '.$v[1] .'.'.
												$v[2].' '.$v[0];
								} else {
									$results .= ' '.$v[1] .' '.$v[0].' '.$this->db->quoteSmart($wrap.$v[2].$wrap);
								}
								break;
							case(4):
								if ($v[0] == 'IN') {
									$columnType = $this->columnInfo->getColumnType($v[1], $v[2]);
									if (!is_array($v[3])) {
										$v[3] = array($v[3]);
									}
									foreach ($v[3] as $tk => $tv) {
										$v[3][$tk] = $this->db->quoteSmart($tv,$columnType);
									}
									$results .= ' '.$v[1] .'.'.
									$v[2] .' '.$v[0].' ('.implode(',', $v[3]).')';
								} else {
									$isString = false;
									$columnType = $this->columnInfo->getColumnType($v[1], $v[2]);
									if (in_array(strtolower($columnType), $this->quotedTypes)) {
										$isString = true;
									}
									if (is_null($v[3])) {
										$v[3] = ($isString) ? $this->db->quoteSmart($wrap . $wrap) : 0;
									} else {
										$v[3] = ($isString) ? $this->db->quoteSmart($wrap.$v[3].$wrap) : $v[3];
									}
									$results .= ' '.$v[1] .'.'.
													$v[2] .' '.$v[0].' '.$v[3];
								}
								break;
							case(5):
								$results .= ' '.$v[1] .'.'.
												$v[2] .' '.$v[0].' '.
												$v[3] .'.'.
												$v[4];
								break;
						}
					//}


				} else {
					$results .= ' ( ' . $this->parseWhere($v) . ' ) ';
				}
			} else {
				if (in_array($v,$sigils)) {
					return $this->parseWhere(array($whereArray));
				} else {
					$results .= ' '.$v.' ';
				}
			}
		}

		// Can't have ORs running amok
		return '('.$results.')';

		// Amok amok amok!!
	}

	/** dbLock
	 *  Attempts to lock all of the tables we are working with in bf_sysconfig
	 * if it can't lock one it will push back onto stack and wait a second and try again
	 * if it can't lock one after ten retries, it will return unlocked
	 * after 8 tries it will try to force the issue.
	 */
	public function dbLock() {
		// Get list of tables to lock - in the event there are more than one..
		$tableStack = $this->presql['tables'];
		if (!is_array($tableStack)) { return false; }

		$retries = 0;
		while(!empty($tableStack) && $retries < 10) {
			$tableName = array_pop($tableStack);
			if (!empty($tableName)) {
				if ($this->getLock($tableName) == TABLE_UNLOCKED || $retries > 5) {
					$lock = $this->setLock($tableName);
					if (DB::isError($lock) || $lock == TABLE_UNLOCKED) {
						$retries++;
						array_push($tableStack, $tableName);
						// ERROR: wasn't able to lock the table.. Why fer?
						// Push back onto stack.. let it try some more.. but at 10 tries.. FAIL
					}						
				} else {
					// Table is locked currently, push back on stack
					$retries++;
					array_push($tableStack, $tableName);
					sleep(1);
				}
			}
		}

		return (empty($tableStack))?TABLE_LOCKED:TABLE_UNLOCKED;
	}

	public function dbUnlock() {
		// Get list of tables to lock - in the event there are more than one..
		$tableStack = $this->presql['tables'];
		if (!is_array($tableStack)) { return false; }

		foreach ($tableStack as $tableName) {
			$res = $this->db->query('UPDATE bf_sysconfig SET bf_value=\'0\' WHERE bf_param=\'lock_' . $tableName . '\'');
			if (DB::isError($res)) { return false; }
		}
		return true;
	}

	protected function setLock($tableName) {
		if ($this->db->getOne('SELECT count(*) FROM bf_sysconfig WHERE bf_param=\'lock_' . $tableName . '\'') == '0') {
			$res = $this->db->query('INSERT INTO bf_sysconfig VALUES(\'N\',\'lock_' . $tableName . '\',\'1\',\'0\',\'\',\'\',\'\',\'\',\'\')');
			return (! DB::isError($res))?TABLE_LOCKED:TABLE_UNLOCKED;
		} else {
			$res = $this->db->query('UPDATE bf_sysconfig SET bf_value=\'1\' WHERE bf_param=\'lock_' . $tableName . '\'');
			return (! DB::isError($res))?TABLE_LOCKED:TABLE_UNLOCKED;
		}
	}

	protected function getLock($tableName) {
		return $this->db->getOne('SELECT count(*) FROM bf_sysconfig WHERE bf_param=\'lock_' . $tableName . '\' AND bf_value=\'1\'');
	}

	function __construct($db, $type = BFQUERYBUILDER_SELECT, $tableLock = TABLE_UNLOCKED) {
		parent::__construct($db, $type, $tableLock);
		$this->quotedTypes = array ('char', 'varchar', 'string', 'text');
	}

}

?>
