<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFScope.php
 * 
 * @package Console
 * @subpackage AdaptorLinks
 */

/**
 * BFScope
 */
class BFScope extends BFDBO {
	public $table = 'bf_scopes';
	public $primary_key = 'bf_sid';
	public $descriptor_key = 'bf_link';
	public $field_defaults = array(
									'bf_sid' => null,
									'bf_pid' => 0,
									'bf_active' => 'Y',
									'bf_stamp' => 0,
									'bf_status' => 'Idle',
									'bf_jobid' => 0,
									'bf_link' => '',
									'bf_name' => '',
									'bf_eid' => 0
									);
	public $type_name = 'adaptor link';
	public $log_changes = false;  // We'll log our own changes, thanks.
	public $field_descriptions = array(
			'bf_sid' => '',
			'bf_pid' => 'project',
			'bf_active' => 'state',
			'bf_stamp' => '',
			'bf_status' => '',
			'bf_jobid' => '',
			'bf_link' => 'adaptor',
			'bf_name' => '',
			'bf_eid' => 'environment'
			);
		
	/**
	 * If we have an environment to work with, put the vars into the env.
	 * This actually takes a bf_store-d string, splits on the first '=' it finds and assigns
	 * the trimmed left value as the param and the trimmed right value as the value.
	 */
	public function populateEnv() {
		if ($this->fields['bf_eid'] != 0) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_text', 'bf_interfacetemplates', 'bf_text', 'DISTINCT');
			$qb->join(
				array('LEFT', 'bf_interfaces',
						array(
							array('=', 'bf_interfacetemplates', 'bf_name', 'bf_interfaces', 'bf_template')
						)
					)
				);
			$qb->where(array(
					'(',
					array('=', 'bf_interfaces', 'bf_link', $this->fields['bf_link']),
					'OR',
					array('=', 'bf_interfacetemplates', 'bf_name', $this->fields['bf_link']),
					')',
					array('=', 'bf_interfacetemplates', 'bf_type', 'env')
					));
			$envs = $qb->getCol();

			if (DB::isError($envs)) {
				throw new BFDBException($envs->getMessage);
			} else {
				// If an actual environment...
				$this->controller->require_module('BFEnv');
//				$temp_env = new BFEnv();
//				$temp_env->select(array($temp_env->primary_key, 'bf_eid'), array(1, 0));
//				$temp_level = $temp_env->get('bf_level');

				foreach ($envs as $pairstring) {
					$string = bf_get_storage($pairstring); // bf_store ID to stored string
					$pair = explode('=', $string, 2); // 'foo = bar' to array('foo ', ' bar')
					// If no param/value combination, nothing to set
					if (count($pair) < 2) {
						continue;
					}
					$temp_env = new BFEnv();
					$temp_env->set($temp_env->primary_key, $this->fields['bf_eid']);
					$temp_env->set('bf_eid', null);
					$temp_env->set('bf_level', '0');
					$temp_env->set('bf_param', trim($pair[0]));
					$temp_env->set('bf_value', trim($pair[1]));
					$temp_env->set('bf_action', 'S');
					$temp_env->set('bf_mode', 'N');
					// Actually save the new variable.
					$temp_env->save();
				}
			}
		}
	}
	
	public function save()  {
		// Get the number of adaptor links with the current project
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_sid', 'bf_scopes', 'count', 'COUNT');
		$qb->where(array(
				array('=', 'bf_scopes', 'bf_pid', $this->fields['bf_pid']),
				array('!=', 'bf_scopes', 'bf_sid', $this->fields['bf_sid']),
				));
		$count = $qb->getOne();

		if (DB::isError($count)) {
			throw new BFDBException($count->getMessage());
		} else if (intval($count) > 0) {
			throw new BFException(translate('2251', translate('server')));
		}
		parent::save();
	}
	
	public function insert() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('AddLink')) {
			throw new BFAccessException('AddLink');
		}
		
		// Inserting this object
		if (parent::insert())  {
			return true;
		}
		return false;
	}
	
	
	/**
	 * Overridden for access check
	 */
	public function delete() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteLink')) {
			throw new BFAccessException('DeleteLink');
		}
		
		return parent::delete();
	}
	
	public function logChange($type)  {
		if ($type != 'Updated' || $this->controller->getSysconfig('log_all_changes') != 'Y')  {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_description', 'bf_projects', 'name');
			$qb->where(array(
							array('=', 'bf_projects', 'bf_pid', $this->fields['bf_pid'])
						));
			$projectName = $qb->getOne();
			$this->controller->logAudit('Link' . $type, array($this->fields['bf_link'], $projectName));
		}
	}

	/**
	 * A BFDBO-centered logging function used to log before-and-after state on changed fields.  
	 */
	public function logChangedFields($field, $before, $after)  {
		$fieldname = isset($this->field_descriptions[$field]) ? 
				($this->field_descriptions[$field] ? translate($this->field_descriptions[$field]) : '') : 
				$field;
		if ($this->inserted && $fieldname && $this->controller->getSysconfig('log_all_changes') == 'Y')  {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_description', 'bf_projects', 'name');
			$qb->where(array(
							array('=', 'bf_projects', 'bf_pid', $this->fields['bf_pid'])
						));
			$projectName = $qb->getOne();
			$this->controller->logAudit('LinkUpdatedField', array($fieldname,
					$this->fields['bf_link'], $projectName, $before, $after));
		}
	}
}

?>
