<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFSelector
 * A database object providing access to CRUDing the selectors stored in the
 * database.
 * @package Console
 * @subpackage Selectors
 */


$GLOBALS['controller']->require_module('BFDBO');

class BFSelector extends BFDBO {
	public $table		   = 'bf_selector';
	public $primary_key	 = 'bf_selector_id';
	public $descriptor_key = 'bf_selector_id';
	public $required_fields = array('bf_selector_id');
	public $stored_values = array('bf_selector_id' => null, 'bf_prop' => null, 'bf_operator' => null, 'bf_value' => null, 'bf_level' => null);
	protected $field_defaults = array('bf_selector_id' => '', 'bf_prop' => '', 'bf_operator' => '', 'bf_value' => '', 'bf_level' => 0);
	public $field_descriptions = array(
			'bf_selector_id' => 'name',
			'bf_level' => 'access',
			'bf_prop' => 'property',
			'bf_value' => 'value',
			'bf_operator' => 'operator',
			'bf_required' => 'required'
			);

	/**
	 * Whee...validation.
	 */
	public function save() {
		global $_REQUEST;
		// Test for duplicate selector names
		if ($this->fields['bf_selector_id'] != $this->stored_values['bf_selector_id']) {
			// Access check
			if (!$this->inserted && !$GLOBALS['controller']->userHasAccessTo('AddSelector')) {
				throw new BFAccessException('AddSelector');
			}
			if (!$this->fields['bf_prop'])  {
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_prop', 'bf_selector', 'props', 'COUNT');
				$qb->where(array(
								array('=', 'bf_selector', 'bf_prop', ''),
								array('=', 'bf_selector', 'bf_selector_id', $this->fields['bf_selector_id']),
								array('!=', 'bf_selector', 'bf_selector_id', $this->stored_values['bf_selector_id'])
							));
				$count = $qb->getOne();
				if (DB::isError($count)) {
					throw new BFDBException($count->getMessage());
				} else if (intval($count) > 0) {
					throw new BFException(translate('2195', translate('selector')));
				}
			}
		}
		// Test for no access level
		if (!$this->fields['bf_prop'] && !$this->fields['bf_level'])  {
			 throw new BFException(translate('2256', translate('selector'), translate('access group')));
		}
		parent::save();
	}

	public function update() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('EditSelector')) {
			throw new BFAccessException('EditSelector');
		}
		$qb = BFQueryBuilder::update();
		$updated = false;
		
		foreach ($this->tablefields as $field) {
			if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
				if (!$updated) {
					$updated = true;
				}
				$qb->setColumn($field, $this->table, $this->fields[$field]);
			}
		}
		
		// No updates? Eh, it worked.
		if (!$updated) {
			return true;
		}
	   
		// Update the selector as a whole
		if ($this->fields['bf_selector_id'] != $this->stored_values['bf_selector_id'] || 
				$this->fields['bf_level'] != $this->stored_values['bf_level']) {
			$qb->where(array(
						 array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key])
					 ));
		// Or update a single variable in the selector
		} else {
			$qb->where(array(
						 array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]),
						 array('=', $this->table, 'bf_prop', $this->stored_values['bf_prop']),
						 array('=', $this->table, 'bf_operator', $this->stored_values['bf_operator']),
						 array('=', $this->table, 'bf_value', $this->stored_values['bf_value'])
					 ));
		}
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			if ($this->updateDependents())  {
				$this->logChange('Updated');
				return true;
			}
			return false;
		}
	}
		
	/**
	 * Work done solely because of someone's allergy to primary keys
	 */
	protected function updateDependents() {
		// Renamed the references from projects
		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_platform', 'bf_projects', $this->fields[$this->primary_key]);
		$qb->where(array('=', 'bf_projects', 'bf_platform', $this->stored_values[$this->primary_key]));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}
		
		// Renamed the references from steps
		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_resource', 'bf_steps', $this->fields[$this->primary_key]);
		$qb->where(array('=', 'bf_steps', 'bf_resource', $this->stored_values[$this->primary_key]));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		}
		
		return true;
	}


	/**
	 * Overridden in order to copy all the selector variables as well as the selector itself on a top-level copy.
	 */
	public function copy()  {
		if ($this->fields['bf_prop'])  {
			$this->inserted = false;
			return $this->save();
		}  else {
			//  We need to select all the selector variables from the database and re-insert them with the new ID.
			$oldid = $this->fields['bf_selector_id'];
			parent::copy();
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_prop', 'bf_selector');
			$qb->addColumn('bf_value', 'bf_selector');
			$qb->addColumn('bf_operator', 'bf_selector');
			$qb->addColumn('bf_required', 'bf_selector');
			$qb->where(array(
							array('!=', 'bf_selector', 'bf_prop', ''),
							array('=', 'bf_selector', 'bf_selector_id', $oldid)
						));
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			} else {
				$new_id = $this->get($this->primary_key);
				while ($var = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
					$new_var = new BFSelector();
					$new_var->set('bf_selector_id', $new_id);
					$new_var->set('bf_level', 0);
					$new_var->set('bf_prop', $var['bf_prop']);
					$new_var->set('bf_value', $var['bf_value']);
					$new_var->set('bf_operator', $var['bf_operator']);
					$new_var->set('bf_required', $var['bf_required']);
					$new_var->save();
				}
				return true;
			}
			return false;
		}
	}


	public function delete() {
		if ($this->stored_values['bf_prop'] == '') {
			return $this->deleteSelector();
		} else {
			return $this->deleteSelectorVariable();
		}
	}

	protected function deleteSelector() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('DeleteSelector')) {
			throw new BFAccessException('DeleteSelector');
		}
		if ($this->referenced())  {
			throw new BFException(translate('2051'));
		}
		
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key])
					));
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}

	protected function deleteSelectorVariable() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('EditSelector')) {
			throw new BFAccessException('EditSelector');
		}
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
						array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]),
						array('=', $this->table, 'bf_prop', $this->stored_values['bf_prop']),
						array('=', $this->table, 'bf_operator', $this->stored_values['bf_operator']),
						array('=', $this->table, 'bf_value', $this->stored_values['bf_value'])
					));
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}


	/**
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		$tables = array('bf_projects' => 'bf_platform', 'bf_steps' => 'bf_resource', 'bf_cron' => 'bf_selector');
		$count = 0;
		foreach ($tables as $table => $fieldname) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn($fieldname, $table, $fieldname, 'COUNT');
			$qb->where(array('=', $table, $fieldname, $this->fields[$this->primary_key]));
			$count += intval($qb->getOne());
		}
		return $count > 0;
	}
	
	
	/**
	 * Overridden to deal with the selector / selector property dichotomy.
	 */
	public function logChange($type)  {
		if ($this->log_changes && ($type != 'Updated' || 
				$this->controller->getSysconfig('log_all_changes') != 'Y'))  {
			if ($this->fields['bf_prop'])  {
				$this->controller->logAudit('SelectorProp' . $type, 
						array($this->fields['bf_prop'], $this->fields['bf_selector_id']));
			}  else {
				$this->controller->logAudit('Selector' . $type, array($this->fields['bf_selector_id']));
			}
		}
	}

	/**
	 * A BFDBO-centered logging function used to log before-and-after state on changed fields.  
	 */
	public function logChangedFields($field, $before, $after)  {
		$fieldname = isset($this->field_descriptions[$field]) ? 
				($this->field_descriptions[$field] ? translate($this->field_descriptions[$field]) : '') : 
				$field;
		if ($this->log_changes && $this->inserted && 
				$this->controller->getSysconfig('log_all_changes') == 'Y')  {
			if ($this->fields['bf_prop'])  {
				$this->controller->logAudit('SelectorPropUpdatedField', array($fieldname,
						$this->fields['bf_prop'], $this->fields['bf_selector_id'], $before, $after));
			}  else {
				$this->controller->logAudit('SelectorUpdatedField', array($fieldname,
						$this->fields['bf_selector_id'], $before, $after));
			}
		}
	}
}

?>
