<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFServer
 * A database object providing access to CRUDing the servers stored in the
 * database.
 * @package Console
 * @subpackage Agents
 */

if (!defined('TABLE_LOCK')) { define('TABLE_LOCK', true); }

$GLOBALS['controller']->require_module('BFDBO');

class BFServer extends BFDBO {
	public $table		   = 'bf_servers';
	public $primary_key	 = 'bf_id';
	public $descriptor_key = 'bf_name';
	public $required_fields = array('bf_name', 'bf_host', 'bf_path', 'bf_level');
	public $stored_values = array('bf_name' => null, 'bf_level' => null, 'bf_active' => null);
	public $manifest = array();
	protected $auto_test = false;
	protected $field_defaults = array(
									'bf_name' => '',
									'bf_level' => 0,
									'bf_host' => '',
									'bf_path' => '',
									'bf_publish' => '',
									'bf_eid' => 0,
									'bf_authid' => '',
									'bf_active' => 'Y',
									'bf_coll_id' => '',
									'bf_err' => 'T',
									'bf_test' => '',
									'bf_upstate' => 'R'
									);
	protected $bfstores = array('bf_test' => null);
	protected $trigger_connection_test = array('bf_host', 'bf_auth');
	public $type_name = 'Server';
	public $field_descriptions = array(
			'bf_id' => '',
			'bf_name' => 'name',
			'bf_level' => 'access',
			'bf_host' => 'host',
			'bf_path' => 'path',
			'bf_publish' => 'files',
			'bf_eid' => 'environment',
			'bf_authid' => 'authentication',
			'bf_active' => 'active',
			'bf_coll_id' => 'collector',
			'bf_err' => '',
			'bf_test' => '',
			'bf_upstate' => ''
			);

	private $request	= array();  // Handy way to pass $req around

	/** setRequest($this->request)
	 *	A handy way to pass this information around in a secure manner
	 */
	public function setRequest($req) {
		$this->request = $req;
	}

	/**
	 * Override BFDBO::set() so it can act like it has an inactive and active field, rather then
	 * just one. This makes code for forms that say one or the other a little more readable.
	 */
	public function set($field, $value) {

		// Access check
		if ($this->inserted && ($field == 'bf_inactive' || $field == 'bf_active') && !$this->controller->userHasAccessTo('ActivateServer')) {
			$newval = $field == 'bf_active' ? $value : ($value == 'Y' ? 'N' : 'Y');
			if ($this->stored_values['bf_active'] != $newval)  {
				throw new BFAccessException('ActivateServer');
			}
		}

		switch ($field) {
			case 'bf_inactive':
				parent::set('bf_active', ($value == 'Y') ? 'N' : 'Y');
				break;
			case 'bf_host':
			case 'bf_host':
			case 'bf_host':
				$this->auto_test = true;
			default:
				parent::set($field, $value);
		}
	}

	/**
	 * Really, you can only get a couple of manifest values
	 */
	public function getManifest($field) {
		foreach ($this->manifest as $entry)  {
			if ($entry['name'] == $field)  {
				return $entry['value'];
			}
		}

		return false;
	}

	/**
	 * Really, you can only set a couple of manifest values
	 */
	public function setManifest($field, $value, $unique = false) {
		if ($field == 'BF_JOBS' && !$GLOBALS['controller']->userHasAccessTo('ResetServerUse')) {
			throw new BFAccessException('ResetServerUse');
		}
		if ($unique)  {
			foreach ($this->manifest as $key => $entry)  {
				if ($entry['name'] == $field) {
					$this->manifest[$key]['value'] = $value;
					return true;
				}
			}
		}
		$this->manifest[] = array('name' => $field, 'value' => $value);
		return true;
	}

	/**
	 * Override the selection of the database object so that (as long as it
	 * gets loaded from the database) the manifest gets loaded as well.
	 */
	public function selectByFields($fields, $values, $comp = '=') {
		if (parent::selectByFields($fields, $values, $comp)) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn(BF_FIELD_PREFIX . 'prop', BF_TABLE_PREFIX . 'manifest', 'name');
			$qb->addColumn(BF_FIELD_PREFIX . 'value', BF_TABLE_PREFIX . 'manifest', 'value');
			$qb->where(array('=', BF_TABLE_PREFIX . 'manifest', BF_FIELD_PREFIX . 'name', $this->get(BF_FIELD_PREFIX . 'name')));
			$qb->orderBy('ASC', 'name');
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}
			while($man = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$this->manifest[] = array(
					'name' => $man['name'],
					'value' => $man['value']
				);
			}
			return true;
		}

		return false;
	}

	/**
	 * Override save() to keep from duplicating server names
	 */
	public function save() {
		if (!$this->hasRequiredFields()) {
			// Catch this and handle it sanely per-object
			throw new BFException(translate('2181'));
		}
		// Only check if changed or inserting
		if ($this->updated['bf_name']) {
			// Manage the manifest entries...
			if ($this->stored_values['bf_name'] && $this->stored_values['bf_name'] != $this->fields['bf_name'])  {
				$tmpname = $this->fields['bf_name'];
				$this->fields['bf_name'] = $this->stored_values['bf_name'];
				if (!$this->deleteManifest()) {
					$GLOBALS['controller']->addError(translate('2203'));
				}
				$this->fields['bf_name'] = $tmpname;
			}
			if (!$this->saveManifest()) {
				$GLOBALS['controller']->addError(translate('2204'));
			}
			// Get the number of servers with the current name
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_name', 'bf_servers', 'namecount', 'COUNT');
			$qb->where(array(
				array('=', 'bf_servers', 'bf_name', $this->fields['bf_name']),
				array('!=', 'bf_servers', 'bf_name', $this->stored_values['bf_name'])
			));
			$count = $qb->getOne();

			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('server')));
			}
		}

		return parent::save();
	}

	/**
	 * Override for access check
	 */
	public function insert() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('AddServer')) {
			return parent::insert();
		} else {
			throw new BFAccessException('AddServer');
		}
	}

	/**
	 * Override for access check
	 */
	public function update() {
		// Access check
		if (!$GLOBALS['controller']->userHasAccessTo('EditServer') || 
				((int)$GLOBALS['user']->get('bf_userid') !== 1 && 
				!in_array($this->stored_values['bf_level'], 
				bf_get_levels($GLOBALS['user']->get('bf_userid'), true)))) {
			throw new BFAccessException('EditServer');
		} else {
			// Collector change?
			if (array_key_exists('bf_coll_id', $this->updated) && $this->updated['bf_coll_id']) {
				$this->deleteManifest();
				$this->saveManifest();
			}

			// Core data change?
			if ($this->auto_test) {
				$this->set('bf_err', 'T');
				$this->auto_test = false;
			}
			return parent::update();
		}
	}

	/**
	 * Override to clear out the manifest on success
	 */
	public function delete() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('DeleteServer')) {
			if (parent::delete()) {
				$this->deleteManifest();
				return true;
			}
			return false;
		} else {
			throw new BFAccessException('DeleteServer');
		}
	}

	/**
	 * Delete the manifest for this server
	 */
	public function deleteManifest() {
		$qb = BFQueryBuilder::delete();
		$qb->where(array('=', BF_TABLE_PREFIX . 'manifest', BF_FIELD_PREFIX . 'name', $this->fields[BF_FIELD_PREFIX . 'name']));
		$res = $qb->getResult();

		if (DB::isError($res)) {
			$GLOBALS['controller']->addError($res->getMessage());
		} else {
			return true;
		}
	}

	/**
	 * Manage the manifest info
	 */
	public function saveManifest() {
		$GLOBALS['controller']->require_module('BFManifest');
		$this->setManifest('BF_LAST_UPDATE', 0, true);

		foreach ($this->manifest as $man => $fest) {
			$manfest = new BFManifest();

			// Try to load the old value
			$manfest->select(
							 array(BF_FIELD_PREFIX . 'name', BF_FIELD_PREFIX . 'prop'),
							 array($this->fields[BF_FIELD_PREFIX . 'name'], $fest['name'])
							 );

			// Regardless, set values
			$manfest->set(BF_FIELD_PREFIX . 'name', $this->fields[BF_FIELD_PREFIX . 'name']);
			$manfest->set(BF_FIELD_PREFIX . 'prop', $fest['name']);
			$manfest->set(BF_FIELD_PREFIX . 'value', $fest['value']);

			// Save and panic on failure
			$manfest->save();
		}

		return true;
	}

	/**
	 * Populate the manifest with the default info
	 */
	public function populateManifest() {
		$manfest	= array();
		$manfest[]	= array('name' => 'BF_LAST_UPDATE', 'value' => 0);

		foreach ($manfest as $man => $fest) {
			$qb = BFQueryBuilder::insert();
			$qb->setColumn(BF_FIELD_PREFIX . 'name', BF_TABLE_PREFIX . 'manifest', $this->get(BF_FIELD_PREFIX . 'name'));
			$qb->setColumn(BF_FIELD_PREFIX . 'prop', BF_TABLE_PREFIX . 'manifest', $fest['name']);
			$qb->setColumn(BF_FIELD_PREFIX . 'value', BF_TABLE_PREFIX . 'manifest', $fest['value']);
			$qb->setColumn(BF_FIELD_PREFIX . 'category', BF_TABLE_PREFIX . 'manifest', 'A');
			$res = $qb->getResult();
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			}
		}
		return true;
	}

	/**
	 * loadFields() sets up a couple of added fields from other tables,
	 * as well as a couple of automatic joins to go with.
	 */
	public function loadFields() {
		global $user;

		parent::loadFields();
		$this->fields['bf_level']   = null;
		$this->fields['bf_env']	 = null;

		// If not root, you need to have access
		$uid = (int)$user->get('bf_userid');
		if ($uid !== 1) {
			$l = array_keys(bf_get_levels($uid));
			if (is_array($l) && count($l) > 0) {
				$this->querybuilder->where(array(array('IN', 'bf_servers', 'bf_level', $l)));
			} else {
				$this->querybuilder->where(array('=', 'bf_servers', 'bf_level', 0));
			}

		}
	}


	/** saveNote($serverID, $noteID)
	 *	Saves note to database, if no NID then it's a new note..
	 */
	public function saveNote($serverID, $noteID) {
		$noteText = $this->request['note_text'];
		$noteUser = (array_key_exists('note_user_id', $this->request))?$this->request['note_user_id']:'0';
		$noteDate = (array_key_exists('note_date', $this->request))?strtotime($this->request['note_date']):strtotime('now');
		$noteDate = ($noteDate == -1)?strtotime('now'):$noteDate; // IE is causing some funky stuff with dates

		$storage = new BFStore();
		$storeID = $storage->setStorage($noteText, $noteID);

		if ($noteID == '') {

			$qb = BFQueryBuilder::insert(null, TABLE_LOCK);
			$qb->setColumn('bf_created',	'bf_notes',	 $noteDate);
			$qb->setColumn('bf_userid',		'bf_notes',	 $noteUser);
			$qb->setColumn('bf_type',		'bf_notes',	 'Server');
			$qb->setColumn('bf_pid',		'bf_notes',	 $serverID);
			$qb->setColumn('bf_id',			'bf_notes',	 '0');
			$qb->setColumn('bf_data',		'bf_notes',	 $storeID);
			$res = $qb->getResult();

			if (DB::isError($res)) {
				$this->controller->addError(translate('2222'));
				return false; }
		}
		return true;
	}


	/** deleteNote($serverID, $noteID)
	 * This function does the job of deleting the note from the database and delete the storage item associated with it
	 */
	public function deleteNote($serverID, $noteID) {
		$storage = new BFStore();

		// Delete the item from bf_store
		$res = $storage->removeStorage($noteID);
		if (! $res) { return false; }

		$qb = BFQueryBuilder::delete(null, TABLE_LOCK);
		$qb->where(array(
			array('=', 'bf_notes', 'bf_type',	'Server'),
			array('=', 'bf_notes', 'bf_pid',	$serverID),
			array('=', 'bf_notes', 'bf_data',	$noteID)
		));
		$res = $qb->getResult();
		if (DB::isError($res)) { return false; }

		return true;
	}




}

?>
