<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFServerAuth
 * @package Console
 * @subpackage ServerAuth
 */

$GLOBALS['controller']->require_file('Module.php');

class BFServerAuth extends BFDBO {
	public $table	   = 'bf_serverauth';
	public $primary_key = 'bf_authid';
	public $descriptor_key = 'bf_authid';
	public $stored_values = array('bf_authid' => null);
	public $required_fields = array('bf_authid');
	protected $field_defaults = array('bf_username' => '', 'bf_password' => '');
	public $type_name = 'ServerAuth';
	public $field_descriptions = array(
			'bf_authid' => 'name',
			'bf_username' => 'login',
			'bf_password' => '',
			'bf_level' => 'access'
			);
	
	/**
	 * Override so we can look for dupes
	 */
	public function save() {
		if (!$this->fields[$this->primary_key] )  {
			throw new BFException(translate('2256', translate('server auth'), translate('name')));
		}
		if ($this->updated['bf_authid']) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_authid', $this->table, 'auths', 'COUNT');
			$qb->where(array(
				array('=', $this->table, 'bf_authid', $this->fields['bf_authid']),
				array('!=', $this->table, 'bf_authid', $this->stored_values['bf_authid'])
			));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('server auth')));
			}
		}
		
		return parent::save();
	}
	
	public function getReferences() {
		if (!$this->fields[$this->primary_key] )  {
			return false;
		}

		$qb = BFQueryBuilder::select();
		$qb->addColumn(BF_FIELD_PREFIX . 'authid', BF_TABLE_PREFIX . 'servers', BF_FIELD_PREFIX . 'authid');
		$qb->where(array('=', BF_TABLE_PREFIX . 'servers', BF_FIELD_PREFIX . 'authid', $this->fields[$this->primary_key]));
		$res = $qb->getCol();
		
		return (DB::isError($res)) ? false : $res;
	}
	
	public function insert()  {
		// Access check
		if (!$this->controller->userHasAccessTo('AddServerAuth')) {
			throw new BFAccessException('AddServerAuth');
		}
		
		return parent::insert();
	}
			
		
	/**
	 * Overloaded, since the "primary key" really = the name...
	 */
	public function update() {
		// Access check
		if (!$this->controller->userHasAccessTo('EditServerAuth')) {
			throw new BFAccessException('EditServerAuth');
		}
		
		if (!$this->hasRequiredFields()) {
			// Catch this and handle it sanely per-object
			throw new BFException(translate('2181'));
		}
		$qb = BFQueryBuilder::update();
		$updated = false;
		foreach ($this->tablefields as $field) {
			if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
				if (!$updated) {
					$updated = true;
				}
				$qb->setColumn($field, $this->table, $this->fields[$field]);
			}
		}
		
		// No wasting queries.		
		if (!$updated) {
			return true;
		}
		
		$qb->where(array('=', $this->table, $this->primary_key, $this->stored_values['bf_authid']));
		
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			// Now politely change any associations in the servers
			$fix = BFQueryBuilder::update();
			$fix->setColumn('bf_authid', 'bf_servers', $this->fields[$this->primary_key]);
			$fix->where(array('=', 'bf_servers', 'bf_authid', $this->stored_values[$this->primary_key]));
			$fixres = $fix->getResult();
			if (DB::isError($fixres)) {
				throw new BFDBException($fixres->getMessage());
			} else {
				$this->logChange('Updated');
				return true;
			}
		}
	}
	
	/**
	 * Overloaded, since the "primary key" really = the name...
	 */
	public function delete() {
		// Access check
		if (!$this->controller->userHasAccessTo('DeleteServerAuth')) {
			throw new BFAccessException('DeleteServerAuth');
		}
		
		$refs = $this->getReferences();
		$refs = $refs ? count($refs) : false;
		if ($refs)  {
			throw new BFDBException(translate('2061', $refs));
		}

		$qb = BFQueryBuilder::delete();
		$qb->where(array('=', $this->table, $this->primary_key, $this->stored_values['bf_authid']));
		
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			throw new BFDBException($res->getMessage());
		} else {
			$this->logChange('Deleted');
			return true;
		}
	}
}

?>
