<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFStart
 *
 * @package Console
 * @subpackage Core
 */

if (!defined('TABLE_LOCK')) {
	define('TABLE_LOCK', true);
}

$GLOBALS['controller']->require_module('BFDBO');
$GLOBALS['controller']->require_module('BFUID');

class BFStart extends BFDBO {

	/** READ ME READ ME READ ME IGNORE ALLONI READ ME READ ME HATE SHAWN READ ME READ ME
	 * 
	 *   Look.. Just come get me if you need to change anything in here.. I'll do it.	
	 *   - Jason	  
	 * 
	 *  READ ME READ ME READ ME EAT POPCORN READ ME READ ME DRINK COKE READ ME READ ME	 
	 */
	 
	 	
	public function quickStart($pid) {
		return $this->jobStarter('quick', $pid);
	}	 	
	
	public function classChangeStart($job, $projectToRun) {
		return $this->jobStarter('classchange', $projectToRun, array('BID'=>$job));
	}
	
	public function manualStart($formSubmission) {
		return $this->jobStarter('manual', $formSubmission['bf_id'], $formSubmission);
	}
	
	public function restart($formSubmission) {
		return $this->jobStarter('restart', $formSubmission['bf_id'], $formSubmission);
	}

	public function clobber($pid) {
		return $this->jobStarter('clobber', $pid);
	}
	 	
	/** Start a project
	 *  This is meant to be used to start any type of job.
	 * 	 Make sure we have a PID, no matter starttype there must be a PID to run
	 * 	 Get the userid and make sure he has access to even run builds
	 * 	 Make sure user hasn't exceeded his build limit
	 * 	 Make sure the user has access to even see this build
	 * 	 Make sure we arent trying to quick start a library (needs a selector)
	 * 	 Make sure there arent required fields in the environment block	 
	 * 	 Make sure this quick start actually has some steps to start
	 * 	 Okay we have checked, and if everything is in order;
	 * 	 Generate the working tag, and environment block
	 * 	 Insert environment block into storage
	 * 	 Insert build into database	else fail
	 * 	 And log it	
	 *
	 * 	Uses:
	 * 	 $this->parseTag	 	 
	 * 	 $this->getEnvironment
	 * 	 $this->getSteps
	 *
	 * 	Job Types:
	 * 	 'quick' 		= quickStart of a job - this is basically the default type
	 * 	 'manual'		= jobs started via the start page	 
	 * 	 'classchange'	= a class change type, has to deal with the environment block differently
	 * 	 'restart'		= like a manual start but a restart - so it's "used"	 	 	 	 
	 *	 'clobber'		= a job that murders other jobs - just pass it an unsuspecting ID
	 */	 	 	 	
	private function jobStarter($startType, $pid, $args = array()) {	
		global $user;

		// Valid types
		$validTypes = array('quick','manual','classchange','restart','clobber');
		if (!in_array($startType, $validTypes)) { return false; }
		
		// Make sure we have a PID
		if (empty($pid)) { return false; }
		
		// Does the user have access to clobber?
		if ($startType == 'clobber') {
			// Check if they are allowed to clobber - not in bf_can_start
			if (!$GLOBALS['controller']->userHasAccessTo('ClobberProject')) {
				$GLOBALS['controller']->addError('NOACCESSTOCLOBBER');
				return false;
			}
		} else {
			// All the testing for permissions, etc. has moved into bf_can_start.
			$canstart = bf_can_start($pid);
			if ($canstart)  {
				$GLOBALS['controller']->addError(translate($canstart));
				return false;
			}
		}

		// Define the users id
		$uid = intval($user->get('bf_userid'));
		
		// Get project information		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_platform',	'bf_projects',	'resource');
		$qb->addColumn('bf_tag',		'bf_projects',	'tag');
		$qb->addColumn('bf_level',		'bf_projects',	'level');
		$qb->addColumn('bf_eid',		'bf_projects',	'env');
		$qb->addColumn('bf_class',		'bf_projects',	'bfclass');
		$qb->addColumn('bf_description','bf_projects',	'name');
		$qb->addColumn('bf_pcid',		'bf_projects',	'pcid');
		$qb->addColumn('bf_fcid',		'bf_projects',	'fcid');
		$qb->addColumn('bf_geo_id',		'bf_projects',	'geo');

		$qb->where(array(
			array('=', 'bf_projects', 'bf_pid',	$pid),
		));
		
		$res = $qb->getResult();	
		if (DB::isError($res)) {
			$GLOBALS['controller']->addError(translate(2174));
			return false;
		}

		// Define our Project
		$ourProject = $res->fetchRow();
		
		// If its not defined then kick out.. most likely due to access..  
		if (empty($ourProject)) {
			$GLOBALS['controller']->addError(translate(2174));
			return false;
		}
		
		$environment  = '';
		$classenv = '';

		/* Generate the inclusion/exclusion list for this project
		 *	and append it to the environment block - disabled steps
		 *	are intentionally left off completely.
		 *	
		 * This is done differently for manual versus quick starts
		 *  -- What do we do about CHILDREN of CHILDREN etc here?
		 *  
		 * This is done here so that we know to pass or fail before updating tags
		 * this information will be used later down the line		 		 		 		 		 		 
		 */		
		if (($startType == 'manual' || $startType == 'restart') && isset($args['stepstate']) && is_array($args['stepstate'])) {
			$activeSteps = array();
			foreach ($args['stepstate'] as $sk=>$sv) {
				if ($sv == '10' || $sv == '11') {
					$activeSteps[] = "+".$sk;
				} else {
					$activeSteps[] = "-".$sk;
				}
			}			
		} else {		 
			$activeSteps = $this->getSteps($pid);
		}

		// Are we trying to start a build with no steps selected?
		if(empty($activeSteps) && $startType != 'clobber') {
			$GLOBALS['controller']->addError(translate(2176));
			return false;						
		}

		/** Create alternate information based on manual / not manual */
		$fParam	= isset($args['parameter']) ? $args['parameter'] : array();			
		$tSelector = (($startType == 'manual' || $startType == 'restart') && $fParam['selector'] != '')?$fParam['selector']:$ourProject->resource;
		$tClass = (($startType == 'manual' || $startType == 'restart') && $fParam['class'] != '')?$fParam['class']:$ourProject->bfclass;
		$tLevel = ($startType == 'manual' && $fParam['level'] != '')?$fParam['level']:$ourProject->level;
		$tGeo = (isset($fParam['geo']) && $startType == 'manual' && $fParam['geo'] != '')?$fParam['geo']:$ourProject->geo;
		
		/**
		 * Check if the selector exists and the user has access to the selector and class
		 * We completely don't care about either, though, if we're clobbering the project.
		 */

		if ($startType != 'clobber')  {
			if (!$tSelector)  {
				$GLOBALS['controller']->addError(translate('2031'));
				return false;
			}
			$access = bf_get_levels($uid);
	
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_level', 'bf_selector');
			$qb->where(array(
					array('=', 'bf_selector', 'bf_selector_id', $tSelector),
					array('=', 'bf_selector', 'bf_prop', '')
					));
			$selectorlevel = $qb->getOne();
			if (!$selectorlevel)  {
				$GLOBALS['controller']->addError(translate('2031'));
				return false;
			}  elseif (!isset($access[$selectorlevel]))  {
				$GLOBALS['controller']->addError(translate('2039'));
				return false;
			}

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_level', 'bf_classes');
			$qb->where(array('=', 'bf_classes', 'bf_name', $tClass));
			$classlevel = $qb->getOne();
			if (!isset($access[$classlevel]))  {
				$GLOBALS['controller']->addError(translate('2039'));
				return false;
			}			
				
		}

		/* Generate both the actual working tag and the parsed list of tags
		 *	used in the environment block && append it to the environment
		 * Then generate the Environment block		 
		 *	- if this is a manual start then generate the lists from the form
		 *	otherwise generate it from the database
		 */		 		 
		if ($startType == 'manual') {
		
			// Extract all the form submitted crap into shorter vars
			$fParam	= isset($args['parameter']) ? $args['parameter'] : array();			
			$fTags 	= isset($args['tag']) ? $args['tag'] : array();
			$fEnvs	= isset($args['environment']) ? $args['environment'] : array();
		
			/** Dirty tags
			 * there is now this new fangled check boxy thingy on the start form which will
			 * flag tags as either being dirty or not.  Dirty tags mean we need to trust the 
			 * form, otherwise it will just pull them directly from the database as you would
			 * on a chain or quick start.
			 */			 			 			 			 			
			if ($GLOBALS['controller']->userHasAccessTo('EditStartTagVars') && (isset($args['dirtyTags']))) {		
				list ($workingTag, $parsedTags) = $this->parseManualTag($pid, $fParam['tagformat'], $fTags);
			} else {				
				list ($workingTag, $parsedTags) = $this->parseTag($pid, $fParam['tagformat']);
			}

			foreach ($parsedTags as $tagKey=>$tagValue) {
				$environment .= 'S:'.$tagKey.':'.((is_array($tagValue))?$tagValue['value']:$tagValue)."\n";
			}
		
			// Generate the environment variables for this project	and append it to the environment block
			$parsedEnv = $this->getManualEnvironment($pid, $fEnvs, ((array_key_exists('saveenv',$fParam))?$fParam['saveenv']:''));

		} else if ($startType == 'restart') {
			
			// Extract all the form submitted crap into shorter vars
			$fParam	= isset($args['parameter']) ? $args['parameter'] : array();			
			$fTags 	= isset($args['tag']) ? $args['tag'] : array();
			$fEnvs	= isset($args['environment']) ? $args['environment'] : array();
					
			// Generate the environment variables for this project	and append it to the environment block
			$parsedEnv = $this->getManualEnvironment($pid, $fEnvs, '');			

		} else {
		
			list ($workingTag, $parsedTags) = $this->parseTag($pid);
			if ($startType == 'classchange') {
				foreach ($parsedTags as $tagKey=>$tagValue) {
					$classenv.= 'S:'.$tagKey.':'.$tagValue."\n";
				}
			}
			else {
				foreach ($parsedTags as $tagKey=>$tagValue) {
					$environment .= 'S:'.$tagKey.':'.$tagValue."\n";
				}
			}
			
			// Generate the environment variables for this project and append it to the environment block 
			$parsedEnv = $this->getEnvironment($ourProject->env);
			
		}

		/** Check if the tag is empty */
		if ((!isset($workingTag) || !$workingTag) && $startType != 'restart')  {
			$GLOBALS['controller']->addError(translate('2252'));
			return false;
		}
		
		/**  Reformat the parsedENV to the environment block
		 * Check for required / must change fields - if its a classchange block then ignore
		 */		 
		foreach ($parsedEnv as $env) {
			if ($startType == 'classchange') {
				$classenv .= $env['action'].':'.$env['key'].':'.$env['value']."\n";
			}
			else {
				$environment .= $env['action'].':'.$env['key'].':'.$env['value']."\n";
			}
			
			// Check for pipe matches
			$matches = explode("|", $env['origvalue'], 2);
			if (is_array($matches) && isset($matches[0])) {
				$firstPipe = $matches[0];
			}

			// Test to see if we have any "must changes" that havent been changed - ignore for classchange
			if (( $env['mode'] == 'C' && ($env['origvalue'] == $env['value'] || ($firstPipe == $env['value'])) ) && $startType != 'classchange' && $startType != 'clobber') {
				$GLOBALS['controller']->addError(translate(2178));
				//THE FOLLOWING CODE HAS BEEN ADDED TO PREVENT UNINTENTIONAL ITERATION OF THE BUILD NUMBER WHEN THE EXECUTE FAILS WHEN
				//A MUST CHANGE VAR IS NOT CHANGED - RATLC01271622
				//First - get all the values of bf_tid associated with this pid
				$qb1 = BFQueryBuilder::select();
				$qb1->addColumn('bf_tid', 'bf_tags');
				$qb1->where(array('=', 'bf_tags', 'bf_pid', $pid));
				$result = $qb1->getResult();
				
				while($row = $result->fetchRow(DB_FETCHMODE_ASSOC)){
					//Then iterate thorough each tid 
					foreach($row as $key=>$value) {
						//establish the value of the build number
						$qb2 = BFQueryBuilder::select();
						$qb2->addColumn('bf_value', 'bf_tags');
						$qb2->addColumn('bf_autoinc','bf_tags');
						$qb2->where(array('=', 'bf_tags', 'bf_tid', $value));
						$qb2->where(array('=', 'bf_tags', 'bf_pid', $pid));
						$res = $qb2->getResult();
						$buildValue = $res->fetchRow();
						//then decrement bf_value for each bf_tid associated with this bf_pid
						//IF it was incremented. - KWP
						$newValue = ($buildValue->bf_autoinc == 'Y') ? --$buildValue->bf_value : $buildValue->bf_value;

						$qb = BFQueryBuilder::update(null, TABLE_LOCK);
						$qb->setColumn('bf_value', 'bf_tags', $newValue);
						$qb->where(	array (
						array('=', 'bf_tags',  'bf_pid', $pid),
						array('=', 'bf_tags',  'bf_tid', $value)
						));
						if (DB::isError($qb->getResult())) { return false; }
					}
				}
				//END OF ADDED CODE - RAROSSI											
				return false;						
			}
			
			// Test to see if we have any "requireds" that havent been filled out - ignore for classchange
			if (( $env['mode'] == 'R' && $env['value'] == '' ) && $startType != 'classchange' && $startType != 'clobber') {
				$GLOBALS['controller']->addError(translate(2177));
				return false;						
			}
		}

		if ($startType == 'classchange') {
			
			/** Using $BID we must do the caller stuff - This is for chaining.
			 * This whole thing needs refactoring like you wouldn't believe  */
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_pid',		'bf_builds');
			$qb->addColumn('bf_start',		'bf_builds');
			$qb->addColumn('bf_resource',	'bf_builds');
			$qb->addColumn('bf_env',		'bf_builds');
			$qb->addColumn('bf_tag',		'bf_builds');
			$qb->where(array('=', 'bf_builds', 'bf_bid', $args['BID']));
			$res = $qb->getResult();
			$origBuild = $res->fetchRow();
	
			// Get EnvStorage
			$storage = new BFStore();
			$origEnvs = $storage->getStorage($origBuild->bf_env);
	
			// Get the original environment black from the build
			$origBuildEnvs = preg_split("|\n|", $origEnvs['TEXT']);
					
			// Get stack sys param
			$stack = $GLOBALS['controller']->getSysconfig('stack_caller');
	
			/** Get the original builds environments and weed out everyting but the BF_ 
			 * Then replace BF_CALLER with BF_.. if stack is turned on then allow for 
			 * BF_CALLER_CALLER .. otherwise dont..  this is lame. */		 		 		
			$origBFs = array();
			foreach ($origBuildEnvs as $v) {
				if ($v)  {
					list($action, $key, $value) = split(":", $v, 3);
					if (preg_match("/BF_(.*)/", $key, $match)) {
						if (strtoupper($stack) == 'Y') {
							// if we are stacking then bf_caller_caller is allowed
							$origBFs[] = array($action,'BF_CALLER_'.$match[1],$value);				
						} else {
							if (!preg_match("/BF_CALLER_/",$key)) {
								$origBFs[] = array($action,'BF_CALLER_'.$match[1],$value);									
							}
						}
					} else {
						// not a BF_* variable, don't add BF_CALLER to it, but do add
						// it to the ENV
						$origBFs[] = array($action,$key,$value);
          }
				}
			}

			// Reconstitute into the new and improved env block.		
			foreach ($origBFs as $v) {	 		
				$environment .= implode(':',$v)."\n";				
			}	
			
			// Add Tag
			$environment .= 'S:BF_CALLER_TAG:'.$origBuild->bf_tag."\n";
			
			// Add Start Time of orig - in a jacked up format.
			$environment .= 'S:BF_CALLER_LASTRUN:'.date('Y-M-d H:i:s',$origBuild->bf_start)."\n";
			
			// Add Project name - For mmmkay sake we don't store this information
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_description', 'bf_projects');
			$qb->where(array('=','bf_projects','bf_pid',$origBuild->bf_pid));
			$environment .= 'S:BF_CALLER_PROJECTNAME:'.$qb->getOne()."\n";
			
			// Add Project Selector
			$environment .= 'S:BF_CALLER_SELECTOR:'.$origBuild->bf_resource."\n";
			
			//Put the initial build to the end of the class change build
			$environment .= $classenv;
		 
		}

		// Now include those into the environment block based on Active Steps
		$environment .= "X:X:".implode(',',$activeSteps)."\n";
		if ($startType == 'classchange') {
			// Because Robin says so - this is due to chaining?  dunno
			$environment .= "Z:Z:Z\n";
		}

		/** Restart Devation.  If a restart then simply redo that which we already have
		 * and step off..  Chad do not change my comments!  Chris will change them!
		 * Replace the environment and then simple flip the build table back to running
		 * Then break out and do nothing else below.		 		 
		 */		 		
		if ($startType == 'restart') {
			// Sync Commands
			if ($fParam['synccommands'] == 'sync') {
				$environment .= "Y:Y:1\n";
			}

			/** Save the environment block BACK to storage. */		
			$storage = new BFStore();
			$storage->setStorage($environment, $fParam['env']);
			
			// Now update the build record
			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_state', 		'bf_builds',	'W');
			$qb->setColumn('bf_jobid', 		'bf_builds',	'0');
			$qb->setColumn('bf_can',   		'bf_builds',	'N');
			$qb->setColumn('bf_result',		'bf_builds',	'');
			$qb->setColumn('bf_resource',	'bf_builds',	$tSelector);
			$qb->setColumn('bf_class',		'bf_builds',	$tClass);

			$qb->setColumn('bf_userid','bf_builds',	$uid);
			$qb->where(array('=', 'bf_builds', 'bf_bid', $fParam['bid']));
			$res = $qb->getResult();
	
			if (DB::isError($res)) {
				return false;
			}

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tag', 'bf_builds');
			$qb->where(array('=', 'bf_builds', 'bf_bid', $fParam['bid']));
			$restartTag = $qb->getOne();
			
			$this->controller->logAudit('BuildQueued', array($ourProject->name, $restartTag));						
			return true;
		}
		
		/** At this point we have created the environment and need to save
		 * it to storage - we do this before we insert the build so that 
		 * there isn't a race condition with the engine		 
		 */		 		
		$storage = new BFStore();
		$EnvID = $storage->setStorage($environment);

		// Get the build id for this build
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_value',   'bf_sysconfig',	'value');
		$qb->where(array (array('=', 'bf_sysconfig',  'bf_param', 'build_id')));
		$thisBuildID = $qb->getOne(false);
		$nextBuildID = $thisBuildID + 1; // Probably need to run some check here to make sure that id hasn't been used or isn't in use.

		// Set the next build id to be used
		$qb = BFQueryBuilder::update(null, TABLE_LOCK);
		$qb->setColumn('bf_value',  'bf_sysconfig',  $nextBuildID);
		$qb->where(array(
			array('=', 'bf_sysconfig', 'bf_param', 'build_id'),
			array('=', 'bf_sysconfig', 'bf_value', $thisBuildID)
		));
		$qb->getResult();

		/** Manual Link
		 * Get the Run Link setting from the form.  If it's checked, set as "Queued" otherwise ""
		 */	
		if ($startType == 'quick') {
			$runManLink = $GLOBALS['controller']->getSysconfig('run_man_link');
			$bfStage = (isset($runManLink) && $runManLink == 'Y') ? 'Queued' : '';
		} else {
			$bfStage = (isset($fParam['manlink']) && $fParam['manlink'] == 'Y') ? 'Queued' : '';
		}

		// Determine bf_process
		$bfProcess = ($startType == 'clobber')?'N':'B';

		/** Create entry in builds table
		 */
		$qb = BFQueryBuilder::insert(null, TABLE_LOCK);
		$qb->setColumn('bf_bid',		'bf_builds',	$thisBuildID);	//Use THIS buildid as pulled from bf_sysconfig
		$qb->setColumn('bf_pid',		'bf_builds',	$pid);			//Project ID
		$qb->setColumn('bf_start',		'bf_builds',	'0');			//When run starts, for us set to zero
		$qb->setColumn('bf_state',		'bf_builds',	'W');			//State of build, for us 'W'aiting
		$qb->setColumn('bf_process',	'bf_builds',	$bfProcess);	//Type of build, for us 'B'uild or 'N' if we are nuking.
		$qb->setColumn('bf_prg',		'bf_builds',	'N');			//No longer used, 'N'
		$qb->setColumn('bf_can',		'bf_builds',	'N');			//Cancelled, 'N'
		$qb->setColumn('bf_resource',	'bf_builds',	$tSelector);	//Server
		$qb->setColumn('bf_tag',		'bf_builds',	$workingTag);	//Tag parse-see bf_tags table
		$qb->setColumn('bf_userid',		'bf_builds',	$uid);			//User ID of user that started build
		$qb->setColumn('bf_level',		'bf_builds',	$tLevel);		//Project Level
		$qb->setColumn('bf_result',		'bf_builds',	'');			//Result,''
		$qb->setColumn('bf_duration',	'bf_builds',	'0');			//Time taken to execute,'0'
		$qb->setColumn('bf_size',		'bf_builds',	'0');			//Overloaded,Reflector setting,'0'
		$qb->setColumn('bf_stage',		'bf_builds',	$bfStage);		//Stage of build,''
		$qb->setColumn('bf_fired',		'bf_builds',	gmdate('U'));	//UTC of when person clicked "RUN" in GMT
		$qb->setColumn('bf_jobid',		'bf_builds',	'0');			//No longer used,'0'
		$qb->setColumn('bf_env',		'bf_builds',	$EnvID);		//Project Environment
		$qb->setColumn('bf_class',		'bf_builds',	$tClass);		//Project Class
		$qb->setcolumn('bf_engineid',	'bf_builds', 	'');			// Whats this for?
		$qb->setcolumn('bf_pcid',		'bf_builds', 	$ourProject->pcid);
		$qb->setcolumn('bf_fcid',		'bf_builds', 	$ourProject->fcid);
		$qb->setcolumn('bf_cbid',		'bf_builds', 	'0');			// Whats this for?
		$qb->setcolumn('bf_geo_id',		'bf_builds', 	$tGeo);			// GDD or blank		
		$res = $qb->getResult();

		if (DB::isError($res)) {
			return false;
		} else {
			// Log it
			$this->controller->logAudit('BuildQueued', array($ourProject->name, $workingTag));			
			return $thisBuildID;
		}
	}


	/** parseTag($ProjectID)
	 *  This function does the job of parsing out the tag information
	 */
	private function parseTag($PID, $workingTag = '') {
		// An array of each tag and final value to be returned
		$returnTags = array();

		// Define Built in tags
		// Get tag_d_format - Set S:BF_D:...
		$e = BFQueryBuilder::select();
		$e->addColumn('bf_value', 'bf_sysconfig', 'tag_d_format');
		$e->where(array('=', 'bf_sysconfig', 'bf_param', 'tag_d_format'));
		$builtInTag['BF_D'] = date($e->getOne());

		// Get tag_t_format - Set S:BF_T:...
		$e = BFQueryBuilder::select();
		$e->addColumn('bf_value', 'bf_sysconfig', 'tag_t_format');
		$e->where(array('=', 'bf_sysconfig', 'bf_param', 'tag_t_format'));
		$builtInTag['BF_T'] = date(str_replace('h', 'H', str_replace('m', 'i', $e->getOne())));

		// Set Julian Date
		$builtInTag['BF_J'] = date('z');

		// Set Week Day
		$builtInTag['BF_W'] = date('w');

		// Get our project information
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_platform',	'bf_projects','resource');
		$qb->addColumn('bf_tag',		'bf_projects','tag');
		$qb->addColumn('bf_level',		'bf_projects','level');
		$qb->addColumn('bf_eid',		'bf_projects','env');
		$qb->addColumn('bf_class',		'bf_projects','bfclass');
		$qb->addColumn('bf_tagsync',	'bf_projects','sync');
		$qb->where(	array('=', 'bf_projects', 'bf_pid',	$PID) );
		$res = $qb->getResult();
		$ourProject = $res->fetchRow();

		// Actually start doing stuff with the tag
		$workingTag = ($workingTag == '')?$ourProject->tag:$workingTag;
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tid',	'bf_tags','id');
		$qb->addColumn('bf_tid',	'bf_tags','master_id');	// used later
		$qb->addColumn('bf_name',	'bf_tags','name');
		$qb->addColumn('bf_value',	'bf_tags','value');
		$qb->addColumn('bf_parent',	'bf_tags','padding');
		$qb->addColumn('bf_autoinc','bf_tags','autoinc');
		$qb->addColumn('bf_autoinc','bf_tags','child_autoinc');	// used later
		$qb->where( array('=', 'bf_tags', 'bf_pid', $PID) );
		$res = $qb->getResult();

		// Define the tags and values and do any refinement here necessary
		$projectTags = array();
		while($row = $res->fetchRow()) {
			if (is_object($row)) {
				$projectTags[$row->name] = $row;
			}
		}

		/** Tag Syncing
 		 * Get the master projects values and stick them in our projectsTags if
		 * we have a project tag of similar name.
		 */
		if ($ourProject->sync != 0) {
			// Get the master values
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tid',	'bf_tags','id');
			$qb->addColumn('bf_name',	'bf_tags','name');
			$qb->addColumn('bf_value',	'bf_tags','value');
			$qb->addColumn('bf_parent',	'bf_tags','padding');
			$qb->addColumn('bf_autoinc','bf_tags','master_autoinc');
			$qb->where( array('=', 'bf_tags', 'bf_pid', $ourProject->sync) );
			$res = $qb->getResult();

			// Overwrite the projects tag with the masters			
			while($row = $res->fetchRow()) {
				if (is_object($row) && isset($projectTags[$row->name])) {

					// update the important fields and remember the childs autoinc setting
					$projectTags[$row->name]->value = $row->value;
					$projectTags[$row->name]->autoinc = $row->master_autoinc;
					$projectTags[$row->name]->master_id = $row->id;

					// Update the value in case of Master Y Child N
					if ($projectTags[$row->name]->autoinc == 'Y' && $projectTags[$row->name]->child_autoinc == 'N') {
						--$projectTags[$row->name]->value;
					}

					// Write these tags BACK to the database
					$qb1 = BFQueryBuilder::update(null, TABLE_LOCK);
					$qb1->setColumn('bf_value', 'bf_tags', $projectTags[$row->name]->value);
					$qb1->where(
						array (
							array('=', 'bf_tags',  'bf_pid', $PID),
							array('=', 'bf_tags',  'bf_name', $row->name) // no way for us to know child id
							)
						);

					if (DB::isError($qb1->getResult())) { return false; }				

				}
			}

		}

		// Reorder the tags so they are longest to shortest - so $BOB will parse before $BO
		uksort($projectTags, array($this, 'tagLength'));

		// Parse out the various built in tags - So $BF_D will parse out before $B
		foreach($builtInTag as $k=>$v) {
			$returnTags[$k] = $v;
			$workingTag = preg_replace("/\\\$".$k."/", $v, $workingTag);
		}

		// Parse out the users project tags - Giggity
		foreach($projectTags as $ourTag) {

			// Add Padding
			$paddedValue = $ourTag->value;
			if ($ourTag->padding != '0') {
				for($t = strlen($paddedValue); $t < $ourTag->padding; $t++ , $paddedValue = '0'.$paddedValue);
			}

			// Write back just the Incremented Tag
			if ($ourTag->child_autoinc == 'Y') {
				$qb1 = BFQueryBuilder::update(null, TABLE_LOCK);
				$qb1->setColumn('bf_value', 'bf_tags', ++$ourTag->value);
				$qb1->where(
					array (
						array('=', 'bf_tags',  'bf_pid', $PID),
						array('=', 'bf_tags',  'bf_tid', $ourTag->id)
						)
					);

				if (DB::isError($qb1->getResult())) { return false; }
			}

			// when TagSyncing AND autoinc, update the master value
			if ($ourProject->sync != 0 && $ourTag->autoinc == 'Y') {
				$qb2 = BFQueryBuilder::update(null, TABLE_LOCK);
				// Only autoinc the value in the master, if it wasn't inc'd in the child. So YY tags only get inc'd once.
				$qb2->setColumn('bf_value', 'bf_tags', (($ourTag->child_autoinc == 'Y')?$ourTag->value:++$ourTag->value));
				$qb2->where(
					array (
						array('=', 'bf_tags',  'bf_pid', $ourProject->sync),
						array('=', 'bf_tags',  'bf_tid', $ourTag->master_id)
						)
					);
				if (DB::isError($qb2->getResult())) { return false; }
			}

			$returnTags[$ourTag->name] = $paddedValue;
			$workingTag = preg_replace("/\\\${$ourTag->name}/", $paddedValue, $workingTag);
		}

		// Handle any %{ISO} date/time stuff in the tag
		$workingTag = strftime($workingTag, time());

		return array($workingTag, $returnTags);
	}
	
	/** parseTag($ProjectID, $workingTag, $projectTags)
	 *  This function does the job of parsing out the tag information from the
	 *  manually entered form - its different enough that trying to merge them 
	 *  into one function was painful	 	 
	 */	
	private function parseManualTag($PID, $workingTag, $projectTags) {
		// Get our projects tag info, but use the value from the form
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tid',	'bf_tags','id');
		$qb->addColumn('bf_name',	'bf_tags','name');
		$qb->addColumn('bf_parent',	'bf_tags','padding');
		$qb->addColumn('bf_autoinc','bf_tags','autoinc');
		$qb->where(	array('=', 'bf_tags', 'bf_pid', $PID) );
		$res = $qb->getResult();

		// Define the tags as they are in the database, sans value
		while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$projectTags[$row['name']] = array_merge($row,array('value'=>$projectTags[$row['name']]));
		}

		// make sure all tags play by the same rules
		$fixTags = array();
		foreach($projectTags as $k=>$v) {
			if (is_array($v)) {
				$fixTags[$v['name']] = $v;
			} else {
				$fixTags[$k] = array( 'value'=>$v, 'padding'=>'0', 'autoinc'=>'N', 'name'=>$k );
			}
		}
		$projectTags = $fixTags;

		// Reorder the tags so they are longest to shortest - so $BOB will parse before $BO
		uksort($projectTags, array($this, 'tagLength'));

		// Parse out the users project tags - Giggity
		foreach($projectTags as $tagKey=>$ourTag) {
			// Add Padding
			$paddedValue = $ourTag['value'];
			if ($ourTag['padding'] != '0') {
				for($t = strlen($paddedValue); $t < $ourTag['padding']; $t++ , $paddedValue = '0'.$paddedValue);
				$projectTags[$tagKey]['value'] = $paddedValue;
			}
			// Increment Tag if needed
			if ($ourTag['autoinc'] == 'Y') {
				$qb = BFQueryBuilder::update(null, TABLE_LOCK);
				$qb->setColumn('bf_value', 'bf_tags', ++$ourTag['value']);
				$qb->where(	array (
						array('=', 'bf_tags',  'bf_pid', $PID),
						array('=', 'bf_tags',  'bf_tid', $ourTag['id'])
					));
				if (DB::isError($qb->getResult())) { return false; }
			}
			$workingTag = preg_replace("/\\\${$ourTag['name']}/", $paddedValue, $workingTag);
		}

		// Handle any %{ISO} date/time stuff in the tag
		$workingTag = strftime($workingTag, time());

		return array($workingTag,$projectTags);
	}
	
	
	
	// sort function used only by parseTag
	private function tagLength($a, $b) {
		if (strlen($a) == strlen($b)) {
			return 0;
		}
		return (strlen($a) > strlen($b))?-1:1;
	}

	/** getEnfironment
	 *	Get the full environment based on EID and create an array for it
	 */	 	
	private function getEnvironment($EID) {
		$env = array();
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_param', 'bf_env', 'key');
		$qb->addColumn('bf_value', 'bf_env', 'value');
		$qb->addColumn('bf_action','bf_env', 'action');
		$qb->addColumn('bf_mode',  'bf_env', 'mode');
		$qb->where(array(
			array('=', 'bf_env', 'bf_bid', $EID),
			array('!=','bf_env', 'bf_eid', '0'),
		));
		$qb->orderBy('ASC', 'bf_eid');
		$res = $qb->getResult();
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			// Add the environment - force to default if |'d input
			$work = preg_split("/\|/",$row['value']);
			$env[] = array(
				"key"		=> $row['key'],
				"value"		=> $work[0],
				"action"	=> $row['action'],
				"mode"		=> $row['mode']
			);
		}
		return $env;
	}

	/** getManualEnvironment
	 * Gets the full environment based on the EID and the form that you submitted for a manual start
	 * Due to the way env's are stored some of this information may end up being wrong - but I am
	 * doing my best to pull the info out.
	 * I MUST get both the action and the mode from the bf_env table based on eid and bid..
	 * 	  
	 * Its a bit of a hack, but it works for now - the form input names will look something like this;
	 * name="environment[4579906e834651_1_2_OSName]	  
	 * which is;
	 * name="environment[{parent token/unique key id(ignore it)}_{bf_env::bid}_{bf_env::eid}_{bf_env::param}]	 
	 */	 	
	private function getManualEnvironment($PID, $env, $saveenv) {
		$results = array();
		if (is_array($env)) {
			foreach ($env as $k=>$v) {
				if(strlen($v) > 255) {
					next;
				}	
				
				list($oneUP, $bid, $eid, $key) = split("_", $k, 4);
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_action', 'bf_env');
				$qb->addColumn('bf_value', 'bf_env');
				$qb->addColumn('bf_mode', 'bf_env');
				$qb->where(array(
					array('=', 'bf_env', 'bf_bid', $bid),
					array('!=', 'bf_env', 'bf_eid', '0'),
					array('=', 'bf_env', 'bf_eid', $eid),
					array('=', 'bf_env', 'bf_param', $key)
				));

				$res = $qb->getResult();
				$row = $res->fetchRow(DB_FETCHMODE_ASSOC);
				
				// Add to the results array this set
				$results[] = array(
					'key' 		=> $key,
					'value' 	=> $v,
					'origvalue'	=> (isset($row['bf_value']))?$row['bf_value']:'',
					'action'	=> (isset($row['bf_action']))?$row['bf_action']:'S',
					'mode'		=> (isset($row['bf_mode']))?$row['bf_mode']:'N',
				);

				// If we are supposed to save the env, and it's original value isn't a pipe delimited list...
				// then save it.
				if ($saveenv == 'save' && strpos($row['bf_value'], '|') === false
						&& $GLOBALS['controller']->userHasAccessTo('EditEnvValue')) {
					$eb = BFQueryBuilder::update();
					$eb->setColumn('bf_value', 'bf_env', $v);
					$eb->where(array(
						array('=', 'bf_env', 'bf_bid', $bid),
						array('=', 'bf_env', 'bf_param', $key)
					));
					$eb->getResult();
					// TODO Erro checking
				}
			}

			// origvalue must be expanded if expand_list_vars is set.
			// otherwise the 'mustchange' is not honored as values differ.
			$expand = $GLOBALS['controller']->getSysconfig('expand_list_vars');
			if ($expand == 'Y') {
				$indexes = array();
				foreach ($results as $idx => $arr) {
					$indexes[$arr['key']] = $idx;
				}

				foreach ($results as $idx=>$arr) {
					$value = $arr['origvalue'];
					// prevent recursive loop references. 5 times, then end.
					for ($i=0; $i<=5; $i++) {
						$list = preg_split("/\|/", $value);
						if (strpos($list[0], "$") === 0) {
							$var = substr($list[0],1);
							$value = $results[$indexes[$var]]['value'];
						} else {
							break;
						}
					}
					$results[$idx]['origvalue'] = $value;
				}
			}
		}		
		return $results;
	}

	/* getSteps
	*	This function does the work of making the inclusion/exclusion list for the
	* environment block
	*/
	private function getSteps($PID) {
		$steps = array();
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_uid',	'bf_steps', 'uid');
		$qb->where(array(
			array('=',	'bf_steps',	'bf_pid',	$PID),
			array('!=',	'bf_steps',	'bf_active','N')
		));
		$res = $qb->getResult();
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$steps[] = '+'.$row['uid'];
		}
		return $steps;
	}






}


?>
