<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFStep
 * @package Console
 * @subpackage Projects
 */

if (!defined('TABLE_LOCK')) { define('TABLE_LOCK', true); }

$GLOBALS['controller']->require_module('BFDBO');
$GLOBALS['controller']->require_module('BFStore');
$GLOBALS['controller']->require_module('BFUID');
$GLOBALS['controller']->require_module('BFNotes');

class BFStep extends BFDBO {

	public $table		= 'bf_steps';
	public $primary_key	= 'bf_uid';
	public $descriptor_key = 'bf_description';
	public $primary_key_type_override = 'step';
	protected $field_defaults = array('bf_inline'=>'N');
	protected $required_fields = array(	'bf_pid', 'bf_description', 'bf_dir', 'bf_command' );
	protected $bfstores = array('bf_command' => null);
	public $type_name = 'Step';
	public $field_descriptions = array(
			'bf_uid' => '',
			'bf_pid' => '',
			'bf_sid' => 'sequence number',
			'bf_eid' => 'environment',
			'bf_did' => 'result',
			'bf_cid' => 'inline',
			'bf_description' => 'name',
			'bf_dir' => 'directory',
			'bf_pnotify' => 'pass notify',
			'bf_fnotify' => 'fail notify',
			'bf_onfail' => 'on fail',
			'bf_resource' => 'selector',
			'bf_canbg' => 'thread',
			'bf_absolute' => 'absolute path',
			'bf_timeout' => 'timeout',
			'bf_inline' => '',
			'bf_level' => 'access',
			'bf_dist' => 'broadcast',
			'bf_command' => 'command',
			'bf_nchange' => '',
			'bf_pcid' => 'pass chain',
			'bf_fcid' => 'fail chain',
			'bf_active' => 'active',
			'bf_passwait' => 'pass wait',
			'bf_failwait' => 'fail wait'
			);

	protected $recursion = false;

	/** Save 
	 * Overridden to check for duplicate names and check for permission
	 */
	public function save($checkfields = true) {	
		//$this->inserted=1 means we're doing an update, otherwise its a create.
		if (($this->controller->userHasAccessTo('AddProjectStep') && !$this->inserted) ||
				($this->controller->userHasAccessTo('UpdateProjectStep') && $this->inserted)) {
			
			// check for variable issues.
			if ($checkfields)  {
				if (!$this->fields['bf_description'])  {
					throw new BFDBException(translate('2256', translate('step'), translate('name')));
				}
				if (preg_match('/^\s+$/', $this->fields['bf_description']))  {
					throw new BFDBException(translate('2000'));
				}
				if (strlen($this->fields['bf_description']) > 255)  {
					throw new BFDBException(translate('2001'));
				}
				if (!$this->fields['bf_dir'])  {
					throw new BFDBException(translate('2004'));
				}
				if (preg_match('/^\s+$/', $this->fields['bf_dir']))  {
					throw new BFDBException(translate('2005'));
				}
				if (!$this->bfstores['bf_command']['TEXT'])  {
					throw new BFDBException(translate('2007'));
				}
				if (preg_match('/^\s+$/', $this->bfstores['bf_command']['TEXT']))  {
					throw new BFDBException(translate('2008'));
				}
			}
			// check for duplicate names
			$qb = BFQueryBuilder::select();
			$qb->where(array(
					array('=', $this->table, 'bf_pid',	$this->fields['bf_pid']),
					array('=', $this->table, 'bf_description', $this->fields['bf_description']),
					array('!=', $this->table, 'bf_uid', $this->fields['bf_uid'])
					));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
				return false;
			} else if (intval($count) > 0) {
				throw new BFException(translate('2195', translate('step')));
				return false;
			}
				
			// if sid = nothing, then put it at the bottom
			if ($this->fields['bf_sid'] == '') {
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_sid', 'bf_steps', 'max', 'MAX');
				$qb->where(array('=', 'bf_steps', 'bf_pid', $this->fields['bf_pid']));
				$this->set('bf_sid', ($qb->getOne(false) + 1));			
			}

			// save it			
			if (parent::save()) {
				if (!empty($_REQUEST['abovestep']) && !$this->recursion ) { 

					// Now perform a move to where it should live relative to where it was actually created and the target is currently
					// This is confusing so feel free to ask.
					$this->moveStep($this->fields['bf_pid'], $this->get('bf_uid'), ((int)$_REQUEST['abovestep'] - $this->get('bf_sid')));
				}
				return true;
			} 
			throw new BFException(translate('2205'));
			return false;
		}
		if($this->inserted) {
			throw new BFAccessException('UpdateProjectStep');
		} else {
			throw new BFAccessException('AddProjectStep');
		}
		return false;	
	}

	/** Toggle Step Field
	 * Toggle a single field in a step to the choice given.	
	 * It doesn't actually know the possible states - and really is only here to check permissions
	 * Required: Project Id, Step Unique Id, Toggle Choice, and field to toggle.	 
	 */	 	 
	public function toggleStepField($PID, $SUID, $choice, $field) {
		if ($this->controller->userHasAccessTo('UpdateProjectStep')) {
			$tempStep = new BFStep(); 
			$tempStep->select(array('bf_pid','bf_uid'),array($PID, $SUID));
			$tempStep->set($field, $choice);
			$tempStep->save(false);									
			return true;			
		}
		throw new BFAccessException('UpdateProjectStep');
		return false;	
	}	

	/** Move Step
	 * Moves the step by a number of places either up or down
	 * It will need to adjust ALL steps to reflect the change.
	 * Required: Project ID, Step Unique ID, Distance to move +/-
	 */
	public function moveStep($PID, $SUID, $distance) {
		if ($this->controller->userHasAccessTo('UpdateProjectStep')) {
			if ($distance != '0') { 
				// Must know the max number of steps
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_sid', 'bf_steps', 'max', 'MAX');
				$qb->where(array('=', 'bf_steps', 'bf_pid', $PID));
				$maxStepSID = (int)$qb->getOne(false);
				
				// Get step current location			
				$stepCurrentSID = $this->get('bf_sid');
				
				// Determine the destination
				$stepDestinationSID = (int)$stepCurrentSID + (int)$distance;
				
				// May not move to a point beyond how many steps we have
				if ($stepDestinationSID > $maxStepSID) { $stepDestinationSID = $maxStepSID; }
	
				// May not move to a negative number, SID of 1 is as low as it goes.
				if ($stepDestinationSID < 1) { $stepDestinationSID = 1;	}
				
				if ($stepCurrentSID == $stepDestinationSID)  {
					return true;
				}

				// Set the current sid to max + 1 to prevent collisions
				$this->recursion = true;
				$this->set('bf_sid', $maxStepSID + 1);
				$this->save(false);
				
				// Determine direction of move and make it so
				if ($stepDestinationSID < $stepCurrentSID) {
					// Moving steps from a higher SID to a lower one	
					for ($t = ($stepCurrentSID-1); $t >= $stepDestinationSID; $t--) {
						$tempStep = new BFStep(); 
						$tempStep->select(array('bf_pid','bf_sid'),array($PID, $t));
						$tempStep->set('bf_sid', ($t+1));
						$tempStep->recursion = true;						
						$tempStep->save(false);
						// Now update any templates set to this step...
						$template_qb = BFQueryBuilder::update();
						$template_qb->setColumn('bf_sid', 'bf_templates', $t+1);
						$template_qb->where(array(
							array('=', 'bf_templates', 'bf_pid', $PID),
							array('=', 'bf_templates', 'bf_sid', $t),
						));
						$template_qb->getResult();
					}
					$this->set('bf_sid', $stepDestinationSID);
					$this->save(false);
				} else {					
					// Moving steps from a lower SID to a higher one
					for ($t = ($stepCurrentSID+1); $t <= $stepDestinationSID; $t++) {					
						$tempStep = new BFStep(); 
						$tempStep->select(array('bf_pid','bf_sid'),array($PID, $t));
						$tempStep->set('bf_sid', ($t-1));
						$tempStep->recursion = true;						
						$tempStep->save(false);
						// Now update any templates set to this step...
						$template_qb = BFQueryBuilder::update();
						$template_qb->setColumn('bf_sid', 'bf_templates', $t-1);
						$template_qb->where(array(
							array('=', 'bf_templates', 'bf_pid', $PID),
							array('=', 'bf_templates', 'bf_sid', $t),
						));
						$template_qb->getResult();
					}
					$this->set('bf_sid', $stepDestinationSID);
					$this->save(false);
					// Now update any templates set to this step...
					$template_qb = BFQueryBuilder::update();
					$template_qb->setColumn('bf_sid', 'bf_templates', $stepDestinationSID);
					$template_qb->where(array(
						array('=', 'bf_templates', 'bf_pid', $PID),
						array('=', 'bf_templates', 'bf_sid', $stepCurrentSID),
					));
					$template_qb->getResult();
				}							
			}
			$this->recursion = false;
			return true;
		}
		$this->recursion = false;
		throw new BFAccessException('UpdateProjectStep');
		return false;	
	}	 	
	
	/** Clone Step
	 * Clones a step by simply editing the one we have now and inserting it
	 * then performing a move to its new position.  This means you should be
	 * able to clone a step to any point in the list.	 
	 * Required: Project ID, Step Unique ID to be cloned, Distance to move +/- from target	 	 	
	 */
	public function cloneStep($PID, $SUID, $distance) {
		if ($this->controller->userHasAccessTo('UpdateProjectStep')) {
			if ($this->controller->userHasAccessTo('AddProjectStep')) {
				if ($distance != '0') {
					// Get the Cloners position so we can work with it
					$distanceRelativeTo = $this->get('bf_sid');
								
					// Create new instance of BFStep
					$newStep = new BFStep();
					foreach($this->fields as $key=>$value) {
						if (is_int($key) && $value != 'bf_uid' && $value != 'bf_sid' && $value != 'bf_description') {
							$newStep->set($value,$this->get($value));
						}						
					}
				 	// Set a new SID for our new step
					$qb = BFQueryBuilder::select();
					$qb->addColumn('bf_sid', 'bf_steps', 'max', 'MAX');
					$qb->where(array('=', 'bf_steps', 'bf_pid', $PID));
					$newStep->set('bf_sid', ($qb->getOne(false) + 1));
					
					// Get a list of all step names in this project
					$qb = BFQueryBuilder::select();
					$qb->addColumn('bf_description', 'bf_steps');
					$qb->where(array(
						array('=', 'bf_steps', 'bf_pid', $PID)
					));
					$names = $qb->getCol();
					
					// Uniquely name the new step ( I hate this but.. feh. )
					$maxTries = 100000;
					for($i = 1; $i <= $maxTries; $i++) {
						$newName = $this->get('bf_description') .' '. $i;
						$match = false;
						foreach($names as $v) {
							if ($v == $newName) {
								$match = true;
							}
						}
						if ($match == false) {
							break;
						}
					}
					$newStep->set('bf_description', $newName);				
					
					// Insert it
					if ($newStep->save()) {
						// Due to how we calculate distance to travel										
						if ($distance < 0) { $distance++; }						
						// Now perform a move to where it should live relative to where it was actually created and the target is currently
						// This is confusing so feel free to ask.
						$newStep->moveStep($PID, $newStep->get('bf_uid'), (((int)$distanceRelativeTo + (int)$distance) - $newStep->get('bf_sid')));
						return true;
					} 
					throw new BFException(translate('2205'));
					return false;
				}
			}
			throw new BFAccessException('AddProjectStep');
			return false; 		
		}
		throw new BFAccessException('UpdateProjectStep');
		return false;	
	}
	
	/** Delete Step 	 	
	 * Over Ride to delete Notes and crap
	 */
	public function deleteStep($PID, $SUID) {
		// Do stuff here.. 
		if ($GLOBALS['controller']->userHasAccessTo('DeleteProjectStep')) {
			// First move the step to the bottom of the list (so all other steps slide up)
			$this->moveStep($PID, $SUID, 100000);
		
			// Delete the associated Notes
			$notes = new BFNotes();
			$notes->noteType = 'ProjectStep';
			$notes->deleteNote('All',$PID, $SUID);
						
			return parent::delete();
		}
		throw new BFAccessException('DeleteProjectStep');
		return false;	
	}	 	
	
	/**
	 * Gets the project name for the project this step belongs to.
	 */
	public function getProjectName()  {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_description', 'bf_projects', 'name');
		$qb->where(array('=', 'bf_projects', 'bf_pid', $this->fields['bf_pid']));
		$name = $qb->getOne();
		return DB::isError($name) ? '' : $name;
	}
	
	public function logChange($type)  {
		if ($this->log_changes && ($type != 'Updated' || 
				$this->controller->getSysconfig('log_all_changes') != 'Y'))  {
			$this->controller->logAudit('Step' . $type, array($this->fields[$this->descriptor_key], 
					$this->getProjectName()));
		}
	}

	/**
	 * A BFDBO-centered logging function used to log before-and-after state on changed fields.  
	 */
	public function logChangedFields($field, $before, $after)  {
		$fieldname = isset($this->field_descriptions[$field]) ? 
				($this->field_descriptions[$field] ? translate($this->field_descriptions[$field]) : '') : 
				$field;
		if ($this->log_changes && $this->inserted && $fieldname &&
				$this->controller->getSysconfig('log_all_changes') == 'Y')  {
			$this->controller->logAudit($this->getObjectTypeName() . 'UpdatedField', array($fieldname,
					$this->fields[$this->descriptor_key], $this->getProjectName(), $before, $after));
		}
	}
}

?>
