<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFStore
 * A database object that does the reading/writing from the bf_store table
 *
 * @package Console
 * @subpackage Core
 */


$GLOBALS['controller']->require_module('BFDBO');
$GLOBALS['controller']->require_module('BFUID');

class BFStore extends BFDBO {
	public $table		= 'bf_store';
	public $primary_key = 'bf_uid';

	/** getStorage
	 * returns the UID, PART Number (or blank of more than one), the imploded TEXT and the NUMber of parts
	 */
	public function getStorage($uid, $part = '') {

		$qb = BFQueryBuilder::select();
		if ($part == '') {
			$qb->where(array(
				array('=', $this->table, $this->primary_key, $uid)
			));
		} else {
			$qb->where(array(
				array('=', $this->table, $this->primary_key, $uid),
				array('=', $this->table, 'bf_part', $part)
			));
		}
		$res = $qb->getResult();

		if (DB::isError($res)) {
			$this->controller->addError(translate('2225'));
			return false;
			}
		$text = array();
		$toHex = false;
		while ($row = $res->fetchRow()) {
			$toHex	= $row->bf_hex;
			$text[$row->bf_part] = $row->bf_text;
		}
		ksort($text);
		$returnText = (is_array($text))?implode('', $text):$text;
		if ($toHex) { $returnText = pack("H*", $returnText); }

		return array(
			'UID'   => $uid,
			'PART'  => $part,
			'TEXT'  => $returnText,
			'NUM'   => count($text)
		);
	}

	/** setStorage
	 * does the work of chopping up a given blob into bits and stores it in the DB
	 * if UID is set it will update it by deleting the old one and inserting the new one
	 * using the same UID.  This is because we cant know how long the old one might have been
	 * parts wise, and its just easier this way.
	 */
	public function setStorage($text, $uid = '') {

		$hex = 1;  // hex encode everything until I figure out what is supposed to be and whats not
		// Where do I get this?

		// GET A NEW ID FROM BFUID IF ONE ISN'T PROVIDED
		if ($uid == '') {
			$newUID = new BFUID();
			$uid = $newUID->getUID();
		}

		// Convert the text to hex
		if ($hex) {
			$hextext = unpack("H*hex", $text);
			$text = $hextext['hex'];
			// Random NULLs at the end = bad
			$len = strlen($text);
			if ($len > 0 && $text{$len - 1} == '0') {
				$text = substr($text, 0, -1);
			}
		}

		// split the text into bits.
		$text = str_split($text, 4000);

		// Delete ALL of the previous stored objects with this UID
		$res = $this->removeStorage($uid);
		if (! $res) { return false; }

		// Now insert all of the new objects with this UID
		foreach ($text as $k=>$v) {
			$qb = BFQueryBuilder::insert();
			$qb->setColumn('bf_uid',  $this->table, $uid);
			$qb->setColumn('bf_part', $this->table, $k);
			$qb->setColumn('bf_hex',  $this->table, $hex);
			$qb->setColumn('bf_text', $this->table, $v);
			$res = $qb->getResult();

			if (DB::isError($res)) {
				$this->controller->addError(translate('2226'));
				return false;
			}
		}
		return $uid;
	}

	/** removeStorage
	 * Removes the storage with the uid of $UID
	 */
	public function removeStorage($UID) {
		// Delete ALL of the previous stored objects with this UID
		$qb = BFQueryBuilder::delete();
		$qb->where(array(
			array('=', $this->table, 'bf_uid', $UID)
		));		
		$res = $qb->getResult();
		
		if (DB::isError($res)) {
			$this->controller->addError(translate('2227'));
			return false;
		}
		return true;
	}

}

?>
