<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFTemplate.php
 * 
 * @package Console
 * @subpackage Templates
 */

/**
 * BFTemplate
 */
class BFTemplate extends BFDBO {
	public $table =			'bf_templates';
	public $primary_key =	'bf_name';
	public $descriptor_key = 'bf_description';
	public $stored_values = array('bf_name' => null, 'bf_pid' => null, 'bf_sid' => null, 'bf_lang' => null);
	protected $bfstores = array('bf_body' => null);
	public $type_name = 'Template';
	public $field_descriptions = array(
			'bf_pid' => 'project',
			'bf_name' => 'key',
			'bf_description' => 'description',
			'bf_frm' => 'from',
			'bf_subj' => 'subject',
			'bf_body' => 'body',
			'bf_sid' => 'step',
			'bf_lang' => 'language'
			);

	/**
	 * Overriden to check for an existing interface with that project ID and step ID
	 */
	public function save() {
		// Only check this if things have changed...
		if ($this->fields['bf_name'] != $this->stored_values['bf_name']
				|| $this->fields['bf_pid'] != $this->stored_values['bf_pid']
				|| $this->fields['bf_sid'] != $this->stored_values['bf_sid']
				|| $this->fields['bf_lang'] != $this->stored_values['bf_lang']) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_name', $this->table, 'bf_name', 'COUNT');
			$qb->where(array(
							array('=', $this->table, 'bf_name', $this->fields['bf_name']),
							array('=', $this->table, 'bf_pid', $this->fields['bf_pid']),
							array('=', $this->table, 'bf_sid', $this->fields['bf_sid']),
							array('=', $this->table, 'bf_lang', $this->fields['bf_lang'])
						));
			$count = $qb->getOne();
			if (DB::isError($count)) {
				throw new BFDBException($count->getMessage());
				return false;
			} else if (intval($count) > 0) {
				throw new BFException(translate('2206'));
				return false;
			}
		}
		if ($this->stored_values['bf_pid'] == 0
			&& $this->fields['bf_pid'] != $this->stored_values['bf_pid']) {
			$this->inserted = false;
		}
		return parent::save();
	}
	
	/**
	 * Overridden for access check
	 */
	public function insert() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('AddTemplate')) {
			return parent::insert();
		} else {
			throw new BFAccessException('AddTemplate');
		}
	}
	
	/**
	 * Overridden for access check and multiple keys...
	 */
	public function update() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('UpdateTemplate')) {
			if (!$this->hasRequiredFields()) {
				// Catch this and handle it sanely per-object
				throw new BFException(translate('2181'));
			}
			$qb = BFQueryBuilder::update();
			
			// No sense in sending empty queries. Just return true to keep everyone happy
			$updated = false;
			foreach ($this->tablefields as $field) {
				if (array_key_exists($field, $this->updated) && $this->updated[$field]) {
					if (!$updated) {
						$updated = true;
					}
					// If a bfstored value, need to save that first to get the ID
					if (array_key_exists($field, $this->bfstores)) {
						$this->fields[$field] = $this->bfstore->setStorage($this->bfstores[$field]['TEXT'], $this->bfstores[$field]['UID']);
					}
					$qb->setColumn($field, $this->table, $this->fields[$field]);
				}
			}
			
			if ($updated) {
				$qb->where(array(
								 array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]),
								 array('=', $this->table, 'bf_pid', $this->stored_values['bf_pid']),
								 array('=', $this->table, 'bf_sid', $this->stored_values['bf_sid']),
								 array('=', $this->table, 'bf_lang', $this->stored_values['bf_lang'])
								 ));
				$res = $qb->getResult();
				if (!DB::isError($res))  {
					$this->logChange('Updated');
					return true;
				}
				return false;
				
			} else {
				return true;
			}
		} else {
			throw new BFAccessException('UpdateTemplate');
		}
	}
	
	/**
	 * Overridden for access check and multiple keys...
	 */
	public function delete() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('DeleteTemplate')) {
			$qb = BFQueryBuilder::delete();
			$qb->where(array(
							array('=', $this->table, $this->primary_key, $this->stored_values[$this->primary_key]),
							array('=', $this->table, 'bf_pid', $this->stored_values['bf_pid']),
							array('=', $this->table, 'bf_sid', $this->stored_values['bf_sid']),
							array('=', $this->table, 'bf_lang', $this->stored_values['bf_lang'])
							));
			
			$res = $qb->getResult();
			
			if (DB::isError($res)) {
				throw new BFDBException($res->getMessage());
			} else {
				$this->logChange('Deleted');
				return true;
			}
		} else {
			throw new BFAccessException('DeleteTemplate');
		}
	}
}

?>
