<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFUID
 * A database object that creates the latest UID
 *
 * @package Console
 * @subpackage Core
 */

$GLOBALS['controller']->require_module('BFDBO');

class BFUID extends BFDBO {
	public $table	   = 'bf_uid';
	public $primary_key = 'bf_type';

	private $uidLock;
	private $uidType;

	/**
	 * getUID
	 * - attempt to pull the next UID for type $inType
	 * - do so in a nice and friendly manner for 5 seconds at which point, force the issue and take it!
	 */
	public function getUID($inType = 'generic') {
		global $GLOBAL_DATABASE;
		$this->uidType = (in_array($inType,$GLOBAL_DATABASE['valid_types']))?$inType:'generic';

		$lockObject = $this->setLock();
		if (!is_object($lockObject)) {
			$lockObject = $this->forceLock();
			if (!is_object($lockObject)) {
				throw new BFException(translate('2207'));
				die;
			}
		}
		$this->resetLock($inType);
		return $lockObject->bf_stamp.$this->uidType.$lockObject->bf_num;
	}

	/**
	 * resetLock
	 * - takes the locked object and unlocks it by setting the bf_lock value to zero.
	 */
	public function resetLock($inType = 'generic') {
		global $GLOBAL_DATABASE;
		$this->uidType = (in_array($inType,$GLOBAL_DATABASE['valid_types']))?$inType:'generic';

		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_lock', 'bf_uid', '0');
		$qb->where(array(
				array('=', 'bf_uid', 'bf_type', $this->uidType),
				array('=', 'bf_uid', 'bf_lock', $this->uidLock)
			));
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFException(translate('2208'));
			return false;
			}
		return true;
	}

	/**
	 * setLock
	 * - locks the entry we are working with so that no one else can come along and get the wrong UID
	 */
	protected function setLock() {
		// Attempt to get a lock on the db 5 times - 1 per second
		$result = '';
		for ($t=0;$t<5;$t++) {

			$num = 0;
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_num', 'bf_uid', 'bf_num', 'MAX');
			$qb->where(array('=', 'bf_uid', 'bf_type', $this->uidType));
			$num = $qb->getOne(false);

			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_lock', 'bf_uid', $this->uidLock);
			$qb->setColumn('bf_num', 'bf_uid', ++$num);
			$qb->where(array(
					array('=', 'bf_uid', 'bf_type', $this->uidType),
					array('=', 'bf_uid', 'bf_lock', '0')
				));
			$resA = $qb->getResult();

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_stamp', 'bf_uid', 'bf_stamp');
			$qb->addColumn('bf_num', 'bf_uid', 'bf_num');
			$qb->where(array(
					array('=', 'bf_uid', 'bf_type', $this->uidType),
					array('=', 'bf_uid', 'bf_lock', $this->uidLock),
				));

			$resB = $qb->getResult();

			if (DB::isError($resA) || DB::isError($resB)) {
				throw new BFException(translate('2209'));
				return false;
				}
			$result = $resB->fetchRow();
			if (is_object($result)) {
				// this is a good break - we want this break
				break;
				}
			usleep(10000);
		}
		return $result;
	}

	/**
	 * forceLock
	 * - after 5 seconds of attempting to get the uid the nice way, this is the brute force method
	 */
	protected function forceLock() {
		// If we haven't gotten anything at this point, then force the issue

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_num', 'bf_uid', 'bf_num', 'MAX');
			$qb->where(array('=', 'bf_uid', 'bf_type', $this->uidType));
			$num = $qb->getOne(false);

			$qb = BFQueryBuilder::update();
			$qb->setColumn('bf_lock', 'bf_uid', $this->uidLock);
			$qb->setColumn('bf_num', 'bf_uid', ++$num);
			$qb->where(array('=', 'bf_uid', 'bf_type', $this->uidType));
			$resA = $qb->getResult();

			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_stamp', 'bf_uid', 'bf_stamp');
			$qb->addColumn('bf_num', 'bf_uid', 'bf_num');
			$qb->where(array(
					array('=', 'bf_uid', 'bf_type', $this->uidType),
					array('=', 'bf_uid', 'bf_lock', $this->uidLock),
				));
			$resB = $qb->getResult();

		if (DB::isError($resA) || DB::isError($resB)) {
			throw new BFException(translate('2188'));
			return false;
			}
		$result = $resB->fetchRow();

		return $result;
	}

	function __construct() {
		parent::__construct();
		$s = explode(" ", microtime());
		$m = explode(".", $s[0]);
		$this->uidLock = (substr($s[1], 4, 4).substr($m[1], 0, 4));
		$this->uidType = 'generic';
	}
}

?>
