<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * BFUser
 * A database object providing access to CRUDing the users stored in the
 * database. It also provides access to the user preferences.
 * @package Console
 * @subpackage Users
 */

$GLOBALS['controller']->require_file('Module.php');
$GLOBALS['controller']->require_module('BFLDAP');

class BFUser extends BFDBO {
	public $table	   = 'bf_users';
	public $primary_key = 'bf_userid';
	public $descriptor_key = 'bf_login';
	protected $preferences;
	protected $required_fields = array('bf_userid', 'bf_name', 'bf_password', 'bf_login');
	protected $field_defaults = array(
									'bf_level' => 0,
									'bf_name' => '',
									'bf_email' => '',
									'bf_login' => '',
									'bf_password' => '',
									'bf_maxbuilds' => 0,
									'bf_session' => 0,
									'bf_pupdate' => 0,
									'bf_pexpire' => '1',
									'bf_pwexpired' => 'N',
									'bf_tzone' => '',
									'bf_datefmt' => 'short',
									'bf_priority' => '',
									'bf_lang' => '',
									'bf_tiplevel' => 0,
									'bf_data' => ''
									);
	public $sessionid;
	protected $errors = array();
	protected $messages = array();
	protected $ldap = false;
	protected $type;
	public $type_name = 'User';
	public $field_descriptions = array(
			'bf_userid' => '',
			'bf_level' => '',
			'bf_name' => 'name',
			'bf_email' => 'email',
			'bf_login' => 'username',
			'bf_password' => '',
			'bf_maxbuilds' => 'limit',
			'bf_session' => '',
			'bf_pupdate' => '',
			'bf_pexpire' => 'password expires',
			'bf_pwexpired' => '',
			'bf_tzone' => 'time zone',
			'bf_datefmt' => 'date format',
			'bf_priority' => 'priority login',
			'bf_lang' => 'language',
			'bf_tiplevel' => '',
			'bf_data' => ''
			);

	/**
	 * Overwrite to keep LDAP users from changing most fields
	 */
	public function set($field, $value) {
		if ($this->inserted && $this->ldap && $field != 'bf_lang' && $field != 'bf_tzone' && $field != 'bf_datefmt' && $field != 'bf_priority') {
			return false;
		} else {
			return parent::set($field, $value);
		}
	}

	/**
	 * Override to ensure that the user's language always has some value.
	 */	
	public function get($field) {
		if ($field == 'bf_lang') {
			return $this->getLang();
		} else {
			return parent::get($field);
		}
	}

	public function getLang() {
		if (!empty($this->fields['bf_lang'])
				&& file_exists(BF_ROOT . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR . $this->fields['bf_lang'])) {
			return $this->fields['bf_lang'];
		} else {
			return $this->getBrowserLang();
		}
	}
	
	/**
	 * Laziness...abstract what defines a user as root
	 */
	public function isRoot() {
		return (int)$this->fields['bf_userid'] === 1;
	}

	/**
	 * Simple method to try guessing the user's language preference from the request headers
	 * Example:
		[Accept-Language] => eo,en;q=0.7,en-us;q=0.3
	 * Firefox 2.0b requested Esperanto, English, English_US
	 */
	public function getBrowserLang() {
		global $LANG_DEFAULT, $LANG_LOCALES;

		$lang_header = $this->controller->getRequestHeader('Accept-Language');
		$lang_split = preg_split('/[.;,=.0-9]/', $lang_header, -1, PREG_SPLIT_NO_EMPTY);
		foreach ($lang_split as $option) {
			// Force correct formatting
			$option = str_replace('-', '_', $option);
			// Prefix if the full string fails to match
			$prefix = $option;
			if (strlen($option) == 5) {
				$option[3] = strtoupper($option[3]);
				$option[4] = strtoupper($option[4]);
				$prefix = $option[0] . $option[1];
			}
			$langpath = BF_ROOT . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR . $option;
			if (is_dir($langpath)
					&& file_exists($langpath . DIRECTORY_SEPARATOR . 'language.php')) {
				return $option;
			}
			if (isset($LANG_LOCALES[$prefix])) {
				$langpath = BF_ROOT . DIRECTORY_SEPARATOR . 'lang' . DIRECTORY_SEPARATOR . $LANG_LOCALES[$prefix];
				if (is_dir($langpath)
						&& file_exists($langpath . DIRECTORY_SEPARATOR . 'language.php')) {
					return $LANG_LOCALES[$prefix];
				}
			}
		}
		
		// Use the default if it comes to it
		return $LANG_DEFAULT;
	}

	/**
	 * A method written because of old Java habits
	 */
	public function isLDAPUser() {
		return $this->ldap;
	}

	public function hasLicenseSeat() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_userid', 'bf_sessions', 'bf_userid');
		$qb->where(array('=', 'bf_sessions', 'bf_userid',  $this->get('bf_userid')));
		$res = $qb->getOne();
		if (DB::isError($res)) {
			return false;
		} else {
			return $res;
		}
	}

	/**
	 * Since the BFUser object manages preferences, it also needs to allow access to them. But only
	 * if they exist.
	 */
	public function getPreference($name) {
		if (!is_array($this->preferences)) {
			$this->loadPreferences();
		}
		foreach ($this->preferences as $pref) {
			if ($pref->get('bf_name') == $name) {
				return $pref->get('bf_value');
			}
		}

		return false;
	}

	/**
	 * Cycle through the known preferences, looking for the one referenced in the parameter.
	 * If not found, make a new one and save that instead. Takes two strings as paramters.
	 */
	public function setPreference($name, $value) {
		if (!is_array($this->preferences)) {
			$this->loadPreferences();
		}
		foreach ($this->preferences as $pref) {
			if ($pref->get('bf_name') == $name) {
				$pref->set('bf_value', $value);
				return $pref->save();
			}
		}

		// If you made it this far, you need a new preference entry
		$pref = new BFPreference();
		$pref->set('bf_userid', $this->get('bf_userid'));
		$pref->set('bf_name',   $name);
		$pref->set('bf_value',  $value);

		return $pref->save();
	}

	/**
	 * Quick check to make sure that the password (if specified) meets standards
	 */
	public function save() {
		if (!empty($this->fields[BF_FIELD_PREFIX . 'password'])
				&& $this->fields[BF_FIELD_PREFIX . 'password'][0] != '!'
				&& (!$this->inserted || $this->updated[BF_FIELD_PREFIX . 'password'])) {
			$valid = self::validPassword(bf_decrypt($this->fields[BF_FIELD_PREFIX . 'password']));
			if ($valid !== true) {
				$GLOBALS['controller']->addError($valid);
				return false;
			}
		}
		// Check for dupes
		$qb = BFQueryBuilder::select();
		$qb->addColumn(BF_FIELD_PREFIX . 'login', $this->table, BF_FIELD_PREFIX . 'login', 'COUNT');
		$qb->where(array(
						array('=', $this->table, BF_FIELD_PREFIX . 'login', $this->fields[BF_FIELD_PREFIX . 'login']),
						array('!=', $this->table, BF_FIELD_PREFIX . 'userid', $this->fields[BF_FIELD_PREFIX . 'userid'])
						));
		$count = $qb->getOne();
		if (DB::isError($count)) {
			throw new BFDBException($count->getMessage());
		} else if (intval($count) > 0) {
			$GLOBALS['controller']->addError(translate('2210'));
			return false;
		} else {
			return parent::save();
		}
	}

	/**
	 * Overridden for access check
	 */
	public function insert() {
		// License seat available?
		if ($this->controller->getSysconfig('feature_fixedusers') == '1'
				&& $this->controller->getUsersCount() > (int)$this->controller->getSysconfig('lic_users')) {
			$this->controller->addError(translate('user limit reached'));
			return false;
		}
		// Access check
		if ($this->controller->userHasAccessTo('AddUser')) {
			// No matter what, generate one.
			$this->generatePrimaryKey(true);
			return (parent::insert() && $this->addToDefaultGroups());
		} else {
			throw new BFAccessException('AddUser');
		}
	}

	/**
	 * Add the user to any group marked as a default
	 */
	protected function addToDefaultGroups() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_level', 'bf_levels', 'level');
		$qb->where(array('=', 'bf_levels', 'bf_def', 'Y'));
		$levels = $qb->getCol();
		$this->controller->require_module('BFLevel');
		foreach ($levels as $level) {
			$bflevel = new BFLevel();
			$bflevel->select($bflevel->primary_key, $level);
			$pk = $this->fields[$this->primary_key];
			$this->controller->sudo(
								    array(&$bflevel, 'addMember'),
								    array($pk)
								    );
			$this->controller->sudo(
										   array(&$bflevel, 'save'),
										   array()
										   );
		}
		return true;
	}

	/**
	 * Overridden for access check and modification of access level memberships
	 */
	public function update() {
		// Access check
		if ($GLOBALS['controller']->userHasAccessTo('UpdateUser') || $this->get($this->primary_key) == $GLOBALS['user']->get($this->primary_key)) {
			return parent::update();
		} else {
			throw new BFAccessException('UpdateUser');
		}
	}

	public function addMemberOf($groups) {
		$this->controller->require_module('BFLevel');
		foreach ($groups as $groupid)  {
			$level = new BFLevel();
			$level->select($level->primary_key, $groupid);
			$level->addMember($this->get($this->primary_key));
			try {
				$level->save();
			} catch (Exception $e) {
				$this->controller->addError($e->getMessage());
				return false;
			}
		}

		return true;
	}

	public function deleteMemberOf($groups) {
		$this->controller->require_module('BFLevel');
		foreach ($groups as $groupid)  {
			$level = new BFLevel();
			$level->select($level->primary_key, $groupid);
			$level->deleteMember($this->get($this->primary_key));
			try {
				$level->save();
			} catch (Exception $e) {
				$this->controller->addError($e->getMessage());
				return false;
			}
		}

		return true;
	}

	/**
	 * Basically just a method alias to BFDBO::select() make things a little easier.
	 */
	protected function loadBySession($session) {
		// If empty, don't even try
		if (empty($session)) {
			return false;
		}

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_userid', 'bf_sessions', 'userid');
		$qb->addColumn('bf_type', 'bf_sessions', 'type');
		$qb->where(array('=', 'bf_sessions', 'bf_id', $session));
		$session_record = $qb->getRow();

		if (DB::isError($session_record)) {
			throw new BFDBException($session_record->getMessage());
		}

		$sid = isset($session_record['userid']) ? (int)$session_record['userid'] : 0;

		if ($sid > 0 && $this->select('bf_userid', $session_record['userid'])) {
			$this->sessionid = $session;
			return $this->serveCookie();
		} else {
			return false;
		}
	}

	/**
	 * Calls the appropriate type of loadByCredentials method as decided by the request type.
	 */
	protected function loadByCredentials() {

		switch ($this->controller->request_type) {
			case BF_REQUEST_XHTML:
				return $this->authFromRequest();
			case BF_REQUEST_RSS2:
				return $this->authFromHTTP();
		}
	}

	protected function authFromRequest() {
		global $_REQUEST;

		$ldap = null;
		if (array_key_exists('domain', $_REQUEST)) {
			$ldap = new BFLDAP();
			// Only work with valid LDAP domains.
			if (!$ldap->select(BF_FIELD_PREFIX . 'domain', $_REQUEST['domain'])) {
				$ldap = null;
			}
		}

		if (array_key_exists('username', $_REQUEST) && array_key_exists('password', $_REQUEST)) {
			return $this->login($_REQUEST['username'], $_REQUEST['password'], $ldap);
		}
		return false;
	}

	/**
	 * Use HTTP Authentication to authenticate the user.
	 *
	 * ISSUE: LDAP users, when their passwords do not get saved in the database, must add
	 * DOMAIN\name to the username field in the authentication dialog.
	 */
	protected function authFromHTTP() {
		global $_SERVER;

		if (array_key_exists('PHP_AUTH_USER', $_SERVER) && array_key_exists('PHP_AUTH_PW', $_SERVER)) {
			// Lazy hack to always have the username first
			@list($username, $domain) = array_reverse(explode('\\', $_SERVER['PHP_AUTH_USER'], 2));
			$ldap = null;
			if (!empty($domain)) {
				$ldap = new BFLDAP();
				// Only work with valid LDAP domains.
				if (!$ldap->select(BF_FIELD_PREFIX . 'domain', $domain)) {
					$ldap = null;
				}
			}
			return $this->login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'], $ldap, false);
		}
		return false;
	}

	/**
	 * Very similar to loadByCredentials, but authenticates against a specified LDAP
	 * server and then creates the user's record/session pair in the database from
	 * the returned results.
	 */
	protected function loadByLDAP($nopass = null) {
		global $_REQUEST, $controller, $LANG_DEFAULT;
		// If we have everything we need to attempt authentication
		if (array_key_exists('username', $_REQUEST) && ($nopass || array_key_exists('password', $_REQUEST)) && 
				array_key_exists('domain', $_REQUEST)) {
			// Attempt to retrieve the LDAP database object based on the passed domain
			$ldap = new BFLDAP();
			if ($ldap->select(BF_FIELD_PREFIX . 'domain', $_REQUEST['domain'])) {
				if ($results = $this->ldapValidUser($ldap, $_REQUEST['username'], 
						isset($_REQUEST['password']) ? $_REQUEST['password'] : '', $nopass)) {
					$this->set('bf_login', strtolower($_REQUEST['username']));
					$this->set('bf_name', $results['fullname']);
					$this->set('bf_email', $results['email']);
					// Time to get the time zone!
					$qb = BFQueryBuilder::select();
					$qb->addColumn('bf_tzone', 'bf_users', 'bf_tzone');
					$qb->where(array('=', 'bf_users', 'bf_userid', 1));
					$tzone = $qb->getOne();
					$this->set('bf_tzone', $tzone);
					// Only save the password if specified that we should
					if (strtolower($this->controller->getSysconfig('store_ldap_auth')) == 'yes' ||
							strtolower($this->controller->getSysconfig('store_ldap_auth')) == 'y') {
						$password = isset($_REQUEST['password']) ? $_REQUEST['password'] : '';
						$this->set('bf_password', '!' . bf_encrypt($password));
					} else {
						$this->set('bf_password', '!');
					}
					$this->set('bf_lang', $LANG_DEFAULT);
					$this->controller->sudo(
											array(&$this, 'save'),
											array()
											);
					// Once saved, NOW we can add the user to the LDAP groups if necessary
					if (strtolower($ldap->get('access_groups')) == 'yes') {
						$this->addLDAPGroups($results['grouplist']);
					}
					$this->ldap = true;

					return $this->serveCookie(true);
				}
			}
		}
		return false;
	}

	/**
	 * More or less a port of ldapValidUser from the 3.* branch.
	 */
	protected function ldapValidUser($ldap, $username, $password, $nopass = null) {
		global $controller;

		// Attempt to connect to the LDAP server
		$ldap_connection_string = $ldap->get('protocol') . '://' . $ldap->get('host');
		$ldap_connection = ldap_connect($ldap_connection_string);
		if (!$ldap_connection && !($ldap_connection = ldap_connect($ldap->get('host')))) {
			$this->errors[] = translate('2200');
			return false;
		}
		// Otherwise, attempt to bind based on the credentials supplied
		ldap_set_option($ldap_connection, LDAP_OPT_PROTOCOL_VERSION, 3);

		if(!($r = @ldap_bind($ldap_connection, $ldap->get('admin_cn'), bf_decrypt($ldap->get('admin_password'))))) {
			// This means something bad happened during the bind process
			$this->errors[] = translate('2267');
		};
		if ($r) {
			$ldap_dname = $ldap->get('host');
			// sub in the login string and go find the user account record
			$ldap_search_filter = str_replace("%",$username,$ldap->get('unique_identifier'));
			$ldap_query_fields = array(
									$ldap_dname,
									$ldap->get('displayname'),
									$ldap->get('mailname'),
									$ldap->get('groupname')
								);
			
			//escape parenthesis in search filter...
			if(strpos($ldap_search_filter,'(') || strpos($ldap_search_filter, ')'))
			{
				$ldap_search_filter = str_replace('(', '\(',$ldap_search_filter);
				$ldap_search_filter = str_replace(')', '\)',$ldap_search_filter);
			}
			$sr = ldap_search($ldap_connection, $ldap->get('search_base'), $ldap_search_filter, $ldap_query_fields);
			if ($sr) {
				// there must be only one...
				$records = ldap_count_entries($ldap_connection,$sr);
				if ($records == 1) {
					// read the record and use the DN field to re-bind or crypt and compare to the ldap entry to validate
	   				$ldap_info = ldap_get_entries($ldap_connection, $sr);

					$ldap_user_bind_string = $ldap_info[0]['dn'];
					if (is_array($ldap_user_bind_string)) {
						$ldap_user_bind_string = $ldap_user_bind_string[0];
					}
					if($nopass)  {
						$r = true;
					}  elseif ($ldap->get('user_bind') == "no") {
						//$r = ldap_valid_password($password,$ldap_info[0]);
						$r = false;
					} else {
						// Attempt to reconnect to the LDAP server
						ldap_close($ldap_connection);
						$ldap_connection = ldap_connect($ldap_connection_string);
						if (!$ldap_connection) {
							$ldap_connection = ldap_connect($ldap->get('host'));
						}
						// Otherwise, attempt to bind based on the credentials supplied
						ldap_set_option($ldap_connection, LDAP_OPT_PROTOCOL_VERSION, 3);
						$r = ldap_bind($ldap_connection, $ldap_user_bind_string, $password);
					}
					if ($r) {
						// re-bind is ok or it crypt'd ok, valid user login.
						$results['fullname'] = $ldap_info[0][strtolower($ldap->get('displayname'))][0];
						$results['fullname'] = str_replace('\'', '&#39;', $results['fullname']);
						$results['email'] = (isset($ldap_info[0][$ldap->get('mailname')]))?$ldap_info[0][strtolower($ldap->get('mailname'))][0]:'';

						$results['grouplist'] =array();
						$this->ldap_get_groups($ldap_connection,$ldap_info[0],$results['grouplist'], $ldap);

						$memberof = strtolower($ldap->get('memberof'));
						if ($memberof == '') {
							return $results;
						} else {
							foreach ($results['grouplist'] as $group => $val) {
								if ($memberof == $group) {
									return $results;
								}
							}
							$this->errors[] = translate('2154', translate('login'));
						}
					} else {
						$this->errors[] = translate('2231');
					}
				} elseif ($records == 0) {
					$this->errors[] = translate('2232');
				} else {
					$this->errors[] = translate('2233');
				}
			} else {
				$this->errors[] = translate('2234');
			}
		} else {
			$this->errors[] = translate('2235');
		}

		return false;
	}

	/**
	 * Add the user to the LDAP groups matching those in the database
	 * TODO: Fix the database schema so we don't use split up, hex-encoded
	 * values as our freakin' keys. Writing this hurt me.
	 */
	protected function addLDAPGroups($groups) {

		// Cheating a bit...
		$this->controller->require_file('modules/BFLevel/BFLevel.php');
		$this->controller->require_file('modules/BFReport/BFReport.php');

		$rep = new BFReport();
		$rep->addColumn('bf_level', 'bf_levels', 'level');
		$rep->addColumn('bf_name', 'bf_levels', 'name');
		$rep->addColumn('bf_dn', 'bf_levels', 'dn');
		$res = $rep->getReportResults();

		while ($rep->valid() && $rec = $res->next()) {
			$store = new BFStore();
			$real_dn = $store->getStorage($rec['dn']);
			$real_dn = $real_dn['TEXT'];
			// Try to keep memory usage slightly under monstrous
			unset($store);
			$addme = false;

			// If this group matches our requirements, add the user
			if ($real_dn != "") {
				// '*' means everyone gets this
				if ($real_dn == '*') {
					$addme = true;
				} else {
					// iterate over the list of access DNs looking for them in the user's DN list
					$real_dns = explode(';',$real_dn);
					if (!is_array($real_dns)) {
						$real_dns = array($real_dns);
					}
					foreach ($real_dns as $bf_grp) {
						$bf_grp = strtolower($bf_grp);
						foreach ($groups as $group => $gcount) {
							// If the access control DN matches the LDAP DN then they have access to this group.
							if ($bf_grp == $group) {
								$addme = true;
							}
						}
					}
				}
			} else {
				$pat = '/^[^=]+=' . $rec['name'] . ',/i';
				foreach ($groups as $group => $gcount) {
					if (preg_match($pat, $group)) {
						$addme = true;
					}
				}
			}

			// Select groups that match the dn
			if ($addme) {
				$group = new BFlevel();
				$group->select($group->primary_key, intval($rec['level']));
				// Add the user and save
				$this->controller->sudo(
										array(&$group, 'addMember'),
										array(intval($this->fields[$this->primary_key]))
										);
				$this->controller->sudo(
										array(&$group, 'save'),
										array()
										);
			}
		}
		return true;
	}

	/**
	 * Actually check credentials and log the user into the system.
	 */
	protected function login($name, $pass, $ldap = null, $needcookie = true) {
		$fields = array('bf_login');
		$values = array($name);

		/**
		 * We can only get away with a blanket deletion of where clauses because
		 * this DBO has no joins. If it did, we'd need to bring those back after
		 */
		$this->cleanWhere();

		/**
		 * If you can't bump active users, we need another SQL call to check for active sessions
		 */
		$bump = $this->controller->getSysConfig('bump_active_users');
		if ($bump[0] != 'Y' && $bump[0] != 'y' && $bump[0] != '1') {
			// Need to take auto_logoff into account or someone would never never logout
			$auto_logoff = intval($this->controller->getSysConfig('auto_logoff'));
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_id', 'bf_sessions', 'bf_id', 'COUNT');
			// Root does not get kept out of the system.
			$qb->where(array('!=', 'bf_users', 'bf_userid', 1));
			if ($auto_logoff > 0) {
				$qb->where(array(
								 array('=', 'bf_users', 'bf_userid', 'bf_sessions', 'bf_userid'),
								 array('!=', 'bf_users', 'bf_userid', 1),
								 array('=', 'bf_users', 'bf_login', $name),
								 array('!=', 'bf_sessions', 'bf_id', ''),
								 array('>', 'bf_sessions', 'bf_modified', time() - ($auto_logoff * 60))
								 ));
			} else {
				$qb->where(array(
								 array('=', 'bf_users', 'bf_userid', 'bf_sessions', 'bf_userid'),
								 array('!=', 'bf_users', 'bf_userid', 1),
								 array('!=', 'bf_sessions', 'bf_id', ''),
								 array('=', 'bf_users', 'bf_login', $name)
								 ));
			}
			$still_logged_in = $qb->getOne();

			if (intval($still_logged_in) > 0) {
				$this->errors[] = translate(2070);
				return false;
			}
		}
		/**
		 * Warning! This uses ancient bf_decrypt() technologies, because each and every time a
		 * string gets encrypted it has a different encrypted value. Completely different
		 * functionality than hashing (which it SHOULD do, but can't.  Ask Chris or Robin if
		 * you don't understand why that is).
		 */
		$idfound = $this->select($fields, $values);
		
		/**
		 * RATLC01267094
		 * 
		 * Check to see if we're a ldap user (i.e. lowercase the name and then
		 * check the db for this id, and if we are, then setup the user info.
		 * 
		 * rest of this...from if(!DB::Error())...was copied from bfdbo.php since i couldnt 
		 * figure out how to reuse the this->select() functionality properly...
		 */
		if(!$idfound) {
			$qbuser = BFQueryBuilder::select();
			$qbuser->addColumn('bf_userid', 'bf_users', 'bf_userid');
			$qbuser->addColumn('bf_level', 'bf_users', 'bf_level');
			$qbuser->addColumn('bf_name', 'bf_users', 'bf_name');
			$qbuser->addColumn('bf_email', 'bf_users', 'bf_email');
			$qbuser->addColumn('bf_login', 'bf_users', 'bf_login');
			$qbuser->addColumn('bf_password', 'bf_users', 'bf_password');
			$qbuser->addColumn('bf_maxbuilds', 'bf_users', 'bf_maxbuilds');
			$qbuser->addColumn('bf_session', 'bf_users', 'bf_session');
			$qbuser->addColumn('bf_pupdate', 'bf_users', 'bf_pupdate');
			$qbuser->addColumn('bf_pexpire', 'bf_users', 'bf_pexpire');
			$qbuser->addColumn('bf_pwexpired', 'bf_users', 'bf_pwexpired');
			$qbuser->addColumn('bf_tzone', 'bf_users', 'bf_tzone');
			$qbuser->addColumn('bf_datefmt', 'bf_users', 'bf_datefmt');
			$qbuser->addColumn('bf_priority', 'bf_users', 'bf_priority');
			$qbuser->addColumn('bf_lang', 'bf_users', 'bf_lang');
			$qbuser->addColumn('bf_tiplevel', 'bf_users', 'bf_tiplevel');
			$qbuser->addColumn('bf_data', 'bf_users', 'bf_data');
			$qbuser->where(array('=', 'bf_users', 'bf_login', strtolower($name)));
			$myres = $qbuser->getResult();
			if(!DB::isError($myres)) {
				$cols = $myres->fetchRow(DB_FETCHMODE_ASSOC);
				$pw = $cols{'bf_password'};
				if ($cols && strlen($pw) > 0 && $pw{0} == '!') {
					//yes we are and ldap user, continue....
					$this->updated  = array();  // Keeps track of which fields get modified
					$this->inserted = true;
					foreach ($cols as $col => $val) {
						$this->fields[$col]	 = $val;
						$this->updated[$col]	= false;
						// Keep a stored copy where specified
						if (array_key_exists($col, $this->stored_values)) {
							$this->stored_values[$col] = $val;
						}
					}
					$idfound = true;
				}
			}
		}
		
		if ($idfound){

			// Override those fields, guys!  If the password starts with a bang, ldap user...
			$password = $this->get('bf_password');
			if (strlen($password) > 0 && $password{0} == '!') {
				$password = substr($password, 1);
				$this->ldap = true;
			}

			if (bf_decrypt($password) == $pass || ($this->ldap && $ldap instanceof BFLDAP && $this->ldapValidUser($ldap, $name, $pass))) {
				// If not an LDAP user, and their password expired - force them to make a new one
				if (!$this->ldap && $this->fields['bf_pwexpired'] != 'N' && !$this->changePassword()) {
					$this->forceXHTMLLogin(true);
				} else if ($this->fields['bf_userid'] != 1 && $this->controller->getUsersCount() >= (int)$this->controller->getSysconfig('lic_users') && !$this->forceEntry() && $this->controller->currentUserHasSeat($this->get('bf_userid')) != 1) {
					$this->errors[] = translate('login limit reached');
					return false;
				// Note login and serve the cookie
				} else {
					if ($this->ldap && $info = $ldap->getUserInfo($this->fields['bf_login'])) {

						$update = BFQueryBuilder::update();
						$update->setColumn('bf_name', $this->table, $info['fullname']);
						$update->setColumn('bf_email', $this->table, $info['email']);

						if ( bf_decrypt($password) != $pass)  {
	
							if ( (strtolower($this->controller->getSysconfig('store_ldap_auth')) == 'yes') ||
								 (strtolower($this->controller->getSysconfig('store_ldap_auth')) == 'y') ) {

                        		if(isset($pass)) {
									$update->setColumn('bf_password', $this->table, '!' . bf_encrypt($pass));
								}	
							}
                  		}
						
						$update->where(array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]));
						$res = $update->getResult();
					}
					$this->controller->logAudit('SelectedUserByLogin', array($name, $this->fields['bf_email']));
					$this->controller->loadPermissions($this->fields['bf_userid'], true);
					return ($needcookie) ? $this->serveCookie(true) : true;
				}
			}
		} else {
			$this->controller->logError('InvalidUserId', array($name));
		}
		$this->errors[] = translate('invalid credentials');

		return false;
	}

	protected function forceEntry() {
		if ($this->controller->getSysconfig('feature_fixedusers') != '1' && $this->fields['bf_priority'] == 'Y') {
			// Get the user with the longest session and bump them
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_userid', 'bf_users', 'userid');
			$qb->addColumn('bf_modified', 'bf_sessions', 'modified');
			$qb->where(array(
					array('=', 'bf_users', 'bf_userid', 'bf_sessions', 'bf_userid'),
					array('!=', 'bf_users', 'bf_userid', 1),
					array('!=', 'bf_sessions', 'bf_id', '')
				));
			$qb->orderBy('ASC', 'bf_users', 'bf_priority');
			$qb->orderBy('DESC', 'bf_sessions', 'bf_modified');
			
			if ($res = $qb->getOne()) {
				$now = time();
				$qb = BFQueryBuilder::update();
				$qb->setColumn('bf_id',		 'bf_sessions',  '');
				$qb->setColumn('bf_type',	   'bf_sessions',  'lic_users');
				$qb->setColumn('bf_modified',   'bf_sessions',  $now);
				$qb->where(array('=', 'bf_sessions', 'bf_userid',  $res));
				$res = $qb->getResult();
		
				return true;
			}
		}
		return false;
	}

	/**
	 * This only gets called when a user's password has expired and
	 * they create a new one.
	 */
	protected function changePassword() {
		global $_REQUEST;

		if (array_key_exists('new_password', $_REQUEST) && array_key_exists('new_password_confirm', $_REQUEST)) {
			if ($_REQUEST['new_password'] != $_REQUEST['new_password_confirm']) {
				$this->errors[] = translate(2271);
			} else if (($valid = BFUser::validPassword($_REQUEST['new_password'])) !== true) {
				$this->errors[] = $valid;
			} else if (bf_decrypt($this->fields['bf_password']) == $_REQUEST['new_password']) {
				$this->errors[] = translate(2272);
			} else {
				$this->set('bf_password', bf_encrypt($_REQUEST['new_password']));
				$this->set('bf_pwexpired', 'N');
				$update = BFQueryBuilder::update();
				$update->setColumn('bf_password', $this->table, $this->fields['bf_password']);
				$update->setColumn('bf_pwexpired', $this->table, 'N');
				$update->where(array('=', $this->table, $this->primary_key, $this->fields[$this->primary_key]));
				$res = $update->getResult();

				// Puke accordingly or return true
				if (DB::isError($res)) {
					$this->errors[] = $res->getMessage();
				} else if (!$this->serveCookie(true)) {
					$this->errors[] = translate(2273);
				} else {
					return true;
				}
			}
		}

		return false;
	}

	protected function serveCookie($new = false) {
		global $GLOBAL_COOKIE, $_SERVER, $BF_PARSED_URL;

		// Only create a new cookie if not set already
		if ($new) {
			$this->sessionid = md5($this->get('bf_login') . $this->sessionid . time() . $GLOBAL_COOKIE['salt']);
		}
		// set expires and use auto_logoff * 60 as its in minutes, not seconds
		// if the autologoff is less than 1 minute, set it to 1 minute.
		$expiration = ($this->sysconfig->get('auto_logoff') < 1 && $this->sysconfig->get('auto_logoff') != 0)?60:intval($this->sysconfig->get('auto_logoff')) * 60;

		if ($this->setCookie($this->sessionid, $expiration)) {
			// TODO: Set the cookie path to the webroot
			if (setcookie($GLOBAL_COOKIE['name'], $this->sessionid, (($expiration > 0) ? time() + $expiration : null), '/')) {
				return true;
			} else {
				throw new BFException(BF_LANG_ERR_COOKIE_UNSET);
			}
		}

		return false;
	}

	/**
	 * Set the cookie value as stored in the users' session record in the database.
	 */
	public function setCookie($value, $expires) {
		// First, we need to know if this updates an existing cookie with this value
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_id', 'bf_sessions', 'id');
		$qb->addColumn('bf_userid', 'bf_sessions', 'userid');
		$qb->addColumn('bf_type', 'bf_sessions', 'type');
		// This selects either a record with the current value or an expired record
		$qb->where(array('=', 'bf_sessions', 'bf_userid', $this->get('bf_userid')));
		// We only need one match. Really, we should only ever HAVE one match, but you never know...
		$qb->limit(1);
		$session_record = $qb->getRow();
		// Thanks for documenting this, guys! DB::getOne() returns null when no results...
		if (DB::isError($session_record) || empty($session_record)) {
			return $this->insertCookie($value, $expires);
		} else {
			return $this->updateCookie($session_record['id'], $value, $expires);
		}
	}

	protected function insertCookie($value, $expires) {
		$now = time();
		$qb = BFQueryBuilder::insert();
		$qb->setColumn('bf_id',		 'bf_sessions', $value);
		$qb->setColumn('bf_userid',	 'bf_sessions', $this->get('bf_userid'));
		$qb->setColumn('bf_type',	   'bf_sessions', 'lic_users');
		$qb->setColumn('bf_created',	'bf_sessions', $now);
		$qb->setColumn('bf_modified',   'bf_sessions', $now);
		$qb->setColumn('bf_expires',	'bf_sessions', $expires);
		$res = $qb->getResult();
		return !DB::isError($res);
	}

	public function updateCookie($oldvalue, $newvalue, $expires) {
		// If blanking out, set now to 0, otherwise set to now
		$now = ($newvalue == '') ? 0 : time();
		$qb = BFQueryBuilder::update();
		$qb->setColumn('bf_id',		 'bf_sessions',  $newvalue);
		$qb->setColumn('bf_type',	   'bf_sessions',  'lic_users');
		$qb->setColumn('bf_modified',   'bf_sessions',  $now);
		$qb->setColumn('bf_expires',	'bf_sessions',  $expires);
		$qb->where(array(
						 array('=', 'bf_sessions', 'bf_userid',  $this->get('bf_userid')),
						 array('=', 'bf_sessions', 'bf_id',	  $oldvalue)
						 ));
		$res = $qb->getResult();
		return !DB::isError($res);
	}

	public function getCookie() {
		if (!isset($this->sessionid) && !empty($this->fields['bf_userid'])) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_id', 'bf_sessions', 'bf_id');
			$qb->where(array('=', 'bf_sessions', 'bf_userid',  $this->get('bf_userid')));
			$res = $qb->getOne();
			if (DB::isError($res)) {
				return false;
			} else {
				$this->sessionid = $res;
			}
		} else {
			return false;
		}

		return $this->sessionid;
	}

	/**
	 * Log out!
	 *
	 * If there is a message then tell them why they are being logged out.
	 * If we are logged in and have a sessionid, set it to nothing.
	 * Then attempt to set the cookie to NOW so it is expired after one second else throw
	 */
	public function logout($msg = null) {
		global $_COOKIE, $GLOBAL_COOKIE, $BF_PARSED_URL, $user;
		$sid = isset($_COOKIE[$GLOBAL_COOKIE['name']]) ? $_COOKIE[$GLOBAL_COOKIE['name']] : '';
		if (!empty($sid)) {
			// Clear the cookie record in the sessions table
			$this->updateCookie($sid, '', 0);
			$expiration = intval($this->sysconfig->get('auto_logoff'));
			$_COOKIE[$GLOBAL_COOKIE['name']] = '';
			if (!setcookie($GLOBAL_COOKIE['name'], '', 1, '/')) {
				throw new BFException(BF_LANG_ERR_COOKIE_UNSET);
			}
		}
		// No matter what, prompt for authentication
		$this->controller->logAudit('UserLogout', array(isset($this->fields['bf_name']) ? 
				$this->fields['bf_name'] : ''));
		$this->forceLogin(true);
	}
	
	public function su($password) {
		global $user;
		if ($user->isRoot() || bf_decrypt($this->fields['bf_password']) == $password) {
			// Note login and serve the cookie
			$this->controller->logAudit('SelectedUserByLogin', array($this->fields['bf_name'], $this->fields['bf_email']));
			$this->controller->loadPermissions($this->fields['bf_userid'], true);
			return $this->serveCookie(true);
		}
		throw new BFException(translate('invalid credentials'));
	}

	/**
	 * Purge the users' seat out of the license pool
	 */
	public function purge() {
		$qb = BFQueryBuilder::delete();
		$qb->where(array('=', 'bf_sessions', 'bf_userid',  $this->get('bf_userid')));
		$res = $qb->getResult();
		if (!DB::isError($res))  {
			$this->controller->logAudit('UserPurged', array($this->fields['bf_name']));
			return true;
		}
		return false;			
	}

	/**
	 * Need to purge the user when deleting so they don't keep taking up a seat
	 */
	public function delete() {
		// Access check
		if ($this->controller->userHasAccessTo('DeleteUser')) {
			if ($this->referenced())  {
				throw new BFException(translate('2066'));
			} elseif (!$this->purge()) {
				throw new BFDBException(translate('2211'));
			} elseif (!$this->removeAllAccess()) {
				throw new BFDBException(translate('2212'));
			} else {
				return parent::delete();			
			}
		} else {
			throw new BFAccessException('DeleteUser');
		}
	}

	/**
	 * Kill all records in bf_access and bf_accesscache
	 */
	protected function removeAllAccess() {
		$qb = BFQueryBuilder::delete();
		$qb->where(array('=', 'bf_access', 'bf_userid',  $this->get('bf_userid')));
		$res = $qb->getResult();
		if (DB::isError($res)) {
			return false;
		}
		$qb = BFQueryBuilder::delete();
		$qb->where(array('=', 'bf_accesscache', 'bf_userid',  $this->get('bf_userid')));
		$res = $qb->getResult();
		return !DB::isError($res);
	}

	/**
	 * Force the user to change their password the next time they login
	 */
	public function expirePassword() {
		// Access check
		if (!$this->controller->userHasAccessTo('ExpireUserPasswords')) {
			throw new BFAccessException('ExpireUserPasswords');
		}

		$this->set('bf_pwexpired', 'Y');
		return parent::update();
	}

	/**
	 * TODO: return or otherwise convey useful info.
	 */
	public function referenced() {
		if (isset($this->fields[$this->primary_key]) && (int)$this->fields[$this->primary_key] > 0) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_owner', 'bf_cron', 'bf_owner', 'COUNT');
			$qb->where(array('=', 'bf_cron', 'bf_owner', $this->fields[$this->primary_key]));
			return intval($qb->getOne()) > 0;
		} else {
			return false;
		}
	}


	/**
	 * Load the user previously authenticated using HTTP Authentication
	 */
	public function loadByRemoteUser($name) {
		$fields = array('bf_login');
		$values = array($name);

		/**
		 * We can only get away with a blanket deletion of where clauses because
		 * this DBO has no joins. If it did, we'd need to bring those back after
		 */
		$this->cleanWhere();

		if ($this->select($fields, $values)) {
			return $this->serveCookie(true);
		}  elseif ($this->controller->getSysconfig('sso_http_domain'))  {
			/**
			 * We'll try logging on via LDAP, using the sso_http_domain setting as the domain and the name as
			 * the username.
			 */
			$_REQUEST['username'] = $name;
			$_REQUEST['domain'] = $this->controller->getSysconfig('sso_http_domain');
			return $this->loadByLDAP(true);
		}

		return false;
	}

	/** LDAP request desguised as a Single Sign On HTTP request
	 *  Gets the Username/Password and Domain from the http header, and then makes an LDAP connection
	 */
	public function loadBySSOHTTPLDAP() {
		global $_SERVER;

		$_REQUEST['username'] = '';
		$_REQUEST['password'] = '';
		$_REQUEST['domain'] = '';

		$sso_http_username = $this->controller->getSysconfig('sso_http_username');
		$sso_http_password = $this->controller->getSysconfig('sso_http_password');
		$sso_http_domain = $this->controller->getSysconfig('sso_http_domain');

		if (!empty($sso_http_username) && !empty($sso_http_password) && !empty($sso_http_domain)
				&& strpos($sso_http_username, '|') !== false && strpos($sso_http_password, '|') !== false) {
			// Split the header identifiers and the regexp out of the sysconfig
			list($header_username, $regexp_username) = preg_split("/\|/", $sso_http_username, 0);
			list($header_password, $regexp_password) = preg_split("/\|/", $sso_http_password, 0);
			$header_domain = NULL;
			$regexp_domain = NULL;

			// In case someone needs to hardcode the domain
			if(strpos($sso_http_domain,'|') !== false) {
				list($header_domain, $regexp_domain) = preg_split("/\|/", $sso_http_domain, 0);
			} else {
				$domain = $sso_http_domain;
			}		

			// Extract the username, password and domain from the passed header using the regexp in sysconfig
			// $1 will always be the object to collect
			$username = (isset($_SERVER[$header_username]))?
				(preg_match('~'.$regexp_username.'~', $_SERVER[$header_username], $matches))?
					((isset($matches[1]))?$matches[1]:''):'':'';

			$password = (isset($_SERVER[$header_password]))?
				(preg_match('~'.$regexp_password.'~', $_SERVER[$header_password], $matches))?
					((isset($matches[1]))?$matches[1]:''):'':'';

			if(!empty($regexp_domain)) {
				$domain = (isset($_SERVER[$header_domain]))?
					(preg_match('~'.$regexp_domain.'~', $_SERVER[$header_domain], $matches))?
						((isset($matches[1]))?$matches[1]:''):'':'';
			}

			if (!empty($username) && !empty($password) && !empty($domain)) {

				$_REQUEST['username'] = $username;
				$_REQUEST['password'] = $password;
				$_REQUEST['domain'] = $domain;

				return $this->authFromRequest();

			}

		}
		return false;
	}


	/**
	 * Attempt to authenticate the user first by cookie, then by credentials (from a login form),
	 * and then finally by simply forcing the login page.
	 *
	 * If the user clicks LOGOUT then by god, log out and demand the login prompt.
	 *  This is so if someone is http authed in or SSO http/ldap authed in, they can logout and login as root or something.
	 * If username exists in the post then we know they are trying to login via the form,
	 *  so ignore all other information and log them in via username/password.
	 *  This is so that when I do login via SSO or some other method, if I click log out and then type in some username
	 *  and password it will actually listen to what I typed in and not relog me in as whatever the auth was.
	 */
	public function authenticate($sso_logout = false) {
		global $_REQUEST, $GLOBAL_COOKIE;
		$auth = false;
		
		// Cookie?
		if (array_key_exists($GLOBAL_COOKIE['name'], $_REQUEST) && !empty($_REQUEST[$GLOBAL_COOKIE['name']])
				&& !array_key_exists('username', $_REQUEST)) {
			$auth = $this->loadBySession($_REQUEST[$GLOBAL_COOKIE['name']]);
			if (!$auth) {
				$this->errors[] = translate(2274);
			}
		}

		// HTTP Auth?
		if (!$auth && $this->controller->getSysconfig('sso_remote_user') == 'Y' && 
				array_key_exists('PHP_AUTH_USER', $_SERVER) && !array_key_exists('username', $_REQUEST)) {
			$auth = $this->loadByRemoteUser($_SERVER['PHP_AUTH_USER']);
			if (!$auth) {
				$this->errors[] = translate(2275);
			}
		}

		// Username and password?
		if (!$auth) {
			$auth = $this->loadByCredentials();
		}

		// SSO HTTP/LDAP Auth?
		if (!$auth && !array_key_exists('username', $_REQUEST) && !$sso_logout) {
			$auth = $this->loadBySSOHTTPLDAP();
		}

		// LDAP Auth?
		if (!$auth && empty($this->fields['bf_userid'])) {
			$auth = $this->loadByLDAP();
		}

		// If authenticate, congrats! Have your prefs and off you go.
		if ($auth) {
			$this->loadPreferences();
			$this->loadView();
			return true;
		// If not, you need to try logging in...
		} else {
			$lang = bf_get_language();
			require_once 'lang/'.$lang.'/language.php';
			$this->forceLogin();
		}
	}

	/**
	 * Since this needs to get called from a couple of places, we
	 * might as well just pull it out into a convinient little method.
	 */
	public function displayLoginMessage() {
		$msg = $this->controller->getSysconfig('login_message');
		if (!empty($msg)) {
			$strings = explode("\n", $msg);
			$msg = $strings[rand(0, count($strings) - 1)];
			$this->messages[] = $msg;
		}
	}

	/**
	 * Different request types have different behaviors, so we can't just
	 * use a different template.
	 */
	public function forceLogin($logout = false) {
		global $_REQUEST;
		if (!array_key_exists(BF_REQUEST_KEY_TYPE, $_REQUEST)) {
			$_REQUEST[BF_REQUEST_KEY_TYPE] = $this->controller->request_type;
		}
		switch ($_REQUEST[BF_REQUEST_KEY_TYPE]) {
			case BF_REQUEST_XHTML:
				$this->forceXHTMLLogin(false, $logout);
				break;
			case BF_REQUEST_XML:
				$this->forceXMLLogin();
				break;
			case BF_REQUEST_RSS2:
				$this->forceHTTPLogin();
				break;
		}
	}

	/**
	 * Spit out the login page and exit();
	 */
	public function forceXHTMLLogin($password_expired = false, $logout = false) {
		global $controller, $GLOBALS;
		// Need to pass LDAP servers, if any
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_domain', 'bf_ldap', 'bf_domain', 'DISTINCT');
		$domains = $qb->getCol();

		$this->controller->require_file('BFJTE.php');
		$jte = new BFJTE();
		$jte->assign('webroot', BF_WEBROOT);

		if (!empty($GLOBALS['proxyHost']) || !empty($GLOBALS['proxyPath'])) {
			$jte->assign('formaction', BF_WEBROOT);
		} else {
			$jte->assign('formaction', str_replace('&logout','',$_SERVER['REQUEST_URI']));
		}

		// Inform the user, even when not logged in yet
		if ($controller->license_error) {
			$this->errors[] = $controller->license_error;
		}

		if ($password_expired) {
			$jte->assign('password_expired', 1);
			$jte->assign('username', $this->fields['bf_login']);
			$this->errors[] = translate(2276);
		}

		// Only assign the LDAP domains if we have results
		if (!DB::isError($domains) && is_array($domains)) {
			$jte->assign('domains', $domains);
		}

		$this->displayLoginMessage();
		
		$jte->assign('errors', $this->errors);
		$jte->assign('messages', $this->messages);
		if ($logout) {
			$jte->display('logout.tpl');
		} else {
			$jte->display('login.tpl');
		}
		exit();
	}

	/**
	 * Spit out an appropriate error and exit();
	 */
	public function forceXMLLogin() {
		$this->controller->require_file('BFJTE.php');
		$jte = new BFJTE();
		$jte->assign('webroot', BF_WEBROOT);
		$jte->assign('errors', array('LOGGEDOUT'));
		header($GLOBALS['GLOBAL_HEADERS'][BF_REQUEST_XML]);
		$jte->display('xml.tpl');
		exit();
	}

	protected function forceHTTPLogin($msg = null) {
		header('WWW-Authenticate: Basic realm=""');
		header('HTTP/1.0 401 Unauthorized');
		echo $msg;
		exit;
	}

	/**
	 * Create an array of BFPreference objects.
	 *
	 * Something to consider: depending on just how long the list of user preferences could get,
	 * we may want to pull this out into an associative array of names and values and just create
	 * and destroy database objects as necessary.
	 */
	protected function loadPreferences() {
		global $GLOBAL_SETTINGS, $controller;
		if (!$controller->require_module('BFPreference')) return false;
		$pref = new BFPreference();
		$uid = $this->get('bf_userid');
		$this->preferences = $pref->search('bf_userid', ($uid != null) ? $uid : 0);
	}

	protected function loadView() {
		global $GLOBAL_SETTINGS;
		// Override the default count setting
		$action = isset($_REQUEST['action']) ? $_REQUEST['action'] : 
				(isset($_REQUEST['mod']) ? bf_get_default_action($_REQUEST['mod']) : '');

		$count = $this->getPreference($action ? ($action . '_count') : 'count');
		if ($count) {
			$GLOBAL_SETTINGS['reporting']['count'] = (int)$count;
		}
	}

	function __construct() {
		parent::__construct();
		$this->sysconfig =	  $GLOBALS['controller']->getSysConfig();
	}

	/**
	 * Password format check based on an OMA-standardized sysconfig setting.
	 */
	public static function validPassword($password) {
		$fields = explode('.', $GLOBALS['controller']->getSysconfig('pw_fmt'));

		$len = array_shift($fields);
		if ($len < 1) {
			$len = 1;
		}

		if (strlen($password) < $len) {
			return translate("password short of x chars", $len);
		}

		$required = intval(array_shift($fields));
		$u_case = 0;
		$l_case = 0;
		$n_case = 0;
		$s_case = 0;
		for ($i = 0 ; $i < strlen($password) ; $i++) {
			if (strpos('ABCDEFGHIJKLNMOPQRSTUVWXYZ',$password{$i}) !== FALSE) {
				$u_case++;
			}
			if (strpos('abcdefghijklnmopqrstuvwxyz',$password{$i}) !== FALSE) {
				$l_case++;
			}
			if (strpos('0123456789',$password{$i}) !== FALSE) {
				$n_case++;
			}
			if (strpos("\"\'~!@#$%^&*(){}[]|,./?-_=+\\<>",$password{$i}) !== FALSE) {
				$s_case++;
			}
		}

		$matches = 0;
		foreach ($fields as $req) {
			$cat = substr($req,0,1);
			$len = intval(substr($req,1));
			if ($len < 1) {
				$len = 1;
			}
			switch($cat) {
				case 'N':
					if ($n_case < $len)
						return translate('password too few numerics');
					break;
				case 'S':
					if ($s_case < $len)
						return translate('password too few special');
					break;
				case 'U':
					if ($u_case < $len)
						return translate('password too few upper');
					break;
				case 'L':
					if ($l_case < $len)
						return translate('password too few lower');
					break;
				case 'n':
					if ($n_case >= $len)
						$matches++;
					break;
				case 's':
					if ($s_case >= $len)
						$matches++;
					break;
				case 'u':
					if ($u_case >= $len)
						$matches++;
					break;
				case 'l':
					if ($l_case >= $len)
						$matches++;
					break;
				default:
					// ??
					break;
			}
		}
		return ($matches >= $required ? true : translate('password too few variations'));
	}

	public function fetch() {
		if (parent::fetch()) {
			if ($this->fields['bf_password']{0} == '!') {
				$this->ldap = true;
			}
			return true;
		} else {
			return false;
		}
	}


	public function ldap_get_groups($ldap_connection,$ldap_info,&$results,&$ldap_config) {

		$ldap_filter = "";
		$ldap_keys = array();
		$ldap_values = array();
		$groupname = $ldap_config->get('groupname');
		$groups = array();
		$sr = false;

		//flatten the results so we can add into the filter
		foreach ($ldap_info as $key => $val) {
			if (!preg_match("/\d+/",$key)) {
				if (($groupname != "")&&($groupname == $key)) { // check for the groupname before we flatten the results
					if (is_array($val) && array_key_exists('count',$val)) {
						$subcount = $val['count'];
						for ($j = 0 ; $j < $subcount ; $j++) {
							array_push($groups,$val[$j]);
						}
					}
				}
				if (is_array($val)) {
					$val = $val[0];
				}
				array_push($ldap_keys,"/%$key%/");
				array_push($ldap_values,$val);
			}
		}

		if ($ldap_config->get('group_search_base') == "") {
			if (count($groups) != 0) { //The groups where in the user record (ala Active Directory)
				foreach ($groups as $group) {
					$this->ldap_unwind_groups($ldap_connection,$group,$results,$ldap_config);
				}
				return("");
			} else {	//Its not in the user record but you didn't specify a group search? Yeah, bye
			    return("");
			}
		}

		//Okay, its kept somewhere completely different and we need to use the group_search_base
		ldap_set_option($ldap_connection, LDAP_OPT_PROTOCOL_VERSION, 3);
		// bind as the admin account - this has to have already worked
		if(!ldap_bind($ldap_connection, $ldap_config->get('admin_cn'),  bf_decrypt($ldap_config->get('admin_password')))) {
			return(""); //coudn't bind
		}

		$ldap_filter = preg_replace($ldap_keys,$ldap_values,$ldap_config->get('group_unique_identifier'));
		//Escape parenthesis in $ldap_filter.
		if(strpos($ldap_filter,'(') || strpos($ldap_filter, ')'))
		{
			$ldap_filter = str_replace('(', '\(',$ldap_filter);
			$ldap_filter = str_replace(')', '\)',$ldap_filter);
		}
		$sr = ldap_search($ldap_connection, $ldap_config->get('group_search_base'), $ldap_filter, array("dn"));

		if ($sr) {
			$records = ldap_count_entries($ldap_connection,$sr);
			$ldap_recs = ldap_get_entries($ldap_connection,$sr);
			for ($i = 0 ; $i < $records ; $i++) {
				if (($groupname != "") && isset($ldap_recs[$i][$groupname]) && isset($ldap_recs[$i][$groupname]['count'])) {
					$subcount = $ldap_recs[$i][$key]['count'];
					for ($j = 0 ; $j < $subcount ; $j++) {
						$this->ldap_unwind_groups($ldap_connection,$ldap_recs[$i][$groupname][$j]['dn'],$results,$ldap_config);
					}
				} else {
					$this->ldap_unwind_groups($ldap_connection,$ldap_recs[$i]["dn"],$results,$ldap_config);
				}
			}
		}
		// exit(0); // DEBUG uncomment this line and the above DEBUG line with the exit(0) to get a list of groups on user login
		return($ldap_filter);
	}


	public function ldap_unwind_groups($ldap_connection,$group,&$results,$ldap_config) {
		// print("$group\n"); // DEBUG uncomment this line and the above DEBUG line with the exit(0) to get a list of groups on user login
		if (!isset($results[strtolower($group)])) {
			$results[strtolower($group)] = 1;
		} else if ($results[strtolower($group)]++ > 5) { // try and protect against recursion
			return;
		}

		return; //this function
	}

	public function getObjectName()  {
		return $this->fields['bf_name'];
	}
}



?>
