<?php

if (file_exists("../../includes/config.php")) {
	$path_relative = "../..";
} else if (file_exists("../includes/config.php")) {
	$path_relative = "..";
}
require_once($path_relative . "/includes/config.php");
require_once("../../modules/BFDBO/SQL/CommonSQL.php");
require_once("../../includes/BFColumnTypeCache.php");

if(!defined('DB_FETCHMODE_ORDERED')) {
	define('DB_FETCHMODE_ORDERED', 1);
}
if(!defined('DB_FETCHMODE_ASSOC')) {
	define('DB_FETCHMODE_ASSOC', 2);
}

abstract class DBHandle {
	protected $db;
	protected $config;
	protected $sqlLookup;
	protected $db_type;
	protected $dsn;
	protected $canPrep = true;
	protected $errorMessage = null;
	protected $columnCache = null;

	function __construct($dbType = null, $cfg = null) {
		$this->db_type = $dbType;
		$this->config = $cfg;
	}

	public abstract function native_prepare($query);

	public abstract function native_param_fixup($types = array(), $params = array());

	public abstract function native_execute($sth, $types, $values);

	public abstract function native_query($statement);

	public abstract function native_list_tables();

	public abstract function native_fetchrow($sth, $options);

	public abstract function native_disconnect($dbh);


	public function getDB() {
		return $this->db;
	}

	public function canPrepare($can = true) {
		$this->canPrep = $can;
	}

	public function getColumnCache() {
		return $this->columnCache;
	}

	public function generateColumnCache($databaseId = 'db2') {
		$this->columnCache = BFColumnTypeCache::getCache($this, $databaseId, $this->getDBVersion(), true);
	}

	public function setDSN($newDSN) {
		$this->dsn = $newDSN;
	}

	public function getConnection($bfconfig = null) {
		global $GLOBAL_DB_OPTIONS;
		if ($bfconfig == null) {
			if ($this->config != null) {
				$bfconfig = $this->config;
				$this->dsn = $bfconfig->getDSN();
			} else {
				if ($this->dsn == null)
					return null;
			}
		}
		$this->db = $this->native_connect($this->dsn, $GLOBAL_DB_OPTIONS);

		return $this->db;
	}


	public function getDBVersion() {
		$sth = $this->simpleQuery("SELECT bf_value as version FROM bf_sysconfig WHERE bf_param='schema_version'");

		if (!$sth) {
			return null;
		}
		$ver = $this->fetchRow($sth, DB_FETCHMODE_ASSOC);
		return $ver['version'];
	}

	public function fetchRow($sth, $options) {
		return $this->native_fetchrow($sth, $options);
	}

	public function listTables() {
		$tables = array();
		$sql = $this->native_list_tables();
		// odbc handles this differently.  It will do the odbc_tables function in this case
		// and return an array of table names.
		if (is_array($sql)) {
			return $sql;
		}

		$res = $this->simpleQuery($sql);

		if (!$res) {
			return $tables;
		} else {
			while($cols = $this->fetchRow($res, DB_FETCHMODE_ASSOC)) {
				foreach ($cols as $col => $val) {
					if (strtoupper(substr($val, 0, 2)) == 'BF') {
						array_push($tables, strtolower($val));
					}
				}
			}
			return $tables;
		}
	}


	public function lookup($key) {
		if ($this->sqlLookup == null) {
			$obj = $this->db_type . "SQL";
			require_once("../../modules/BFDBO/SQL/" . $obj . ".php");
			$this->sqlLookup = new $obj;
		}
		return $this->sqlLookup->getSQL($key);
	}


	public function getKeys() {
		// make sure we populate keys first
		$this->lookup('');

		return $this->sqlLookup->statements;
	}


	public function connect($bfconfig = null) {
		if ($bfconfig != null) {
			$this->config = $bfconfig;
		}
		$dbh = $this->getConnection();
		if (!$dbh) {
			include("offline.php");
			exit;
		}
	}


	public function disconnect() {
		$this->native_disconnect($this->db);
	}


	public function prepare($query) {
		$query = $this->lookup($query);
		//print "preparing statement [$query]\n";
		$this->lastQuery = $query;
		return $this->native_prepare($query);
	}

	public function simpleQuery($statement) {
		$this->lastQuery = $statement;
		return $this->native_query($this->lookup($statement));
	}

	public function execute($sth, $types = array(), $values = array()) {
		return $this->native_execute($sth, $types, $values);
	}

	public function do_query($statement, $types = array(), $params = array()) {
		if (sizeof($params) > 0) {
			if (!$this->canPrep) {
				$params = $this->native_param_fixup($types, $params);
				return $this->fake_prepare_exec($statement, $params);
			}
			$sth = $this->prepare($statement);
			if (!$sth) {
				return null;
			}
			if (!$r = $this->execute($sth, $types, $params)) {
				return null;
			}
			return $sth;
		} else {
			return $this->simpleQuery($statement);
		}
	}

	public function freeHandle($id) {
		$id->free();
	}

	public function finish($id) {
		$id->free();
	}


	protected function setErrorMessage($msg = null) {
		$this->errorMessage = $msg;
	}

	public function getErrorMessage() {
		return $this->errorMessage;
	}

	public function clearErrorMessage() {
		$this->errorMessage = null;
	}

	// probably better ways to do this...
	public function fake_prepare_exec ($statement, $params = array()) {
		$statement = $this->lookup($statement);

		$q = preg_split("/(\?)/", $statement, -1, PREG_SPLIT_DELIM_CAPTURE);
		$newStatement = "";
		$pCount = 0;
		for ($i = 0; $i < sizeof($q); $i++) {
			$newStatement .= ($q[$i] == '?') ? $params[$pCount++] : $q[$i];
		}
		//print "executing faked statement [$newStatement]\n";
		return $this->simpleQuery($newStatement);
	}

}

?>
