<?php

require_once("DBHandle.php");

class Db2Handle extends DBHandle {
	
	public function native_connect($dsn, $options = null) {
		$db_username = $dsn['username'];
		$db_password = $dsn['password'];
		$db_hostname = $dsn['hostname'];
		$db_database = $dsn['database'];
		$db_port = $dsn['tcp_port'];

		$dbase = explode("?", $db_database);
		// attempt to connect uncatalogued first.
		if ($db_port && $dbh = db2_connect("DATABASE=$db_database;HOSTNAME=$db_hostname;PORT=$db_port;PROTOCOL=TCPIP;UID=$db_username;PWD=$db_password")) {
			return $dbh;
		} else {
			// otherwise assume we connect catalogued
			if ($dbh = db2_connect($db_database, $db_username, $db_password, array("autocommit" => DB2_AUTOCOMMIT_ON))) {
				return $dbh;
			} else {
				$this->setErrorMessage(db2_conn_error());
				return null;
			}
		}
	}
	
	public function native_prepare($query) {
		if (!$stmt = db2_prepare($this->db, $query)) {
			$this->setErrorMessage("Failed to prepare query [$query] : " .db2_stmt_errormsg());
		}
		return $stmt;
	}


	function native_param_fixup($types = array(), $params = array()) {
		// This is where we do any needed substitution on bind parameters for this db.
		return $params;
	}


	public function native_query($statement) {
		if (!$result = db2_exec($this->db, $statement)) {
			$this->setErrorMessage("Failed to query [$statement] : " . db2_stmt_error());
			return null;
		}
		return $result;
	}


	public function native_execute($stmt, $types, $params) {
		$ret = db2_execute($stmt, $params);
		if (!$ret) {
			$this->setErrorMessage("Failed to execute prepared query [$this->lastQuery] with bind values of ["
					. join(",",$params)."]: " . db2_stmt_errormsg($stmt));
		}
		return $ret;
	}
	
	public function native_list_tables() {
		return "SELECT table_name from SYSIBM.tables_s where is_insertable_into='YES' and table_schema = CURRENT_SCHEMA";
	}

	public function native_fetchrow($sth, $options) {
		$result = array();
		if ($options != DB_FETCHMODE_ASSOC) {
			if (!$results = db2_fetch_array($sth)) {
				return null;
			}
			return $result;	
		}
		if (!$results = db2_fetch_assoc($sth)) {
			return null;
		}
		$lowered = array();
	  	foreach ($results as $key => $val) {
	  			$k = strtolower($key);
	  			$lowered[$k] = $val;
	  	}
	  	return $lowered;
	}


	// have to immulate PEAR::db's tableInfo.
	public function tableInfo($tbl) {
		$id = @db2_exec($this->db, "SELECT * FROM $tbl FETCH FIRST 1 ROW ONLY");
		if (!$id) {
		$this->setErrorMessage(db2_stmt_errormsg());
			return array();
		}
		$count = @db2_num_fields($id);
		$res   = array();
	
		for ($i = 0; $i < $count; $i++) {
		$res[$i] = array(
			'table' => $tbl,
			'name'  => strtolower(@db2_field_name($id, $i)),
			'type'  => @db2_field_type($id, $i),
			'len'   => @db2_field_width($id, $i),
			'flags' => '',
		);
		}   
		@db2_free_result($id);
			return $res; 
	}
	
	public function native_disconnect($dbh) {
		db2_close($dbh);
	}
        
}

?>
