<?php

require_once("DBHandle.php");

class MysqlHandle extends DBHandle {
	
	public function native_connect($dsn, $options = null) {
		$db_username = $dsn['username'];
		$db_password = $dsn['password'];
		$db_hostname = $dsn['hostname'];
		$db_database = $dsn['database']; 
		
		// $this->quotedTypes = array(253, 254);
		$this->quotedTypes = array('char', 'varchar', 'text', 253, 254);
		$this->intTypes = array(1, 2, 3, 8, 9);
		$this->doubleTypes = array(5, 4, 0, 246);
		if ($dbh = mysqli_connect($db_hostname,$db_username,$db_password, $db_database)) {
			if (function_exists('mysqli_set_charset') && mysqli_set_charset($dbh, "utf8")) {
				$charset = mysqli_character_set_name($dbh);
			}
			return $dbh;
		} else {
			$this->setErrorMessage(mysqli_connect_error());
			return null;
		}
	}
	
	public function native_prepare($query) {
		/* if (!$stmt = mysqli_prepare($this->db, $query)) {
			$this->setErrorMessage("Error in preparing query [$query] : " . mysqli_error());	
		}
		return $stmt; */
		$this->setErrorMessage("Don't call execute on a MysqlHandle.\n");
		return null;
	}
	
	function native_param_fixup($types = array(), $params = array()) {
		// This is where we do any needed substitution on bind parameters for this db.
		$newParams = array();
		if (count($types) != count($params)) {
			return null;
		}
		for($count = 0; $count < count($types); $count++) {
			if (substr($types[$count], 0, 1) == ':') {
				// we have a ':type' instead of a column.
				if (substr($types[$count], 1) == 'string') {
					$newParams[$count] = "'". $params[$count] . "'";
				} else {
					$newParams[$count] = $params[$count];
				}
			} else {
				// 'table.column' format was used
				$info = split("\.", $types[$count]);
				$type = $this->columnCache->getColumnType($info[0], $info[1]);
				if (in_array($type, $this->quotedTypes)) {
					$newParams[$count] = "'". $params[$count] . "'";
				} else {
					$newParams[$count] = $params[$count];
				}
			}
		}
		
		return $newParams;
	}
	

	public function native_query($statement) {
		if (!$result = mysqli_query($this->db, $statement)) {
			$this->setErrorMessage("Error running query [$statement] : " . mysqli_error($this->db));
		}
		return $result;
	}

	/*   giving up on this, but we might fix it in the future to gain prepare/execute abilities.
	public function process_mysql_types($types) {
			$newTypes = "";
			for($count = 0; $count < count($types); $count++) {
				if (substr($types[$count], 0, 1) == ':') {
					// we have a ':type' instead of a column.
					$typeval = substr($types[$count], 1);
					if ($typeval  == 'string') {
						$newTypes .= "s";
					} else if ($typeval == 'int') {
						$newTypes .= "i";
					} else if ($typeval == 'double') {
						$newTypes .= "d";
					} else {
						$newTypes .= "b";
					}
				} else {
					// 'table.column' format was used
					$info = split("\.", $types[$count]);
					$type = $this->columnCache->getColumnType($info[0], $info[1]);
					if (in_array($type, $this->quotedTypes)) {
						$newTypes .= "s";
					} else if (in_array($type, $this->intTypes)) {
						$newTypes .= "i";
					} else if (in_array($type, $this->doubleTypes)) {
						$newTypes .= "d";
					} else {
						$newTypes .= "b";
					}
				}
				$count++;
			}
		return $newTypes;
	}
	*/

	public function native_execute($stmt, $types, $params) {
		// mysqli has prepare / execute but they are jacked up... 
		/* $typeString = $this->process_mysql_types($types);
		print "sql [$this->lastQuery] : bind types [$typeString] : params [" . join(",",$params)."]\n";
		if (mysqli_stmt_bind_param($stmt, $typeString, $params)) {
			if (mysqli_stmt_execute($stmt)) {
				mysqli_stmt_store_result($stmt);
				return $stmt;
			} 
		}
		$this->setErrorMessage("Error executing prepared statment : " . mysqli_stmt_error($stmt));
		mysqli_stmt_close($stmt);
		return $stmt; */
		$this->setErrorMessage("Don't call execute on a MysqlHandle.\n");
		return null;
	}
	
	public function native_list_tables() {
		return "show tables";
	}

	public function native_fetchrow($sth, $options) {
		if (!$sth) {
			$this->setErrorMessage(mysqli_error($this->db));
			return null;
		}
		if ($options != DB_FETCHMODE_ASSOC) {
			return mysqli_fetch_array($sth, MYSQLI_NUM);
		}
		return mysqli_fetch_array($sth, MYSQLI_ASSOC);
	}


	// have to immulate PEAR::db's tableInfo.
	public function tableInfo($tbl) {
		if (!$id = $this->native_query("SELECT * FROM $tbl LIMIT 0")) {
        	return array();
        }
        
        $count = @mysqli_num_fields($id);
        $res   = array();

        for ($i = 0; $i < $count; $i++) {
            $tmp = @mysqli_fetch_field($id);
            $col = $i + 1;
			$res[$i] = array(
                'table' => strtolower($tmp->table),
                'name'  => strtolower($tmp->name),
                'type'  => $tmp->type,
                'len'   => $tmp->max_length,
                'flags' => '',
            );
        }   
		@mysqli_free_result($id);
		return $res; 
	}
	
	public function native_disconnect($dbh) {
		mysqli_close($dbh);
	}
        
}

?>
