<?php

require_once("DBHandle.php");

class OdbcHandle extends DBHandle {
	
	public function native_connect($dsn, $options = null) {
		$db_username = $dsn['username'];
		$db_password = $dsn['password'];
		$db_hostname = $dsn['hostname'];
		$db_database = $dsn['database'];

		$this->quotedTypes = array('char', 'varchar', 'string', 'text');	
		
		if ($dbh = odbc_connect($db_database, $db_username, $db_password)) {
			return $dbh;
		} else {
			$this->setErrorMessage(odbc_error() . " : " . odbc_errormsg());
			return null;
		}
	}
	
	public function native_param_fixup($types = array(), $params = array()) {
		global $db_provider;
		// This is where we do any needed substitution on bind parameters for this db.
		if (count($types) != count($params)) {
			return null;
		}
		if ($db_provider == "sqlserver") {
			//sqlserver section
			// This is where we do any needed substitution on bind parameters for this db.
			$newParams = array();
			for($count = 0; $count < count($types); $count++) {
				if (substr($types[$count], 0, 1) == ':') {
					// we have a ':type' instead of a column.
					if (substr($types[$count], 1) == 'string') {
						$newParams[$count] = "'". $params[$count] . "'";
					} else {
						$newParams[$count] = $params[$count];
					}
				} else {
					// 'table.column' format was used
					$info = split("\.", $types[$count]);
					$type = $this->columnCache->getColumnType($info[0], $info[1]);
					if (in_array($type, $this->quotedTypes)) {
						$newParams[$count] = "'". $params[$count] . "'";
					} else {
						$newParams[$count] = $params[$count];
					}
				}
			}
		
			return $newParams;
		} else {
			//sybase section
			// This is where we do any needed substitution on bind parameters for this db.
			$newParams = array();
			for($count = 0; $count < count($types); $count++) {
				if (substr($types[$count], 0, 1) == ':') {
					// we have a ':type' instead of a column.
					if (substr($types[$count], 1) == 'string') {
						$newParams[$count] = "'". $params[$count] . "'";
					} else {
						$newParams[$count] = $params[$count];
					}
				} else {
					// 'table.column' format was used
					$info = split("\.", $types[$count]);
					$type = $this->columnCache->getColumnType($info[0], $info[1]);
					if (in_array($type, $this->quotedTypes)) {
						$newParams[$count] = "'". $params[$count] . "'";
					} else {
						$newParams[$count] = $params[$count];
					}
				}
			}
		
			return $newParams;
		}
	}
	
	
	public function native_prepare($query) {
		$this->setErrorMessage("You should not call prepare on ODBC.");
		return null;
	}


	public function native_query($statement) {
		$result = odbc_exec($this->db, $statement);
		if (!$result) {
			$this->setErrorMessage(odbc_error($this->db) . " : " . odbc_errormsg($this->db));
			return null;
		}
		return $result;
	}


	public function native_execute($sth, $types, $params) {
		$this->setErrorMessage("You should not call execute on ODBC.");
		return null;
	}

	
	public function native_list_tables() {
		$results = array();
		$r = odbc_tables($this->db);
		while (odbc_fetch_row($r)) {
			if (odbc_result($r, "TABLE_TYPE") == "TABLE") {
				if ($tbl = odbc_result($r, "TABLE_NAME")) {
					if (strtoupper(substr($tbl, 0, 2)) == 'BF') {
						array_push($results, $tbl);
					}
				}
			}
		}
		return $results;
	}


	public function native_fetchrow($sth, $options) {
		$result = array();
		
		$r = odbc_fetch_into($sth,$result);
		if ($r == FALSE) {
			DBError("Error fetching results.");	
			return null;
		} 
		if ($options != DB_FETCHMODE_ASSOC) {
			return $result;	
		}
		$newResults = array();
		$num_fields = odbc_num_fields($sth);
		for ($i = 0 ; $i < $num_fields ; $i++) {
			$name = strtolower(odbc_field_name($sth,$i+1));
			$newResults[$name] = $result[$i];
		}
		return $newResults;
	}


	// have to immulate PEAR::db's tableInfo.
	public function tableInfo($tbl) {
		$id = @odbc_exec($this->db, "SELECT * FROM $tbl");
        if (!$id) {
        	return $this->odbcRaiseError();
        }
        $count = @odbc_num_fields($id);
        $res   = array();

        for ($i = 0; $i < $count; $i++) {
            $col = $i + 1;
            $res[$i] = array(
                'table' => $tbl,
                'name'  => strtolower(@odbc_field_name($id, $col)),
                'type'  => @odbc_field_type($id, $col),
                'len'   => @odbc_field_len($id, $col),
                'flags' => '',
            );
        }   
		@odbc_free_result($id);
		return $res; 
	}

	
	public function native_disconnect($dbh) {
		odbc_close($dbh);
	}
        
}

?>
