<?php

require_once("DBHandle.php");

class OracleHandle extends DBHandle {
	
	public function native_connect($dsn, $options = null) {
		$db_username = $dsn['username'];
		$db_password = $dsn['password'];
		$db_hostname = $dsn['hostname'];
		$db_database = $dsn['database']; 
		
		$this->quotedTypes = array('CHAR', 'VARCHAR', 'VARCHAR2');

		if ($dbh = oci_connect($db_username, $db_password,$db_database)) {
			return $dbh;
		} else {
			$e = ocierror();
			$this->setErrorMessage("Unable to connect to Oracle : [" . $e['message']);
			return null;
		}
	}
	
	public function native_prepare($query) {
		$this->setErrorMessage("You should not call prepare on Oracle.");
		return null;
	}

	function native_param_fixup($types = array(), $params = array()) {
		// This is where we do any needed substitution on bind parameters for this db.
		// due to Oracle not having an empty string value (equates it to null)
		// we are using a space for Oracle. 
		$newParams = array();
	
		for($count = 0; $count < count($types); $count++) {
			$params[$count] = ($params[$count] == '') ? ' ' : $params[$count];
			if (substr($types[$count], 0, 1) == ':') {
				// we have a ':type' instead of a column.
				if (substr($types[$count], 1) == 'string') {
					$newParams[$count] = "'". $params[$count] . "'";
				} else {
					$newParams[$count] = $params[$count];
				}
			} else {
				// 'table.column' format was used
				$info = split("\.", $types[$count]);
				$type = $this->columnCache->getColumnType($info[0], $info[1]);
				if (in_array($type, $this->quotedTypes)) {
					$newParams[$count] = "'". $params[$count] . "'";
				} else {
					$newParams[$count] = $params[$count];
				}
			}
		}
	
		return $newParams;
	}




	public function native_query($statement) {
		if (!$stmt = ociparse($this->db, $statement)) {
			$e = ocierror($this->db);
			$this->setErrorMessage("[oracle error] ". $e['code'] . ": ". $e['messsage'] . " : " . $e['sqltext']);
			return null;
		}
		if (!ociexecute($stmt, OCI_COMMIT_ON_SUCCESS)) {
			$e = ocierror($stmt);
			$this->setErrorMessage("[oracle error] ". $e['code'] . " : " . $e['sqltext']);
			return null;
		}
		return $stmt;
	}


	public function native_execute($stmt, $types, $params) {
		$this->setErrorMessage("You should not call execute on Oracle.");
		return null;
	}
	
	public function native_list_tables() {
		return "SELECT table_name FROM user_tables WHERE table_name like 'BF_%'";
	}

	public function native_fetchrow($sth, $options) {
		$result = array();
		
		if ($options != DB_FETCHMODE_ASSOC) {
			ocifetchinto($sth, $result, OCI_RETURN_NULLS);
		  	for ($i = 0; $i < count($result); $i++) {
		  		if ($result[$i] == " ") {
		  			$result[$i] = "";	
		  		}
		  	}
		  	return $result;
		} 
		
		ocifetchinto($sth, $result, OCI_ASSOC | OCI_RETURN_NULLS);
	  	$lowered = array();
	  	foreach ($result as $key => $value) {
	  			$k = strtolower($key);
	  			if ($value == " ") {
	  				$value = "";
	  			}
	  			$lowered[$k] = $value;
	  	}
	  	return $lowered;
	}


	// have to immulate PEAR::db's tableInfo.
	public function tableInfo($tbl) {
		$tbl = strtoupper($tbl);
        $q_fields = 'SELECT column_name, data_type, data_length, '
                    . 'nullable '
                    . 'FROM user_tab_columns '
                    . "WHERE table_name='$tbl' ORDER BY column_id";

		$tbl = strtolower($tbl);
		if (!$stmt = $this->native_query($q_fields)) {
			return array();	
		}
		
		$i = 0;
        while ($row = $this->native_fetchrow($stmt, DB_FETCHMODE_ASSOC)) {
            $res[$i] = array(
                'table' => $tbl,
                'name'  => strtolower($row['column_name']),
                'type'  => $row['data_type'],
                'len'   => $row['data_length'],
                'flags' => ($row['nullable'] == 'N') ? 'not_null' : '',
            );
            $i++;
        }   
		$this->native_free_statement($stmt);
		return $res; 
	}
	
	public function native_free_statement($stmt) {
		oci_free_statement($stmt);
	}
	
	public function native_disconnect($dbh) {
		oci_close($dbh);
	}
        
}

?>
