<?php

global $APILanguage;

/**
 * Message Class used for localization of the API messages
 * 
 */

class Messages {
	private $language = "";
	private $languagePackPath = "./lang/";
	private $packName = "language.php";

	/*
	 * Set the current language.
	 * Should be set to language "_" Region (ex. "en_US")
	 */
	
	public function setLang($lang) {
		global $APILanguage;
		if ($lang !== $this->language || empty($APILanguage))
			$this->getDictionary($lang);
		$this->language = $lang;
		
	}	

	/*
	 * Get the current language.
	 * Should be set to language "_" Region (ex. "en_US")
	 */
	public function getLang() {
		return $this->language;
	}
	
	/*
	 * Get a language pack message with substituted values
	 * for any %s characters.
	 */
	public function getMessage($messageKey, $params = array()) {
		global $APILanguage;
		
		if (empty($APILanguage)) {
			$this->getDictionary($this->language);
		}
		$msg = isset($APILanguage[$messageKey]) ? $APILanguage[$messageKey] : $messageKey;
		$message = $msg;
		if (count($params) > 0) {
			$message = vsprintf($msg, $params);	
		}
		if (empty($msg) && !empty($messageKey))
			$message = $messageKey;
		return $message;
	}
	
	
	/*
	 *	Populate APILanguage variable by loading the appropriate language pack. 
	 */
	private function getDictionary($lang) {
		global $APILanguage;
		$path = $this->languagePackPath . $lang . '/' . $this->packName;
		if (file_exists($path)) {
			include($path);
		}
	}
	
	/*
	 * Expand any Message Keys in the given buffer and return the new string.
	 * Note parameters to the keys are ':' delimited and keys should be hex
	 * encoded.
	 */
	public function ExpandMessageTokens($buffer = "") {
		$values = explode("\n", $buffer);
		$results = "";
		foreach ($values as $v) {
			$msgs = explode(":::", $v);
			$key = $msgs[0];
			$params = array();
			for ($i=1; $i < count($msgs); $i++) {
				$params[$i - 1] = pack('H*',$msgs[$i]);
			}
			$results .= $this->getMessage($key,$params);
		}
		return $results;
	}
	
	/*
	 * Build up a string that contains the message key and delimited parameters
	 * in a format that ExpandMessageTokens will accept.
	 * Format is the string that is the key followed by ':' delimited hex encoded 
	 * parameters. (hex encoded to insure no delimiter issues)
	 */
	public static function buildMessageParamString($message, $params = array()) {
		$results = $message;
		foreach ($params as $parm) {
			$results .= ':::'; 
			$enc = @unpack("H*stuff",$parm);
			$results .= $enc['stuff'];
		}		
		return $results;
	}
	
	public static function getUserLang($dbh, $user = "root") {
		
	}
	
	function __construct($lang = "en_US") {
		$this->setLang($lang);
		$this->getDictionary($lang);
	}

}


?>