<?php
/*
Copyright (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/


// os-safe directories
function fixpath($dir) {
	$dir = str_replace('/', DIRECTORY_SEPARATOR, $dir);
	$dir = str_replace('\\', DIRECTORY_SEPARATOR, $dir);

	return $dir;
}

// remove a directory. Recursive, like rm -rf
function recrmdir($dir) {
	$dh = opendir($dir);
	if (! $dh) return false;

	while ($entry = readdir($dh)) {
		if ($entry == '.' || $entry == '..') {
			continue;
		}

		$tmp = $dir . DIRECTORY_SEPARATOR . $entry;
		if (is_dir($tmp)) {
			if (! recrmdir($tmp) )
				return false;
		}
		else {
			if (! unlink($tmp) )
				return false;
		}
	}

	closedir($dh);

	if (rmdir($dir))
		return true;
	else
		return false;
}

// make a directory. Recursive, like mkdir -p
function recmkdir($dir) {
	if ($dir == '.' || $dir == ('.' . DIRECTORY_SEPARATOR . '.'))
		return true;

	$part = split('[/\\]', $dir);
	$tmpd = '.';

	foreach ($part as $segment) {
		$tmpd .= DIRECTORY_SEPARATOR . $segment;
		mkdir($tmpd);
	}

	return is_dir($dir);
}

// error exit point. do all your cleanup here
function err_quit($dir, $msg, $initdir = 0) {
	global $PageError, $Msg;
	$PageError = $msg;

	if (! recrmdir($dir) ) {
		$PageError .= "\n" . $Msg->buildMessageParamString("PageErrorReflectFilesDelete");
	}

	if ($initdir)
		chdir($initdir);
}

// create dir and unzip file
function unzip_stuff($basedir, $newdir, $index) {
	global $relpath, $Msg;

	// unzip files into the right path
	// each file has an associated path
	$path = $newdir . DIRECTORY_SEPARATOR . $relpath[$index];
	$path = fixpath($path);

	// create path
	if (!recmkdir($path)) {
		$PageError = $Msg->buildMessageParamString("PageErrorReflectFilesSpace");
		return false;
	}

	// unzip files into it
	chdir($path);
	if (! unzip_file($_FILES['zipfile']['tmp_name'][$index]) ) {
		chdir($basedir);
		return false;
	}
	chdir($basedir);

	return true;
}

// unzip one file
function unzip_file($zipfile) {
	global $PageError, $Msg;

	$zip = zip_open($zipfile);

	if (!$zip) {
		$PageError = $Msg->buildMessageParamString("PageErrorReflectZipOpen");
		return false;
	}

	while ( $zip_entry = zip_read($zip) ) {
		$full_path = zip_entry_name($zip_entry);
		$full_path = fixpath($full_path);

		$path = dirname($full_path);

		if ($path != '.') {
			if (! recmkdir($path) ) {
				$PageError = $Msg->buildMessageParamString("PageErrorReflectUnzip");
				return false;
			}
		}

		if (! zip_entry_open($zip, $zip_entry, 'rb') ) {
			$PageError = $Msg->buildMessageParamString("PageErrorReflectZipEntry", array($zip_entry));
			return false;
		}

		$fp = fopen($full_path, 'w');
		
		if (! $fp) {
			$PageError = $Msg->buildMessageParamString("PageErrorReflectUnZipFiles");
			return false;
		}

		$buf = zip_entry_read($zip_entry, zip_entry_filesize($zip_entry));

		if ($buf != false) {
			if (! fwrite($fp, $buf) ) {
				$PageError = $Msg->buildMessageParamString("PageErrorReflectUnzipWrite");
				return false;
			}
		}

		fclose($fp);
		zip_entry_close($zip_entry);
	}

	zip_close($zip);

	return true;
}

// Handle reflector requests
if ($Command == "Reflectar") {
	_apisend('Debug1','Entered reflector module');

	// do we have the right modules?
	if (!extension_loaded('zip') && !extension_loaded('php_zip')) {
		$PageError = $Msg->buildMessageParamString("PageErrorReflectPHPZipExt");
		return;
	}

	// where are we?
	$initdir = getcwd();

	// check if we have a license
	if(!GetSysParam($dbh,"feature_reflectors")) {
		$PageError = $Msg->buildMessageParamString("PageErrorReflectLicense");
		return;
	}

	// create dir for this build
	$basedir = '..' . DIRECTORY_SEPARATOR .  '..' . DIRECTORY_SEPARATOR . '..';
	if (is_file($basedir . DIRECTORY_SEPARATOR . 'Platform' .  DIRECTORY_SEPARATOR . 'buildforge.pl'))
		$basedir = $basedir . DIRECTORY_SEPARATOR .  'Platform';
	elseif (is_file($basedir . DIRECTORY_SEPARATOR . 'engine' .  DIRECTORY_SEPARATOR . 'buildforge.pl'))
		$basedir = $basedir . DIRECTORY_SEPARATOR . 'engine';
	elseif (is_file($basedir . DIRECTORY_SEPARATOR . 'buildforge.exe'))
		$basedir = $basedir;
	else {
		$PageError = $Msg->buildMessageParamString("PageErrorReflectEngineLocation");
		return;
	}
	$basedir = $basedir . DIRECTORY_SEPARATOR . 'reflector';

	$tmpdir = $basedir . DIRECTORY_SEPARATOR .
		$_SERVER['REMOTE_ADDR'] . '.' . time();


	_apisend('Debug1','Unzipping project files');
	foreach ($Project as $index => $foo) {
		$newdir = $tmpdir . DIRECTORY_SEPARATOR . $SID[$index];

		if (! recmkdir($newdir, 0755) ) {
			$PageError = $Msg->buildMessageParamString("PageErrorReflectTempDir");
			return;
		}

		// unzip all files into that dir
		if (! unzip_stuff($initdir, $newdir, $index) ) {
			err_quit($tmpdir, $PageError);
			return;
		}
	}

	// fire off build
	_apisend('Debug1','Firing prelimnary build');
	include('c_build.php');
	$BID = FireBuild($RunProject, 1);
	if ($BID < 0) {
		$PageError .= $Msg->buildMessageParamString("PageErrorReflectBuildCreate");
		err_quit($tmpdir, $PageError);
		return;
	}

	// build is in the 'P' state, so it won't run.
	// rename the dir to the buildID so the engine
	// knows where to pick stuff up.
	_apisend('Debug1','Renaming unzipped files');
	_apisend('Debug1', "Moving unzipped files from [" . $tmpdir . "] to [" . $basedir . DIRECTORY_SEPARATOR . $BID . "].");
	if (! rename($tmpdir, $basedir .  DIRECTORY_SEPARATOR . $BID) ) {
		err_quit($tmpdir, $Msg->buildMessageParamString("PageErrorReflectZipMove", array($tmpdir,$basedir . DIRECTORY_SEPARATOR . $BID)));
		return;
	}

	// okay. Now reset the state so that it will run.
	_apisend('Debug1','Firing build for realz');
	param_query($dbh,  'CREFLECT_UPDATE_FIRE_BUILD', 
				array('bf_builds.bf_bid'), array($BID));

	// and back to where we were
	chdir($initdir);

	// send data back to user
	$PageInfo = "Build ID is $BID";
	$PageInfo = $Msg->buildMessageParamString("PageInfoReflectBuildId", array($BID));
	_apisend('BuildID', $BID);
}
addaudit(0,$PageInfo);
?>
