<?php
/*
Copyright (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/


require_once("DB/Db2Handle.php");
require_once("DB/DBLock.php");

$quotedTypes = array('char', 'varchar', 'string', 'text');

function db_open($config) {
	global $PageError, $columnCache;
	$dbh = new Db2Handle('db2');
	$dbh->connect($config);
	if ($dbh->getErrorMessage() != null) {
		DBError($dbh->getErrorMessage());
		$dbh->clearErrorMessage();
	}
	$dbh->generateColumnCache('db2');
	
	return $dbh;
}

function db_close($dbh) {
	return $dbh->disconnect($dbh);
}

function db_fetchrow_array($sth) {
	global $PageError, $Msg;
	if (!$sth) {
		DBError($Msg->getMessage("DBInvalidStatementHandle"));
		return null;		
	}		
	return db2_fetch_array($sth);
}

function db_fetchrow_hashref($sth) {
	global $PageError, $Msg;
	$results = array();
	
	if (!$sth) {
		DBError($Msg->getMessage("DBInvalidStatementHandle"));
		return null;		
	}	
	$results = db2_fetch_assoc($sth);

	if ($results == null) {
		return null;
	}
	$lowered = array();
  	foreach ($results as $key => $val) {
  			$k = strtolower($key);
  			$lowered[$k] = $val;
  	}
  	return $lowered;
}


function db_stmt_fixup($statement) {
 
  $statement = preg_replace('/\\\"/','"', $statement); //undo addslashes
  $statement = str_replace("\\\\","\\",$statement);  //undo addslashes
  $statement = preg_replace("/\\\'/","''", $statement);

  $statement = preg_replace("/SUBSTRING\(/", "SUBSTR(", $statement);
  if (preg_match("/^\s*create/i", $statement)) {
	$statement = preg_replace("/TEXT/", "VARCHAR2(4000)", $statement);
  }
  
  return $statement;
}


function db_query($dbh,$statement) {
	$statement = db_stmt_fixup($statement);

	$ret = $dbh->do_query($statement);
	if ($dbh->getErrorMessage() != null) {
		DBError($dbh->getErrorMessage());
		$dbh->clearErrorMessage();
	} else if ($ret == null) {
		DBError(odbc_error() . " : " . odbc_errormsg());
	}
	return $ret;
}


function param_query($dbh, $statement, $types = array(), $params = array()) {
	// $params = param_fixup($types, $params);
	/* if ($params === null) {
		DBError("Parameter mismatch with statement [$statement] types [". 
				join(",", $types)."] parameters [". join(",", $params)."]");
	} */
	return $dbh->do_query($statement, $types, $params);
}

function db_do($dbh,$statement) {
	return db_query($dbh, $statement);
}


function db_free($id) {
  	if (is_a($id, 'DB_result')) {
		$id->free();
	} else {
		return true;
	}
}


function db_finish($id) {
	return(db_free($id));
}


function getDBErrorMessage($db = null, $msg = null) {
	return db2_stmt_errormsg();
}


function DBError($db_err) {
	global $PageError;

	$db_err = $Msg->getMessage($db_err);
	// debug_print_backtrace();
	if (is_a($db_err, 'DB_Error')) {
		$PageError .= $db_err->toString();
		trigger_error($db_err->toString(),E_USER_WARNING);
	} else {
		$PageError .= "$db_err\n";
		trigger_error("$db_err",E_USER_WARNING);
	}
}


?>
