<?php
/*
Copyright (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/


include_once("ldap_password.php");

global $domain, $Msg;

function ldap_load_config($domain,&$ldap_config) {
	global $ldap_debug;
	global $dbh;

	if (!isset($domain)) {
		return;
	}


	// load the LDAP config data for this domain into an associative array
	$h = db_query($dbh,"SELECT bf_param,bf_value FROM bf_ldap WHERE bf_domain='$domain'");
	while ($r = db_fetchrow_array($h)) {
		$ldap_config['ldap_'.$r[0]] = $r[1];
	}

	// split up/fix up the host/port fields
	$ldap_config['ldap_host']= $ldap_config['ldap_protocol'] . "://" . $ldap_config['ldap_host'];
	//if (!strlen($ldap_config['ldap_port']))
		//$ldap_config['ldap_port'] = "389";

	// decrypt the admin bind password
	if ($ldap_config['ldap_admin_password'] != "")
		$ldap_config['ldap_admin_password'] = bf_decrypt($ldap_config['ldap_admin_password']);

	// some field cleanup
	if ($ldap_config['ldap_distinguishedname']     == "") $ldap_config['ldap_distinguishedname']    = "distinguishedname";
	if ($ldap_config['ldap_displayname']           == "") $ldap_config['ldap_displayname'] = "displayname";
	if ($ldap_config['ldap_mailname']              == "") $ldap_config['ldap_mailname']    = "mail";
	if ($ldap_config['ldap_groupname']             == "") $ldap_config['ldap_groupname']    = "memberof";
	$ldap_config['ldap_distinguishedname']   = strtolower($ldap_config['ldap_distinguishedname']);
	$ldap_config['ldap_displayname']         = strtolower($ldap_config['ldap_displayname']);
	$ldap_config['ldap_mailname']            = strtolower($ldap_config['ldap_mailname']);
	$ldap_config['ldap_groupname']           = strtolower($ldap_config['ldap_groupname']);
	$ldap_config['ldap_memberof']            = strtolower($ldap_config['ldap_memberof']);

	// $ldap_config now has a clean set of ldap conf params for 
	if ($ldap_debug) ldap_dump("BuildForge LDAP Configuration",$ldap_config);
}

function ldap_dump($title,$var) {
	print "<hr size=1 noshade width=100%>$title<br><pre>";print_r($var);print "</pre>";
}

// returns an array of user fields
// or one element 'error'
function ldap_valid_user($domain,$login,$password,&$ldap_config) {
	global $ldap_debug, $Msg;
	$results = array();

	$results["error"] = "";
	if (!count($ldap_config) || !isset($login))
		return($results);


	$ldap_dname = $ldap_config['ldap_distinguishedname'];
	$ldap_query_fields = array($ldap_dname, $ldap_config['ldap_displayname'], $ldap_config['ldap_mailname'], $ldap_config['ldap_groupname']); //JT

	// connect to the ldap server
	$ds = ldap_connect($ldap_config['ldap_host']);
	if ($ds) {
		// bind as the admin account
		ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);
		$r = ldap_bind($ds,$ldap_config['ldap_admin_cn'],$ldap_config['ldap_admin_password']);
		if ($r) {
			// sub in the login string and go find the user account record
			$ldap_search_filter = str_replace("%",$login,$ldap_config['ldap_unique_identifier']);
			if ($ldap_debug) ldap_dump("User Search",array($ldap_config['ldap_search_base'],$ldap_search_filter,$ldap_dname));
			//escape parenthesis in search filter...
			if(strpos($ldap_search_filter,'(') || strpos($ldap_search_filter, ')'))
			{
				$ldap_search_filter = str_replace('(', '\(',$ldap_search_filter);
				$ldap_search_filter = str_replace(')', '\)',$ldap_search_filter);
			}
			$sr = ldap_search($ds, $ldap_config['ldap_search_base'], $ldap_search_filter, $ldap_query_fields); 
			if ($sr) {
				// there must be only one...
				$records = ldap_count_entries($ds,$sr);
				if ($records == 1) {
					// read the record and use the DN field to re-bind
					// or crypt and compare to the ldap entry to validate
	   				$ldap_info = ldap_get_entries($ds, $sr);
					if ($ldap_debug) ldap_dump("User Record",$ldap_info[0]);
					$ldap_user_bind_string = $ldap_info[0][$ldap_dname];
					if (is_array($ldap_user_bind_string))
						$ldap_user_bind_string = $ldap_user_bind_string[0];
					if (strtolower(substr($ldap_config['ldap_user_bind'],0,1)) == "n") {
						if ($ldap_debug) ldap_dump("Scripted Authentication",array($password,$ldap_info[0]));
						$r = ldap_valid_password($password,$ldap_info[0]);
					}
					else {
						if ($ldap_debug) ldap_dump("Re-bind Authentication",array($ldap_user_bind_string,$password));
						$r = ldap_bind($ds,$ldap_user_bind_string,$password);
					}
					if ($r) {
						// re-bind is ok or it crypt'd ok, valid user login.
						$results["fullname"] = $ldap_info[0][$ldap_config['ldap_displayname']][0];
						$results["fullname"] = preg_replace("@'@", "&#39;", $results["fullname"]);
						$results["email"] = $ldap_info[0][$ldap_config['ldap_mailname']][0];
						$results["map_groups"] = strtolower(substr($ldap_config['ldap_access_groups'],0,1));

						// get a list of groups this users is a member of, need to re-bind as the root user
						ldap_bind($ds,$ldap_config['ldap_admin_cn'],$ldap_config['ldap_admin_password']);
						$results["grouplist"] = array();
						ldap_get_groups($ds,$ldap_info[0],$results["grouplist"],$ldap_config);

						// the ldap_memberof defines a global "you can log in" group. if defined
						// you must be a member of that group
						if ($ldap_config['ldap_memberof'] != "") {
							
							foreach (array_keys($results["grouplist"]) as $grp) { //JT
								
								if ($ldap_config['ldap_memberof'] == $grp) {
									return($results);
								}
							}
							$results = array(); // not an authorized user!!
							$results["error"] = $Msg->buildMessageParamString("ErrorLDAPNoAuth");
						}
					}
					else
						$results["error"] = $Msg->buildMessageParamString("ErrorLDAPInvalidAuth");
				}
				elseif ($records == 0) 
					$results["error"] = $Msg->buildMessageParamString("ErrorLDAPUserNameMissing");
				else
					$results["error"] = $Msg->buildMessageParamString("ErrorLDAPAuthExists");
			}
			else
				$results["error"] = $Msg->buildMessageParamString("ErrorLDAPAuthUnknown");
		}
		else
			$results["error"] = $Msg->buildMessageParamString("ErrorLDAPAuthInvalid");
	}
	else
		$results["error"] = $Msg->buildMessageParamString("ErrorLDAPConnect");
	return($results);
}


function ldap_get_groups($ds,$ldap_info,&$results,&$ldap_config) {	

	global $ldap_debug;

	$ldap_filter = "";
 	$ldap_keys = array();
	$ldap_values = array();
	$groupname = $ldap_config['ldap_groupname'];
 	$groups = array();
	$sr = false;


	foreach ($ldap_info as $key => $val) {
		if (!preg_match("/\d+/",$key)) {
			if (($groupname != "")&&($groupname == $key)) { // check for the groupname before we flatten the results
       				if (is_array($val) && array_key_exists('count',$val)) {
              				$subcount = $val['count'];
              				for ($j = 0 ; $j < $subcount ; $j++) {
                       				array_push($groups,$val[$j]);
               				}
       				}
 			}

			if (is_array($val)) {
				$val = $val[0];
			}
			array_push($ldap_keys,"/%$key%/");
			array_push($ldap_values,$val);
		}
	}


 	if ($ldap_config['ldap_group_search_base'] == "") {
       		if (count($groups) != 0) { //The groups where in the user record (ala Active Directory)
               		foreach ($groups as $group) {
                       		ldap_unwind_groups($ds,$group,$results,$ldap_config);
               		}
               		return("");
       		} else {  
			//Its not in the user record but you didn't specify a group search? Yeah, bye
       			return("");
       		}
	}

	// Why are we reconnecting?? JT
  	$ds = ldap_connect($ldap_config['ldap_host']);

	// bind as the admin account - this has to have already worked
	ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);
	if(!ldap_bind($ds,$ldap_config['ldap_admin_cn'],$ldap_config['ldap_admin_password'])) {
		return(""); //couldn't bind
	}

	$ldap_filter = preg_replace($ldap_keys,$ldap_values,$ldap_config['ldap_group_unique_identifier']);
	$sr = ldap_search($ds, $ldap_config['ldap_group_search_base'], $ldap_filter, array("dn")); 
	if ($sr) {
		$records = ldap_count_entries($ds,$sr);
		$ldap_recs = ldap_get_entries($ds,$sr);
		for ($i = 0 ; $i < $records ; $i++) {
			//recursive support
			if(($groupname != "") && is_array($ldap_recs[$i][$groupname]) && array_key_exists('count',$ldap_recs[$i][$groupname])) {
				$subcount = $ldap_recs[$i][$groupname]['count'];
				for ($j = 0 ; $j < $subcount ; $j++) {
					ldap_unwind_groups($ds,$ldap_recs[$i][$groupname][$j]['dn'],$results,$ldap_config);
				}
			} else {
				ldap_unwind_groups($ds,$ldap_recs[$i]["dn"],$results,$ldap_config);
			}

		}

		return($ldap_filter);
	}

	return("");
}

function ldap_unwind_groups($ds,$group,&$results,&$ldap_config) {
	global $ldap_config;
	global $ldap_debug;

	$group = strtolower($group);
	if ($results[$group]++ > 5) // try and protect against recursion
		return;

	//This is a place holder for a better unwind support	

	return;

}

function ldap_map_groups($userid,$results) {
	global $ldap_debug;
	global $dbh;

	// this won't be defined for local accounts but that's ok.
	if (!isset($results['map_groups']) || $results['map_groups'] != "y" || 
			!isset($results['grouplist']) || count($results['grouplist']) == 0)
		return;

	$lev_list = array();

	$h = db_query($dbh,"SELECT bf_level,bf_dn AS bf_dname,bf_name FROM bf_levels");
	while ($r = db_fetchrow_array($h)) {
		$level = $r[0];
		$r[1] = BlockFetch($r[1]);
		// if there is a DN pattern defined, check the incoming user to see if they're in one of them.
		if ($r[1] != "") {
			// '*' means everyone gets this
			if ($r[1] == "*") {
				$lev_list[$level] = $r[2];
			}
			else {
				// iterate over the list of access DN's looking for them in the user's DN list
				foreach (explode(";",$r[1]) as $bf_grp) {
					$bf_grp = strtolower($bf_grp);
					foreach (array_keys($results["grouplist"]) as $ldp_grp) {
						// if the access control DN (bf_dname matches the user's DN ($ldp_grp)
						// then they have access to this group.
						if ($bf_grp == $ldp_grp) {
							if ($ldap_debug) print "specific match: [$bf_grp]<br>";
							$lev_list[$level] = $r[2];
						}
					}
				}
			}
		}
		else {
			$pat = "/^..=$r[2],/i";
			if (isset($results['grouplist']))  {
				foreach (array_keys($results["grouplist"]) as $ldp_grp) {
					if (preg_match($pat,$ldp_grp)) {
						if ($ldap_debug) print "name match: [$pat][$ldp_grp]<br>";
						$lev_list[$level] = $r[2];
					}
				}
			}
		}
	}
	db_query($dbh,"DELETE FROM bf_access WHERE bf_userid=$userid");
	foreach (array_keys($lev_list) as $level) {
		db_query($dbh,"INSERT INTO bf_access VALUES($level,$userid,0)");
		if ($ldap_debug) print "grant: $userid - $lev_list[$level]<br>";
	}

	resetAccessCacheDirty($dbh);

}







// including this file loads up a global ldap_config array
$ldap_config = array();
ldap_load_config($domain,$ldap_config);

?>
