<?php
/*
Copyright (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

//require_once("DBHandle.php");
require_once("DB/DBLock.php");
require_once("DB/OdbcHandle.php");


function db_open($config) {
	global $PageError, $db_provider, $db_type,$columnCache;
	$dbh = new OdbcHandle($db_provider);
	$dbh->connect($config);
	if ($dbh->getErrorMessage() != null) {
		DBError($dbh->getErrorMessage());
		$dbh->clearErrorMessage();
	}
	
	$dbh->canPrepare(false);
	$dbh->generateColumnCache($db_provider);
	
	return $dbh;
}


function db_close($dbh) {
	return $dbh->disconnect($dbh);
}

function db_fetchrow_array($sth) {
	global $PageError, $db_provider, $Msg;
	$result = array();
	$num_fields = 0;
	if (!$sth) {
		DBError($Msg->getMessage("DBInvalidStatementHandle"));
		return null;		
	}
	$r = odbc_fetch_into($sth,$result);
	return $result;
}


function db_fetchrow_hashref($sth) {
	global $Msg;
	if (!$sth) {
		DBError($Msg->getMessage("DBInvalidStatementHandle"));
		return null;		
	}
	if (!($row = db_fetchrow_array($sth)))
		return(null);

	// make $row (an array) into an associative array
	$result = array();
	
	$num_fields = odbc_num_fields($sth);
	for ($i = 0 ; $i < $num_fields ; $i++) {
		$name = strtolower(odbc_field_name($sth,$i+1));
		$result[$name] = $row[$i];
	}
	return($result);
}


function db_stmt_fixup($statement) {
	global $db_provider;
	if ($db_provider == "sybase") {
		// SUBSTRING(field,1,4096) => field
		//$statement = preg_replace("/SUBSTRING\((.*?),\d+,\d+\)/","$1",$statement);
		//$statement = str_replace("''","' '",$statement);
		//$statement = str_replace("\\\"","\"\"",$statement);
		//$statement = str_replace("\\'","''",$statement);
		$statement = str_replace("\\\\","\\",$statement);
	}
	elseif ($db_provider == "sqlserver") {
		$statement = str_replace("\\\\","\\",$statement);
		$statement = str_replace("\\\"","\"",$statement);
	}
	return($statement);
}

function db_query($dbh,$statement) {
	$statement = db_stmt_fixup($statement);
	$ret = $dbh->do_query($statement);
	if ($dbh->getErrorMessage() != null) {
		DBError($dbh->getErrorMessage());
		$dbh->clearErrorMessage();
	} else if ($ret == null) {
		DBError(odbc_error() . " : " . odbc_errormsg());
	}
	return $ret;
}




function param_query($dbh, $statement, $types = array(), $params = array()) {
	if (!$ret = $dbh->do_query($statement, $types, $params)) {
		DBError($dbh->getErrorMessage());
	}
	return $ret;
}


function db_do($dbh,$statement) {
	$statement = db_stmt_fixup($statement);
	if (!$ret = $dbh->do_query($statement)) {
		if ($dbh->getErrorMessage() != null) {
			DBError($dbh->getErrorMessage());
			$dbh->clearErrorMessage();
		} else if ($ret == null) {
			DBError(odbc_error() . " : " . odbc_errormsg());
		}
	}
	return $ret;
}

function db_free($id) {
		odbc_free_result($id);
}


function db_finish($id) {
	return(db_free($id));
}


function getDBErrorMessage($db, $msg = null) {
	return odbc_errormsg($db);
}


function DBError($db_err) {
	global $PageError;
	
	$db_err = $Msg->getMessage($db_err);
	//debug_print_backtrace();
	$PageError .= "$db_err\n";
	trigger_error("$db_err",E_USER_WARNING);
}


?>
