<?php
/*
Copyright (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

require_once("DB/OracleHandle.php");
require_once("DB/DBLock.php");


function db_open($config) {
	$dbh = new OracleHandle('oci8');
	$dbh->connect($config);
	if ($dbh->getErrorMessage() != null) {
		DBError($dbh->getErrorMessage());
		$dbh->clearErrorMessage();
	}
	
	$dbh->canPrepare(false);
	$dbh->generateColumnCache('oci8');
	
	return $dbh;
}

function db_close($dbh) {
	return $dbh->disconnect($dbh);
}

function db_fetchrow_array($sth) {
	global $PageError;
	$results = array();
	if (!ocifetchinto($sth, $results, OCI_RETURN_NULLS)) {
		return NULL;
	}
	
	for ($i = 0; $i < count($results); $i++) {
  		if ($results[$i] == " ") {
  			$results[$i] = "";	
  		}
  	}
  	
  	return $results;
}

function db_fetchrow_hashref($sth) {
	global $PageError;
	$results = array();
	if (!ocifetchinto($sth, $results, OCI_ASSOC | OCI_RETURN_NULLS)) {
		return NULL;
	}

	// ocifetchinto creates an associative array with UPPER case keys.  lets lower them.
  	$lowered = array();
  	foreach ($results as $key => $value) {
  			$k = strtolower($key);
  			if ($value == " ") {
  				$value = "";
  			}
  			$lowered[$k] = $value;
  	}
  	
  	return $lowered;
}


function db_stmt_fixup($statement) {

  $statement = preg_replace("/([,(])\s*\'\'\s*(?=[,\)])/","\\1' '", $statement);
  $statement = preg_replace("/\=\s*\'\'/","= ' '", $statement);
  $statement = preg_replace("/\\\'/","''", $statement);
  $statement = preg_replace('/\\\"/','"', $statement); //undo addslashes
  $statement = str_replace("\\\\","\\",$statement);  //undo addslashes

  $statement = preg_replace("/SUBSTRING\(/", "SUBSTR(", $statement);
  if (preg_match("/^\s*create/i", $statement)) {
  	$statement = preg_replace("/TEXT/", "VARCHAR2(4000)", $statement);
  }
  
  return $statement;
}


function db_query($dbh,$statement) {
	$statement = db_stmt_fixup($statement);
	
	$ret = $dbh->do_query($statement);
	if ($dbh->getErrorMessage() != null) {
		DBError($dbh->getErrorMessage());
		$dbh->clearErrorMessage();
	}
	return $ret;
}



function param_query($dbh, $statement, $types = array(), $params = array()) {
	$ret = $dbh->do_query($statement, $types, $params);
	if ($dbh->getErrorMessage() != null) {
		DBError($dbh->getErrorMessage());
		$dbh->clearErrorMessage();
	}
	return $ret;
}


function db_do($dbh,$statement) {
  	$statement = db_stmt_fixup($statement);
  	
  	$ret = $dbh->do_query($statement);
	if ($dbh->getErrorMessage() != null) {
		DBError($dbh->getErrorMessage());
		$dbh->clearErrorMessage();
	}
	return $ret;
}


function db_free($id) {
	oci_free_statement($id);
}

function db_finish($id) {
	return(db_free($id));
}



function getDBErrorMessage($db, $msg = null) {
	$e = oci_error($db);
	return $e['code'] . " : " . $e['message'];
}


function DBError($db_err) {
	global $PageError;

	// debug_print_backtrace();
	$db_err = $Msg->getMessage($db_err);
	if (is_a($db_err, 'DB_Error')) {
		$PageError .= $db_err->toString();
		trigger_error($db_err->toString(),E_USER_WARNING);
	} else {
		$PageError .= "$db_err\n";
		trigger_error("$db_err",E_USER_WARNING);
	}
}
?>
