<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Administration
 *
 * @package Console
 * @subpackage Administration
 */

$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['BF_ADMIN_KEYS'] = array(
		'ACTION_GROUPS'		=> array(
				'match' => '/groups\.[\w.]+/',
				'class' => 'Groups',
				'label' => translate('access groups'),
				'default' => 'groups.view',
				'primary' => false,			
				'display' => true),

		'ACTION_USERS'		=> array('match' => '/users\.[\w.]+/',
				'class' => 'Users',
				'label' => translate('users'),
				'default' => 'users.view',
				'primary' => true,			
				'display' => true),

		'ACTION_PERMISSIONS'	=> array('match' => '/permissions\.[\w.]+/',
				'class' => 'Permissions',
				'label' => translate('permissions'),
				'default' => 'permissions.view',
				'primary' => false,			
				'display' => true),

		'ACTION_LDAP'			=> array('match' => '/ldap\.[\w.]+/',
				'class' => 'LDAP',
				'label' => translate('ldap'),
				'default' => 'ldap.view',
				'primary' => false,			
				'display' => true),

		'ACTION_SYSTEM'		=> array('match' => '/system\.[\w.]+/',
				'class' => 'System',
				'label' => translate('system'),
				'default' => 'system.view',
				'primary' => false,			
				'display' => true),

		'ACTION_MESSAGES'		=> array('match' => '/messages\.[\w.]+/',
				'class' => 'Messages',
				'label' => translate('messages'),
				'default' => 'messages.view',
				'primary' => false,			
				'display' => true),

		'ACTION_IMPORT'		=> array('match' => '/import\.[\w.]+/',
				'class' => 'Import',
				'label' => translate('import'),
				'default' => 'import.view',
				'primary' => false,			
				'display' => true)
);

if ($GLOBALS['controller']->local || $GLOBALS['controller']->master) {
	$GLOBALS['BF_ADMIN_KEYS']['ACTION_GEOS'] = array('match' => '/geo\.[\w.]+/',
				'class' => 'Geos',
				'label' => translate('geo'),
				'default' => 'geo.view',
				'primary' => false,			
				'display' => true);
}

class Administration extends Module implements ViewableXHTML {
	protected $navigation_jte;
	protected $current_admin_key;
	public $action;
	protected $request_type;
	
	/**
	 * Pass that buck!
	 */
	public function handlesRequestType($req) {
		return $this->mod->handlesRequestType($req);
	}

	/**
	 * Woo-hoo, Factory-friendly and I didn't even mean to!
	 * Since Administration really covers N modules worth
	 * of functionality, we've broken them up into eleven modules
	 * that can each get returned based on the request.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$admin = new Administration();
		$admin->setRequest($req, $type, $post);
		return $admin;
	}
	
	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		global $GLOBAL_SETTINGS;
		
		// Get the action out
		if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
			$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
		} else {
			$this->action = $this->submodules['ACTION_USERS']['default'];
		}
		$this->request_type = $type;
		
		// Okay, so fetch the classname from the request and SAFELY use it based on the global
		if ($class = self::parseRequestedClass($req)) {
			
			if ($this->controller->require_file(BF_PRODUCT_ROOT . DIRECTORY_SEPARATOR . 'modules' . 
					DIRECTORY_SEPARATOR . 'Administration' . DIRECTORY_SEPARATOR . $class . '.php')) {
				// TODO: fix PHP so we can just do $class::handleRequest($req, $type, $post); 
				// I f@#$ing hate eval();
				$this->mod = eval('return ' . $class . '::handleRequest($req, $type, $post);');
				
				// What did we navigate to again?
				foreach ($this->submodules as $key => $info) {
					if ($info['class'] == $class) {
						$this->current_admin_key = $key;
						break;
					}
				}
			}
		} else {
			throw new BFException(translate('invalid request'));
		}

	}

	/**
	 * Scan the incomming request for an action that matches one of the Admin keys
	 */
	private static function parseRequestedClass($req) {
		if (array_key_exists('action', $req)) {
			foreach ($GLOBALS['BF_ADMIN_KEYS'] as $key => $attr) {
				if (preg_match($attr['match'], $req['action'])) {
					return $attr['class'];
				}
			}
		// Default to Users
		} else {
			return 'Users';
		}

		return false;
	}
	public function getNavigation() {
		return false;
	}
	
	/**
	 * Passing the buck
	 */
	public function render() {
		$this->mod->render();
		$this->content =& $this->mod->content;
		if ($this->request_type == BF_REQUEST_XHTML || array_key_exists(BF_REQUEST_XML_NAV, $_REQUEST)) {
			$this->renderNavigation();
		}		
	}

	private function renderNavigation() {
		$this->nav_content = $this->mod->navigationBlock();		
	}
	
	/**
	 * Passing the buck
	 */
	public function getIncludes() {
		return $this->mod->getIncludes();
	}
	
	/**
	 * Passing the buck
	 */
	public function getTTL() {
		return $this->mod->getTTL();
	}
	
	/**
	 * Passing the buck
	 */
	public function getLastBuildDate() {
		return $this->mod->getLastBuildDate();
	}
	
	public function __construct() {
		parent::__construct();
		$this->submodules = $GLOBALS['BF_ADMIN_KEYS'];
	}
}

?>
