<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Admin Geos
 *
 * @package Console
 * @subpackage Geos
 */

$GLOBALS['controller']->require_module('BFGeo');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';

class Geos extends Administration {
	const ACTION_VIEW = 'geo.view';
	const ACTION_EDIT = 'geo.edit';
	const ACTION_TEST = 'geo.test';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $geo;

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			$this->geo = new ViewGeo();
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$geo = new Geos();
		$temp = new BFGeo();
		$geo->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$geo->load($req[$temp->primary_key]);
		} else {
			$geo->autoLoad();
		}
		return $geo;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS;
		$rep = new BFReport();

		// Setup the columns, most of which will get displayed
		$rep->addColumn('bf_location', 'bf_geo', 'location');
		$rep->addColumn('bf_value', 'bf_sysconfig', 'outage');
		$rep->addColumn('bf_hostname', 'bf_geo', 'hostname');
		$rep->addColumn('bf_database', 'bf_geo', 'database');
		$rep->addColumn('bf_active', 'bf_geo', 'active');
		$rep->addColumn('bf_geo_id', 'bf_geo', 'id');

		$rep->join(array(
				'LEFT',
				'bf_sysconfig',
				array(
					array('=', 'bf_geo', 'bf_geo_id', 'bf_sysconfig', 'bf_geo_id'),
					array('=', 'bf_sysconfig', 'bf_param', 'gdd_site_outage')
				)
			));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'location', array(array('LIKE', 'bf_geo', 'bf_location')));
			$userDefinedFilter->setKey( 'hostname', array(array('LIKE', 'bf_geo', 'bf_hostname')));
			$userDefinedFilter->setKey( 'database', array(array('LIKE', 'bf_geo', 'bf_database')));
			$userDefinedFilter->setKey( 'username', array(array('LIKE', 'bf_geo', 'bf_username')));
			$userDefinedFilter->setKey( 'db', array(array('LIKE', 'bf_geo', 'bf_db')));
			$userDefinedFilter->setKey( 'active', array(array('LIKE', 'bf_geo', 'bf_active')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_geo', 'bf_location'),
											'OR',
											array('LIKE', 'bf_geo', 'bf_hostname'),
											'OR',
											array('LIKE', 'bf_geo', 'bf_database'),
											'OR',
											array('LIKE', 'bf_geo', 'bf_username'),
											'OR',
											array('LIKE', 'bf_geo', 'bf_db'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'location')
		);

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			// Needed for proper activity time display
			$this->jte->assign('now', time());
			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the user object,
	 * but it does call manage() (which makes changes) and then
	 * registers the user object with the jTE object for use in
	 * templates.
	 */
	protected function edit() {
		$this->manage();
		$this->jte->register_object('editable', $this->geo, null, false);
	}

	protected function manage() {
		global $GLOBAL_OBJECT_KEY;
		if (array_key_exists($GLOBAL_OBJECT_KEY, $this->post) && is_array($this->post[$GLOBAL_OBJECT_KEY])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBAL_OBJECT_KEY])) {
				$this->geo->delete();
			}  elseif (isset($this->post['copy'])) {
				$this->geo->copy();
			} else {
				foreach ($this->post[$GLOBAL_OBJECT_KEY] as $field => $value) {
					if ($field == 'bf_password') {
						// If not matching throw exception
						if (!array_key_exists('bf_password_verify', $this->post[$GLOBAL_OBJECT_KEY])
								|| $this->post[$GLOBAL_OBJECT_KEY]['bf_password'] != $this->post[$GLOBAL_OBJECT_KEY]['bf_password_verify']) {
							throw new BFException(translate('2087'));
						} else if ($value == '') {
							continue;
						} else {
							$value = bf_encrypt($value);
						}
					} else if ($field == 'bf_failover' && $value == 'BF_NULL') {
						$value = null;
					}
					$this->geo->set($field, $value);
				}
				// Only save if not testing, since testing needs to set values and then attempt auth-ing
				try {
					if (array_key_exists('test', $this->post[$GLOBAL_OBJECT_KEY])) {
						$res = $this->geo->test();
						$this->jte->assign('test', $res);
					} else {
						$this->geo->save();
					}
				} catch(Exception $e) {
					$this->controller->addError($e->getMessage());
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFUser object (if necessary)
		5. Set thenav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller, $GLOBAL_DATABASE_TYPES;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Administration/templates');
		$this->jte->assign('db_types', $GLOBAL_DATABASE_TYPES);
		$this->jte->register_function('bf_get_geo_location', 'bf_get_geo_location');
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->register_object('editable', $this->geo, null, false);
			$this->nav_content = '';
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Administration/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		if ($this->action == self::ACTION_EDIT || $this->action == self::ACTION_TEST) {
			$this->edit();
		}
		$this->frontPageReport();
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Administration/javascripts/geo.content.js')
				 );
	}

	public function navigationBlock() {
		return '';
	}

	public function load($id) {
		if (isset($id)) {
			$this->geo->select($this->geo->primary_key, $id);
		}
	}
	
	public function autoLoad() {
		$geo_qb = BFQueryBuilder::select();
		$geo_qb->addColumn('bf_geo_id', 'bf_geo', 'id');
		$geo_qb->orderBy('ASC', 'bf_geo', 'bf_location');
		$this->load($geo_qb->getOne());
	}
}

/**
 * Hack to get null vs. empty string so we can tell which one means 'primary'
 */
class ViewGeo extends BFGeo {
	public function get($field) {
		if ($field == 'bf_failover' && $this->fields[$field] === null) {
			return 'BF_NULL';
		} else {
			return parent::get($field);
		}
	}
}

?>
