<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Admin Groups
 *
 * @package Console
 * @subpackage AccessGroups
 */

$GLOBALS['controller']->require_module('BFLevel');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';
$GLOBALS['GLOBAL_MEMBERS_KEY'] = 'members';
$GLOBALS['GLOBAL_CHILDREN_KEY'] = 'children';
$GLOBALS['GLOBAL_ADD_KEY'] = 'add';
$GLOBALS['GLOBAL_DEL_KEY'] = 'del';

class Groups extends Administration {
	const ACTION_VIEW = 'groups.view';
	const ACTION_EDIT = 'groups.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $user;
	protected $children;
	protected $nonchildren;
	protected $members;
	protected $nonmembers;

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			$this->level = new BFLevel();
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$users = new Groups();
		$temp = new BFLevel();
		$users->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$users->load($req[$temp->primary_key]);
		}
		return $users;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS, $user;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Setup the columns, most of which will get displayed
		$rep->addColumn('bf_level', 'bf_levels', 'id');
		$rep->addColumn('bf_def', 'bf_levels', 'def');
		$rep->addColumn('bf_dn', 'bf_levels', 'dns');
		$rep->addColumn('bf_name', 'bf_levels', 'name');
		$rep->where(array('!=', 'bf_levels', 'bf_level', 0));
		
		// Add level constraints
		$uid = intval($user->get('bf_userid'));
		if ($uid !== 1) {
			if (isset($this->request['dialog'])) {
				$uid_bf_levels = array_keys(bf_get_levels($uid));
			} else {
				$uid_bf_levels = array_keys(bf_get_levels($uid, false, true));
			}
			$rep->where(
				array(
					array('IN', 'bf_levels', 'bf_level', $uid_bf_levels),
				)
			);
		}

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_levels', 'bf_name'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'name')
		);

		// LEFT JOIN throws the COUNT out of whack
	   // $rep->countBy('bf_levels', 'bf_userid');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the user object,
	 * but it does call manage() (which makes changes) and then
	 * registers the user object with the jTE object for use in
	 * templates.
	 */
	protected function edit() {
		global $user;
		
		$this->manage();
		$this->manageMembers();
		$this->manageChildren();

		$uid = intval($user->get('bf_userid'));
		$owningLevel = $this->level->get('bf_lvl');

		$hasAccess = ($uid === 1 || in_array($owningLevel, array_keys(bf_get_levels($uid))))?true:false;

		if ($hasAccess)  {
			$this->jte->register_object('editable', $this->level, null, false);
			$this->jte->assign('members', $this->members);
			$this->jte->assign('nonmembers', $this->nonmembers);
			$this->jte->assign('children', $this->children);
			$this->jte->assign('nonchildren', $this->nonchildren);
		}
	}

	protected function manageMembers() {
		global $GLOBAL_MEMBERS_KEY, $GLOBAL_ADD_KEY, $GLOBAL_DEL_KEY;

		// Manage the group members
		if (array_key_exists($GLOBAL_MEMBERS_KEY, $this->post) && is_array($this->post[$GLOBAL_MEMBERS_KEY])) {
			if (array_key_exists($GLOBAL_ADD_KEY, $this->post[$GLOBAL_MEMBERS_KEY]) && 
					is_array($this->post[$GLOBAL_MEMBERS_KEY][$GLOBAL_ADD_KEY])) {
				$this->level->addMembers($this->post[$GLOBAL_MEMBERS_KEY][$GLOBAL_ADD_KEY]);
			}
			if (array_key_exists($GLOBAL_DEL_KEY, $this->post[$GLOBAL_MEMBERS_KEY]) && 
					is_array($this->post[$GLOBAL_MEMBERS_KEY][$GLOBAL_DEL_KEY])) {
				$this->level->removeMembers($this->post[$GLOBAL_MEMBERS_KEY][$GLOBAL_DEL_KEY]);
			}
			try {
				$this->level->save();
			} catch (Exception $e) {
				$this->controller->addError($e->getMessage());
				return false;
			}
		}
	}

	protected function manageChildren() {
		global $GLOBAL_CHILDREN_KEY, $GLOBAL_ADD_KEY, $GLOBAL_DEL_KEY;

		// Manage the group children
		if (array_key_exists($GLOBAL_CHILDREN_KEY, $this->post) && is_array($this->post[$GLOBAL_CHILDREN_KEY])) {
			if (array_key_exists($GLOBAL_ADD_KEY, $this->post[$GLOBAL_CHILDREN_KEY]) && is_array($this->post[$GLOBAL_CHILDREN_KEY][$GLOBAL_ADD_KEY])) {
				$this->level->addChildren($this->post[$GLOBAL_CHILDREN_KEY][$GLOBAL_ADD_KEY]);
			}
			if (array_key_exists($GLOBAL_DEL_KEY, $this->post[$GLOBAL_CHILDREN_KEY]) && is_array($this->post[$GLOBAL_CHILDREN_KEY][$GLOBAL_DEL_KEY])) {
				$this->level->removeChildren($this->post[$GLOBAL_CHILDREN_KEY][$GLOBAL_DEL_KEY]);
			}
			try {
				$this->level->save();
			} catch (Exception $e) {
				$this->controller->addError($e->getMessage());
				return false;
			}
		}
	}

	protected function manage() {
		global $GLOBAL_OBJECT_KEY;
		// Manage the group details
		if (array_key_exists($GLOBAL_OBJECT_KEY, $this->post) && is_array($this->post[$GLOBAL_OBJECT_KEY])) {
			// Delete?
			if (isset($this->post[$GLOBAL_OBJECT_KEY]['delete'])) {
				$this->level->delete();
			}  elseif (isset($this->post[$GLOBAL_OBJECT_KEY]['copy'])) {
				$this->level->copy();
			} else {
				if (array_key_exists('bf_def', $this->post[$GLOBAL_OBJECT_KEY])) {
					$this->post[$GLOBAL_OBJECT_KEY]['bf_def'] = ($this->post[$GLOBAL_OBJECT_KEY]['bf_def'] == 'true') ? 'Y' : 'N';
				} else if (array_key_exists('bf_dn', $this->post[$GLOBAL_OBJECT_KEY]) && $this->post[$GLOBAL_OBJECT_KEY]['bf_dn'] == '') {
					unset($this->post[$GLOBAL_OBJECT_KEY]['bf_dn']);
				}
				$this->level->setMany($this->post[$GLOBAL_OBJECT_KEY]);
				try {
					$this->level->save();
				} catch (Exception $e) {
					$this->controller->addError($e->getMessage());
					return false;
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFLevel object (if necessary)
		5. Set thenav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Administration/templates');
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->register_object('editable', $this->level, null, false);
			$this->nav_content = '';
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Administration/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		switch ($this->action) {
			case self::ACTION_EDIT:
				$this->edit();
				$this->frontPageReport();
				break;
			case self::ACTION_VIEW:
			default:
				$this->frontPageReport();
		}
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Administration/javascripts/groups.content.js')
				 );
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$group = $this->level;			
		}  else {
			$group = new BFLevel();
			$group->select($group->primary_key, $id);
		}

		$id = $group->get($group->primary_key);
		if (!$id)  {
			return '';
		}
		$members = $group->members;

		$temp = bf_get_nav_projects_and_steps(array('=', '', 'bf_level', $id));
		
		$nav = array();
		$nav['permissions'] = bf_get_nav_perms($id);
		$nav['projects'] = array_key_exists('projects', $temp) ? $temp['projects'] : '';
		$nav['library'] = array_key_exists('library', $temp) ? $temp['library'] : '';		
		$nav['access groups'] = bf_get_nav_objects('access groups', 
					array(
							array('=', 'bf_levels', 'bf_lvl', $id),
							'OR',
							array('=', 'bf_levels', 'bf_level', $group->get('bf_lvl'))
					));
		$nav['environments'] = bf_get_nav_objects('environments', array(
				array('=', 'bf_env', 'bf_level', $id),
				array('=', 'bf_env', 'bf_eid', 0)));
		$nav['adaptors'] = bf_get_nav_objects('adaptors', array('=', 'bf_interfaces', 'bf_level', $id));
		$nav['classes'] = bf_get_nav_objects('classes', array('=', 'bf_classes', 'bf_level', $id));
		$nav['servers'] = bf_get_nav_objects('servers', array('=', 'bf_servers', 'bf_level', $id));
		$nav['selectors'] = bf_get_nav_objects('selectors', array('=', 'bf_selector', 'bf_level', $id));
		$nav['server auth'] = bf_get_nav_objects('server auth', array('=', 'bf_serverauth', 'bf_level', $id));
		if ($members)  {
			$nav['users'] = bf_get_nav_objects('users', array(array('IN', 'bf_users', 'bf_userid', array_keys($members))));
		}

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}


	protected function loadMembers() {
		$this->members = $this->level->getMembers();
		$this->nonmembers = $this->level->getNonMembers();
	}

	protected function loadChildren() {
		$this->children = $this->level->getChildren();
		$this->nonchildren = $this->level->getNonChildren();
	}

	public function load($id) {
		if (!empty($id)) {
			$this->level->select($this->level->primary_key, $id);
			$this->loadMembers($this->level->get('bf_level'));
			$this->loadChildren($this->level->get('bf_level'));
		}
	}
}

?>
