<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Import
 *
 * The Import viewable module provides an interface to engine/bfimport.pl:
 usage: bfimport [opts] [file]
 [opts] are:
        -L    list xml file contents
	or:
		-p    import projects
		-I    include chained projects
		-P    purge logs if replacing projects
		-s    import servers
		-e    import environments
		-c    import classes
		-g    import notification groups
		-u    import users
		-T    import notification templates
		-f    import filters
		-d    import ldap domain settings
		-r    replace rather than rename imported items
 *
 * @package Console
 * @subpackage Import
 */

$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'import';
$GLOBALS['GLOBAL_FILE_KEY'] = 'import';

// The power to turn parameters into switches - behold!
$GLOBALS['GLOBAL_IMPORT_SWITCHES'] = array(
	'projects' => '-p',
	'chained' => '-I',
	'purge' => '-P',
	'servers' => '-s',
	'environments' => '-e',
	'class' => '-c',
	'groups' => '-g',
	'users' => '-u',
	'templates' => '-T',
	'filters' => '-f',
	'selectors' => '-S',
	'collectors' => '-C',
	'ldap' => '-d',
	'replace' => '-r'
);

class Import extends Administration {
	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$import = new Import();
		$import->setRequest($req, $type, $post);
		return $import;
	}

	protected function edit() {
		global $GLOBAL_OBJECT_KEY, $GLOBAL_FILE_KEY, $GLOBAL_IMPORT_SCRIPT, $_FILES;

		// We do NOT want to attempt running a non-existent file
		$runme = false;
		foreach ($GLOBAL_IMPORT_SCRIPT as $script) {
			if (@file_exists($script)) {
				$runme = $script;
				break;
			}
		}

		if ($runme !== false) {
			// This will call the engine to actually pass things off and import the project
			if (array_key_exists($GLOBAL_OBJECT_KEY, $this->post)) {
				if (is_array($_FILES) && array_key_exists($GLOBAL_FILE_KEY, $_FILES)) {
					$runme .= $this->parseParams();
					$importfile = $_FILES[$GLOBAL_FILE_KEY]['tmp_name']['file'];
					if (!file_exists($importfile)) {
						$this->controller->addError(translate('2228'));
						return false;
					}
					chmod($importfile,0644);
					$runme .= ' -w -o "' . $importfile . '" "' . $importfile . '" 2>&1';
					$feedback = '';
					$feedback = translate('importing file %', $_FILES[$GLOBAL_FILE_KEY]['name']['file']) . "\n\n";

					$f = @popen($runme,"r");
					while (!@feof($f)) {
						$buf = @fgets($f,1024);
						$buf = ereg_replace("^Import: [0-9]+: ","",$buf);
						$feedback .= "$buf\n";
					}
					@pclose($f);

					$this->jte->assign('feedback', $feedback);
					return true;
				} else {
					$this->controller->addError(translate('2228'));
					return false;
				}
			}
		} else {
			$this->controller->addError(translate('2229'));
		}
	}

	/**
	 * Translate parameters from the _REQUEST into switches for the command
	 */
	protected function parseParams() {
		global $GLOBAL_OBJECT_KEY, $GLOBAL_IMPORT_SWITCHES, $user;
		$params = ' -U ' . $user->get($user->primary_key);

		foreach ($this->post[$GLOBAL_OBJECT_KEY] as $key => $val) {
			switch ($key) {
				case 'conflict':
					if ($val == 'replace') {
						$params .= ' ' . $GLOBAL_IMPORT_SWITCHES['replace'];
					}
					break;
				default:
					if (array_key_exists($key, $GLOBAL_IMPORT_SWITCHES)) {
						$params .= ' ' . $GLOBAL_IMPORT_SWITCHES[$key];
					}
			}
		}

		return $params;
	}

	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Administration/templates');
		$this->handleAction();

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Administration/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		$this->edit();
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					'stylesheets' => array(BF_WEBROOT . 'fullcontrol/modules/Administration/styles/import.view.css')
					);
	}

	public function navigationBlock() {
		return '';
	}

}

?>
