<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Admin Permissions
 *
 * @package Console
 * @subpackage Permissions
 */

$GLOBALS['controller']->require_module('BFPermission');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';
$GLOBALS['GLOBAL_ADD_KEY'] = 'add';
$GLOBALS['GLOBAL_DEL_KEY'] = 'del';

class Permissions extends Administration {
	const ACTION_VIEW = 'permissions.view';
	const ACTION_EDIT = 'permissions.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $permission;

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			$this->permission = new BFPermission();
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$permissions = new Permissions();
		$temp = new BFPermission();
		$permissions->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$permissions->load($req[$temp->primary_key]);
		} else {
			$permissions->load();
		}
		return $permissions;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Setup the columns, most of which will get displayed
		$rep->addColumn('bf_tag', 'bf_perms', 'id');
		$rep->addColumn('bf_message', 'bf_strings', 'name');
		$rep->addColumn('bf_group', 'bf_perms', 'type');
		$rep->where(array(
			array('!=', 'bf_perms', 'bf_description', ''),
			array('=', 'bf_perms', 'bf_description', 'bf_strings', 'bf_key'),
			array('=', 'bf_strings', 'bf_lang', $GLOBALS['user']->get('bf_lang'))
		));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_strings', 'bf_message')));
			$tags = array('projects' => 'P', 'groups' => 'G', 'Environment' => 'E', 'Access' => 'A', 'Users' => 'U', 'Execution' => 'R', 'Classes' => 'C', 'Servers' => 'B', 'Security' => 'S', 'Tuning' => 'T', 'Source' => 'L', 'Interface' => 'I');
			$userDefinedFilter->setKey( 'type', array(array('LIKE', 'bf_perms', 'bf_group')), $tags);
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_strings', 'bf_message'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'name')
		);

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}
	
	protected function assignGroups() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tag', 'bf_perms', 'id');
		$qb->addColumn('bf_group', 'bf_perms', 'type');
		$qb->where(array('=', 'bf_perms', 'bf_description', ''));
		$res = $qb->getResult();

		if (DB::isError($res)) {
			$this->controller->addError($res->getMessage());
		} else {
			$groups = array();
			while($rec = $res->fetchRow()) {
				$groups[$rec->type] = $rec->id;
			}
			$this->jte->assign('groups', $groups);
		}
	}

	/**
	 * This method doesn't directly change anything in the user object,
	 * but it does call manage() (which makes changes) and then
	 * registers the user object with the jTE object for use in
	 * templates.
	 */
	protected function edit() {
		$this->manage();
		$this->jte->register_object('editable', $this->permission, null, false);
	}

	protected function manage() {
		global $GLOBAL_OBJECT_KEY, $GLOBAL_ADD_KEY, $GLOBAL_DEL_KEY;

		// Manage the group members
		if (array_key_exists($GLOBAL_OBJECT_KEY, $this->post) && is_array($this->post[$GLOBAL_OBJECT_KEY])) {
			if (array_key_exists($GLOBAL_ADD_KEY, $this->post[$GLOBAL_OBJECT_KEY]) && is_array($this->post[$GLOBAL_OBJECT_KEY][$GLOBAL_ADD_KEY])) {
				$this->permission->addMembers($this->post[$GLOBAL_OBJECT_KEY][$GLOBAL_ADD_KEY]);
			}
			if (array_key_exists($GLOBAL_DEL_KEY, $this->post[$GLOBAL_OBJECT_KEY]) && is_array($this->post[$GLOBAL_OBJECT_KEY][$GLOBAL_DEL_KEY])) {
				$this->permission->removeMembers($this->post[$GLOBAL_OBJECT_KEY][$GLOBAL_DEL_KEY]);
			}
			try {
				$this->permission->save();
			} catch (Exception $e) {
				$this->controller->addError($e->getMessage());
				return false;
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFUser object (if necessary)
		5. Set thenav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Administration/templates');
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->register_object('editable', $this->permission, null, false);
			$this->nav_content = '';
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Administration/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		switch ($this->action) {
			case self::ACTION_EDIT:
				$this->edit();
				$this->frontPageReport();
				break;
			case self::ACTION_VIEW:
			default:
				$this->assignGroups();
				$this->frontPageReport();
		}
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Administration/javascripts/permissions.content.js')
				 );
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$perm = $this->permission;
		}  else {
			$perm = new BFPermission();
			$perm->select($perm->primary_key, $id);
		}
		
		$id = $perm->get($perm->primary_key);
		if (!$id)  {
			return '';
		}
		$members = $perm->getMembers();
		$curnav = array();
		
		foreach($members as $groupid => $groupname)  {
			$curnav[$groupname] = array(
					'mod' => 'administration',
					'action' => 'groups.view',
					'id' => 'bf_level=' . $groupid,
					'text' => $groupname
			);
		}

		// We also want to get groups that are owned by groups that have this permission.
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_level', 'bf_accesscache', 'id');
		$qb->addColumn('bf_name', 'bf_levels', 'name');
		$qb->where(array(
				array('=', 'bf_accesscache', 'bf_level', 'bf_levels', 'bf_level'),
				array('IN', 'bf_accesscache', 'bf_groupid', array_keys($members)),
				));
		$res = $qb->getResult();
		if (!DB::isError($res))  {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC))  {
				$curnav[$row['name']] = array(
						'mod' => 'administration',
						'action' => 'groups.view',
						'id' => 'bf_level=' . $row['id'],
						'text' => $row['name']
				);
			}
		}
				
		
		if ($curnav)  {
			ksort($curnav);
			$nav = array(
					'access groups' => array(
							'mod' => 'administration',
							'action' => 'groups.view',
							'text' => translate('access groups'),
							'subnav' => $curnav
					));
			return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
		}
		
		return '';
		
	}


	public function load($id = null) {
		// Get the first if none selected
		if (empty($id)) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tag', 'bf_perms', 'bf_tag');
			$qb->where(array('!=', 'bf_perms', 'bf_description', ''));
			$qb->orderBy('ASC', 'bf_perms', 'bf_description');
			$qb->limit(1);
			$id = $qb->getOne();
		}
		$this->permission->select($this->permission->primary_key, $id);
	}
}

?>
