<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Admin System
 *
 * @package Console
 * @subpackage System
 */

$GLOBALS['controller']->require_module('BFSysConfig');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';

class System extends Administration {
	const ACTION_VIEW = 'system.view';
	const ACTION_EDIT = 'system.edit';
	const ACTION_INFO = 'system.info';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $sysconfig;

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			$this->sysconfig = new BFSysConfig();
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$sysconfigs = new System();
		$temp = new BFSysConfig();
		$sysconfigs->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$sysconfigs->load($req[$temp->primary_key]);
		} else {
			$sysconfigs->defaultSysConfig();
		}
		return $sysconfigs;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Setup the columns, most of which will get displayed
		$rep->addColumn('bf_param', 'bf_sysconfig', 'id');
		$rep->addColumn('bf_message', 'bf_strings', 'name');
		$rep->addColumn('bf_description', 'bf_sysconfig', 'description');
		$rep->addColumn('bf_value', 'bf_sysconfig', 'value');
		$rep->addColumn('bf_datatype', 'bf_sysconfig', 'datatype');

		$rep->where(array(
				array('=', 'bf_sysconfig', 'bf_title', 'bf_strings', 'bf_key'),
				array('=', 'bf_strings', 'bf_lang', $GLOBALS['user']->get('bf_lang')),
				array('=', 'bf_sysconfig', 'bf_public', 'Y'),
				array('=', 'bf_sysconfig', 'bf_geo_id', '')
			));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_strings', 'bf_message')));
			$userDefinedFilter->setKey( 'value', array(array('LIKE', 'bf_sysconfig', 'bf_value')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_strings', 'bf_message'),
											'OR',
											array('LIKE', 'bf_sysconfig', 'bf_value'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'name')
		);

		// LEFT JOIN throws the COUNT out of whack
	   // $rep->countBy('bf_users', 'bf_userid');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			if ($this->controller->local) {
				$this->loadGeoConfigs();
			}

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * Load the configs overridden for this geo location
	 */
	protected function loadGeoConfigs() {
		$qb = BFQueryBuilder::select();
		// Only need the param and value pairs
		$qb->addColumn('bf_param', 'bf_sysconfig', 'id');
		$qb->addColumn('bf_value', 'bf_sysconfig', 'value');
		$qb->where(array(
				array('=', 'bf_sysconfig', 'bf_public', 'Y'),
				array('=', 'bf_sysconfig', 'bf_geo_id', $this->controller->geo->get('bf_geo_id'))
			));
		$configs = array();
		if ($res = $qb->getResult()) {
			while ($row = $res->fetchRow()) {
				$configs[$row->id] = $row->value;
			}
		}
		$this->jte->assign('geo_configs', $configs);
	}

	/**
	 * This method doesn't directly change anything in the user object,
	 * but it does call manage() (which makes changes) and then
	 * registers the user object with the jTE object for use in
	 * templates.
	 */
	protected function edit() {
		$this->manage();
		$this->jte->register_object('editable', $this->sysconfig, null, false);
	}

	protected function manage() {
		global $GLOBAL_OBJECT_KEY;
		if (array_key_exists($GLOBAL_OBJECT_KEY, $this->post) && is_array($this->post[$GLOBAL_OBJECT_KEY])) {
			// Sysconfigs ONLY get their values changed
			$param = 'bf_value';
			if (isset($this->post[$GLOBAL_OBJECT_KEY]['bf_datatype']))  {
				if ($this->post[$GLOBAL_OBJECT_KEY]['bf_datatype'] == 'bool')  {
					$param = 'bf_value_radio';
				}  elseif ($this->post[$GLOBAL_OBJECT_KEY]['bf_datatype'] == 'password')  {
					$param = 'bf_value_password';
				}  else {
					$param = 'bf_value_text';
				}
			}
			if (array_key_exists($param, $this->post[$GLOBAL_OBJECT_KEY])) {
				$this->sysconfig->set('bf_value', $this->post[$GLOBAL_OBJECT_KEY][$param]);
			}
			$this->sysconfig->save();
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFUser object (if necessary)
		5. Set thenav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Administration/templates');
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->register_object('editable', $this->sysconfig, null, false);
			$this->nav_content = '';
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Administration/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Only root can trigger this.
	 */
	protected function showInfo() {
		phpinfo();
		exit();
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		if ($this->action == self::ACTION_INFO && (int)$GLOBALS['user']->get('bf_userid') === 1) {
			$this->showInfo();
		}
//		else {
//			$this->action = self::ACTION_VIEW;
//		}
		switch ($this->action) {
			case self::ACTION_EDIT:
				$this->edit();
				$this->frontPageReport();
				break;
			case self::ACTION_VIEW:
			default:
				$this->frontPageReport();
		}
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Administration/javascripts/system.content.js')
				 );
	}


	public function navigationBlock() {
		return '';
	}


	public function load($id) {
		$fields = array($this->sysconfig->primary_key, 'bf_public');
		$values = array($id, 'Y');
		$this->sysconfig->select($fields, $values);
	}

	public function defaultSysConfig() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn($this->sysconfig->primary_key, 'bf_sysconfig');
		$qb->where(array(
					array('=', 'bf_sysconfig', 'bf_public', 'Y'),
					array('=', 'bf_sysconfig', 'bf_title', 'bf_strings', 'bf_key'),
					array('=', 'bf_strings', 'bf_lang', $GLOBALS['user']->get('bf_lang'))
					));

		// Add filter constraints
		if (isset($this->request['filter']) && $this->request['filter'])  {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_strings', 'bf_message')));
			$userDefinedFilter->setKey( 'value', array(array('LIKE', 'bf_sysconfig', 'bf_value')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_strings', 'bf_message'),
											'OR',
											array('LIKE', 'bf_sysconfig', 'bf_value'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$qb->filter($filterArray);
		}

		$qb->limit(1);
		$qb->orderBy('ASC', 'bf_strings', 'bf_message');
		$id = $qb->getOne();
		$this->load($id);
	}
}

?>
