<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Admin Users
 *
 * @package Console
 * @subpackage Users
 */

$GLOBALS['controller']->require_module('BFUser');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';
$GLOBALS['GLOBAL_MEMBEROF_KEY'] = 'members';
$GLOBALS['GLOBAL_ADD_KEY'] = 'add';
$GLOBALS['GLOBAL_DEL_KEY'] = 'del';

// For the template's use
$GLOBALS['GLOBAL_SETTINGS']['site']['limits'] = array(0, 1, 2, 3, 4, 5, 10, 20, 35, 50);

class Users extends Administration {
	const ACTION_VIEW = 'users.view';
	const ACTION_EDIT = 'users.edit';
	const ACTION_SU = 'users.su';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $user;
	protected $user_levels;			// Levels the user is directly or indirectly a member of
	protected $direct_user_levels;	// Levels the user is directly a member of
	protected $non_user_levels;		// Levels the user is not a member of

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			$this->user = new BFUser();
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$users = new Users();
		$temp = new BFUser();
		$users->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$users->load($req[$temp->primary_key]);
		} else {
			$users->load();
		}
		return $users;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Setup the columns, most of which will get displayed
		$rep->addColumn('bf_name', 'bf_users', 'name');
		$rep->addColumn('bf_login', 'bf_users', 'username');
		$rep->addColumn('bf_email', 'bf_users', 'email');
		$rep->addColumn('bf_maxbuilds', 'bf_users', 'limit');
		$rep->addColumn('bf_modified', 'bf_sessions', 'activity');
		$rep->addColumn('bf_tzone', 'bf_users', 'tz');
		$rep->addColumn('bf_userid', 'bf_users', 'id');
		$rep->addColumn('bf_password', 'bf_users', 'ldap');
		$rep->addColumn('bf_priority', 'bf_users', 'priority');
		$rep->addColumn('bf_id', 'bf_sessions', 'session');

		$rep->where(array('!=', 'bf_users', 'bf_userid', 0));

		$rep->join(array('LEFT', 'bf_sessions', array('=', 'bf_users', 'bf_userid', 'bf_sessions', 'bf_userid')));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_users', 'bf_name')));
			$userDefinedFilter->setKey( 'username', array(array('LIKE', 'bf_users', 'bf_login')));
			$userDefinedFilter->setKey( 'email', array(array('LIKE', 'bf_users', 'bf_email')));
			$userDefinedFilter->setKey( 'tz', array(array('LIKE', 'bf_users', 'bf_tzone')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_users', 'bf_name'),
											'OR',
											array('LIKE', 'bf_users', 'bf_login'),
											'OR',
											array('LIKE', 'bf_users', 'bf_email'),
											'OR',
											array('LIKE', 'bf_users', 'bf_tzone'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'name')
		);

		// LEFT JOIN throws the COUNT out of whack
	   $rep->countBy('bf_userid', 'bf_users');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			// Needed for proper activity time display
			$this->jte->assign('now', time());
			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
			$this->jte->assign('dow', array('sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 
					'saturday'));
		}
	}

	/**
	 * This method doesn't directly change anything in the user object,
	 * but it does call manage() (which makes changes) and then
	 * registers the user object with the jTE object for use in
	 * templates.
	 */
	protected function edit() {
		$this->loadOptions();
		$this->manage();
		$this->manageMemberOf();
		$this->jte->assign('limits', $GLOBALS['GLOBAL_SETTINGS']['site']['limits']);
		$this->get_user_levels();
		$this->jte->assign('levels', $this->user_levels);
		$this->jte->assign('directlevels', $this->direct_user_levels);
		$this->jte->assign('nonlevels', $this->non_user_levels);
		$this->jte->register_object('editable', $this->user, null, false);
	}

	/**
	 * Go through the process of loading things for select inputs....ugh.
	 */
	protected function loadOptions() {
		$this->levels = bf_get_levels();
		$this->dateformats = bf_get_dateformats();
	}
	
	/**
	 * Switch user to the passed user
	 */
	protected function su() {
		global $user;
		if (isset($this->post['password']) || $user->isRoot()) {
			try {
				$this->user->su($this->post['password']);
			} catch (BFException $e) {
				$this->controller->addError($e->getMessage());
			}
		} else {
			$this->controller->addError(translate('missing password'));
		}
	}

	protected function manage() {
		global $controller, $GLOBAL_OBJECT_KEY, $_COOKIE, $GLOBAL_COOKIE;
		if (array_key_exists($GLOBAL_OBJECT_KEY, $this->post) && is_array($this->post[$GLOBAL_OBJECT_KEY])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBAL_OBJECT_KEY])) {
				try {
					$this->user->delete();
				} catch (Exception $e) {
					$this->controller->addError($e->getMessage());
				}
			}  elseif (isset($this->post[$GLOBAL_OBJECT_KEY]['copy'])) {
				$this->user->copy();
			} else if (array_key_exists('purge', $this->post[$GLOBAL_OBJECT_KEY])) {
				if ($this->user->purge()) {
					$controller->addMessage(translate('2230'));
				}
			} else if (array_key_exists('logout', $this->post[$GLOBAL_OBJECT_KEY])) {
				//if ($this->user->updateCookie($_COOKIE[$GLOBAL_COOKIE['name']], '', 0)) {
				if ($this->user->setCookie('', 0)) {
					$controller->addMessage(translate('logged out'));
				}
			} else if (array_key_exists('expire', $this->post[$GLOBAL_OBJECT_KEY])) {
				if ($this->user->expirePassword()) {
					$controller->addMessage(translate('password expired'));
				}
			} else {
				foreach ($this->post[$GLOBAL_OBJECT_KEY] as $field => $value) {
					if ($field == 'bf_screenreader') {
						$this->user->setPreference('screenreader', ($value && $value != '0'));
						continue;
					} elseif ($field == 'bf_trunc') {
						$this->user->setPreference('truncation', $value); 
						continue;
					} elseif ($field == 'bf_schedule_start_dow') {
						$this->user->setPreference('schedule_start_dow', $value);
						continue;
					// Passwords need verification and "encryption"
					}  elseif ($field == 'bf_password') {
						// If not matching throw exception
						if (!array_key_exists('bf_password_verify', $this->post[$GLOBAL_OBJECT_KEY]) || $value != $this->post[$GLOBAL_OBJECT_KEY]['bf_password_verify']) {
							throw new BFException(translate('2087'));
						// Don't save an empty password
						} else if ($value == '') {
							continue;
						} else {
							$value = bf_encrypt($value);
						}
					}
					$this->user->set($field, $value);
				}
				try {
					$this->user->save();
				} catch (Exception $e) {
					$this->controller->addError($e->getMessage());
				}
			}
		}
	}


	protected function manageMemberOf() {
		global $GLOBAL_MEMBEROF_KEY, $GLOBAL_ADD_KEY, $GLOBAL_DEL_KEY;
		// Manage the group children
		if (array_key_exists($GLOBAL_MEMBEROF_KEY, $this->post) && is_array($this->post[$GLOBAL_MEMBEROF_KEY])) {
			if (array_key_exists($GLOBAL_ADD_KEY, $this->post[$GLOBAL_MEMBEROF_KEY]) && is_array($this->post[$GLOBAL_MEMBEROF_KEY][$GLOBAL_ADD_KEY])) {
				$this->user->addMemberOf($this->post[$GLOBAL_MEMBEROF_KEY][$GLOBAL_ADD_KEY]);
			}
			if (array_key_exists($GLOBAL_DEL_KEY, $this->post[$GLOBAL_MEMBEROF_KEY]) && is_array($this->post[$GLOBAL_MEMBEROF_KEY][$GLOBAL_DEL_KEY])) {
				$this->user->deleteMemberOf($this->post[$GLOBAL_MEMBEROF_KEY][$GLOBAL_DEL_KEY]);
			}
		}
	}


	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFUser object (if necessary)
		5. Set the nav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		// Need to substring in the templates to see ldap or not
		$this->jte->register_function('substr', 'substr', false);
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Administration/templates');
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->assign('limits', $GLOBALS['GLOBAL_SETTINGS']['site']['limits']);
			$this->jte->register_object('editable', $this->user, null, false);
			$this->nav_content = '';
			
			// Info just for display
			$users_count = (int)$this->controller->getUsersCount();
			$lic_users = (int)$controller->getSysConfig('lic_users');
			// Do not display that you take up more license seats than you have allowed to use
			if ($users_count > $lic_users) {
				$users_count = $lic_users;
			}
			$this->jte->assign('users_count', $users_count);
			$this->jte->assign('lic_users', $lic_users);
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Administration/templates/' . $this->request_type . '.' . 
				$this->action . '.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		switch ($this->action) {
			case self::ACTION_SU:
				$this->su();
				break;
			case self::ACTION_EDIT:
				$this->edit();
				$this->frontPageReport();
				break;
			case self::ACTION_VIEW:
			default:
				$this->frontPageReport();
		}
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Administration/javascripts/users.content.js')
				 );
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$user = $this->user;
		}  else {
			$user = new BFUser();
			$user->select($user->primary_key, $id);
		}
		
		$id = $user->get($user->primary_key);
		if (!$id)  {
			return '';
		}
		$nav = array();
		if (bf_get_levels($id))  {
			$nav['permissions'] = bf_get_nav_perms(array_keys(bf_get_levels($id)));
			$nav['access groups'] = bf_get_nav_objects('access groups', array(array('IN', 'bf_levels', 'bf_level', 
					array_keys(bf_get_levels($id)))));
		}
		$nav['schedules'] = bf_get_nav_objects('schedules', array('=', 'bf_cron', 'bf_owner', $id));

		return bf_output_nav_xml($nav);
	}

	protected function get_user_levels()  {
		$id = $this->user->get($this->user->primary_key);
		$levels = BFQueryBuilder::select();
		$levels->addColumn('bf_name', 'bf_levels', 'level');
		$levels->addColumn('bf_level', 'bf_levels', 'id');
		$levels->addColumn('bf_level', 'bf_access', 'direct');
		$levels->addColumn('bf_level', 'bf_accesscache', 'member');
		$levels->join(
					  array('LEFT', 'bf_access',
							array(
								  array('=', 'bf_access', 'bf_level', 'bf_levels', 'bf_level'),
								  array('=', 'bf_access', 'bf_userid', $id)
								  )
							)
					  );
		$levels->join(
					  array('LEFT', 'bf_accesscache',
							array(
								  array('=', 'bf_accesscache', 'bf_level', 'bf_levels', 'bf_level'),
								  array('=', 'bf_accesscache', 'bf_userid', $id)
								  )
							)
					  );
		$levels->orderBy('ASC', 'bf_levels', 'bf_level');
		$res = $levels->getResult();

		if (DB::isError($res)) {
			$GLOBALS['controller']->addError($res->getMessage());
			return false;
		}

		$this->user_levels = array();
		$this->direct_user_levels = array();
		$this->non_user_levels = array();

		while ($level = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			if ($level['member'] != '' || $level['direct'] != '')  {
				$this->user_levels[$level['level']] = ($level['direct'] == '') ? 0 : 1;
			}
			if ($level['direct'] != '')  {
				$this->direct_user_levels[$level['id']] = $level['level'];
			}  else {
				$this->non_user_levels[$level['id']] = $level['level'];						
			}
		}
	}

	public function load($id = null) {
		if (!empty($id)) {
			$this->user->select($this->user->primary_key, $id);
			// Get the group list
			$this->get_user_levels();
		} else {
			// Time to get the time zone!
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_tzone', 'bf_users', 'bf_tzone');
			$qb->where(array('=', 'bf_users', 'bf_userid', 1));
			$tzone = $qb->getOne();
			$this->user->set('bf_tzone', $tzone);
		}
	}
}

?>
