<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Environments
 *
 * @package Console
 * @subpackage Environments
 */

$GLOBALS['controller']->require_module('BFEnv');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');

/**
 * Used to make an array in the request to keep the environment fields
 * by themselves to reduce confusion when making changes to the environment.
 */
define('GLOBAL_ENVIRONMENT_KEY',		'environment');
define('GLOBAL_ENVIRONMENT_VAR_KEY',	'variable');

class Environments extends Module implements ViewableXHTML {
	const ACTION_VIEW_ENVIRONMENT = 'environment.view';
	const ACTION_EDIT_ENVIRONMENT = 'environment.edit';
	const ACTION_VIEW_VARIABLE = 'variable.view';
	const ACTION_EDIT_VARIABLE = 'variable.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;
	public $content;
	public $nav_content;
	protected $environment;
	protected $environment_var;
	protected $project_references;
	protected $step_references;

	public function setRequest($req, $type, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			// Need this no matter what...
			$this->environment = new BFEnv();
			$this->environment_var = new BFEnv();
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$environments = new Environments();
		$temp = new BFEnv();
		$environments->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			// Key an eye out for the eid as well
			$eid = (array_key_exists('bf_eid', $req)) ? $req['bf_eid'] : null;
			$environments->load($req[$temp->primary_key], $eid);
		}
		return $environments;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $user, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_bid', 'bf_env', 'id');
		$rep->addColumn('bf_param', 'bf_env', 'name');
		$rep->addColumn('bf_name', 'bf_levels', 'access');
		$rep->addColumn('bf_level', 'bf_env', 'level');
		// Add constraints
		$uid = intval($user->get('bf_userid'));
		if ($uid === 1) {
			$rep->where(array(
				array('=', 'bf_env', 'bf_level', 'bf_levels', 'bf_level'),
				array('=', 'bf_env', 'bf_eid', 0)
			));
		} else {
			$rep->where(array(
				array('=', 'bf_env', 'bf_level', 'bf_levels', 'bf_level'),
				array('IN', 'bf_env', 'bf_level', array_keys(bf_get_levels($uid))),
				array('=', 'bf_env', 'bf_eid', 0)
			));
		}

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_env', 'bf_param')));
			$userDefinedFilter->setKey( 'access', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_env', 'bf_param'),
											'OR',
											array('LIKE', 'bf_levels', 'bf_name'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'name')
		);
		
		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);
		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * Listing for a particular environment's parameters/values
	 */
	protected function envParamsReport($id) {
		global $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_eid', 'bf_env', 'id');
		$rep->addColumn('bf_param', 'bf_env', 'name');
		$rep->addColumn('bf_value', 'bf_env', 'value');
		$rep->addColumn('bf_action', 'bf_env', 'action');
		$rep->addColumn('bf_mode', 'bf_env', 'project');
		$rep->addColumn('bf_bid', 'bf_env', 'bid');
		// Add constraints
		$rep->where(array(
				array('=', 'bf_env', 'bf_bid', $id),
				array('>', 'bf_env', 'bf_eid', 0)
			));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_env', 'bf_param')));
			$userDefinedFilter->setKey( 'access', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_env', 'bf_param'),
											'OR',
											array('LIKE', 'bf_env', 'bf_value'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy('ASC', 'bf_env', 'bf_eid');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);
		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
			$this->jte->assign('bf_bid', $this->environment->get('bf_bid'));
		}
	}


	/**
	 * This method doesn't directly change anything in the environment object,
	 * but it does call manageEnvironment() (which makes changes) and then
	 * registers the environment object with the jTE object for use in
	 * templates.
	 */
	protected function editEnv() {
		if (!($this->environment_var instanceof BFEnv)) {
			$this->environment_var = new BFEnv();
		}
		$this->manageEnvironment();
		$this->jte->register_object('parent', $this->environment, null, false);
		$this->jte->register_object('editable', $this->environment_var, null, false);
		$this->jte->assign('project_count', count($this->project_references));
		$this->jte->assign('step_count', count($this->step_references));
	}

	/**
	 * This should get altered to accept different forms of requests (ie. other than POST
	 * but it'll do for now. This method runs through the request and sets any values it
	 * finds from the request. The environment object weeds out the garbage and validates/escapes.
	 */
	protected function manageEnvironment() {
		global $controller;

		if (array_key_exists(GLOBAL_ENVIRONMENT_KEY, $this->post) && is_array($this->post[GLOBAL_ENVIRONMENT_KEY])) {
			try {
				// Delete?
				if (array_key_exists('delete', $this->post[GLOBAL_ENVIRONMENT_KEY])) {
					$this->environment->delete();
				// Copy?
				} else if (array_key_exists('copy', $this->post[GLOBAL_ENVIRONMENT_KEY])) {
					return $this->environment->copy();
				} else {
					// Making a new one?
					$inserting = ($this->environment->get('bf_bid') === false);
					foreach ($this->post[GLOBAL_ENVIRONMENT_KEY] as $field => $value) {
						$this->environment->set($field, $value);
					}
					$this->environment->save();
				}
			} catch (Exception $e) {
				$controller->addError($e->getMessage());
				return false;
			}
		}
		// Yes, you could theoretically edit both the Environment and one of its variables concurrently
		if (array_key_exists(GLOBAL_ENVIRONMENT_VAR_KEY, $this->post) && is_array($this->post[GLOBAL_ENVIRONMENT_VAR_KEY])) {
			try {
				// Delete?
				if (array_key_exists('delete', $this->post[GLOBAL_ENVIRONMENT_VAR_KEY])) {
					return $this->environment_var->delete();
				} else {
					foreach ($this->post[GLOBAL_ENVIRONMENT_VAR_KEY] as $field => $value) {
						$this->environment_var->set($field, $value);
					}
						$this->environment_var->save();
				}
			} catch (Exception $e) {
				$controller->addError($e->getMessage());
				return false;
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFEnv object (if necessary)
		5. Set the nav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW_ENVIRONMENT;

		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Environments/templates');
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML || array_key_exists(BF_REQUEST_XML_NAV, $_REQUEST)) {
			$this->nav_content = $this->navigationBlock();		
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		$tpl = BF_ROOT . '/public/fullcontrol/modules/Environments/templates/' . $this->request_type . '.' . 
				$this->action . '.tpl';
		if ($this->request_type != BF_REQUEST_XHTML && file_exists($tpl)) {
			return $tpl;
		} else if ($this->action == self::ACTION_VIEW_VARIABLE || $this->action == self::ACTION_EDIT_VARIABLE) {
			return BF_ROOT . '/public/fullcontrol/modules/Environments/templates/xhtml.variable.view.tpl';
		} else {
			return BF_ROOT . '/public/fullcontrol/modules/Environments/templates/xhtml.environment.view.tpl';
		}
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		// Really, this could get called from anywhere
		$this->editEnv();
		switch ($this->action) {
			case self::ACTION_EDIT_VARIABLE:
			case self::ACTION_VIEW_VARIABLE:
				$this->envParamsReport($this->environment->get('bf_bid'));
				break;
			case self::ACTION_EDIT_ENVIRONMENT:
			case self::ACTION_VIEW_ENVIRONMENT:
			default:
				$this->frontPageReport();
		}
	}

	public function getIncludes() {
		$location = ($this->action == self::ACTION_EDIT_VARIABLE || 
				$this->action == self::ACTION_VIEW_VARIABLE) ? 'variable' : 'environment';
		
		return array(
					 'javascripts' => array(
								BF_WEBROOT . 'javascripts/mouseOvers.js',
								BF_WEBROOT . 'fullcontrol/javascripts/actions_menu.js',
								BF_WEBROOT . 'fullcontrol/modules/Environments/javascripts/' . $location . '.content.js'
							)
					 );
	}

	public function getNavigation() {
		return false;
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$env = $this->environment;
		}  else {
			$env = new BFEnv();
			$env->select($env->primary_key, $id);
		}
		
		$id = $env->get($env->primary_key);
		$name = $env->get('bf_param');
		if (!$id)  {
			return '';
		}
		$access = $env->get('bf_level');

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_env');
		$qb->where(array(array('=', 'bf_env', 'bf_param', '.include'), array('=', 'bf_env', 'bf_value', 
				$env->getEnvGroupName())));

		$res = $qb->getCol();
		if (!DB::isError($res) && $res)  {
			$where = array('IN', 'bf_env', 'bf_bid', $res);
		}  else {
			$where = array();
		}

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_value', 'bf_env');
		$qb->where(array(array('=', 'bf_env', 'bf_param', '.include'), array('=', 'bf_env', 'bf_bid', $id)));
		$res = $qb->getCol();
		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_env');
		$qb->where(array(array('=', 'bf_env', 'bf_param', '.include'), array('=', 'bf_env', 'bf_bid', $id)));
		if (!DB::isError($res) && $res)  {
			if ($where)  {
				$where = array('(', $where, 'OR', array('IN', 'bf_env', 'bf_param', $res), ')');
			}  else {
				$where = array(array('IN', 'bf_env', 'bf_param', $res));
			}
		}

		$nav = bf_get_nav_projects_and_steps(array('=', '', 'bf_eid', $id));
		if ($where)  {
			$nav['environments'] = bf_get_nav_objects('environments', array($where,
					array('=', 'bf_env', 'bf_eid', 0)));
		}
		$nav['schedules'] = bf_get_nav_objects('schedules', array('=', 'bf_cron', 'bf_eid', $id));
		$nav['servers'] = bf_get_nav_objects('servers', array('=', 'bf_servers', 'bf_eid', $id));
		$nav['adaptor links'] = bf_get_nav_objects('adaptor links', array('=', 'bf_scopes', 'bf_eid', $id));
		$nav['access groups'] = bf_get_nav_objects('access groups', array('=', 'bf_levels', 'bf_level', $access));

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}


	public function load($bid, $eid = null) {
		global $controller, $user;

		$hasAccess = false;
		$uid = intval($user->get('bf_userid'));

		if (!empty($bid)) {
			// Load the Environment itself
			$fields = array($this->environment->primary_key, 'bf_eid');
			$values = array($bid, 0);
			$this->environment->select($fields, $values);
		}

		$envLevel = $this->environment->get('bf_level');
		# User is Root or New Envgroup or EnvLevel is in Users list.
		$hasAccess = ($uid === 1 || $envLevel == '' || in_array($envLevel, array_keys(bf_get_levels($uid))));
		// If the user doesn't have access to this, redirect to list.
		if (!$hasAccess) {
			$controller->addHeader('Location: ' . BF_WEBROOT . 'fullcontrol/?mod=environments');
		}

		// Need to know whether or not the environment has any references anywhere
		// NOTE: I know someone will ask for a listing of the projects and/or steps
		// still using the environment in question, so might as well just get them now.
		$this->project_references = $this->environment->getProjectReferences();
		$this->step_references = $this->environment->getStepReferences();

		// If necessary, also load the particular variable
		if (!is_null($eid) && $eid != '') {
			$fields = array($this->environment->primary_key, 'bf_eid');
			$values = array($bid, $eid);
			$this->environment_var->select($fields, $values);
		} else {
			// We need a bid no matter what
			$this->environment_var->set($this->environment->primary_key, $bid);
		}
	}

	public function __construct() {
		parent::__construct();
		$this->jte = new BFJTE();
	}
}

?>
