<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Home
 *
 * @package Console
 * @subpackage Home
 */

$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');
$GLOBALS['controller']->require_module('BFReport');


class Builds extends Module implements ViewableXHTML {
	const VIEW_ACTIVE 			= 'home.view.active';
	const VIEW_COMPLETED 		= 'home.view.completed';

	public $title = 'Home';
	public $content;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML);
	protected $jte;
	protected $request;
	protected $request_type;
	public $action;

	protected function generateContent() {
		global $user;
		$this->jte = new BFJTE();

		if ($this->action == self::VIEW_ACTIVE || empty($this->action)) {
			$params = array();
			if (array_key_exists('active', $this->request) && is_array($this->request['active'])) {
				$params = $this->request['active'];
			}
			$active = $this->getBuilds(array('W', 'R'));
			$this->jte->register_object('listing', $active, null, false);
			$user->setPreference('home_defaultpage', self::VIEW_ACTIVE);
		}

		if ($this->action == self::VIEW_COMPLETED) {
			$params = array();
			if (array_key_exists('completed', $this->request) && is_array($this->request['completed'])) {
				$params = $this->request['completed'];
			}
			$completed = $this->getBuilds(array('C'));
			$this->jte->register_object('listing', $completed, null, false);
			$user->setPreference('home_defaultpage', self::VIEW_COMPLETED);
		}
		$this->jte->register_object('module', $this, null, false);

		// Get the purge list and submit it to the template engine
		$this->jte->assign('purges', $this->getPurges());
		
		// Set up function to check the number of filter matches for a build.
		$this->jte->register_function('bf_get_num_filter_matches',	'bf_get_num_filter_matches');
	}

	protected function getBuilds($states) {
		global $GLOBAL_SETTINGS, $user;
		$rep = new BFReport();

		// Index
		$rep->addColumn('bf_bid',			'bf_builds',	'bid');
		$rep->addColumn('bf_tag',			'bf_builds',	'tag');
		$rep->addColumn('bf_description',	'bf_projects',	'project');
		$rep->addColumn('bf_class',			'bf_builds',	'class');
		$rep->addColumn('bf_state',			'bf_builds',	'state');
		$rep->addColumn('bf_result',		'bf_builds',	'result');
		$rep->addColumn('bf_fired',			'bf_builds',	'date');
		$rep->addColumn('bf_duration',  	'bf_builds',	'runtime');
		$rep->addColumn('bf_name',			'bf_users',	 	'owner');
		$rep->addColumn('bf_pid',			'bf_builds',	'pid');
		$rep->addColumn('bf_process',   	'bf_builds',	'process');
		$rep->addColumn('bf_userid',		'bf_builds',	'userid');
		$rep->addColumn('bf_can',			'bf_builds',	'cancelled');
		$rep->addColumn('bf_platform',		'bf_projects',	'selector');
		$rep->addColumn('bf_stage',			'bf_builds',	'stage');
		if ($this->controller->master || $this->controller->local) {
			$rep->addColumn('bf_geo_id',	'bf_builds',		'geo_id');
		}

		$uid = intval($user->get('bf_userid'));
		if ($uid === 1) {
			$rep->where(array(
				array('=', 'bf_builds', 'bf_userid', 'bf_users', 'bf_userid'),
				array('=', 'bf_projects', 'bf_pid', 'bf_builds', 'bf_pid'),
				array('=', 'bf_builds', 'bf_process', 'B')
			));
		} else {
			$rep->where(array(
				array('=', 'bf_builds', 'bf_userid', 'bf_users', 'bf_userid'),
				array('=', 'bf_projects', 'bf_pid', 'bf_builds', 'bf_pid'),
				array('=', 'bf_builds', 'bf_process', 'B'),
				array('=', 'bf_builds', 'bf_level', 'bf_levels', 'bf_level'),
				array('IN', 'bf_builds', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}

		$rep->where(array('IN', 'bf_builds', 'bf_state', $states));

		if ($this->controller->master || $this->controller->local) {
			$this->geos = bf_get_geo();
		}

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey('tag', array(array('LIKE', 'bf_builds', 'bf_tag')));
			$userDefinedFilter->setKey('canceled', array(array('=', 'bf_builds', 'bf_can')), 
					array('canceled' => 'Y', 'Canceled' => 'Y'));
			$userDefinedFilter->setKey('result', array(array('=', 'bf_builds', 'bf_result')), 
					array('passed' => 'P', 'failed' => 'F', 'warning' => 'W', 'Passed' => 'P', 'Failed' => 'F', 'Warning' => 'W'));
			$userDefinedFilter->setKey('state', array(array('=', 'bf_builds', 'bf_state')), 
					array('completed' => 'C', 'running' => 'R', 'Completed' => 'C', 'Running' => 'R'));
			$userDefinedFilter->setKey('class', array(array('LIKE', 'bf_projects', 'bf_class')));
			$userDefinedFilter->setKey('project',array(array('LIKE', 'bf_projects', 'bf_description')));
			$userDefinedFilter->setKey('library',array(array('LIKE', 'bf_projects', 'bf_description')));
			$userDefinedFilter->setKey('owner', array(array('LIKE', 'bf_users', 'bf_name')));			
			
			$userDefinedFilter->setKey( '~',	// ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_builds', 'bf_tag'),
											'OR',										
											array('LIKE', 'bf_projects', 'bf_description'),
											'OR',
											array('LIKE', 'bf_builds', 'bf_class'),
											'OR',
											array('LIKE', 'bf_builds', 'bf_state'),
											'OR',
											array('LIKE', 'bf_builds', 'bf_result'),
											'OR',
											array('LIKE', 'bf_users', 'bf_name'),
											')'
										));
										
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}


		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'DESC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'date')
			);

		// If requested, set a new page to display. Otherwise, use defaults
		$count	= (array_key_exists('count', $this->request))	? intval($this->request['count'])	: $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request))	? intval($this->request['offset'])	: $GLOBAL_SETTINGS['reporting']['offset'];		
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();
	}

	protected function getPurges() {
		$qb	= BFQueryBuilder::select();

		$qb->addColumn('bf_pid', 'bf_builds', 'pid'); // the id of the build to be purged
		$qb->addColumn('bf_bid', 'bf_builds', 'bid'); // the purge builds id
		$qb->addColumn('bf_can', 'bf_builds', 'can'); // are we canceling?
		$qb->join(array('LEFT', 'bf_users',	array( array('=', 'bf_users', 'bf_userid', 'bf_builds', 'bf_userid') ) ));
		$qb->where(array('=', 'bf_builds', 'bf_process', 'P'));
		$res = $qb->getResult();
	
		$results = array();
		while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$results[$row['pid']] = array(
					'pid'	=> $row['pid'],
					'bid'	=> $row['bid'],
					'can'	=> $row['can']
				);
		}
		return $results;	
	}


	public function getNavigation() {
		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function setRequest($req, $type = BF_REQUEST_XHTML) {
		global $GLOBAL_SETTINGS, $user;
		$this->request = $req;
		$this->request_type = $type;

		// Get the action out
		if (isset($req[$GLOBAL_SETTINGS['site']['actionkey']]) && 
				$req[$GLOBAL_SETTINGS['site']['actionkey']] && 
				$req[$GLOBAL_SETTINGS['site']['actionkey']] != 'home.view') {
			$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
		} else {
			
			// If no action is set, check to see if we have done one previously and use that
			$home_defaultpage = $user->getPreference('home_defaultpage');
			if (!empty($home_defaultpage)) {
				$this->action = $home_defaultpage;
			}  else {
				$this->action = 'home.view.active';
			}
		}
	}

	public function render() {
		$this->generateContent();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Home/templates');

		if (isset($this->geos)) {
			$this->jte->assign('geos', $this->geos);
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XML) {
			return BF_ROOT . '/public/fullcontrol/modules/Home/templates/xml.home.view.builds.tpl';
		} else {
			return BF_ROOT . '/public/fullcontrol/modules/Home/templates/xhtml.home.view.builds.tpl';
		}
	}


	public function getIncludes() {
		return array(
				'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Home/javascripts/builds.content.js')
				);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$home = new Builds();
		$home->setRequest($req, $type);
		return $home;
	}

}

?>
