<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Home Messages
 *
 * @package Console
 * @subpackage Messages
 */

$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';

class Messages extends Home implements ViewableXHTML {
	protected $jte;
	protected $request;
	protected $post;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $ttl = 5;
	protected $lastBuildDate;
	
	public $request_type;
	public $content;
	public $nav_content;
	protected $location_db = null; // GDD override
	protected $usernames; // GDD nonsense

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (isset($this->request['geo_id']) && !empty($this->request['geo_id'])) {
				$this->location_db = $this->controller->getDB($this->request['geo_id']);
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$messages = new Messages();
		$messages->setRequest($req, $type, $post);
		return $messages;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	public function frontPageReport() {
		global $GLOBAL_SETTINGS, $user;
		
		$messages = array();
		$rep = new BFReport($this->location_db);

		// Two very different ways of pulling this info: GDD or not?
		if ($this->controller->local || $this->controller->master) {
			$rep->addColumn('bf_userid', 'bf_messages', 'bf_user');
			$rep->addColumn('bf_msgkey', 'bf_messages', 'message');
			// Also need to pull in the userids for this...
			$userids_qb = BFQueryBuilder::select($this->location_db);
			$userids_qb->addColumn('bf_userid', 'bf_messages', 'bf_userid');
			if (array_key_exists('severity', $this->request) && $this->request['severity']) {
				$userids_qb->where(array('=', 'bf_messages', 'bf_severity', $this->request['severity']));
			}
			if (array_key_exists('last', $this->request) && $this->request['last'] != '') {
				$time = strtotime($this->request['last']);
				$userids_qb->where(array('>', 'bf_messages', 'bf_stamp', $time));
			}
			if (array_key_exists('user', $this->request) && $this->request['user'] != '') {
				$userids_qb->where(array('=', 'bf_messages', 'bf_userid', $this->request['user']));
			}
			$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : 
					$GLOBAL_SETTINGS['reporting']['count'];
			$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : 
					$GLOBAL_SETTINGS['reporting']['offset'];
			$userids_qb->limit($count, $offset);
			// If this actually works and we get a listing...
			if ($userids = $userids_qb->getCol()) {
				$userids = array_unique($userids);
				// In order to get the user names...
				$usernames_qb = BFQueryBuilder::select();
				$usernames_qb->addColumn('bf_userid', 'bf_users', 'bf_userid');
				$usernames_qb->addColumn('bf_name', 'bf_users', 'bf_name');
				$usernames_qb->where('IN', 'bf_users', 'bf_userid', $userids);
				if ($usernames_res = $usernames_qb->getResult()) {
					$this->usernames = array();
					while ($username_row = $usernames_res->fetchRow()) {
						$this->usernames[$username_row->bf_userid] = $username_row->bf_name;
					}
				}
			}
		}
		$rep->addColumn('bf_stamp', 'bf_messages', 'stamp');
		$rep->addColumn('bf_severity', 'bf_messages', 'severity');
		$rep->addColumn('bf_msgargs', 'bf_messages', 'msgargs');
		
		if (!$this->controller->local && !$this->controller->master) {
			$rep->addColumn('bf_name', 'bf_users', 'bf_user');
			$rep->addColumn('bf_message', 'bf_strings', 'message');
			$rep->join(array('INNER', 'bf_strings', 
					array('=', 'bf_messages', 'bf_msgkey', 'bf_strings', 'bf_key')));
			$rep->join(array('LEFT', 'bf_users', 
					array('=', 'bf_messages', 'bf_userid', 'bf_users', 'bf_userid')));
			$rep->where(array('=', 'bf_strings', 'bf_lang', $user->get('bf_lang')));
		}
		
		if (array_key_exists('severity', $this->request) && $this->request['severity']) {
			$rep->where(array('=', 'bf_messages', 'bf_severity', $this->request['severity']));
		}
		if (array_key_exists('last', $this->request) && $this->request['last'] != '') {
			$time = strtotime($this->request['last']);
			$rep->where(array('>', 'bf_messages', 'bf_stamp', $time));
		}
		if (array_key_exists('user', $this->request) && $this->request['user'] != '') {
			$rep->where(array('=', 'bf_messages', 'bf_userid', $this->request['user']));
		}
		
		$rep->orderBy('DESC', 'bf_messages', 'bf_stamp');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : 
				$GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : 
				$GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);
		$res = $rep->getReportResults();
		$this->jte->register_object('listing', $res);
	}

	/**
	 * Generate the current view and assigns to the template
	 */
	public function render() {
		global $controller;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Home/templates');
		$this->frontPageReport();
		if ($this->controller->local || $this->controller->master) {
			$this->jte->assign('usernames', $this->usernames);
		}

		// Nobody else needs these
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->nav_content = '';
		} else if ($this->request_type == BF_REQUEST_RSS2) {
			$controller->addHeader('Last-Modified: ' . $this->lastBuildDate);
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Home/templates/' . $this->request_type . 
				'.home.view.messages.tpl';
	}

	/**
	 * We can just go ahead and use the Admin versions of things, since this is basically the same page.
	 */
	public function getIncludes() {
		return array(
				'stylesheets' => array(BF_WEBROOT . 
						'fullcontrol/modules/Administration/styles/messages.view.css'),
				'javascripts' => array(BF_WEBROOT . 
						'fullcontrol/modules/Home/javascripts/messages.content.js')
				 );
	}
	
	/**
	 * Return the cacheable period in minutes
	 */
	public function getTTL() {
		return $this->ttl;
	}
	
	/**
	 * Get the stamp from the last (chronologically speaking) message and return it.
	 */
	public function getLastBuildDate() {
		return $this->lastBuildDate;
	}


	public function navigationBlock() {
		return '';
	}

}

?>
