<?php
/* (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the copyright and other intellectual property rights in this software.
 Duplication or use of the Software is not permitted except as expressly provided in a written agreement between your company and BuildForge, Inc. */

$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_module('BFNotes');
$GLOBALS['controller']->require_module('BFStore');

class Build extends Module implements ViewableXHTML {

	public $title = 'Project Runs';
	public $content;

	protected $jte;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $request;
	protected $request_type;
	public $action;	

	/** Handle Request
	 * First thing run by central controller
	 * NO CLASS is created when this is run.  This does the primary job of creating the class
	 *  which then defines the rest of the behaviour throughout the object.
	 */	 
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$build = new Build();
		$build->setRequest($req, $type, $post);
		return $build;
	}
	
	/** Valid Request Types
	 * Check to see what request types this module handles and return true if we can do it
	 */	 	
	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}
	
	/** includes
	 * This is where we define what other things need to be included.. like javascript
	 */	 	
	public function getIncludes() {
		return array( 
			'javascripts' => array(
					BF_WEBROOT . 'fullcontrol/modules/Jobs/javascripts/build.'.$this->action[0].'.content.js',
					BF_WEBROOT . 'fullcontrol/modules/Jobs/javascripts/build.common.content.js'
					) 
		);
	}
	
	/** Set Request
	 * this passes the get/post request to the object and gets the action out
	 */	 	
	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		global $GLOBAL_SETTINGS;
		$this->request		= $req;
		$this->post			= $post;
		$this->request_type = $type;

		if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $this->request)) {
			$this->action = array($this->request[$GLOBAL_SETTINGS['site']['actionkey']]);
		} else {
			$this->action = array();
		}
	}
	
	/** Render
	 * This is the primary render function..  Here we make the magic happen.
	 * This is called by Central Controller	 
	 */	 	
	public function render() {
		global $user;

		// If this is a raw view of the log or the bom, then divert the flow and return
		if ($this->request_type == BF_REQUEST_RAW) {
			$GLOBALS['controller']->addHeader($GLOBALS['GLOBAL_HEADERS'][BF_REQUEST_XHTML]);

			$uid = $user->get('bf_userid');

			// Make with the log view!		
			if (isset($this->request['action']) && $this->request['action'] == 'edit.log') {				

				if ($uid != 1)  {
					$levels = bf_get_levels($uid);
					
					// Check if the user has legitimate access to this step log.
					$db = $this->getDBFromBID($this->request['bf_id']);
					$qb = BFQueryBuilder::select($db);
					$qb->addColumn('bf_level', 'bf_results');
					$qb->where(array(
							array('=', 'bf_results', 'bf_bid', $this->request['bf_id']),
							array('=', 'bf_results', 'bf_lid', $this->request['lid'])
							));
					$level = $qb->getOne();
					if (!isset($levels[$level]))  {
						return;
					}
				}

				$this->content = new LogViewer($this->request['bf_id'],$this->request['lid'],
					((isset($this->request['filter']))?$this->request['filter']:''), $this);
				return;
			}

			// Make with the Bom view!
			if (isset($this->request['action']) && $this->request['action'] == 'edit.bom') {				

				$this->content = new BomViewer($this->request['bf_id'], $this);
				return;
			}

		}

		$this->generateContent();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Jobs/templates');
		$template = $this->chooseTemplate();
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;


	}
	
	/** Choose Templates
	 * this function determines what templates we should be using
	 * in this case it will either be an xml template, or the main xhtml.view template
	 * our five main views use the same template as they are all the same	 
	 */	 	  	
	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XML) {		
			return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/xml.build.'.$this->action[0].'.tpl';
		} else if ($this->request_type == BF_REQUEST_RAW) {
			return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/raw.build.'.$this->action[0].'.tpl';
		} else {
			return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/xhtml.build.edit.tpl';
		}
	}

	/** Generate Navigation
	 * This is where the lower left nav is generated.. 
	 * Or rather where we used to generate it.. I guess its dead now.. 	  
	 */	 		
	public function getNavigation() {
		return false;
	}
		
	/** Generate the lower left Nav block
	 * ... figuring this out.. 	
	 */	
	public function navigationBlock($id = 0)  {
		global $user;
		
		if (empty($id) && array_key_exists('bf_id', $this->request) && !empty($this->request['bf_id'])) {
			$id = $this->request['bf_id'];
		}		
		
		$tags = false;
		$db = $this->getDBFromBID($id);
		$qb = BFQueryBuilder::select($db);
		$qb->addColumn('bf_lid',			'bf_results',	'step');		// Step unique ID
		$qb->addColumn('bf_description',	'bf_results',	'stepname');	// Step Name
		$qb->addColumn('bf_cbid',			'bf_results',	'chain');		// Chained build ID
		$qb->addColumn('bf_canbg', 			'bf_results',	'canbg');
		
		$uid = intval($user->get('bf_userid'));
		if ($uid !== 1 && !$GLOBALS['controller']->userHasAccessTo('ExecStepPerm')) {
			$qb->where(array('IN', 'bf_results', 'bf_level', array_keys(bf_get_levels($uid))));
		}
		
		if (!$this->controller->local && !$this->controller->master) {
			$qb->addColumn('bf_tag',			'bf_builds',	'tag');		// Non-GDD Chained build tag
			$qb->join(array('LEFT', 'bf_builds', array('=', 'bf_builds', 'bf_bid', 'bf_results', 'bf_cbid')));
		} else {
			// First get the list of chain build IDs
			$cbids_qb = BFQueryBuilder::select($db);
			$cbids_qb->addColumn('bf_cbid', 'bf_results', 'chain');			// Chained build ID
			$cbids_qb->where( array('=', 'bf_results', 'bf_bid', $id) );
			$cbids = $cbids_qb->getCol();
			if  (!empty($cbids)) {
				// Then get the tags from the list of chain build IDs
				$tags_qb = BFQueryBuilder::select();
				$tags_qb->addColumn('bf_bid', 'bf_builds', 'bid');			// Selected to "JOIN" via array
				$tags_qb->addColumn('bf_tag', 'bf_builds', 'tag');			// GDD Chained build tag
				$tags_qb->where('IN', 'bf_builds', 'bf_bid', $cbids);
				if ($tags_res = $tags_qb->getResult()) {
					$tags = array();
					while ($tags_row = $tags_res->fetchRow()) {
						$tags[$tags_row->bid] = $tags_row->tag;
					}
				}
			}
		}
		$qb->where( array('=', 'bf_results', 'bf_bid', $id) );
		$qb->orderBy('ASC', 'bf_results', 'bf_sid');
		
		$res = $qb->getResult();
		if (DB::isError($res)) { return; }
		
		$steps = array();
		$chains = array();
		while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC))  {
			$steps[$row['step']] = array(
					'mod'		=> 'jobs',
					'action'	=> 'edit.log',
					'id'		=> 'bf_id=' . $id . '&lid=' . $row['step'],
					'image'		=> ($row['canbg'] == 'Y' ? '../images/icons/step_threaded.gif' : ($row['canbg'] == 'J' ? 
							'../images/icons/step_joined.gif' : '../images/icons/step.gif')),
					'text'		=> $row['stepname']
			);
			if ($row['chain'])  {
				$chains[$row['step']] = array(
					'mod'		=> 'jobs',
					'action'	=> 'edit',
					'id'		=> 'bf_id=' . $row['chain'],
					'text'		=> ($tags ? $tags[$row['chain']] : $row['tag']),
					'image'		=> '../images/nav_icon_project_runs_blue.gif',
				);
			}
		}

		$qb = BFQueryBuilder::select($db);
		$qb->addColumn('bf_cbid', 'bf_builds', 'chain');			// Project level chained build ID
		$qb->where( array('=', 'bf_builds', 'bf_bid', $id) );
		$cbid = $qb->getOne();

		if (!DB::isError($cbid) && $cbid)  {
			$qb = BFQueryBuilder::select($db);
			$qb->addColumn('bf_bid',			'bf_builds',	'bid');		// Project level chained build ID
			$qb->addColumn('bf_tag',			'bf_builds',	'tag');		// Project level chained build tag
			$qb->where(array('=', 'bf_builds', 'bf_bid', $cbid));
			$qb->orderBy('ASC', 'bf_builds', 'bf_bid');
			$res = $qb->getResult();
			if (DB::isError($res)) { return; }
			
			while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC))  {
				$chains[] = array(
					'mod'		=> 'jobs',
					'action'	=> 'edit',
					'id'		=> 'bf_id=' . $row['bid'],
					'text'		=> $row['tag'],
					'image'		=> '../images/nav_icon_project_runs_blue.gif',
				);
			}
		}

		$nav = array(
			'Results' => array(
					'mod' => 'jobs',
					'action' => 'edit',
					'id' => 'bf_id=' . $id,
					'text' => translate('results'),
					'image' => '../images/icons/closed_folder.gif',
			),
			'Bill of Materials' => array(
					'mod' => 'jobs',
					'action' => 'edit.bom',
					'id' => 'bf_id=' . $id,
					'text' => translate('bill of materials'),
					'image' => '../images/icons/closed_folder.gif',
			),
			'Notes' => array(
					'mod' => 'jobs',
					'action' => 'edit.notes',
					'id' => 'bf_id=' . $id,
					'text' => translate('notes') . ' (' . $this->jte->getEntity('noteslistcount') . ')',
					'image' => '../images/icons/closed_folder.gif',
			),
			'Step Logs' => array(
					'mod' => 'jobs',
					'action' => 'edit.log',
					'text' => translate('step logs'),
					'image' => '../images/icons/open_folder.gif',
					'subnav' => $steps,
			),
			'Chains' => array(
					'mod' => 'jobs',
					'action' => 'edit.bom',
					'text' => translate('chains'),
					'image' => '../images/icons/open_folder.gif',
					'subnav' => $chains
			),
		);
		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}
	
	
	
	
	/** Generate Content
	 * This is where we rush off to generate the content, and assign it back to the template engine
	 */
	protected function generateContent() {
		global $user;
		$this->jte = new BFJTE();
			
		// Begin: Edit/Save/Delete Notes
		$this->notes = new BFNotes();
		$this->notes->noteType = 'ProjectRun';
		if (isset($this->post['edit_note_id'])) {
			$this->action[0] = 'edit.notes';
			$this->notes->currentNote = $this->post['edit_note_id'];		
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid']);	
		} else if (isset($this->post['delete_note_id'])) {
			$this->action[0] = 'edit.notes';
			$this->notes->deleteNote($this->post, $this->post['pid'], $this->post['sid']);
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid']);	
		} else if (isset($this->post['save_note'])) {
			$this->action[0] = 'edit.notes';
			$this->notes->saveNote($this->post, $this->post['pid'], $this->post['sid']);
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid']);	
		} else {
			if (isset($this->request['bf_id'])) {
				$this->notes->registerNotesList($this->jte, 'noteslist', $this->request['bf_id']);
			}	
		}		
		// End: Edit/Save/Delete Notes

		// Begin: Pausing/Unpausing steps of a running job		
		// Pause Step
		if ($this->action[0] == 'edit' && isset($this->request['pausestep']) && isset($this->request['bf_id']) && isset($this->request['bf_lid'])) {
			$jobObject = new BFJobs();
			$results = $jobObject->pauseStep($this->request['bf_id'], $this->request['bf_lid']);
		}
		// Unpause Step
		if ($this->action[0] == 'edit' && isset($this->request['unpausestep']) && isset($this->request['bf_id']) && isset($this->request['bf_lid'])) {
			$jobObject = new BFJobs();
			$results = $jobObject->unpauseStep($this->request['bf_id'], $this->request['bf_lid']);
		}
		// End: Pausing/Unpausing steps of a running job		


		// Make sure we have an ID and then gather stuff for it.				
		if (array_key_exists('bf_id', $this->request) && !empty($this->request['bf_id'])) {
			
			// Do some sort of XML request
			if ($this->request_type == BF_REQUEST_XML) {
		
				// Generate edit stuff
				if ($this->action[0] == 'edit') {
		
					// cancel the job if its set
					if (isset($this->request['job_action']) && $this->request['job_action'] == 'Cancel') {
						$jobObject = new BFJobs();
						$results = $jobObject->cancelJobs(array($this->request['bf_id']));						
					}
					
					// Change the jobs class
					if (isset($this->request['changeclass'])) {
						$jobObject = new BFJobs();
						$results = $jobObject->changeClass($this->request['bf_id'], $this->request['changeclass']);
					}
							
					// Generate Step results information
					$steps = $this->getBuildResults($this->request['bf_id']);
					$this->jte->register_object('listing', $steps, null, false);
										
					// Generate Step depths for indenting
					$depths = $this->getStepDepths($this->request['bf_id']);
					$this->jte->assign('depths', $depths);

					// Generate Job specific data for header and timing etc..
					$jobinformation = $this->getBuildInformation($this->request['bf_id']);
					$this->jte->register_object('jobinfo', $jobinformation, null, false);
				}
/*
				// Generate job bom step manifest information
				if ($this->action[0] == 'edit.bom' && isset($this->request['manifest'])) {
					$manifest = $this->getStepManifest($this->request['bf_id'], $this->request['manifest']);
					$this->jte->assign('manifest', $manifest);
					$this->jte->assign('manifestid', $this->request['manifest']);
				}

				// Generate job bom step checkpoint information
				if ($this->action[0] == 'edit.bom' && isset($this->request['checkpoint'])) {
					$checkpoint = $this->getStepCheckpoint($this->request['bf_id'], $this->request['checkpoint']);
					$this->jte->assign('checkpoint', $checkpoint);
					$this->jte->assign('checkpointid', $this->request['checkpoint']);
				}

				// Generate job bom step filter information
				if ($this->action[0] == 'edit.bom' && isset($this->request['bomfilters'])) {
					$stepfilters = $this->getStepFilters($this->request['bf_id'], $this->request['bomfilters']);
					$this->jte->assign('bomfilter', $stepfilters);
					$this->jte->assign('bomfilterid', $this->request['bomfilters']);
				}

				// Generate job custom bom information
				if ($this->action[0] == 'edit.bom' && isset($this->request['custombom'])) {
					$custombom = $this->getBomData($this->request['bf_id'], $this->request['custombom']);
					$this->jte->assign('custombom', $custombom);
					$this->jte->assign('custombomid', $this->request['custombomid']);
				}
*/				
			// Do some sort of XHTML request
			} else {

				// Generate raw BUILD information			
				$build = $this->getBuildInformation($this->request['bf_id']);
				$this->jte->register_object('build', $build, null, false);
				
				
				// Generate step results information
				if ($this->action[0] == 'edit') {			
					$steps = $this->getBuildResults($this->request['bf_id']);
					$this->jte->register_object('listing', $steps, null, false);
					$depths = $this->getStepDepths($this->request['bf_id']);
					$this->jte->assign('depths', $depths);
				}
				
				// Generate step LOG information - but not the log
				if ($this->action[0] == 'edit.log') {

					// step information
					$step = $this->getBuildResults($this->request['bf_id'], $this->request['lid']);
					$this->jte->register_object('step', $step, null, false);
				
					// log buckets - from startpoint (line number) on
					$logbuckets = $this->getLogBuckets($this->request['bf_id'], $this->request['lid']);
					$this->jte->assign('logBuckets', $logbuckets);
					
// No longer needed due to jellis' improvements to LogViewer::display() - KWP
//					$filterloginfo = $this->getStepFilterInformation($this->request['bf_id'], $this->request['lid']);
//					$this->jte->assign('filterloginfo', $filterloginfo);
				}
/*				
				// Generate job bom information
				if ($this->action[0] == 'edit.bom') {
					// step information	for steps list
					$steps = $this->getBuildResults($this->request['bf_id']);
					$this->jte->register_object('steps', $steps, null, false);

					// Define the custom bom categories
					$bomfilters = $this->getFilteredSteps($this->request['bf_id']);
					$this->jte->register_object('bomfilters', $bomfilters, null, false);
					$customBom = $this->getBomCategories($this->request['bf_id']);
					$this->jte->register_object('customBom', $customBom, null, false);
					// Are there any checkpoints?
					$checkpoints = $this->getBuildCheckpoints($this->request['bf_id']);
					$this->jte->assign('checkpoints', $checkpoints);
				}
*/
				
			}
			
						
		}
				
	}

	/** get build information 
	 *	Gather all the build information we need about $BID
	 */
	protected function getBuildInformation($bid) {
		$rep = new BFReport();
		$rep->addColumn('bf_bid',		'bf_builds',	'bid');
		$rep->addColumn('bf_pid',		'bf_builds',	'pid');
		$rep->addColumn('bf_tag',		'bf_builds',	'tag');
		$rep->addColumn('bf_start',		'bf_builds',	'start');
		$rep->addColumn('bf_start',		'bf_builds',	'started');
		$rep->addColumn('bf_state',		'bf_builds',	'state');
		$rep->addColumn('bf_result',	'bf_builds',	'result');
		$rep->addColumn('bf_can',		'bf_builds',	'cancelled');
		$rep->addColumn('bf_stage',		'bf_builds',	'stage');
		$rep->addColumn('bf_userid',	'bf_builds',	'owner');
		$rep->addColumn('bf_resource',	'bf_builds',	'selector');
		$rep->addColumn('bf_class',		'bf_builds',	'class');
		$rep->addColumn('bf_location',	'bf_geo',		'geo');
		$rep->addColumn('bf_description',	'bf_projects',	'projectname');

		$rep->where(array(
				array('=', 'bf_builds', 'bf_bid', $bid),
			));
		
		$rep->join(array(
				'INNER',
				'bf_projects',
				array('=', 'bf_builds', 'bf_pid', 'bf_projects', 'bf_pid')
			));

		$rep->join(array(
				'left',
				'bf_geo',
				array('=', 'bf_builds', 'bf_geo_id', 'bf_geo', 'bf_geo_id')
			));

		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();	
	}	

	/** get build results
	 *  Gather the build steps from results table
	 */
	public function getBuildResults($bid, $lid = '') {
		global $GLOBAL_SETTINGS, $user;
		
		$db = $this->getDBFromBID($bid);
		$rep = new BFReport($db);
		$rep->addColumn('bf_sid',			'bf_results',	'step');		// Step Number
		$rep->addColumn('bf_description',	'bf_results',	'stepname');	// Step Name
		$rep->addColumn('bf_server',		'bf_results',	'server');		// Step Server
		$rep->addColumn('bf_duration',	  	'bf_results',	'runtime');		// RunTime
		$rep->addColumn('bf_result',		'bf_results',	'result');		// Result
		$rep->addColumn('bf_bid',			'bf_results',	'bid');			// Build ID
		$rep->addColumn('bf_uid',			'bf_results',	'uid');			// Step UniqueID
		$rep->addColumn('bf_lid',			'bf_results',	'lid');			// Log ID
		$rep->addColumn('bf_spid',			'bf_results',	'spid');		// Step Project ID
		$rep->addColumn('bf_dist',			'bf_results',	'broadcast');	// {Y}es or {N}o
		$rep->addColumn('bf_canbg',			'bf_results',	'thread');	 	// {Y}es, {N}o or {J}oin
		$rep->addColumn('bf_resource',		'bf_results',	'selector');	// Step Selector
		$rep->addColumn('bf_flag',			'bf_results',	'flag');		// Flag
		$rep->addColumn('bf_cbid',			'bf_results',	'chainbuild');	// chainBuildId
		$rep->addColumn('bf_command',		'bf_results',	'command');		// StepCommand
		$rep->addColumn('bf_owner',			'bf_results',	'owner');		// Owning step ID
		
		$rep->where(array('=', 'bf_results', 'bf_bid', $bid));
		if ($lid)  {
			$rep->where(array('=', 'bf_results', 'bf_lid', $lid));
		}
		$uid = intval($user->get('bf_userid'));
		if ($uid !== 1 && !$GLOBALS['controller']->userHasAccessTo('ExecStepPerm')) {
			$rep->where(array('IN', 'bf_results', 'bf_level', array_keys(bf_get_levels($uid))));
		}

		// Add filter constraints
		if (array_key_exists('filter', $this->request) && empty($lid)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// If someone is filtering for a running :: just to be nice
			if ($userDefinedFilter->keyHasValue('~','canceled')) {
				$userDefinedFilter->removeKeyValue('~','canceled');
				$userDefinedFilter->addKeyValue('result','C');
			}

			// If someone is filtering for a running :: just to be nice
			if ($userDefinedFilter->keyHasValue('~','running') || $userDefinedFilter->keyHasValue('result','running')) {
				$userDefinedFilter->removeKeyValue('result','running');
				$userDefinedFilter->removeKeyValue('~','running');
				$userDefinedFilter->addKeyValue('result','R');
			}

			// If someone is filtering for a failed :: just to be nice
			if ($userDefinedFilter->keyHasValue('~','failed')) {
				$userDefinedFilter->removeKeyValue('~','failed');
				$userDefinedFilter->addKeyValue('result','failed');
			}

			// If someone is filtering for a passed :: just to be nice
			if ($userDefinedFilter->keyHasValue('~','passed')) {
				$userDefinedFilter->removeKeyValue('~','passed');
				$userDefinedFilter->addKeyValue('result','passed');
			}

			// If someone is filtering for a warning :: just to be nice
			if ($userDefinedFilter->keyHasValue('~','warning')) {
				$userDefinedFilter->removeKeyValue('~','warning');
				$userDefinedFilter->addKeyValue('result','warning');
			}

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey('step', array(array('LIKE', 'bf_results', 'bf_sid')));
			$userDefinedFilter->setKey('stepname', array(array('LIKE', 'bf_results', 'bf_description')));
			$userDefinedFilter->setKey('name', array(array('LIKE', 'bf_results', 'bf_description')));
			$userDefinedFilter->setKey('server', array(array('LIKE', 'bf_results', 'bf_server')));
			$userDefinedFilter->setKey('selector', array(array('LIKE', 'bf_results', 'bf_resource')), 
					array('default' => ''));
			$userDefinedFilter->setKey('runtime', array(array('LIKE', 'bf_results', 'bf_duration')));
			$userDefinedFilter->setKey('result',array(array('LIKE', 'bf_results', 'bf_result')),
					array('canceled' => 'C', 'passed' => 'P', 'failed' => 'F',
					'warning' => 'W', 'stopped' => 'B', 'skipped' => 'S',
					'Canceled' => 'C', 'Passed' => 'P', 'Failed' => 'F',
					'Warning' => 'W', 'Stopped' => 'B', 'Skipped' => 'S'));
			
			$userDefinedFilter->setKey( '~',	// ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_results', 'bf_description'),
											'OR',
											array('LIKE', 'bf_results', 'bf_server'),
											'OR',
											array('LIKE', 'bf_results', 'bf_resource'),
											'OR',
											array('LIKE', 'bf_results', 'bf_result'),
											')'
										));
										
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		$rep->orderBy(
			(isset($this->request['sort']) ? "{$this->request['sort']}" : 'ASC'),
			(isset($this->request['orderby']) ? "{$this->request['orderby']}" : 'step')
			);

		if (!isset($this->request['action']) || $this->request['action'] != 'edit.bom')  {
			$count  = (isset($this->request['count']))  ? intval($this->request['count'])  : $GLOBAL_SETTINGS['reporting']['count'];
			$offset = (isset($this->request['offset'])) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
			$rep->limit($count, $offset);
		}

		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();	
		
	}	  	

	/** Get Log Buckets
	 * This gets the various log buckets (categories) 
	 * and ultimately presents them in a checkbox format so that the user 
	 * can turn on and off the various sections of the log
	 * Note: Startpoint is the starting LID, but defaults to zero.
	 */
	protected function getLogBuckets($bid, $lid, $startpoint = 0) {
		global $user; 
		
		$db = $this->getDBFromBID($bid);
		$qb = BFQueryBuilder::select($db);
		$qb->where(array(
			array('=', 'bf_logbuckets', 'bf_uid', $lid),
			array('>=', 'bf_logbuckets', 'bf_line', $startpoint)
		));
		$qb->orderBy('ASC','bf_line');
		$res = $qb->getResult();
	
		$results = array();		
		if (!db::isError($res)) {		
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$userPref = $user->getPreference('logbucket_'.$row['bf_bucket']);
				$results[$row['bf_bucket']] = array(
					'bucket' => $row['bf_bucket'],
					'preference' => (($userPref == 'N')?'N':'Y')
					);
			}
		}
		return $results;			
	}	 	 	 	

	/** Get Log
	 * This gathers the entire steps log for output - go team!
	 */	 	
	protected function getLog($bid, $lid) {
		global $user;
		
		// Get the log buckets for this one (this is fast)
		$buckets = $this->getLogBuckets($bid, $lid, 0);		

		$rep = new BFReport();
		$rep->addColumn('bf_lid', 'bf_logs', 'lid');
		$rep->addColumn('bf_stamp', 'bf_logs', 'date');
		$rep->addColumn('bf_type', 'bf_logs', 'type');
		$rep->addColumn('bf_message', 'bf_logs', 'message');
		$rep->addColumn('bf_faction', 'bf_logs', 'faction');
		$rep->addColumn('bf_messageid', 'bf_logs', 'stored');
		$rep->where(array(
			array('=', 'bf_logs', 'bf_uid', $lid),
		));
		
		foreach ($buckets as $key=>$value) {
			if ($value['preference'] == 'N') {
				$rep->where(array('!=', 'bf_logs', 'bf_type', $value['bucket']));
			}			
		}
		
		$rep->orderBy('ASC','lid');
		
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();		
	}

	/** Get a steps manifest
	 * This gathers the steps manifest information
	 */	 	
/*
	protected function getStepManifest($bid, $lid) {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_bom_manifests');
		$qb->addColumn('bf_lid', 'bf_bom_manifests');
		$qb->addColumn('bf_name', 'bf_bom_manifests');
		$qb->addColumn('bf_prop', 'bf_bom_manifests');
		$qb->addColumn('bf_value', 'bf_bom_manifests');

		$qb->where(array(
			array('=', 'bf_bom_manifests', 'bf_bid', $bid),
			array('=', 'bf_bom_manifests', 'bf_lid', $lid)
		));
		$res = $qb->getResult();
		
		$results = array();		
		if (!db::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$results[] = array(
					'server'	=> $row['bf_name'],
					'property'	=> $row['bf_prop'],
					'value'		=> $row['bf_value']
					);
			}
		}		
		return $results;	
	}
*/

	/** Any checkpoint information for this build?
	 * Returns true or false.. If any checkpoints set true, else false.
	 */
/*
	public function getBuildCheckpoints($bid) {
		$qb = BFQueryBuilder::select();
		$qb->where(array(
			array('=', 'bf_bom', 'bf_bid', $bid)
		));
		$res = $qb->getCol();
		if (!db::isError($res)) {
			return !empty($res);	
		}
	}
*/

	/** Get a steps checkpoints
	 * Gathers the action, md5 and pathname of the checkpoints
	 * for display.  Note this is a much simpler version than the 
	 * previously implemented system
	 */	 	 	 	
/*
	public function getStepCheckpoint($bid, $lid) {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_data', 'bf_bom');
		$qb->addColumn('bf_type', 'bf_bom');
		$qb->addColumn('bf_path', 'bf_bom');
	
		$qb->where(array(
			array('=', 'bf_bom', 'bf_bid', $bid),
			array('=', 'bf_bom', 'bf_sid', $lid)
		));
		$res = $qb->getResult();
		
		$results = array();		
		if (!db::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$results[] = array(
					'action'	=> ($lid)?'Addition':'Base',
					'md5'		=> $row['bf_data'],
					'pathname'	=> '['.(($row['bf_type']=='F')?'File':'Dir').'] '.$row['bf_path']
					);
			}
		}
		return $results;	
	}
*/
	
	/** Get filtered steps
	 * Find out which steps have filter events in this build.  That's it.
	 */	 	 	
/*
	public function getFilteredSteps($bid) {
		$qb = BFQueryBuilder::select();
		
		$rep = new BFReport();
		$rep->addColumn('bf_sid',			'bf_results',		'step',		'DISTINCT');	// Step Number
		$rep->addColumn('bf_description',	'bf_results',		'stepname');				// Step Name
		$rep->addColumn('bf_logid',			'bf_filterevents',	'lid');						// Log ID
		$rep->where( array(
				array('=', 'bf_filterevents', 'bf_bid', $bid),
				array('=', 'bf_results', 'bf_lid', 'bf_filterevents', 'bf_logid'),				
				));
		$rep->countBy('bf_sid', 'bf_results');
		$res = $rep->getResult();
	
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}		
	}
*/

	public function getStepFilterInformation($bid, $lid) {
		if (!$bid || !$lid)  { return array(); }
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_line', 'bf_filterevents');
		$qb->addColumn('bf_pattern', 'bf_filterevents');
		$qb->addColumn('bf_type', 'bf_filterevents');
		$qb->addColumn('bf_action', 'bf_filterevents');
		//$qb->addColumn('bf_message', 'bf_logs');
		$qb->where(array(
			array('=', 'bf_filterevents', 'bf_bid', $bid),
			array('=', 'bf_filterevents', 'bf_logid', $lid),
		));
		$qb->orderBy('ASC', 'bf_filterevents', 'bf_line');
		$res = $qb->getResult();

		$results = array();
		$preresults = array();
		if (!db::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				if (!$row['bf_line'] || $row['bf_type'] == 'U') continue;
				$preresults[($row['bf_line'])][] = $row;
			}

			// Preprocess the filter results now by incrementing the line number by the count. 
			// this way we are keying on the actual line, not the FILT line
			// Frankly I think this is compensation for an engine bug.. but whatever
			foreach ($preresults as $key=>$value) {
				$results[($key + count($value))] = $value;
			}

		}
		return $results;
	}


	/** Get a steps filters ######@TODO Stop using this, it's murder on the database.. 
	 * Gathers the line #, pattern, action, and data of the filter events for display.
	 * NOTE:  This function currently does a lot of extraneous selects because the engine is 
	 * not entering the right line number in bf_filterevents - the messages logged by bf_filterevents
	 * are pushing the actual log output row down.  When and if this is ever taken care of, every query but the
	 * first will mysteriously vanish.
	 * PPS: Both the LogViewer and BomViewer objects use this, so don't move it.
	 */	 	 	 	
	public function getStepFilters($bid, $lid) {
		if (!$bid || !$lid)  {
			return array();
		}
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_line', 'bf_filterevents');
		$qb->addColumn('bf_pattern', 'bf_filterevents');
		$qb->addColumn('bf_type', 'bf_filterevents');
		$qb->addColumn('bf_action', 'bf_filterevents');
		//$qb->addColumn('bf_message', 'bf_logs');
		$qb->where(array(
			array('=', 'bf_filterevents', 'bf_bid', $bid),
			array('=', 'bf_filterevents', 'bf_logid', $lid),
			//array('=', 'bf_logs', 'bf_uid', $suid),
			//array('=', 'bf_filterevents', 'bf_logid', 'bf_logs', 'bf_uid'),
			//array('=', 'bf_filterevents', 'bf_line', 'bf_logs', 'bf_lid')
		));
		$qb->orderBy('ASC', 'bf_filterevents', 'bf_line');
		$res = $qb->getResult();
		
		$results = array();
		$i = 0;
		if (!db::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				if (!$row['bf_line'])  {
					continue;
				}
				$qb2 = BFQueryBuilder::select();
				$qb2->addColumn('bf_message', 'bf_logs');
				$qb2->addColumn('bf_lid', 'bf_logs');
				$qb2->where(array(
					array('=', 'bf_logs', 'bf_uid', $lid),
					array('>=', 'bf_logs', 'bf_lid', $row['bf_line']),
					array('=', 'bf_logs', 'bf_type', 'EXEC')
				));
				$qb2->orderBy('ASC', 'bf_logs', 'bf_lid');
				$qb2->limit(1);
				$message = $qb2->getRow();

				$results[$i] = array(
					'line'		=> $message['bf_lid'],
					'filter'	=> $row['bf_pattern'],
					'action'	=> $row['bf_action'] . ' ' . $row['bf_type'],
					'actionkey'	=> $row['bf_action'],
					'typekey'	=> $row['bf_type'],
					'data'		=> $message['bf_message']
					);
				switch($row['bf_type'])  {
					case 'E':
						$results[$i]['action'] = translate('error');  
						break;
					case 'F':  
						$results[$i]['action'] = 
								$row['bf_action'] == 'H' ? translate('set fail / halt') : translate('set fail');  
						break;
					case 'W':
						$results[$i]['action'] = translate('warning');
						break;
					case 'C':
						$results[$i]['action'] = 
								$row['bf_action'] == 'H' ? translate('clear fail / halt') : translate('clear fail');  
						break;
					case 'R':
						$results[$i]['action'] = translate('notify changers');  
						break;
					case 'U':  
						$results[$i]['action'] = 
								$row['bf_action'] == 'H' ? translate('clear warning / halt') : 
								translate('clear warning');  
						break;
					case 'S':  
						$results[$i]['action'] = translate('halt');  
						break;
				}
				$i++;
			}
		}
		return $results;	
	}

	
	/**
	 * Given a build ID, gets the steps in that build and figures out how deeply inlined they are (basically,
	 * how much to indent them).
	 */
	public function getStepDepths($bid)  {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_sid',			'bf_results',	'sid');			// Log ID
		$qb->addColumn('bf_owner',			'bf_results',	'owner');		// Owning step ID
		$qb->where(array('=', 'bf_results', 'bf_bid', $bid));
		$qb->orderBy('ASC', 'bf_results', 'bf_sid');
		$res = $qb->getResult();
		
		$results = array();
		$i = 0;
		if (!db::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$results[$row['sid']] = $row['owner'] && isset($results[$row['owner']]) ?
						$results[$row['owner']] + 1 : 0;
			}
		}
		return $results;
	}
	
	/** Get Custom Bom Categories
	 * users can add their own bom categories and content through various dot commands
	 * so we have to be able to get that information back out again.
	 * using a report instead of an array since foreach is bad	 
	 */	 	 	
/*
	public function getBomCategories($bid) {
		$qb = BFQueryBuilder::select();
		$rep = new BFReport();
		
		$rep->addColumn('bf_title', 	'bf_interfacecategories', 'title');
		$rep->addColumn('bf_category',	'bf_interfacecategories', 'category');
		
		$rep->where(array(
			array('=', 'bf_interfacecategories', 'bf_bid', $bid)
		));
		$res = $rep->getResult();
	
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}		
	}
*/	
	/** Get Bom Data
	 * This gets the data for a given custom bom section based on the build id and the category
	 * The data block is now in store and must be yanked from there
	 * Will have to use foreach for this one, and not sure what interfacedata sections are
	 */	 	 	 	

/** getBomData version 2
 * Due to a complete rehash of how this is done by the engine, we have to rethink this completely.
 * Note: I think the way this is done is completely retarded and counter intuitive. It's unnecessarily 
 * complex and doesn't lend itself to .. well anything.. 
 *
 * First: Get the sections (containers) from interfacesections and hang on to them - cause you need them
 *
 * Second: Step through the entire category's data and build an array based that can be passed to the xml
 */
/*
	public function getBomData($bid, $category) {
		$storage = new BFStore();
		
		$qb = BFQueryBuilder::select();
		$qb->where(array(
			array('=', 'bf_interfacesections', 'bf_bid', $bid),
			array('=', 'bf_interfacesections', 'bf_category', $category)
		));
		$qb->orderBy('asc', 'bf_order');
		$res = $qb->getResult();

		// Get the sections in this category and place them in an array for later use	
		$sections = array();
		if (!DB::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$sections[$row['bf_section']] = $row;
			}
		}

		// Now get the data from this category and begin the process of placing it

		// Final result set
		$results = array();
		// The line id {key} has the following section {value} and field {value}
		$lineID = array();
		$qb = BFQueryBuilder::select();
		$qb->where(array(
			array('=', 'bf_interfacedata', 'bf_bid', $bid),
			array('=', 'bf_interfacedata', 'bf_category', $category),
		));
		$qb->orderBy('asc', 'bf_lid');
		$res = $qb->getResult();

		if (!DB::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$data = $storage->getStorage($row['bf_data']);
			
				// If this is a parent section, then add it to the root and set the index informationi
				if ($sections[$row['bf_section']]['bf_parent'] == '') {
					// Capture the line id so we have some sort of index to use
					$lastIndex = $row['bf_lid'];
					// Add the data to the result set
					$results[$row['bf_section']][$row['bf_field']][$lastIndex] = array(array($data['TEXT'],$sections[$row['bf_section']]['bf_expand']));
					// Capture the index information so we have it for childrensesses
					$lineID[$lastIndex] = array('section'=>$row['bf_section'],'field'=>$row['bf_field']);
				// If this isn't a parent and this lines section is a child of the last indexes section then
				// add it to the array as such.  This crap right here makes baby jebus cry.
				} else if ($lineID[$lastIndex]['section'] == $sections[$row['bf_section']]['bf_parent']) {						
					$results[ $lineID[$lastIndex]['section'] ][ $lineID[$lastIndex]['field'] ][$lastIndex][$row['bf_field']][] = $data['TEXT'];
				// If the next line isn't a parent and it isnt a child of the previous parent then your data is borked.
				} else {
					// Not sure what to do in this event.. probably should ponder it.
				}
			}
		}
//print_r($results);die;
		return $results;

	}
*/
	protected function getDBFromBID($bid) {
		global $controller;

		// The database connection may change depending on the GDD info
		$db = null;
		if ($controller->master || $controller->local) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_geo_id', 'bf_builds', 'bf_geo_id');
			$qb->where(array('=', 'bf_builds', 'bf_bid', $bid));
			$geo = $qb->getOne();
			$db = $controller->getDB($geo);
		}
		return $db;
	}
}


class BomViewer extends jTEPHP {

	protected $bfid;	// The ID of the build
	protected $parent; 	// used to call paretn functions when needed.
	protected $storage;

	// All of the below variables are used during custom bom creation.
	protected $sectionLayoutTitles;		
	protected $sectionLayoutFields;
	protected $lastValue;
	protected $lastField;
	protected $parentage;
	protected $expandable;
	protected $sectionDepth;
	protected $hasKids;

	public function __construct($bfid,$parent) {
		$this->bfid = $bfid;
		$this->parent = $parent;
		$this->storage = new BFStore();
	}

	public function display() {
		global $user;

		// Step Information
		$steps = $this->parent->getBuildResults($this->bfid);

		// Checkpoints
		$hasCheckpoints = $this->getBuildCheckpoints($this->bfid);
		$stepCheckpoints = array();
		if (!empty($hasCheckpoints)) {
			foreach ($hasCheckpoints as $k=>$v) {
				$stepCheckpoints[$v] = $this->getStepCheckpoint($this->bfid, $v);
			}
		}
		// Filters
		$bomFilters = $this->getFilteredSteps($this->bfid);

		// Custom Bom Categories
		$bomCats = $this->getBomCategories($this->bfid);


		// For now - and likely forever.
		include('templates/php.build.edit.bom.tpl');

	}

	/** Get Custom Bom Categories
	 * users can add their own bom categories and content through various dot commands
	 * so we have to be able to get that information back out again.
	 * using a report instead of an array since foreach is bad	 
	 */	 	 	
	public function getBomCategories($bid) {
		$qb = BFQueryBuilder::select();
		$rep = new BFReport();
		
		$rep->addColumn('bf_title', 	'bf_interfacecategories', 'title');
		$rep->addColumn('bf_category',	'bf_interfacecategories', 'category');
		
		$rep->where(array(
			array('=', 'bf_interfacecategories', 'bf_bid', $bid)
		));
		$res = $rep->getResult();
	
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}		
		return new BFReportResults();		
	}

	/** In the course of building the custom bom, we have to gather all manner of data
	 * so that we know what to display.  This function will do just that on a category basis.
	 */
	public function getBomData($bid, $category) {
		$storage = new BFStore();
		
		$qb = BFQueryBuilder::select();
		$qb->where(array(
			array('=', 'bf_interfacesections', 'bf_bid', $bid),
			array('=', 'bf_interfacesections', 'bf_category', $category)
		));
		$qb->orderBy('asc', 'bf_order');
		$res = $qb->getResult();
		
		if (!db::isError($res)) {

			$sectionLayoutTitles = array();
			$sectionLayoutFields = array();
			$lastValue = array();
			$lastField = array();
			$parentage = array();
			$expandable = array();
			$sectionDepth = array();
			$hasKids = array();

			while($layout = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				if (isset($lastValue[$layout['bf_section']][$layout['bf_field']])) {
					continue;
				}
				$sectionLayoutTitles[$layout['bf_section']][] = $layout['bf_title'];
				$sectionLayoutFields[$layout['bf_section']][] = $layout['bf_field'];
				$lastValue[$layout['bf_section']][$layout['bf_field']] = '';
				$lastField[$layout['bf_section']] = $layout['bf_field'];
				if (!empty($layout['bf_parent'])) {
					$parentage[$layout['bf_section']] = $layout['bf_parent'];
					$hasKids[$layout['bf_parent']] = 1;
				}
				$expandable[$layout['bf_section']] = $layout['bf_expand'];
			}
		}
		
		foreach ($lastField as $section=>$v) {
			$sectionDepth[$section] = $this->depth($section,$parentage);
		}

		$nqb = BFQueryBuilder::select();
		$nqb->where(array(
			array('=', 'bf_interfacedata', 'bf_bid', $bid),
			array('=', 'bf_interfacedata', 'bf_category', $category)
		));
		$nqb->orderBy('asc', 'bf_lid');

		$datahandle = $nqb->getResult();

		return array($datahandle,$sectionLayoutTitles,$sectionLayoutFields,$lastValue,$lastField,$parentage,$expandable,$sectionDepth,$hasKids);

	}

	private function depth($sec,$pHash) {
		if (isset($pHash[$sec])) {
			$depth = $this->depth($pHash[$sec],$pHash);
			return ++$depth;
		}
		return 1;
	}

/*  Newly inserted function getDBFromBID 
	needed for GDD functionality */
	
	protected function getDBFromBID($bid) {
		global $controller;
		// The database connection may change depending on the GDD info
		$db = null;
		if ($controller->master || $controller->local) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_geo_id', 'bf_builds', 'bf_geo_id');
			$qb->where(array('=', 'bf_builds', 'bf_bid', $bid));
			$geo = $qb->getOne();
			$db = $controller->getDB($geo);
		}
		return $db;
	}

	/** Get a steps manifest
	 * This gathers the steps manifest information
	 */	 	
	public function getStepManifest($bid, $lid) {
		// Only in GDD in certain circumstances in certain phases of the moon, and when you're name is randy do you need to do this
		$db = $this->getDBFromBID($bid);
		$qb = BFQueryBuilder::select($db);

		$qb->addColumn('bf_bid', 'bf_bom_manifests');
		$qb->addColumn('bf_lid', 'bf_bom_manifests');
		$qb->addColumn('bf_name', 'bf_bom_manifests');
		$qb->addColumn('bf_prop', 'bf_bom_manifests');
		$qb->addColumn('bf_value', 'bf_bom_manifests');

		$qb->where(array(
			array('=', 'bf_bom_manifests', 'bf_bid', $bid),
			array('=', 'bf_bom_manifests', 'bf_lid', $lid)
		));
		$res = $qb->getResult();
		
		$results = array();		
		if (!db::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$results[] = array(
					'server'	=> $row['bf_name'],
					'property'	=> $row['bf_prop'],
					'value'		=> $row['bf_value']
					);
			}
		}		
		return $results;	
	}

	/** Any checkpoint information for this build?
	 * Return a list of SID's that have checkpoint information
	 * TODO: merge into one query and test. Doing this change
	 * just prior to code freeze and do NOT want to rock the boat
	 */
	public function getBuildCheckpoints($bid) {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_sid', 'bf_bom');
		$qb->where(array(
			array('=', 'bf_bom', 'bf_bid', $bid),
			array('!=', 'bf_bom', 'bf_sid', 0)
		));
		$res1 = $qb->getCol();
		if (db::isError($res1)) {
			return;	
		}
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_sid', 'bf_bom');
		$qb->where(array(
			array('=', 'bf_bom', 'bf_bid', $bid),
			array('=', 'bf_bom', 'bf_sid', 0)
		));
		$qb->limit(1);
		$res2 = $qb->getCol();
		if (db::isError($res2)) {
			return;	
		}
		return array_merge($res1, $res2);
	}

	/** Get a steps checkpoints
	 * Gathers the action, md5 and pathname of the checkpoints
	 * for display.  Note this is a much simpler version than the 
	 * previously implemented system
	 */	 	 	 	
	public function getStepCheckpoint($bid, $lid) {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_data', 'bf_bom');
		$qb->addColumn('bf_type', 'bf_bom');
		$qb->addColumn('bf_path', 'bf_bom');
	
		$qb->where(array(
			array('=', 'bf_bom', 'bf_bid', $bid),
			array('=', 'bf_bom', 'bf_sid', $lid)
		));
		$res = $qb->getResult();
		
		$results = array();		
		if (!db::isError($res)) {
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$results[] = array(
					'action'	=> ($lid)?'Addition':'Base',
					'md5'		=> $row['bf_data'],
					'pathname'	=> '['.(($row['bf_type']=='F')?'File':'Dir').'] '.$row['bf_path']
					);
			}
		}
		return $results;	
	}

	/** Get filtered steps
	 * Find out which steps have filter events in this build.  That's it.
	 * 
	 * I have some questions as to whether or not this query will work in oracle.. -Jason
	 */	 	 	
	public function getFilteredSteps($bid) {
		$qb = BFQueryBuilder::select();
		
		$rep = new BFReport();
		$rep->addColumn('bf_sid',			'bf_results',		'step',		'DISTINCT');	// Step Number
		$rep->addColumn('bf_description',	'bf_results',		'stepname');				// Step Name
		$rep->addColumn('bf_logid',			'bf_filterevents',	'lid');						// Log ID
		$rep->where( array(
				array('=', 'bf_filterevents', 'bf_bid', $bid),
				array('=', 'bf_results', 'bf_lid', 'bf_filterevents', 'bf_logid'),				
				));
		$rep->countBy('bf_sid', 'bf_results');
		$res = $rep->getResult();
	
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}		
	}


}



class LogViewer extends jTEPHP {
	
	protected $bfid;
	protected $lid;
	protected $parent;  // Used to call parent functions when needed.

	public function __construct($bfid,$lid,$filter,$parent) {
		$this->bfid = $bfid;
		$this->lid = $lid;
		$this->filter = $filter;
		$this->parent = $parent;
	}

	public function display() {
		global $user;
		$steplog = $this->getLog($this->bfid, $this->lid);
		$temp = $this->parent->getStepFilters($this->bfid, $this->lid);
		$filterlog = array();
		foreach ($temp as $value) {
			$filterlog[$value['line']][] = $value;
		}

		// For now - and likely forever
		include('templates/php.build.edit.log.tpl');

	}

	protected function getDBFromBID($bid) {
		global $controller;

		// The database connection may change depending on the GDD info
		$db = null;
		if ($controller->master || $controller->local) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_geo_id', 'bf_builds', 'bf_geo_id');
			$qb->where(array('=', 'bf_builds', 'bf_bid', $bid));
			$geo = $qb->getOne();
			$db = $controller->getDB($geo);
		}
		return $db;
	}

	/** Get Log
	 * This gathers the entire steps log for output - go team!
	 */	 	
	protected function getLog($bid, $lid) {
		global $user;
		
		// Get the log buckets for this one (this is fast)
		$buckets = $this->getLogBuckets($bid, $lid, 0);		

		$db = $this->getDBFromBID($bid);
		$rep = new BFReport($db);
		$rep->addColumn('bf_lid', 'bf_logs', 'lid');
		$rep->addColumn('bf_stamp', 'bf_logs', 'date');
		$rep->addColumn('bf_type', 'bf_logs', 'type');
		$rep->addColumn('bf_message', 'bf_logs', 'message');
		$rep->addColumn('bf_faction', 'bf_logs', 'faction');
		$rep->addColumn('bf_messageid', 'bf_logs', 'stored');
		$rep->where(array(
			array('=', 'bf_logs', 'bf_uid', $lid),
		));
		
		foreach ($buckets as $key=>$value) {
			if ($value['preference'] == 'N') {
				$rep->where(array('!=', 'bf_logs', 'bf_type', $value['bucket']));
			}			
		}
		
		// Add filter constraints - if there are any
		if (isset($this->filter)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->filter );
			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey('type', array(array('LIKE', 'bf_logs', 'bf_type')));
			$userDefinedFilter->setKey('message', array(array('LIKE', 'bf_logs', 'bf_message')));

			$userDefinedFilter->setKey( '~',	// ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_logs', 'bf_type'),
											'OR',
											array('LIKE', 'bf_logs', 'bf_message'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}



		$rep->orderBy('ASC','lid');
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();		
	}

	/** Get Log Buckets
	 * This gets the various log buckets (categories) 
	 * and ultimately presents them in a checkbox format so that the user 
	 * can turn on and off the various sections of the log
	 * Note: Startpoint is the starting LID, but defaults to zero.
	 */
	protected function getLogBuckets($bid, $lid, $startpoint = 0) {
		global $user; 
		
		$db = $this->getDBFromBID($bid);
		$qb = BFQueryBuilder::select($db);
		$qb->where(array(
			array('=', 'bf_logbuckets', 'bf_uid', $lid),
			array('>=', 'bf_logbuckets', 'bf_line', $startpoint)
		));
		$qb->orderBy('ASC','bf_line');
		$res = $qb->getResult();
	
		$results = array();		
		if (!db::isError($res)) {		
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$userPref = $user->getPreference('logbucket_'.$row['bf_bucket']);
				$results[$row['bf_bucket']] = array(
					'bucket' => $row['bf_bucket'],
					'preference' => (($userPref == 'N')?'N':'Y')
					);
			}
		}
		return $results;			
	}	 	 	 	

}

?>
