<?php
/* (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the copyright and other intellectual property rights in this software.
 Duplication or use of the Software is not permitted except as expressly provided in a written agreement between your company and BuildForge, Inc. */

// insert into bf_modules values ('jobs','Jobs','Jobs',6,'Y','Y',25)

/**
 * Jobs
 *
 * @package Console
 * @subpackage Jobs
 */

$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');
$GLOBALS['controller']->require_module('BFReport');

class Runs extends Module implements ViewableXHTML {

	const VIEW				= 'view'; // do user pref
	const VIEW_ALL			= 'view.all';
	const VIEW_COMPLETED	= 'view.completed';
	const VIEW_RUNNING		= 'view.running';
	const VIEW_ARCHIVED		= 'view.archived';
	const VIEW_LOCKED		= 'view.locked';

	const JOB_ACTION		= 'job_action';
	const JOB_PURGE			= 'Purge';
	const JOB_LOCK			= 'Lock';
	const JOB_UNLOCK		= 'Unlock';
	const JOB_CANCELPURGE	= 'cancelpurge';
	const JOB_CANCEL		= 'Cancel';

	const CHECKBOXNAME 		= 'jobs';

	public $title = 'Project Runs';
	public $content;

	protected $jte;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML);
	protected $request;
	protected $request_type;
	public $action;

	/** Handle Request
	 * First thing run by central controller
	 * NO CLASS is created when this is run.  This does the primary job of creating the class
	 *  which then defines the rest of the behaviour throughout the object.
	 */	 
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$jobs = new Runs();
		$jobs->setRequest($req, $type);
		return $jobs;
	}
	
	/** Valid Request Types
	 * Check to see what request types this module handles and return true if we can do it
	 */	 	
	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}
	
	/** includes
	 * This is where we define what other things need to be included.. like javascript
	 */	 	
	public function getIncludes() {
		return array( 
					'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Jobs/javascripts/content.js'), 
					'stylesheets' => array(BF_WEBROOT . 'fullcontrol/modules/Jobs/styles/global.css') 
					);
	}
	
	/** Set Request
	 * this passes the get/post request to the object and gets the action out
	 */	 	
	public function setRequest($req, $type = BF_REQUEST_XHTML) {
		global $GLOBAL_SETTINGS;
		$this->request = $req;
		$this->request_type = $type;

		if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $this->request)) {
			$this->action = array($this->request[$GLOBAL_SETTINGS['site']['actionkey']]);
		} else {
			$this->action = array();
		}
	}
	
	/** Render
	 * This is the primary render function..  Here we make the magic happen.
	 * This is called by Central Controller	 
	 */	 	
	public function render() {
		$this->generateContent();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Jobs/templates');
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}
	
	/** Choose Templates
	 * this function determines what templates we should be using
	 * in this case it will either be an xml template, or the main xhtml.view template
	 * our five main views use the same template as they are all the same	 
	 */	 	  	
	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XML) {
			switch($this->action[0]) {
				case self::VIEW_ALL:
				case self::VIEW_COMPLETED:
				case self::VIEW_RUNNING:
				case self::VIEW_ARCHIVED:
				case self::VIEW_LOCKED:
					return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/xml.view.tpl';
					break;
				default:
					return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/xml.' . $this->action[0] . '.tpl';
					break;
			}
		} else {
			return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/xhtml.view.tpl';
		}
	}

	public function getNavigation() {
		return false;
	}
	
	/** Generate Content
	 * This is where we rush off to generate the content, and assign it back to the template engine
	 */	 	
	protected function generateContent() {
		global $user;
		$this->jte = new BFJTE();	
					
		$this->jte->assign('is_styx', $this->controller->configuration->styx()); // Styx Hack

		// If no action is set, check to see if we have done one previously and use that
		$jobs_defaultpage = $user->getPreference('jobs_defaultpage');
		if ((empty($this->action) && !empty($jobs_defaultpage)) || (in_array(self::VIEW, $this->action))) {
			$this->action[] = $jobs_defaultpage;
		}
		
		/** Perform some actions
		 * Some actions may be taken which depends on a key to exist in the request		 
		 */
		if (array_key_exists(self::JOB_ACTION, $this->request)) {
				
			// Purge 		 		
			if ($this->request[self::JOB_ACTION] == self::JOB_PURGE) {
				$jobObject = new BFJobs();
				$results = $jobObject->purgeJobs($this->request[self::CHECKBOXNAME]);
			}

			// Lock
			if ($this->request[self::JOB_ACTION] == self::JOB_LOCK) {
				$jobObject = new BFJobs();
				$results = $jobObject->lockJobs($this->request[self::CHECKBOXNAME]);
			}

			// Unlock 
			if ($this->request[self::JOB_ACTION] == self::JOB_UNLOCK) {
				$jobObject = new BFJobs();
				$results = $jobObject->unlockJobs($this->request[self::CHECKBOXNAME]);
			}
			
			// Cancel
			if ($this->request[self::JOB_ACTION] == self::JOB_CANCEL) {			
				$jobObject = new BFJobs();
				$results = $jobObject->cancelJobs($this->request[self::CHECKBOXNAME]);
			}
				
		}
		
		// Cancel Purge - Single button click, so no job_action
		if (array_key_exists(self::JOB_CANCELPURGE, $this->request)) {
			$jobObject = new BFJobs();
			$results = $jobObject->cancelPurge($this->request[self::JOB_CANCELPURGE]);
		}
		 
		 
		/** Display the data
		 * These are the various methods by which we display the data
		 */		 		
		
		// view.all
		if (in_array(self::VIEW_ALL, $this->action)) {
			$builds = $this->getJobs(array('C','R','W','F','A','L'));
			$user->setPreference('jobs_defaultpage', self::VIEW_ALL);
		}		
		
		// view.completed or action empty then assume completed
		if (in_array(self::VIEW_COMPLETED, $this->action) || empty($this->action)) {
			$builds = $this->getJobs(array('C'));
			$user->setPreference('jobs_defaultpage', self::VIEW_COMPLETED);
		}

		// view.running (running, waiting and fetal)
		if (in_array(self::VIEW_RUNNING, $this->action)) {
			$builds = $this->getJobs(array('R','W','F'));
			$user->setPreference('jobs_defaultpage', self::VIEW_RUNNING);
		}
		
		// view.archived
		if (in_array(self::VIEW_ARCHIVED, $this->action)) {
			$builds = $this->getJobs(array('A'));
			$user->setPreference('jobs_defaultpage', self::VIEW_ARCHIVED);
		}

		// view.locked
		if (in_array(self::VIEW_LOCKED, $this->action)) {
			$builds = $this->getJobs(array('L'));
			$user->setPreference('jobs_defaultpage', self::VIEW_LOCKED);
		}
		
		// Set builds to template engine		
		$this->jte->register_object('listing', $builds, null, false);
		$this->jte->register_object('module', $this, null, false);
		
		// Get the purge list and submit it to the template engine
		$this->jte->assign('purges', $this->getPurges());
		if (isset($this->geos)) {
			$this->jte->assign('geos', $this->geos);
		}
		
		// Set up function to check the number of filter matches for a build.
		$this->jte->register_function('bf_get_num_filter_matches',	'bf_get_num_filter_matches');
	}

	/** Get Jobs
	 * This does the actual magic of building the report based on the view we are currently in.
	 * We are getting BUILDS here or Project Runs.. or Jobs.. whatever we are calling them now.	 	 
	 */	 	
	protected function getJobs($states) {
		global $controller, $GLOBAL_SETTINGS, $user;
		$rep = new BFReport();
		$rep->addColumn('bf_bid',			'bf_builds',	'bid');
		$rep->addColumn('bf_tag',			'bf_builds',	'tag');
		$rep->addColumn('bf_description',	'bf_projects',	'project');
		$rep->addColumn('bf_class',			'bf_builds',	'class');
		$rep->addColumn('bf_state',			'bf_builds',	'state');
		$rep->addColumn('bf_result',		'bf_builds',	'result');
		$rep->addColumn('bf_fired',			'bf_builds',	'date');
		$rep->addColumn('bf_duration',  	'bf_builds',	'runtime');
		$rep->addColumn('bf_name',			'bf_users',	 	'owner');
		$rep->addColumn('bf_pid',			'bf_builds',	'pid');
		$rep->addColumn('bf_process',   	'bf_builds',	'process');
		$rep->addColumn('bf_userid',		'bf_builds',	'userid');
		$rep->addColumn('bf_can',			'bf_builds',	'cancelled');
		$rep->addColumn('bf_platform',		'bf_projects',	'selector');
		$rep->addColumn('bf_stage',			'bf_builds',	'stage');
		if ($controller->master || $controller->local) {
			$rep->addColumn('bf_geo_id',	'bf_builds',		'geo_id');
		}

		$uid = intval($user->get('bf_userid'));

		$rep->join(array('LEFT', 'bf_users',	array( array('=', 'bf_users', 'bf_userid', 'bf_builds', 'bf_userid') ) ));

		if ($uid === 1) {
			$rep->join(array('INNER', 'bf_projects',	array( array('=', 'bf_projects', 'bf_pid', 'bf_builds', 'bf_pid') ) ));
			$rep->where(array(
				array('=', 'bf_builds', 'bf_process', 'B'),
			));
		} else {
			$rep->join(array('INNER', 'bf_projects',	array( array('=', 'bf_projects', 'bf_pid', 'bf_builds', 'bf_pid') ) ));
			$rep->join(array('INNER', 'bf_levels',	array( array('=', 'bf_builds', 'bf_level', 'bf_levels', 'bf_level') ) ));
			$rep->where(array(
				array('=', 'bf_builds', 'bf_process', 'B'),
				array('IN', 'bf_builds', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}

		if ($controller->master || $controller->local) {
			$this->geos = bf_get_geo();
		}

		$rep->where(array('IN', 'bf_builds', 'bf_state', $states));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// If someone is filtering for a cancel, here is how we do that since its not directly related to the result
			if ($userDefinedFilter->keyHasValue('~','canceled') || $userDefinedFilter->keyHasValue('result','canceled')) {
				$userDefinedFilter->removeKeyValue('~','canceled');
				$userDefinedFilter->removeKeyValue('result','canceled');
				$userDefinedFilter->addKeyValue('canceled','canceled');
			}

			// If someone is filtering for a running, here is how we do that since its not directly related to the result
			if ($userDefinedFilter->keyHasValue('~','running') || $userDefinedFilter->keyHasValue('result','running')) {
				$userDefinedFilter->removeKeyValue('~','running');
				$userDefinedFilter->removeKeyValue('result','running');
				$userDefinedFilter->addKeyValue('state','running');
			}

			// If someone is filtering for a failed :: just to be nice
			if ($userDefinedFilter->keyHasValue('~','failed')) {
				$userDefinedFilter->removeKeyValue('~','failed');
				$userDefinedFilter->addKeyValue('result','failed');
			}

			// If someone is filtering for a passed :: just to be nice
			if ($userDefinedFilter->keyHasValue('~','passed')) {
				$userDefinedFilter->removeKeyValue('~','passed');
				$userDefinedFilter->addKeyValue('result','passed');
			}

			// If someone is filtering for a warning :: just to be nice
			if ($userDefinedFilter->keyHasValue('~','warning')) {
				$userDefinedFilter->removeKeyValue('~','warning');
				$userDefinedFilter->addKeyValue('result','warning');
			}

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey('tag', array(array('LIKE', 'bf_builds', 'bf_tag')));
			$userDefinedFilter->setKey('canceled', array(array('=', 'bf_builds', 'bf_can')), 
					array('canceled' => 'Y', 'Canceled' => 'Y'));
			$userDefinedFilter->setKey('result', array(array('=', 'bf_builds', 'bf_result')), 
					array('passed' => 'P', 'failed' => 'F', 'warning' => 'W', 'Passed' => 'P', 'Failed' => 'F', 'Warning' => 'W'));
			$userDefinedFilter->setKey('state', array(array('=', 'bf_builds', 'bf_state')), 
					array('completed' => 'C', 'running' => 'R', 'Completed' => 'C', 'Running' => 'R'));
			$userDefinedFilter->setKey('class', array(array('LIKE', 'bf_projects', 'bf_class')));
			$userDefinedFilter->setKey('project',array(array('LIKE', 'bf_projects', 'bf_description')));
			$userDefinedFilter->setKey('library',array(array('LIKE', 'bf_projects', 'bf_description')));
			$userDefinedFilter->setKey('owner', array(array('LIKE', 'bf_users', 'bf_name')));			
			if ($controller->master || $controller->local) {
				$userDefinedFilter->setKey('console', array(array('LIKE', 'bf_builds', 'bf_geo_id')), array_merge(array('primary'=>''),array_flip($this->geos)));
			}
			
			$userDefinedFilter->setKey( '~',	// ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_builds', 'bf_tag'),
											'OR',										
											array('LIKE', 'bf_projects', 'bf_description'),
											'OR',
											array('LIKE', 'bf_builds', 'bf_class'),
											'OR',
											array('LIKE', 'bf_builds', 'bf_state'),
											'OR',
											array('LIKE', 'bf_users', 'bf_name'),
											')'
										));
										
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'DESC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'date')
			);
		
		// If requested, set a new page to display. Otherwise, use defaults
		$userPrefCount = $user->getPreference('view_count');
		$count	= (array_key_exists('count', $this->request))?intval($this->request['count']):((!empty($userPrefCount))?intval($userPrefCount):$GLOBAL_SETTINGS['reporting']['count']);
		$offset = (array_key_exists('offset', $this->request))	? intval($this->request['offset'])	: $GLOBAL_SETTINGS['reporting']['offset'];		
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();
	}


	protected function getPurges() {
		$qb	= BFQueryBuilder::select();

		$qb->addColumn('bf_pid', 'bf_builds', 'pid'); // the id of the build to be purged
		$qb->addColumn('bf_bid', 'bf_builds', 'bid'); // the purge builds id
		$qb->addColumn('bf_can', 'bf_builds', 'can'); // are we canceling?
		$qb->join(array('LEFT', 'bf_users',	array( array('=', 'bf_users', 'bf_userid', 'bf_builds', 'bf_userid') ) ));
		$qb->where(array('=', 'bf_builds', 'bf_process', 'P'));
		$res = $qb->getResult();
	
		$results = array();
		while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$results[$row['pid']] = array(
					'pid'	=> $row['pid'],
					'bid'	=> $row['bid'],
					'can'	=> $row['can']
				);
		}
		return $results;	
	}


	public function navigationBlock() {
		return '';
	}


}

function bf_get_num_filter_matches($bid, $type = 'F')  {
	$qb	= BFQueryBuilder::select();

	$qb->addColumn('bf_logid', 'bf_filterevents', 'count', 'COUNT');
	$qb->where(array(
		array('=', 'bf_filterevents', 'bf_bid', $bid),
		array('=', 'bf_filterevents', 'bf_type', $type),
		));
	$ret = $qb->getOne();
	return DB::isError($ret) ? 0 : $ret;
}

?>
