<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Admin Semaphores
 *
 * @package Console
 * @subpackage Semaphores
 */

$GLOBALS['controller']->require_module('BFSemaphore');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';

class Semaphores extends Jobs {
	const ACTION_VIEW = 'semaphores.view';
	const ACTION_EDIT = 'semaphores.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $semaphores;
	protected $references;

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			$this->semaphores = new BFSemaphore();
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$semaphores = new Semaphores();
		$temp = new BFSemaphore();
		$semaphores->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$semaphores->load($req[$temp->primary_key]);
		} else {
			$semaphores->load();
		}
		return $semaphores;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $GLOBAL_SETTINGS;
		$rep = new BFReport();

		// Setup the columns, most of which will get displayed
		$rep->addColumn('bf_bid', 'bf_semaphores', 'id');
		$rep->addColumn('bf_semaphore', 'bf_semaphores', 'semaphore');
		$rep->addColumn('bf_active', 'bf_semaphores', 'status');
		$rep->addColumn('bf_stamp', 'bf_semaphores', 'obtained');
		$rep->addColumn('bf_requestor', 'bf_semaphores', 'requestor');
		$rep->where(array('!=', 'bf_semaphores', 'bf_semaphore', ''));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'semaphore', array(array('LIKE', 'bf_semaphores', 'bf_semaphore')));
			$userDefinedFilter->setKey( 'requestor', array(array('LIKE', 'bf_semaphores', 'bf_requestor')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_semaphores', 'bf_semaphore'),
											'OR',
											array('LIKE', 'bf_semaphores', 'bf_requestor'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'semaphore')
		);

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the user object,
	 * but it does call manage() (which makes changes) and then
	 * registers the user object with the jTE object for use in
	 * templates.
	 */
	protected function edit() {
		$this->manage();
		$this->jte->register_object('editable', $this->semaphores, null, false);
	}

	protected function manage() {
		global $GLOBAL_OBJECT_KEY, $controller;
		if (array_key_exists($GLOBAL_OBJECT_KEY, $this->post) && is_array($this->post[$GLOBAL_OBJECT_KEY])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBAL_OBJECT_KEY])) {
				$this->semaphores->delete();
			} else {
				foreach ($this->post[$GLOBAL_OBJECT_KEY] as $field => $value) {
					$this->semaphores->set($field, $value);
				}
				try {
					$this->semaphores->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFSemaphore object (if necessary)
		5. Set thenav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Jobs/templates');
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->register_object('editable', $this->semaphores, null, false);
			$this->nav_content = '';
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		if ($this->action == self::ACTION_EDIT) {
			$this->edit();
		}
		$this->frontPageReport();
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Jobs/javascripts/semaphores.content.js')
				 );
	}


	public function navigationBlock() {
		return '';
	}


	public function load($id = null) {
		if (empty($id)) {
			$qb = BFQueryBuilder::select();
			$qb->addColumn('bf_bid', 'bf_semaphores', 'id');
			$qb->orderBy('ASC', 'bf_semaphores', 'bf_semaphore');
			$qb->limit(1);
			$id = $qb->getOne();
		}
		$this->semaphores->select($this->semaphores->primary_key, $id);
	}
}

?>
