<?php
/* (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the copyright and other intellectual property rights in this software.
 Duplication or use of the Software is not permitted except as expressly provided in a written agreement between your company and BuildForge, Inc. */

$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_module('BFStart');
class Start extends Module implements ViewableXHTML {

	const START_LIST		= 'start.list';
	const START				= 'start';
	const RESTART			= 'restart';

	public $title = 'Project Runs';
	public $content;

	protected $jte;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML);
	protected $request;
	protected $request_type;
	public $action;
	
	
	/** Handle Request
	 * First thing run by central controller
	 * NO CLASS is created when this is run.  This does the primary job of creating the class
	 *  which then defines the rest of the behaviour throughout the object.
	 */	 
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$start = new Start();
		$start->setRequest($req, $type);
		return $start;
	}
	
	/** Valid Request Types
	 * Check to see what request types this module handles and return true if we can do it
	 */	 	
	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}
	
	/** includes
	 * This is where we define what other things need to be included.. like javascript
	 */	 	
	public function getIncludes() {
		return array( 
			'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Jobs/javascripts/'.
				(($this->action[0] == 'start' || $this->action[0] == 'restart')?'start':$this->action[0]).'.content.js'),
			'stylesheets' => array(BF_WEBROOT . 'fullcontrol/modules/Jobs/styles/global.css')
		);
	}
	
	/** Set Request
	 * this passes the get/post request to the object and gets the action out
	 */	 	
	public function setRequest($req, $type = BF_REQUEST_XHTML) {
		global $GLOBAL_SETTINGS;
		$this->request = $req;
		$this->request_type = $type;

		if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $this->request)) {
			$this->action = array($this->request[$GLOBAL_SETTINGS['site']['actionkey']]);
		} else {
			$this->action = array();
		}
	}
	
	/** Render
	 * This is the primary render function..  Here we make the magic happen.
	 * This is called by Central Controller	 
	 */	 	
	public function render() {
		$this->generateContent();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Jobs/templates');
		$template = $this->chooseTemplate();
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}
	
	/** Choose Templates
	 * this function determines what templates we should be using
	 * in this case it will either be an xml template, or the main xhtml.view template
	 * our five main views use the same template as they are all the same	 
	 */	 	  	
	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XML) {		
			return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/xml.'.$this->action[0].'.tpl';
		} else {
			return BF_ROOT . '/public/fullcontrol/modules/Jobs/templates/xhtml.start.tpl';
		}
	}

	/** Generate Navigation
	 * This is where the lower left nav is generated.. and a terrible way to do it.. 
	 */	 		
	public function getNavigation() {
		return false;
	}
	public function navigationBlock() {
		return '';
	}
	/** Generate Content
	 * This is where we rush off to generate the content, and assign it back to the template engine
	 */	 	
	protected function generateContent() {
		global $user;
		$this->jte = new BFJTE();	

		$this->jte->assign('is_styx', $this->controller->configuration->styx()); // Styx Hack

		// Quickstart the project given		
		if (isset($this->request['quickstart'])) {
			$startObj = new BFStart();
			$startObj->quickStart($this->request['quickstart']);
			// Return out of this because we dont need any content
			return;
		}
		
		// Are we starting a build?
		if (in_array(self::START, $this->action) && isset($this->request['parameter'])) {
			$startObj = new BFStart();
			$newBuildID = $startObj->manualStart($this->request);
			if ($newBuildID !== false) {
				$this->controller->addHeader('Location: ' . BF_WEBROOT . 'fullcontrol/index.php?mod=jobs&action=edit&bf_id='.$newBuildID);
			}
		}

		// Are we restarting a build?
		if (in_array(self::RESTART, $this->action) && isset($this->request['parameter'])) {
			$startObj = new BFStart();
			if ($startObj->restart($this->request)) {
				$this->controller->addHeader('Location: ' . BF_WEBROOT . 'fullcontrol/index.php?mod=jobs&action=edit&bf_id='.$this->request['parameter']['bid']);
			}
		}
		
		// Generate and display the information for a single starting project
		if ((in_array(self::START, $this->action) || in_array(self::RESTART, $this->action)) && isset($this->request['bf_id'])) {
		
			if (in_array(self::RESTART, $this->action)) {
				// Define the project we are trying to restart
				$this->jte->register_object('start', $this->getRestartProject($this->request['bf_id']), null, false);
			}
			
			if (in_array(self::START, $this->action)) {
				// Define the project we are trying to start
				$this->jte->register_object('start', $this->getProject($this->request['bf_id']), null, false);
			}
			
			// Define the tags list
			$this->jte->register_object('tagslist', $this->getTaglist($this->request['bf_id']), null, false);
			
			// Define Built in tags to jTE.. just.. cause this is easier.
			// Get tag_d_format - Set S:BF_D:...
			$e = BFQueryBuilder::select();
			$e->addColumn('bf_value', 'bf_sysconfig', 'tag_d_format');
			$e->where(array('=', 'bf_sysconfig', 'bf_param', 'tag_d_format'));
			$this->jte->assign('BF_D', date($e->getOne()) );

			// Get tag_t_format - Set S:BF_T:...
			$e = BFQueryBuilder::select();
			$e->addColumn('bf_value', 'bf_sysconfig', 'tag_t_format');
			$e->where(array('=', 'bf_sysconfig', 'bf_param', 'tag_t_format'));
			$this->jte->assign('BF_T', date(str_replace('h', 'H', str_replace('m', 'i', $e->getOne()))) );

			// Set Julian Date
			$this->jte->assign('BF_J', date('z') );
			// Set Week Day
			$this->jte->assign('BF_W', date('w') );
			
		}
		
		// Display the start list of projects/libraries
		if (in_array(self::START_LIST, $this->action)) {
		
			// Set builds to template engine		
			$this->jte->register_object('listing', $this->getProjects(), null, false);
			$this->jte->assign('clobbers', $this->getClobbers());

			$this->jte->register_object('module', $this, null, false);
			$this->jte->register_function('bf_can_quickstart', 'bf_can_quickstart');
		}
		

	}

	/** Get Projects List
	 * This function generates the start list in jobs
	 */	 	
	protected function getProjects() {
		global $GLOBAL_SETTINGS, $user;
		$rep = new BFReport();
		$rep->addColumn('bf_description',	'bf_projects',	'project');
		$rep->addColumn('bf_tag',			'bf_projects',	'tag');
		$rep->addColumn('bf_class',			'bf_projects',	'class');
		$rep->addColumn('bf_param',			'bf_env',		'env');
		$rep->addColumn('bf_platform',		'bf_projects',	'selector');
		$rep->addColumn('bf_snotify',		'bf_projects',	's');
		$rep->addColumn('bf_pnotify',		'bf_projects',	'p');
		$rep->addColumn('bf_fnotify',		'bf_projects',	'f');
		$rep->addColumn('bf_name',			'bf_levels',	'access');
		$rep->addColumn('bf_pid',			'bf_projects',	'pid');
		$rep->addColumn('bf_eid',			'bf_projects',	'envid');
		
		$rep->join(array('INNER', 'bf_levels', array(array('=', 'bf_projects', 'bf_level', 'bf_levels', 'bf_level')) ));
		$rep->join(array('LEFT', 'bf_env', array(
				array('=', 'bf_env', 'bf_bid', 'bf_projects', 'bf_eid'),
				array('=', 'bf_env', 'bf_eid', 0)
				) ));

		$uid = intval($user->get('bf_userid'));
		
		if ($uid !== 1) {
			$rep->where(array(
				array('IN', 'bf_projects', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}


		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey('access', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey('f', array(array('LIKE', 'bf_projects', 'bf_fnotify')));
			$userDefinedFilter->setKey('p', array(array('LIKE', 'bf_projects', 'bf_pnotify')));
			$userDefinedFilter->setKey('s', array(array('LIKE', 'bf_projects', 'bf_snotify')));
			$userDefinedFilter->setKey('platform', array(array('LIKE', 'bf_projects', 'bf_platform')));
			$userDefinedFilter->setKey('env', array(array('LIKE', 'bf_env', 'bf_param')));
			$userDefinedFilter->setKey('class', array(array('LIKE', 'bf_projects', 'bf_class')));
			$userDefinedFilter->setKey('tag', array(array('LIKE', 'bf_projects', 'bf_tag')));
			$userDefinedFilter->setKey('project',
										array(
											'(',
											array('LIKE', 'bf_projects', 'bf_description'),
											')'
										));
			$userDefinedFilter->setKey( '~',	// ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_projects', 'bf_description'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_tag'),
											'OR',
											array('LIKE', 'bf_levels', 'bf_name'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_platform'),
											'OR',
											array('LIKE', 'bf_env', 'bf_param'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_class'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_fnotify'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_pnotify'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_snotify'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'project')
			);

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : 
				$GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : 
				$GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();
	}
	
	/** Return an array of projects being clobbered
	 */
	protected function getClobbers() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid', 'bf_builds', 'bf_pid');
		$qb->where(array(
			array('=', 'bf_builds', 'bf_process', 'N')
		));
		$res = $qb->getResult();
		$results = array();
		while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$results[$row['bf_pid']] = $row['bf_pid'];
		}
		return $results;
	}
	
	protected function getRestartProject($BID) {
		global $user;
		
		$rep = new BFReport();
		$rep->addColumn('bf_bid',			'bf_builds',	'bid');
		$rep->addColumn('bf_pid',			'bf_builds',	'id');
		$rep->addColumn('bf_env',			'bf_builds',	'env');
		$rep->addColumn('bf_tag',			'bf_builds',	'tag');
		$rep->addColumn('bf_class',			'bf_builds',	'class');
		$rep->addColumn('bf_resource',		'bf_builds',	'selector');
		$rep->addColumn('bf_description',   'bf_projects',  'project');
		$rep->addColumn('bf_geo_id',		'bf_projects',  'geo_id');

		$rep->join( array('LEFT', 'bf_projects', array( array('=', 'bf_builds', 'bf_pid', 'bf_projects', 'bf_pid') ) ) );

		$rep->where(
				array(
					array('=', 'bf_builds', 'bf_bid', $BID)
				)
		);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {		
			$res = $rep->getReportResults();			
			return $res;
		}
		return new BFReportResults();		
		
	}	
	
	/** Get Project
	 * This function generates all the information required 
	 *  for a single project to be started in jobs
	 * Required: ProjectID of job to be started	 
	 */	 	 	
	protected function getProject($PID) {
		global $user;
		
		$rep = new BFReport();
		$rep->addColumn('bf_pid',			'bf_projects',  'id');
		$rep->addColumn('bf_description',   'bf_projects',  'project');
		$rep->addColumn('bf_tag',			'bf_projects',  'tag');
		$rep->addColumn('bf_class',			'bf_projects',  'class');
		$rep->addColumn('bf_param',			'bf_env',		'env');
		$rep->addColumn('bf_platform',		'bf_projects',  'selector');
		$rep->addColumn('bf_eid',			'bf_projects',  'eid');
		$rep->addColumn('bf_level',			'bf_projects',	'level');
		$rep->addColumn('bf_uid',			'bf_steps',		'hassteps');
		$rep->addColumn('bf_active',		'bf_scopes',	'activescope');
		$rep->addColumn('bf_geo_id',		'bf_projects',	'geo_id');
		$rep->addColumn('bf_location',		'bf_geo',	'geo_name');
		$rep->join( array('INNER', 'bf_levels', array( array('=', 'bf_projects', 'bf_level', 'bf_levels', 'bf_level') ) ) );
		$rep->join( array('LEFT', 'bf_steps', array( 
				array('=', 'bf_steps', 'bf_pid', 'bf_projects', 'bf_pid'),
				array('=', 'bf_steps', 'bf_sid', 1)
				) ) );
		$rep->join( array('LEFT', 'bf_env', array(
				array('=', 'bf_env', 'bf_bid', 'bf_projects', 'bf_eid'),
				array('=', 'bf_env', 'bf_eid', 0)
				) ) );
		$rep->join( array('LEFT', 'bf_geo',
				array('=', 'bf_geo', 'bf_geo_id', 'bf_projects', 'bf_geo_id'),
				) );
		$rep->join( array('LEFT', 'bf_scopes', array( array('=', 'bf_projects', 'bf_pid', 'bf_scopes', 'bf_pid') ) ) );

		$uid = (int)$user->get('bf_userid');
		if ($uid === 1) {
			$rep->where(
						array(
							  array(
									array('=', 'bf_projects', 'bf_pid', $PID)
									)
							  )
						);
		} else {
			$rep->where(
						array(
								array('=', 'bf_projects', 'bf_pid', $PID),
								array('IN', 'bf_projects', 'bf_level', array_keys(bf_get_levels($uid)))
						));
		}

		// Primary Countby field.
		$rep->countBy('bf_pid', 'bf_projects');

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {	
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();		
	}

	/** getClasses
	 * Returns a list of all classes -- pays attention to level
	 */
	protected function getClasses() {
		global $user;
		$uid = (int)$user->get('bf_userid');

		$rep = new BFReport();
		$rep->addColumn('bf_name',   'bf_classes', 'name');
		if ($uid !== 1) {
			$rep->where(array(array('IN', 'bf_classes', 'bf_level', array_keys(bf_get_levels($uid)))));
		}

		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();		
	}

	/** getSelectors
	 * Returns a list of all selectors
	 */
	protected function getSelectors() {
		global $user;
		$uid = (int)$user->get('bf_userid');
		
		$rep = new BFReport();
		$rep->addColumn('bf_selector_id', 'bf_selector', 'name');
		$rep->where(array(array('=', 'bf_selector', 'bf_prop', '')));
		if ($uid !== 1) {
			$rep->where(array(array('IN', 'bf_selector', 'bf_level', array_keys(bf_get_levels($uid)))));
		}
		
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();		
	}	 	

	/** getTaglist
	 * Returns a list of tags for this particular PID
	 */
	protected function getTaglist($PID) {
		$rep = new BFReport();
		$rep->addColumn('bf_tid',		'bf_tags',  'id');
		$rep->addColumn('bf_name',		'bf_tags',  'name');
		$rep->addColumn('bf_value',		'bf_tags',  'value');
		$rep->addColumn('bf_parent',	'bf_tags',  'padding');
		$rep->addColumn('bf_autoinc',	'bf_tags',  'autoincrement');
		$rep->where(array('=', 'bf_tags', 'bf_pid', $PID));

		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			return $res;
		}
		return new BFReportResults();		
	}

}


?>
