<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * AdaptorLinks
 *
 * @package Console
 * @subpackage AdaptorLinks
 */

$GLOBALS['controller']->require_module('BFScope');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_module('BFInterface');

$GLOBALS['GLOBAL_OBJECT_KEY']	   = 'object';

class AdaptorLinks extends Projects {
	// A special view just to snag project steps for a given project
	const ACTION_VIEW_PROJECTS = 'adaptorlinks.projects';
	const ACTION_VIEW_ADAPTORLINKS = 'adaptorlinks.view';
	const ACTION_EDIT_ADAPTORLINKS = 'adaptorlinks.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;
	public $content;
	public $nav_content;
	protected $adaptorlink;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			// Need this no matter what...
			$this->adaptorlink = new BFScope();
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$adaptorlinks = new AdaptorLinks();
		$temp = new BFScope();
		$adaptorlinks->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$adaptorlinks->load($req[$temp->primary_key]);
		}
		return $adaptorlinks;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport($regName = 'listing') {
		global $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_sid', 'bf_scopes', 'id');
		$rep->addColumn('bf_link', 'bf_scopes', 'adaptor');
		$rep->addColumn('bf_description', 'bf_projects', 'project');
		$rep->addColumn('bf_pid', 'bf_scopes', 'pid');
		$rep->join(
			array('LEFT', 'bf_projects',
					array(
						array('=', 'bf_projects', 'bf_pid', 'bf_scopes', 'bf_pid'),
					)
				)
			);

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'adaptor', array(array('LIKE', 'bf_scopes', 'bf_link')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_scopes', 'bf_link'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$possible = array(
						  'adaptor' => array('bf_scopes', 'bf_link'),
						  'project' => array('bf_projects', 'bf_description')
						  );
		if (array_key_exists('orderby', $this->request) && array_key_exists($this->request['orderby'], $possible)) {
			$rep->orderBy($sort, $possible[$this->request['orderby']][0], $possible[$this->request['orderby']][1]);
		} else {
			$rep->orderBy('ASC', 'bf_scopes', 'bf_link');
		}

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object($regName, $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	protected function editAdaptorLink() {
		$this->manageAdaptorLink();
		$this->jte->register_object('editable', $this->adaptorlink, null, false);
		$this->jte->register_function('bf_get_interfaces', 'bf_get_interfaces', null, false);
		$this->jte->register_function('bf_get_interface_template_name', 'bf_get_interface_template_name', null, false);
		$this->jte->register_function('bf_get_interfacetemplates', 'bf_get_interfacetemplates', null, false);
		$this->jte->register_function('bf_get_entry_points', 'bf_get_entry_points', null, false);
	}

	/**
	 * This should get altered to accept different forms of requests (ie. other than POST
	 * but it'll do for now. This method runs through the request and sets any values it
	 * finds from the request. The environment object weeds out the garbage and validates/escapes.
	 */
	protected function manageAdaptorLink() {
		global $controller;
		if (array_key_exists($GLOBALS['GLOBAL_OBJECT_KEY'], $this->post) && is_array($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
				$this->adaptorlink->delete();
			} else {
				foreach ($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']] as $field => $value) {
					$this->adaptorlink->set($field, $value);
				}
				try {
					$this->adaptorlink->save();
					// See if it should attempt to populate the env
					if (array_key_exists('populate_env', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])
							&& $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']]['populate_env'] == '1') {
						$this->adaptorlink->populateEnv();
					}
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFScope object (if necessary)
		5. Set the nav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW_ADAPTORLINKS;

		$this->handleAction();
		$this->jte->register_function('bf_get_steps', 'bf_get_steps', null, false);
		$this->jte->register_function('bf_get_entry_points', 'bf_get_entry_points', null, false);

		// Projects view
		if ($this->action == self::ACTION_VIEW_PROJECTS) {
			$this->jte->assign('unused_projects', $this->getUnusedProjects());
		}
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		$this->editAdaptorLink();
		$this->frontPageReport();
	}

	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/adaptorlinks.content.js')
					 );
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$link = $this->adaptorlink;
		}  else {
			$link = new BFScope();
			$link->select($link->primary_key, $id);
		}

		$id = $link->get($link->primary_key);
		if (!$id)  {
			return '';
		}

		$nav = bf_get_nav_projects_and_steps(array('=', 'bf_projects', 'bf_pid', $link->get('bf_pid')), null);
		$nav['adaptors'] = bf_get_nav_objects('adaptors', array('=', 'bf_interfaces', 'bf_link', 
				$link->get('bf_link')));
		$nav['environments'] = bf_get_nav_objects('environments', array(
				array('=', 'bf_env', 'bf_bid', $link->get('bf_eid')), array('=', 'bf_env', 'bf_eid', 0)));

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}

	public function getUnusedProjects() {
		$qb = new BFReport();
		$qb->addColumn('bf_description',	'bf_projects',   'name');
		$qb->addColumn('bf_pid',			'bf_projects',   'pid');
		$qb->addColumn('bf_link',			'bf_scopes',   'linked');
		$qb->join(
			array('LEFT', 'bf_scopes',
					array(
						array('=', 'bf_projects', 'bf_pid', 'bf_scopes', 'bf_pid'),
					)
				)
			);
	
		$uid = (int)$GLOBALS['user']->get('bf_userid');

		if ($uid !== 1) {
			$qb->where(array(
					array('IN', 'bf_projects', 'bf_level', array_keys(bf_get_levels($uid))),
					array('IS NULL', 'bf_scopes', 'bf_link')
					));
		}
		$qb->orderBy('ASC', 'bf_projects', 'bf_description');

		$res = $qb->getReportResults();
		return $res;
	}

	public function load($id) {
		if (!empty($id)) {
			// Load the Adaptor itself
			$fields = array($this->adaptorlink->primary_key);
			$values = array($id);
			$this->adaptorlink->select($fields, $values);
		}
	}

	public function __construct() {
		parent::__construct();
		$this->jte = new BFJTE();
	}
}

?>
