<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Adaptors
 *
 * @package Console
 * @subpackage Adaptors
 */

$GLOBALS['controller']->require_module('BFInterface');
$GLOBALS['controller']->require_module('BFReport');

$GLOBALS['GLOBAL_OBJECT_KEY']	   = 'object';

class Adaptors extends Projects {
	const ACTION_VIEW_ADAPTORS = 'adaptors.view';
	const ACTION_EDIT_ADAPTORS = 'adaptors.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;
	public $content;
	public $nav_content;
	protected $adaptor;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			// Need this no matter what...
			$this->adaptor = new BFInterface();
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$adaptors = new Adaptors();
		$temp = new BFInterface();
		$adaptors->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$adaptors->load($req[$temp->primary_key]);
		}
		return $adaptors;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $GLOBAL_SETTINGS, $user;
		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_link', 'bf_interfaces', 'id');
		$rep->addColumn('bf_link', 'bf_interfaces', 'name');
		$rep->addColumn('bf_mode', 'bf_interfaces', 'type');
		$rep->where(array('=', 'bf_interfaces', 'bf_link', 'bf_interfaces', 'bf_link'));
		$rep->countBy('bf_link', 'bf_interfaces');
		
		$uid = intval($user->get('bf_userid'));
		if ($uid === 1) {
			$rep->where(array('=', 'bf_interfaces', 'bf_level', 'bf_levels', 'bf_level'));
		} else {
			$rep->where(array(
							  array('=', 'bf_interfaces', 'bf_level', 'bf_levels', 'bf_level'),
							  array('IN', 'bf_interfaces', 'bf_level', array_keys(bf_get_levels($uid)))
							  ));
		}

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_interfaces', 'bf_prop')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_interfaces', 'bf_link'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$rep->orderBy($sort, 'bf_interfaces', 'bf_link');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	protected function editAdaptor() {
		$this->manageAdaptor();
		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_type', 'bf_interfacetemplates', 'type', 'DISTINCT');
		$types = $qb->getCol();
		
		$this->jte->assign('types', $types);
		$this->jte->register_object('editable', $this->adaptor, null, false);
	}

	/**
	 * This should get altered to accept different forms of requests (ie. other than POST
	 * but it'll do for now. This method runs through the request and sets any values it
	 * finds from the request. The environment object weeds out the garbage and validates/escapes.
	 */
	protected function manageAdaptor() {
		global $controller;
		if (array_key_exists($GLOBALS['GLOBAL_OBJECT_KEY'], $this->post) && is_array($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
				$this->adaptor->delete();
			}  elseif (isset($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']]['copy'])) {
				$this->adaptor->copy();
			} else {
				foreach ($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']] as $field => $value) {
					if ($field == 'bf_mode') {
						switch ($value) {
							case 'S':
								$this->adaptor->set('bf_template', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']]['bf_template_S']);
								break;
							case 'T':
								$this->adaptor->set('bf_template', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']]['bf_template_T']);
								break;
							case 'D':
								$this->adaptor->set('bf_template', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']]['bf_template_D']);
								break;
						}
					}
					$this->adaptor->set($field, $value);
				}
				try {
					$this->adaptor->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFInterface object (if necessary)
		5. Set the nav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW_ADAPTORS;

		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Projects/templates');
		$this->jte->register_function('bf_get_interfacetemplates', 'bf_get_interfacetemplates', null, false);
		$this->jte->register_function('bf_get_interfacetemplate', 'bf_get_interfacetemplate', null, false);
		$this->handleAction();

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		$this->editAdaptor();
		$this->frontPageReport();
	}

	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/adaptors.content.js')
					 );
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$adaptor = $this->adaptor;
		}  else {
			$adaptor = new BFInterface();
			$adaptor->select($adaptor->primary_key, $id);
		}
		
		$id = $adaptor->get($adaptor->primary_key);
		if (!$id)  {
			return '';
		}

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid', 'bf_scopes');
		$qb->addColumn('bf_eid', 'bf_scopes');
		$qb->where(array('=', 'bf_scopes', 'bf_link', $id));
		$res = $qb->getResult();
		if (!DB::isError($res))  {
			$pids = array();
			$eids = array();
			while($row = $res->fetchRow(DB_FETCHMODE_ASSOC))  {
				$pids[$row['bf_pid']] = $row['bf_pid'];
				$eids[$row['bf_eid']] = $row['bf_eid'];
			}
			$nav = $pids ? 
					bf_get_nav_projects_and_steps(array('IN', 'bf_projects', 'bf_pid', array_keys($pids)), null) :
					array();
			$nav['environments'] = $eids ? bf_get_nav_objects('environments', array(array('IN', 'bf_env', 'bf_bid', 
					array_keys($eids)), array('=', 'bf_env', 'bf_eid', 0))) : '';
		}  else {
			$nav = array();
		}
		
		$nav['adaptor links'] = bf_get_nav_objects('adaptor links', array('=', 'bf_scopes', 'bf_link', $id));
		$nav['access groups'] = bf_get_nav_objects('access groups', array('=', 'bf_levels', 'bf_level', 
				$adaptor->get('bf_level')));
		
		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}


	public function load($id) {
		// Load the Adaptor itself
		$fields = array($this->adaptor->primary_key);
		$values = array($id);
		$this->adaptor->select($fields, $values);
	}

	public function __construct() {
		parent::__construct();
		$this->jte = new BFJTE();
	}
}

?>
