<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Admin Classes
 *
 * @package Console
 * @subpackage Classes
 */

$GLOBALS['controller']->require_module('BFClass');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';

class Classes extends Projects {
	const ACTION_VIEW = 'classes.view';
	const ACTION_EDIT = 'classes.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $class;

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			$this->class = new BFClass();
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$classes = new Classes();
		$temp = new BFClass();
		$classes->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$classes->load($req[$temp->primary_key]);
		}
		return $classes;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS, $user;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Setup the columns, most of which will get displayed
		$rep->addColumn('bf_name', 		'bf_classes',	'buildclass');
		$rep->addColumn('bf_name', 		'bf_levels',	'access');
		$rep->addColumn('bf_precious', 	'bf_classes',	'delete');
		$rep->addColumn('bf_purge_days','bf_classes',	'days');
		$rep->addColumn('bf_level', 	'bf_classes',	'level');

		$uid = intval($user->get('bf_userid'));
		if ($uid === 1) {
			$rep->where(array('=', 'bf_classes', 'bf_level', 'bf_levels', 'bf_level'));
		} else {
			$rep->where(array(
				array('=', 'bf_classes', 'bf_level', 'bf_levels', 'bf_level'),
				array('IN', 'bf_classes', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'class', array(array('LIKE', 'bf_classes', 'bf_name')));
			$userDefinedFilter->setKey( 'delete', array(array('LIKE', 'bf_classes', 'bf_precious')));
			$userDefinedFilter->setKey( 'days', array(array('LIKE', 'bf_classes', 'bf_purge_days')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_classes', 'bf_name'),
											'OR',
											array('LIKE', 'bf_classes', 'bf_precious'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'buildclass')
		);

		// LEFT JOIN throws the COUNT out of whack
	   // $rep->countBy('bf_users', 'bf_userid');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the user object,
	 * but it does call manage() (which makes changes) and then
	 * registers the user object with the jTE object for use in
	 * templates.
	 */
	protected function edit() {
		$this->manage();
		$this->jte->register_object('editable', $this->class, null, false);
	}

	protected function manage() {
		global $GLOBAL_OBJECT_KEY;
		if (array_key_exists($GLOBAL_OBJECT_KEY, $this->post) && is_array($this->post[$GLOBAL_OBJECT_KEY])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBAL_OBJECT_KEY])) {
				$this->class->delete();
			}  elseif (isset($this->post[$GLOBAL_OBJECT_KEY]['copy'])) {
				$this->class->copy();
			} else {
				foreach ($this->post[$GLOBAL_OBJECT_KEY] as $field => $value) {
					// Annoying checkboxes, give me a useful value!
					if ($field == 'bf_priority' || $field == 'bf_pwexpired') {
						if ($value == 'true') {
							$value = 'Y';
						} else {
							$value = 'N';
						}
					// Passwords need verification and "encryption"
					} else if ($field == 'bf_password') {
						// If not matching throw exception
						if (!array_key_exists('bf_password_verify', $this->post[$GLOBAL_OBJECT_KEY])) {
							throw new BFException(translate('2087'));
						} else {
							$value = bf_encrypt($value);
						}
					}
					$this->class->set($field, $value);
				}
				$this->class->save();
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFUser object (if necessary)
		5. Set thenav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Projects/templates');
		$this->jte->register_function('bf_get_project_names', 'bf_get_project_names', null, false);
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->register_object('editable', $this->class, null, false);
			$this->nav_content = '';
		}

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/' . $this->request_type . '.' . $this->action .
		'.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		switch ($this->action) {
			case self::ACTION_EDIT:
				$this->edit();
				$this->frontPageReport();
				break;
			case self::ACTION_VIEW:
			default:
				$this->frontPageReport();
		}
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/classes.content.js')
				 );
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$class = $this->class;
		}  else {
			$class = new BFClass();
			$class->select($class->primary_key, $id);
		}

		$id = $class->get($class->primary_key);
		if (!$id)  {
			return '';
		}
		$arr = array();
		foreach (array($class->get('bf_pid'), $class->get('bf_npid'), $class->get('bf_xpid')) as $v)  {
			if ($v)  {
				$arr[] = $v;
			}
		}
		if ($arr)  {
			$arr = array(array('=', 'bf_projects', 'bf_class', $id),
						'OR',
						array('IN', 'bf_projects', 'bf_pid', $arr));
		}  else {
			$arr = array('=', 'bf_projects', 'bf_class', $id);
		}

		$nav = bf_get_nav_projects_and_steps($arr, null);
		$nav['schedules'] = bf_get_nav_objects('schedules', array('=', 'bf_cron', 'bf_class', $id));
		$nav['access groups'] = bf_get_nav_objects('access groups', array('=', 'bf_levels', 'bf_level', 
				$class->get('bf_level')));

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}


	public function load($id) {
		$this->class->select($this->class->primary_key, $id);
	}
}

?>
