<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Filters
 *
 * @package Console
 * @subpackage LogFilters
 */

$GLOBALS['controller']->require_module('BFFilter');
$GLOBALS['controller']->require_module('BFReport');

$GLOBALS['GLOBAL_OBJECT_KEY']	   = 'parent';
$GLOBALS['GLOBAL_SUBOBJECT_KEY']	= 'variable';

$GLOBALS['sigils'] = array('=','!=','<>','<','>','<=','>=');

class Filters extends Projects {
	const ACTION_VIEW_FILTERS = 'filters.view';
	const ACTION_EDIT_FILTERS = 'filters.edit';
	const ACTION_VIEW_FILTER = 'filters.view.pattern';
	const ACTION_EDIT_FILTER = 'filters.edit.pattern';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;
	public $content;
	public $nav_content;
	protected $filter;
	protected $filter_pattern;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			// Need this no matter what...
			$this->filter = new BFFilter();
			$this->filter_pattern = new BFFilter();
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$filters = new Filters();
		$temp = new BFFilter();
		$filters->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			// Key an eye out for the prop as well
			$prop = (array_key_exists('bf_fid', $req)) ? $req['bf_fid'] : null;
			$filters->load($req[$temp->primary_key], $prop);
		}
		return $filters;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS, $user;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_did', 'bf_filters', 'id');
		$rep->addColumn('bf_pattern', 'bf_filters', 'name');
		// Add constraints
		$rep->where(array('=', 'bf_filters', 'bf_fid', 0));

		$uid = intval($user->get('bf_userid'));
		if ($uid === 1) {
			$rep->where(array('=', 'bf_filters', 'bf_level', 'bf_levels', 'bf_level'));
		} 
		else {
			$rep->where(array(
				array('=', 'bf_filters', 'bf_level', 'bf_levels', 'bf_level'),
				array('IN', 'bf_filters', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}
		
		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_filters', 'bf_pattern')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_filters', 'bf_pattern'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}
		
		/* Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$possible = array(
						  'collector' => array('bf_servers', 'bf_coll_id'),
						  'host' => array('bf_servers', 'bf_host'),
						  'path' => array('bf_servers', 'bf_path'),
						  'name' => array('bf_servers', 'bf_name')
						  );
		if (array_key_exists('orderby', $this->request) && array_key_exists($this->request['orderby'], $possible)) {
			$rep->orderBy($sort, $possible[$this->request['orderby']][0], $possible[$this->request['orderby']][1]);
		} else {
			$rep->orderBy('ASC', 'bf_servers', 'bf_name');
		}*/

		// Add a order by and sort direction
		if (array_key_exists('sort', $this->request)) {
			switch (strtolower($this->request['orderby'])) {
				case 'name':
				default:
					$rep->orderBy(
								  ($this->request['sort'] == 'desc' ? 'DESC' : 'ASC'),
								  'bf_filters', 'bf_pattern'
								  );
			}
		} else {
			$rep->orderBy(
						  'ASC',
						  'bf_filters', 'bf_pattern'
						  );
		}

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : 
				$GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : 
				$GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	protected function patternPageReport($id) {
		global $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_did', 'bf_filters', 'did');
		$rep->addColumn('bf_fid', 'bf_filters', 'id');
		$rep->addColumn('bf_pattern', 'bf_filters', 'pattern');
		$rep->addColumn('bf_action', 'bf_filters', 'action');
		$rep->addColumn('bf_name', 'bf_levels', 'notify');
		// Add constraints
		$rep->where(array(
						array('!=', 'bf_filters', 'bf_fid', 0),
						array('=', 'bf_filters', 'bf_did', $id)
						  ));
		$rep->join(array('LEFT', 'bf_levels', array('=', 'bf_levels', 'bf_level', 'bf_filters', 'bf_level')));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'pattern', array(array('LIKE', 'bf_filters', 'bf_pattern')));
			$userDefinedFilter->setKey( 'action', array(array('LIKE', 'bf_filters', 'bf_action')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_filters', 'bf_pattern'),
											  'OR',
											  array('LIKE', 'bf_filters', 'bf_action'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$rep->orderBy($sort, 'bf_filters', 'bf_fid');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : 
				$GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : 
				$GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the filter object,
	 * but it does call manageFilter() (which makes changes) and then
	 * registers the filter object with the jTE object for use in
	 * templates.
	 */
	protected function editFilter() {
		if (!($this->filter_pattern instanceof BFFilter)) {
			$this->filter_pattern = new BFFilter();
		}
		$this->manageFilter();
		$this->jte->register_object('parent', $this->filter, null, false);
		$this->jte->register_object('editable', $this->filter_pattern, null, false);
	}

	/**
	 * This should get altered to accept different forms of requests (ie. other than POST
	 * but it'll do for now. This method runs through the request and sets any values it
	 * finds from the request. The filter object weeds out the garbage and validates/escapes.
	 */
	protected function manageFilter() {
		global $controller;
		if (array_key_exists($GLOBALS['GLOBAL_OBJECT_KEY'], $this->post) && 
				is_array($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
				$this->filter->delete();
			}  elseif (isset($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']]['copy'])) {
				$this->filter->copy();
			} else {
				// Making a new one?
				$inserting = ($this->filter->get('bf_did') === false);
				foreach ($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']] as $field => $value) {
					$this->filter->set($field, $value);
				}
				try {
					$this->filter->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
			}
		}
		// Yes, you could theoretically edit both the Environment and one of its variables concurrently
		if (array_key_exists($GLOBALS['GLOBAL_SUBOBJECT_KEY'], $this->post) && 
				is_array($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']])) {
			// Delete?
			if (isset($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['move'])) {
				$this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_fid'] = $this->filter_pattern->get('bf_fid') + 
						$this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['move'];
				if ($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_fid'] < 1)  {
					$this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_fid'] = 1;
				}
			}
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']])) {
				$this->filter_pattern->delete();
			}  elseif (isset($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['copy'])) {
				$this->filter_pattern->copy($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['copy']);
			} else {
				foreach ($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']] as $field => $value) {
					$this->filter_pattern->set($field, $value);
				}
				try {
					$this->filter_pattern->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFFilter object (if necessary)
		5. Set the nav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller, $sigils;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW_FILTERS;

		$this->jte->assign('sigils', $sigils);
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Projects/templates');
		$this->handleAction();

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XHTML) {
			if ($this->action == self::ACTION_VIEW_FILTERS || $this->action == self::ACTION_EDIT_FILTERS) {
				return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/xhtml.filters.view.tpl';
			} else {
				return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/xhtml.filters.view.pattern.tpl';
			}
		} else {
			$target_path = BF_ROOT . '/public/fullcontrol/modules/Projects/templates/' . $this->request_type . 
					'.' . $this->action . '.tpl';
			if (is_readable($target_path)) {
				return $target_path;
			} else {
				if ($this->action == self::ACTION_VIEW_FILTERS || $this->action == self::ACTION_EDIT_FILTERS) {
					return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/xml.filters.view.tpl';
				} else {
					return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/xml.filters.view.pattern.tpl';
				}
			}
		}
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		// Really, this could get called from anywhere
		$this->editFilter();
		switch ($this->action) {
			case self::ACTION_EDIT_FILTER:
			case self::ACTION_VIEW_FILTER:
				$this->patternPageReport($this->filter->get('bf_did'));
				break;
			case self::ACTION_VIEW_FILTERS:
			case self::ACTION_EDIT_FILTERS:
			default:
				$this->frontPageReport();
		}
	}

	public function getIncludes() {
		return array(
					 'javascripts' => array(
							BF_WEBROOT . 'javascripts/mouseOvers.js',
							BF_WEBROOT . 'fullcontrol/javascripts/actions_menu.js',
							BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/' . $this->action . 
									'.content.js'
					)
				);
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$filter = $this->filter;
		}  else {
			$filter = new BFFilter();
			$filter->select($filter->primary_key, $id);
		}
		
		$id = $filter->get($filter->primary_key);
		if (!$id)  {
			return '';
		}

		$nav = bf_get_nav_projects_and_steps(array('=', 'bf_steps', 'bf_did', $id), 
				array('=', 'bf_steps', 'bf_did', $id));
		$nav['access groups'] = bf_get_nav_objects('access groups', array('=', 'bf_levels', 'bf_level', 
				$filter->get('bf_level')));

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}


	public function load($id, $prop = null) {
		if (!empty($id)) {
			// Load the Filter itself
			$fields = array($this->filter->primary_key, 'bf_fid');
			$values = array($id, 0);
			$this->filter->select($fields, $values);
		}

		// If necessary, also load the particular pattern
		if (!is_null($prop) && $prop != '') {
			$fields = array($this->filter_pattern->primary_key, 'bf_fid');
			$values = array($id, $prop);
			$this->filter_pattern->select($fields, $values);
		} else {
			// We need a did no matter what
			$this->filter_pattern->set($this->filter->primary_key, $id);
		}
	}


	public function __construct() {
		parent::__construct();
		$this->jte = new BFJTE();
	}
}

?>
