<?php /* (c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the copyright and other intellectual property rights in this software.
    Duplication or use of the Software is not permitted except as expressly provided in a written agreement between your company and BuildForge, Inc. */

/**
 * Projects
 *
 * @package Console
 * @subpackage Projects
 */

$GLOBALS['controller']->require_module('BFProject');
$GLOBALS['controller']->require_module('BFStep');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_module('BFStore');
$GLOBALS['controller']->require_module('BFStart');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

/**
 * Used to make an array in the request to keep the project fields
 * by themselves to reduce confusion when making changes to the project.
 */
$GLOBALS['GLOBAL_PROJECT_KEY']	= 'project';
$GLOBALS['GLOBAL_STEP_KEY']		= 'steps';

class Plans extends Module implements ViewableXHTML {

	// CONSTANTS

	// Actions ~ action=
	const ACTION_VIEW_PROJECTS	= 'projects.view';	// Show Index for projects
	const ACTION_VIEW_STEPS	 	= 'steps.view';	 	// Show Index for steps in a project
		
	const ACTION_EDIT_PROJECT	= 'projects.edit';	// Shows the bottom pane for a project ***
	const ACTION_EDIT_STEP		= 'steps.edit';			// Shows the bottom pane for the steps
	const ACTION_EDIT_NOTES		= 'steps.notes';		// Updates the notes area with new notes -- 
													// special circumstance ***
	
	// Sub-Actions ~ sa= ~ via AJAX-POST ~ used by details pane form and options menu popup
	
	// Project Constants
	const SUBACTION_RUNPROJECT		= 'Run Project';
	const SUBACTION_COPYPROJECT		= 'Copy Project';
	const SUBACTION_DELPROJECT		= 'Delete Project';
	const SUBACTION_CLOBBER			= 'Clobber';
	// Step Constants
	const SUBACTION_NEWABOVE		= 'New Step Above';
	const SUBACTION_NEWBELOW		= 'New Step Below';
	const SUBACTION_SAVESTEP		= 'Save Step';
	const SUBACTION_CLONESTEP		= 'Clone Step';
	const SUBACTION_MOVESTEP		= 'Move Step';
	const SUBACTION_DELETESTEP		= 'Delete Step';
	const SUBACTION_TOGGLESTEP		= 'Toggle Step';
	const SUBACTION_TOGGLECONT		= 'Toggle Continue';
	const SUBACTION_TOGGLEABSOLUTE	= 'Toggle Absolute';
	const SUBACTION_TOGGLETHREAD	= 'Toggle Thread';
	// Tag Constants
	const SUBACTION_EDITTAG			= 'Edit Tag';
	const SUBACTION_LOADTAG			= 'Load Tag';
	const SUBACTION_DELETETAG		= 'Delete Tag';
	// Register Constants
	const SUBACTION_EDITREG			= 'Edit Register';
	const SUBACTION_LOADREG			= 'Load Register';
	const SUBACTION_DELETEREG		= 'Delete Register';

	protected $jte;        				// jTE Template
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;				//
	public $content;					//
	public $action;

	
	protected $project;		 			// Reference to DB Object BFProject
	protected $step;					// Reference to DB Object BFStep

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function setRequest($req, $type, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request		= $req;
			$this->post			= $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action	= $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	protected function handleAction() {
		global $GLOBAL_HEADERS;
		
		// One off for QuickStarting
		if (isset($this->request['sa']) && $this->request['sa'] == self::SUBACTION_RUNPROJECT && isset($this->request['bf_id'])) {
			$startObj = new BFStart();
			$startObj->quickStart($this->request['bf_id']);
			return;
		}

		// Instance of BFProject
		if (!isset($this->project))  {  $this->project = new BFProject();  }  // Instantiate BFProject

		// Instance of BFStep
		if (!isset($this->step))  {  $this->step = new BFStep();  }

		// Catch if there is no project id
		if (!array_key_exists('bf_id', $this->request) && ($this->action == self::ACTION_EDIT_STEP || 
				$this->action == self::ACTION_VIEW_STEPS)) {
			$this->request['bf_id'] = '1';
		}
				
		// Begin: Edit/Save/Delete Notes
		$this->notes = new BFNotes();
		$this->notes->noteType = 'ProjectStep';
		if (isset($this->post['edit_note_id'])) {
			$this->action = 'steps.notes';
			$this->notes->currentNote = $this->post['edit_note_id'];
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid'], $this->post['sid']);	
		} else if (isset($this->post['delete_note_id'])) {
			$this->action = 'steps.notes';
			$this->notes->deleteNote($this->post, $this->post['pid'], $this->post['sid']);
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid'], $this->post['sid']);	
		} else if (isset($this->post['save_note'])) {
			$this->action = 'steps.notes';
			$this->notes->saveNote($this->post, $this->post['pid'], $this->post['sid']);
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid'], $this->post['sid']);	
		} else {
			// load notes note done here but in the steps list below
		}		
		// End: Edit/Save/Delete Notes
		
		
		##  TO DO:  Pair these down so that each action only does the MINIMUM to perform that action
		##   right now there is a lot of unnecessary duplication

		switch ($this->action) {


			// "steps.edit" - Primarily used to view and edit details about a step
			case self::ACTION_EDIT_STEP:
			// "steps.view" - Primarily used to view steps list of a project and perform actions
			case self::ACTION_VIEW_STEPS:
				// Handle all subactions
				list ($projectID, $stepID, $uniqueSID, $noteID) = $this->handleSubaction();					
				// Register all information with the template engine
				$this->registerSTEPS($projectID, $stepID, $uniqueSID, $noteID);
				
				break;

			// "projects.view" - Primarily used to view projects list and perform new project action
			case self::ACTION_VIEW_PROJECTS:
			case self::ACTION_EDIT_PROJECT:

				list ($projectID, $stepID, $uniqueSID, $noteID) = $this->handleSubaction();
				$this->registerPROJECTS($projectID, $stepID, $uniqueSID, $noteID);

				break;


		}
		$this->jte->register_object('module', $this, null, false);
	}




### BEGIN Projects/Library Commonality Code #######################################################

	/** handleRequest
	 * This is the very first thing that is run by the central controller prior to rendering.
	 * NO CLASS is created when this is run.  This does the primary job of creating the class
	 *  which then defines the rest of the behaviour throughout the object.
	 * This class is written to know itself as either Projects or Library and behave accordingly
	 * The first line will need to reflect that for Projects versus Library
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$project = new Plans();
		$temp = new BFProject();
		$tempstep = new BFStep();
		if (array_key_exists('bf_id', $req) && !array_key_exists($temp->primary_key, $req))  {
			$req[$temp->primary_key] = $req['bf_id'];
		}
		if (array_key_exists('action', $req) && ($req['action'] == self::ACTION_VIEW_STEPS || 
				$req['action'] == self::ACTION_EDIT_STEP) && !array_key_exists('bf_sid', $req) && 
				!array_key_exists('step', $req) && !array_key_exists('suid', $req))  {
			$req['bf_sid'] = 1;
		}
		$project->setRequest($req, $type, $post);
		$project->request_type = $type;

		if (array_key_exists($temp->primary_key, $req) && $req[$temp->primary_key]) {
			if (array_key_exists('bf_sid', $req))  {
				$project->load($req[$temp->primary_key], 'bf_sid', $req['bf_sid']);
			}  elseif (array_key_exists('step', $req))  {				
				$project->load($req[$temp->primary_key], 'bf_sid', $req['step']);
			}  elseif (array_key_exists('suid', $req))  {
				$project->load($req[$temp->primary_key], 'bf_uid', $req['suid']);
			}  else {
				$project->load($req[$temp->primary_key]);
			}
		}

		return $project;
	}

### END Projects/Library Commonality Code #########################################################

	/** Render
	 *  This is the magic renderer..
	 */
	public function render() {
		global $controller, $user;

		// if no action is defined, define the default one
		if (!isset($this->action)) {
			$this->action = self::ACTION_VIEW_PROJECTS;
		}

		// Create an instance of smarty
		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Projects/templates');

		$this->jte->assign('is_styx', $this->controller->configuration->styx()); // Styx Hack

		// Start the process
		$this->handleAction();

		// Add the userobj to jTE
		$this->jte->register_object('userobj', $user, null, false);

		// Grab that whacky template
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
		$this->nav_content = $this->navigationBlock();		
	}
	
	/** ChooseTemplate
	 * Does the work of grabbing the templates we need, based on 3 things;
	 * 1.  our class name determines the directory (used by Projects and Library)
	 * 2.  request type (xml or xhtml)
	 * 3.  their action
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/' . $this->request_type . '.' . $this->action . '.tpl';
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 * 
	 * if action = edit then we are editing a projects steps so do that stuff else we are either
	 * viewing the projects list, or editing the project so do that.
	 * 	 
 	 * Important
 	 * File names don't relate to actions or anything else for that matter.
	 * projects.edit / steps.edit is the details object 
	 * projects.view / steps.view is the paginator
	 */   
	public function getIncludes() {		
		if ($this->action == 'steps.view') {
			return array(
						'javascripts' => array(
							BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/steps.view.content.js',
							BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/steps.edit.content.js'
							)
					);
		} else {
			return array(
						'javascripts' => array(
							BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/projects.view.content.js',
							BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/projects.edit.content.js'
							)
					);
		}
	}

	public function getNavigation() {
		return false;
	}

	public function load($id, $sidtype = 'bf_sid', $sid = 0) {

		if ($id) {		
			$this->project = new BFProject();
			$this->project->select($this->project->primary_key, $id);
		}
		
		if ($sid)  {
			$this->step = new BFStep();
			$this->step->select(array($this->project->primary_key, $sidtype), array($id, $sid));
		}
		
	
	}


	public function handleSubaction() {
		global $controller, $user;
		// Does the user have access to this project/step?
		$hasAccess = false;
		$uid = intval($user->get('bf_userid'));
		
		/** Define the various forms of identification
		 */		 
		$projectID	= (isset($this->request['bf_id']))?$this->request['bf_id']:0;
		$uniqueSID	= (isset($this->request['suid']) && !empty($this->request['suid']))?$this->request['suid']:'';
		$stepNumber	= (isset($this->request['step']) && !empty($this->request['step']))?$this->request['step']:'';
		$noteID		= (isset($this->request['note_id']))?$this->request['note_id']:''; //???
		
		// Update means we know the value of UniqueSID before positively setting it below
		$step_update = true;
		// Get uniquesid of pid/sid or get first one of pid if not set
		if (!empty($projectID) && empty($uniqueSID)) {
			// $qb = BFQueryBuilder::select();
			// $qb->addColumn('bf_uid', 'bf_steps');
			// $qb->where(array(
				// array('=', 'bf_steps', 'bf_pid', $projectID),
				// array('=', 'bf_steps', 'bf_sid', ((!empty($stepNumber))?$stepNumber:'1'))
			// ));
			// $uniqueSID = $qb->getOne();
			$step_update = false;			
		}
		
		// SAVE PROJECT - saves the edited project information or creates a new one
		// DELETE and CLOBBER can't exist here.
		// Copy?				
		if (array_key_exists('copy', $this->post)) {
			$this->project->copy();
		// Save, if nothing else
		} elseif (array_key_exists($GLOBALS['GLOBAL_PROJECT_KEY'], $this->request)) {
			$this->project->setMany($this->post[$GLOBALS['GLOBAL_PROJECT_KEY']]);
			$this->project->save();
			
			// Mmmm Hacky.. Send along the project ID of the insert so we can refresh to it
			if (empty($projectID)) {
				$this->jte->assign('jumptoid', $this->project->get('bf_pid'));
			}
		}
	
		/** Check to see if there is a subaction
		 * if there is, check to see if the user has access.
		 * If he does, then do it.
		 */		 		 		
		if (isset($this->request['sa'])) {
			// Check to make sure they have access to the project to edit it.
			if (!empty($projectID)) {
				$projectLevel = $this->project->get('bf_level');
				$hasAccess = ($uid === 1 || in_array($projectLevel, array_keys(bf_get_levels($uid))));
	
				// If the user doesn't have access to this and this is an edit request, redirect to list.
				if (!$hasAccess && $this->action == self::ACTION_EDIT_STEP) {
					$controller->addHeader('Location: ' . BF_WEBROOT . 'fullcontrol/?mod=' . 
							((get_class($this)=='Plans')?'projects':'library'));
				}
			} else {
				$hasAccess = true;
			}
	
			// Check that they have access to the step to edit it.
			if ((!empty($stepNumber) || !empty($uniqueSID)) && $hasAccess) {
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_level', 'bf_steps');
				if (!empty($stepNumber)) {
					$qb->where(array(
						array('=', 'bf_steps', 'bf_pid', $projectID),
						array('=', 'bf_steps', 'bf_sid', $stepNumber)
						));
				} else {
					$qb->where(array(
						array('=', 'bf_steps', 'bf_pid', $projectID),
						array('=', 'bf_steps', 'bf_uid', $uniqueSID)
						));
				}
				$stepLevel = $qb->getOne();
				$validStepLevels = array_keys(bf_get_levels($uid));
				$validStepLevels[] = 0;	// ZERO is a valid step level
	
				$hasAccess = ($uid === 1 || in_array($stepLevel, $validStepLevels))?true:false;
			}

			if ($hasAccess || !$step_update) {	
				// Make sure you know what step we are talking about for updates
				if (!empty($projectID) && $step_update) {				
					$this->step->select(array('bf_pid','bf_uid'),array($projectID, $uniqueSID));
				}
				switch($this->request['sa']) {
				
					// Load the tag data into the form
					case self::SUBACTION_LOADTAG:
						$this->register_singleTag('tagsingle', $this->request['tid']);
						break;
				
					// Load the register data into the form
					case self::SUBACTION_LOADREG:
						$this->register_singleReg('regsingle', $this->request['rid']);
						break;
				
					// STEP SWITCHES				
				
					// TOGGLE STEP - Flips a step between enabled and disabled
					case self::SUBACTION_TOGGLESTEP:
						$this->step->toggleStepField($projectID, $uniqueSID, $this->request['toggle'], 'bf_active');
						break;
						
					// TOGGLE CONTINUE - Flips a step between continue on fail and halt on fail
					case self::SUBACTION_TOGGLECONT:
						$this->step->toggleStepField($projectID, $uniqueSID, $this->request['toggle'], 'bf_onfail');
						break;
											
					// TOGGLE ABSOLUTE - Flips a step between absolute path and relative path
					case self::SUBACTION_TOGGLEABSOLUTE:				
						$this->step->toggleStepField($projectID, $uniqueSID, $this->request['toggle'], 'bf_absolute');
						break;					
			
					// TOGGLE THREADED - Rolls the threading type from Non-threaded -> Threaded -> Joined -> 
					// Non-threaded...
					case self::SUBACTION_TOGGLETHREAD:
						$this->step->toggleStepField($projectID, $uniqueSID, $this->request['toggle'], 'bf_canbg');
						break;					
			
					/** Save Step
					 * Set the step information and then try to save
					 */					 					
					case self::SUBACTION_SAVESTEP:			
						foreach ($this->post[$GLOBALS['GLOBAL_STEP_KEY']] as $field => $value) {
							$this->step->set($field, $value);
						}						
						try {
							$this->step->save();
							$stepNumber = $this->step->get('bf_sid');
							$uniqueSID = $this->step->get('bf_uid');
						} catch (Exception $e) {
							$controller->addError($e->getMessage());
							return false;
						}					
						break;
	
					/** Move Step
					 * Moves the step up or down by X number of places
					 */					 					
					case self::SUBACTION_MOVESTEP:
						$this->step->moveStep($projectID, $uniqueSID, $this->post['direction']);
						break;
						
					/** Clone Step
					 * Clones a step up or down by X number of places
					 */					 					
					case self::SUBACTION_CLONESTEP:
						$this->step->cloneStep($projectID, $uniqueSID, $this->post['direction']);
						break;
						
					/** Delete Step
					 * Deletes a step and its associated notes from the database
					 */					 					
					case self::SUBACTION_DELETESTEP:
						$this->step->deleteStep($projectID, $uniqueSID);
						break;
	
					/** Everything required to save an edited tag.. *gleee* */
					case self::SUBACTION_EDITTAG:					
						$this->project->editTag($projectID, $this->post);
						break;
	
					/** Delete Tag **/
					case self::SUBACTION_DELETETAG:
						$this->project->deleteTag($projectID, $this->post);
						break;
	
					/** Edit Register **/
					case self::SUBACTION_EDITREG:
						$this->project->editRegister($projectID, $this->post);
						break;
						
					/** Delete Register **/
					case self::SUBACTION_DELETEREG:
						$this->project->deleteRegister($projectID, $this->post);
						break;
						
					/** Delete Project - Deletes a lot of stuff **/
					case self::SUBACTION_DELPROJECT:
						$this->project->deleteProject($projectID);
						break;
							
					/** Clobber Project **/
					case self::SUBACTION_CLOBBER:
						$startObj = new BFStart();
						$startObj->clobber($projectID);
						break;

				}
			}
		}
		return array($projectID, $stepNumber, $uniqueSID, $noteID);
	}


	############################################################################
	#	Below are the various methods by which to register objects with jTE	#
	############################################################################

	/** registerSTEPS
	 * Registers everything that is needed for a steps list with the template engine.  There is a lot.
	 */
	protected function registerSTEPS($projectID, $stepID, $uniqueSID, $noteID) {
		// Register this steps notes list with the template engine - this is a special case situation
		$this->notes->registerNotesList($this->jte, 'noteslist', $projectID, $uniqueSID);
		
		// object registers
		$this->register_StepsList('listing');					// Primary listing of the steps for this project
		$this->register_Project('project', $projectID);			// Selected Project information
		$this->register_Step('step', $uniqueSID);				// Selected Step Information (Defaults: 1)
		$this->register_CurrentBuilds('numbuilds');				// the number of builds a user has

		// assigns
		$this->jte->assign('hasbuilds', $this->project->hasBuilds());	// Boolean - Checks to see if project has builds
		$this->jte->assign('abovestep', ((array_key_exists('abovestep',$this->request))?$this->request['abovestep']:''));	// What step to insert above?
		$this->jte->assign('hassteps', $this->project->hasSteps());		// Boolean - Checks to see if a project has steps
	}

	/**
	 * registerPROJECTS - Registers all project info..
	 */
	protected function registerPROJECTS($projectID, $stepID, $uniqueSID, $noteID) {
		// sanity checks
		$this->project->validateSelectors();					// This will now check to make sure that selectors that are assigned are real.
		
		// object registers
		$this->register_PrimaryList('listing',true);			// Primary listing of the projects or libraries depending on classname
		$this->jte->assign('clobbers', $this->getClobbers());

		// If we are clobbering this project, don't let the user edit it.
		if(	isset($this->jte->_jTE_entities['clobbers']) && 
			is_array($this->jte->_jTE_entities['clobbers']) && 
			in_array($projectID, $this->jte->_jTE_entities['clobbers'])) {
				if (isset($this->request['sa']) && $this->request['sa'] == 'Clobber') {
					// Er.. never mind.. do nothing here
				} else {
					// We are trying to edit a clobbered message.. say no!
					$GLOBALS['controller']->addError(translate('2035'));
				}
		} else {
			// There are cases were these three are set to -1 so that the object is generated, but with nothing
			$this->register_Project('project', $projectID);			// Selected Project information
			$this->jte->assign('projectid', $projectID);			// Assigns our project id to a variable for use
			$this->register_TagsList('tagslist', $projectID);		// Registers our current projects tags		
			$this->register_RegList('reglist', $projectID);			// Registers a list of registers with jTE
		}

	}

	/** registerProjectsList
	 * registers a list of all projects with jTE
	 * if regName == false; then return the object instead of registering it
	 */
	protected function register_PrimaryList($regName=FALSE, $allowLimit=TRUE) {
		global $GLOBAL_SETTINGS, $user;

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_description',	'bf_projects',	'project');
		$rep->addColumn('bf_tag',			'bf_projects',	'tag');
		$rep->addColumn('bf_class',			'bf_projects',	'class');
		$rep->addColumn('bf_param',			'bf_env',		'env');
		$rep->addColumn('bf_platform',		'bf_projects',	'selector');
		$rep->addColumn('bf_snotify',		'bf_projects',	's');
		$rep->addColumn('bf_pnotify',		'bf_projects',	'p');
		$rep->addColumn('bf_fnotify',		'bf_projects',	'f');
		$rep->addColumn('bf_name',			'bf_levels',	'access');
		$rep->addColumn('bf_pid',			'bf_projects',	'id');
		$rep->addColumn('bf_bid',			'bf_env',		'envid');

		$rep->join(
			array('INNER', 'bf_levels',
					array(
						array('=', 'bf_projects', 'bf_level', 'bf_levels', 'bf_level'),
					)
				)
			);
		$rep->join(
			array('LEFT', 'bf_env',
					array(
						array('=', 'bf_env', 'bf_bid', 'bf_projects', 'bf_eid'),
					)
				)
			);

		$rep->where(array(array('(', array('=', 'bf_env', 'bf_eid', 0), 'OR',	array('=', 'bf_projects', 'bf_eid', 0), ')')));

		// If BOTH then get both, else based on the class ... != for projects, = for libraries
		if (!isset($this->request['both'])) {
			$rep->where(array(array(((get_class($this)=='Plans')?'!=':'=') , 'bf_projects', 'bf_platform', '')));
		}

		$uid = intval($user->get('bf_userid'));
		if ($uid !== 1) {
			$rep->where(array(
				array('=', 'bf_projects', 'bf_level', 'bf_levels', 'bf_level'),
				array('IN', 'bf_projects', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey('access', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey('f', array(array('LIKE', 'bf_projects', 'bf_fnotify')));
			$userDefinedFilter->setKey('p', array(array('LIKE', 'bf_projects', 'bf_pnotify')));
			$userDefinedFilter->setKey('s', array(array('LIKE', 'bf_projects', 'bf_snotify')));
			$userDefinedFilter->setKey('platform', array(array('LIKE', 'bf_projects', 'bf_platform')));
			$userDefinedFilter->setKey('selector', array(array('LIKE', 'bf_projects', 'bf_platform')));
			$userDefinedFilter->setKey('env', array(array('LIKE', 'bf_env', 'bf_param')));
			$userDefinedFilter->setKey('environment', array(array('LIKE', 'bf_env', 'bf_param')));
			$userDefinedFilter->setKey('class', array(array('LIKE', 'bf_projects', 'bf_class')));
			$userDefinedFilter->setKey('tag', array(array('LIKE', 'bf_projects', 'bf_tag')));
			$userDefinedFilter->setKey('project',
										array(
											'(',
											array('LIKE', 'bf_projects', 'bf_description'),
											')'
										));
			$userDefinedFilter->setKey('library',
										array(
											'(',
											array('LIKE', 'bf_projects', 'bf_description'),
											')'
										));
			$userDefinedFilter->setKey( '~',	// ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_projects', 'bf_description'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_tag'),
											'OR',
											array('LIKE', 'bf_levels', 'bf_name'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_platform'),
											'OR',
											array('LIKE', 'bf_env', 'bf_param'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_class'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_fnotify'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_pnotify'),
											'OR',
											array('LIKE', 'bf_projects', 'bf_snotify'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'project')
			);

		// If requested, set a new page to display. Otherwise, use defaults
		if ($allowLimit) {
			$count	= (array_key_exists('count', $this->request))	? intval($this->request['count'])	: $GLOBAL_SETTINGS['reporting']['count'];
			$offset = (array_key_exists('offset', $this->request))	? intval($this->request['offset'])	: $GLOBAL_SETTINGS['reporting']['offset'];
			$rep->limit($count, $offset);
		}
		// Primary Countby field.
		$rep->countBy('bf_pid', 'bf_projects');

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			$this->temp_project_count = $rep->getCount();

			if ($regName === FALSE) {
				return $res;
			} else {
				$this->jte->register_object($regName, $res, null, false);
				$this->jte->register_function('bf_can_quickstart', 'bf_can_quickstart');
			}
			return true;
		}

		return false;
	}

	/**
	 * registerStepsList
	 *
	 * registers all steps for this project as $regname
	 */
	protected function register_StepsList($regName, $allowLimit=true) {
		global $GLOBAL_SETTINGS;

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_uid',			'bf_steps',		'uid');			// Step Unique ID
		$rep->addColumn('bf_active',		'bf_steps',		'active');	 	// Active
		$rep->addColumn('bf_sid',			'bf_steps',		'step');		// Step Number
//		$rep->addColumn('bf_dist',			'bf_steps',		'broadcast');	// {Y}es or {N}o
//		$rep->addColumn('bf_dir',			'bf_steps',		'dir');			// Directory
		$rep->addColumn('bf_description',	'bf_steps',		'stepname');	// Step Name
		$rep->addColumn('bf_resource',		'bf_steps',		'selector');	// Selector (resource)
		$rep->addColumn('bf_param',			'bf_env',		'env');			// Environment
		$rep->addColumn('bf_absolute',		'bf_steps',		'absolute'); 	// Absolute Checkbox
		$rep->addColumn('bf_onfail',		'bf_steps',		'continue'); 	// Continue Checkbox
		$rep->addColumn('bf_pattern',		'bf_filters',	'result');	 	// ID to filters
		$rep->addColumn('bf_pnotify',		'bf_steps',		'pnotify');		// Pass Notification Group
		$rep->addColumn('bf_fnotify',		'bf_steps',		'fnotify');		// Fail Notification Group
		$rep->addColumn('bf_cid',			'bf_steps',		'inline');	 	// Threaded
		$rep->addColumn('bf_pcid',			'bf_steps',		'pass');		// Pass Chain
		$rep->addColumn('bf_fcid',			'bf_steps',		'fail');		// Fail Chain
		$rep->addColumn('bf_name',			'bf_levels',	'access');	 	// Threaded
		$rep->addColumn('bf_canbg',			'bf_steps',		'thread');	 	// Threaded
		$rep->addColumn('bf_pid',			'bf_steps',		'pid');			// Project
		$rep->addColumn('bf_bid',			'bf_env',		'envid');		// Environment ID (for link)
		$rep->addColumn('bf_did',			'bf_filters',	'resultid');	// Filter ID (for link)
		$rep->join(
			array('LEFT', 'bf_filters',
					array(
						array('=', 'bf_steps',	'bf_did', 'bf_filters', 'bf_did'),
					)
				)
			);
		$rep->join(
			array('LEFT', 'bf_env',
					array(
						array('=', 'bf_env', 'bf_bid', 'bf_steps', 'bf_eid'),
					)
				)
			);
		$rep->join(
			array('LEFT', 'bf_levels',
					array(
						array('=', 'bf_steps',	'bf_level', 'bf_levels', 'bf_level'),
					)
				)
			);

		// Add constraints
		$rep->where(
				array(
					'(', array('=', 'bf_filters', 'bf_fid', 0), 'OR', array('=', 'bf_steps', 'bf_did', 0), ') AND',
					'(', array('=', 'bf_env', 'bf_eid', 0), 'OR', array('=', 'bf_steps', 'bf_eid', 0), ')',
					array('=', 'bf_steps',	'bf_pid',	$this->request['bf_id']),
					)
			);

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey('access', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey('step', array(array('LIKE', 'bf_steps', 'bf_sid')));
			$userDefinedFilter->setKey('name', array(array('LIKE', 'bf_steps', 'bf_description')));
			$userDefinedFilter->setKey('~',	// ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_steps', 'bf_description'),
											'OR',
											array('LIKE', 'bf_levels', 'bf_name'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'step')
			);

		// If requested, set a new page to display. Otherwise, use defaults
		if ($allowLimit) {
			$count	= (array_key_exists('count', $this->request))	? intval($this->request['count'])	: $GLOBAL_SETTINGS['reporting']['count'];
			$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
			$rep->limit($count, $offset);
		}


		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$stepsRes = $rep->getReportResults();
			$this->jte->register_object($regName, $stepsRes, null, false);
			return true;
		}
		return false;
	}

	/**
	 * registerProject
	 *
	 * registers a single project as $regName
	 */
	protected function register_Project($regName, $projectID) {
		global $user;

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_level', 'bf_projects');
		$qb->where(array(array('=', 'bf_projects', 'bf_pid', $projectID)));
		$projectLevel = $qb->getOne();
		$uid = intval($user->get('bf_userid'));
		$projectID = ($uid === 1 || in_array($projectLevel, array_keys(bf_get_levels($uid))))?$projectID:0;

		$rep = new BFReport();
		$rep->addColumn('bf_pid',			'bf_projects',	'id');
		$rep->addColumn('bf_description',	'bf_projects',	'name');
		$rep->addColumn('bf_tag',			'bf_projects',	'tag');
		$rep->addColumn('bf_class',			'bf_projects',	'class');
		$rep->addColumn('bf_param',			'bf_env',		'env');
		$rep->addColumn('bf_eid',			'bf_projects',	'envid');		
		$rep->addColumn('bf_platform',		'bf_projects',	'selector');
		$rep->addColumn('bf_level',			'bf_projects',	'level');
		$rep->addColumn('bf_tagsync',		'bf_projects',	'tagsync');
		$rep->addColumn('bf_maxthread',		'bf_projects',	'maxthread');
		$rep->addColumn('bf_exclusive',		'bf_projects',	'runlimit');
		$rep->addColumn('bf_sticky',		'bf_projects',	'sticky');
		$rep->addColumn('bf_pnotify',		'bf_projects',	'pnotify');
		$rep->addColumn('bf_fnotify',		'bf_projects',	'fnotify');
		$rep->addColumn('bf_snotify',		'bf_projects',	'snotify');
		$rep->addColumn('bf_pcid',			'bf_projects',	'passchain');
		$rep->addColumn('bf_fcid',			'bf_projects',	'failchain');
		$rep->addColumn('bf_geo_id',		'bf_projects',	'geo_id');
		$rep->addColumn('bf_location',		'bf_geo',		'geo_name');

		$rep->join(
			array('LEFT', 'bf_env',
					array(
						array('=', 'bf_env', 'bf_bid', 'bf_projects', 'bf_eid'),
						array('=', 'bf_env', 'bf_eid', 0)
					)
				)
			);
		$rep->join(
			array('LEFT', 'bf_geo',
					array(
						array('=', 'bf_geo', 'bf_geo_id', 'bf_projects', 'bf_geo_id')
					)
				)
			);

		$rep->where(array(
				array('=', 'bf_projects', 'bf_pid',	$projectID),
			));

		if ($rep->compile() && $rep->valid()) {
			$projectRes = $rep->getReportResults();
			$this->jte->register_object($regName, $projectRes, null, false);
			$this->jte->assign('referenced', $this->project->referenced());	// Boolean - Checks to see if project is referenced (chained, inlined, class chain, etc.)
			return true;
		}
		return false;
	}

	/** registerReglist
	 * registers the list of registers for this project
	 */
	protected function register_RegList($regName, $projectID) {
		$rep = new BFReport();
		$rep->addColumn('bf_data',		'bf_data',	'id');  // Used as an "ID" since we don't use unique IDs in this program
		$rep->addColumn('bf_register',	'bf_data',	'name');
		$rep->addColumn('bf_data',		'bf_data',	'content');
		$rep->where(array(
			array('=',	'bf_data',	'bf_uid',	$projectID),
			array('=',	'bf_data',	'bf_bid',	'0')
		));
		$rep->orderBy('ASC','name');
		if ($rep->compile() && $rep->valid()) {
			$rep = $rep->getReportResults();
			$this->jte->register_object($regName, $rep, null, false);
			return true;
		}
		return false;
	}

	/** registerFiltersList
	 * registers a list of filters
	 */
	protected function register_FiltersList($regName) {
		global $GLOBAL_SETTINGS;

		$rep = new BFReport();
		$rep->addColumn('bf_did',		'bf_filters' ,'id');
		$rep->addColumn('bf_pattern',	'bf_filters' ,'name');
		$rep->where(array(
			array('=', 'bf_filters', 'bf_fid',	'0'),
			array('=', 'bf_filters', 'bf_action', '')
		));
		if ($rep->compile() && $rep->valid()) {
			$rep = $rep->getReportResults();
			$this->jte->register_object($regName, $rep, null, false);
			return true;
		}
		return false;
	}

	protected function register_Step($regName, $uniqueSID) {
		global $GLOBAL_SETTINGS, $user;
		
		$noStepMatch = false;
		$whereArray = undef;

		/** Change the WHERE clause
		 * Defining our where clause depending on a number of factors;
		 * Is a step number set but not a unique SID (for backwards compatibility)
		 * is a unique SID set?
		 * are neither set? - assume step number one
		 */		
		 
		if ( isset($this->request['step']) && empty($uniqueSID) ) {		
			$whereArray = array (
					array('=', 'bf_steps', 'bf_pid', $this->request['bf_id']),
					array('=', 'bf_steps', 'bf_sid', $this->request['step'])
				);
		} else if ($uniqueSID) {
			$whereArray = array (
					array('=', 'bf_steps', 'bf_pid', $this->request['bf_id']),
					array('=', 'bf_steps', 'bf_uid', $uniqueSID)
				);			
		} else {
			$noStepMatch = true;
		}

		// Does the user have access to view this step?
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_level', 'bf_steps');
		$qb->where($whereArray);
		$stepLevel = $qb->getOne();

		$validStepLevels = array_keys(bf_get_levels(intval($user->get('bf_userid'))));
		$validStepLevels[] = 0;	// ZERO is a valid

		$rep = new BFReport();
		$rep->addColumn('bf_pid',			'bf_steps',	'pid');			// Project ID
		$rep->addColumn('bf_uid',			'bf_steps',	'uid');			// Step Unique ID
		$rep->addColumn('bf_sid',			'bf_steps',	'step');		// Step ID 0-however many steps
		$rep->addColumn('bf_description',	'bf_steps',	'name');		// Name.. duh.
		$rep->addColumn('bf_command',		'bf_steps',	'command');		// ID to storage
		$rep->addColumn('bf_dir',			'bf_steps',	'dir');			// Path
		$rep->addColumn('bf_active',		'bf_steps',	'active');	 	// {Y}es or {N}o
		$rep->addColumn('bf_absolute',		'bf_steps',	'absolute');	// {Y}es or {N}o
		$rep->addColumn('bf_level',			'bf_steps',	'level');		// ID to levels
		$rep->addColumn('bf_timeout',		'bf_steps',	'timeout');		// Time in seconds
		$rep->addColumn('bf_onfail',		'bf_steps',	'onfail');	 	// {C}ontinue or {H}alt
		$rep->addColumn('bf_dist',			'bf_steps',	'broadcast');	// {Y}es or {N}o
		$rep->addColumn('bf_canbg',			'bf_steps',	'thread');	 	// {Y}es, {N}o or {J}oin
		$rep->addColumn('bf_cid',			'bf_steps',	'inline');	 	// PID of other possible inlines
		$rep->addColumn('bf_pcid',			'bf_steps',	'passchain');	// PID of passchain
		$rep->addColumn('bf_fcid',			'bf_steps',	'failchain');	// PID of failchain
		$rep->addColumn('bf_passwait',		'bf_steps',	'passwait');	// {Y}es or {N}o
		$rep->addColumn('bf_failwait',		'bf_steps',	'failwait');	// {Y}es or {N}o
		$rep->addColumn('bf_did',			'bf_steps',	'result');	 	// ID to filters
		$rep->addColumn('bf_eid',			'bf_steps',	'env');			// ID to environments
		$rep->addColumn('bf_resource',		'bf_steps',	'selector');	// Selector (resource)
		$rep->addColumn('bf_pnotify',		'bf_steps',	'pnotify');		// Pass Notification Group
		$rep->addColumn('bf_fnotify',		'bf_steps',	'fnotify');		// Fail Notification Group
		$rep->addColumn('bf_nchange',		'bf_steps',	'nchange');		// 
		$rep->addColumn('bf_param',			'bf_env',	'envname');			// ID to environments

		$rep->join(
			array('LEFT', 'bf_env',
					array(
						array('=', 'bf_env', 'bf_bid', 'bf_steps', 'bf_eid'),
						array('=', 'bf_env', 'bf_eid', 0)
					)
				)
			);

		$rep->where($whereArray);
	
		// Is it a valid report?  Does the user have access to see this step?  Is he root?
		if ($noStepMatch) {
			return false;
		}
		if ($rep->compile() && $rep->valid() && 
			(intval($user->get('bf_userid')) === 1 || (in_array($stepLevel, $validStepLevels)))) {
			$rep = $rep->getReportResults();
			$this->jte->register_object($regName, $rep, null, false);
			return true;
		}
		$GLOBALS['controller']->addError(translate('2039'));
		return false;
	}

	/** registerTagsList($jteRegisterAsName)
	 * registers a list of the tags for a specific project with jTE
	 */
	protected function register_TagsList($regName, $projectID) {
		global $GLOBAL_SETTINGS;
		$rep = new BFReport();

		$rep->addColumn('bf_tid',		'bf_tags',	'id');
		$rep->addColumn('bf_name',		'bf_tags',	'name');
		$rep->addColumn('bf_value',		'bf_tags',	'value');
		$rep->addColumn('bf_parent',	'bf_tags',	'padding');
		$rep->addColumn('bf_autoinc',	'bf_tags',	'autoincrement');

		$rep->where(
				array('=', 'bf_tags', 'bf_pid', $projectID)
		);

		if ($rep->compile() && $rep->valid()) {
			$rep = $rep->getReportResults();
			$this->jte->register_object($regName, $rep, null, false);
			return true;
		}
		return false;
	}

	/** registerCurrentBuilds($jteRegisterAsName)
	 * registers the number of builds this user has with jTE
	 */
	protected function register_CurrentBuilds($regName) {
		global $user;
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid', 'bf_builds', 'bf_bid', 'COUNT');
		$qb->where(array(
			array('=', 'bf_builds', 'bf_userid', $user->get('bf_userid'))
		));
		$results = $qb->getOne();
		$this->jte->assign($regName, $results);
		return true;
	}

	/** Return an array of projects being clobbered
	 */
	protected function getClobbers() {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_pid', 'bf_builds', 'bf_pid');
		$qb->where(array(
			array('=', 'bf_builds', 'bf_process', 'N')
		));
		$res = $qb->getResult();
		$results = array();
		while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$results[$row['bf_pid']] = $row['bf_pid'];
		}
		return $results;
	}

	/** register_singleTag
	 *  registers a single tag with the template engine, usually for editing	
	 */
	protected function register_singleTag($regName, $tid) {
		$PID = $this->request['bf_id'];

		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_tid',	'bf_tags',	'tid');
		$qb->addColumn('bf_name',	'bf_tags',	'name');
		$qb->addColumn('bf_value',	'bf_tags',	'value');
		$qb->addColumn('bf_parent',	'bf_tags',	'parent');
		$qb->addColumn('bf_autoinc','bf_tags',	'autoinc');

		$qb->where(array(
			array('=', 'bf_tags', 'bf_pid', $PID),
			array('=', 'bf_tags', 'bf_tid', $tid)
		));

		$res = $qb->getResult();
		$results = $res->fetchRow(DB_FETCHMODE_ASSOC);
				
		$this->jte->assign($regName, $results);
		return true;
	}

	/** register singleRegister
	 * registers with jte a single register for editing.
	 */	 	
	protected function register_singleReg($regName, $rid) {
		$PID = $this->request['bf_id'];
		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_data', 		'bf_data', 'rid');
		$qb->addColumn('bf_register', 	'bf_data', 'register');
		$qb->addColumn('bf_data', 		'bf_data', 'content');
		$qb->where(array(
			array('=', 'bf_data', 'bf_uid', $PID),
			array('=', 'bf_data', 'bf_data', $rid)
		));
		
		$res = $qb->getResult();
		$results = $res->fetchRow(DB_FETCHMODE_ASSOC);
				
		$this->jte->assign($regName, $results);
		return true;
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if ($this->action && ($this->action == self::ACTION_VIEW_STEPS || $this->action == self::ACTION_EDIT_STEP))  {
			return $this->stepNavigationBlock();
		}
			
		if ($id)  {
			$project = new BFProject;
			$project->select($project->primary_key, $id);
		}  else  {
			$project = $this->project;
		}

		$id = $project->get($project->primary_key);
		if (!$id)  {
			return '';
		}
		
		// Show in navigation list the projects that we're tag sync'd to or that are tag sync'd to us,
		// or that pass / fail chain us or that we're pass / fail chained to.
		$temp = bf_get_nav_projects_and_steps(array(
				array('=', 'bf_projects', 'bf_pid', $project->get('bf_tagsync')),
				'OR',
				array('=', 'bf_projects', 'bf_tagsync', $id),
				'OR',
				array('=', 'bf_projects', 'bf_pid', $project->get('bf_pcid')),
				'OR',
				array('=', 'bf_projects', 'bf_pcid', $id),
				'OR',
				array('=', 'bf_projects', 'bf_pid', $project->get('bf_fcid')),
				'OR',
				array('=', 'bf_projects', 'bf_fcid', $id),
				), null);
				
		// Complicated where clause to show in the navigation list projects that we have steps that
		// pass / fail / inline chain to or that have steps that pass / fail / inline chain to us.
		$temp2 = bf_get_nav_projects_and_steps(
				array(
						'(',
							array('=', 'bf_steps', 'bf_pid', $id),
							'AND',
							'(', 
								array('=', 'bf_steps', 'bf_cid', 'bf_projects', 'bf_pid'),
								'OR',
								array('=', 'bf_steps', 'bf_pcid', 'bf_projects', 'bf_pid'),
								'OR',
								array('=', 'bf_steps', 'bf_fcid', 'bf_projects', 'bf_pid'), 
							')',
						')',
						'OR',
						'(', 
							array('=', 'bf_steps', 'bf_pid', 'bf_projects', 'bf_pid'),
							'AND',
							'(',
								array('=', 'bf_steps', 'bf_cid', $id),
								'OR',
								array('=', 'bf_steps', 'bf_pcid', $id),
								'OR',
								array('=', 'bf_steps', 'bf_fcid', $id),
							')',
						')'
				), 
				array(),
				'NOJOIN');
		foreach (array('projects', 'library') as $which)  {
			if (array_key_exists($which, $temp) && $temp[$which])  {
				if (!array_key_exists($which, $temp2))  {
					$temp2[$which] = $temp[$which];
				}  else {
					foreach ($temp[$which]['subnav'] as $k => $v)  {
						$temp2[$which]['subnav'][$k] = $v;
					}
				}
			}
		}
		
		$nav = array();
		$nav['environments'] = bf_get_nav_objects('environments', array(
				array('=', 'bf_env', 'bf_bid', $project->get('bf_eid')), array('=', 'bf_env', 'bf_eid', 0)));
		$nav['selectors'] = bf_get_nav_objects('selectors', 
				array('=', 'bf_selector', 'bf_selector_id', $project->get('bf_platform')));
		$nav['projects'] = array_key_exists('projects', $temp2) ? $temp2['projects'] : '';
		$nav['library'] = array_key_exists('library', $temp2) ? $temp2['library'] : '';
		$nav['adaptor links'] = bf_get_nav_objects('adaptor links', 
				array('=', 'bf_scopes', 'bf_pid', $id));
		$nav['schedules'] = bf_get_nav_objects('schedules', 
				array('=', 'bf_cron', 'bf_pid', $id));
		$nav['classes'] = bf_get_nav_objects('classes', array(
				array('=', 'bf_classes', 'bf_name', $project->get('bf_class')),
				'OR',
				array('=', 'bf_classes', 'bf_pid', $id),
				'OR',
				array('=', 'bf_classes', 'bf_npid', $id),
				'OR',
				array('=', 'bf_classes', 'bf_xpid', $id)				
				));
		$nav['templates'] = bf_get_nav_objects('templates', 
				array('=', 'bf_templates', 'bf_pid', $id));
		$arr = array();
		foreach (array($project->get('bf_level'), $project->get('bf_snotify'),
				$project->get('bf_pnotify'), $project->get('bf_fnotify')) as $v)  {
			if ($v)  {
				$arr[] = $v;
			}
		}
		if ($arr)  {
			$nav['access groups'] = bf_get_nav_objects('access groups', 
					array(array('IN', 'bf_levels', 'bf_level', $arr)));
		}

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function stepNavigationBlock($id = 0, $sid = 0)  {
	
		if ($id)  {
			$project = new BFProject;
			$project->select($project->primary_key, $id);
		}  else  {
			$project = $this->project;
		}
		$id = $project->get($project->primary_key);

		if ($sid)  {
			$step = new BFStep;
			$step->select(array($project->primary_key, $step->primary_key), array($id, $sid));
		}  else  {
			$step = $this->step;
		}
		
		$sid = $step->get($step->primary_key);
		
		if (!$id || !$sid)  {
			return '';
		}
		$arr = array();
		foreach (array($step->get('bf_cid'), $step->get('bf_pcid'), $step->get('bf_fcid')) as $v)  {
			if ($v)  {
				$arr[] = $v;
			}
		}
		if ($arr)  {
			$temp = bf_get_nav_projects_and_steps(
					//array(
					array('IN', 'bf_projects', 'bf_pid', $arr )  
					     //)  
					, NULL 
				);
		}  else {
			$temp = array();
		}
		
		$nav = array();
		
		$nav['environments'] = bf_get_nav_objects('environments', array(
				array('=', 'bf_env', 'bf_bid', $step->get('bf_eid')), array('=', 'bf_env', 'bf_eid', 0)));
		$nav['selectors'] = bf_get_nav_objects('selectors', 
				array('=', 'bf_selector', 'bf_selector_id', $step->get('bf_resource')));
		$nav['log filters'] = bf_get_nav_objects('log filters', 
				array(
						array('=', 'bf_filters', 'bf_did', $step->get('bf_did')),
						array('=', 'bf_filters', 'bf_fid', 0)
				));
		$nav['projects'] = array_key_exists('projects', $temp) ? $temp['projects'] : '';
		$nav['library'] = array_key_exists('library', $temp) ? $temp['library'] : '';
		$arr = array();
		foreach (array($project->get('bf_level'), $project->get('bf_pnotify'), $project->get('bf_fnotify'),
				$project->get('bf_snotify'), $step->get('bf_level'), $step->get('bf_pnotify'), 
				$step->get('bf_fnotify')) as $v)  {
			if ($v)  {
				$arr[] = $v;
			}
		}
		if ($arr)  {
			$nav['access groups'] = bf_get_nav_objects('access groups', 
					array(array('IN', 'bf_levels', 'bf_level', $arr)));
		}

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}
}


?>
