<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Templates
 *
 * @package Console
 * @subpackage Templates
 */

$GLOBALS['controller']->require_module('BFTemplate');
$GLOBALS['controller']->require_module('BFReport');

$GLOBALS['GLOBAL_OBJECT_KEY']	   = 'object';

class Templates extends Projects {
	// A special view just to snag project steps for a given project
	const ACTION_VIEW_PROJECTSTEPS = 'templates.steps';
	const ACTION_VIEW_TEMPLATES = 'templates.view';
	const ACTION_EDIT_TEMPLATES = 'templates.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;
	public $content;
	public $nav_content;
	protected $template;
	protected $templates;

	public function setRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			// Need this no matter what...
			$this->template = new BFTemplate();
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$templates = new Templates();
		$temp = new BFTemplate();
		$templates->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req) && array_key_exists('bf_pid', $req) && 
				array_key_exists('bf_sid', $req)) {
			$templates->load($req[$temp->primary_key], $req['bf_pid'], $req['bf_sid'], $req['bf_lang']);
		}
		return $templates;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS;

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_name', 'bf_templates', 'id');
		$rep->addColumn('bf_sid', 'bf_templates', 'sid');
		$rep->addColumn('bf_pid', 'bf_templates', 'pid');		
		$rep->addColumn('bf_description', 'bf_templates', 'description');
		$rep->addColumn('bf_description', 'bf_projects', 'project');
		$rep->addColumn('bf_description', 'bf_steps', 'step');
		$rep->addColumn('bf_subj', 'bf_templates', 'subject');
		$rep->addColumn('bf_lang', 'bf_templates', 'lang');
		$rep->join(
				array('LEFT', 'bf_projects',
					array(
						array('=', 'bf_projects',	'bf_pid', 'bf_templates', 'bf_pid'),
					)
				)
			);
		$rep->join(
				array('LEFT', 'bf_steps',
					array(
						array('=', 'bf_projects',	'bf_pid', 'bf_steps', 'bf_pid'),
						array('=', 'bf_steps',	'bf_sid', 'bf_templates', 'bf_sid'),
					)
				)
			);
		
		//$rep->countBy('bf_name', 'bf_templates');

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_templates', 'bf_description')));
			$userDefinedFilter->setKey( 'subject', array(array('LIKE', 'bf_templates', 'bf_subj')));
			$userDefinedFilter->setKey( 'language',array(array('LIKE', 'bf_templates', 'bf_lang')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_templates', 'bf_description'),
											  'OR',
											  array('LIKE', 'bf_templates', 'bf_subj'),
											  'OR',
											  array('LIKE', 'bf_templates', 'bf_lang'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$possible = array(
						  'description' => array('bf_templates', 'bf_description'),
						  'subject' => array('bf_templates', 'bf_subj'),
						  'project' => array('bf_projects', 'bf_description'),
						  'step' => array('bf_steps', 'bf_description')
						  );
		if (array_key_exists('orderby', $this->request) && array_key_exists($this->request['orderby'], $possible)) {
			$rep->orderBy($sort, $possible[$this->request['orderby']][0], $possible[$this->request['orderby']][1]);
		} else {
			$rep->orderBy('ASC', 'bf_templates', 'bf_description');
		}

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	protected function editTemplate() {
		$this->manageTemplate();
		$this->jte->register_object('editable', $this->template, null, false);
	}
	
	/**
	 * This should get altered to accept different forms of requests (ie. other than POST
	 * but it'll do for now. This method runs through the request and sets any values it
	 * finds from the request. The environment object weeds out the garbage and validates/escapes.
	 */
	protected function manageTemplate() {
		global $controller;
		if (array_key_exists($GLOBALS['GLOBAL_OBJECT_KEY'], $this->post) && is_array($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
				$this->template->delete();
			} else {
				foreach ($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']] as $field => $value) {
					$this->template->set($field, $value);
				}
				try {
					$this->template->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFTemplate object (if necessary)
		5. Set the nav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW_TEMPLATES;

		$this->handleAction();
		$this->jte->register_function('bf_get_steps', 'bf_get_steps', null, false);
		$this->jte->register_function('bf_get_templates', 'bf_get_templates', null, false);

		// Steps view
		if ($this->action != self::ACTION_VIEW_PROJECTSTEPS) {
			$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Projects/templates');
			$this->jte->register_function('bf_get_project_names', 'bf_get_project_names', null, false);
		}
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Projects/templates/' . $this->request_type . '.' . 
				$this->action . '.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		$this->editTemplate();
		$this->frontPageReport();
	}

	public function getIncludes() {
		return array(
				'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Projects/javascripts/templates.content.js')
		);
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = '', $pid = 0, $sid = 0, $lang = 'en_US')  {
		if (!$id)  {
			$template = $this->template;
		}  else {
			$template = new BFTemplate();
			$template->select(array($template->primary_key, 'bf_pid', 'bf_sid', 'bf_lang'), array($id, $pid, $sid, $lang));
		}

		$id = $template->get($template->primary_key);
		if (!$id)  {
			return '';
		}
		$pid = $template->get('bf_pid');
		$sid = $template->get('bf_sid');

		$nav = array();
		if (!$pid)  {
			$type = $template->get('bf_name');
			
			if ($type != 'system_alert' && $type != 'purge_fail' && $type != 'filter_fire' && 
					$type != 'artifact_fire')  {
				//  TODO:  Add a "NOT IN" in addition to an "IN".  For now, the stupid time & memory consuming way.
				$qb = BFQueryBuilder::select();
				$qb->addColumn('bf_pid', 'bf_templates', 'pid', 'DISTINCT');
				$templatepids = $qb->getCol();
				if (!DB::isError($templatepids))  {
					$where = array(array('!=', 'bf_projects', 'bf_pid', 0));
					$stepwhere = null;
					foreach ($templatepids as $pid)  {
						$where[] = array('!=', 'bf_projects', 'bf_pid', $pid);
					}
					switch($type)  {
						case 'build_start':
							$where[] = array('!=', 'bf_projects', 'bf_snotify', 0);
							break;
						case 'build_pass':
						case 'build_warn':
							$where[] = array('!=', 'bf_projects', 'bf_pnotify', 0);
							break;
						case 'build_fail':
						case 'build_break':
							$where[] = array('!=', 'bf_projects', 'bf_fnotify', 0);
							break;
						case 'step_pass':
						case 'step_warn':
							$stepwhere = array('!=', 'bf_steps', 'bf_pnotify', 0);
						case 'step_fail':
							$stepwhere = array('!=', 'bf_steps', 'bf_fnotify', 0);
					}
						
					$nav = bf_get_nav_projects_and_steps($where, $stepwhere, 'AND');
				}
			}
		}  elseif (!$sid)  {
			$nav = bf_get_nav_projects_and_steps(array('=', '', 'bf_pid', $pid), null);
		}  else {
			$nav = bf_get_nav_projects_and_steps(
					array('=', '', 'bf_pid', $pid),
					array('=', '', 'bf_sid', $sid), 'AND');
		}

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}


	public function load($id, $pid, $sid, $lang) {
		if (!empty($id)) {
			$pid = intval($pid);
			$sid = intval($sid);
			// Load the Template itself
			$fields = array($this->template->primary_key, 'bf_pid', 'bf_sid', 'bf_lang');
			$values = array($id, $pid, $sid, $lang);
			$this->template->select($fields, $values);
		}
	}

	public function __construct() {
		parent::__construct();
		$this->jte = new BFJTE();
	}
}

/**
 * Returns an array of template ids/descriptions if id isn't set or the corresponding description
 * if id is set.
 */
function bf_get_templates($lang = null, $id = null) {
	if (!isset($lang)) {
		$lang = $GLOBALS['user']->get('bf_lang');
	}
	$qb = BFQueryBuilder::select();
	$qb->addColumn('bf_description', 'bf_templates', 'description');
	$qb->addColumn('bf_name', 'bf_templates', 'name');
	if ($id)  {
		$qb->where(array(
				array('=', 'bf_templates', 'bf_pid', 0),
				array('=', 'bf_templates', 'bf_name', $id),
				array('=', 'bf_templates', 'bf_lang', $lang),
				));
		$res = $qb->getOne();
		return DB::isError($res) ? '' : $res;
	}  else {
		$qb->where(array(
				array('=', 'bf_templates', 'bf_pid', 0),
				array('=', 'bf_templates', 'bf_lang', $lang),
				));
		$qb->orderBy('ASC', 'bf_templates', 'bf_description');
		$res = $qb->getResult();
		if (DB::isError($res)) {
			return array();
		}
		
		$templates = array();
		while ($template = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$templates[$template['name']] = $template['description'];
		}
		return $templates;
	}
}


?>
