<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Query
 *
 * A viewable module that does not support XHTML. You can only access this module
 * via AJAX or the API.
 *
 * @package Console
 * @subpackage Query
 */

define('BF_REQUEST_API', 'api');

$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

class Query extends Module implements Viewable {
	const ACTION_VIEW = 'view';

	protected $jte;
	protected $request;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XML, BF_REQUEST_API);
	protected $report;

	public $request_type;
	public $content;

	public function setRequest($req, $type) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request =	$req;
			$this->request_type =   $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XML) {
		$query = new Query();
		$query->setRequest($req, $type);
		return $query;
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Query/templates');
		$this->handleAction();

		$this->jte->assign('results', $this->report->getReportResults());
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		return BF_ROOT . '/public/fullcontrol/modules/Query/templates/' . $this->request_type . '.' . 
				$this->action . '.tpl';
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		switch ($this->action) {
			case self::ACTION_VIEW:
			default:
				global $_POST;
				if (array_key_exists(BF_REQUEST_KEY_QUERY, $_POST)) {
					$this->parseQuery($_POST[BF_REQUEST_KEY_QUERY]);
				}
		}
	}

	/**
	 * Someone needs to do the dirty work of actually pulling a valid
	 * query out of the request. This could come in unimaginable amounts
	 * of combinations.
	 *
	 * Take the posted, serialized array and return a BFQueryBuilder
	 * created object.
	 */
	protected function parseQuery($q) {
		$this->report = new BFReport();
		$this->report->unserialize($q);
		if (!$this->report->compile()) {
			throw new BFException(BF_LANG_ERR_INVALID_QUERY);
		}
	}

	/**
	 * For now, return nothing. Could potentially link to XSL, CSS, etc.
	 */
	public function getIncludes() {
		return array();
	}
}

?>
