<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Schedules
 *
 * @package Console
 * @subpackage Schedules
 */

$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

class Schedules extends Module implements ViewableXHTML {
	const ACTION_VIEW = 'schedule.view';
	const ACTION_VIEW_CALENDAR = 'schedule.calendar.view';
	const ACTION_VIEW_LIST = 'schedule.list.view';
	const ACTION_EDIT = 'schedule.edit';

	protected $includes;
	protected $nav_report;

	// Key for the cron object
	const GLOBAL_CRON_KEY		= 'object';
	const GLOBAL_CRON_ENV_KEY	= 'environment';

	protected $jte;	  	// Instance of jTE
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;   //
	public $content;		//
	protected $request;
	protected $post;
	public $action;
	protected $cron;

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function setRequest($req, $type, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}
		return false;
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$Schedules = new Schedules();
		$Schedules->request_type = $type;
		$Schedules->setRequest($req, $type, $post);
		return $Schedules;
	}

	protected function handleAction() {
		// In the event of mulitple actions.. deal with them here
		switch ($this->action) {
			case self::ACTION_EDIT:
				$this->register_calendar('scheduler');
				break;
			case self::ACTION_VIEW:
			case self::ACTION_VIEW_CALENDAR:
			case self::ACTION_VIEW_LIST:
			default:
				$this->register_calendar('scheduler');
		}
	}

	public function render() {
		global $controller, $user;

		$schedule_defaultpage = $user->getPreference('schedule_defaultpage');
		if (!isset($this->action) || $this->action == self::ACTION_VIEW)  {
			if (!empty($schedule_defaultpage))  {
				$this->action = $schedule_defaultpage;
			}  else {
				$this->action = self::ACTION_VIEW_LIST;
			}
		}

		if ($this->request_type == BF_REQUEST_XHTML && $this->action != $schedule_defaultpage && 
				$this->action != self::ACTION_EDIT)  {
			$user->setPreference('schedule_defaultpage', $this->action);
		}

		// Hellooo template engine
		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Schedules/templates');

		// Add the userobj to jTE
		$this->jte->register_object('userobj', $user, null, false);

		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->nav_content = $this->navigationBlock();		
		}

		$this->jte->assign(BF_GIMME_REFRESH, 1);
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	protected function chooseTemplate() {
		if ($this->request_type == 'xhtml')  {
			return BF_ROOT . '/public/fullcontrol/modules/Schedules/templates/xhtml.schedule.view.tpl';
		} elseif ($this->action == self::ACTION_EDIT) {
			return BF_ROOT . '/public/fullcontrol/modules/Schedules/templates/xml.schedule.edit.tpl';
		}  else {
			return BF_ROOT . '/public/fullcontrol/modules/Schedules/templates/xml.schedule.view.tpl';
		}
	}


	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					'stylesheets' => array(BF_WEBROOT . 'fullcontrol/modules/Schedules/styles/schedule.global.css',),
					'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Schedules/javascripts/schedule.content.js')
				);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * Creates and registers a calendar for scheduling builds
	 */
	protected function register_calendar($regName) {
		// Calendar management
		$this->controller->require_module('BFCalendar');
		$cal = new BFCalendar();
		$filter = isset($this->request['filter']) ? $this->request['filter'] : '';
		$sortby = isset($this->request['orderby']) ? $this->request['orderby'] : '';
		$sortorder = isset($this->request['sort']) ? $this->request['sort'] : '';
		$cal->loadAllEvents($filter, $sortby, $sortorder);
		$cal->generateCalendar();

		// Cron management
		$this->controller->require_module('BFCron');
		$this->cron = new BFCron();
		if (array_key_exists($this->cron->primary_key, $this->request) && 
				!empty($this->request[$this->cron->primary_key])) {
			$this->cron->select($this->cron->primary_key, $this->request[$this->cron->primary_key]);
			if (!$this->cron->get($this->cron->primary_key))  {
				$this->controller->addError(translate('2264'));
			}
		}
		// Make any changes/insertions
		if (array_key_exists(self::GLOBAL_CRON_KEY, $this->post)) {

			// Scrub the various cron fields for double or single quotes
			$scrubFields = array('bf_minute','bf_hour','bf_dom','bf_month','bf_dow');
			$illegalCharacters = array('/\"/','/\'/');
			foreach($scrubFields as $k) {
				if (isset($this->post[self::GLOBAL_CRON_KEY][$k])) {
					$this->post[self::GLOBAL_CRON_KEY][$k] = preg_replace($illegalCharacters,"", 
							$this->post[self::GLOBAL_CRON_KEY][$k]);
				}
			}


			if (array_key_exists('delete', $this->post[self::GLOBAL_CRON_KEY])) {
				try {
					$this->cron->delete();
				} catch (Exception $e) {
					$this->controller->addError($e->getMessage());
				}
			}  elseif (isset($this->post[self::GLOBAL_CRON_KEY]['copy'])) {
				$this->cron->copy();
			} else if ($this->cron->setMany($this->post[self::GLOBAL_CRON_KEY])) {
				try {
					if (array_key_exists(self::GLOBAL_CRON_ENV_KEY, $this->post) && 
							is_array($this->post[self::GLOBAL_CRON_ENV_KEY])) {
						$this->cron->setEnvironment($this->post[self::GLOBAL_CRON_ENV_KEY]);
					}
					$this->cron->save();
				} catch (Exception $e) {
					$this->controller->addError($e->getMessage());
				}
			}
		}

		// Actually assign objects to the template engine
		$this->jte->register_object('module', $this, null, false);
		$this->jte->register_object($regName, $cal, null, false);
		$this->jte->register_object('listing', $cal->getAllEvents(), null, false);
		$this->jte->register_object('editable', $this->cron, null, false);
		$this->jte->assign('dow', array('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat'));
		$this->jte->register_function('bf_get_project_names','bf_get_project_names', false);
		$this->jte->register_function('bf_env_to_html',		'bf_env_to_html',		false);
		$this->jte->register_function('bf_get_users',		'bf_get_users',			false);
		$this->jte->register_function('bf_format_date',		'bf_format_date',			false);
	}



	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$cron = $this->cron;
		}  else {
			$cron = new BFCron();
			$cron->select($cron->primary_key, $id);
		}
		$id = $cron->get($cron->primary_key);
		if (!$id)  {
			return '';
		}

		$temp = bf_get_nav_projects_and_steps(array('=', 'bf_projects', 'bf_pid', $cron->get('bf_pid')), null);
		$nav = array(
			'environments' => bf_get_nav_objects('environments', array(array('=', 'bf_env', 'bf_bid', 
					$cron->get('bf_eid')), array('=', 'bf_env', 'bf_eid', 0))),
			'projects' => array_key_exists('projects', $temp) ? $temp['projects'] : '',
			'library' => array_key_exists('library', $temp) ? $temp['library'] : '',	
			'selectors' => bf_get_nav_objects('selectors', array(array('=', 'bf_selector', 'bf_selector_id', 
					$cron->get('bf_selector')), array('=', 'bf_selector', 'bf_prop', ''))),
			'users' => bf_get_nav_objects('users', array('=', 'bf_users', 'bf_userid', $cron->get('bf_owner'))),
			'classes' => bf_get_nav_objects('classes', array('=', 'bf_classes', 'bf_name', 
					$cron->get('bf_class')))
		);
		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}

}

?>
