<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Agents
 *
 * @package Console
 * @subpackage Agents
 */

$GLOBALS['controller']->require_module('BFServer');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');
$GLOBALS['controller']->require_module('BFNotes');

/**
 * Used to make an array in the request to keep the server fields
 * by themselves to reduce confusion when making changes to the server.
 */
$GLOBALS['GLOBAL_SERVER_KEY'] = 'server';
$GLOBALS['GLOBAL_MANIFEST_KEY'] = 'manifest';

/**
 * Agents
 *
 * The Agents object extends Module in order to transparently take advantage of some of the BFDBO
 * methods and permission management. It also implements ViewableXHTML, a subinterface of Viewable,
 * since all viewable modules that handle the XHTML request type need to implement this interface
 * (primarily for the controller to have a way of requesting the navigaiton structure or content).
 *
 * The object itself acts as the controller for the BFServer model
 * and uses the templates in this folder for the views. Since it
 * accepts different request types, it has different views to go
 * along with them (ex: XML listing of the Servers used via AJAX).
 *
 * @package Console
 * @subpackage Agents
 */
class Agents extends Servers implements ViewableRSS2 {

	const ACTION_VIEW = 'servers.view';
	const ACTION_EDIT = 'servers.edit';
	const ACTION_TEST = 'servers.test';

	const ACTION_EDIT_NOTES		= 'servers.edit.notes';		// Updates the notes area with new notes -- special circumstance

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RSS2, BF_REQUEST_RAW);
	protected $server;
	protected $statii;
	protected $ttl = 5;
	protected $lastBuildDate;

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request =	$req;
			$this->post = $post;
			$this->request_type =   $type;
			$this->server = new BFServer();
			$this->server->setRequest($this->request);	 	// Send along the request information

			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$server = new Agents();
		$temp = new BFServer();
		$server->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$server->load($req[$temp->primary_key]);
		}
		return $server;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS, $user;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_id', 'bf_servers', 'id');
		$rep->addColumn('bf_err', 'bf_servers', 'err');
		$rep->addColumn('bf_name', 'bf_servers', 'name');
		$rep->addColumn('bf_name', 'bf_levels', 'access');
		$rep->addColumn('bf_coll_id', 'bf_servers', 'collector');
		$rep->addColumn('bf_host', 'bf_servers', 'host');
		$rep->addColumn('bf_path', 'bf_servers', 'path');
		$rep->addColumn('bf_level', 'bf_servers', 'level');
		$rep->addColumn('bf_active', 'bf_servers', 'active');
		// Add constraints
		$uid = intval($user->get('bf_userid'));
		if ($uid === 1) {
			$rep->where(array('=', 'bf_servers', 'bf_level', 'bf_levels', 'bf_level'));
		} else {
			$rep->where(array(
				array('=', 'bf_servers', 'bf_level', 'bf_levels', 'bf_level'),
				array('IN', 'bf_servers', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'host', array(array('LIKE', 'bf_servers', 'bf_host')));
			$userDefinedFilter->setKey( 'path', array(array('LIKE', 'bf_servers', 'bf_path')));
			$userDefinedFilter->setKey( 'access', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey( 'collector', array(array('LIKE', 'bf_servers', 'bf_coll_id')));
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_servers', 'bf_name')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_servers', 'bf_name'),
											'OR',
											array('LIKE', 'bf_levels', 'bf_name'),
											'OR',
											array('LIKE', 'bf_servers', 'bf_coll_id'),
											'OR',
											array('LIKE', 'bf_servers', 'bf_host'),
											'OR',
											array('LIKE', 'bf_servers', 'bf_path'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$possible = array(
						  'collector' => array('bf_servers', 'bf_coll_id'),
						  'host' => array('bf_servers', 'bf_host'),
						  'path' => array('bf_servers', 'bf_path'),
						  'name' => array('bf_servers', 'bf_name')
						  );
		if (array_key_exists('orderby', $this->request) && array_key_exists($this->request['orderby'], $possible)) {
			$rep->orderBy($sort, $possible[$this->request['orderby']][0], $possible[$this->request['orderby']][1]);
		} else {
			$rep->orderBy('ASC', 'bf_servers', 'bf_name');
		}

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();
			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the server object,
	 * but it does call manageServer() (which makes changes) and then
	 * registers the server object with the jTE object for use in
	 * templates.
	 */
	protected function editServer() {
		$this->manageServer();
		$this->jte->register_object('editable', $this->server, null, false);
	}

	/**
	 * This should get altered to accept different forms of requests (ie. other than POST
	 * but it'll do for now. This methodruns through the request and sets any values it
	 * finds from the request. The server object weeds out the garbage and validates/escapes.
	 */
	protected function manageServer() {
		global $GLOBAL_SERVER_KEY, $GLOBAL_MANIFEST_KEY, $controller;
		try {
			if (is_array($this->post)
					&& array_key_exists($GLOBAL_SERVER_KEY, $this->post)
					&& is_array($this->post[$GLOBAL_SERVER_KEY])) {
				$this->jte->assign('saving', true);
				// Delete?
				if (array_key_exists('delete', $this->post[$GLOBAL_SERVER_KEY])) {
					return $this->server->delete();
				}  elseif (isset($this->post[$GLOBAL_SERVER_KEY]['copy'])) {
					return $this->server->copy();
				} else {
					foreach ($this->post[$GLOBAL_SERVER_KEY] as $field => $value) {
						// Annoying checkboxes, give me a useful value!
						if ($field == 'bf_inactive') {
							$this->server->set('bf_active', ($value == 'true') ? 'N' : 'Y');
						} else {
							$this->server->set($field, $value);
						}
					}
					$this->server->save();
				}
			}
			// Any manifest stuff?
			if (is_array($this->post)                                                                                                                                                                                                                                               
                    && array_key_exists($GLOBAL_MANIFEST_KEY, $this->post)
					&& is_array($this->post[$GLOBAL_MANIFEST_KEY])) {
				foreach ($this->post[$GLOBAL_MANIFEST_KEY] as $field => $value) {
					$this->server->setManifest($field, $value, 1);
				}
				$this->server->saveManifest();
			}
		} catch (Exception $e) {
			$controller->addError($e->getMessage());
		}
	}
	
	/**
	 * Gets the status of a server to get output via RSS
	 */
	protected function serverStatusReport() {
		$rep = new BFReport();
		$rep->addColumn('bf_id', 'bf_servers');
		$rep->addColumn('bf_name', 'bf_servers');
		$rep->addColumn('bf_test', 'bf_servers');
		$rep->addColumn('bf_active', 'bf_servers');
		$rep->addColumn('bf_err', 'bf_servers');
		$res = $rep->getReportResults();

		if (!$res->valid()) {
			return false;
		}

		$this->statii = array();
		$this->lastBuildDate = 0;

		while ($row = $res->next()) {
			$updated = false;
			$test = bf_get_storage($row['bf_test']);
			
			// TODO: Update this when we internationalize the output
			$offset = strpos($test, translate('completed'));
			if ($offset && preg_match('/\d+/', $test, $matches, null, $offset)) {
				$updated = (int)$matches[0];
				if ($updated > $this->lastBuildDate) {
					$this->lastBuildDate = $updated;
				}
			}
			$this->statii[$row['bf_id']] = array(
								'id' => $row['bf_id'],
								'name' => $row['bf_name'],
								'test' => $test,
								'active' => $row['bf_active'],
								'err' => $row['bf_err'],
								'updated' => ($updated ? $updated : time())
							);
		}
		$this->jte->assign('servers', $this->statii);
		return true;
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFServer object (if necessary)
		5. Set thenav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller, $user;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Servers/templates');
		$this->handleAction();

		$this->jte->assign('is_styx', $this->controller->configuration->styx()); // Styx Hack

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->register_object('editable', $this->server, null, false);
			$this->nav_content = '';
		} else if ($this->request_type == BF_REQUEST_RSS2) {
			$controller->addHeader('Last-Modified: ' . $this->lastBuildDate);
		}
		// We need some functions registered
		$this->jte->register_function('bf_get_collectors',   'bf_get_collectors');
		$this->jte->register_function('bf_translate_servertest',  'bf_translate_servertest');
		$this->jte->register_object('userobj', $user, null, false);

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
		
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XHTML) {
			return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xhtml.servers.view.tpl';
		} else {
			return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/' . $this->request_type . '.' . 
					$this->action . '.tpl';
		}
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		// If an RSS request, just get the feed contents and return
		if ($this->request_type == BF_REQUEST_RSS2) {
			$this->serverStatusReport();
			return true;
		}

		// Begin: Edit/Save/Delete Notes
		$this->notes = new BFNotes();
		$this->notes->noteType = 'ServerNote';
		if (isset($this->post['edit_note_id'])) {
			$this->action = 'servers.edit.notes';
			$this->notes->currentNote = $this->post['edit_note_id'];
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid']);	
		} else if (isset($this->post['delete_note_id'])) {
			$this->action = 'servers.edit.notes';
			$this->notes->deleteNote($this->post, $this->post['pid'], $this->post['sid']);
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid']);	
		} else if (isset($this->post['save_note'])) {
			$this->action = 'servers.edit.notes';
			$this->notes->saveNote($this->post, $this->post['pid'], $this->post['sid']);
			$this->notes->registerNotesList($this->jte, 'noteslist', $this->post['pid']);	
		} else {
			if (isset($this->request['bf_id']) && !empty($this->request['bf_id'])) {
				$this->notes->registerNotesList($this->jte, 'noteslist', $this->request['bf_id']);
			}	
		}		
		// End: Edit/Save/Delete Notes
				
		if ($this->action == self::ACTION_EDIT || $this->action == self::ACTION_TEST) {
			$this->editServer();
		}
		if ($this->action == self::ACTION_EDIT || $this->action == self::ACTION_VIEW) {
			$this->frontPageReport();
		}
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
				'stylesheets' => array(BF_WEBROOT . 'fullcontrol/modules/Servers/styles/servers.view.css'),
				'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Servers/javascripts/servers.view.content.js'),
				'alternates' => array(array(
								'id' => 'alt_rss2',
								'type' => 'application/rss+xml',
								'title' => 'RSS 2.0',
								'href' => BF_WEBROOT . 'fullcontrol/?mod=servers&type=rss2'
								))
				);
	}

	public function load($id) {
		if (!empty($id)) {
			$this->server->select($this->server->primary_key, $id);
		}
	}




	/**
	 * Return the cacheable period in minutes
	 */
	public function getTTL() {
		return $this->ttl;
	}
	
	/**
	 * Get the stamp from the last (chronologically speaking) message and return it.
	 */
	public function getLastBuildDate() {
		return $this->lastBuildDate;
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$server = $this->server;
		}  else {
			$server = new BFServer();
			$server->select($this->server->primary_key, $id);
		}
		
		$id = $server->get($server->primary_key);
		if (!$id)  {
			return '';
		}

		$nav = array(
			'collectors' => bf_get_nav_objects('collectors', array('=', 'bf_collector', 'bf_coll_id', 
					$server->get('bf_coll_id'))),
			'environments' => bf_get_nav_objects('environments', array(
					array('=', 'bf_env', 'bf_bid', $server->get('bf_eid')),
					array('=', 'bf_env', 'bf_eid', 0)
			)),
			'server auths' => bf_get_nav_objects('server auth', 
					array('=', 'bf_serverauth', 'bf_authid', $server->get('bf_authid'))),
			'access groups' => bf_get_nav_objects('access groups', 
					array('=', 'bf_levels', 'bf_level', $server->get('bf_level')))
		);

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}

}

function bf_translate_servertest($messages = null)  {
	global $user;

	if(!$messages)  {
		return '';
	}
	if (preg_match('/[^a-zA-Z0-9,!\n]/', $messages))  {
		return $messages;
	}

	$messages = explode("\n", $messages);
	$return = '';
	foreach($messages as $message)  {
		$parameters = explode(',', $message);
		$message = bf_translate_string(pack("H*", array_shift($parameters)));
		if ($message)  {
			foreach ($parameters as $i => $p) {
				$parameters[$i] = pack("H*", $p);
			}
			$return .= @vsprintf($message, $parameters) . "\n";
		}
	}
	return $return;
}

?>
