<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Collectors
 *
 * @package Console
 * @subpackage Collectors
 */

$GLOBALS['controller']->require_module('BFCollector');
$GLOBALS['controller']->require_module('BFReport');

$GLOBALS['GLOBAL_OBJECT_KEY']	   = 'parent';
$GLOBALS['GLOBAL_SUBOBJECT_KEY']	= 'variable';

$GLOBALS['COLL_BUILTINS'] = array(
								'CPU_ARCH',
								'CPU_LOAD',
								'CPU_LOAD1',
								'CPU_LOAD15',
								'CPU_LOAD5',
								'CPU_MANUFACTURER',
								'CPU_MHZ',
								'CPU_MODEL',
								'CPU_SERIAL',
								'DISK_FREE',
								'DISK_TOTAL',
								'MEM_FREE',
								'MEM_LOAD',
								'MEM_PAGESIZE',
								'MEM_TOTAL',
								'NET_FQDN',
								'NET_HWADDR',
								'NET_IFACE',
								'NET_IPV',
								'NET_IPV4',
								'NET_IPV6',
								'NET_SPEED',
								'NUM_CPU',
								'OS_HOSTID',
								'OS_MACHINE',
								'OS_RELEASE',
								'OS_SYSNAME',
								'OS_VERSION',
								'WIN_SERVICEPACK'
							);

class Collectors extends Servers {
	const ACTION_VIEW_COLLECTORS = 'collectors.view';
	const ACTION_EDIT_COLLECTORS = 'collectors.edit';
	const ACTION_VIEW_COLLECTOR = 'collectors.view.prop';
	const ACTION_EDIT_COLLECTOR = 'collectors.edit.prop';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;
	public $content;
	public $nav_content;
	protected $collector;
	protected $collector_prop;

	public function setRequest($req, $type, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			// Need this no matter what...
			$this->collector = new BFCollector();
			$this->collector_prop = new BFCollector();
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$collectors = new Collectors();
		$temp = new BFCollector();
		$collectors->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			// Key an eye out for the prop as well
			$prop = (array_key_exists('bf_seq', $req)) ? $req['bf_seq'] : null;
			$collectors->load($req[$temp->primary_key], $prop);
		}
		return $collectors;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $GLOBAL_SETTINGS;
		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_coll_id', 'bf_collector', 'id', 'DISTINCT');
		$rep->addColumn('bf_coll_id', 'bf_collector', 'name');
		$rep->where(array('!=', 'bf_collector', 'bf_coll_id', ''));
		$rep->countBy('bf_coll_id', 'bf_collector');

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_collector', 'bf_prop')));
			$userDefinedFilter->setKey( 'access', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_collector', 'bf_coll_id'),
											  'OR',
											  array('LIKE', 'bf_collector', 'bf_prop'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$rep->orderBy($sort, 'bf_collector', 'bf_coll_id');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : 
				$GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : 
				$GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	protected function propPageReport($id) {
		global $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_seq', 'bf_collector', 'id');
		$rep->addColumn('bf_prop', 'bf_collector', 'property');
		$rep->addColumn('bf_value', 'bf_collector', 'value');
		$rep->addColumn('bf_user', 'bf_collector', 'type');
		$rep->addColumn('bf_regex', 'bf_collector', 'regex');
		// Add constraints
		$rep->where(array(
						  array('!=', 'bf_collector', 'bf_prop', ''),
						  array('=', 'bf_collector', 'bf_coll_id', $id)
					  ));
		$rep->orderBy('ASC', 'bf_collector', 'bf_seq');

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'property', array(array('LIKE', 'bf_collector', 'bf_prop')));
			$userDefinedFilter->setKey( 'value', array(array('LIKE', 'bf_collector', 'bf_value')));
			$userDefinedFilter->setKey( 'regex', array(array('LIKE', 'bf_collector', 'bf_regex')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_collector', 'bf_prop'),
											  'OR',
											  array('LIKE', 'bf_collector', 'bf_value'),
											  'OR',
											  array('LIKE', 'bf_collector', 'bf_regex'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the environment object,
	 * but it does call manageEnvironment() (which makes changes) and then
	 * registers the environment object with the jTE object for use in
	 * templates.
	 */
	protected function editCollector() {
		if (!($this->collector_prop instanceof BFCollector)) {
			$this->collector_prop = new BFCollector();
		}
		$this->manageCollector();
		$this->jte->register_object('parent', $this->collector, null, false);
		$this->jte->register_function('bf_get_collectors', 'bf_get_collectors', null, false);
		$this->jte->register_object('editable', $this->collector_prop, null, false);
	}

	/**
	 * This should get altered to accept different forms of requests (ie. other than POST
	 * but it'll do for now. This method runs through the request and sets any values it
	 * finds from the request. The environment object weeds out the garbage and validates/escapes.
	 */
	protected function manageCollector() {
		global $controller;
		if (array_key_exists($GLOBALS['GLOBAL_OBJECT_KEY'], $_REQUEST) && is_array($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
				$this->collector->deleteCollector();
			}  elseif (isset($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']]['copy'])) {
				$this->collector->copy();
			} else {
				// Making a new one?
				$inserting = ($this->collector->get('bf_coll_id') === false);
				foreach ($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']] as $field => $value) {
					$this->collector->set($field, $value);
				}
				try {
					$this->collector->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
			}
		}
		// Yes, you could theoretically edit both the collector and one of its variables concurrently
		if (array_key_exists($GLOBALS['GLOBAL_SUBOBJECT_KEY'], $_REQUEST) && is_array($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']])) {
			// Delete?
			if (isset($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['move'])) {
				$this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_seq'] = $this->collector_prop->get('bf_seq') + 
						$this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['move'];
				if ($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_seq'] < 1)  {
					$this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_seq'] = 1;
				}
			}
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']])) {
				$this->collector_prop->delete();
			}  elseif (isset($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['copy'])) {
				$this->collector_prop->copy($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['copy']);
			} else {
				if (array_key_exists('bf_type', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']])) {
					$this->collector_prop->set('bf_seq', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_seq']);
					if ($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_type'] == 'Run Command') {
						// If a Set Value or Run Command
						$this->collector_prop->set('bf_prop', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_prop']);
						$this->collector_prop->set('bf_user', 'Y');
						$this->collector_prop->set('bf_value', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_value']);
						$this->collector_prop->set('bf_regex', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_regex']);
						// If a set value
					} else if ($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_type'] == 'Set Value') {
						$this->collector_prop->set('bf_prop', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_prop']);
						$this->collector_prop->set('bf_user', 'N');
						$this->collector_prop->set('bf_value', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_value']);
						$this->collector_prop->set('bf_regex', '');
						// If a built-in
					} else if ($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_type'] == '.include') {
						$this->collector_prop->set('bf_prop', '.include');
						$this->collector_prop->set('bf_user', 'N');
						$this->collector_prop->set('bf_value', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_value_include']);
						$this->collector_prop->set('bf_regex', '');
						// If a built-in
					} else {
						$this->collector_prop->set('bf_prop', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['bf_prop_default']);
						$this->collector_prop->set('bf_user', 'N');
						$this->collector_prop->set('bf_value', '');
						$this->collector_prop->set('bf_regex', '');
					}
				// If not editing, you must want to re-order the thing or something
				} else {
					foreach ($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']] as $field => $value) {
						$this->collector_prop->set($field, $value);
					}
				}

				try {
					$this->collector_prop->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFCollector object (if necessary)
		5. Set the nav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller, $COLL_BUILTINS;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW_COLLECTORS;

		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Servers/templates');
		$this->jte->assign('builtins', $COLL_BUILTINS);
		$this->handleAction();

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}
	
	public function getNavigation() {
		return false;
	}
	
	public function getSubNavigation() {
		global $GLOBAL_SETTINGS;
		$rep = new BFReport();
		
		// Add the necessary fields
		$rep->addColumn('bf_seq', 'bf_collector', 'id');
		$rep->addColumn('bf_prop', 'bf_collector', 'name');
		// Add constraints
		$rep->where(array(
						  array('!=', 'bf_collector', 'bf_prop', ''),
						  array('=', 'bf_collector', 'bf_coll_id', $this->collector->get('bf_coll_id'))
						  ));
		$rep->orderBy('ASC', 'bf_collector', 'bf_seq');
		
		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();
			
			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );
			
			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'property', array(array('LIKE', 'bf_collector', 'bf_prop')));
			$userDefinedFilter->setKey( 'value', array(array('LIKE', 'bf_collector', 'bf_value')));
			$userDefinedFilter->setKey( 'regex', array(array('LIKE', 'bf_collector', 'bf_regex')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_collector', 'bf_prop'),
											  'OR',
											  array('LIKE', 'bf_collector', 'bf_value'),
											  'OR',
											  array('LIKE', 'bf_collector', 'bf_regex'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();
			
			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}
		
		return $rep->getReportResults();
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XHTML) {
			if ($this->action == self::ACTION_VIEW_COLLECTORS || $this->action == self::ACTION_EDIT_COLLECTORS) {
				return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xhtml.collectors.view.tpl';
			} else {
				return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xhtml.collectors.view.prop.tpl';
			}
		} else {
			$target_path = BF_ROOT . '/public/fullcontrol/modules/Servers/templates/' . $this->request_type . '.' . $this->action . '.tpl';
			if (is_readable($target_path)) {
				return $target_path;
			} else {
				if ($this->action == self::ACTION_VIEW_COLLECTORS || $this->action == self::ACTION_EDIT_COLLECTORS) {
					return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xml.collectors.view.tpl';
				} else {
					return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xml.collectors.view.prop.tpl';
				}
			}
		}
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		// Really, this could get called from anywhere
		$this->editCollector();
		switch ($this->action) {
			case self::ACTION_EDIT_COLLECTOR:
			case self::ACTION_VIEW_COLLECTOR:
				$this->propPageReport($this->collector->get('bf_coll_id'));
				break;
			case self::ACTION_EDIT_COLLECTORS:
			case self::ACTION_VIEW_COLLECTORS:
			default:
				$this->frontPageReport();
		}
	}

	public function getIncludes() {
		return array(
					 'javascripts' => array(
											BF_WEBROOT . 'javascripts/mouseOvers.js',
											BF_WEBROOT . 'fullcontrol/javascripts/actions_menu.js',
											BF_WEBROOT . 'fullcontrol/modules/Servers/javascripts/' . $this->action . '.content.js')
					 );
	}


	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$coll = $this->collector;
		}  else {
			$coll = new BFCollector();
			$coll->select($coll->primary_key, $id);
		}

		$id = $coll->get($coll->primary_key);
		if (!$id)  {
			return '';
		}

		$nav = array(
			'servers' => bf_get_nav_objects('servers', array('=', 'bf_servers', 'bf_coll_id', $id))
		);

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}


	public function load($id, $prop = null) {
		// Load the Environment itself
		$fields = array($this->collector->primary_key, 'bf_seq');
		$values = array($id, 0);
		$this->collector->select($fields, $values);

		// If necessary, also load the particular variable
		if ($prop) {
			$fields = array($this->collector->primary_key, 'bf_seq');
			$values = array($id, $prop);
			$this->collector_prop->select($fields, $values);
		} else {
			// We need a bid no matter what
			$this->collector_prop->set($this->collector->primary_key, $id);
			// The wonderful world of not having primary keys...
			$this->collector_prop->stored_values[$this->collector->primary_key] = $id;
		}
	}

	public function __construct() {
		parent::__construct();
		$this->jte = new BFJTE();
	}
}

?>
