<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Selectors
 *
 * @package Console
 * @subpackage Selectors
 */

$GLOBALS['controller']->require_module('BFSelector');
$GLOBALS['controller']->require_module('BFReport');

$GLOBALS['GLOBAL_OBJECT_KEY']	   = 'parent';
$GLOBALS['GLOBAL_SUBOBJECT_KEY']	= 'variable';

$GLOBALS['sigils'] = array('==','!=','<','>','<=','>=','><');

class Selectors extends Servers {
	const ACTION_VIEW_SELECTORS = 'selectors.view';
	const ACTION_EDIT_SELECTORS = 'selectors.edit';
	const ACTION_VIEW_SELECTOR = 'selectors.view.prop';
	const ACTION_EDIT_SELECTOR = 'selectors.edit.prop';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	public $request_type;
	public $content;
	public $nav_content;
	protected $selector;
	protected $selector_prop;

	public function setRequest($req, $type, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			// Need this no matter what...
			$this->selector = new BFSelector();
			$this->selector_prop = new BFSelector();
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$selectors = new Selectors();
		$temp = new BFSelector();
		$selectors->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			// Key an eye out for the prop as well
			$prop = (array_key_exists('bf_prop', $req)) ? $req['bf_prop'] : null;
			$op = (array_key_exists('bf_operator', $req)) ? $req['bf_operator'] : null;
			$value = (array_key_exists('bf_value', $req)) ? $req['bf_value'] : null;
			$selectors->load($req[$temp->primary_key], $prop, $op, $value);
		}
		return $selectors;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $user, $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();
		// Add the necessary fields
		$rep->addColumn('bf_selector_id', 'bf_selector', 'id', 'DISTINCT');
		$rep->addColumn('bf_selector_id', 'bf_selector', 'name');
		$rep->addColumn('bf_name', 'bf_levels', 'access');
		$rep->addColumn('bf_level', 'bf_selector', 'level');
		
		// Add constraints
		$uid = intval($user->get('bf_userid'));
		if ($uid === 1) {
			$rep->where(array('=', 'bf_selector', 'bf_level', 'bf_levels', 'bf_level'));
		} else {
			$rep->where(array(
				array('=', 'bf_selector', 'bf_level', 'bf_levels', 'bf_level'),
				array('IN', 'bf_selector', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}
		$rep->countBy('bf_selector_id', 'bf_selector');

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_selector', 'bf_prop')));
			$userDefinedFilter->setKey( 'access', array(array('LIKE', 'bf_levels', 'bf_name')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_selector', 'bf_selector_id'),
											  'OR',
											  array('LIKE', 'bf_selector', 'bf_prop'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$possible = array(
						  'name' => array('bf_selector', 'bf_selector_id'),
						  'access' => array('bf_levels', 'bf_name')
						  );
		if (array_key_exists('orderby', $this->request) && array_key_exists($this->request['orderby'], $possible)) {
			$rep->orderBy($sort, $possible[$this->request['orderby']][0], $possible[$this->request['orderby']][1]);
		} else {
			$rep->orderBy('ASC', 'bf_selector', 'bf_selector_id');
		}
		
		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);
		
		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	protected function propPageReport($id) {
		global $controller, $GLOBAL_SETTINGS;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_prop',		'bf_selector', 'id');
		$rep->addColumn('bf_prop',		'bf_selector', 'variable');
		$rep->addColumn('bf_operator',	'bf_selector', 'operator');
		$rep->addColumn('bf_value',		'bf_selector', 'value');
		$rep->addColumn('bf_required',	'bf_selector', 'required');
		// Add constraints
		$rep->where(array(
						array('!=', 'bf_selector', 'bf_prop', ''),
						array('=', 'bf_selector', 'bf_selector_id', "$id")
					));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_selector', 'bf_prop')));
			$userDefinedFilter->setKey( 'value', array(array('LIKE', 'bf_selector', 'bf_value')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_selector', 'bf_selector_id'),
											  'OR',
											  array('LIKE', 'bf_selector', 'bf_prop'),
											  'OR',
											  array('LIKE', 'bf_selector', 'bf_value'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$sort = array_key_exists('sort', $this->request) && $this->request['sort'] == 'desc' ? 'DESC' : 'ASC';
		$possible = array(
						  'variable' => array('bf_selector', 'bf_prop'),
						  'operator' => array('bf_selector', 'bf_operator'),
						  'value' => array('bf_selector', 'bf_value'),
						  'required' => array('bf_selector', 'bf_required')
						  );
		if (array_key_exists('orderby', $this->request) && array_key_exists($this->request['orderby'], $possible)) {
			$rep->orderBy($sort, $possible[$this->request['orderby']][0], $possible[$this->request['orderby']][1]);
		} else {
			$rep->orderBy('ASC', 'bf_selector', 'bf_prop');
		}

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the environment object,
	 * but it does call manageEnvironment() (which makes changes) and then
	 * registers the environment object with the jTE object for use in
	 * templates.
	 */
	protected function editSelector() {
		if (!($this->selector_prop instanceof BFSelector)) {
			$this->selector_prop = new BFSelector();
		}
		$this->manageSelector();
		$this->jte->register_object('parent', $this->selector, null, false);
		$this->jte->register_object('editable', $this->selector_prop, null, false);
		$this->jte->register_function('bf_get_collector_props',  'bf_get_collector_props', false);
	}

	/**
	 * This should get altered to accept different forms of requests (ie. other than POST
	 * but it'll do for now. This method runs through the request and sets any values it
	 * finds from the request. The environment object weeds out the garbage and validates/escapes.
	 */
	protected function manageSelector() {
		global $controller;
		if (array_key_exists($GLOBALS['GLOBAL_OBJECT_KEY'], $_REQUEST) && is_array($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_OBJECT_KEY']])) {
				$this->selector->delete();
			}  elseif (isset($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']]['copy'])) {
				$this->selector->copy();
			} else {
				// Making a new one?
				$inserting = ($this->selector->get('bf_selector_id') === false);
				foreach ($this->post[$GLOBALS['GLOBAL_OBJECT_KEY']] as $field => $value) {
					$this->selector->set($field, $value);
				}
				try {
					$this->selector->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
				if ($inserting && $this->request_type == BF_REQUEST_XHTML) {
					$controller->addHeader('Location: ' . BF_WEBROOT . 'fullcontrol/?mod=administration&action=' . self::ACTION_VIEW_SELECTOR . '&bf_selector_id=' . $this->selector->get('bf_selector_id'));
				}
			}
		}
		// Yes, you could theoretically edit both the Environment and one of its variables concurrently
		if (array_key_exists($GLOBALS['GLOBAL_SUBOBJECT_KEY'], $_REQUEST) && is_array($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']])) {
				$this->selector_prop->delete();
			}  elseif (isset($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']]['copy'])) {
				$this->selector_prop->copy();
			} else {
				foreach ($this->post[$GLOBALS['GLOBAL_SUBOBJECT_KEY']] as $field => $value) {
					$this->selector_prop->set($field, $value);
				}
				try {
					$this->selector_prop->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
					return false;
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFSelector object (if necessary)
		5. Set the nav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller, $sigils;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW_SELECTORS;

		$this->jte->assign('sigils', $sigils);
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Servers/templates');
		$this->handleAction();

		$this->jte->assign('is_styx', $this->controller->configuration->styx()); // Styx Hack

		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}

	public function getNavigation() {
		return false;
	}

	public function getSubNavigation() {
		global $user;

		$rep = new BFReport();

		// Add the necessary fields
		$rep->addColumn('bf_prop',		'bf_selector', 'id');
		$rep->addColumn('bf_prop',		'bf_selector', 'name');
		$rep->addColumn('bf_operator',	'bf_selector', 'operator');
		$rep->addColumn('bf_value',		'bf_selector', 'value');
		// Add constraints
		$rep->where(array(
						array('!=', 'bf_selector', 'bf_prop', ''),
						array('=', 'bf_selector', 'bf_selector_id', $this->selector->get('bf_selector_id'))
					));

		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'name', array(array('LIKE', 'bf_selector', 'bf_prop')));
			$userDefinedFilter->setKey( 'value', array(array('LIKE', 'bf_selector', 'bf_value')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											  '(',
											  array('LIKE', 'bf_selector', 'bf_selector_id'),
											  'OR',
											  array('LIKE', 'bf_selector', 'bf_prop'),
											  'OR',
											  array('LIKE', 'bf_selector', 'bf_value'),
											  ')'
											  ));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy('ASC', 'bf_selector', 'bf_prop');

		return $rep->getReportResults();
	}

	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XHTML) {
			if ($this->action == self::ACTION_VIEW_SELECTORS || $this->action == self::ACTION_EDIT_SELECTORS) {
				return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xhtml.selectors.view.tpl';
			} else {
				return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xhtml.selectors.view.prop.tpl';
			}
		} else {
			$target_path = BF_ROOT . '/public/fullcontrol/modules/Servers/templates/' . $this->request_type . '.' . $this->action . '.tpl';
			if (is_readable($target_path)) {
				return $target_path;
			} else {
				if ($this->action == self::ACTION_VIEW_SELECTORS || $this->action == self::ACTION_EDIT_SELECTORS) {
					return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xml.selectors.view.tpl';
				} else {
					return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xml.selectors.view.prop.tpl';
				}
			}
		}
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		// Really, this could get called from anywhere
		$this->editSelector();
		switch ($this->action) {
			case self::ACTION_EDIT_SELECTOR:
			case self::ACTION_VIEW_SELECTOR:
				$this->propPageReport($this->selector->get('bf_selector_id'));
				break;
			case self::ACTION_VIEW_SELECTORS:
			case self::ACTION_EDIT_SELECTORS:
			default:
				$this->frontPageReport();
		}
	}

	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Servers/javascripts/' . $this->action . '.content.js')
					 );
	}

	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$selector = $this->selector;
		}  else {
			$selector = new BFSelector();
			$selector->select($selector->primary_key, $id);
		}

		$id = $selector->get($selector->primary_key);
		if (!$id)  {
			return '';
		}

		$nav = bf_get_nav_projects_and_steps(array('=', '', 'bf_platform', $id), 
				array('=', '', 'bf_resource', $id));
		$nav['schedules'] = bf_get_nav_objects('schedules', array('=', 'bf_cron', 'bf_selector', $id));
		$nav['access groups'] = bf_get_nav_objects('access groups', array('=', 'bf_levels', 'bf_level', 
				$selector->get('bf_level')));

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}

	public function load($id, $prop = null, $op = null, $value = null) {
		// Load the Selector itself
		$fields = array($this->selector->primary_key, 'bf_prop');
		$values = array($id, '');
		$this->selector->select($fields, $values);

		// If necessary, also load the particular variable
		if (!is_null($prop) && $prop != '' && !is_null($op) && $op != '' && !is_null($value)) {
			$fields = array($this->selector->primary_key, 'bf_prop', 'bf_operator', 'bf_value');
			$values = array($id, $prop, $op, $value);
			$this->selector_prop->select($fields, $values);
		} else {
			// We need a bid no matter what
			$this->selector_prop->set($this->selector->primary_key, $id);
			// The wonderful world of not having primary keys...
			$this->selector_prop->stored_values[$this->selector->primary_key] = $id;
		}
	}

	public function __construct() {
		parent::__construct();
		$this->jte = new BFJTE();
	}
}

?>
