<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Admin ServerAuth
 *
 * @package Console
 * @subpackage ServerAuth
 */

$GLOBALS['controller']->require_module('BFServerAuth');
$GLOBALS['controller']->require_module('BFReport');
$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['GLOBAL_OBJECT_KEY'] = 'object';

class ServerAuth extends Servers {
	const ACTION_VIEW = 'serverauth.view';
	const ACTION_EDIT = 'serverauth.edit';

	protected $jte;
	protected $request;
	protected $post;
	public $action;
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML, BF_REQUEST_RAW);
	protected $serverauth;
	protected $references;

	public $request_type;
	public $content;
	public $nav_content;

	public function setRequest($req, $type, $post = null) {
		if ($this->handlesRequestType($type)) {
			global $GLOBAL_SETTINGS;
			$this->request = $req;
			$this->post = $post;
			$this->request_type = $type;
			$this->serverauth = new BFServerAuth();
			if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
				$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
			}
			return true;
		}

		return false;
	}

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * I'd really like PHP to support extending classes that have some abstract method.
	 * Then we wouldn't have to have to either replicate methods like this everywhere
	 * or make a subclass of Module to implement these for Viewable Modules.
	 *
	 * Bastards.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$serverauth = new ServerAuth();
		$temp = new BFServerAuth();
		$serverauth->setRequest($req, $type, $post);
		if (array_key_exists($temp->primary_key, $req)) {
			$serverauth->load($req[$temp->primary_key]);
		}
		return $serverauth;
	}

	/**
	 * This stupidly long method constructs a report using the BFReport object,
	 * runs the report and assigns the BFReportResults object to the jTE object
	 * for use in the template.
	 */
	protected function frontPageReport() {
		global $controller, $GLOBAL_SETTINGS, $user;
		try {
			$controller->require_module('BFReport');
		} catch (Exception $e) {
			$controller->exitWithError($e);
			return false;
		}

		$rep = new BFReport();

		// Setup the columns, most of which will get displayed
		$rep->addColumn('bf_authid', 'bf_serverauth', 'id');
		$rep->addColumn('bf_username', 'bf_serverauth', 'username');
		
		$uid = intval($user->get('bf_userid'));
		if ($uid === 1) {
			$rep->where(array('!=', 'bf_serverauth', 'bf_authid', ''));
		} else {
			$rep->where(array(
					array('!=', 'bf_serverauth', 'bf_authid', ''),
					array('IN', 'bf_serverauth', 'bf_level', array_keys(bf_get_levels($uid)))
			));
		}
		
		// Add filter constraints
		if (array_key_exists('filter', $this->request)) {
			$userDefinedFilter = new BFFilterRequest();

			// Set the user request
			$userDefinedFilter->setFilter( $this->request['filter'] );

			// Set Keys, Actions and optional Value filters - See class for instructions
			$userDefinedFilter->setKey( 'id', array(array('LIKE', 'bf_serverauth', 'bf_authid')));
			$userDefinedFilter->setKey( 'username', array(array('LIKE', 'bf_serverauth', 'bf_username')));
			$userDefinedFilter->setKey( '~',   // ~ = NO keyword
										array(
											'(',
											array('LIKE', 'bf_serverauth', 'bf_authid'),
											'OR',
											array('LIKE', 'bf_serverauth', 'bf_username'),
											')'
										));
			// Get the results
			$filterArray = $userDefinedFilter->getFilter();

			// Send our newly generated filter array to the report
			$rep->filter($filterArray);
		}

		// Add a order by and sort direction
		$rep->orderBy(
			(array_key_exists('sort', $this->request) ? "{$this->request['sort']}" : 'ASC'),
			(array_key_exists('orderby', $this->request) ? "{$this->request['orderby']}" : 'id')
		);

		// LEFT JOIN throws the COUNT out of whack
	   // $rep->countBy('bf_serverauth', 'bf_authid');

		// If requested, set a new page to display. Otherwise, use defaults
		$count = (array_key_exists('count', $this->request)) ? intval($this->request['count']) : $GLOBAL_SETTINGS['reporting']['count'];
		$offset = (array_key_exists('offset', $this->request)) ? intval($this->request['offset']) : $GLOBAL_SETTINGS['reporting']['offset'];
		$rep->limit($count, $offset);

		// Make sure it works
		if ($rep->compile() && $rep->valid()) {
			$res = $rep->getReportResults();

			$this->jte->register_object('listing', $res, null, false);
			$this->jte->register_object('module', $this, null, false);
		}
	}

	/**
	 * This method doesn't directly change anything in the user object,
	 * but it does call manage() (which makes changes) and then
	 * registers the user object with the jTE object for use in
	 * templates.
	 */
	protected function edit() {
		global $user;

		$this->manage();
		$uid = intval($user->get('bf_userid'));
		$level = $this->serverauth->get('bf_level');

		$hasAccess = ($uid === 1 || in_array($level, array_keys(bf_get_levels($uid))))?true:false;

		if ($hasAccess)  {
			$this->jte->register_object('editable', $this->serverauth, null, false);
		}
	}

	protected function manage() {
		global $GLOBAL_OBJECT_KEY, $controller;
		if (array_key_exists($GLOBAL_OBJECT_KEY, $_REQUEST) && is_array($this->post[$GLOBAL_OBJECT_KEY])) {
			// Delete?
			if (array_key_exists('delete', $this->post[$GLOBAL_OBJECT_KEY])) {
				$this->serverauth->delete();
			}  elseif (isset($this->post[$GLOBAL_OBJECT_KEY]['copy'])) {
				$this->serverauth->copy();
			} else {
				foreach ($this->post[$GLOBAL_OBJECT_KEY] as $field => $value) {
					// Passwords need verification and "encryption"
					if ($field == 'bf_password') {
						// If not matching throw exception
						if (!array_key_exists('bf_password_verify', $this->post[$GLOBAL_OBJECT_KEY]) || $this->post[$GLOBAL_OBJECT_KEY]['bf_password_verify'] != $value) {
							throw new BFException(translate('2087'));
						} else if ($value == '') {
							continue;
						} else {
							$value = bf_encrypt($value);
						}
					}
					$this->serverauth->set($field, $value);
				}
				try {
					$this->serverauth->save();
				} catch (Exception $e) {
					$controller->addError($e->getMessage());
				}
			}
		}
	}

	/**
	 * In order, this:
		1. Sets the action to view if unset
		2. Creates a jTE object to render the module contents
		3. Calls handlAction() to act on whatever action got set/requested
		4. If XHTML request, make and register a new BFServerAuth object (if necessary)
		5. Set thenav_content to an empty string, so the nav falls back on the array
		6. Churn the template + vars into content
	 */
	public function render() {
		global $controller;
		if (!isset($this->action)) $this->action = self::ACTION_VIEW;

		$this->jte = new BFJTE();
		$this->jte->assign('templates', BF_ROOT . '/public/fullcontrol/modules/Servers/templates');
		$this->handleAction();

		// Nobody else needs this
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->jte->register_object('editable', $this->serverauth, null, false);
			$this->nav_content = '';
		}

		$this->jte->assign('reference_count', count($this->references));
		$this->jte->render($this->chooseTemplate());
		$this->content = $this->jte->_jTE_parser;
	}
	
	/**
	 * Simple method to choose the template based on the request type and action sent
	 */
	protected function chooseTemplate() {
		if ($this->request_type == BF_REQUEST_XHTML) {
			return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/xhtml.serverauth.view.tpl';
		} else {
			return BF_ROOT . '/public/fullcontrol/modules/Servers/templates/' . $this->request_type . '.' . $this->action . '.tpl';
		}
	}

	/**
	 * Call different methods depending on the action in the request
	 */
	protected function handleAction() {
		switch ($this->action) {
			case self::ACTION_EDIT:
				$this->edit();
				$this->frontPageReport();
				break;
			case self::ACTION_VIEW:
			default:
				$this->frontPageReport();
		}
	}

	/**
	 * The includes referred to here actually mean linked files in the XHTML templates.
	 * Obviously only used for XHTML requests (for now, but could get used for XML and others)
	 * this particular implementation tells the template to include a module-specific
	 * stylesheet and JavaScript file.
	 */
	public function getIncludes() {
		return array(
					 'javascripts' => array(BF_WEBROOT . 'fullcontrol/modules/Servers/javascripts/serverauth.content.js')
				 );
	}

	/**
	 * Constructs the navigation block.  It basically gloms together a bunch of queries and their outputs
	 * into a navigation XML block.  (A BUNCH of queries.  No way around it for this form of navigation.)
	 */
	public function navigationBlock($id = 0)  {
		if (!$id)  {
			$auth = $this->serverauth;
		}  else {
			$auth = new BFServerAuth();
			$auth->select($auth->primary_key, $id);
		}
		
		$id = $auth->get($auth->primary_key);
		if (!$id)  {
			return '';
		}

		$nav = array(
			'servers' => bf_get_nav_objects('servers', array('=', 'bf_servers', 'bf_authid', $id)),
			'access groups' => bf_get_nav_objects('access groups', array('=', 'bf_levels', 'bf_level', 
					$auth->get('bf_level')))
		);

		return ($this->request_type == BF_REQUEST_XHTML) ? bf_output_nav_xhtml($nav) : bf_output_nav_xml($nav);
	}

	public function load($id) {
		$this->serverauth->select($this->serverauth->primary_key, $id);
		$this->references = $this->serverauth->getReferences();
	}
}

?>
