<?php
/*
(c)2003-2006, BuildForge, Inc. All rights reserved. BuildForge, Inc.
owns the copyright and other intellectual property rights in this
software.
Duplication or use of the Software is not permitted except as expressly
provided in a written agreement between your company and BuildForge,
Inc.
*/

/**
 * Servers
 *
 * @package Console
 * @subpackage Servers
 */

$GLOBALS['controller']->require_file('Viewable.php');
$GLOBALS['controller']->require_file('Module.php');

$GLOBALS['BF_ADMIN_KEYS'] = array(
	'ACTION_SERVERS'	=> array(	'match' => '/servers\.[\w.]+/',	
									'class' => 'Agents',	
									'label' => translate('agents'),	
									'default' => 'servers.view',	
									'primary' => true,			
									'display' => false),

	'ACTION_SELECTORS'	=> array(	'match' => '/selectors\.[\w.]+/',	
									'class' => 'Selectors',	
									'label' => translate('selectors'),	
									'default' => 'selectors.view',	
									'primary' => false,			
									'display' => true),

	'ACTION_COLLECTORS'	=> array(	'match' => '/collectors\.[\w.]+/',	
									'class' => 'Collectors',	
									'label' => translate('collectors'),	
									'default' => 'collectors.view',	
									'primary' => false,			
									'display' => (($GLOBALS['controller']->configuration->styx())?false:true)),

	'ACTION_SERVERAUTH'	=> array(	'match' => '/serverauth\.[\w.]+/',	
									'class' => 'ServerAuth',	
									'label' => translate('server auth'),	
									'default' => 'serverauth.view',	
									'primary' => false,			
									'display' => (($GLOBALS['controller']->configuration->styx())?false:true))
);

class Servers extends Module implements ViewableXHTML {
	protected $acceptable_request_types = array(BF_REQUEST_XHTML, BF_REQUEST_XML);
	protected $navigation_jte;
	protected $current_admin_key;
	public $action;
	protected $request_type;
	
	public $submodules;

	public function handlesRequestType($req) {
		return (array_search($req, $this->acceptable_request_types) !== false);
	}

	/**
	 * Woo-hoo, Factory-friendly and I didn't even mean to!
	 * Since Servers really covers four modules worth of functionality, we've broken them up into
	 * four modules that can each get returned based on the request.
	 */
	public static function handleRequest($req, $type = BF_REQUEST_XHTML, $post = null) {
		$admin = new Servers();
		$admin->setRequest($req, $type, $post);
		return $admin;
	}

	public function setRequest($req, $type, $post = null) {
		global $GLOBAL_SETTINGS;


		// Get the action out
		if (array_key_exists($GLOBAL_SETTINGS['site']['actionkey'], $req)) {
			$this->action = $req[$GLOBAL_SETTINGS['site']['actionkey']];
		} else {
			$this->action = $GLOBALS['BF_ADMIN_KEYS']['ACTION_SERVERS']['default'];
		}
		$this->request_type = $type;

		// Okay, so fetch the classname from the request and SAFELY use it based on the global
		if ($class = self::parseRequestedClass($req)) {

			if ($this->controller->require_file(BF_PRODUCT_ROOT . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'Servers' . DIRECTORY_SEPARATOR . $class . '.php')) {
				// TODO: fix PHP so we can just do $class::handleRequest($req, $type); - I f@#$ing hate eval();
				$this->mod = eval('return ' . $class . '::handleRequest($req, $type, $post);');

				// What did we navigate to again?
				foreach ($GLOBALS['BF_ADMIN_KEYS'] as $key => $info) {
					if ($info['class'] == $class) {
						$this->current_admin_key = $key;
						break;
					}
				}
			}
		} else {
			throw new BFException(translate('invalid request'));
		}

	}

	/**
	 * Scan the incomming request for an action that matches one of the Admin keys
	 */
	private static function parseRequestedClass($req) {
		global $BF_ADMIN_KEYS;
		if (array_key_exists('action', $req)) {
			foreach ($BF_ADMIN_KEYS as $key => $attr) {
				if (preg_match($attr['match'], $req['action'])) {
					return $attr['class'];
				}
			}
		// Default to Agents
		} else {
			return 'Agents';
		}

		return false;
	}

	public function getNavigation() {
		return false;
	}

	/**
	 * Passing the buck
	 */
	public function render() {
		$this->mod->render();
		$this->content =& $this->mod->content;
		if ($this->request_type == BF_REQUEST_XHTML) {
			$this->renderNavigation();
		}
	}

	private function renderNavigation() {
		$this->nav_content = $this->mod->navigationBlock();		
	}

	/**
	 * Passing the buck
	 */
	public function getIncludes() {
		return $this->mod->getIncludes();
	}

	/**
	 * Return the cacheable period in minutes
	 */
	public function getTTL() {
		return $this->mod->getTTL();
	}
	
	/**
	 * Get the stamp from the last (chronologically speaking) message and return it.
	 */
	public function getLastBuildDate() {
		return $this->mod->getLastBuildDate();
	}
	
	public function __construct() {
		parent::__construct();
		$this->submodules = $GLOBALS['BF_ADMIN_KEYS'];
	}
}

?>
