/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */
package com.buildforge.buildmodel.core.internal;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import com.buildforge.buildmodel.core.CorePlugin;
import com.buildforge.buildmodel.core.IBuildRecord;
import com.buildforge.buildmodel.core.exception.BuildForgeAuthenticationException;
import com.buildforge.buildmodel.core.util.Log;


public class Builds extends BuildForgeConnection {
	
	public Object[] getAllBuildRecords (int projectId) {
		ArrayList completeBuildIds = (ArrayList) getBuildByProject(projectId);
		if (isHasError()) {
			// throw exception?
			Log.errorLogMessage(Messages.getString("Builds.BUILD_ID_RETRIEVAL_ERROR_CONS") + projectId + "] : " + getErrorMessage(),  //$NON-NLS-1$ //$NON-NLS-2$
					Log.ERROR, this.getClass().getName(), 
					CorePlugin.getDefault().getBundle().getSymbolicName());
			return new IBuildRecord[] { };
		}
		IBuildRecord[] results = new IBuildRecord[completeBuildIds.size() - 1];
		for (int i = 1; i < completeBuildIds.size(); i++) {
			Hashtable element = (Hashtable) completeBuildIds.get(i);
			int bid = 0;
			int pid = 0;
			try {
				bid = Integer.parseInt((String) element.get("bf_bid")); //$NON-NLS-1$
				pid = Integer.parseInt((String) element.get("bf_pid")); //$NON-NLS-1$
			} catch (NumberFormatException nfe) {
				Log.trace("Error converting bf_bid to an integer : " + nfe.getMessage(), getClass().getName()); //$NON-NLS-1$
			}
				results[i - 1] = new BuildRecord((String) element.get("bf_tag"),  //$NON-NLS-1$
												bid,
												pid,
												(String) element.get("bf_state"), //$NON-NLS-1$
												(String) element.get("bf_username"),  //$NON-NLS-1$
												(String) element.get("bf_result") //$NON-NLS-1$
												);

		}
		return results;
	}
		
	
	public List getStepListResults(int bid, int pid) {
		StringBuffer content = new StringBuffer();
		
		String vers = getConsoleVersion();
		if (vers != null && !"".equals(vers)) {
			content.append("&Command=" + String.valueOf(hexEncode("PluginListStepStatus"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&BID=" + String.valueOf(hexEncode(String.valueOf(bid))));
		} else { 
			String clause = "WHERE r.bf_bid=" + bid + " ORDER BY r.bf_sid"; //$NON-NLS-1$ //$NON-NLS-2$
			String fields = "bf_bid, bf_sid, bf_description, bf_result";		
			content.append("&Command=" + String.valueOf(hexEncode("APIQueryRecords")));
			content.append("&Table=" + String.valueOf(hexEncode("bf_results r"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Fields=" + String.valueOf(hexEncode(fields))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Clause=" + String.valueOf(hexEncode(clause))); //$NON-NLS-1$
		}
		
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}
		ArrayList responseList = (ArrayList) processResponseRecords(response);
		
		return responseList;		
	}
	
	
	// Used for 3.8 Consoles
	public List getStepListInfoResults(int bid, int sid) {
		StringBuffer content = new StringBuffer();
		String clause = "WHERE bf_bid='" + bid + "' AND bf_sid='" + sid + "' ORDER BY bf_sid, bf_lid"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		
		content.append("&Command=" + String.valueOf(hexEncode("APIQueryRecords"))); //$NON-NLS-1$ //$NON-NLS-2$
		content.append("&Table=" + String.valueOf(hexEncode("bf_logs"))); //$NON-NLS-1$ //$NON-NLS-2$
		content.append("&Fields=" + String.valueOf(hexEncode("bf_lid, bf_stamp, bf_message"))); //$NON-NLS-1$ //$NON-NLS-2$
		content.append("&Clause=" + String.valueOf(hexEncode(clause))); //$NON-NLS-1$
		
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}
		ArrayList responseList = (ArrayList) processResponseRecords(response);
		
		return responseList;
		
	}
	
	//Used for 7.0+ Consoles
	public List getStepListInfoResults(String logId, int bid) {
		StringBuffer content = new StringBuffer();
		content.append("&Command=" + String.valueOf(hexEncode("PluginListStepDetails"))); //$NON-NLS-1$ //$NON-NLS-2$
		content.append("&LID=" + String.valueOf(hexEncode(logId)));
		content.append("&BID=" + String.valueOf(hexEncode(Integer.toString(bid))));
		
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}
		ArrayList responseList = (ArrayList) processResponseRecords(response);
		
		return responseList;
		
	}
		
	public List getBuildByProject(int projectId) {
		StringBuffer content = new StringBuffer();
		
		String vers = getConsoleVersion();
		if (vers != null && !"".equals(vers)) {
			content.append("&Command=" + String.valueOf(hexEncode("PluginListBuilds"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&PID=" + String.valueOf(hexEncode(String.valueOf(projectId))));
		} else {
			String clause = "WHERE b.bf_process='B' AND b.bf_pid=" + projectId + " AND b.bf_userid=u.bf_userid"; //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Command=" + String.valueOf(hexEncode("APIQueryRecords")));
			content.append("&Table=" + String.valueOf(hexEncode("bf_builds b, bf_users u"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Fields=" + String.valueOf(hexEncode("b.*, u.bf_name as bf_username"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Clause=" + String.valueOf(hexEncode(clause))); //$NON-NLS-1$ */
		}
		
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}
		
		ArrayList responseList = (ArrayList) processResponseRecords(response);
	
		return responseList;
	}


}


