/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */
package com.buildforge.buildmodel.core.internal;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Hashtable;

import com.buildforge.buildmodel.core.IBuildRecord;
import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.IProjectNode;
import com.buildforge.buildmodel.core.IStep;
import com.buildforge.buildmodel.core.util.Log;


public class ProjectNode implements IProjectNode, Serializable {
	private int projectID;
	private String name = null;
	private IConsole parent = null;
	private IBuildRecord[] buildRecs;
	private ArrayList steps = null;
	
	public ProjectNode() {
		super();
	}
	
	public ProjectNode(int projectID, String name, IConsole parent) {
		this.projectID = projectID;
		this.name = name;
		this.parent = parent;
	}

	public void getBuildIdRecords() {
		if (parent == null) {
			return;
		}
		Builds build = new Builds();
		build.setUserName(parent.getUser());
		build.setHost(parent.getHost());
		build.setPassword(parent.getPassword());
		if (!"".equals(parent.getLdapDomain())) {
			build.setDomain(parent.getLdapDomain());
		}
		build.setToken(parent.getAuthenticationTokens());
		
		setBuildRecs((IBuildRecord[]) build.getAllBuildRecords(projectID));
		for (int i = 0; i < buildRecs.length; i++) {
			buildRecs[i].setParent(this);
		}
		if (build.isAuthError()) {
			parent.setConnected(false);
		}
	}
	
	public IBuildRecord getBuildRecordById(int buildId) {
		for (int i = 0; i < buildRecs.length; i++) {
			if (buildRecs[i].getBuildId() == buildId) {
				return buildRecs[i];
			}
		}
		return null;
	}
		
	/* (non-Javadoc)
	 * @see com.buildforge.internal.IProjectNode#getName()
	 */
	public String getName() {
		return name;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IProjectNode#setName(java.lang.String)
	 */
	public void setName(String name) {
		this.name = name;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IProjectNode#getParent()
	 */
	public IConsole getParent() {
		return parent;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IProjectNode#setParent(com.buildforge.buildmodel.core.IConsole)
	 */
	public void setParent(IConsole parent) {
		this.parent = parent;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IProjectNode#getProjectID()
	 */
	public int getProjectID() {
		return projectID;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IProjectNode#setProjectID(int)
	 */
	public void setProjectID(int projectID) {
		this.projectID = projectID;
	}
	
	public String toString() {
		return name;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IProjectNode#getBuildRecs()
	 */
	public IBuildRecord[] getBuildRecs() {
		getBuildIdRecords();
		return buildRecs;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IProjectNode#setBuildRecs(com.buildforge.internal.BuildRecord[])
	 */
	public void setBuildRecs(IBuildRecord[] buildRecs) {
		this.buildRecs = buildRecs;
	}
	
	public IStep[] getSteps() {
		if (parent == null)
			return null;
		
		Projects projects = new Projects();
		projects.setUserName(parent.getUser());
		projects.setHost(parent.getHost());
		projects.setPassword(parent.getPassword());
		projects.setDomain(parent.getLdapDomain());
		projects.setToken(parent.getAuthenticationTokens());
		String vers = parent.getVersion();
		projects.setConsoleVersion(vers);
		
		ArrayList stepsList = (ArrayList) projects.listSteps(projectID);
		if (steps == null)
			steps = new ArrayList();
		steps.clear();
		
		for (int i = 0; i < stepsList.size(); i++) {
			if (((Object) stepsList.get(i)) instanceof Hashtable) {
				Hashtable element = (Hashtable) stepsList.get(i);
				// store valid projects as children to this server.
				if ((String) element.get("bf_description") != null) {
					//Log.trace("Steps for projectId [" + projectID + "] : " + element.get("bf_sid"), this.getClass().getName());
					if (vers == null || "".equals(vers)) {
						steps.add(new Step(Integer.parseInt((String) element.get("bf_sid")),
							(String) element.get("bf_sid"),
							(String) element.get("bf_description"), this));
					} else {
						steps.add(new Step( Integer.parseInt((String) element.get("bf_sid")),
								(String) element.get("bf_uid"),
								(String) element.get("bf_description"), this));
					}
				}
			}
		}
		IStep[] results = new IStep[steps.size()];
		for (int i = 0; i < steps.size(); i++) {
			results[i] = (IStep) steps.get(i);
		}
		
		return results;
	}

}
