/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */
package com.buildforge.buildmodel.core.internal;

import java.net.HttpURLConnection;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.buildforge.buildmodel.core.util.Log;



public class Projects extends BuildForgeConnection {
	HttpURLConnection httpClient = null;
	String orderName = " ORDER BY bf_description"; //$NON-NLS-1$
	
	public List listProjects() {
		return listProjects("", "WHERE bf_platform != '' " + orderName); //$NON-NLS-1$
	}
	
	public List listProjects(int id) {
		return listProjects("ID", "WHERE bf_pid='" + id + "'"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	public List listProjects(String title) {
		return listProjects("TITLE", "WHERE bf_description='" + title + "'"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	public List listProjects(String type, String clause) {
		String whereClause = clause;
		StringBuffer content = new StringBuffer();
		String vers = getConsoleVersion();
		if (vers != null && !"".equals(vers)) {
			Log.trace("Newer method called for listProjects since version is [" + vers + "]", getClass().getName());
			content.append("&Command=" + String.valueOf(hexEncode("PluginListProjects"))); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			Log.trace("*** Older method called since version is not obtained\n", getClass().getName());
			content.append("&Command=" + String.valueOf(hexEncode("APIQueryRecords")));
			content.append("&Table=" + String.valueOf(hexEncode("bf_projects"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Fields=" + String.valueOf(hexEncode("*"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Clause=" + String.valueOf(hexEncode(whereClause))); //$NON-NLS-1$
		}
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}
		Log.trace("Pre-processed response from listProjects :\n[" + response + "]\nError Message [" + getErrorMessage() + "]\nInfo Message [" + getInfoMessage() + "]\n", getClass().getName());
		return processResponseRecords(response);
	}
	
	public int executeQuickBuild (int pid) {
		StringBuffer content = new StringBuffer();
		int buildID = 0;
		String buildTag = ""; //$NON-NLS-1$
		
		content.append("&Command=" + String.valueOf(hexEncode("FireBuild")));	 //$NON-NLS-1$ //$NON-NLS-2$
		content.append("&Project=" + String.valueOf(hexEncode(String.valueOf(pid)))); //$NON-NLS-1$
		
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}
		
		ArrayList responseList = (ArrayList) processResponseRecords(response);
		Pattern buildIDregx = Pattern.compile(Messages.getString("Projects.BUILD_ID_LOOKUP_STRING")); //$NON-NLS-1$
		Matcher m = buildIDregx.matcher(""); //$NON-NLS-1$
		
		for (int i = 0; i < responseList.size(); i++) {
			Hashtable element = (Hashtable) responseList.get(i);
			for(Iterator it = element.keySet().iterator(); it.hasNext(); ) {
				String currKey = (String) it.next();
				String currVal = (String) element.get(currKey);
				if ("_info".equals(currKey)) { //$NON-NLS-1$
					m.reset(currVal);
					if (m.find()) {
						buildTag = m.group(1);
						// System.out.println("buildTag : " + buildTag);
						buildID = Integer.parseInt(m.group(2));
					}
				}
			}
		}
		
		if (isHasError()) {
			return -1;
		}
		
		return buildID;
	}
	
/*
	public EnvList getEnvironmentsForRun (int pid) {
		return EnvList.getProjectRunList(pid);
	}
*/
	
	public List listSteps(int pid) {
		StringBuffer content = new StringBuffer();
		String vers = getConsoleVersion();
		if (vers != null && !"".equals(vers)) {
			content.append("&Command=" + String.valueOf(hexEncode("PluginListSteps"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&PID=" + String.valueOf(hexEncode(String.valueOf(pid))));
		} else {
			String whereClause = "where bf_pid=" + pid + " order by bf_sid"; //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Command=" + String.valueOf(hexEncode("APIQueryRecords")));
			content.append("&Table=" + String.valueOf(hexEncode("bf_steps"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Fields=" + String.valueOf(hexEncode("*"))); //$NON-NLS-1$ //$NON-NLS-2$
			content.append("&Clause=" + String.valueOf(hexEncode(whereClause))); //$NON-NLS-1$
		}
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}
	
		return processResponseRecords(response);
	}

	public List listProjectEnvironment(int projectId) {
		StringBuffer content = new StringBuffer();
		String vers = getConsoleVersion();
		if (vers != null && !"".equals(vers)) {
			content.append("&Command=" + String.valueOf(hexEncode("PluginListProjEnv")));
			content.append("&PID=" + String.valueOf(hexEncode(String.valueOf(projectId))));
		} else {
			String whereClause = "WHERE p.bf_eid=e.bf_bid AND p.bf_pid=" + projectId + " order by bf_eid";
			content.append("&Command=" + String.valueOf(hexEncode("APIQueryRecords")));
			content.append("&Table=" + String.valueOf(hexEncode("bf_projects p, bf_evn e")));
			content.append("&Fields=" + String.valueOf(hexEncode("e.bf_eid, e.bf_level, e.bf_param, e.bf_value, e.bf_action, e.bf_mode")));
			content.append("&Clause=" + whereClause);
		}
		
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}

		return processResponseRecords(response);
	}

	public int executeBuild(int pid, EnvironmentGroup env) {
		StringBuffer content = new StringBuffer();
		int buildID = 0;
		String buildTag = ""; //$NON-NLS-1$
		
		Log.trace("***** FIRING BUILD OFF ******", this.getClass().getName());
		content.append("&Command=" + String.valueOf(hexEncode("FireBuild")));	 //$NON-NLS-1$ //$NON-NLS-2$
		content.append("&Project=" + String.valueOf(hexEncode(String.valueOf(pid)))); //$NON-NLS-1$
		
		ArrayList envList = (ArrayList) env.getList();
		if (envList.size() > 0) {
			StringBuffer subcontent = new StringBuffer();
			content.append("&AddEnv=");
			for (int i = 0; i < envList.size(); i++) {
				Environment el = (Environment) envList.get(i);
				subcontent.append(el.getAction() + ":" + el.getName() + ":" + el.getChoice() + "\n");
			}
			Log.trace("Environment Block For requested build : " + subcontent.toString(), this.getClass().getName());
			content.append(String.valueOf(hexEncode(subcontent.toString())));
		}
		
		String response = postQuery(content.toString());
		if (repost) {
			response = postQuery(content.toString());
			repost = false;
		}

		
		ArrayList responseList = (ArrayList) processResponseRecords(response);
		Pattern buildIDregx = Pattern.compile(Messages.getString("Projects.BUILD_ID_LOOKUP_STRING")); //$NON-NLS-1$
		Matcher m = buildIDregx.matcher(""); //$NON-NLS-1$
		
		for (int i = 0; i < responseList.size(); i++) {
			Hashtable element = (Hashtable) responseList.get(i);
			for(Iterator it = element.keySet().iterator(); it.hasNext(); ) {
				String currKey = (String) it.next();
				String currVal = (String) element.get(currKey);
				if ("_info".equals(currKey)) { //$NON-NLS-1$
					m.reset(currVal);
					if (m.find()) {
						buildTag = m.group(1);
						// System.out.println("buildTag : " + buildTag);
						buildID = Integer.parseInt(m.group(2));
					}
				}
			}
		}
		
		if (isHasError()) {
			return -1;
		}
		
		return buildID;
	}


}
