/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.core.internal;

import java.util.ArrayList;
import java.util.Hashtable;

import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.IStepDetailInfo;
import com.buildforge.buildmodel.core.IStepDetailRecord;
import com.buildforge.buildmodel.core.IStepLogInfo;
import com.buildforge.buildmodel.core.util.Log;

public class StepDetailRecord implements IStepDetailRecord {

	private int buildId;
	private int stepId; 
	private String logId;
	private String stepName = null;
	private IStepDetailInfo details = null;
	private IStepLogInfo logInfo = null;
	private char resultsStatus;
	private boolean is40plus = false;
	
	public char getResultsStatus() {
		return resultsStatus;
	}

	public void setResultsStatus(char resultsStatus) {
		this.resultsStatus = resultsStatus;
	}
	
	// Constructor used for 3.8
	public StepDetailRecord (int buildId, int stepId, String stepName, String[] details, String[] info, char resultsStatus) {
		this.buildId = buildId;
		this.stepId = stepId;
		this.stepName = stepName;
		if (details != null)
			this.details = new StepDetailInfo(details, this);
		if (logInfo != null)
			this.logInfo = new StepLogInfo(info, this);
		this.resultsStatus = resultsStatus;
	}

	// 4.0 constructor
	public StepDetailRecord (int buildId, int stepId, String stepName, String[] details, String[] info, char resultsStatus, String logId) {
		this(buildId, stepId, stepName, details, info, resultsStatus);
		this.logId = logId;
		is40plus = true;
	}
	
	public void populateLogInfo(IConsole cons) {
		Log.trace("In StepDetailRecord: populateLogInfo()  console used is : " + cons.getName(), getClass().getName());
		Builds build = new Builds();
		build.setUserName(cons.getUser());
		build.setHost(cons.getHost());
		build.setPassword(cons.getPassword());
		if (!"".equals(cons.getLdapDomain())) {
			build.setDomain(cons.getLdapDomain());
		}
		build.setToken(cons.getAuthenticationTokens());
		
		ArrayList info;
		if (is40plus) {
			info = (ArrayList) build.getStepListInfoResults(getLogId(), getBuildId());
		} else {			
			info = (ArrayList) build.getStepListInfoResults(getBuildId(), getStepId());
		}
		
		
		ArrayList logDetail = new ArrayList();
		ArrayList logInfo = new ArrayList();
		boolean isDetail = true;
				
		for (int i=1; i < info.size(); i++) {
			Hashtable element = (Hashtable) info.get(i);
			// lineId and timestamp aren't really used.  
			// Just leaving them in comments because they are there.
			// int lineId = Integer.parseInt((String) element.get("bf_lid"));
			// String timestamp = (String) element.get("bf_stamp");
			String msg = (String) element.get("bf_message");
			if (!is40plus) {
				if (isDetail) {
					if ((msg.length() > 5) && ("EXEC:".equals(msg.substring(0,5)))) {
						isDetail = false;
						logInfo.add((String) msg);
					} else {
						logDetail.add((String) msg);
					}
				} else {
					logInfo.add((String) msg);
				}
			} else {
				String type = (String) element.get("bf_type");
				String lid = (String) element.get("bf_lid");
				if (isDetail) {
					if ("EXEC".equals(type)) {
						isDetail = false;
						logInfo.add(msg);
					} else {
						logDetail.add(type + ": " + msg);
					}
				} else {
					logInfo.add(msg);
				}
			}
			
		}
		
		String[] details = new String[logDetail.size()];
		for (int j=0; j < logDetail.size(); j++) {
			details[j] = (String) logDetail.get(j);
		}
		String[] stepInfo = new String[logInfo.size()];
		for (int j=0; j < logInfo.size(); j++) {
			stepInfo[j] = (String) logInfo.get(j);
		}
		
		setLogInfo((IStepLogInfo) new StepLogInfo(stepInfo, this));
		StepDetailInfo detailInfo = new StepDetailInfo(details, this);
		detailInfo.generateTypeLists();
		setDetails((IStepDetailInfo) detailInfo);
		

	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#getBuildId()
	 */
	public int getBuildId() {
		return buildId;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#setBuildId(int)
	 */
	public void setBuildId(int buildId) {
		this.buildId = buildId;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#getDetails()
	 */
	public IStepDetailInfo getDetails() {
		return details;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#setDetails(java.lang.String)
	 */
	public void setDetails(IStepDetailInfo details) {
		this.details = details;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#getStepId()
	 */
	public int getStepId() {
		return stepId;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#setStepId(int)
	 */
	public void setStepId(int stepId) {
		this.stepId = stepId;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#getLogInfo()
	 */
	public IStepLogInfo getLogInfo() {
		return logInfo;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#setLogInfo(java.lang.String)
	 */
	public void setLogInfo(IStepLogInfo logInfo) {
		this.logInfo = logInfo;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#getStepName()
	 */
	public String getStepName() {
		return stepName;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailRecord#setStepName(java.lang.String)
	 */
	public void setStepName(String stepName) {
		this.stepName = stepName;
	}
	
	public String toString() {
		return stepName;
	}

	public String getLogId() {
		return logId;
	}

	public void setLogId(String logId) {
		this.logId = logId;
	}
	
}
