/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.reflector.internal;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.multipart.FilePart;
import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
import org.apache.commons.httpclient.methods.multipart.Part;
import org.apache.commons.httpclient.methods.multipart.StringPart;

import com.buildforge.buildmodel.core.internal.BuildForgeConnection;
import com.buildforge.buildmodel.core.internal.IConstants;
import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.reflector.Messages;

public class HttpFilePoster extends BuildForgeConnection implements IConstants {
	private String formPath = API_URL_APPEND_STRING;
	private String formItemName = "zipfile[]"; //$NON-NLS-1$
	private boolean filePostError = false;
	private Map params;
	
	public HttpFilePoster (String user, String password, String host) {
		super();
		this.userName = user;
		this.password = password;
		this.host = host;
		params = new HashMap();
	}
	
	public void setParameter(String name, String value) {
		if ("token".equals(name) && value != null) { //$NON-NLS-1$
			value = decodeString(value);
		}
		params.put(name, value);
	}
	
	
	public String postFormData(String url, File[] files) {
		String results = ""; //$NON-NLS-1$
		
		PostMethod filePost = new PostMethod(url);
		filePost.setContentChunked(true);
		
		Part[] parts = new Part[files.length + params.size()];
		try {
			int i;
			for (i = 0; i < files.length; i++) {
				parts[i] = new FilePart(formItemName, files[i].getName(), files[i]);
			}
			
			Set keys = params.keySet();
			for(Iterator keyIt = keys.iterator(); keyIt.hasNext();i++) {
				String name = (String) keyIt.next();
				parts[i] = new StringPart(name, (String) params.get(name), "UTF-8");
				//filePost.getParams().setParameter(name, (String) params.get(name));
			}

			
		} catch (FileNotFoundException fnf) {
			Log.trace("Problem posting Archived file.  Couldn't Find the archive. " + fnf.getMessage(), getClass().getName()); //$NON-NLS-1$
		}
		filePost.setRequestEntity(new MultipartRequestEntity(parts, filePost.getParams()));
		
		HttpClient client = new HttpClient();
        //client.getHttpConnectionManager().getParams().setConnectionTimeout(5000);
        
        try {
		    int status = client.executeMethod(filePost);
		    if (status == HttpStatus.SC_OK) {
		    	results = filePost.getResponseBodyAsString();
		        Log.trace("response=" + filePost.getResponseBodyAsString(), getClass().getName()); //$NON-NLS-1$
		    } else {
		        Log.trace("failed! response=" + HttpStatus.getStatusText(status), getClass().getName()); //$NON-NLS-1$
		    }
        } catch (IOException ioe) {
        	Log.trace(ioe.getMessage(), getClass().getName());
        }
        
		return results;
	}

	public String getFormItemName() {
		return formItemName;
	}

	public void setFormItemName(String formItemName) {
		this.formItemName = formItemName;
	}

	public String getFormPath() {
		return formPath;
	}

	public void setFormPath(String formPath) {
		this.formPath = formPath;
	}

	public boolean isFilePostError() {
		return filePostError;
	}

	public void setFilePostError(boolean filePostError) {
		this.filePostError = filePostError;
	} 
	
}
