/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.reflector.internal;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;

import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.reflector.BuildForgeProjectNature;
import com.buildforge.buildmodel.reflector.Messages;
import com.buildforge.buildmodel.reflector.ReflectorPlugin;

public class NatureToggle {
	private String nature_id = BuildForgeProjectNature.NATURE_ID;
	
	public void addBuildForgeNature(IProject project) {
		Log.trace("Adding Build Forge Nature", this.getClass().getName()); //$NON-NLS-1$
		
		try {
			IProjectDescription description = project.getDescription();
			String[] natures = description.getNatureIds();
			for (int i = 0; i < natures.length; i++) {
				if (nature_id.equals(natures[i])) {
					Log.trace("Nature already exists in this project.", getClass().getName()); //$NON-NLS-1$
					return;
				}
			}
			String[] newNatures = new String[natures.length + 1];
			System.arraycopy(natures, 0, newNatures, 0, natures.length);
			newNatures[natures.length] = nature_id;
			IStatus status = project.getWorkspace().validateNatureSet(natures);
			
			if (status.getCode() == IStatus.OK) {
				description.setNatureIds(newNatures);
				project.setDescription(description, null);
			} else {
				Log.trace("Error setting Nature!!!", getClass().getName()); //$NON-NLS-1$
			}
		} catch (CoreException ce) {
			ce.printStackTrace();
			Log.errorLogMessage(Messages.getString("NatureToggle.ADD_NATURE_ERROR_CONS") + nature_id + "] : " + ce.getMessage(), Log.ERROR,  //$NON-NLS-1$ //$NON-NLS-2$
					this.getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
		} 
 	}  

	public void removeBuildForgeNature(IProject project) {
		Log.trace("Removing Build Forge Nature", this.getClass().getName()); //$NON-NLS-1$
		
		try {
			IProjectDescription description = project.getDescription();
			String[] natures = description.getNatureIds();
			int location = 0;
			
			for (int i = 0; i < natures.length; i++) {
				if (nature_id != null && nature_id.equals(natures[i])) {
					location = i;
				}
			}
			
			if (location != -1) {
				String[] newNatures = new String[natures.length -1];
				System.arraycopy(natures, 0, newNatures, 0, location);
				System.arraycopy(natures, location + 1, newNatures, location, newNatures.length - location);
				project.setDescription(description, new NullProgressMonitor());
			}
			
		} catch (CoreException ce) {
			Log.errorLogMessage(Messages.getString("NatureToggle.PROJECT_DESCRIPTION_ERROR_CONS") + ce.getMessage(), Log.ERROR,  //$NON-NLS-1$
					this.getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());			
		}
		
	}

}
