/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.reflector.internal;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.reflector.Messages;
import com.buildforge.buildmodel.reflector.ReflectorPlugin;

public class Packager {
	private static final int MAX_BUFFER = 10240;
	private boolean fileNotFound = false;
	private String zipFile;
	private String[] fileNames;
	private String baseDir;
	
	public Packager () {
		super();
	}
	
	public Packager (String[] fileNames, String zipFile, String baseDir) {
		this.zipFile = zipFile;
		this.fileNames = fileNames;
		this.baseDir = baseDir;
		
	}
	
	public static void zipFiles (String[] fileNames, String zipFile, String baseDir, String workingDir) {
		Packager pkgr = new Packager(fileNames, zipFile, baseDir);
		pkgr.createZip();
	}
	
	public void createZip () {
		File[] files = new File[fileNames.length];
		String[] filesName = new String[fileNames.length];
		byte[] buffer = new byte[MAX_BUFFER];
		
		for (int i = 0; i < fileNames.length; i++) {
			files[i] = getFileFromPath(baseDir + fileNames[i]);
			Log.trace(fileNames[i] + " processed.", getClass().getName()); //$NON-NLS-1$
			filesName[i] = new String("." + fileNames[i]); //$NON-NLS-1$
		}
		
		if (fileNotFound) {
			Log.trace("Not all files in the list to archive exist!", getClass().getName()); //$NON-NLS-1$
			Log.errorLogMessage(Messages.getString("Packager.MISSING_FILES_TO_ARCHIVE_ERROR_CONS"),  //$NON-NLS-1$
					Log.INFO, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
		}
		
		File archive = new File(zipFile);
		try {
			FileOutputStream stream = new FileOutputStream(archive);
			ZipOutputStream out = new ZipOutputStream(stream);
			
			for(int i=0; i < files.length; i++) {
				if (files[i] == null || !files[i].exists() || files[i].isDirectory()) 
					continue;
				Log.trace("Adding " + filesName[i] + " to Archive " + zipFile, getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
				
				ZipEntry addition = new ZipEntry(filesName[i]);
				addition.setTime(files[i].lastModified());
				out.putNextEntry(addition);
				
				FileInputStream in = new FileInputStream(files[i]);
				int buff;
				while ((buff = in.read(buffer, 0, buffer.length)) > 0) {
					out.write(buffer, 0, buff);
				}
				in.close();
			}
			out.close();
			stream.close();
			
			Log.trace("Created Archive " + zipFile, getClass().getName()); //$NON-NLS-1$
		} catch (FileNotFoundException fnf) {
			Log.errorLogMessage(Messages.getString("Packager.FILE_NOT_FOUND_ERROR_CONS") + fnf.getMessage(),  //$NON-NLS-1$
					Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
		} catch (IOException ioe) {
			Log.errorLogMessage(Messages.getString("Packager.ADD_FILE_ERROR_CONS") + ioe.getMessage(),  //$NON-NLS-1$
					Log.ERROR, getClass().getName(),ReflectorPlugin.getDefault().getBundle().getSymbolicName());
		}
	}
	
	
	private File getFileFromPath(String path) {
		File result = new File(path);
		if (result.exists()) {
			return result;
		}
		Log.trace("File [" + path + "] does not exist!", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
		Log.errorLogMessage(Messages.getString("Packager.PRE_FILE_PATH_NO_EXIST_ERROR_CONS") + path + Messages.getString("Packager.POST_FILE_PATH_NO_EXIST_ERROR_CONS"),  //$NON-NLS-1$ //$NON-NLS-2$
				Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
		fileNotFound = true;
		return null;
	}

	public String getBaseDir() {
		return baseDir;
	}

	public void setBaseDir(String baseDir) {
		this.baseDir = baseDir;
	}

	public String[] getFileNames() {
		return fileNames;
	}

	public void setFileNames(String[] fileNames) {
		this.fileNames = fileNames;
	}

	public String getZipFile() {
		return zipFile;
	}

	public void setZipFile(String zipFile) {
		this.zipFile = zipFile;
	}
	
	
}
