/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.reflector.popupmenus;

import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import org.eclipse.core.resources.IProject;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.IProjectNode;
import com.buildforge.buildmodel.core.IStep;
import com.buildforge.buildmodel.core.dialogs.JobRunDialog;
import com.buildforge.buildmodel.core.exception.BuildForgeAuthenticationException;
import com.buildforge.buildmodel.core.exception.BuildForgeException;
import com.buildforge.buildmodel.core.internal.Environment;
import com.buildforge.buildmodel.core.internal.EnvironmentGroup;
import com.buildforge.buildmodel.core.internal.EnvironmentValue;
import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.reflector.IConstants;
import com.buildforge.buildmodel.reflector.Messages;
import com.buildforge.buildmodel.reflector.ReflectorPlugin;
import com.buildforge.buildmodel.reflector.internal.HttpFilePoster;
import com.buildforge.buildmodel.reflector.internal.Packager;
import com.buildforge.buildmodel.reflector.internal.ReflectorFavoriteItem;
import com.buildforge.buildmodel.reflector.internal.ReflectorStore;


public class RemoteBuildActionDelegate implements IObjectActionDelegate, IConstants {
	private IProject project = null;
	
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		
		ISelection selection = targetPart.getSite().getSelectionProvider().getSelection();
		if (selection instanceof IStructuredSelection) {
			Object first = ((IStructuredSelection) selection).getFirstElement();
			
			if (first instanceof IProject) {
				project = (IProject) first;
				action.setText(Messages.getString("RemoteBuildActionDelegate.EXECUTE_PROJECT_ACTION_LABEL")); //$NON-NLS-1$
				action.setEnabled(false);
				if (ReflectorFavoriteItem.getBuildForgeEnabledPropertyValue(project)) {
					if (project.isAccessible()) {
						action.setEnabled(true);
					} 
				}
			} else if (first instanceof IJavaProject) {
				project = ((IJavaProject) first).getProject();
				action.setText(Messages.getString("RemoteBuildActionDelegate.EXECUTE_PROJECT_ACTION_LABEL")); //$NON-NLS-1$
				action.setEnabled(false);
				if (ReflectorFavoriteItem.getBuildForgeEnabledPropertyValue(project)) {
					if (project.isAccessible()) {
						action.setEnabled(true);
					} 
				}
			}
			
		}
		
	}

	public void run(IAction action) {
		Log.trace("Mr.Pibb + Red Vines = crazy delicious", getClass().getName()); //$NON-NLS-1$

		//get configured information for Buildforge console 
		IConsole console = ReflectorFavoriteItem.getConsoleSelection(project);
		String bfProject = ReflectorFavoriteItem.getProjectSelection(project);
		String stepId = ReflectorFavoriteItem.getStepUniqueId(project);
		
		String buildPath = ReflectorFavoriteItem.getRelativeBuildForgePath(project);
		//String[] projFiles = ReflectorFavoriteItem.getCheckedFiles(project);
		Log.trace("project [" + bfProject + "] stepId [" + stepId + "]", getClass().getName());
		ArrayList projectFiles;
		Hashtable environmentAdd;
		ArrayList envOrder;
		ReflectorStore priorStore = ReflectorPlugin.getProjectInfoByLabel(project.getProject().getFullPath().toString());
    	if (priorStore != null) {    		
    		Log.trace("Found existing ReflectorStore",getClass().getName()); //$NON-NLS-1$
    		projectFiles = (ArrayList) priorStore.getCheckTree();
    		if (projectFiles == null)
    			projectFiles = new ArrayList();
    		environmentAdd = priorStore.getEnvironment();
    		envOrder = priorStore.getEnvironmentOrder();
    		if (environmentAdd == null || envOrder == null) {
    			environmentAdd = new Hashtable();
    			envOrder = new ArrayList();
    		}
    	} else {
    		Log.trace("No prior ReflectorStore found", getClass().getName()); //$NON-NLS-1$
    		projectFiles = new ArrayList();
    		environmentAdd = new Hashtable();
    		envOrder = new ArrayList();
    	}

    	
    	String[] projFiles = new String[projectFiles.size()];
    	for (int i=0; i < projectFiles.size(); i++) {
    		projFiles[i] = (String) projectFiles.get(i);
    	}
		
		if (console == null) {
			// The console was likely deleted since configuring the eclipse project with this console.
			ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
					Messages.getString("RemoteBuildActionDelegate.PROJECT_EXECUTION_NO_CONSOLE_ERROR_DLG"), Messages.getString("RemoteBuildActionDelegate.PROJECT_EXECUTION_NO_CONSOLE_ERROR_SUB_DLG"), //$NON-NLS-1$ //$NON-NLS-2$
					Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), new BuildForgeException(Messages.getString("RemoteBuildActionDelegate.EXCEPTION_CONFIGURATION_ERROR_TEXT")), Log.ERROR)); //$NON-NLS-1$
			return;
		}
		
		try {
			String checktoken = console.getAuthenticationTokens();
			if (!console.isConnected() || checktoken == null || checktoken.length() == 0) {
				console.setAuthEnabled(true);
				console.invalidateAuthenticationTokens();
				if (console.authenticate())
					console.update();
			}
		} catch (BuildForgeAuthenticationException bae) {
			Log.trace("Error Authenticating with the buildforge console", getClass().getName()); //$NON-NLS-1$
			Log.errorLogMessage(Messages.getString("RemoteBuildActionDelegate.AUTH_ERROR_CONS") + bae.getMessage(),  //$NON-NLS-1$
					Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
			ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
					Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_DLG"), Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_SUB_DLG") + bae.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
					Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), bae, Log.ERROR));
			return;
		} catch (BuildForgeException be) {
			Log.trace("Error communicating with the buildforge console", getClass().getName()); //$NON-NLS-1$
			Log.errorLogMessage(Messages.getString("RemoteBuildActionDelegate.COMMUNICATION_ERROR_CONS") + be.getMessage(),  //$NON-NLS-1$
					Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
			ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
					Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_DLG"), Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_SUB_DLG") + be.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
					Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), be, Log.ERROR));
			return;
		}


    	// Gather the default Project Environment and display it to the user with the Added Environments from 
    	// the reflector properties page.
		
		int pId = getConsoleProjectId(console, bfProject);
		Log.trace("Project Id is [" + pId + "] : checked with project name of [" + bfProject + "] and console of [" + console.getName() + "]\n", getClass().getName());
    	
		EnvironmentGroup envGroup = mergeEnvs(console.getEnvironmentGroup(pId), environmentAdd);
		
    	/* --- inserting dialog here for dealing with Environment issues. --- */
    	JobRunDialog dialog = new JobRunDialog(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), envGroup);
    	if (dialog.open() != InputDialog.OK)
    		return;
    	EnvironmentGroup finalEnv = dialog.environment;
		
		Log.trace("Console User [" + console.getUser() + "]   Host [" + console.getHost() + "]", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		HttpFilePoster poster = new HttpFilePoster(console.getUser(), console.getPassword(), console.getHost());
		
		//get Zip files for each project involved.
		String projectBaseDir = project.getLocation().toOSString();
		String tmpZipFile = project.getPluginWorkingLocation(ReflectorPlugin.getDefault().getDescriptor()).append("zip1.zip").toOSString(); //$NON-NLS-1$
		Packager archive = new Packager(projFiles, tmpZipFile, projectBaseDir);
		Log.trace("directory for project is " + projectBaseDir, getClass().getName()); //$NON-NLS-1$
		
		archive.createZip();
		Log.trace("zip should be created at " + tmpZipFile, getClass().getName()); //$NON-NLS-1$
		
		String sId = getConsoleProjectsStepId(console, bfProject, stepId); 
		
		if (pId < 0 || "".equals(sId)) {
			// The project or step wasn't found in the configured console.
			// They may have been deleted from the buildforge project.
			String errStr = ""; //$NON-NLS-1$
			if (pId < 0)
				errStr = Messages.getString("RemoteBuildActionDelegate.PROJECT_ID_NOT_FOUND_ERROR_DLG"); //$NON-NLS-1$
			else 
				errStr = Messages.getString("RemoteBuildActionDelegate.STEP_ID_NOT_FOUND_ERROR_DLG"); //$NON-NLS-1$
			
			ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
					Messages.getString("RemoteBuildActionDelegate.PROJECT_EXECUTION_NO_CONSOLE_ERROR_DLG"), errStr, Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(),  //$NON-NLS-1$
							new BuildForgeException(Messages.getString("RemoteBuildActionDelegate.EXCEPTION_CONFIGURATION_ERROR_TEXT")), Log.ERROR)); //$NON-NLS-1$
			return;
		}
		
		//post files to the server
		Log.trace("Posting to project [" + pId + "] with step [" + sId + "] and buildPath of [" + buildPath + "].", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		poster.setParameter( "Command", REFLECTOR_COMMAND); //$NON-NLS-1$
		poster.setParameter("token",console.getAuthenticationTokens()); //$NON-NLS-1$
		poster.setParameter("plaintext", "1"); //$NON-NLS-1$ //$NON-NLS-2$
		poster.setParameter("RunProject", Integer.toString(pId)); //$NON-NLS-1$
		poster.setParameter("Client", "EclipseReflector"); //$NON-NLS-1$ //$NON-NLS-2$
		poster.setParameter(POST_PROJECT_ID + "[]", Integer.toString(pId)); //$NON-NLS-1$
		poster.setParameter(POST_STEP_ID + "[]", sId); //$NON-NLS-1$
		poster.setParameter(POST_ZIP_PATH + "[]", buildPath); //$NON-NLS-1$
		StringBuffer envPost = new StringBuffer();
		/* for(int i = 0; i < envOrder.size(); i++) {
			envPost.append("S:" + (String) envOrder.get(i) + ":" + environmentAdd.get((String) envOrder.get(i)) + "\n");
		}*/
		ArrayList envList = (ArrayList) finalEnv.getList();
		for (int i = 0; i < envList.size(); i++) {
			Environment el = (Environment) envList.get(i);
			envPost.append(el.getAction() + ":" + el.getName() + ":" + el.getChoice() + "\n");
		}
		poster.setParameter(ENVIRONMENT_ID , envPost.toString());
		
		Log.trace("Authenticated.  Posting to " + console.getHost() + "full URL [" + poster.getURL(console.getHost()), getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
		String response = poster.postFormData(poster.getURL(console.getHost()),new File[] { new File(tmpZipFile) });
		ArrayList result = (ArrayList) poster.processResponseRecords(response);
		
		String buildID = ""; //$NON-NLS-1$
		String buildTag = ""; //$NON-NLS-1$
		for (int i = 0; i < result.size(); i++) {
			Hashtable res = (Hashtable) result.get(i);
			if (res.containsKey("_error")) { //$NON-NLS-1$
				Log.errorLogMessage(Messages.getString("RemoteBuildActionDelegate.POSTING_PROJECT_ERROR_CONS") + poster.getErrorMessage(),  //$NON-NLS-1$
						Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
				ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
						Messages.getString("RemoteBuildActionDelegate.PROJECT_EXECUTION_NO_CONSOLE_ERROR_DLG"), Messages.getString("RemoteBuildActionDelegate.POSTING_FILE_ERROR_SUB_DLG") + res.get("_error"), //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), new BuildForgeException(Messages.getString("RemoteBuildActionDelegate.EXCEPTION_POSTING_ERROR_TEXT")), Log.ERROR)); //$NON-NLS-1$
				return;
			}
			if (res.containsKey("tag")) { //$NON-NLS-1$
				buildTag = (String) res.get("tag"); //$NON-NLS-1$
			}
			if (res.containsKey("BuildID")) { //$NON-NLS-1$
				buildID = (String) res.get("BuildID"); //$NON-NLS-1$
			}
		}
		
		int id = -1;
		if (!"".equals(buildID)) { //$NON-NLS-1$
			try {
				id = Integer.parseInt(buildID);
			} catch (NumberFormatException nfe) {
			}
		}
		
		if (!"".equals(buildTag) && id >= 0) { //$NON-NLS-1$
			MessageDialog.openInformation(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
					Messages.getString("RemoteBuildActionDelegate.BUILD_ID_INFO_DLG"), Messages.getString("RemoteBuildActionDelegate.BUILD_ID_IS_DLG") + buildTag); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			// bad things happened 
			ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
					Messages.getString("RemoteBuildActionDelegate.PROJECT_EXECUTION_ERROR_DLG"), Messages.getString("RemoteBuildActionDelegate.UNKNOWN_ERROR_SUB_DLG"), //$NON-NLS-1$ //$NON-NLS-2$
					Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), 
							new BuildForgeException(Messages.getString("RemoteBuildActionDelegate.EXCEPTION_ERROR_POSTING_TEXT1") + //$NON-NLS-1$
									Messages.getString("RemoteBuildActionDelegate.EXCEPTION_ERROR_POSTING_TEXT2")),  //$NON-NLS-1$
							Log.ERROR));
			Log.errorLogMessage(Messages.getString("RemoteBuildActionDelegate.EXCEPTION_POSTING_ERROR_TEXT") + poster.getErrorMessage(),  //$NON-NLS-1$
					Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
		}
		
	}

	/**
	 *  mergeEnvs is used to overlay the reflector properties Environment tab settings on top of the 
	 *  EnvironmentGroup for the project we are dealing with, pre display to the user for change 
	 *  before running.
	 * @param envGroup
	 * @param environmentAdd
	 * @return the altered EnvironmentGroup
	 */
	private EnvironmentGroup mergeEnvs(EnvironmentGroup envGroup, Hashtable environmentAdd) {
		Iterator it = environmentAdd.keySet().iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Environment envExist = envGroup.getEnv(key);
			if (envExist == null) {
				// add the entry to the environment group as it doesn't exist
				envGroup.newEnvironmentEntry(key, ((EnvironmentValue) environmentAdd.get(key)).getValue(), "S", "N");
			} else {
				// update the existing environment as the user has overriden it in reflector properties dialog
				envExist = new Environment();
				String val = "";
				try {
					val = ((EnvironmentValue) environmentAdd.get(key)).getValue();
				} catch (NullPointerException npe) { val = ""; }
				envExist.setValue(val);
				envGroup.replaceEnv(key, envExist);
			}
		}
		envGroup.orderByPosition();
		return envGroup;
	}
	

	private int getConsoleProjectId(IConsole console, String projectName) {
		int result = -1;
		if (projectName == null) 
			return result;
		
		Object[] projs = (Object[]) console.getProjects().toArray();
		
		for (int i = 0; i < projs.length; i++) {
			if (projectName.equals(((IProjectNode)projs[i]).getName())) {
				Log.trace("Found Project", getClass().getName());
				return ((IProjectNode)projs[i]).getProjectID();
			}
		}
		Log.trace("No ID for Project Found", getClass().getName());
		return result;
	}
	
	
	private String getConsoleProjectsStepId(IConsole console, String projectName, String stepUId) {
		String result = "";
		if (projectName == null || stepUId == null)
			return result;
		
		Object[] projs = (Object[]) console.getProjects().toArray();
		
		for (int i = 0; i < projs.length; i++) {
			if (projectName.equals(((IProjectNode)projs[i]).getName())) {
				IStep steps[] = ((IProjectNode)projs[i]).getSteps();
				for (int j=0; j < steps.length; j++) {
					Log.trace("[step] " + steps[j].getName() + " [id] " + steps[j].getStepId() + " [uid] " + steps[j].getStepUniqueIdentifier(), getClass().getName()); 
					if (stepUId.equals(steps[j].getStepUniqueIdentifier())) {
						Log.trace("Found Step Id", getClass().getName());
						return steps[j].getStepUniqueIdentifier();
					}
				}
			}
		}
		Log.trace("No ID for Step Found", getClass().getName());
		return result;
	}
	
	
	public void selectionChanged(IAction action, ISelection selection) {
		
	}
	
}
