/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.ui;

import org.eclipse.ui.model.IWorkbenchAdapter;
import org.eclipse.ui.plugin.*;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.core.runtime.IAdapterManager;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.osgi.framework.BundleContext;

import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.ui.adapters.ConsoleAdapterFactory;

/**
 * The main plugin class to be used in the desktop.
 */
public class UiPlugin extends AbstractUIPlugin {

	//The shared instance.
	private static UiPlugin plugin;
	
	/**
	 * The constructor.
	 */
	public UiPlugin() {
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		Log.trace("starting Build Forge UI Plugin.", this.getClass().getName());
		IAdapterManager manager = Platform.getAdapterManager();
		IAdapterFactory factory = new ConsoleAdapterFactory();
		manager.registerAdapters(factory, IWorkbenchAdapter.class);
		manager.registerAdapters(factory, IPropertySource.class);
		
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		Log.trace("Stoping Build Forge UI Plugin.", this.getClass().getName());
		plugin = null;
	}

	/**
	 * Returns the shared instance.
	 */
	public static UiPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path.
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin("com.buildforge.buildmodel.ui", path);
	}
}
