/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.ui.property;

import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PropertyPage;

import com.buildforge.buildmodel.core.CorePlugin;
import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.exception.BuildForgeException;
import com.buildforge.buildmodel.core.internal.Messages;
import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.ui.UiPlugin;
import com.buildforge.buildmodel.ui.adapters.ConsolePropertySource;


public class ConsolePropertyPage extends PropertyPage {

	private static final int TEXT_WIDTH = 50;
	
	private Text hostText;
	private Text userText;
	private Text ldapDomainText;
	private Text passwordText;
	private Text consoleNameText;
	
	protected Control createContents(Composite parent) {
		Log.trace("Console property page: createing contents.", this.getClass().getName()); //$NON-NLS-1$

		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		composite.setLayout(layout);
		
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL_VERTICAL;
		data.horizontalAlignment = GridData.FILL_HORIZONTAL;
		composite.setLayoutData(data);
		
		addConsoleNameInfo(composite);
		addSeparator(composite);
		addConsoleHostInfo(composite);
		addSeparator(composite);
		addConsoleLoginInfo(composite);
		addSeparator(composite);
		
		Log.trace("Console property page: finished creating.", this.getClass().getName()); //$NON-NLS-1$
		return composite;
	}

	private void addConsoleLoginInfo(Composite parent) {
		IConsole console = (IConsole) getElement();
		GridData gd = new GridData();
	    gd.widthHint = convertWidthInCharsToPixels(TEXT_WIDTH );
	    
		Label consoleUserLabel = new Label(parent, SWT.NONE);
		consoleUserLabel.setText(ConsolePropertySource.PROP_DISPLAY_USER);
		
		userText = new Text(parent, SWT.BORDER);
		userText.setLayoutData(gd);
	   
		String usr = console.getUser();
		userText.setText( (usr != null) ? usr : "" ); //$NON-NLS-1$
		
		GridData gd2 = new GridData();
	    gd2.widthHint = convertWidthInCharsToPixels(TEXT_WIDTH );
		
		Label consolePasswordLabel = new Label(parent, SWT.NONE);
		consolePasswordLabel.setText(ConsolePropertySource.PROP_DISPLAY_PASSWORD);
		
		passwordText = new Text(parent, SWT.BORDER | SWT.PASSWORD );
		passwordText.setLayoutData(gd2);

		String pass = (console != null) ? console.getPassword() : null;
		passwordText.setEchoChar('*');
		passwordText.setText((pass != null) ? pass : "");  //$NON-NLS-1$
		
		GridData gd3 = new GridData();
	    gd3.widthHint = convertWidthInCharsToPixels(TEXT_WIDTH );
		
		Label consoleLdapDomainLabel = new Label(parent, SWT.NONE);
		consoleLdapDomainLabel.setText(ConsolePropertySource.PROP_DISPLAY_LDAP_DOMAIN);
		
		ldapDomainText = new Text(parent, SWT.BORDER);
		ldapDomainText.setLayoutData(gd3);
		
		String ldapDomain = console.getLdapDomain();
		ldapDomainText.setText( (ldapDomain != null) ? ldapDomain : "" ); //$NON-NLS-1$
		
	}
	
	private void addConsoleHostInfo(Composite parent) {
		IConsole console = (IConsole) getElement();
		GridData gd = new GridData();
	    gd.widthHint = convertWidthInCharsToPixels(TEXT_WIDTH);
		
		Label consoleHostLabel = new Label(parent, SWT.NONE);
		consoleHostLabel.setText(ConsolePropertySource.PROP_DISPLAY_HOST);
		
		hostText = new Text(parent, SWT.BORDER);
		hostText.setLayoutData(gd);
		
		String host = (console != null) ? console.getHost() : null;
		hostText.setText((host != null) ? host : ""); //$NON-NLS-1$
		
	}

	private void addConsoleNameInfo(Composite parent) {
		IConsole console = (IConsole) getElement();
		GridData gd = new GridData();
	    gd.widthHint = convertWidthInCharsToPixels(TEXT_WIDTH);
	    
		Label consoleNameLabel = new Label(parent, SWT.NONE);
		consoleNameLabel.setText(ConsolePropertySource.PROP_DISPLAY_NAME);
		
		consoleNameText = new Text(parent, SWT.BORDER);
		consoleNameText.setLayoutData(gd);
		
		String consName = (console != null) ? console.getName() : null;
		consoleNameText.setText((consName != null) ? consName : ""); //$NON-NLS-1$
	}

	private void addSeparator(Composite parent) {
		 Label separator = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		 GridData gridData = new GridData();
		 gridData.horizontalAlignment = GridData.FILL;
		 gridData.grabExcessHorizontalSpace = true;
		 gridData.horizontalSpan = 2;
		 separator.setLayoutData(gridData);
	}
	
	public boolean performOk() {
		Log.trace("Console property page: Ok clicked. Storing information in the current console.",  //$NON-NLS-1$
				this.getClass().getName());
		((IConsole) getElement()).logOff();
		((IConsole) getElement()).setHost(hostText.getText());
		((IConsole) getElement()).setUser(userText.getText());
		((IConsole) getElement()).setPassword(passwordText.getText());
		((IConsole) getElement()).setName(consoleNameText.getText());
		((IConsole) getElement()).setLdapDomain(ldapDomainText.getText());
		((IConsole) getElement()).setAuthEnabled(true);
		
		if ("".equals(hostText.getText()) || //$NON-NLS-1$
				"".equals(userText.getText()) || //$NON-NLS-1$
				"".equals(passwordText.getText()) || //$NON-NLS-1$
				"".equals(consoleNameText.getText())) { //$NON-NLS-1$
			ErrorDialog.openError(UiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
					Messages.getString("ConsolePropertyPage.CONSOLE_CONFIGURATION_ERROR"), Messages.getString("ConsolePropertyPage.EMPTY_PROPERTY_ERROR"), //$NON-NLS-1$ //$NON-NLS-2$
					Log.getMultiStatus(UiPlugin.getDefault().getBundle(), new BuildForgeException(Messages.getString("ConsolePropertyPage.NO_EMPTY_CONSOLE_INFO_ERROR")), Log.ERROR)); //$NON-NLS-1$
			return false;
		}
		
		CorePlugin.getDefault().save();
		return true;
	}
	
}
