/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.ui.view;


import java.net.URL;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.ui.part.ViewPart;

import com.buildforge.buildmodel.core.CorePlugin;
import com.buildforge.buildmodel.core.IProjectNode;
import com.buildforge.buildmodel.core.internal.Messages;
import com.buildforge.buildmodel.ui.UiPlugin;
import com.buildforge.buildmodel.ui.filters.ArchivedStateFilter;
import com.buildforge.buildmodel.ui.filters.CompletedStateFilter;
import com.buildforge.buildmodel.ui.filters.LockedStateFilter;
import com.buildforge.buildmodel.ui.filters.RunningStateFilter;
import com.buildforge.buildmodel.ui.filters.WaitStateFilter;
import com.buildforge.buildmodel.ui.sorters.BuildRecordIdSorter;
import com.buildforge.buildmodel.ui.sorters.BuildRecordOwnerSorter;
import com.buildforge.buildmodel.ui.sorters.BuildRecordResultSorter;
import com.buildforge.buildmodel.ui.sorters.BuildRecordStateSorter;
import com.buildforge.buildmodel.ui.viewer.BuildIdViewContentProvider;
import com.buildforge.buildmodel.ui.viewer.BuildIdViewLabelProvider;


public class BuildIdView extends ViewPart implements ISelectionListener {
	public static final String ID = "BuildForgeBuildIdView.view"; //$NON-NLS-1$

	private Action filterWaitAction;
	private ViewerFilter filterWait;
	private Action filterRunningAction;
	private ViewerFilter filterRunning;
	private Action filterCompletedAction;
	private ViewerFilter filterCompleted;
	private Action filterArchivedAction;
	private ViewerFilter filterArchived;
	private Action filterLockedAction;
	private ViewerFilter filterLocked;
	
	TableViewer viewer  = null;

	private IMemento memento = null;
	private boolean mementoApplied = false;
	
	public void createPartControl(Composite parent) {
		viewer = createTableViewer(parent);
		viewer.setContentProvider(new BuildIdViewContentProvider());
		viewer.setLabelProvider(new BuildIdViewLabelProvider());
		viewer.setInput(CorePlugin.getConsoleList().getActiveProjectNode());
		viewer.setSorter(new BuildRecordIdSorter(true));
		
		WorkbenchHelp.setHelp(viewer.getControl(), "com.buildforge.buildmodel.help.BuildInfoContextId"); //$NON-NLS-1$
		getViewSite().getPage().addSelectionListener(this);
		getViewSite().setSelectionProvider(viewer);
		
		createViewActions();
		
		getViewSite().getActionBars().getToolBarManager().add(filterWaitAction);
		getViewSite().getActionBars().getToolBarManager().add(filterRunningAction);
		getViewSite().getActionBars().getToolBarManager().add(filterCompletedAction);
		getViewSite().getActionBars().getToolBarManager().add(filterArchivedAction);
		getViewSite().getActionBars().getToolBarManager().add(filterLockedAction);
	    getViewSite().getActionBars().getToolBarManager().add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	    getViewSite().getActionBars().getToolBarManager().add(new Separator());
	    
	    viewer.getControl().addPaintListener(new PaintListener() {
	        public void paintControl(PaintEvent e) {
	        	initToolState();
	            mementoApplied = true;
	        }
	      }); 
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {		
				ISelection selection = event.getSelection();
	
				if (selection instanceof IProjectNode) {
					Object first = ((IStructuredSelection) selection).getFirstElement();
					CorePlugin.getConsoleList().setActiveProjectNode((IProjectNode)first);
					CorePlugin.getConsoleList().setActiveConsole(((IProjectNode)first).getParent());
				}
			}
		}); 
		
	}
	
	private void initToolState() {
		
		if (!mementoApplied && memento != null) {
			try {
				Integer running = memento.getInteger("filterRunning"); //$NON-NLS-1$
				Integer waiting = memento.getInteger("filterWaiting"); //$NON-NLS-1$
				Integer complete = memento.getInteger("filterComplete"); //$NON-NLS-1$
				Integer archive = memento.getInteger("filterArchive"); //$NON-NLS-1$
				Integer filter = memento.getInteger("filterLock"); //$NON-NLS-1$
				
				if (running != null && running.intValue() > 0) { 
					filterRunningAction.setChecked(true);
					filterRunningAction.run();
				}
				if (waiting != null && waiting.intValue() > 0) { 
					filterWaitAction.setChecked(true);
					filterWaitAction.run();
				}
				if (complete != null && complete.intValue() > 0) { 
					filterCompletedAction.setChecked(true);
					filterCompletedAction.run();
				}
				if (archive == null || archive.intValue() > 0) {
					filterArchivedAction.setChecked(true);
					filterArchivedAction.run();
				}
				if (filter == null || filter.intValue() > 0) {
					filterLockedAction.setChecked(true);
					filterLockedAction.run();
				} 
				
				if (CorePlugin.getConsoleList().getActiveProjectNode() != null) {
					viewer.setInput((IProjectNode) CorePlugin.getConsoleList().getActiveProjectNode());
				} 
			} catch (NullPointerException npe) {}
			
		} else if (!mementoApplied){
			// memento is not null or not applied.  Lets default the filterArchive and Locked to being pressed.
			filterArchivedAction.setChecked(true);
			filterArchivedAction.run();
			filterLockedAction.setChecked(true);
			filterLockedAction.run();
		}
		
	}


	public void init(IViewSite site, IMemento memento) 
		throws PartInitException {
			super.init(site, memento);
			this.memento = memento;
	}
	

	public void saveState(IMemento memento) {
		if (filterRunningAction.isChecked())
			memento.putInteger("filterRunning", 1); //$NON-NLS-1$
		else 
			memento.putInteger("filterRunning", 0); //$NON-NLS-1$
		
		if (filterWaitAction.isChecked())
			memento.putInteger("filterWaiting", 1); //$NON-NLS-1$
		else 
			memento.putInteger("filterWaiting", 0); //$NON-NLS-1$
		
		if (filterCompletedAction.isChecked()) 
			memento.putInteger("filterComplete", 1); //$NON-NLS-1$
		else
			memento.putInteger("filterComplete", 0); //$NON-NLS-1$
		
		if (filterArchivedAction.isChecked())
			memento.putInteger("filterArchive", 1); //$NON-NLS-1$
		else 
			memento.putInteger("filterArchive", 0); //$NON-NLS-1$
		
		if (filterLockedAction.isChecked())
			memento.putInteger("filterLock", 1); //$NON-NLS-1$
		else 
			memento.putInteger("filterLock", 0); //$NON-NLS-1$
	}
	
	private TableViewer createTableViewer(Composite parent) {
		Table table = new Table(parent, SWT.H_SCROLL | SWT.V_SCROLL
		        | SWT.MULTI | SWT.FULL_SELECTION);

		    TableLayout layout = new TableLayout();
		    table.setLayout(layout);
		    
		    table.setLinesVisible(true);
		    table.setHeaderVisible(true);
		    String[] STD_HEADINGS = {
		    		Messages.getString("BuildIdView.TABLE_HEADING_TAG"),
		    		Messages.getString("BuildIdView.TABLE_HEADING_STATE"),
		    		Messages.getString("BuildIdView.TABLE_HEADING_RESULT"),
		    		Messages.getString("BuildIdView.TABLE_HEADING_OWNER"),
		    	}; 
		    
		    layout.addColumnData(new ColumnWeightData(35, 30, true));
		    TableColumn tc0 = new TableColumn(table, SWT.NONE);
		    tc0.setText(STD_HEADINGS[0]);
		    tc0.setAlignment(SWT.LEFT);
		    tc0.setResizable(true);
		    tc0.addSelectionListener(new SelectionAdapter() {
		    	boolean descending = true;
		    	public void widgetSelected(SelectionEvent e) {
		    		if (descending) {
		    			descending = false;
		    			viewer.setSorter(new BuildRecordIdSorter(descending));
		    		} else {
		    			descending = true;
		    			viewer.setSorter(new BuildRecordIdSorter(descending));
		    		}
		    	}
		    }); 

		    layout.addColumnData(new ColumnWeightData(15, true));
		    TableColumn tc1 = new TableColumn(table, SWT.NONE);
		    tc1.setText(STD_HEADINGS[1]);
		    tc1.setAlignment(SWT.CENTER);
		    tc1.setResizable(true);
		    tc1.addSelectionListener(new SelectionAdapter() {
		    	boolean descending = true;
		    	public void widgetSelected(SelectionEvent e) {
		    		if (descending) {
		    			descending = false;
		    			viewer.setSorter(new BuildRecordStateSorter(descending));
		    		} else {
		    			descending = true;
		    			viewer.setSorter(new BuildRecordStateSorter(descending));
		    		}
		    	}
		    });
		    
		    layout.addColumnData(new ColumnWeightData(15, true));
		    TableColumn tc2 = new TableColumn(table, SWT.NONE);
		    tc2.setText(STD_HEADINGS[2]);
		    tc2.setAlignment(SWT.CENTER);
		    tc2.setResizable(true);
		    tc2.addSelectionListener(new SelectionAdapter() {
		    	boolean descending = true;
		    	public void widgetSelected(SelectionEvent e) {
		    		if (descending) {
		    			descending = false;
		    			viewer.setSorter(new BuildRecordResultSorter(descending));
		    		} else {
		    			descending = true;
		    			viewer.setSorter(new BuildRecordResultSorter(descending));
		    		}
		    	}
		    });
		    
		    
		    layout.addColumnData(new ColumnWeightData(35, true));
		    TableColumn tc3 = new TableColumn(table, SWT.NONE);
		    tc3.setText(STD_HEADINGS[3]);
		    tc3.setAlignment(SWT.LEFT);
		    tc3.setResizable(true);
		    tc3.addSelectionListener(new SelectionAdapter() {
		    	boolean descending = true;
		    	public void widgetSelected(SelectionEvent e) {
		    		if (descending) {
		    			descending = false;
		    			viewer.setSorter(new BuildRecordOwnerSorter(descending));
		    		} else {
		    			descending = true;
		    			viewer.setSorter(new BuildRecordOwnerSorter(descending));
		    		}
		    	}
		    });
		    
		    return new TableViewer(table);
	}

	public void setFocus() {
		viewer.getControl().setFocus();
	}
	

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if (part != this) {
			if (selection instanceof IStructuredSelection) {
	            Object first = ((IStructuredSelection)selection).getFirstElement();
	            if (first instanceof IProjectNode) {
	            	if (first != null && viewer != null && viewer.getContentProvider() != null) {
	            		viewer.setInput((IProjectNode) first);
	            	}
	            }
			}
		}
	}
	
	public void createViewActions() {
		
	    filterWaitAction = new Action(Messages.getString("BuildIdView.HIDE_WAIT_STATUS_ACTION_LABEL"), //$NON-NLS-1$
	            Action.AS_CHECK_BOX) {
	          public void run() {
	            if (isChecked()) {
	              if (filterWait == null)
	                filterWait = new WaitStateFilter();
	              viewer.addFilter(filterWait);
	            } else
	              viewer.removeFilter(filterWait);
	          }
	        };
	        filterWaitAction
	            .setToolTipText(Messages.getString("BuildIdView.HIDE_WAIT_STATUS_TOOLTIP_LABEL")); //$NON-NLS-1$
	        URL url = UiPlugin.getDefault().find(new Path("icons/status_wait_hide.gif")); //$NON-NLS-1$
		    ImageDescriptor id = ImageDescriptor.createFromURL(url);
		    filterWaitAction.setImageDescriptor(id);
	
	        
	        filterRunningAction = new Action(Messages.getString("BuildIdView.HIDE_RUNNING_STATUS_ACTION_LABEL"), //$NON-NLS-1$
		            Action.AS_CHECK_BOX) {
		          public void run() {
		            if (isChecked()) {
		              if (filterRunning == null)
		                filterRunning = new RunningStateFilter();
		              viewer.addFilter(filterRunning);
		            } else
		              viewer.removeFilter(filterRunning);
		          }
		        };
		        filterRunningAction
		            .setToolTipText(Messages.getString("BuildIdView.HIDE_RUNNING_STATUS_TOOLTIP_LABEL")); //$NON-NLS-1$
		        URL url2 = UiPlugin.getDefault().find(new Path("icons/status_running_hide.gif")); //$NON-NLS-1$
			    ImageDescriptor id2 = ImageDescriptor.createFromURL(url2);
		        filterRunningAction.setImageDescriptor(id2);
		        
		        
		        filterCompletedAction = new Action(Messages.getString("BuildIdView.HIDE_COMPLETED_STATUS_ACTION_LABEL"), //$NON-NLS-1$
			            Action.AS_CHECK_BOX) {
			          public void run() {
			            if (isChecked()) {
			              if (filterCompleted == null)
			                filterCompleted = new CompletedStateFilter();
			              viewer.addFilter(filterCompleted);
			            } else
			              viewer.removeFilter(filterCompleted);
			          }
		        };
		        filterCompletedAction
		            .setToolTipText(Messages.getString("BuildIdView.HIDE_COMPLETED_STATUS_TOOLTIP_LABEL")); //$NON-NLS-1$
		        URL url3 = UiPlugin.getDefault().find(new Path("icons/status_completed_hide.gif")); //$NON-NLS-1$
			    ImageDescriptor id3 = ImageDescriptor.createFromURL(url3);
		        filterCompletedAction.setImageDescriptor(id3);
        
		        filterArchivedAction = new Action(Messages.getString("BuildIdView.HIDE_ARCHIVED_STATUS_ACTION_LABEL"),  //$NON-NLS-1$
		        		Action.AS_CHECK_BOX) {
		        	public void run() {
		        		if (isChecked()) {
		        			if (filterArchived == null) 
		        				filterArchived = new ArchivedStateFilter();
		        			viewer.addFilter(filterArchived);
		        		} else {
		        			viewer.removeFilter(filterArchived);
		        		}
		        	}
		        };
		        filterArchivedAction.setToolTipText(Messages.getString("BuildIdView.HIDE_ARCHIVED_STATUS_TOOLTIP_LABEL")); //$NON-NLS-1$
		        URL url4 = UiPlugin.getDefault().find(new Path("icons/status_archived_hide.gif")); //$NON-NLS-1$
		        ImageDescriptor id4 = ImageDescriptor.createFromURL(url4);
		        filterArchivedAction.setImageDescriptor(id4);
		        
		        filterLockedAction = new Action(Messages.getString("BuildIdView.HIDE_LOCKED_STATUS_ACTION_LABEL"),  //$NON-NLS-1$
		        		Action.AS_CHECK_BOX) {
		        	public void run() {
		        		if (isChecked()) {
		        			if (filterLocked == null)
		        				filterLocked = new LockedStateFilter();
		        			viewer.addFilter(filterLocked);
		        		} else {
		        			viewer.removeFilter(filterLocked);
		        		}
		        	}
		        };
		        filterLockedAction.setToolTipText(Messages.getString("BuildIdView.HIDE_LOCKED_STATUS_TOOLTIP_LABEL")); //$NON-NLS-1$
		        URL url5 = UiPlugin.getDefault().find(new Path("icons/status_locked_hide.gif")); //$NON-NLS-1$
		        ImageDescriptor id5 = ImageDescriptor.createFromURL(url5);
		        filterLockedAction.setImageDescriptor(id5);
	}
	

	
}
