/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.ui.view;

import java.net.URL;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.dialogs.PropertyDialogAction;
import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.WorkbenchJob;

import com.buildforge.buildmodel.core.CorePlugin;
import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.IConsoleList;
import com.buildforge.buildmodel.core.IProjectNode;
import com.buildforge.buildmodel.core.dialogs.JobRunDialog;
import com.buildforge.buildmodel.core.exception.BuildForgeAuthenticationException;
import com.buildforge.buildmodel.core.exception.BuildForgeException;
import com.buildforge.buildmodel.core.internal.EnvironmentGroup;
import com.buildforge.buildmodel.core.internal.Messages;
import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.ui.UiPlugin;
import com.buildforge.buildmodel.ui.viewer.ConsoleExplorerTreeViewProvider;


public class ConsoleExplorerView extends ViewPart  {
	public static final String ID = "BuildForgeConsoleExplorer.view"; //$NON-NLS-1$
	
	private boolean initialSelectionComplete = false;
	private TreeViewer viewer;
	Action addConsoleAction, deleteConsoleAction, executeBuild, propertiesAction;
	protected PropertyDialogAction propertiesDialogAction;

	private Action refreshAction;

	 class ViewLabelProvider extends LabelProvider implements ILabelProvider {
		
		public String getText(Object obj) {
			if (obj instanceof IProjectNode) {
				return ((IProjectNode) obj).getName();
			}
			if (obj instanceof IConsole)
				return ((IConsole) obj).getName();
			return null;
		}

		public Image getImage(Object obj) {
			if (obj instanceof IConsole) {
				if (! ((IConsole) obj).isConnected() || ((IConsole) obj).isDisabled()) {
					URL url = UiPlugin.getDefault().find(new Path("icons/server_disconnected.gif")); //$NON-NLS-1$
					ImageDescriptor id = ImageDescriptor.createFromURL(url);
				    Image failImage = id.createImage();
					return failImage;
				}
				URL url = UiPlugin.getDefault().find(new Path("icons/server_connected.gif")); //$NON-NLS-1$
			    ImageDescriptor id = ImageDescriptor.createFromURL(url);
			    Image passImage = id.createImage();
				return passImage;
			} 
			if (obj instanceof IProjectNode) {
				URL url = UiPlugin.getDefault().find(new Path("icons/project.gif")); //$NON-NLS-1$
			    ImageDescriptor id = ImageDescriptor.createFromURL(url);
			    Image execImage = id.createImage();
				return execImage;
			} 
			return null;
		}
 		
	}  


	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	public void createPartControl(Composite parent) {
		Log.trace("Initializing tree viewer", this.getClass().getName()); //$NON-NLS-1$
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		viewer.setContentProvider(new ConsoleExplorerTreeViewProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setInput(CorePlugin.getConsoleList());
		
		getViewSite().setSelectionProvider(viewer);
		WorkbenchHelp.setHelp(viewer.getControl(), "com.buildforge.buildmodel.help.ConsoleContextId"); //$NON-NLS-1$
				
		Shell shell = getViewSite().getShell();
		propertiesDialogAction = new PropertyDialogAction(shell, viewer);
		createActions();
		getViewSite().getActionBars().setGlobalActionHandler(ActionFactory.PROPERTIES.getId(), propertiesDialogAction);
		
		IMenuManager mgr = getViewSite().getActionBars().getMenuManager();
		mgr.add(addConsoleAction);
		mgr.add(deleteConsoleAction);
		mgr.add(refreshAction);
		mgr.add(executeBuild);
		mgr.add(propertiesAction);	

		createContextMenu();
		
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				addConsoleAction.setEnabled(true);
				deleteConsoleAction.setEnabled(false);
				executeBuild.setEnabled(false);
				propertiesAction.setEnabled(false);

				if (event.getSelection() instanceof IStructuredSelection) {
					IStructuredSelection selection = (IStructuredSelection)event.getSelection();
					
					for (Iterator iter = selection.iterator(); iter.hasNext();) {
						Object subject = iter.next();

						if (subject instanceof IProjectNode) {
							if (((IProjectNode)subject).getName() != null) {
								
								deleteConsoleAction.setEnabled(false);
								executeBuild.setEnabled(true);
								propertiesAction.setEnabled(false);

								// Mark the Active Project
								CorePlugin.getConsoleList().setActiveProjectNode((IProjectNode) subject);
								CorePlugin.getConsoleList().setActiveConsole(((IProjectNode)subject).getParent());
								if ( ((IProjectNode) subject).getParent().isDisabled() ||
										!((IProjectNode) subject).getParent().isConnected() ) {
									viewer.refresh();
								}
								
							}
						}

						if (subject instanceof IConsole) {
							if (((IConsole)subject).getName() != null) {
								deleteConsoleAction.setEnabled(true);
								executeBuild.setEnabled(false);
								propertiesAction.setEnabled(true);
							
								// Mark the Active Console
								IConsoleList consoleList = CorePlugin.getConsoleList();
								CorePlugin.getConsoleList().setActiveConsole((IConsole) subject);

								consoleList.setActiveConsole((IConsole)subject);
								
								WorkbenchJob execJob = new WorkbenchJob(Messages.getString("ConsoleExplorerView.WORKBENCH_JOB_BUILD_PROCESSING_MESG")) { //$NON-NLS-1$
									public IStatus runInUIThread(IProgressMonitor monitor) {
										IConsole console = (IConsole) CorePlugin.getConsoleList().getActiveConsole();

										if (console != null) {
											try {
												String checktoken = console.getAuthenticationTokens();
												if (!console.isConnected() || checktoken == null || checktoken.length() == 0 || console.isDisabled()) {
													Log.trace("Attempting to Authenticate with console [" + console.getName(), getClass().getName()); //$NON-NLS-1$
													Log.trace("Reconnecting to console " + console.getName(), getClass().getName()); //$NON-NLS-1$
													console.invalidateAuthenticationTokens();
													console.setAuthEnabled(true);
													if (console.authenticate()) 
														console.update();
												}
												
											} catch (BuildForgeAuthenticationException bae) {
												Log.errorLogMessage(Messages.getString("ConsoleExplorerView.AUTHENTICATION_ERROR_CONS"), Log.ERROR,  //$NON-NLS-1$
														this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName());
												ErrorDialog.openError(UiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
														Messages.getString("ConsoleExplorerView.BUILDFORGE_AUTHENTICATION_ERROR_DLG"), Messages.getString("ConsoleExplorerView.BUILDFORGE_AUTHENTICATION_SUB_ERROR_DLG") + bae.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
														Log.getMultiStatus(UiPlugin.getDefault().getBundle(), bae, Log.ERROR));
												console.setAuthEnabled(false);
											} catch (BuildForgeException be) {
												Log.errorLogMessage(Messages.getString("ConsoleExplorerView.BUILDFORGE_COMMUNICATION_ERROR_CONS"), Log.ERROR,  //$NON-NLS-1$
														this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName());
												ErrorDialog.openError(UiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
														Messages.getString("ConsoleExplorerView.GENERIC_ERROR_DLG"), Messages.getString("ConsoleExplorerView.GENERIC_ERROR_SUB_DLG") + be.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
														Log.getMultiStatus(UiPlugin.getDefault().getBundle(), be, Log.ERROR));
												console.setAuthEnabled(false);
											}
										}
										return Status.OK_STATUS;
									}
								};
								
								execJob.setSystem(true);
								if (((IConsole) subject).isDisabled()) {
									try {
										boolean verified = ((IConsole)subject).verifiedConnection();
										if (!verified)
											return;
									} catch (Exception e){ return; }
								}
								
								execJob.schedule(); 
								viewer.refresh();
							}
						}
						
					}					
				}				
				
			}
		});
		
		 viewer.getControl().addPaintListener(new PaintListener() {
				public void paintControl(PaintEvent e) {
		        	if (!initialSelectionComplete)
		        		initSelection();
		        	initialSelectionComplete = true;
		        }
		      }); 
		
		 refreshAction.run();
	}

	private void initSelection() {
		if (CorePlugin.getConsoleList() != null) {
			IConsole activeConsole = CorePlugin.getConsoleList().getActiveConsole();
			IProjectNode activeProject = CorePlugin.getConsoleList().getActiveProjectNode();
			if (activeConsole != null) {
				boolean verifiedHost = false;
				try {
					Log.trace("verifying console connection.", this.getClass().toString());
					verifiedHost = activeConsole.verifiedConnection();
				} catch (Exception e) { }
				
				if (verifiedHost) {
					String projectName = ""; //$NON-NLS-1$
					StructuredSelection sel = new StructuredSelection(activeConsole);
					StructuredSelection projSel = null;
					if (activeProject != null) 
						projectName = activeProject.getName();
					try {
							activeConsole.authenticate();
							activeConsole.update();
					} catch (BuildForgeException bae) {}
					
					
					IProjectNode foundProj = null;
					if (activeProject != null) {
						for (int i = 0; i < activeConsole.getProjects().size(); i++) {
							IProjectNode p =  (IProjectNode) activeConsole.getProjects().get(i);
							if (projectName.equals(p.getName())) {
								foundProj = p;
							}
						}
						if (foundProj != null)
							projSel = new StructuredSelection(foundProj);
					}
					
					viewer.setExpandedElements(new Object [] {activeConsole });
				
					if (activeProject != null && foundProj != null) {
						CorePlugin.getConsoleList().setActiveProjectNode(foundProj);
						viewer.setSelection(projSel, true);
					}
					else 
						viewer.setSelection(sel, true);
				} else {
					activeConsole.invalidateAuthenticationTokens();
					activeConsole.setAuthEnabled(false);
					activeConsole.setConnected(false);
				}
			}
		}
	}
		

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	public void createActions() {
		
		deleteConsoleAction = new Action(Messages.getString("ConsoleExplorerView.DELETE_CONSOLE_ACTION_LABEL")) { //$NON-NLS-1$
			public void run () {
				Log.trace("ConsoleExplorerView : executing delete action", this.getClass().getName()); //$NON-NLS-1$
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				Object first = selection.getFirstElement();
				if (first instanceof IConsole) {
					Log.trace("ConsoleExplorerView : deleting console " + ((IConsole) first).getName(), this.getClass().getName()); //$NON-NLS-1$
					CorePlugin.getConsoleList().removeConsole((IConsole) first);
					viewer.refresh();
				}
			}
		};
		
				
		refreshAction = new Action(Messages.getString("ConsoleExplorerView.REFRESH_CONSOLE_ACTION_LABEL")) { //$NON-NLS-1$
			public void run() {
				Log.trace("ConsoleExplorerView : executing refresh action", this.getClass().getName()); //$NON-NLS-1$
				
				WorkbenchJob execJob = new WorkbenchJob(Messages.getString("ConsoleExplorerView.WORKBENCH_JOB_BUILD_PROCESSING_MESG")) { //$NON-NLS-1$
				
					public IStatus runInUIThread(IProgressMonitor monitor) {
						
						IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
						Object first = selection.getFirstElement();
						String consoleName = ""; //$NON-NLS-1$
						if (first instanceof IConsole) {
							consoleName = ((IConsole)first).getName();
						}
						Object[] consoles = CorePlugin.getConsoleList().getConsoles();
						for (int i=0; i < consoles.length; i++) {
							IConsole cons = (IConsole) consoles[i];
							String checktoken = cons.getAuthenticationTokens();
							if ((cons != null && !cons.isConnected()) || checktoken == null || checktoken.length() ==0 ) {
								try {
									if (consoleName.equals(((IConsole) consoles[i]).getName()))
										((IConsole)consoles[i]).setAuthEnabled(true);
									((IConsole)consoles[i]).invalidateAuthenticationTokens();
									if (((IConsole) consoles[i]).authenticate())
										((IConsole) consoles[i]).update();
									else {
										((IConsole) consoles[i]).setAuthEnabled(false);
									}
								} catch (BuildForgeAuthenticationException bae) {
									Log.errorLogMessage(Messages.getString("ConsoleExplorerView.AUTHENTICATION_ERROR_CONS"), Log.ERROR,  //$NON-NLS-1$
											this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName());
									ErrorDialog.openError(UiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
											Messages.getString("ConsoleExplorerView.BUILDFORGE_AUTHENTICATION_ERROR_DLG"), Messages.getString("ConsoleExplorerView.BUILDFORGE_AUTHENTICATION_SUB_ERROR_DLG") + bae.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
											Log.getMultiStatus(UiPlugin.getDefault().getBundle(), bae, Log.ERROR));
									((IConsole) consoles[i]).setAuthEnabled(false);
								}  catch (BuildForgeException be) {
									Log.errorLogMessage(Messages.getString("ConsoleExplorerView.BUILDFORGE_COMMUNICATION_ERROR_CONS"), Log.ERROR,  //$NON-NLS-1$
											this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName());
									ErrorDialog.openError(UiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
											Messages.getString("ConsoleExplorerView.GENERIC_ERROR_DLG"), Messages.getString("ConsoleExplorerView.GENERIC_ERROR_SUB_DLG") + be.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
											Log.getMultiStatus(UiPlugin.getDefault().getBundle(), be, Log.ERROR));
									((IConsole) consoles[i]).setAuthEnabled(false);
								}
							}
						}
						viewer.refresh();
						return Status.OK_STATUS;
					}
				};
				execJob.setSystem(true);
				execJob.schedule();
			}
		};

		executeBuild = new Action(Messages.getString("ConsoleExplorerView.EXECUTE_BUILD_ACTION_LABEL")) { //$NON-NLS-1$
			public void run() {
				Log.trace("ConsoleExplorerView : execute action called", this.getClass().getName()); //$NON-NLS-1$
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				final Object first = selection.getFirstElement();
				if (first instanceof IProjectNode) {
					
					WorkbenchJob execJob = new WorkbenchJob(Messages.getString("ConsoleExplorerView.WORKBENCH_JOB_BUILD_PROCESSING_MESG")) { //$NON-NLS-1$
						public IStatus runInUIThread(IProgressMonitor monitor) {
							int projectId = ((IProjectNode) first).getProjectID();
							EnvironmentGroup envGrp = ((IProjectNode) first).getParent().getEnvironmentGroup(projectId);
							
							if (envGrp != null && (envGrp.getList()).size() > 0) {
								/* --- inserting dialog here for dealing with Environment issues. --- */
						    	JobRunDialog dialog = new JobRunDialog(UiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
						    			envGrp);
						    	if (dialog.open() != InputDialog.OK)
						    		return Status.CANCEL_STATUS;
						    	envGrp = dialog.environment;
							}
					    	
							try {
								Log.trace("++++++ FIRE!!! ++++++", this.getClass().getName());
								final int buildId = ((IProjectNode) first).getParent().execBuild(projectId, envGrp);
								CorePlugin.getConsoleList().setActiveProjectNode((IProjectNode)first);
								CorePlugin.getConsoleList().setBuildExecuted(projectId);
								CorePlugin.getConsoleList().getActiveProjectNode().getBuildIdRecords();
								CorePlugin.getConsoleList().setActiveBuildRecord(CorePlugin.getConsoleList().getActiveProjectNode().getBuildRecordById(buildId));
								viewer.setSelection(viewer.getSelection());
							} catch (NullPointerException npe) {
								Log.errorLogMessage(Messages.getString("ConsoleExplorerView.NO_BUILDID_AFTER_EXEC_ERROR_CONS"),  //$NON-NLS-1$
										Log.ERROR, this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName());		
							}
							return Status.OK_STATUS;
						}
					};
					execJob.setUser(true);
					execJob.schedule();
					
				} else {
					Log.errorLogMessage(Messages.getString("ConsoleExplorerView.NON_PROJECT_EXECUTE_CONS"),  //$NON-NLS-1$
						Log.ERROR, this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName());
				}
			}
		};
		
		propertiesAction = new Action(Messages.getString("ConsoleExplorerView.PROPERTIES_ACTION_LABEL")) { //$NON-NLS-1$
			public void run() {
				Log.trace("ConsoleExplorerView : executing console properties", this.getClass().getName()); //$NON-NLS-1$
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				Object first = selection.getFirstElement();
				if (first instanceof IConsole) {
					Log.trace("ConsoleExplorerView : creating properties page", this.getClass().getName()); //$NON-NLS-1$
					propertiesDialogAction.selectionChanged(selection);
					propertiesDialogAction.run();
					try {
						Log.trace("ConsoleExplorerView : forcing reauthentication with console after properties change", this.getClass().getName()); //$NON-NLS-1$
						((IConsole)first).invalidateAuthenticationTokens();
						((IConsole) first).setAuthEnabled(true);
						((IConsole)first).authenticate();
						((IConsole)first).update();
					} catch (BuildForgeAuthenticationException bfe) {
						Log.errorLogMessage(Messages.getString("ConsoleExplorerView.CHANGE_PROPS_AUTH_ERROR_CONS"), //$NON-NLS-1$
								Log.ERROR, this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName());
						ErrorDialog.openError(UiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
								Messages.getString("ConsoleExplorerView.BUILDFORGE_AUTHENTICATION_ERROR_DLG"), Messages.getString("ConsoleExplorerView.BUILDFORGE_AUTHENTICATION_CHECK_PROPS_ERROR_SUB_DLG"), //$NON-NLS-1$ //$NON-NLS-2$
								Log.getMultiStatus(UiPlugin.getDefault().getBundle(), bfe, Log.ERROR));
						((IConsole) first).setAuthEnabled(false);
					} catch (BuildForgeException be) {
						Log.errorLogMessage(Messages.getString("ConsoleExplorerView.BUILDFORGE_COMMUNICATION_ERROR_CONS"), Log.ERROR,  //$NON-NLS-1$
								this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName());
						ErrorDialog.openError(UiPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
								Messages.getString("ConsoleExplorerView.GENERIC_ERROR_DLG"), Messages.getString("ConsoleExplorerView.GENERIC_ERROR_SUB_DLG") + be.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
								Log.getMultiStatus(UiPlugin.getDefault().getBundle(), be, Log.ERROR));
						((IConsole) first).setAuthEnabled(false);
					}
					viewer.refresh();
				} else {
					Log.errorLogMessage(Messages.getString("ConsoleExplorerView.NON_PROJECT_PROPERTIES_ACTION_ERROR_CONS"), Log.WARNING, this.getClass().getName(), UiPlugin.getDefault().getBundle().getSymbolicName()); //$NON-NLS-1$
				}
			}
		};
		
		Log.trace("ConsoleExplorerView : Creating Actions", this.getClass().getName()); //$NON-NLS-1$
		addConsoleAction = new Action(Messages.getString("ConsoleExplorerView.NEW_CONSOLE_ACTION_LABEL")) { //$NON-NLS-1$
			public void run () {
				Log.trace("Creating new Console entry.", this.getClass().getName()); //$NON-NLS-1$
				
				IConsole newConsole = CorePlugin.getConsoleList().createEmptyConsoleEntry();
				Log.trace("ConsoleExplorerView : New Console is named " + newConsole.getName(), this.getClass().getName()); //$NON-NLS-1$
				CorePlugin.getConsoleList().setActiveConsole(newConsole);
				StructuredSelection conSel = new StructuredSelection(newConsole);
				Log.trace("Setting Selection to newly created project and calling properties", this.getClass().getName());
				viewer.refresh();
				viewer.setSelection(conSel);
				viewer.refresh();
				propertiesAction.run();
			}
		};
		addConsoleAction.setId("com.buildforge.buildmodel.ui.ConsoleExplorerView.NewConsoleAction"); //$NON-NLS-1$

	}
	
	public void createContextMenu() {
		Log.trace("ConsoleExplorerView : Creating Context Menu", this.getClass().getName()); //$NON-NLS-1$
		MenuManager menuManager = new MenuManager();
        menuManager.setRemoveAllWhenShown(true);
        menuManager.addMenuListener(new IMenuListener() {
                public void menuAboutToShow(IMenuManager manager) {
                        fillContextMenu(manager);
                }
        });
        Menu menu = menuManager.createContextMenu(viewer.getControl());
        viewer.getControl().setMenu(menu);
        getSite().registerContextMenu(menuManager, viewer);
	}
	
	private void fillContextMenu(IMenuManager manager) {
		Log.trace("ConsoleExplorerView : populating context menu", this.getClass().getName()); //$NON-NLS-1$
        manager.add(addConsoleAction);
        manager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
        manager.add(deleteConsoleAction);
        manager.add(refreshAction);
        manager.add(new Separator());
        manager.add(executeBuild);
        manager.add(propertiesAction);
       
	}
	
	
}