/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.ui.viewer;

import java.util.ArrayList;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.Viewer;

import com.buildforge.buildmodel.core.IBuildRecord;
import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.IStepDetailInfo;
import com.buildforge.buildmodel.core.IStepDetailRecord;
import com.buildforge.buildmodel.core.IStepDetailTypeInfo;
import com.buildforge.buildmodel.core.IStepLogInfo;
import com.buildforge.buildmodel.core.internal.Messages;

public class BuildLogTreeViewContentProvider implements ITreeContentProvider {

	StructuredViewer viewer = null;
	IConsole logConsole = null;
	
	public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof IStepDetailRecord) {
			ArrayList children = new ArrayList();
			((IStepDetailRecord) parentElement).populateLogInfo(logConsole);
			String[] info = ((IStepDetailRecord) parentElement).getLogInfo().getInfo();
			IStepDetailInfo detailInfo = ((IStepDetailRecord)parentElement).getDetails();
			Object[] det = detailInfo.getDetailTypes();
			for (int i = 0; i < det.length; i++) {
				children.add((IStepDetailTypeInfo)det[i]);
			}
			for (int i = 0; i < info.length; i++) {
				children.add(info[i]);
			}
			if (info.length == 0 && children.size() == 0) {
				return new String[] {Messages.getString("BuildLogTreeViewContentProvider.LOG_VIEW_INFO_NO_RESULTS_MESSAGE")}; //$NON-NLS-1$
			}
			return children.toArray();
		}
/*		if (parentElement instanceof IStepDetailInfo) {
			if (parentElement == null) 
				((IStepDetailInfo) parentElement).getParent().populateLogInfo(logConsole);
			//String[] detailsFolders = ((IStepDetailInfo)parentElement).getDetailTypes();
			//if (detailsFolders.length > 0)
			//	return detailsFolders;
			//return new String[] {Messages.getString("BuildLogTreeViewContentProvider.LOG_VIEW_DETAIL_NO_RESULTS_MESSAGE")};
			return ((IStepDetailInfo)parentElement).getDetailTypes();
		}
*/
		if (parentElement instanceof IStepDetailTypeInfo) {
			if (parentElement == null)
				((IStepDetailTypeInfo)parentElement).getParent().getParent().populateLogInfo(logConsole);
			String[] details = ((IStepDetailTypeInfo) parentElement).getInfo();
			if (details.length == 0)
				return new String[] {Messages.getString("BuildLogTreeViewContentProvider.LOG_VIEW_DETAIL_NO_RESULTS_MESSAGE")}; //$NON-NLS-1$
			return details;
		}
		if (parentElement instanceof IStepLogInfo) {
			if (parentElement == null)
				((IStepLogInfo)parentElement).getParent().populateLogInfo(logConsole);
			String[] info = ((IStepLogInfo) parentElement).getInfo();
			if (info.length == 0) 
				return new String[] {Messages.getString("BuildLogTreeViewContentProvider.LOG_VIEW_NO_STEP_RESULTS_MESSAGE")}; //$NON-NLS-1$
			return info;
		}
		return null;
	}

	public Object getParent(Object element) {
		if (element instanceof IStepDetailInfo)
			return ((IStepDetailInfo) element).getParent();
		if (element instanceof IStepDetailTypeInfo)
			return ((IStepDetailTypeInfo) element).getParent();
		return null;
	}

	public boolean hasChildren(Object element) {
		if (element instanceof IBuildRecord)
			return true;
		if (element instanceof IStepDetailRecord)
			return true;
		if (element instanceof IStepDetailInfo)
			return true;
		if (element instanceof IStepDetailTypeInfo)
			return true;
		return false;
	}

	public Object[] getElements(Object inputElement) {
		if (inputElement instanceof IBuildRecord) {
			logConsole = ((IBuildRecord) inputElement).getParent().getParent();
			Object[] objs =  ((IBuildRecord) inputElement).getStepDetailRecords();
			if (objs.length == 0) {
				return new String[] {Messages.getString("BuildLogTreeViewContentProvider.LOG_VIEW_NO_BUILD_STEPS_MESSAGE")}; //$NON-NLS-1$
			}
			return objs;
		}
		return null;
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		
	}

	public void dispose() {
		
		
	}

}
