/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.core;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;

import org.eclipse.ui.plugin.*;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.osgi.framework.BundleContext;

import com.buildforge.buildmodel.core.internal.ConsoleList;
import com.buildforge.buildmodel.core.util.Log;


/**
 * The main plugin class to be used in the Build Forge core plugin.
 * Access to 
 */
public class CorePlugin extends AbstractUIPlugin {

	//The shared instance.
	private static CorePlugin plugin;
	//private ResourceBundle resourceBundle;
	private static Path consolesPath = new Path("buildforge_consoles.txt");
	private static IConsoleList consoleList = null;
	
	/**
	 * The constructor.
	 */
	public CorePlugin() {
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		consoleList = getConsoleList();
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		Object[] cons = consoleList.getConsoles();
		
		for (int i=0; i < cons.length; i++) {
			((IConsole)cons[i]).logOff();
		}
		
		if (consoleList != null)
			save();
		
		super.stop(context);
		plugin = null;
	}

	/**
	 * Returns the shared instance.
	 */
	public static CorePlugin getDefault() {
		return plugin;
	}

	/**
	 * Read the list of consoles from the serialized data file.
	 * @return IConsoleList - List containing IConsole objects.
	 */
	public static IConsoleList getConsoleList() {
		Log.trace("getting console list", CorePlugin.getDefault().getClass().getName());
	
		if (consoleList == null) {
			try {
				IPath statedir = CorePlugin.getDefault().getStateLocation();
				File locdata = statedir.append(consolesPath).toFile();
				BufferedReader in = new BufferedReader(new FileReader(locdata));
 
				StringBuffer savedString = new StringBuffer();
				String str;
				while((str = in.readLine()) != null) {
					savedString.append(str);
				}
								
				consoleList = new ConsoleList();
				consoleList.initializeConsoleListWithSaveData(savedString.toString());	
			} catch (IOException e) {
				consoleList = new ConsoleList();
			}
		} 
		return consoleList;
	}
	
	public static void setConsoleList(IConsoleList consoleList) {
		CorePlugin.consoleList = consoleList;
	}

	/**
	 * Serializes the existing IConsoleList to a file. 
	 *
	 */
	public void save() {

		IPath statedir = CorePlugin.getDefault().getStateLocation();
		File locdata = statedir.append(consolesPath).toFile();

		Log.trace("Saving information to : " + locdata.getAbsolutePath(), CorePlugin.getDefault().getClass().getName());
		
		try {
			FileOutputStream out = new FileOutputStream(locdata);
			String clSave = consoleList.getConsoleListSaveData();
			out.write(clSave.getBytes());
			Log.trace("Data Written : " + clSave, CorePlugin.getDefault().getClass().getName());
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path.
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin("com.buildforge.buildmodel.core", path);
	}
}
