package com.buildforge.buildmodel.core.dialogs;

import java.util.ArrayList;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

import com.buildforge.buildmodel.core.internal.Environment;
import com.buildforge.buildmodel.core.internal.EnvironmentGroup;
import com.buildforge.buildmodel.core.internal.Messages;
import com.buildforge.buildmodel.core.util.Log;


/***
 *  This dialog is used to set things about a job up at runtime.
 *  Currently only environment variables are set here, but we 
 *  should probably do inclusion/exclusion of steps and selector/console
 *  selection as well in the future.
 * @author heathm
 *
 */
public class JobRunDialog extends Dialog {

	public EnvironmentGroup environment = null;

	public JobRunDialog(Shell parentShell, EnvironmentGroup env) {
		super(parentShell);
		environment = env;
	}

	public Control createDialogArea(Composite parent) {
		Composite top = (Composite) super.createDialogArea(parent);
		GridLayout gLay = new GridLayout();
	    gLay.numColumns = 1;
		top.setLayout(gLay);
		top.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		final Label label = new Label(top, SWT.FILL);
		label.setText(Messages.getString("JobRunDialog.HEADER_INFO1"));
		label.getBounds();
		
		final Composite comp = new Composite(top, SWT.BORDER);
		comp.setLayout(new GridLayout(2,true));
		comp.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		final Label envNameLabel = new Label (comp, SWT.NONE);
		envNameLabel.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		final Label envValueLabel = new Label (comp, SWT.NONE);
		envValueLabel.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		
		environment.orderByPosition();
		ArrayList envs = (ArrayList) environment.getList();
		final Label[] envNamesList = new Label[envs.size()];
		final CCombo[] envValueList = new CCombo[envs.size()];
		int defaultChoice = 0;
		for (int i = 0; i < envs.size(); i++) {
			envNamesList[i] = new Label(comp, SWT.NONE);
			envNamesList[i].setText( ((Environment) envs.get(i)).getName());
			envNamesList[i].setLayoutData(new GridData(GridData.FILL_VERTICAL));
			envValueList[i] = new CCombo(comp, SWT.FLAT | SWT.FILL);
			String[] options = ((Environment) envs.get(i)).getValueOptions();
			
			if (options != null) {
				for (int op = 0; op < options.length; op++) {
					envValueList[i].add(options[op], op);
				}
				envValueList[i].select(defaultChoice);
			} else {
				String val =  ((Environment) envs.get(i)).getValue();
				if (val != null) {
					envValueList[i].add(val);
					envValueList[i].select(defaultChoice);
				} else {
					envValueList[i].add("");
				}
			}
			environment.setChoice(i, envValueList[i].getText());
			envValueList[i].setData(new Integer(i));
			if (i == 0)
				envValueList[i].setFocus();
			
			envValueList[i].setLayoutData(new GridData(GridData.FILL_BOTH));
			envValueList[i].addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					environment.setChoice(((Integer)((CCombo)e.getSource()).getData()).intValue(), ((CCombo)e.getSource()).getText());
				}
			});
		}
		
		// initContent();
		return top;
	}

	
	public void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(Messages.getString("JobRunDialog.DIALOG_TITLE"));
	}

}
