/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */
package com.buildforge.buildmodel.core.internal;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Hex;

import com.buildforge.buildmodel.core.CorePlugin;
import com.buildforge.buildmodel.core.exception.BuildForgeAuthenticationException;
import com.buildforge.buildmodel.core.exception.BuildForgeException;
import com.buildforge.buildmodel.core.util.Log;


public class BuildForgeConnection  implements IConstants{
	String baseURL = API_URL_APPEND_STRING;
	protected String userName = ""; //$NON-NLS-1$
	protected String password = ""; //$NON-NLS-1$
	protected String domain = ""; //$NON-NLS-1$
	protected String host = ""; //$NON-NLS-1$
	protected String token = ""; //$NON-NLS-1$
	protected String errorMessage = ""; //$NON-NLS-1$
	protected String infoMessage = ""; //$NON-NLS-1$
	String consoleVersion = "";
	boolean hasInfo = false; 
	boolean hasError = false;
	boolean authError = false;
	boolean repost = false;
	
	public BuildForgeConnection () {
		super();
	}
	
	public BuildForgeConnection (String user, String password, String host) {
		super();
		this.userName = user;
		this.password = password;
		this.host = host;	
		this.domain = ""; //$NON-NLS-1$
	}
	
	public BuildForgeConnection (String user, String password, String host, String domain) {
		this(user, password, host);
		this.domain = domain;
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#postQuery(java.lang.String)
	 */
	public String postQuery(String Content)  {
		return postQuery(getURL(), Content);
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#postQuery(java.lang.String, java.lang.String)
	 */
	public String postQuery(String url, String Content)  {
		StringBuffer results = new StringBuffer();
		hasInfo = false;
		hasError = false;
		errorMessage = "";
		
		Log.trace("Content to POST:\n\turl : " + url + "\n\tcontent [" + Content + "]\n\ttoken [" + ( (token == null) ? "" : token) + "]\n", this.getClass().getName());
		try {	
			URL APIUrl = new URL(url);
			HttpURLConnection.setFollowRedirects(true);
			
			if (!"".equals(token) && token != null) { //$NON-NLS-1$
				Content = Content + "&token=" + token; //$NON-NLS-1$
			} 
			Content = Content + "&Client=" + String.valueOf(hexEncode(DEFAULT_CLIENT_STRING)); //$NON-NLS-1$
			
			HttpURLConnection conn = (HttpURLConnection) APIUrl.openConnection();
			conn.setRequestMethod("POST"); //$NON-NLS-1$
			conn.setDoInput(true);
			conn.setDoOutput(true);
			conn.setUseCaches(false);
			// looks like RAD doesn't like the connection timed outs.
			//conn.setConnectTimeout(CONNECT_TIMEOUT);
			// conn.setReadTimeout(READ_TIMEOUT);
			
			OutputStream out = conn.getOutputStream();
			OutputStreamWriter wout = new OutputStreamWriter(out);
			wout.write(Content);
			wout.flush();
			out.close();
			
			InputStream in = conn.getInputStream();
			InputStreamReader inputReader = new InputStreamReader(in);
			BufferedReader buffReader = new BufferedReader(inputReader);
			String line = null;
			boolean continuation = false;
			Log.trace("Results from Post: ", getClass().getName());
			
			while ( (line = buffReader.readLine()) != null) {
				if (line.indexOf("=") == -1)
					continuation = true;
				else 
					continuation = false;
				Log.trace("*** " + line, getClass().getName());
				String[] params = line.split("="); //$NON-NLS-1$
				if ("_error".equals(params[0])) 
					Log.trace("\n\n!!!! ERROR : " + line + "\n\n", getClass().getName());
				results.append(params[0]);
				
				if (params.length > 1) {
					if (!"_token".equals(params[0])) { //$NON-NLS-1$
						try {
							results.append("=");	 //$NON-NLS-1$
							for (int i = 1; i < params.length; i++) {
								results.append(hexDecode(params[i].toCharArray()));
								if (i < (params.length - 1)) {
									results.append("="); //$NON-NLS-1$
								}
							}
							results.append("\n");		 //$NON-NLS-1$
						} catch (DecoderException de) {
							Log.errorLogMessage(Messages.getString("BuildForgeConnection.MESSAGE_DECODE_ERROR")  //$NON-NLS-1$
									+ de.getMessage(), Log.ERROR, this.getClass().getName(), 
									CorePlugin.getDefault().getBundle().getSymbolicName());
							setErrorMessage(Messages.getString("BuildForgeConnection.MESSAGE_DECODE_ERROR_CONS") + params[1] + "]"); //$NON-NLS-1$ //$NON-NLS-2$
						}
						
					} else {
						results.append("="+params[1]+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
						if ("".equals(params[1])) { //$NON-NLS-1$
							authError = true;
						}
						token = params[1];
					}
				} else {
					if (params[0].length() > 0) { 
						// empty string after the '=' or no '='
						if (continuation) {
							results.append("\n");
						}
						results.append("=\n");
					}
				}
			
				if ("_error".equals(params[0])) { //$NON-NLS-1$
					authError = true;
					hasError = true;
					String err = "";
					try {
						err = hexDecode(params[1].toCharArray());
						Log.trace("\tServer Error : " + err + "\n", getClass().getName());
						setErrorMessage(err);
					} catch (Exception de) {
						setErrorMessage(params[1]);
					}
					if (SESSION_INVALID.equals(err)) {
						Log.trace("\tINVALID SESSION : marked for repost\n", getClass().getName());
						
						String defaultError = Messages.getString("BuildForgeConnection.SERVER_AUTH_ERROR"); //$NON-NLS-1$
						if (hasError) {
							defaultError = getErrorMessage();
						}
						setErrorMessage(Messages.getString("BuildForgeConnection.COMMUNICATIONS_ERROR") + defaultError); //$NON-NLS-1$
						
						try {
							token = null;
							authenticate(); 
						} catch (BuildForgeException bfe) { hasError = true; authError = true; } 
						repost = true;
					}
					Log.trace("\tCommunications Error : " + getErrorMessage(), this.getClass().getName()); //$NON-NLS-1$
					Log.trace("\tResults : " + results, this.getClass().getName()); //$NON-NLS-1$

				}
				
				if ("SQL_ERROR".equals(params[0])) {
					hasError = true;
					try {
						setErrorMessage(hexDecode(params[1].toCharArray()));
					} catch (Exception de) {
						setErrorMessage(params[1]);
					}
					Log.trace("\tSQL Error : " + getErrorMessage(), this.getClass().getName());
					Log.trace("\tResults: " + results, this.getClass().getName());
				}
				
				if ("_info".equals(params[0])) { //$NON-NLS-1$
					hasInfo = true;
					setInfoMessage(params[1]);
					Log.trace("\tINFO : " + params[1], this.getClass().getName());
				}
				
				if ("_version".equals(params[0])) {

					try {
						String consoleVer = hexDecode(params[1].toCharArray());
						setConsoleVersion(consoleVer);
						Log.trace("\tVersion : " + consoleVer, this.getClass().getName());
					} catch (DecoderException de) {
						setErrorMessage(params[1]);
					}
					
				}
					
			}
			
			if (authError || token == null || "".equals(token)) {
				String defaultError = Messages.getString("BuildForgeConnection.SERVER_AUTH_ERROR"); //$NON-NLS-1$
				if (hasError) {
					defaultError = getErrorMessage();
				}
				setErrorMessage(Messages.getString("BuildForgeConnection.COMMUNICATIONS_ERROR") + defaultError); //$NON-NLS-1$
			}
			
			in.close();
		    out.close();
		    conn.disconnect();
		} catch (MalformedURLException mue) {
			// report on bad url
			Log.errorLogMessage(Messages.getString("BuildForgeConnection.BAD_URL_ERROR") + url + "\nError: " + mue.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
					Log.ERROR, this.getClass().getName(), 
					CorePlugin.getDefault().getBundle().getSymbolicName());
			authError = true;
			setErrorMessage(BAD_URL + "  " + mue.getMessage()); //$NON-NLS-1$
		} catch (IOException ioe) {
			Log.errorLogMessage(Messages.getString("BuildForgeConnection.IO_ERROR_MESSAGE")  //$NON-NLS-1$
					+ ioe.getMessage(), Log.ERROR, this.getClass().getName(), 
					CorePlugin.getDefault().getBundle().getSymbolicName());
			authError = true;
			setErrorMessage(CONNECT_ERROR + "  " + ioe.getMessage()); //$NON-NLS-1$
		} catch (IllegalArgumentException iae) {
			Log.errorLogMessage(Messages.getString("BuildForgeConnection.UNAVAILABLE_HOST_ERROR")  //$NON-NLS-1$
					+ iae.getMessage(), Log.ERROR, this.getClass().getName(), 
					CorePlugin.getDefault().getBundle().getSymbolicName());
			setErrorMessage(BAD_HOST_ENTRY + "  " + iae.getMessage()); //$NON-NLS-1$
		}
		
		//Log.trace("results are [" + results.toString() + "]\n", getClass().getName());
		
		return results.toString();
	}
		
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#authenticate()
	 */
	public boolean authenticate ( ) throws BuildForgeException {
		boolean validated = false;
		/* if ((token != null && !"".equals(token)) || "".equals(getURL())) //$NON-NLS-1$ //$NON-NLS-2$
			return true; */
		authError = false;
		StringBuffer contentToPost = new StringBuffer();
		
		if (!"".equals(domain)) { //$NON-NLS-1$
			contentToPost.append("&domain=" + String.valueOf(hexEncode(domain))); //$NON-NLS-1$
		}
		contentToPost.append("&login=" + String.valueOf(hexEncode(userName))); //$NON-NLS-1$
		contentToPost.append("&password=" + String.valueOf(hexEncode(password))); //$NON-NLS-1$
		
		if (host == null)
			throw new BuildForgeAuthenticationException(Messages.getString("BuildForgeConnection.UNDEFINED_HOST_ERROR")); //$NON-NLS-1$

		String results = postQuery(getURL(), contentToPost.toString());
		
		Log.trace("results from Auth is : " + results, this.getClass().getName());
		String[] nameValParams = results.split("\n"); //$NON-NLS-1$
		for (int i = 0; i < nameValParams.length; i++) {
			if (nameValParams[i].indexOf("=") != -1) {
				String[] params = nameValParams[i].split("="); //$NON-NLS-1$
				if (params.length > 0 && "_token".equals(params[0])) { //$NON-NLS-1$
					if (params.length > 1) {
						Log.trace("Token found in auth results [" + params[1] + "]\n", getClass().getName());
						token = params[1];
						validated = true;
					} else {
						token = null;
						validated = false;
					}
				}
			}
		} 
		
		if (authError) {
			String err = getErrorMessage();
			if (err == null || "".equals(err)) //$NON-NLS-1$
					err = new String(DEFAULT_AUTHENTICATION_ERROR);
			throw new BuildForgeAuthenticationException(err);
		}
		
		if (hasError) {
			throw new BuildForgeException(getErrorMessage());
		}
		
		return validated;
	}

	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#authenticate(java.lang.String, java.lang.String, java.lang.String, int)
	 */
	public boolean authenticate (String user, String password, String host) throws BuildForgeException {
			setUserName(user);
			setPassword(password);
			setHost(host);
			//setPortNumber(port);
			return authenticate();
	}
	
	public boolean authenticate (String user, String password, String host, String domain) throws BuildForgeException {
		setUserName(user);
		setPassword(password);
		setHost(host);
		setDomain(domain);
		return authenticate();
	}
	
	
	public boolean logOff () {
		if (token == null || "".equals(token))
			return true;

		StringBuffer contentToPost = new StringBuffer();
		contentToPost.append("&Command=" +  String.valueOf(hexEncode(API_LOGOFF_CMD)));
		String post = postQuery(contentToPost.toString());
		
		token = "";
		if (hasError) {
			return false;
		}
		return true;
	}
	
	
	/* returns a List of Hashtables.  One Hashtable per project. */
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#processResponseRecords(java.lang.String)
	 */
	public List processResponseRecords(String response) {
		if (response == null)
			return null;
		ArrayList results = new ArrayList();
		Hashtable values = new Hashtable();
		String currentName = "";
		
		Log.trace("received response record of [" + response.length() + "] size", this.getClass().getName());
		//Log.trace(response, this.getClass().getName());
		String[] nameValParams = response.split("\n"); //$NON-NLS-1$

		for (int i = 0; i < nameValParams.length; i++) {

			StringBuffer paramVal = new StringBuffer();
			StringBuffer name = new StringBuffer();
			boolean continuation = false;
			
			if (nameValParams[i].indexOf("=") != -1) {
				String[] params = nameValParams[i].split("="); //$NON-NLS-1$
			
				// multiple passes in case we have split on '=' in the value field.
				for (int j = 0; j < params.length; j++) {
					if (j < 1) {
						name.append(params[j]);
						currentName = params[j];
					} else {
						paramVal.append(params[j]);
						if (!continuation && j < (params.length - 1)) {
							paramVal.append("="); //$NON-NLS-1$
						}
					}
				}
			} else {
				// if this is a new line in the data handle it.
				continuation = true;
				Log.trace("Data Continuation for [" + currentName + "]\n", getClass().getName());
				name.append(currentName);
				paramVal.append(nameValParams[i]);
			}
			
			if ("_record".equals(name.toString())) { //$NON-NLS-1$
				results.add(values);
				values = new Hashtable();			
			}  else {
				if (continuation) {
					String newValues = (String) values.get(name.toString()) + System.getProperty("line.separator") + paramVal.toString();
					values.put(name.toString(), newValues);
				} else {
					values.put(name.toString(), paramVal.toString());
				}
			}
			
		} 
		results.add(values);
		
		return results;
	} 

	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getURL()
	 */
	public String getURL() {
		String pre = "http://"; //$NON-NLS-1$
		if (host == null) {
			Log.errorLogMessage(Messages.getString("BuildForgeConnection.NULL_HOST_ERROR"),  //$NON-NLS-1$
					Log.WARNING, this.getClass().getName(), CorePlugin.getDefault().getBundle().getSymbolicName());
			return null;
		}
		if (host.length() > 7 && pre.equals(host.substring(0, 7)))
			return host + baseURL;
		else 
			return pre + host + baseURL;
	}
	
	public String getURL(String host, String baseURL) {
		setHost(host);
		//setPortNumber(port);
		setBaseURL(baseURL);
		return getURL();
	}	
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getURL(java.lang.String)
	 */
	public String getURL(String host) {
		setHost(host);
		return getURL();
	}
	
	public String decodeString(String value) {
		String newValue = ""; //$NON-NLS-1$
		try {
			newValue = hexDecode(value.toCharArray());
		} catch(DecoderException de) {
			Log.errorLogMessage(Messages.getString("BuildForgeConnection.TOKEN_DECODE_ERROR")  //$NON-NLS-1$
					+ de.getMessage(), Log.ERROR, this.getClass().getName(), 
					CorePlugin.getDefault().getBundle().getSymbolicName());
			setErrorMessage(Messages.getString("BuildForgeConnection.TOKEN_DECODE_ERROR_CONS") + token + "]"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return newValue;
	}
	
	public static String hexDecode (char[] message) throws DecoderException {
		String val = "";
		try {
			val = new String(Hex.decodeHex(message), "UTF-8");
		} catch (UnsupportedEncodingException e) {
			throw new DecoderException("Unsupported Encoding: UTF-8");
		}
		return val;
	}

	public static char[] hexEncode (String text) {
		char[] val;
		try {
			val = Hex.encodeHex(text.getBytes("UTF-8"));
		} catch (UnsupportedEncodingException e) {
			val = null;
		}
		return val;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getBaseURL()
	 */
	public String getBaseURL() {
		return baseURL;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#setBaseURL(java.lang.String)
	 */
	public void setBaseURL(String baseURL) {
		this.baseURL = baseURL;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getHost()
	 */
	public String getHost() {
		return host;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#setHost(java.lang.String)
	 */
	public void setHost(String host) {
		this.host = host;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getPassword()
	 */
	public String getPassword() {
		return password;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#setPassword(java.lang.String)
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getToken()
	 */
	public String getToken() {
		return token;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#setToken(java.lang.String)
	 */
	public void setToken(String token) {
		Log.trace("Setting token to [" + ((token == null) ? "" : token) + "]" , getClass().getName());
		this.token = token;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getUserName()
	 */
	public String getUserName() {
		return userName;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#setUserName(java.lang.String)
	 */
	public void setUserName(String userName) {
		this.userName = userName;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#isHasError()
	 */
	public boolean isHasError() {
		return hasError;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#setHasError(boolean)
	 */
	public void setHasError(boolean hasError) {
		this.hasError = hasError;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getErrorMessage()
	 */
	public String getErrorMessage() {
		return errorMessage;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#setErrorMessage(java.lang.String)
	 */
	public void setErrorMessage(String errorMessage) {
		if ("".equals(errorMessage)) { //$NON-NLS-1$
			hasError = false;
			this.errorMessage = "";
		}
		hasError = true;
		this.errorMessage = errorMessage;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#getInfoMessage()
	 */
	public String getInfoMessage() {
		return infoMessage;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IBuildForgeConnection#setInfoMessage(java.lang.String)
	 */
	public void setInfoMessage(String infoMessage) {
		this.infoMessage = infoMessage;
	}

	public boolean isAuthError() {
		return authError;
	}

	public void setAuthError(boolean authError) {
		this.authError = authError;
	}

	public String getDomain() {
		return domain;
	}

	public void setDomain(String domain) {
		this.domain = domain;
	}

	public String getConsoleVersion() {
		return consoleVersion;
	}

	public void setConsoleVersion(String consoleVersion) {
		this.consoleVersion = consoleVersion;
	}
	

	
}

