/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */
package com.buildforge.buildmodel.core.internal;

import java.io.Serializable;
import java.util.ArrayList;

import com.buildforge.buildmodel.core.IBuildRecord;
import com.buildforge.buildmodel.core.ILogRecordElement;
import com.buildforge.buildmodel.core.IProjectNode;

public class BuildRecord implements IBuildRecord, Serializable {

	private String buildTag = null;
	private int buildId;
	private int projectId;
	private String status = null;
	private String owner = null;
	private String result = null;
	private IProjectNode parent = null;
	private String consoleVersion;
	
	public BuildRecord() {
		super();
	}
	
	public BuildRecord(String buildTag, int buildId, int projectId, String status, String owner, String result) {
		this.buildTag = buildTag;
		this.buildId = buildId;
		this.projectId = projectId;
		this.status = status;
		this.owner = owner;
		this.result = result;
	}
	
	
	/** 
	 * getBuildLogRecords() fetches all Steps and their Result status for the current buildId and ProjectId
	 * @return ILogRecordElement[] 
	 */
	public ILogRecordElement[] getBuildLogRecords() {
		Builds build = new Builds();
		build.setUserName(parent.getParent().getUser());
		build.setHost(parent.getParent().getHost());
		build.setPassword(parent.getParent().getPassword());
		if (!"".equals(parent.getParent().getLdapDomain())) {
			build.setDomain(parent.getParent().getLdapDomain());
		}
		build.setToken(parent.getParent().getAuthenticationTokens());
		parent.getParent().queryConsoleForVersion();
		build.setConsoleVersion(parent.getParent().getVersion());
		BuildLogRecord log = new BuildLogRecord((ArrayList) build.getStepListResults(buildId, projectId));
		setConsoleVersion(build.getConsoleVersion());
		return log.getLogRecordElements();
	}

	
	/* (non-Javadoc)
	 * @see com.buildforge.internal.IBuildRecord#getBuildId()
	 */
	public int getBuildId() {
		return buildId;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IBuildRecord#setBuildId(int)
	 */
	public void setBuildId(int buildId) {
		this.buildId = buildId;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IBuildRecord#getBuildTag()
	 */
	public String getBuildTag() {
		return buildTag;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IBuildRecord#setBuildTag(java.lang.String)
	 */
	public void setBuildTag(String buildTag) {
		this.buildTag = buildTag;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IBuildRecord#getProjectId()
	 */
	public int getProjectId() {
		return projectId;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IBuildRecord#setProjectId(int)
	 */
	public void setProjectId(int projectId) {
		this.projectId = projectId;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IBuildRecord#getStatus()
	 */
	public String getStatus() {
		return status;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.internal.IBuildRecord#setStatus(java.lang.String)
	 */
	public void setStatus(String status) {
		this.status = status;
	}

	public String toString() {
		return this.buildTag;
	}

	public IProjectNode getParent() {
		return parent;
	}

	public void setParent(IProjectNode project) {
		parent = project;
	}
	
	/**
	 * getStepDetailRecords() - gets all step results and places them into the returned StepDetailRecord[].
	 *   This method is what is used to populate step results in the step log (top level results and step names).
	 * return Object[]
	 */
	public Object[] getStepDetailRecords() {
		ILogRecordElement[] buildLogRecords = getBuildLogRecords();
		ArrayList steps = new ArrayList();
		int currStepId = 1;
		String stepName = null;
		char stepResult = NO_RESULT;

		for (int i = 0; i < buildLogRecords.length; i++) {				
			if (stepName == null) {
				stepName = buildLogRecords[i].getStepName();
			}
			
			if (stepResult == NO_RESULT) {
				stepResult = buildLogRecords[i].getResult().charAt(0);
			}
			currStepId = buildLogRecords[i].getStepId();
			if (consoleVersion == null || "".equals(consoleVersion))    // 3.8
				steps.add(new StepDetailRecord(buildId, currStepId, stepName, null, null, stepResult));
			else   // 4.0+
				steps.add(new StepDetailRecord(buildId, currStepId, stepName, null, null, stepResult, buildLogRecords[i].getLogId()));
			stepResult = NO_RESULT;
			stepName = null;
		}
		return steps.toArray();
	}
	
	public String getOwner() {
		return owner;
	}

	public void setOwner(String owner) {
		this.owner = owner;
	}

	public String getResult() {
		return result;
	}

	public void setResult(String result) {
		this.result = result;
	}

	public String getConsoleVersion() {
		return consoleVersion;
	}

	public void setConsoleVersion(String consoleVersion) {
		this.consoleVersion = consoleVersion;
	}
	
}
