/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */
package com.buildforge.buildmodel.core.internal;

import java.util.ArrayList;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.ListenerList;

import com.buildforge.buildmodel.core.IBuildRecord;
import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.IConsoleList;
import com.buildforge.buildmodel.core.IProjectNode;
import com.buildforge.buildmodel.core.util.Log;

public class ConsoleList extends ArrayList implements IConstants, IConsoleList {

	private transient ListenerList propertyChangeListeners = null;
	
	private  static final String NEW_CONSOLE_NAME = Messages.getString("ConsoleList.NEW_CONSOLE_LABEL"); //$NON-NLS-1$
	private IConsole activeConsole = null;
	private IProjectNode activeProjectNode = null;
	private IBuildRecord activeBuildRecord = null;
	private BuildLogRecord activeBuildLogRecord = null;
	private int buildExecuted = 0;

	
	public ConsoleList() {
		super();
		Log.trace("ConsoleList created.", this.getClass().getName()); //$NON-NLS-1$
		initList();
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#getActiveBuildLogRecord()
	 */
	public BuildLogRecord getActiveBuildLogRecord() {
		return activeBuildLogRecord;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#setActiveBuildLogRecord(com.buildforge.buildmodel.core.IBuildLogRecord)
	 */
	public void setActiveBuildLogRecord(BuildLogRecord activeBuildLogRecord) {
		this.activeBuildLogRecord = activeBuildLogRecord;
	}
	
	public void initList() {
		clear();
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#getConsoles()
	 */
	public Object[] getConsoles() {
		return toArray();
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#addConsole(java.lang.String, java.lang.String, int, java.lang.String, java.lang.String)
	 */
	public IConsole addConsole(String name, String host, int portNumber, String user, String password) {
		Console con = new Console(name, host, portNumber, user, password);
		this.add(con);  
		// firePropertyChangeEvent(ADD_EVENT, null, con);
		return con;
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#removeConsole(com.buildforge.buildmodel.core.IConsole)
	 */
	public void removeConsole(IConsole console) {
		remove(console);
		// firePropertyChangeEvent(REMOVE_EVENT, console, null);
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#addPropertyChangeListener(org.eclipse.jface.util.IPropertyChangeListener)
	 */
	public void addPropertyChangeListener(IPropertyChangeListener listener) {
		 getPropertyChangeListeners().add(listener);
	}
	
	
	private ListenerList getPropertyChangeListeners() {
		 if (propertyChangeListeners == null)
		      propertyChangeListeners = new ListenerList();
		    return propertyChangeListeners;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#getActiveConsole()
	 */
	public IConsole getActiveConsole () {
		return activeConsole;
	}
	
	public IConsole getConsoleByName(String consoleName) {
		if (consoleName == null) 
			return null;
		
		for (int i = 0; i < size(); i++) {
			IConsole cons = (IConsole) get(i);
			if (cons != null) {
				if (consoleName.equals(cons.getName())) {
					return cons;
				}
			}
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#getActiveBuildRecord()
	 */
	public IBuildRecord getActiveBuildRecord() {
		return activeBuildRecord;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#setActiveBuildRecord(com.buildforge.buildmodel.core.IBuildRecord)
	 */
	public void setActiveBuildRecord(IBuildRecord activeBuildRecord) {
		this.activeBuildRecord = activeBuildRecord;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#getActiveProjectNode()
	 */
	public IProjectNode getActiveProjectNode() {
		return activeProjectNode;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#setActiveProjectNode(com.buildforge.buildmodel.core.IProjectNode)
	 */
	public void setActiveProjectNode(IProjectNode activeProjectNode) {
		this.activeProjectNode = activeProjectNode;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IConsoleList#setActiveConsole(com.buildforge.buildmodel.core.IConsole)
	 */
	public void setActiveConsole(IConsole activeConsole) {
		this.activeConsole = activeConsole;
	}
	
	
	public void setBuildExecuted(int projectId) {
		buildExecuted = projectId;
		
	}
	
	public int getBuildExecuted() {
		return buildExecuted;
	}
	
	public IConsole createEmptyConsoleEntry() {
		String consoleName = getUniqueConsoleName();
		return addConsole(consoleName, null, 80, null, null);
	}

	private String getUniqueConsoleName() {
		Log.trace("Aquiring unique name for new console", this.getClass().getName()); //$NON-NLS-1$
		boolean found = false;
		boolean matched = false;
		StringBuffer uniqueName = new StringBuffer(NEW_CONSOLE_NAME);
		int count = 0;
		
		while (!found) {
			matched = false;
			for (int i=0; i < size(); i++) {
				IConsole tmp = (IConsole) get(i);
				if (uniqueName.toString().equals(tmp.getName())) {
					count++;
					uniqueName.replace(NEW_CONSOLE_NAME.length(), uniqueName.length(), String.valueOf(count));
					matched = true;
					break;
				}
			}
			if (!matched)
				found = true;
		}
		Log.trace("Unique name is " + uniqueName.toString(), this.getClass().getName()); //$NON-NLS-1$
		return uniqueName.toString();
	}
	
	
	public String getConsoleListSaveData() {
		StringBuffer data = new StringBuffer("");
	
		Object[] cList = getConsoles();
		for (int i=0; i < cList.length; i++) {
			IConsole tmp = (IConsole) cList[i];
			data.append((i > 0) ? System.getProperty("line.separator") + tmp.getConsoleSaveData() : "" + tmp.getConsoleSaveData());
		}
		return data.toString();
	}
	
	
	public void initializeConsoleListWithSaveData(String cListData) {
		Log.trace("initializing ConsoleList from Saved Data.", getClass().getName());
		String[] lineData = cListData.split(System.getProperty("line.separator"));
		for (int j=0; j < lineData.length; j++) {
			String[] conData = lineData[j].split(String.valueOf(consoleLineDelim));
			
			for (int i=0; i < conData.length; i++) {
					String[] data = conData[i].split(String.valueOf(consoleDelim));
					if (data.length < 5) {
							Log.trace("Existing save data does not have enough elements.  It could be corrupted!!", this.getClass().getName());
							return;
					}
					
					IConsole tmpcon = addConsole(data[0], data[1], Integer.parseInt(data[2]), data[3], data[4]);
					
					// 3.8 version of the API doesn't return a version string for us.
					if (data.length == 5) {
						tmpcon.setLdapDomain("");
						tmpcon.setVersion("3.8");
					} else {
						tmpcon.setLdapDomain(data[5]);
						tmpcon.setVersion(data[6]);
					}
			}
		}
		Log.trace("ConsoleList Initialized.", getClass().getName());
	}
	
}
