package com.buildforge.buildmodel.core.internal;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import com.buildforge.buildmodel.core.util.Log;

/**
 * An Environment Group is used to contain a list of Environment objects
 * specifically this is being used for population of the UI for alteration of
 * environment settings at job run time.
 * 
 * Note this should go away when we move to the services layer as the 
 * services common dbo objects should take it's place.
 * @author heathm
 *
 */
public class EnvironmentGroup {
	private ArrayList envs = new ArrayList();
	private String groupName = ""; 
	private boolean ordered = false;
	/**
	 * Default constructor
	 *
	 */
	public EnvironmentGroup () {
		super();
	}
	
	
	/**
	 * Constructor that populates the Environment Group with a projects environment.
	 * @param projectId
	 */
	public EnvironmentGroup(Projects proj, int projectId) {
		ArrayList eList = (ArrayList) proj.listProjectEnvironment(projectId);
		// e.bf_eid, e.bf_level, e.bf_param, e.bf_value, e.bf_action, e.bf_mode
		for (int i = 0; i < eList.size(); i++) {
			if (((Object) eList.get(i)) instanceof Hashtable) {
				Hashtable element = (Hashtable) eList.get(i);
				Environment env = new Environment();
				
				int eId = 0;
				try { eId = Integer.parseInt( (String) element.get("bf_eid")); } 
				catch (NumberFormatException nfe) { Log.trace("bf_eid is " + (String) element.get("bf_eid"), this.getClass().toString()); }
				Log.trace("eid is " + eId, this.getClass().toString());
				
				if (eId == 0) {
					// This is our main entry used for describing the group. 
					// The table is currently overloaded in this way.
					groupName = (String) element.get("bf_param");
				} else {
					env.setName((String) element.get("bf_param"));
					Log.trace("bf_value was " + (String) element.get("bf_value"), this.getClass().toString());
					env.setValue((String) element.get("bf_value"));
					Log.trace("env.getName() is now " + env.getValue(), this.getClass().toString());
					Log.trace("ValueOptions are " + env.getValueOptions(), this.getClass().toString());
					env.setAction((String) element.get("bf_action"));
					env.setMode((String) element.get("bf_mode"));
					this.addEnv(env);
				}
			}
		}
	}
	
	/**
	 *  Add an Environment Object to our Environment Group.
	 * @param env
	 */
	public void addEnv(Environment env) {
		envs.add(env);
		ordered = false;
	}
	
	public void newEnvironmentEntry(String name, String value, String action, String mode) {
		Environment e = new Environment();
		e.setName(name);
		e.setValue(value);
		e.setAction(action);
		e.setMode(mode);
		if (!ordered) {
			orderByPosition();
		}
		int position = 0;
		if (envs.size() > 0) 
			position = ((Environment) envs.get(envs.size() - 1)).getPosition() + 1;
		e.setPosition(position);
		envs.add(e);
		ordered = true;
	}
	
	/** 
	 * Grab the Environment object with the given name
	 * @param name
	 * @return Environment Object or null
	 */
	public Environment getEnv(String name) {
		
		for (int i=0; i < envs.size(); i++) {
			if ( ((Environment) envs.get(i)).getName().equals(name)) {
				return (Environment) envs.get(i);
			}
		}
		return null;
	}
	
	public void replaceEnv(String name, Environment env) {
		for (int i=0; i < envs.size(); i++) {
			if ( ((Environment) envs.get(i)).getName().equals(name)) {
				envs.set(i, env);
				ordered = false;
			}
		}
	}
	
	
	public List getList() {
		return envs;
	}
	
	public void setChoice(int i, String choice) {
		Log.trace("Setting choice for element " + i + " to [" + choice + "]", this.getClass().getName());
		((Environment) envs.get(i)).setChoice(choice);
	}
	
	public void setChoice(String varName, String choice) {
		for (int i=0; i < envs.size(); i++) {
			if ( ((Environment) envs.get(i)).getName().equals(varName)) {
				Log.trace("Setting choice for element variable " + varName + " to [" + choice + "]", this.getClass().getName());
				((Environment) envs.get(i)).setChoice(choice);		
			}
		}
	}
	
	
	public void orderByPosition() {
		Collections.sort(envs, new Comparator () {
			public int compare(Object o1, Object o2) {
				return ((Environment)o1).compareTo((Environment)o2);
			} 
		});
		ordered = true;
	}
}
