/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */
package com.buildforge.buildmodel.core.internal;

import java.util.ArrayList;

import com.buildforge.buildmodel.core.IStepDetailInfo;
import com.buildforge.buildmodel.core.IStepDetailTypeInfo;


public class StepDetailInfo implements IStepDetailInfo {
	
	private static final String DETAILS_FOLDER_NAME = Messages.getString("StepDetailInfo.DETAILS_FOLDER_NAME_LABEL"); //$NON-NLS-1$
	private String[] detail;
	private StepDetailRecord parent;
	private ArrayList types = null;
		
	public StepDetailInfo (String[] detail, StepDetailRecord parent) {
		this.detail = detail;
		this.parent = parent;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailInfo#getDetail()
	 */
	public String[] getDetail() {
		return detail;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailInfo#setDetail(java.lang.String)
	 */
	public void setDetail(String[] detail) {
		this.detail = detail;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailInfo#getParent()
	 */
	public StepDetailRecord getParent() {
		return parent;
	}

	/* (non-Javadoc)
	 * @see com.buildforge.buildmodel.core.internal.IStepDetailInfo#setParent(com.buildforge.buildmodel.core.internal.StepDetailRecord)
	 */
	public void setParent(StepDetailRecord parent) {
		this.parent = parent;
	}

	
	public String[] getDetailTypesNames () {
		String[] results = new String[types.size()];
		for (int i=0; i < types.size(); i++) {
			results[i] = types.get(i).toString();
		}
		return results;
	}
	
	public Object[] getDetailTypes () {
		return types.toArray();
	}
	
	
	public void generateTypeLists () {
		types = new ArrayList();
		String currType = "";

		int currIndex = -1;
		for (int i = 0; i < detail.length; i++) {
			int delim = detail[i].indexOf(":");
			String type;
			String msg;
			if (delim == -1) {
				type = DETAILS_FOLDER_NAME;
				msg = detail[i];
			} else {
				type = detail[i].substring(0,delim);
				msg = detail[i].substring(delim+1);
			}
			
			if (currType.equals(type)) {
				IStepDetailTypeInfo tInfo = (IStepDetailTypeInfo) types.get(currIndex);
				tInfo.addInfo(msg);
				types.set(currIndex, (IStepDetailTypeInfo) tInfo);
			} else {
				currIndex++;
				IStepDetailTypeInfo tInfo = new StepDetailTypeInfo(type, this);
				tInfo.addInfo(msg);
				types.add((IStepDetailTypeInfo) tInfo);
				currType = type;
			}
		}
	}
	
	
	public String toString () {
		return DETAILS_FOLDER_NAME;
	}

 	
}
