/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.core.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

import com.buildforge.buildmodel.core.internal.Console;
import com.buildforge.buildmodel.core.internal.IConnectable;
import com.buildforge.buildmodel.core.internal.IConstants;

public class ConnectionTest implements IConstants {
	IConnectable conn = null;
	String server = "";
	int port = 80;
	
	public ConnectionTest(Console console) {
		conn = console;
	}

	public boolean testHttpTimeout() throws UnknownHostException, IOException {
		// we should be speaking HTTP so just sending a 
		// GET / HTTP/1.0
		//
		// should get a response back from anything we are 
		// speaking with if not return false.
		
		transformHost(conn.getHost());
		
		try {
			Socket sock = new Socket(server, port);
			sock.setSoTimeout(CONNECT_TIMEOUT);
			
			PrintWriter out = new PrintWriter(sock.getOutputStream(), true);
			BufferedReader in = new BufferedReader(new InputStreamReader(sock.getInputStream()));
			
			out.print("GET / HTTP/1.0\n\n");
			while (in.readLine() != null) { }
			return true;
		} catch (SocketTimeoutException ste) {
			return false;
		}
	}
	
	private void transformHost(String host) {
		if (host == null || "".equals(host)) {
			server = "";
			return;
		}
		
		String pre = "http://";
		if (host.length() > 7 && pre.equals(host.substring(0,7))) {
			host = host.substring(7);
		}
		
		String[] hostinfo = host.split(":");
		server = hostinfo[0];
		if (hostinfo.length > 1 && !"".equals(hostinfo[1])) {
			try {
				String[] portSnatch = hostinfo[1].split("/");
				port = Integer.parseInt(portSnatch[0]);
			} catch (NumberFormatException nfe) {}
		}
	}

}
