/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.core.util;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.osgi.framework.Bundle;

import com.buildforge.buildmodel.core.CorePlugin;

public class Log {
	boolean trace = false;
	public final static int ERROR = IStatus.ERROR;
	public final static int WARNING = IStatus.WARNING;
	public final static int INFO = IStatus.INFO;
	public final static int OK = IStatus.OK;
	private final static String anyClass = "*";
	
	public static void trace(String message, String callingClassName) {
			String traceFilter = Platform.getDebugOption("com.buildforge.buildmodel.core/debug/filter");
			if ((traceFilter != null) && (traceFilter.equals(callingClassName) 
					|| traceFilter.equals(anyClass))) {
				System.out.println(message);
			}
	}
	
	public static void errorLogMessage(String message, int statusLevel, String className, String pluginId) {
		if (pluginId == null) {
			pluginId = CorePlugin.getDefault() .getBundle().getSymbolicName();
		}

		// MultiStatus errorStatus = new MultiStatus(pluginId, statusLevel, message, null);
		Status errorStatus = new Status(statusLevel, pluginId, 0, message, null);
		CorePlugin.getDefault() .getLog() .log(errorStatus);
	}
	
	public static MultiStatus getMultiStatus(Bundle bundle, Exception exception, int statusLevel) {
		String symName = bundle.getSymbolicName();
		String bundleName = bundle.getHeaders().get("Bundle-Name").toString();
		String bundleVendor = bundle.getHeaders().get("Bundle-Vendor").toString();
		String bundleVersion = bundle.getHeaders().get("Bundle-Version").toString();
		
		MultiStatus mStatus = new MultiStatus(symName, statusLevel ,exception.getMessage(), exception);
		
		mStatus.add(createStatus(statusLevel, "Plug-in Vendor: " + bundleVendor, bundle));
		mStatus.add(createStatus(statusLevel, "Plug-in Name: " + bundleName, bundle));
		mStatus.add(createStatus(statusLevel, "Plug-in ID: " + symName, bundle));
		mStatus.add(createStatus(statusLevel, "Version: " + bundleVersion, bundle));
		mStatus.add(createStatus(statusLevel, "Exception: " + exception.getClass().getName(), bundle));
		mStatus.add(createStatus(statusLevel, "Exception Message: " + exception.getMessage(), bundle));
		mStatus.add(createStatus(statusLevel, "Stack Trace: ", bundle));
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintStream ps = new PrintStream(baos);
		exception.printStackTrace(ps);
		ps.close();
		String result = baos.toString();
		String lineSep = System.getProperty("line.separator");
		String[] stackChunks = result.split(lineSep);
		
		for (int i = 0; i < stackChunks.length; i++) {
			mStatus.add(createStatus(statusLevel, "  " + stackChunks[i], bundle));
		}
		return mStatus;
	}
	
	public static Status createStatus(int statusLevel, String msg, Bundle bundle) {
		return new Status(statusLevel, bundle
		        .getSymbolicName(), IStatus.OK, msg, null);
	}
	
}
