package Tests;

import java.util.Random;

import com.buildforge.buildmodel.core.internal.EnvironmentValue;

import junit.framework.TestCase;

public class TestEnvironmentValue extends TestCase {
	
	public void testEnvironmentValue() throws Exception{
		EnvironmentValue value = new EnvironmentValue(null);
		assertNull(value.getValue());
		String testVal = ""+new Random().nextInt();
		value.setValue(testVal);
		assertEquals(testVal, value.getValue());
	}
	
	public void testEnvironmentValueOptions() throws Exception{
		//Test single element
		EnvironmentValue value = new EnvironmentValue("foo");
		assertEquals("foo", value.getValue());
		assertNull(value.getOptions());
		//Test two elements
		value = new EnvironmentValue("foo|bar");
		assertEquals("foo", value.getValue());
		assertNotNull(value.getOptions());
		assertEquals(2, value.getOptions().length);
		assertEquals("foo", value.getOptions()[0]);
		assertEquals("bar", value.getOptions()[1]);
		assertEquals("foo|bar", value.getOptionsAsDelimitedString());		
		//Test three elements
		value = new EnvironmentValue("foo|bar|baz");
		assertEquals("foo", value.getValue());
		assertNotNull(value.getOptions());
		assertEquals(3, value.getOptions().length);
		assertEquals("foo", value.getOptions()[0]);
		assertEquals("bar", value.getOptions()[1]);
		assertEquals("baz", value.getOptions()[2]);
		assertEquals("foo|bar|baz", value.getOptionsAsDelimitedString());
		//Test three elements with prepended multi-delimiters
		value = new EnvironmentValue("||||||||||||||||foo|bar|baz");
		assertEquals("foo", value.getValue());
		assertNotNull(value.getOptions());
		assertEquals(3, value.getOptions().length);
		assertEquals("foo", value.getOptions()[0]);
		assertEquals("bar", value.getOptions()[1]);
		assertEquals("baz", value.getOptions()[2]);
		assertEquals("foo|bar|baz", value.getOptionsAsDelimitedString());
		//Test three elements with appended multi-delimiters
		value = new EnvironmentValue("foo|bar|baz||||||||||||||||");
		assertEquals("foo", value.getValue());
		assertNotNull(value.getOptions());
		assertEquals(3, value.getOptions().length);
		assertEquals("foo", value.getOptions()[0]);
		assertEquals("bar", value.getOptions()[1]);
		assertEquals("baz", value.getOptions()[2]);
		assertEquals("foo|bar|baz", value.getOptionsAsDelimitedString());
		//Test three elements with both prepended and appended multi-delimiters
		value = new EnvironmentValue("||||||||||||||||foo|bar|baz||||||||||||||||");
		assertEquals("foo", value.getValue());
		assertNotNull(value.getOptions());
		assertEquals(3, value.getOptions().length);
		assertEquals("foo", value.getOptions()[0]);
		assertEquals("bar", value.getOptions()[1]);
		assertEquals("baz", value.getOptions()[2]);
		assertEquals("foo|bar|baz", value.getOptionsAsDelimitedString());
		//Test three elements with both prepended and appended multi-delimiters as well as interstitial
		value = new EnvironmentValue("||||||||||||||||foo||||bar||||||baz||||||||||||||||");
		assertEquals("foo", value.getValue());
		assertNotNull(value.getOptions());
		assertEquals(3, value.getOptions().length);
		assertEquals("foo", value.getOptions()[0]);
		assertEquals("bar", value.getOptions()[1]);
		assertEquals("baz", value.getOptions()[2]);
		assertEquals("foo|bar|baz", value.getOptionsAsDelimitedString());
	}
}