/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.reflector.internal;

import java.util.ArrayList;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.QualifiedName;

import com.buildforge.buildmodel.core.CorePlugin;
import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.reflector.Messages;
import com.buildforge.buildmodel.reflector.ReflectorPlugin;



public class ReflectorFavoriteItem {
	public static final QualifiedName BUILDFORGE_ENABLEDKEY = 
		new QualifiedName(ReflectorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), "BUILDFORGE_ENABLED"); //$NON-NLS-1$
	public static final QualifiedName BUILDFORGE_PROJECT_PROPERTYKEY= 
		new QualifiedName(ReflectorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), "BUILDFORGE_PROJECT_PROP"); //$NON-NLS-1$
	public static final QualifiedName BUILDFORGE_CONSOLEKEY = 
		new QualifiedName(ReflectorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), "BUILDFORGE_CONSOLE"); //$NON-NLS-1$
	public static final QualifiedName BUILDFORGE_PROJECTIDKEY = 
		new QualifiedName(ReflectorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), "BUILDFORGE_PROJECTID"); //$NON-NLS-1$
	public static final QualifiedName BUILDFORGE_STEPIDKEY = 
		new QualifiedName(ReflectorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), "BUILDFORGE_STEPID");	 //$NON-NLS-1$
	public static final QualifiedName BUILDFORGE_STEPUNIQUEIDKEY = 
		new QualifiedName(ReflectorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), "BUILDFORGE_STEPUNIQUEID");
	public static final QualifiedName BUILDFORGE_PROJECTPATH = 
		new QualifiedName(ReflectorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), "BUILDFORGE_PROJECTPATH"); //$NON-NLS-1$
	private static final String className = "com.buildforge.buildmodel.reflector.internal.ReflectorFavoriteItem"; //$NON-NLS-1$
	
	private String eclipseProjectPath = null;
	private ArrayList reflectFiles = null;
	private IConsole console = null;
	private int projectId;
	private int stepId;
	

	public ArrayList getReflectFiles() {
		return reflectFiles;
	}

	public void setReflectFiles(ArrayList reflectFiles) {
		this.reflectFiles = reflectFiles;
	}

	public IConsole getConsole() {
		return console;
	}

	public void setConsole(IConsole console) {
		this.console = console;
	}

	public int getProjectId() {
		return projectId;
	}

	public void setProjectId(int projectId) {
		this.projectId = projectId;
	}

	public int getStepId() {
		return stepId;
	}

	public void setStepId(int stepId) {
		this.stepId = stepId;
	}
	
	
	public static boolean getBuildForgeEnabledPropertyValue(IAdaptable element) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		try {
			String value = resource.getPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_ENABLEDKEY);
			if (value == null || "".equals(value)) { //$NON-NLS-1$
				Log.trace("Build Forge Enabled value is null or empty string.", className); //$NON-NLS-1$
				return false;
			}
			Log.trace("Build Forge Enabled value is : " + value, className); //$NON-NLS-1$
			return true;
		} catch (CoreException ce) {
			Log.trace("Core exception while fetching persistent property for on resource [" + resource.getName() + "]", className); //$NON-NLS-1$ //$NON-NLS-2$
			Log.errorLogMessage(Messages.getString("ReflectorFavoriteItem.FETCH_PERSISTANT_PROPERTY_ERROR_CONS"),  //$NON-NLS-1$
					Log.ERROR, className, ReflectorPlugin.getDefault().getBundle().getSymbolicName());
		}
		return false; 
	}

	
	public static void setBuildForgeEnabledPropertyValue(IAdaptable element, boolean isEnabled) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		String value = null;
		if (isEnabled) {
			value = "checked"; //$NON-NLS-1$
		}
		try {
			resource.setPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_ENABLEDKEY, value);
		} catch (CoreException ce) {
			Log.trace("Error setting Persistent property.", className); //$NON-NLS-1$
		}
	}
	
	public static String[] getCheckedFiles(IAdaptable element) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		String resultsString = null;
		
		try {
			resultsString = resource.getPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_PROJECT_PROPERTYKEY);
		} catch (CoreException ce) {
			Log.trace("Core exception while fetching persistent property for checked Tree items", className); //$NON-NLS-1$
			Log.errorLogMessage(Messages.getString("ReflectorFavoriteItem.FETCH_PERSISTANT_TREE_PROPERTY_ERROR_CONS"),  //$NON-NLS-1$
					Log.ERROR, className, ReflectorPlugin.getDefault().getBundle().getSymbolicName());
		}
		
		if (resultsString == null) {
			Log.trace("Get Checked Files : no prior configuration.", className); //$NON-NLS-1$
			return null;
		}
		Log.trace("Get Checked Files : spliting string [" + resultsString + "] on ','.", className); //$NON-NLS-1$ //$NON-NLS-2$
		return resultsString.split(","); //$NON-NLS-1$
	}

	public static void setCheckedFiles(IAdaptable element, String[] filePaths) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		String value = ""; //$NON-NLS-1$
		if (filePaths == null) {
			value = null;
		} else {
			for (int i = 0; i < filePaths.length; i++) {
				value += filePaths[i] + ","; //$NON-NLS-1$
			}
		}
		
		try {
			resource.setPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_PROJECT_PROPERTYKEY, value);
		} catch (CoreException ce) {
			Log.trace("Error setting Persistent property.", className); //$NON-NLS-1$
		}
	}
	
	
	public static void setConsoleSelection(IAdaptable element, String consoleName) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		try {
			resource.setPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_CONSOLEKEY, consoleName);
		} catch (CoreException ce) {
			Log.trace("Error setting Persistent property.", className); //$NON-NLS-1$
		}
	}
	
	public static IConsole getConsoleSelection(IAdaptable element) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		String consoleName = ""; //$NON-NLS-1$
		try {
			consoleName = resource.getPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_CONSOLEKEY);
		} catch (CoreException ce) {
			Log.trace("Core exception while fetching persistent console property for [" + resource.getName() + "]", className); //$NON-NLS-1$ //$NON-NLS-2$
			Log.errorLogMessage(Messages.getString("ReflectorFavoriteItem.FETCH_PERSISTANT_PROPERTY_ERROR_CONS"),  //$NON-NLS-1$
					Log.ERROR, className, ReflectorPlugin.getDefault().getBundle().getSymbolicName());
			return null;
		}
		if (consoleName == null) 
			return null;
		
		return CorePlugin.getConsoleList().getConsoleByName(consoleName);		
	}

	public static void setProjectSelection(IAdaptable element, String selectedProject) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		try {
			resource.setPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_PROJECTIDKEY, selectedProject);
		} catch (CoreException ce) {
			Log.trace("Error setting Persistent property.", className); //$NON-NLS-1$
		}
	}
	
	public static String getProjectSelection(IAdaptable element) {
		IResource resource = (IResource) element.getAdapter(IResource.class);

		try {
			return  resource.getPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_PROJECTIDKEY);
		} catch (CoreException ce) {
			Log.trace("Core exception while fetching persistent console property for [" + resource.getName() + "]", className); //$NON-NLS-1$ //$NON-NLS-2$
			Log.errorLogMessage(Messages.getString("ReflectorFavoriteItem.FETCH_PERSISTANT_PROPERTY_ERROR_CONS"),  //$NON-NLS-1$
					Log.ERROR, className, ReflectorPlugin.getDefault().getBundle().getSymbolicName());
			return null;
		}
		
	}
	
	public static void setStepSelection(IAdaptable element, String selectedStep) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		try {
			resource.setPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_STEPIDKEY, selectedStep);
		} catch (CoreException ce) {
			Log.trace("Error setting Persistent property.", className); //$NON-NLS-1$
		}
	}
	
	public static String getStepSelection(IAdaptable element) {
		IResource resource = (IResource) element.getAdapter(IResource.class);

		try {
			return resource.getPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_STEPIDKEY);
		} catch (CoreException ce) {
			Log.trace("Core exception while fetching persistent console property for [" + resource.getName() + "]", className); //$NON-NLS-1$ //$NON-NLS-2$
			Log.errorLogMessage(Messages.getString("ReflectorFavoriteItem.FETCH_PERSISTANT_PROPERTY_ERROR_CONS"),  //$NON-NLS-1$
					Log.ERROR, className, ReflectorPlugin.getDefault().getBundle().getSymbolicName());
			return null;
		}
		
	}
	
	
	public static void setStepUniqueId(IAdaptable element, String stepUniqueId) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		try {
			resource.setPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_STEPUNIQUEIDKEY, stepUniqueId);
		} catch (CoreException ce) {
			Log.trace("Error setting persistent property.", className);
		}
	}
	
	
	public static String getStepUniqueId(IAdaptable element) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		try {
			return resource.getPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_STEPUNIQUEIDKEY);
		} catch (CoreException ce) {
			Log.trace("Core exception while fetching persistent console property for [" + resource.getName() + "]", className); //$NON-NLS-1$ //$NON-NLS-2$
			Log.errorLogMessage(Messages.getString("ReflectorFavoriteItem.FETCH_PERSISTANT_PROPERTY_ERROR_CONS"),  //$NON-NLS-1$
					Log.ERROR, className, ReflectorPlugin.getDefault().getBundle().getSymbolicName());
			return null;
		}
	}
	
	
	public static void setRelativeBuildForgePath(IAdaptable element, String path) {
		IResource resource = (IResource) element.getAdapter(IResource.class);
		
		try {
			resource.setPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_PROJECTPATH, path);
		} catch (CoreException ce) {
			Log.trace("Error setting Persistent property.", className); //$NON-NLS-1$
		}
	}
	
	public static String getRelativeBuildForgePath(IAdaptable element) {
		IResource resource = (IResource) element.getAdapter(IResource.class);

		try {
			return resource.getPersistentProperty(ReflectorFavoriteItem.BUILDFORGE_PROJECTPATH);
		} catch (CoreException ce) {
			Log.trace("Core exception while fetching persistent console property for [" + resource.getName() + "]", className); //$NON-NLS-1$ //$NON-NLS-2$
			Log.errorLogMessage(Messages.getString("ReflectorFavoriteItem.FETCH_PERSISTANT_PROPERTY_ERROR_CONS"),  //$NON-NLS-1$
					Log.ERROR, className, ReflectorPlugin.getDefault().getBundle().getSymbolicName());
			return null;
		}
		
	}

	public String getEclipseProjectPath() {
		return eclipseProjectPath;
	}

	public void setEclipseProjectPath(String eclipseProject) {
		this.eclipseProjectPath = eclipseProject;
	}
	
}
