/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.reflector.properties;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.dialogs.PropertyPage;

import com.buildforge.buildmodel.core.CorePlugin;
import com.buildforge.buildmodel.core.IConsole;
import com.buildforge.buildmodel.core.IConsoleList;
import com.buildforge.buildmodel.core.IProjectNode;
import com.buildforge.buildmodel.core.IStep;
import com.buildforge.buildmodel.core.exception.BuildForgeException;
import com.buildforge.buildmodel.core.internal.EnvironmentValue;
import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.reflector.IConstants;
import com.buildforge.buildmodel.reflector.Messages;
import com.buildforge.buildmodel.reflector.ReflectorPlugin;
import com.buildforge.buildmodel.reflector.internal.NatureToggle;
import com.buildforge.buildmodel.reflector.internal.ReflectorFavoriteItem;
import com.buildforge.buildmodel.reflector.internal.ReflectorStore;

/**
 * This is the base plugin for the Reflector product.
 * It is to be used with the frequency buildforge plugins
 * com.buildforge.buildmodel.*
 */

public class ReflectorPropertiesPage extends PropertyPage implements IConstants {
	public static final String HAS_BUILDFORGE = "BUILDFORGE_ENABLED"; //$NON-NLS-1$
	Tree check;
	String selectedConsole = null;
	String selectedProject = null;
	String selectedStep = null;
	CTabFolder folders = null;
	Group group = null;
	Combo stepSelection = null;
	Combo projectSelection = null;
	Combo consoleSelection = null;
	boolean isBuildForgeEnabled = false;
	HashMap checkMap = null;
	boolean mapNeedsFilling = true;
	boolean selectDefaultProject = true;
	boolean selectDefaultStep = true;
	private Text directoryPath = null;
	private String[] checkedfiles;
	private String projectID;
	private Hashtable envHash;
	private Text envName;
	private Text envValue;
	private Table envTable;
	private TableItem tItem;
	/**
	 * Constructor for ReflectorPropertiesPage.
	 */
	public ReflectorPropertiesPage() {
		super();
	}

	/** 
	 * This is the top section of the properties page (above the tabs).
	 * @param parent
	 */
	private void addFirstSection(Composite parent) {
		Composite composite = createDefaultComposite(parent);

		//Label for path field
		Label projectLabel = new Label(composite, SWT.NONE);
		projectLabel.setText(TEXT_PROJECT_LABEL);

		// Path text field
		Text pathValueText = new Text(composite, SWT.WRAP | SWT.READ_ONLY);
		Object element = getElement();
		if (element instanceof IProject) {
			pathValueText.setText(((IProject) element).getProjectRelativePath().toString());
		} else if (element instanceof IJavaProject) {
			pathValueText.setText(((IJavaProject) element).getPath().toString());
		}
		
	}

	/**
	 * Adds a line separator.
	 * @param parent
	 */
	private void addSeparator(Composite parent) {
		Label separator = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		separator.setLayoutData(gridData);
	}

	/**
	 * Adds the Configuration Tabs for Reflector.
	 * @param parent
	 */
	private void addTabFolderSections(Composite parent) {		
		Button hasBuildForge = new Button(parent, SWT.CHECK);
		hasBuildForge.setText(TEXT_ENABLE_BUILDFORGE);
		isBuildForgeEnabled = ReflectorFavoriteItem.getBuildForgeEnabledPropertyValue(getElement());
		if (isBuildForgeEnabled) {
			hasBuildForge.setSelection(true);
		}
		
		group = new Group(parent, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL, GridData.FILL_VERTICAL, true, true));
		group.setVisible(isBuildForgeEnabled);
		
		hasBuildForge.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				if (isBuildForgeEnabled) {
					group.setEnabled(false);
					group.setVisible(false);
					isBuildForgeEnabled = false;
				} else {
					group.setEnabled(true);
					group.setVisible(true);
					isBuildForgeEnabled = true;
				}
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				if (isBuildForgeEnabled) {
					group.setEnabled(false);
					isBuildForgeEnabled = false;
				} else {
					group.setEnabled(true);
					isBuildForgeEnabled = true;
				}
			}
		});
		
		folders = new CTabFolder(group, SWT.BORDER);
		folders.setSimple(false);
		folders.setLayout(new GridLayout());
		folders.setLayoutData(new GridData(GridData.FILL_HORIZONTAL, GridData.FILL_VERTICAL, true, true));
		
		CTabItem projects = new CTabItem(folders, SWT.NULL);
		projects.setText(Messages.getString("ReflectorPropertiesPage.CONSOLE_SETTINGS_TAB_LABEL")); //$NON-NLS-1$
		CTabItem workFiles = new CTabItem(folders, SWT.NULL);
		workFiles.setText(Messages.getString("ReflectorPropertiesPage.WORKSPACE_SETTINGS_TAB_LABEL")); //$NON-NLS-1$
		CTabItem environment  = new CTabItem(folders, SWT.NULL);
		environment.setText(Messages.getString("ReflectorPropertiesPage.ENVIRONMENT_SETTINGS_TAB_LABEL"));
		
		populateProjects(projects);
		populateWorkFiles(workFiles);
		populateEnvironment(environment);
		
		folders.setSelection(0);
	}

	/**
	 * Populates the CTabItem folder for files to be passed to 
	 * the Build Forge engine thru a reflector build.
	 * @param workFiles
	 */
	private void populateWorkFiles(CTabItem workFiles) {
		Object element = getElement();
		
		Composite backdrop = new Composite(workFiles.getParent(), SWT.NONE);
		backdrop.setLayout(new FillLayout(SWT.VERTICAL));
		
		if (!(element instanceof IProject) && !(element instanceof IJavaProject)) {
			Log.trace("You may only view properties of a Build Forge Project.", getClass().getName()); //$NON-NLS-1$
			Log.errorLogMessage(Messages.getString("ReflectorPropertiesPage.INVALID_PROJECT_ERROR_CONS") + element.getClass().getName() + "]",  //$NON-NLS-1$ //$NON-NLS-2$
					Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
			return;
		}

		ReflectorStore priorStore = getProjectReflectorStore();	    
	    ArrayList includedFiles = null;

    	if (priorStore != null && priorStore.getCheckTree() != null) {
    		Log.trace("Found existing ReflectorStore",getClass().getName()); //$NON-NLS-1$
    		includedFiles = (ArrayList) priorStore.getCheckTree();
    	} else {
    		Log.trace("No prior ReflectorStore found", getClass().getName()); //$NON-NLS-1$
    		includedFiles = new ArrayList();
    	}
    	
    	checkedfiles = new String[includedFiles.size()];
		for(int i = 0; i < includedFiles.size(); i++) {
			checkedfiles[i] = (String) includedFiles.get(i);
			Log.trace("included files >" + includedFiles.get(i).toString(), getClass().getName()); //$NON-NLS-1$
		}

		check = new Tree(backdrop, SWT.CHECK | SWT.BORDER);
	    check.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				if (e.detail == SWT.CHECK) {
					if (((TreeItem) e.item).getChecked()) {
						setChildrenCheck((TreeItem) e.item, true);
						setParentCheck((TreeItem)e.item, true);
					} else {
						setChildrenCheck((TreeItem) e.item, false);
					}
				}
			}
			
			public void setChildrenCheck(TreeItem itm, boolean checkIt) {
				if (itm == null) 
					return;
				itm.setChecked(checkIt);
				TreeItem[] items = itm.getItems();
				for (int i = 0; i < items.length; i++) {
					setChildrenCheck(items[i], checkIt);
				}
			}

			public void setParentCheck(TreeItem itm, boolean checkIt){
				if(itm == null)
					return;
				itm.getParentItem().setChecked(checkIt);
				setParentCheck(itm.getParentItem(), checkIt);
			}

			
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
    		
    	});
    	
    	try { 
	    	Log.trace("Project Path : " + getProjectName() + getProjectIdentifier(), getClass().getName()); //$NON-NLS-1$    	
	    	addFilesInFolder(check, null, getRealProject(), getRealProject().getFullPath().toString());
	    } catch (CoreException ce) {
	    	Log.errorLogMessage(Messages.getString("ReflectorPropertiesPage.TREE_POPULATION_ERROR_CONS") + ce.getMessage() + "\n" + ce.getStackTrace(), Log.ERROR,  //$NON-NLS-1$ //$NON-NLS-2$
	    			getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
	    	return;
	    }
    	
	    workFiles.setControl(backdrop);
	}
	
	/**
	 * Resolves what is checked and unchecked in the files Tree.
	 * @param checkTree
	 * @param parentItem
	 * @param project
	 * @param path
	 * @return
	 * @throws CoreException
	 */
	private boolean addFilesInFolder(Tree checkTree, TreeItem parentItem, IProject project, String path) throws CoreException {
		IResource[] items;
		boolean allchecked = false;
		
		if (parentItem == null) {
			items = project.members(false); // False does not pick up deleted items (phantoms)
		} else {
			IFolder folder = project.getProject().getFolder(path);
			Log.trace("Folder Path is " + path, getClass().getName()); //$NON-NLS-1$
			items = folder.members(false);  // False does not pick up deleted items (phantoms)
		}
		
		//		 set items checked from stored properties.
		
		if (checkMap == null)
			checkMap = new HashMap();
		
		if (checkedfiles == null) {
			Log.trace("checkedfiles is null", getClass().getName()); //$NON-NLS-1$
			allchecked = true;
		} else if (mapNeedsFilling) {

			for (int i = 0; i < checkedfiles.length; i++) {
				checkMap.put(checkedfiles[i], "1"); //$NON-NLS-1$
			} 
			mapNeedsFilling = false;
		}  
	
		for (int i = 0; i < items.length; i++) {
			TreeItem tItem;
			if (parentItem == null)
				tItem = new TreeItem(checkTree, SWT.NONE);
			else 
				tItem = new TreeItem(parentItem, SWT.NONE);
			
    		if (items[i].getType() == IResource.FOLDER) {
    			Log.trace("Folder [" + items[i].getName() + "]", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
    			tItem.setText(items[i].getName());
    			tItem.setExpanded(true);
    			String itemPath = getPathToNode(tItem);
    			if (checkMap.containsKey(itemPath) || allchecked) 
    				tItem.setChecked(true);
    			if (!addFilesInFolder(checkTree, tItem, project, items[i].getProjectRelativePath().toString())) {
    				Log.trace("Bad things happened in addFilesInFolder", getClass().getName()); //$NON-NLS-1$
    				return false;
    			}
    		} else {
	    		tItem.setText(items[i].getName());
	    		
	    		String itemPath = getPathToNode(tItem);
	    		tItem.setData(itemPath);
	    		if (checkMap.containsKey(itemPath) || allchecked) {
	    			Log.trace("Found " + itemPath + " checking it.", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
	    			tItem.setChecked(true);
	    		}
    		}
		}

		return true;
	}
	
	
	/**
	 * Given a TreeItem this function resolves it's path in the tree by recursively
	 * getting it's parents Text field and appending this with a '/'.
	 * @param target
	 * @return
	 */
	private String getPathToNode(TreeItem target) {
		StringBuffer value = new StringBuffer();
		if (target == null) {
			return ""; //$NON-NLS-1$
		}
		if (target.getParent() == null) {
			return target.getText();
		}
		value.append(getPathToNode(target.getParentItem()) + "/" + target.getText()); //$NON-NLS-1$
		return value.toString();
	}

	private List getFilePaths(Tree checkTree) {
		TreeItem[] rootItems = checkTree.getItems();
		return addFilesFromTree(rootItems);
	}
	
	private List addFilesFromTree(TreeItem[] items) {
		ArrayList results = new ArrayList();
		
		for(int i = 0; i < items.length; i++) {
			if (items[i].getChecked()) {
				if (items[i].getItemCount() > 0) {
					results.add(getPathToNode(items[i]));
					results.addAll(addFilesFromTree(items[i].getItems()));
				} else { 
					results.add(getPathToNode(items[i]));
				} 
			}
		}
		// 
		return results;
	}
	
	/**
	 * Builds the CTabItem for the environment section.
	 * @param envProj
	 */
	private void populateEnvironment(CTabItem envProj) {
		Composite parent = envProj.getParent();
		
		Composite backdrop = new Composite(parent, SWT.NONE);
		GridLayout backLayout = new GridLayout();
		backLayout.numColumns = 2;
		backdrop.setLayout(backLayout);
		
		
		Label environmentLabel = new Label(backdrop, SWT.NONE);
		GridData gData = new GridData();
		gData.horizontalSpan = 2;
		environmentLabel.setLayoutData(gData);
		environmentLabel.setText(Messages.getString("ReflectorPropertiesPage.ENVIRONMENT_LABEL"));
		
		envTable = new Table(backdrop, SWT.BORDER | SWT.FULL_SELECTION);
		gData = new GridData(GridData.FILL_HORIZONTAL);
		gData.horizontalSpan = 2;
		gData.heightHint = convertHeightInCharsToPixels(10);
		//gData.minimumHeight = 40;
		envTable.setLayoutData(gData);
		envTable.setLinesVisible(true);

		final TableEditor editor = new TableEditor(envTable);
		editor.horizontalAlignment = SWT.LEFT;
	    editor.grabHorizontal = true;
	    
		TableColumn c1 = new TableColumn(envTable, SWT.NONE);
		c1.setText("Variable Name");
		
		TableColumn c2 = new TableColumn(envTable, SWT.NONE);
		
		c2.setText("Variable Value");
		
		envTable.setHeaderVisible(true);
		
		this.envHash = getEnvHash();
		
		ReflectorStore refStore = getProjectReflectorStore();
		if (refStore != null) {
			Hashtable oldList = refStore.getEnvironment();
			if (oldList != null) {
				Collection iterableKeys = oldList.keySet();
				ArrayList order = refStore.getEnvironmentOrder();
				if(order != null){ iterableKeys = order; }
				for(Iterator iter = iterableKeys.iterator();iter.hasNext();) {
					TableItem tItem = new TableItem(envTable, SWT.NONE);
					String name = (String)iter.next();
					EnvironmentValue value = EnvironmentValue.getEnvironmentValue(oldList.get(name));
					tItem.setText(0, name);
					tItem.setText(1, value.getValue());
					envHash.put(name, value);
				}
			}
		} else {
			Log.trace("No stored Properties for Environments exist", this.getClass().getName());
		}
		
		Label nameLabel = new Label(backdrop, SWT.NONE);
		nameLabel.setLayoutData(new GridData());
		nameLabel.setText(Messages.getString("ReflectorPropertiesPage.ENV_NAME_LABEL"));
		
		envName = new Text(backdrop, SWT.SINGLE | SWT.BORDER);
		GridData tNameDat = new GridData(GridData.FILL_HORIZONTAL);
		tNameDat.widthHint = convertWidthInCharsToPixels(60);
		envName.setLayoutData(tNameDat);
		
		Label valueLabel = new Label(backdrop, SWT.NONE);
		valueLabel.setLayoutData(new GridData());		
		valueLabel.setText(Messages.getString("ReflectorPropertiesPage.ENV_VALUE_LABEL"));
		
		envValue = new Text(backdrop, SWT.SINGLE | SWT.BORDER);
		GridData tValueDat = new GridData(GridData.FILL_HORIZONTAL);
		tValueDat.widthHint = convertWidthInCharsToPixels(60);
		envValue.setLayoutData(tValueDat);
		
		GridData tButtons = new GridData();
		tButtons.horizontalSpan = 1;
		Button addEnv = new Button(backdrop, SWT.PUSH);
		addEnv.setLayoutData(tButtons);
		addEnv.setText(Messages.getString("ReflectorPropertiesPage.ADD_ENVIRONMENT_BUTTON"));
		
		addEnv.addSelectionListener(new SelectionAdapter() {
			
			public void widgetSelected (SelectionEvent event) {
				if (!"".equals(envName.getText())) {
					EnvironmentValue value = new EnvironmentValue(envValue.getText());
					envHash.put(envName.getText(), value);
					tItem = new TableItem(envTable, SWT.NONE);
					Log.trace("Adding Name " + envName.getText() + " and value " + envValue.getText(), getClass().getName());
					tItem.setText(0, envName.getText());
					tItem.setText(1, value.getValue());
					envName.setText("");
					envValue.setText("");
				}
			}
		});
		
		Button removeEnv = new Button(backdrop, SWT.PUSH);
		removeEnv.setLayoutData(new GridData());
		removeEnv.setText(Messages.getString("ReflectorPropertiesPage.REMOVE_ENVIRONMENT_BUTTON"));
		// add name / value variables and pull the environment variables from the console for the existing 
		// property like a firebuild does.
		removeEnv.addSelectionListener(new SelectionAdapter() {
			
			public void widgetSelected (SelectionEvent event) {
				if (envTable.getSelectionCount() > 0) {
					int[] tmpList = envTable.getSelectionIndices();
					for(int i = 0; i < tmpList.length; i++) {
						int index = tmpList[i];
						String name = envTable.getItem(index).getText(0);
						if (envHash.containsKey(name)) { envHash.remove(name); } 
						envTable.remove(index);
					}
					envName.setText("");
					envValue.setText("");
				}
			}
		});

		envTable.addSelectionListener(new SelectionAdapter() {
			
			public void widgetSelected(SelectionEvent event) {
				if (envTable.getSelectionCount() > 0) {
					int index = envTable.getSelectionIndex();
					envName.setText(envTable.getItem(index).getText(0));
					EnvironmentValue value = (EnvironmentValue)envHash.get(envName.getText());
					if(value.getOptions() != null && value.getOptions().length > 0){
						envValue.setText(value.getOptionsAsDelimitedString());
					} else { envValue.setText(envTable.getItem(index).getText(1)); }
				}
			}
		});
		
		envTable.addMouseListener(new MouseAdapter() {
			public void mouseDown(MouseEvent event) {
				// Dispose any existing editor
				Control old = editor.getEditor();
				if (old != null) old.dispose();

				Point point = new Point(event.x, event.y);
				final TableItem item = envTable.getItem(point);
				if (item != null) {
					int column = getColumnSelected(item, point);
					if (column == 1) {
						final String envVarName = item.getText(0);
						EnvironmentValue value = (EnvironmentValue)envHash.get(envVarName);
						if(value.getOptions() != null && value.getOptions().length > 0){
							// Create the dropdown and add data to it
							final CCombo combo = new CCombo(envTable, SWT.READ_ONLY);
							combo.setItems(value.getOptions());
							combo.select(combo.indexOf(item.getText(1)));
	
							// Compute the width for the editor
							// Also, compute the column width, so that the dropdown fits
							Point size = combo.computeSize(SWT.DEFAULT, SWT.DEFAULT);
							editor.minimumWidth = size.x;
							editor.minimumHeight = size.y;
	
							// Set the focus on the dropdown and set into the editor
							combo.setFocus();
							editor.setEditor(combo, item, column);
	
							// Add a listener to set the selected item back into the cell
							combo.addSelectionListener(new SelectionAdapter() {
								public void widgetSelected(SelectionEvent event) {
									item.setText(1, combo.getText());
									EnvironmentValue value = (EnvironmentValue)envHash.get(envVarName);
									value.setValue(combo.getText());
									combo.dispose();
								}
							});
						}
					}
				}
			}

			private int getColumnSelected(TableItem item, Point pt){
				int column = -1;
				for (int i = 0, n = envTable.getColumnCount(); i < n; i++) {
					Rectangle rect = item.getBounds(i);
					if (rect.contains(pt)) {
						column = i;
						break;
					}
				}
				return column;
			}
		});
		
		c1.pack();
		c2.pack();
		envProj.setControl(backdrop);
	}
	
	/**
	 * Get the current Project's ReflectorStore object.
	 * @return ReflectorStore
	 */
	private ReflectorStore getProjectReflectorStore() {			
		return ReflectorPlugin.getProjectInfoByLabel(getProjectIdentifier());
	}
	
	/**
	 * Get the current Projects Name
	 * @return
	 */
	private String getProjectName() {
		return getRealProject().getName();
	}
	
	
	/**
	 * Get the IProject object for the current selected project.
	 * This resolves IJavaProject to IProject.
	 * @return IProject
	 */
	private IProject getRealProject() {
		IProject proj;
		Object element = getElement();
		if (element instanceof IJavaProject) {
			proj = ((IJavaProject) element).getProject();
		} else {
			proj = (IProject) element;
		}
		return proj;
	}
	
	
	private String getProjectIdentifier() {		
		return getRealProject().getProject().getFullPath().toString();
	}
	

	private void populateProjects(CTabItem projects) {
		Composite parent = projects.getParent();
	
		Composite backdrop = new Composite(parent, SWT.NONE);
		backdrop.setLayout(new RowLayout(SWT.VERTICAL));

		Label consoleText = new Label (backdrop, SWT.NONE);
		consoleText.setText(Messages.getString("ReflectorPropertiesPage.CONSOLE_SELECTION_LABEL")); //$NON-NLS-1$
		
		consoleSelection = new Combo(backdrop, SWT.SIMPLE | SWT.DROP_DOWN | SWT.READ_ONLY);
		IConsoleList consoles = CorePlugin.getConsoleList();
		Log.trace("retrieving Consoles", getClass().getName()); //$NON-NLS-1$
		Object[] cons = consoles.getConsoles();
		
		int defaultConsoleIndex = -1;
		IConsole existingConsole = ReflectorFavoriteItem.getConsoleSelection(getElement());
		for (int i = 0; i < cons.length; i++) {
			IConsole currConsole = (IConsole) cons[i];
			String currConsoleName = (currConsole.getName() != null) ? currConsole.getName() : ""; //$NON-NLS-1$
			Log.trace("console [" + currConsoleName + "]", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
			consoleSelection.add( currConsoleName, i );
			if (existingConsole != null && currConsoleName.equals(existingConsole.getName())) {
				defaultConsoleIndex = i;
				try {
					String checktoken = ((IConsole) cons[i]).getAuthenticationTokens();
					if (!((IConsole)cons[i]).isConnected() || checktoken == null || checktoken.length() == 0) {
						Log.trace("Console isConnected() returned " + ((IConsole)cons[i]).isConnected() + "\nChecktoken is [" + checktoken + "]\n", this.getClass().getName());
						Log.trace("Attempting to Authenticate with console [" + existingConsole + "]\n", this.getClass().getName());
						((IConsole) cons[i]).setAuthEnabled(true);
						((IConsole) cons[i]).invalidateAuthenticationTokens();
						if (((IConsole) cons[i]).authenticate()) {
							((IConsole) cons[i]).update();
						}
					}
				} catch (BuildForgeException be) {
					Log.trace("Error communicating with the buildforge console", getClass().getName()); //$NON-NLS-1$
					Log.errorLogMessage(Messages.getString("RemoteBuildActionDelegate.COMMUNICATION_ERROR_CONS") + be.getMessage(),  //$NON-NLS-1$
							Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
					ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
							Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_DLG"), Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_SUB_DLG") + be.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
							Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), be, Log.ERROR));
				}
			}
		}
		
		RowData rowd = new RowData();
		rowd.width = convertWidthInCharsToPixels(60);
		consoleSelection.setLayoutData(rowd);
		consoleSelection.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				IConsole cons = CorePlugin.getConsoleList().getConsoleByName(consoleSelection.getText());
				try {
					String checktoken = cons.getAuthenticationTokens();
					if (!cons.isConnected() || checktoken == null || checktoken.length() == 0) {
						Log.trace("Attempting to Authenticate with console\n", this.getClass().getName());
						cons.invalidateAuthenticationTokens();
						cons.setAuthEnabled(true);
						if (cons.authenticate()) 
							cons.update();
					}
				} catch (BuildForgeException be) {
					Log.trace("Error communicating with the buildforge console", getClass().getName()); //$NON-NLS-1$
					Log.errorLogMessage(Messages.getString("RemoteBuildActionDelegate.COMMUNICATION_ERROR_CONS") + be.getMessage(),  //$NON-NLS-1$
							Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
					ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
							Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_DLG"), Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_SUB_DLG") + be.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
							Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), be, Log.ERROR));
				}
			}
		});
		
		Label projectText = new Label(backdrop, SWT.NONE);
		projectText.setText(Messages.getString("ReflectorPropertiesPage.PROJECT_ASSOCIATION_LABEL")); //$NON-NLS-1$
		
		final String defaultProjectId = ReflectorFavoriteItem.getProjectSelection(getElement());
		
		projectSelection = new Combo(backdrop, SWT.SIMPLE | SWT.DROP_DOWN | SWT.READ_ONLY);
		RowData rowdat = new RowData();
		rowdat.width = convertWidthInCharsToPixels(60);
		projectSelection.setLayoutData(rowdat);	
		projectSelection.addFocusListener(new FocusAdapter() {
			
			public void focusGained(FocusEvent event) {
				Log.trace(consoleSelection.getText(), getClass().getName());
				Object[] conslist = CorePlugin.getConsoleList().getConsoles();
				
				for (int i = 0; i < conslist.length; i++) {
					IConsole currConsole = (IConsole) conslist[i];
					if (currConsole.getName() != null && 
							currConsole.getName().equals(consoleSelection.getText())) {
						try {
							String checktoken = currConsole.getAuthenticationTokens();
							if (!currConsole.isConnected() || checktoken == null || checktoken.length() == 0) {
								Log.trace("Attempting to Authenticate with console\n", this.getClass().getName());
								currConsole.setAuthEnabled(true);
								currConsole.invalidateAuthenticationTokens();
								if (currConsole.authenticate()) 
									currConsole.update();
							}

							Object[] projects = currConsole.getProjects().toArray();
							projectSelection.clearSelection();
							projectSelection.deselectAll();
							projectSelection.removeAll();
							int defaultIndex = -1;
							for (int j = 0; j < projects.length; j++) {
									projectSelection.add(((IProjectNode)projects[j]).getName(), j);
									if (((IProjectNode)projects[j]).getName().equals(defaultProjectId)) {
										defaultIndex = j; 
									}
							}
							if (selectDefaultProject) {
									if (defaultIndex >= 0) 
										projectSelection.select(defaultIndex);
									selectDefaultProject = false;
							}
							
						} catch (BuildForgeException bfe) {
							Log.trace("Unable to authenticate with Build Forge Server" + bfe.getMessage(), getClass().getName()); //$NON-NLS-1$
							Log.errorLogMessage(Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_CONS") + bfe.getMessage(),  //$NON-NLS-1$
									Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
							ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
									Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_DLG"), Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_SUB_DLG") + bfe.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
									Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), bfe, Log.ERROR));
						}
					}
				}
			}
		});
		
		Label stepText = new Label(backdrop, SWT.NONE);
		stepText.setText(Messages.getString("ReflectorPropertiesPage.STEP_MAPPING_LABEL")); //$NON-NLS-1$

		final String defaultStep = ReflectorFavoriteItem.getStepSelection(getElement());
		
		stepSelection = new Combo(backdrop, SWT.SIMPLE | SWT.DROP_DOWN | SWT.READ_ONLY);
		RowData rowdt = new RowData();
		rowdt.width = convertWidthInCharsToPixels(60);
		stepSelection.setLayoutData(rowdt);
		stepSelection.addFocusListener(new FocusAdapter() {
			
			public void focusGained(FocusEvent event) {
				Object[] conslist = CorePlugin.getConsoleList().getConsoles();
				
				for (int i = 0; i < conslist.length; i++) {
					IConsole currConsole = (IConsole) conslist[i];
					if (currConsole.getName() != null && 
							currConsole.getName().equals(consoleSelection.getText())) {
						ArrayList projects = new ArrayList();
						try {
							String checktoken = currConsole.getAuthenticationTokens();
							if (!currConsole.isConnected() || checktoken == null || checktoken.length() == 0) {
								Log.trace("Attempting to Authenticate with console\n", this.getClass().getName());
								currConsole.setAuthEnabled(true);
								currConsole.invalidateAuthenticationTokens();
								if (currConsole.authenticate()) 
									currConsole.update();
							}
							
							projects = currConsole.getProjects();
						} catch (BuildForgeException bfe) {
							Log.trace("Unable to authenticate with Build Forge Server" + bfe.getMessage(), getClass().getName()); //$NON-NLS-1$
							Log.errorLogMessage(Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_CONS") + bfe.getMessage(),  //$NON-NLS-1$
									Log.ERROR, getClass().getName(), ReflectorPlugin.getDefault().getBundle().getSymbolicName());
							ErrorDialog.openError(ReflectorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
									Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_DLG"), Messages.getString("ReflectorPropertiesPage.AUTHENTICATION_ERROR_SUB_DLG") + bfe.getMessage(), //$NON-NLS-1$ //$NON-NLS-2$
									Log.getMultiStatus(ReflectorPlugin.getDefault().getBundle(), bfe, Log.ERROR));
						}

						stepSelection.clearSelection();
						stepSelection.deselectAll();
						stepSelection.removeAll();
						boolean found = false;
						String vers = currConsole.getVersion();
						for (int j = 0; j < projects.size(); j++) {
							if (projectSelection.getText() != null && projectSelection.getText().equals(((IProjectNode)projects.get(j)).getName())) {
								found = true;
								
								IStep[] steps = ((IProjectNode)projects.get(j)).getSteps();

								int defaultIndex = -1;
								for(int k = 0; k < steps.length; k++) {	
									stepSelection.add((steps[k]).getName(), k);
									if (vers != null && !"".equals(vers)) {
										stepSelection.setData((steps[k]).getName(), (steps[k]).getStepUniqueIdentifier());
									} else {
										stepSelection.setData((steps[k]).getName(), String.valueOf((steps[k]).getStepId()));
									}
									if ((steps[k]).getName().equals(defaultStep)) {
										defaultIndex = k;
									}
								}
								if (selectDefaultStep) {
									if (defaultIndex >= 0) 
										stepSelection.select(defaultIndex);
									selectDefaultStep = false;
									
								}
							}
						}
						
						if (!found) {
							Log.trace("Unable to find project [" + projectSelection.getText() + "] in the console [" + currConsole.getName() + "].", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						}
					}
								
				}
			}
		});
		
		Label projectRelativePath = new Label(backdrop, SWT.WRAP);
		projectRelativePath.setText(Messages.getString("ReflectorPropertiesPage.PROJECT_BUILD_PATH_ASSOCIATION_LABEL")); //$NON-NLS-1$
		directoryPath = new Text(backdrop, SWT.SINGLE | SWT.BORDER);
		RowData tDat = new RowData();
		tDat.width = convertWidthInCharsToPixels(60);
		directoryPath.setLayoutData(tDat);
		String defaultDirPath = ReflectorFavoriteItem.getRelativeBuildForgePath(getElement());
		if (defaultDirPath != null) 
			directoryPath.setText(defaultDirPath);
		directoryPath.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		
		if (defaultConsoleIndex > -1) {
			consoleSelection.select(defaultConsoleIndex);
			// Fake focus
			projectSelection.notifyListeners(SWT.FocusIn, new Event());
			stepSelection.notifyListeners(SWT.FocusIn, new Event());
			directoryPath.forceFocus();
		}
		

		projects.setControl(backdrop);
	}

	
	/**
	 * @see PreferencePage#createContents(Composite)
	 */
	
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		composite.setLayout(layout);
		GridData data = new GridData(GridData.FILL);
		data.grabExcessHorizontalSpace = true;
		data.grabExcessVerticalSpace = true;
		composite.setLayoutData(data);

		addFirstSection(composite);
		addSeparator(composite);
		addTabFolderSections(composite);
		return composite;
	}

	private Composite createDefaultComposite(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());
		
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		composite.setLayoutData(data);

		return composite;
	}

	
	protected void performDefaults() {

	}
	
	
	public boolean performOk() {
		NatureToggle bfNature = new NatureToggle();
		IProject project = null;
		
		if (getElement() instanceof IProject) {
			project = (IProject) getElement();
		} else if (getElement() instanceof IJavaProject) {
			project = ((IJavaProject)getElement()).getProject();
		}
		
		if (! isBuildForgeEnabled) {
			ReflectorFavoriteItem.setBuildForgeEnabledPropertyValue(getElement(), false);
			bfNature.removeBuildForgeNature(project);
			return true;
		}
		
		bfNature.addBuildForgeNature(project);
		ReflectorFavoriteItem.setBuildForgeEnabledPropertyValue(getElement(), true);
		
		String consoleItem = ""; //$NON-NLS-1$
		String projectItem = ""; //$NON-NLS-1$
		String stepItem = ""; //$NON-NLS-1$
		String stepUniqueId = "";
		try {
			consoleItem = consoleSelection.getItem(consoleSelection.getSelectionIndex());
			projectItem = projectSelection.getItem(projectSelection.getSelectionIndex());
			stepItem = stepSelection.getItem(stepSelection.getSelectionIndex());
			stepUniqueId = (String) stepSelection.getData(stepItem);
		} catch (IllegalArgumentException iae) { }
		
		projectID = getProjectIdentifier();
		Log.trace("Saving as projectID of [" + projectID + "]", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
		ReflectorStore refStore = new ReflectorStore(projectID, getFilePaths(check));
		refStore.setEnvironment(envHash);
		
		ArrayList envOrder = new ArrayList();
		for (int i = 0; i < envTable.getItemCount(); i++) {
			Log.trace("Saving envTable Item at index [" + i + "]", getClass().getName());
			envOrder.add(i, envTable.getItem(i).getText());
		}
		
		refStore.setEnvironmentOrder(envOrder);
		ReflectorPlugin.addProjectInfo(refStore);
		ReflectorPlugin.getDefault().save();
		ReflectorFavoriteItem.setConsoleSelection(getElement(), consoleItem);
		Log.trace("Saving console to properties as [" + consoleItem + "]", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
		ReflectorFavoriteItem.setProjectSelection(getElement(), projectItem);
		Log.trace("Saving project to properties as [" + projectItem + "]", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
		ReflectorFavoriteItem.setStepSelection(getElement(), stepItem); 
		Log.trace("Saving step to properties as [" + stepItem + "]", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
		ReflectorFavoriteItem.setStepUniqueId(getElement(), stepUniqueId);
		Log.trace("Saving step to properties as [" + stepUniqueId + "]", getClass().getName());
		ReflectorFavoriteItem.setRelativeBuildForgePath(getElement(), directoryPath.getText());
		Log.trace("Saving Buildforge build path addition to properties as [" + directoryPath.getText() + "]", getClass().getName()); //$NON-NLS-1$ //$NON-NLS-2$
		return true;
	}

	private Hashtable getEnvHash(){ return (envHash != null)?envHash:(envHash = new Hashtable()); }
}
