/*
 * �2005�2006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.ui.view;

import java.net.URL;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.help.WorkbenchHelp;
import org.eclipse.ui.part.ViewPart;

import com.buildforge.buildmodel.core.CorePlugin;
import com.buildforge.buildmodel.core.IBuildRecord;
import com.buildforge.buildmodel.core.IStepDetailRecord;
import com.buildforge.buildmodel.core.util.Log;
import com.buildforge.buildmodel.ui.UiPlugin;
import com.buildforge.buildmodel.ui.viewer.BuildLogTreeViewContentProvider;


public class BuildLogView extends ViewPart implements ISelectionListener {
	public static final String ID = "BuildForgeLogView.view";
	 TreeViewer viewer = null;
	
	class LogViewLabelProvider extends LabelProvider implements
		ILabelProvider {
		
		public Image getImage(Object element) {
			if (element instanceof IStepDetailRecord) {
				if (element == null)
					return null;
				char result = ((IStepDetailRecord) element).getResultsStatus();
				return getResultImage(result);
			}
			return null;
		}
		public String getColumnText(Object element) {
			return getText(element);
		}	
	
		private Image getResultImage(char result) {
			//$ANALYSIS-IGNORE
			if (IBuildRecord.PASS_RESULT == result)
							return createImage("icons/step_pass.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.FAIL_RESULT == result)
							return createImage("icons/step_failed.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.STOP_RESULT == result)
							return createImage("icons/break.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.QUEUED_RESULT == result)
							return createImage("icons/status_queued.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.RUNNING_RESULT == result)
							return createImage("icons/status_running.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.FILTER_WARN_RESULT == result)
							return createImage("icons/step_pass_error.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.WARN_RESULT == result)
							return createImage("icons/step_pass_error.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.SKIP_RESULT == result)
							return createImage("icons/step_skipped.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.NO_RESULT == result)
							return createImage("icons/no_result.gif");
			return createImage("icons/no_result.gif");
		}
	
		 private Image createImage(String gifFile) {
			    URL url = UiPlugin.getDefault().find(new Path(gifFile));
			    ImageDescriptor id = ImageDescriptor.createFromURL(url);
			    return id.createImage();
		 }
	}
	
	
	public void createPartControl(Composite parent) {
		Log.trace("Initializing Build Log View", this.getClass().getName() );
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		viewer.setContentProvider(new BuildLogTreeViewContentProvider());
		viewer.setLabelProvider(new LogViewLabelProvider());
		viewer.setInput(CorePlugin.getConsoleList().getActiveBuildRecord());

		WorkbenchHelp.setHelp(viewer.getControl(), "com.buildforge.buildmodel.help.LogContextId");
		getSite().getPage().addSelectionListener(this);
	}

	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			Object first = ((IStructuredSelection) selection).getFirstElement();
			if (first instanceof IBuildRecord) {
				if (viewer.getContentProvider() == null) {
					viewer.setContentProvider(new BuildLogTreeViewContentProvider());
				}
				viewer.setInput(first);
				viewer.refresh();
			}
		}
	
	}

}
