/*
 * 20052006, BuildForge, Inc. All rights reserved. BuildForge, Inc. owns the
 * copyright and other intellectual property rights in this software.
 * 
 * Duplication or use of the Software is not permitted except as expressly
 * provided in a written agreement between your company and BuildForge, Inc.
 */

package com.buildforge.buildmodel.ui.viewer;

import java.net.URL;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import com.buildforge.buildmodel.core.IBuildRecord;
import com.buildforge.buildmodel.core.internal.Messages;
import com.buildforge.buildmodel.ui.UiPlugin;

public class BuildIdViewLabelProvider extends LabelProvider implements
	ITableLabelProvider, IColorProvider {
	
	public Image getStatusImage(Object obj) {
		IBuildRecord element = (IBuildRecord) obj;
		String status = element.getStatus();
		
		if (IBuildRecord.COMPLETE_STATE.equals(status))
			return createImage("icons/status_completed.gif"); //$NON-NLS-1$
		if (IBuildRecord.WAIT_STATE.equals(status))
			return createImage("icons/status_wait.gif"); //$NON-NLS-1$
		if (IBuildRecord.RUNNING_STATE.equals(status))
			return createImage("icons/status_running.gif");		 //$NON-NLS-1$
		if (IBuildRecord.ARCHIVED_STATE.equals(status))
			return createImage("icons/status_archived.gif"); //$NON-NLS-1$
		if (IBuildRecord.LOCKED_STATE.equals(status))
			return createImage("icons/locked.gif"); //$NON-NLS-1$
		return null;
	}
	
	public Image getColumnImage(Object element, int columnIndex) {
		if (element instanceof IBuildRecord) {
			if (columnIndex == 1) 
				return getStatusImage(element);
			if (columnIndex == 2)
				return getResultImage(((IBuildRecord) element).getResult()); 
		} 
		return null;
	}

	private Image getResultImage(String result) {
		if (result == null || result.length() == 0) 
			return null;
		
		try {
			//$ANALYSIS-IGNORE
			if (IBuildRecord.PASS_RESULT == result.charAt(0))
							return createImage("icons/status_pass.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.FAIL_RESULT == result.charAt(0))
							return createImage("icons/status_fail.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.STOP_RESULT == result.charAt(0))
							return createImage("icons/stop.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.QUEUED_RESULT == result.charAt(0))
							return createImage("icons/status_queued.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.RUNNING_RESULT == result.charAt(0))
							return createImage("icons/status_running.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.FILTER_WARN_RESULT == result.charAt(0))
							return createImage("icons/status_warn.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.WARN_RESULT == result.charAt(0))
							return createImage("icons/status_warn.gif");
			//$ANALYSIS-IGNORE
			if (IBuildRecord.SKIP_RESULT == result.charAt(0))
							return createImage("icons/status_skipped.gif");
		} catch (NullPointerException npe) { }
		return null;
	}

	
	public String getColumnText(Object element, int columnIndex) {
		
		String owner = ((IBuildRecord) element).getOwner();
		String tag = ((IBuildRecord)element).getBuildTag();
		String status = ((IBuildRecord) element).getStatus();
		String result = ((IBuildRecord) element).getResult();
		switch(columnIndex) {
		case 0:
			return (tag != null) ? tag : "";	 //$NON-NLS-1$
		case 1:
			String statusText = ""; //$NON-NLS-1$
			if (status != null) {
				if (status.equals(IBuildRecord.ARCHIVED_STATE)) 
					statusText = Messages.getString("BuildIdViewLabelProvider.ARCHIVED_STATUS_TEXT"); //$NON-NLS-1$
				if (status.equals(IBuildRecord.COMPLETE_STATE))
					statusText = Messages.getString("BuildIdViewLabelProvider.COMPLETE_STATUS_TEXT"); //$NON-NLS-1$
				if (status.equals(IBuildRecord.RUNNING_STATE))
					statusText = Messages.getString("BuildIdViewLabelProvider.RUNNING_STATUS_TEXT"); //$NON-NLS-1$
				if (status.equals(IBuildRecord.WAIT_STATE))
					statusText = Messages.getString("BuildIdViewLabelProvider.WAITING_STATUS_TEXT"); //$NON-NLS-1$
				if (status.equals(IBuildRecord.LOCKED_STATE))
					statusText = Messages.getString("BuildIdViewLabelProvider.LOCKED_STATUS_TEXT"); //$NON-NLS-1$
			}
			return statusText;
		case 2:
			String resultsText = ""; //$NON-NLS-1$
			if (result != null && result.length() > 0) {
				try {
					//$ANALYSIS-IGNORE
					if (IBuildRecord.PASS_RESULT == result.charAt(0))
											resultsText = Messages.getString("BuildIdViewLabelProvider.PASS_RESULTS_TEXT");
					//$ANALYSIS-IGNORE
					if (IBuildRecord.FAIL_RESULT == result.charAt(0))
											resultsText = Messages.getString("BuildIdViewLabelProvider.FAIL_RESULTS_TEXT");
					//$ANALYSIS-IGNORE
					if (IBuildRecord.STOP_RESULT == result.charAt(0))
											resultsText = Messages.getString("BuildIdViewLabelProvider.STOPPED_RESULTS_TEXT");
					//$ANALYSIS-IGNORE
					if (IBuildRecord.QUEUED_RESULT == result.charAt(0))
											resultsText = Messages.getString("BuildIdViewLabelProvider.QUEUED_RESULTS_TEXT");
					//$ANALYSIS-IGNORE
					if (IBuildRecord.RUNNING_RESULT == result.charAt(0))
											resultsText = Messages.getString("BuildIdViewLabelProvider.RUNNING_RESULTS_TEXT");
					//$ANALYSIS-IGNORE
					if (IBuildRecord.FILTER_WARN_RESULT == result.charAt(0))
											resultsText = Messages.getString("BuildIdViewLabelProvider.WARN_RESULTS_TEXT");
					//$ANALYSIS-IGNORE
					if (IBuildRecord.WARN_RESULT == result.charAt(0))
											resultsText = Messages.getString("BuildIdViewLabelProvider.WARN_RESULTS_TEXT");
					//$ANALYSIS-IGNORE
					if (IBuildRecord.SKIP_RESULT == result.charAt(0))
											resultsText = Messages.getString("BuildIdViewLabelProvider.SKIP_RESULTS_TEXT");
				} catch (NullPointerException npe) { }
			}
			return resultsText;
		case 3:
			return (owner != null) ? owner : ""; //$NON-NLS-1$
		default: 
			return getText(element);
		}
	}

	public Color getForeground(Object element) {
		if (element instanceof IBuildRecord) {
			/*
			String status = ((IBuildRecord) element).getStatus();
			if (IBuildRecord.COMPLETE_STATE.equals(status))
				return Display.getDefault().getSystemColor(SWT.COLOR_GRAY);
			if (IBuildRecord.WAIT_STATE.equals(status))
				return Display.getDefault().getSystemColor(SWT.COLOR_BLACK);
			if (IBuildRecord.RUNNING_STATE.equals(status))
				return Display.getDefault().getSystemColor(SWT.COLOR_RED);
			if (IBuildRecord.LOCKED_STATE.equals(status)) 
				return Display.getDefault().getSystemColor(SWT.COLOR_DARK_YELLOW);
			if (IBuildRecord.ARCHIVED_STATE.equals(status)) 
				return Display.getDefault().getSystemColor(SWT.COLOR_BLUE);
				*/
		}
		return null;
	}

	public Color getBackground(Object element) {
		return null;
	}
	
	 private Image createImage(String gifFile) {
		    URL url = UiPlugin.getDefault().find(new Path(gifFile));
		    ImageDescriptor id = ImageDescriptor.createFromURL(url);
		    return id.createImage();
	 }
	 
	 public void dispose() {
		 
	 }
	
}