<?php
/**
 * jTE_parser
 * 
 * @package jTE
 */
class jTE_parser {
	// Reference to parent
	protected $tplRef;
	protected $pretty;

	public function getEntity($eName) {
		return (isset($this->tplRef->_jTE_entities[$eName])?$this->tplRef->_jTE_entities[$eName]:'');
	}

	public function render($tpl) {
		// Our final result
		$result = '';
		$limbo = $this->loadTemplate($tpl);
		$this->pretty = $this->parseElements($limbo);
		unset($limbo);
	}

	public function display() {
		if (is_array($this->pretty)) {
			$this->imploder($this->pretty);
		} else {
			return false;
		}
	}

	/** recursive implode with options to keep the various keys
	 *  Default is No inner clue, No outer glue, and false to keep keys
	 */
	public function imploder($array = null, $innerGlue = "", $outerGlue = "", $keepInnerKey = false, $keepOuterKey = false) {
		$first = true;
		foreach( $array as $key => $item ) {
			if ($first) {
				$first = false;
			} else {
				echo $outerGlue;
			}
		    if (is_array ($item)) {
		        if ($keepOuterKey) {
					echo $key;
				}
		        $this->imploder($item, $innerGlue, $outerGlue, $keepInnerKey, $keepOuterKey);
		    } else if (!is_object($item)) {
		        if ($keepInnerKey) {
					echo $key;
					echo $innerGlue;
				}
				echo $item;
			} else if (is_callable(array($item, 'display'))) {
				$item->display();
			}
		}
	}

	/** Preparse
	 * This is the function that takes a template in and kicks out the initial
	 * array.  this is a public function so that other template::functions can use it.
	 */
	public function preparse($tpl) {
		// Firstly parse the template to get an array of strings and tags
		// At this point our system knows NOTHING about the objects/functions etc..
		$array = $this->createTagsArray($tpl);
		return $array;
	}

	/** Parse
	 * This is the function that actually steps through the array and gives back a useful
	 * result.  This is a public function so that other template::functions can use it.
	 */
	public function parse($array) {
		// Now go through the array we just created and format the array
		$array = $this->handleTagsArray($array);
		return $array;
	}
	
	/**
	 *	This function steps through the entire tags array and deals with those that are object calls
	 */
	public function parseElements($array) {
		if (!is_array($array)) {
			return array($array);
		}
		// Step through the parsed object
		foreach($array as $k => $v) {
			if (is_array($v)) {
				if (isset($v['object']) && isset($v['args'])) {
					// get the name of our object
					$objectName = substr($v['object'], 0, strpos($v['object'], '-'));
					// get the function call or variable call we are making
					$objectCall = substr($v['object'], strpos($v['object'], '>')+1 );
					// get the primary subject of our function
					$pipepos = strpos($v['args'],'|');
					$objectArgs = ($pipepos !== false) ? substr($v['args'], 0, $pipepos) : $v['args'];
					unset($pipepos);
					// Get our working result
					$result = $this->handleObject($objectName, $objectCall, $objectArgs);
					// Get our modifiers
					$modifiers = $this->parseModifiers($v['args']);
					// Pass the result to each of the modifiers for further use.
					// Modifiers are just function calls JUST LIKE EVERY OTHER FUNCTION
					// They must follow the same conventions.
					if ($modifiers !== false && isset($modifiers[0])) {
						foreach($modifiers as $mod) {
							$result = $this->handleFunction($mod, $result);
						}
					}
					$array[$k] = $result;
				} else if (isset($v['function']) && isset($v['args'])) {
					// Get the primary subject of our function
					$pipepos = strpos($v['args'],'|');
					$functionSubject = ($pipepos !== false) ? substr($v['args'], 0, $pipepos) : $v['args'];
					unset($pipepos);
					// Get any secondary functions
					$modifiers = $this->parseModifiers($v['args']);
					// Handle the primary function
					$result = $this->handleFunction($v['function'].'('.$functionSubject.')', $array[$k]);
					// Pass the result to each of the modifiers for further use.
					// Modifiers are just function calls JUST LIKE EVERY OTHER FUNCTION
					// They must follow the same conventions.
					if (!empty($modifiers) && isset($modifiers[0])) {
						foreach($modifiers as $mod) {
							$result = $this->handleFunction($mod, $result, true);
						}
					}
					$array[$k] = $result;
				}
			}
		}
		return $array;
	}

	/** handleTagsArray
	 * This method takes an array of template functions, object references, etc.
	 * and builds it up into a multi-dimensional array with block structure.
	 */
	protected function handleTagsArray($array) {
		// Keep track of the functions and their placement
		$funcs = array();
		$offset = 0;
		foreach ($array as $key => $nub) {
			// For each function in the array, just leave it and add it to the list of functions
			if (isset($nub['function'])) {
				if (!isset($funcs[$nub['function']])) {
					$funcs[$nub['function']] = array(array($key, $offset));
				} else {
					$funcs[$nub['function']][] = array($key, $offset);
				}
				$last_opener = $key;
			// If closing a function, search for the last occurance.
			} else if (isset($nub['closer'])) {
				if (isset($funcs[$nub['closer']])) {
					if (count($funcs[$nub['closer']]) > 0) {
						// Get the index of the last opener for the function and the offset that applies to it
						list($opener, $func_offset) = array_pop($funcs[$nub['closer']]);
						// Calculate the actual openers and closers based on the appropriate offset
						$real_opener = $opener - $func_offset;
						$real_closer = $key - $offset;
						// Create the indented array from 1 past the opener to the closer
						$indent = array_splice($array, $real_opener + 1, $real_closer - $real_opener, null);
						// We don't actually need or want to hang onto the closer "tag"
						$offset += count($indent);
						array_pop($indent);
						// Assign the indent to the block of the opener
						$array[$real_opener]['block'] = $indent;
					} else {
						throw new BFException(translate('2145', $nub['closer']));
					}
				} else {
					throw new BFException(translate('2146', $nub['closer']));
				}
			}
		}
		return $array;
	}

	/** handleObject
	 *	This function handles with a specific object call and returns a result
	 *  This may be the result of a method or property call.
	 */
	protected function handleObject($objectName, $objectCall, $objectArgs) {
		$result = '';
		if (isset($this->tplRef->_jTE_entities[$objectName]) && is_object($this->tplRef->_jTE_entities[$objectName])) {
			if (is_callable(array($this->tplRef->_jTE_entities[$objectName], $objectCall))) {
				$args = explode(',',substr(substr($objectArgs,0,-1), strpos($objectArgs,'(')+1));
				$tempObj = new jTE_functions();
				$tempObj->setParent($this->tplRef);
				foreach ($args as $k => $arg) {
					$args[$k] = $tempObj->getValue($arg);
				}
				$result = call_user_func_array(array($this->tplRef->_jTE_entities[$objectName], $objectCall), $args);
			} else {
				if (isset($this->tplRef->_jTE_entities[$objectName]->$objectCall)) {
					$result = call_user_func(array($this->tplRef->_jTE_entities[$objectName], $objectCall));
				}
			}
		}
		return $result;
	}


	/** handleFunction
	 *	This function handles a function request by creating a temp object of $function type and applying it.
	 *	Functions can be in the form of just about anything so long as they follow the following standard;
	 *  functionname(your list of args,or whatever you want)
	 *  jTE will attempt to load the function from the directory and apply it
	 */
	public function handleFunction($function, $applyAgainst, $pipe = null) {
		$openparen_pos = strpos($function, '(');
		if (!empty($function) && ($openparen_pos !== false || $pipe)) {
			$functionName = ($pos = $openparen_pos) ? substr($function, 0, $pos) : $function;
			$argstr = substr(substr($function,0,-1), $openparen_pos + 1);
			$args = array();
			$instr = false;
			$arg = '';
			$tLength = strlen($argstr);// Pull this out so the function doesn't get called for every iteration
			for ($t = 0;  $t < $tLength; $t++)  {
				switch($argstr[$t])  {
					case '\\':
						$t++;
						break; 
					case "'":
						if ($instr == "'")  {
							$instr = false;
						} elseif (!$instr)  {
							$instr = "'";
						}
						$arg .= $argstr[$t];
						break;
					case '"':
						if ($instr == '"')  {
							$instr = false;
						} elseif (!$instr)  {
							$instr = '"';
						}
						$arg .= $argstr[$t];
						break;
					case ',':
						if (!$instr)  {
							$args[] = $arg;
							$arg = '';
						}  else {
							$arg .= $argstr[$t];
						}
						break;
					default:
						$arg .= $argstr[$t];
				}				
			}
			$args[] = $arg;
			//$args = explode(',',substr(substr($function,0,-1), strpos($function,'(')+1));
			if ($this->requireFunction($functionName)
				|| $this->requireFunction($functionName = 'callout')) {
				$className = 'jTE_' . $functionName;
				$tempObj = new $className();
				$tempObj->setParent($this->tplRef);
				$tempObj->setArgs($args);
				$tempObj->setSource($applyAgainst);
				return $tempObj->apply();
			}
		}
		return $applyAgainst;
	}

	/** requireFunction
	 * 	Go and get the function else false
	 */
	protected function requireFunction($functionName) {
		global $jTE_loaded;
		if (isset($jTE_loaded[$functionName])) {
			return true;
		} else if (file_exists($filepath = BF_ROOT . '/tools/jTE/functions/function.'.$functionName.'.php')) {
			include $filepath;
			if (!isset($jTE_loaded)) {
				$jTE_loaded = array($functionName => true);
			} else {
				$jTE_loaded[$functionName] = true;
			}
			return true;
		}
		return false;
	}

	/** parseModifiers
	 *	Returns an array of modifier functions to be called or nothing
	 */
	public function parseModifiers($modifiers) {
		if (strpos($modifiers, '|') !== false) {
			$modifiers = explode('|',$modifiers);
			array_shift($modifiers);
			return $modifiers;
		}
		return false;
	}


	/** createTagsArray($tpl)
	 *	This function does the work of creating the beastly strings/tags array
	 *  Each element in the array will either be a string or a tag in an array;
	 *   array format:  TYPE, TAG, info to pass
	 *		TYPE ~ There are three types;
	 *		function -	a function
	 *		object -	an object
	 *		closer -	a closer {/if} for easy parsing
	 *	At this point we don't do ANY kind of work against the functions and methods
	 * but rather are just creating an array of the template for easy parsing
	 * this allows each function later to then figure out how it wants to further parse
	 * the array if needed.  This can be good and bad, it adds a level of customizability
	 * and at the same time complexity.
	 *	As ugly as this is.. it works nicely
	 */
	protected function createTagsArray($tpl) {
		$results = array();

		$indexer = 0;			// Where in the parsed array are we?
		$tagOpen = false;		// Are we inside a tag {}
		$haveTag = false;		// Do we know what our tag is yet? (do we have the full tag type yet)
		$tagContent = '';		// What is our tag content
		$tagType = '';			// This is the type of tag we have - built per char to first space
		$tLength = strlen($tpl);// Pull this out so the function doesn't get called for every iteration
		for ($t = 0; $t < $tLength; $t++) {
			$result = '';
			if (!isset($results[$indexer])) {
				$results[$indexer] = '';
			}
			if ($tagOpen == false) {
				// OUTSIDE TAGS
				switch ($tpl[$t]) {
					case '{' :
						$tagOpen = true;
						$indexer++;
						break;
					default :
						$results[$indexer] .= $tpl[$t];
						break;
				}
			} else {
				// INSIDE TAGS
				switch ($tpl[$t]) {
					case '}' :
						// We should now have our entire SINGLE tag - define what is its for the array
						if ($tagType[0] == '*') {
							// Do nothing this is a comment
						} else if ($tagType[0] == '$') {
							// Tag is a variable, echo it
							$result = array ('function'=>'echo', 'args'=>$tagType.$tagContent);
						} else {
							if ($tagType[0] == '/') {
								$result = array ('closer'=>substr($tagType,1));
							} else if (strpos($tagType, '>') !== false) {
								$result = array ('object'=>$tagType, 'args'=>$tagContent);
							} else {
								$result = array ('function'=>$tagType, 'args'=>$tagContent);
							}
						}
						$results[$indexer++] = $result;
						$tp = $t + 1;
						if ($tp != $tLength) {
							if ($tpl[$tp] == "\r") {
								$t++;
							}
							if ($tpl[$tp] == "\n") {
								$t++;
							}
						}

						// Clear the tag for the next one
						$tagOpen = false;
						$tagContent = '';
						$tagType = '';
						$haveTag = false;
						break;
					case ' ' :
//					case "\r" :
//					case "\n" :
					case '|' :
					case '(' :
						if ($haveTag == false) {
							$haveTag = true;
						}
						$tagContent .= $tpl[$t];
						break;
					default :
						if ($haveTag == false) {
							$tagType .= $tpl[$t];
						} else {
							$tagContent .= $tpl[$t];
						}
						break;
				}
			}
		}
		return $results;
	}


	/** loadTemplate
	 *	This function does the job of loading the raw .tpl file
	 */
	protected function loadTemplate($template) {
		if ($template != '' && (file_exists($template) || file_exists($template = $this->tplRef->template_dir . $template))) {
			if (!class_exists('jTE_functions')) {
				include 'jTE/core/core.functions.php';
			}
			$cache_key = md5($template);

			if ($this->hasCache($template, $cache_key)) {
				$limbo = $this->getCache($cache_key);
			} else {
				$raw = file_get_contents($template);
				if (!$raw) {
					return false;
				}
				// Do the preparse work
				$parsed = $this->preparse($raw);
				unset($raw);
				// Do the work of parsing
				try {
					$limbo = $this->parse($parsed);
				} catch (Exception $e) {
					unset($parsed);
					return array(translate('2147', $template, $e->getMessage()));
				}
				unset($parsed);
				// Cache it!
				$this->setCache($template, $limbo, $cache_key);
			}
			return $limbo;
		} else {
			throw new BFException(jTE_TPL_LOAD_ERROR . " :: " . $template);
		}
	}

	protected function hasCache($template, $cache_key) {
		$cached = $this->tplRef->compile_dir . $cache_key;
		return file_exists($cached) && filemtime($template) < filemtime($cached);
	}

	protected function getCache($cache_key) {
		if ($cached = $GLOBALS['controller']->getPCache($cache_key)) {
			return $cached;
		} else {
			$cached = $this->tplRef->compile_dir . $cache_key;
			return unserialize(file_get_contents($cached));
		}
	}

	protected function setCache($template, $limbo, $cache_key) {
		$GLOBALS['controller']->delPCache($cache_key);
		$GLOBALS['controller']->setPCache($cache_key, $limbo);
		$cached = $this->tplRef->compile_dir . $cache_key;
		$contents = serialize($limbo);
		if (file_put_contents($cached, $contents)) {
			@chmod($cached, 0666);
			return true;
		} else {
			return false;
		}
	}

	public function __construct($parent) {
		$this->tplRef = $parent;
	}

}

?>
