<?php

/**	jTE function: bf_env_treeview
 *
 *  Usage:
 *  {bf_env_treeview(EnvID)}
 *
 * @package jTE
 */
class jTE_bf_env_treeview extends jTE_functions {

	var $parentStack = array();		// Parent stack watches for an env .include loop
	var $pipeStack = array();		// Pipe stack watches for a |$foo| loop

	public function apply() {
		$GLOBALS['controller']->require_file('modules/BFQueryBuilder/BFQueryBuilder.php');
		$EID = $this->getValue(trim($this->function_args[0],"()")); // ENV ID
		$parent = (isset($this->function_args[1]))?$this->getValue(trim($this->function_args[1],"()")):''; // parent ENV ID
		$ID = (isset($this->function_args[2]))?$this->getValue(trim($this->function_args[2],"()")):''; // Project ID on restart
		$BID = (isset($this->function_args[3]))?$this->getValue(trim($this->function_args[3],"()")):''; // Project Build on restart
		$genXML = (isset($this->function_args[4]))?$this->function_args[4]:false; // Do we generate XML or HTML?
		$envTree = array();

		if ($parent == 'restart') {
			$envTree = $this->getRestartEnvTree($ID, $BID);			
		}  elseif ($parent == 'schedule') {
			$envTree = $this->getScheduleEnvTree($ID);			
		} else {
			$envTree = $this->getEnvTree($EID);
		}

		if ($genXML)  {
			$results = $this->generateXML($envTree);
		}  else {
			$results = $this->generateHTML($envTree);
		}
		if (!$results && !$genXML)  {
			$results = translate('2045');
		}
		return $results;
	}

	// Get the env block from the database..
	/** Get Environment Tree
	 * This function recursively gets the environment block from the database
	 * first checking to make sure the access is correct against the parent.
	 * BFLevel for children items is always zero for some reason.. 
	 * Say it with me.. Overloading database fields = bad.
	 * It ends up with a complete environment tree which gets passed to the processing functions.
	 */	 
	public function getEnvTree($EID) {
		global $user;
		if (!$EID)  {
			return array();
		}
		
		$this->parentStack[$EID] = $EID;

		$uid = (int)$user->get('bf_userid');

		$qb = BFQueryBuilder::select();	
		if ($uid !== 1) {
			$qb->addColumn('bf_bid',		'bf_env', 'bid');
			$qb->where(array(
				array('=', 'bf_env', 'bf_bid', $EID),
				array('IN', 'bf_env', 'bf_level', array_keys(bf_get_levels($uid)))
			));			
			$validEID = $qb->getOne();						
		} else {
			$validEID = $EID;
		}	

		if (!$validEID)  {
			return array();
		}
		
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_bid',		'bf_env', 'bid');
		$qb->addColumn('bf_param',		'bf_env', 'param');
		$qb->addColumn('bf_value',		'bf_env', 'value');
		$qb->addColumn('bf_mode',		'bf_env', 'mode');
		$qb->addColumn('bf_eid',		'bf_env', 'eid');
		$qb->addColumn('bf_level',		'bf_env', 'level');
		$qb->addColumn('bf_action',		'bf_env', 'action');

		$qb->where(array(
			array('=', 'bf_env', 'bf_bid', $validEID),
			array('>', 'bf_env', 'bf_eid', '0')
		));
		

		$qb->orderBy('ASC', 'bf_eid');
		$res = $qb->getResult();

		if (DB::isError($res)) {
			throw new BFException($res->getMessage());
		}
		$tree = array();
		$counter = 0;
		while ($env = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$uniqid = substr(substr(uniqid(mktime(),4),16),0,7) . $counter++;
			$tree[$uniqid] = $env;
		}		
		foreach ($tree as $uniqid => $env)  {
			if ($env['param'] == '.include')  {
				$subEID = bf_get_env_by_name($env['value']);
				if (isset($this->parentStack[$subEID]))  {
					$tree[$uniqid]['subtree'] = 'recursion';
				}  else {
					$tree[$uniqid]['subtree'] = $this->getEnvTree($subEID);
				}
			}
		}
		unset($this->parentStack[$EID]);
		return $tree;
	}

	// This will allow me to get the EID of a restarted ENV item -- kinda sucks but its what I can od.
	public function getEnvEid($key, $value, $bid) {
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_eid', 'bf_env');
		$qb->where(array(
			array('=', 'bf_env', 'bf_param', $key),
			array('=', 'bf_env', 'bf_value', $value),
			array('=', 'bf_env', 'bf_bid', $bid)
		));
		return $qb->getOne();
	}

	// Restarts rebuild the ENV block from the results table - its a bastardization at best.
	public function getRestartEnvTree($ID, $BID) {
		// Get our Projects Tag Keys
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_name', 'bf_tags');
		$qb->where(array('=', 'bf_tags', 'bf_pid', $ID));
		$tagKeys = $qb->getCol();

		// Get EnvID
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_env', 'bf_builds');
		$qb->where(array('=', 'bf_builds', 'bf_bid', $BID));

		// Get EnvStorage
		$storage = new BFStore();
		$env = $storage->getStorage($qb->getOne());

		// Build the Env Tree
		$envs = preg_split("|\n|", $env['TEXT']);

		// Get projects eid
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_eid', 'bf_projects');
		$qb->where(array('=', 'bf_projects', 'bf_pid', $ID));
		$projectEID = $qb->getOne();

		// Define working ENV
		$workingEnv = array();
		$fakeEID = 0;
		foreach($envs as $v) {
			if ($v)  {
				list($action, $key, $value) = split(":", $v, 3);
	
				// Attempt to get the EnvEid from the env table. This is a bad way to do this, but since we don't
				//  maintain the data in any way to get this information this is the best I can do.
				$tempEID = $this->getEnvEid($key, $value, $projectEID);
				if (empty($tempEID)) { $tempEID = ++$fakeEID; }
				
				if ($action != 'Y' && $action != 'X' && $action != 'Z' && $action != 'Q') {
					if (!empty($key) || !empty($action)) {
					$workingEnv[] = array(
						'action' 	=> $action,
						'param' 	=> $key,
						'value'		=> $value,
						'selected' 	=> $value,
						'mode' 		=> $this->getEnvMode($key, $projectEID),
						'eid'		=> $tempEID, 
						'bid' 		=> $projectEID, // Project EID is the BID of the ENV and needs to be here
						'hide'		=> (preg_match("|^bf_|i", $key) || in_array($key, $tagKeys))?true:false
						);
					}
				}
				if ($action == 'X') {
					$step = $value;
				}
			}
		}

		return $workingEnv;

// This will eventually be the new way to do this.. so don't delete it.

//		// Get the Environment from the new bf_env_builds table and create the working env array
//		$workingEnv = array();
//		$qb = BFQueryBuilder::select();
//		$qb->addColumn('bf_buildid', 'bf_env_builds');
//		$qb->addColumn('bf_blockid', 'bf_env_builds');
//		$qb->addColumn('bf_entryid', 'bf_env_builds');
//		$qb->addColumn('bf_levelid', 'bf_env_builds');
//		$qb->addColumn('bf_param', 'bf_env_builds');
//		$qb->addColumn('bf_value', 'bf_env_builds');
//		$qb->addColumn('bf_action', 'bf_env_builds');
//		$qb->addColumn('bf_mode', 'bf_env_builds');
//		$qb->where(array('=', 'bf_env_builds', 'bf_buildid', $BID));
//		$qb->orderBy('ASC', 'bf_entryid');
//		$res = $qb->getResult();
//		
//		if (DB::isError($res)) {
//			throw new BFException($res->getMessage());
//		}
//		
//		$workingEnv = array();
//		$environmentID = '';
//		while ($env = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
//			$workingEnv[] = array(
//				'action' 	=> $env['bf_action'],
//				'param' 	=> $env['bf_param'],
//				'value' 	=> $env['bf_value'],
//				'mode' 		=> $env['bf_mode'],
//				'bid'		=> $env['bf_blockid'], 
//				'eid' 		=> $env['bf_entryid']
//				);
//		}
//		
//		/** Get Tag Keys
//		 * Pull the projects tag keys so we know not to think they are environment variables
//		 */		 		 		
//		$qb = BFQueryBuilder::select();
//		$qb->addColumn('bf_name', 'bf_tags');
//		$qb->where(array('=', 'bf_tags', 'bf_pid', $ID));
//		$tagKeys = $qb->getCol();
//
//		/** Get Environment ID
//		 * Get the environment build storage id from the builds table.		
//		 */		 		 		 		
//		$qb = BFQueryBuilder::select();
//		$qb->addColumn('bf_env', 'bf_builds');
//		$qb->where(array('=', 'bf_builds', 'bf_bid', $BID));
//		$environmentID = $qb->getOne();		
//
//		/** Get EnvStorage
//		 * Pull from storage the actual environment block so we know what to 
//		 * set as our defaults compared to the "real" environment block
//		 * And split it into an array		 
//		 */
//		$storage = new BFStore();
//		$oldEnv = $storage->getStorage($environmentID);
//		$oldEnvs = preg_split("|\n|", $oldEnv['TEXT']);		 		 		
//
//		/** parseEnvBlock
//		 * Step through the environment block and weed out stuff we don't want
//		 * and try to get the information we do want in some format that is actually
//		 * useful.
//		 */
//		foreach($oldEnvs as $v) {
//			// Split by : only 3 times
//			list($tAction, $tKey, $tValue) = split(":", $v, 3);
//			// No YXZQ actions
//			if ($tAction != 'Y' && $tAction != 'X' && $tAction != 'Z' && $tAction != 'Q') {
//				// No BF_ or Tags
//				if (!preg_match("|^bf_|i", $tKey) and !in_array($tKey, $tagKeys)) {
//					// We now have from the env block our data that we actually want
//					print "$tAction||$tKey||$tValue <br />"; 
//				}				
//			}			
//		}		 		 		 	
//

	}


	// Schedules rebuild the ENV block from the cron table - its also a bastardization at best.
	public function getScheduleEnvTree($CID) {

		// Get environment
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_env', 'bf_cron');
		$qb->where(array('=', 'bf_cron', 'bf_cid', $CID));

		// Get EnvStorage
		$storage = new BFStore();
		$env = $storage->getStorage($qb->getOne());

		// Build the Env Tree
		$envs = preg_split("|\n|", $env['TEXT']);

		// Get cron's eid
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_eid', 'bf_cron');
		$qb->where(array('=', 'bf_cron', 'bf_cid', $CID));
		$EID = $qb->getOne();

		// Define working ENV
		$workingEnv = array();
		$fakeEID = 0; // Don't need to know the real EID, I just need the fields to be unique.
		$fakeBID = 0;
		$i = 0;
		foreach($envs as $v) {
			if (!$v || substr_count($v, ':') < 2)  {
				continue;
			}

			list($action, $key, $value) = split(":", $v, 3);

			if ($action != 'Y' && $action != 'X' && $action != 'Z' && $action != 'Q') {
				if (!preg_match("|^bf_|i", $key)) {
					$workingEnv[$i] = array(
						'action' 	=> $action,
						'param' 	=> $key,
						'value'		=> $value,
						'selected' 	=> $value,
						'mode' 		=> $this->getEnvMode($key, $EID),
						'bid'		=> ++$fakeBID,
						'eid' 		=> ++$fakeEID,
						);
					if ($key == '.include')  {
						$workingEnv[$i]['subtree'] = $this->getEnvTree(bf_get_env_by_name($value));
					}
					$i++;
				}
			}
		}
		return $workingEnv;

// This will eventually be the new way to do this.. so don't delete it.

//		// Get the Environment from the new bf_env_builds table and create the working env array
//		$workingEnv = array();
//		$qb = BFQueryBuilder::select();
//		$qb->addColumn('bf_buildid', 'bf_env_builds');
//		$qb->addColumn('bf_blockid', 'bf_env_builds');
//		$qb->addColumn('bf_entryid', 'bf_env_builds');
//		$qb->addColumn('bf_levelid', 'bf_env_builds');
//		$qb->addColumn('bf_param', 'bf_env_builds');
//		$qb->addColumn('bf_value', 'bf_env_builds');
//		$qb->addColumn('bf_action', 'bf_env_builds');
//		$qb->addColumn('bf_mode', 'bf_env_builds');
//		$qb->where(array('=', 'bf_env_builds', 'bf_buildid', $BID));
//		$qb->orderBy('ASC', 'bf_entryid');
//		$res = $qb->getResult();
//		
//		if (DB::isError($res)) {
//			throw new BFException($res->getMessage());
//		}
//		
//		$workingEnv = array();
//		$environmentID = '';
//		while ($env = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
//			$workingEnv[] = array(
//				'action' 	=> $env['bf_action'],
//				'param' 	=> $env['bf_param'],
//				'value' 	=> $env['bf_value'],
//				'mode' 		=> $env['bf_mode'],
//				'bid'		=> $env['bf_blockid'], 
//				'eid' 		=> $env['bf_entryid']
//				);
//		}
//
//		/** Get Environment ID
//		 * Get the environment build storage id from the builds table.		
//		 */		 		 		 		
//		$qb = BFQueryBuilder::select();
//		$qb->addColumn('bf_env', 'bf_cron');
//		$qb->where(array('=', 'bf_cron', 'bf_cid', $CID));
//		$environmentID = $qb->getOne();		
//
//		/** Get EnvStorage
//		 * Pull from storage the actual environment block so we know what to 
//		 * set as our defaults compared to the "real" environment block
//		 * And split it into an array		 
//		 */
//		$storage = new BFStore();
//		$oldEnv = $storage->getStorage($environmentID);
//		$oldEnvs = preg_split("|\n|", $oldEnv['TEXT']);		 		 		
//
//		/** parseEnvBlock
//		 * Step through the environment block and weed out stuff we don't want
//		 * and try to get the information we do want in some format that is actually
//		 * useful.
//		 */
//		foreach($oldEnvs as $v) {
//			// Split by : only 3 times
//			list($tAction, $tKey, $tValue) = split(":", $v, 3);
//			// No YXZQ actions
//			if ($tAction != 'Y' && $tAction != 'X' && $tAction != 'Z' && $tAction != 'Q') {
//				// No BF_ or Tags
//				if (!preg_match("|^bf_|i", $tKey) and !in_array($tKey, $tagKeys)) {
//					// We now have from the env block our data that we actually want
//					print "$tAction||$tKey||$tValue <br />"; 
//				}				
//			}			
//		}		 		 		 	
//

}
	
	// Only way to get the mode, is to grab the current mode from the env block
	// which is bs and shouldn't be the way we do this.. but so be it.
	private function getEnvMode($param, $eid) {
		if ($param == '' or empty($eid)) {
			return 'N';
		}
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_mode', 'bf_env');
		$qb->where(array(
			array('=', 'bf_env', 'bf_bid', $eid),
			array('!=', 'bf_env', 'bf_eid', '0'),
			array('=', 'bf_env', 'bf_param', $param)
		));

		return $qb->getOne();
	}
	
	/** GetEnvValue
	 * This function will test the param and value in the build env block against all
	 * in the project env block to see if we have more information we need to give.
	 * ... again. complete bullpoo, but here ya go.
	 */	 	 	 	 	
	private function getEnvValue($param, $value, $eid) {
		if ($param == '' or empty($eid)) {
		return $value;
		}
		$qb = BFQueryBuilder::select();
		$qb->addColumn('bf_value', 'bf_env');
		$qb->where(array(
			array('=', 'bf_env', 'bf_bid', $eid),
			array('=', 'bf_env', 'bf_param', $param),
		));
		$newValue = $qb->getOne();
		
		return ($newValue == '')?$value:$newValue;		
	}
	

	public function generateHTML($envTree) {
		$results = "";

		$webroot = $this->getValue('$webroot');
		$counter = 0;

		foreach($envTree as $key=>$val) {

			// Create Unique Name for form elements
			$UID = uniqid() . $counter++;
			$uniqueName = $UID.'_'.$val['bid'].'_'.$val['eid'].'_'.$val['param'];

			// Is this a env key?
			if ($val['eid'] == '0') {
				// Do nothing - though I thought there was something to do here...
			}
			// Not a env key
			else {

				// Is this an include?
				if ($val['param'] == '.include') {
					$childEID = bf_get_env_by_name($val['value']);
					$includeResult = "";
					if (isset($val['subtree']))  {
						if ($val['subtree'] == 'recursion')  {
							$includeResult = '<div class="Step_Solo"><div class="StepData"> </div>' .
									'	<div class="StepData"> </div> [ ' . translate('2143') . ' ]</div>';
						}  else {
							$includeResult = $this->generateHTML($val['subtree']);
						}
					}
					
					if (!empty($includeResult)) {					
						$results .= '<div id="'.$UID.'_Wrapper" class="Env_Wrapper_Collapsed"'.(($val['mode']=='S')?' style="display:none;"':'').'>';
						$results .= '	<label><strong>' . hexentities(substr($val['value'],0,35)) . ':</strong></label><br />';
						$results .= '	<div id="s_'.$UID.'_Children" style="margin-left:20px;clear:left;">';

						$results .= $includeResult;

						$results .= '	</div>';
						$results .= '</div>';
					}

				}

				// This isn't an include - render it.
				else {

					$results .= '<div class="pad" '.(($val['mode'] == 'S' || (isset($val['hide']) && $val['hide'] === true))?' style="display:none;"':'').'>';
					if ($val['mode'] == 'C') {
						$results .= '<label title="' . hexentities(translate('must change')) . '">';
					} else if ($val['mode'] == 'R') {
						$results .= '<label title="' . hexentities(translate('required')) . '">';
					} else {
						$results .= '<label>';
					}
					$results .= hexentities($val['param']) . '</label>';

					//  A hidden variable is uneditable at this point unless you have the EditHiddenTags
					//  permission.
					if ($val['action'] == 'H' && !$GLOBALS['controller']->userHasAccessTo('EditHiddenTags')) {
						$results .= '<input type="password" style="width:200px;" class="textBox" name="fake['.$uniqueName.']" ' .
								'value="'.hexentities($val['value']).'" disabled="disabled" />';
						$results .= '<input type="hidden" name="environment['.$uniqueName.']" value="' .
								hexentities($val['value']).'" />';
					}  else {

						// .ENV
						if (preg_match("/^\.env/", $val['value'])) {
							$envs = bf_get_envs(0);
							if (is_array($envs)) {
								$results .= '<select type="select" class="textBox" style="width:200px;" name="environment['.$uniqueName.']">';
								foreach($envs as $v) {
									$results .= '<option>'.hexentities($v).'</option>';
								}
								$results .= '</select>';
							}
						}

						// .Selectors or .servers
						else if (preg_match("/^\.se(le|rv)/", $val['value'])) {
							$sels = bf_get_selectors();
							if (is_array($sels)) {
								$results .= '<select type="select" class="textBox" style="width:200px;" name="environment['.$uniqueName.']">';
								foreach($sels as $k=>$v) {
									$results .= '<option>'.hexentities($v).'</option>';
								}
								$results .= '</select>';
							}
						}

						// Everything else
						else {
							// Set our input type
							$inputType = ($val['action'] == 'H')?'password':'text';
							$selectlen = '200';
							// Piped content can only be C or N if it's other, then force it to N
							if (preg_match("/\|/", $val['value'])) {
								if($val['mode'] != 'C' and $val['mode'] != 'N' ) {
									$val['mode'] == 'N';
								}
								//definitely a pipe...
								$selectlen = $this->pipe_width($val, $envTree);
							}

							// Do different things based on the mode
							switch($val['mode']) {
								case 'O':	
									// uneditable - create a hidden with value and a fake field that 
									// id disabled (since disabled can't be submitted)
									$results .= '<input type="'.$inputType.'" style="width:200px;" class="text" name="fake[' .
											$uniqueName.']" value="'.hexentities($val['value']) . '" disabled="disabled" />';
									$results .= '<input type="hidden" name="environment[' . $uniqueName . ']" value="'.hexentities($val['value']).'" />';
									break;

								case 'S': // supress - Add it but don't display it.
									$results .= '<input type="hidden" name="environment[' . $uniqueName . ']" value="'.hexentities($val['value']).'" />';
									break;

								case 'C': // must change - require the data to be changed before allowing submit.
										/*
										 * Fix for issue where IE doesnt allow the border of
										 * a <select> tag to be changed. Wrapped the  <select> 
										 * tag in a div, and put a border on the div to mimic what
										 * FF normally supports.
 										 */
										// Piped Content
										if (preg_match("/\|/", $val['value'])) {
											$results .= '<div class="mustchange">';
											$results .= '<select type="select" class="textBox mustchange" style="width:'. $selectlen . ';" name="environment['.$uniqueName.']">';
											$results .= $this->pipe_options($val, $envTree);
											$results .= '</select>';
											$results .= '</div>';
										} else {
											$results .= '<input type="'.$inputType.'" style="width:200px;" class="text mustchange" id="cur_' .
												$uniqueName.'" name="environment['.$uniqueName.']" value="' . hexentities($val['value']).'" />';
										}
									break;

								case 'R': // Required - must not be blank.
									if (preg_match("/\|/", $val['value'])) {
										$results .= '<select type="select" class="textBox mustchange" style="width:'. $selectlen . ';" name="environment['.$uniqueName.']">';
										$results .= $this->pipe_options($val, $envTree);
										$results .= '</select>';
									} else {
										$results .= '<input type="'.$inputType.'" style="width:200px;" class="text mustchange" id="cur_' .
										$uniqueName.'" name="environment['.$uniqueName.']" value="' . hexentities($val['value']).'" />';
									}
									break;

								default: // Everything else.

										// Piped Content
										if (preg_match("/\|/", $val['value'])) {
											$results .= '<select type="select" class="textBox" style="width:'. $selectlen . ';" name="environment['.$uniqueName.']">';
											$results .= $this->pipe_options($val, $envTree);
											$results .= '</select>';
										} else {
											$results .= '<input type="'.$inputType.'" style="width:200px;" class="text" ' . 
													'name="environment['.$uniqueName.']" value="' .	hexentities($val['value']).'" />';
										}
									break;
							}
						}
					}

					$results .= '</div>';

				}
			}
		}

		return $results;
	}


	public function generateXML($envTree) {
		$results = "";
		$webroot = $this->getValue('$webroot');
		$counter = 0;

		foreach($envTree as $key=>$val) {

			// Create Unique Name for form elements
			$UID = uniqid() . $counter++;
			$uniqueName = $UID.'_'.$val['bid'].'_'.$val['eid'].'_'.$val['action'].'_'.$val['param'];

			// Is this a env key?
			if ($val['eid'] == '0') {
				// Do nothing - though I thought there was something to do here...
			}
			// Not a env key
			else {
				// Is this an include?
				if ($val['param'] == '.include') {
					$childEID = bf_get_env_by_name($val['value']);
					if ($childEID)  {
						$value = '----';
					}  else {
						$value = translate('2142');
					}
					$label = hexentities(substr($val['value'],0,35));
					$results .= '<envvar type="parent" id="' . $UID . '" label="' . $label . '" value="' . 
							$value . '">';
					if (isset($val['subtree']))  {
						$results .= $val['subtree'] == 'recursion'  ? '[ ' . 
								translate('2143') . ' ]' : $this->generateXML($val['subtree']);
					}
					$results .= '</envvar>';
				}  else {
				// This isn't an include - render it.
					$label = hexentities($val['param']);
					$name = $uniqueName;
					if (preg_match("/^\.env/", $val['value'])) {
						$envs = bf_get_envs(0);
						if (is_array($envs)) {
							$value = implode('|', $envs);
						}							
					// .Selectors or .servers
					}  else if (preg_match("/^\.se(le|rv)/", $val['value'])) {
					
						$sels = bf_get_selectors();
						if (is_array($sels)) {
							$value = implode('|', $sels);
						}
					}  else {
						if (preg_match("/\|/", $val['value']))  {
							$value = hexentities($this->pipe_options($val['value'], $envTree, true));
						}  else {
							$value = hexentities($val['value']);
						}
					}
					
					if ($val['action'] == 'H' && !$GLOBALS['controller']->userHasAccessTo('EditHiddenTags'))  {
						$type = 'X';
					}  else  {
						$type = $val['action'];
					}
					$results .= '<envvar type="' . $type . '" mode="' . $val['mode'] . '" id="' . $UID . 
							'" label="' . $label . '" name="' . $name . '" value="' . $value . '"/>';
				}
			}
		}

		return $results;
	}


	function flatten_tree($tree)  {
		$ret = array();
		foreach ($tree as $k=>$v)  {
			if (isset($v['subtree']) && is_array($v['subtree']))  {
				$ret = array_merge($ret, $this->flatten_tree($v['subtree']));
			}  else {
				$ret[$k] = $v;
			}
		}
		return $ret;		
	}
	
	function pipe_width($item, $tree) {
		$maxlen = 0;
		$list = $this->pipe_options($item, $tree, 1);
		$eachItem = preg_split("/\|/", $list);

		foreach($eachItem as $v) {
			$tmplen = strlen($v);
			$maxlen = $tmplen > $maxlen ? $tmplen : $maxlen;
		}

		//My worst case scenario...assuming 20 chars is 20 'M' chars, which is about 200 px.
		$maxlen = ($maxlen > 20) ? 'auto' : '200px';
		
		return $maxlen;
	}
	
	/** Pipe_options list
	 * This allows you to have recursive piped options to get around the fact that we can only have so many
	 * characters per value.  If you want you can say $var1 = "A|B|C|$var2|H|I|J" $var2="D|E|F|G"
	 * $var2 will be correctly parsed out in the drop down list.
	 */
	function pipe_options($item, $tree, $xml = false) {
		GLOBAL $controller;
		$result = '';
		$flag = 0;
		$list = $this->flatten_tree($tree);
		$expand = $controller->getSysconfig('expand_list_vars');
		
		if (isset($this->pipeStack[serialize($item)])) {
			$result = $xml ? translate('2143') : '<option value="">[ ' . translate('2143') . ' ]</option>';
		} else {
			$this->pipeStack[serialize($item)] = true;

			$eachItem = preg_split("/\|/", is_string($item) ? $item : $item['value']);
			foreach($eachItem as $v) {
				// If this is a variable, then return that..
// CERL3: List variables should never expand, because this can cause some complicated nesting issues.
// KWPL3: Unless customer specifically wants this behaviour. enabled via sysconfig.
 				if (strpos($v, "$") === 0 && $expand == 'Y') {
 					$var = substr($v,1);
 					foreach($list as $k) {
 						if ($k['param'] == $var) {
 							$newItem = $k;
 							$result .= ($xml && $result ? '|' : '') .
 									$this->pipe_options($newItem, $list, $xml);
 						}
 					}
 				} else {
					if ($xml)  {
						$result .= $flag ? '|' . $v : $v;
						$flag = 1;
					}  else {
						$result .= '<option value="'.hexentities($v).'"' . 
								((isset($item['selected']) && $item['selected'] == $v)?' selected="selected"':'') .
								'>'.hexentities($v).'</option>';
					}
 				}
			}
		}

		unset($this->pipeStack[serialize($item)]);
		return $result;
	}



}

?>
