<?php

/**	jTE function: IF
 *	This function handles an IF block
 *
 *  Usage:
 *  {if $somevalue=='foo'}Do some stuff{/if}
 *  {if 1==0}false{else}true{/if}
 *  {if !1==0}false{elseif $foo}foo{else}true{/if}
 *  {if $somevar} foo.. {/if}
 *  {if !$somevar} foo.. {/if}
 * 
 * @package jTE
 */
class jTE_if extends jTE_functions {
	protected $mainArray;

	public function apply() {
		$result = '';

		if (isset($this->function_source['block'])) {
			// Need to parse the block
			$block = $this->parseBlock($this->function_source['block']);
			if ($this->doComparison($this->function_args[0])) {
				return $this->tplRef->_jTE_parser->parseElements($block['if']);
			} else {
				// Check any elseifs to see if we should use them before resorting to else
				foreach ($block['elseif'] as $i => $elseif) {
					if ($this->doComparison($block['elseif'][$i]['args'])) {
						// Hacked-out, but really just carrying on the tradition...
						// Can't pass the args or they'll get parsed as output
						unset($block['elseif'][$i]['args']);
						return $this->tplRef->_jTE_parser->parseElements($block['elseif'][$i]);
					}
				}
				return $this->tplRef->_jTE_parser->parseElements($block['else']);
			}
		} else {
			return false;
		}
	}

	// Sort the block out
	// Step through each entry in block and push to either
	// $bIf - an array
	// $bElse - an array
	// $bElseif - a nested array that will be tough to deal with
	protected function parseBlock($block) {
		$bIf = array();
		$bElse = array();
		$bElseIf = array();
		$c = -1; // only used by $bElseIf
		$location = 'bIf';

		foreach($block as $k=>$v) {
			if (isset($v['function']) && $v['function'] == 'else') {
				$location = 'bElse';
				continue;
			}
			if (isset($v['function']) && $v['function'] == 'elseif' &&	isset($v['args'])) {
				$location = 'bElseIf';
				$bElseIf[++$c]['args'] = $v['args'];
				continue;
			}
			if ($location == 'bElseIf') {
				array_push($bElseIf[$c], $v);
			} else {
				array_push($$location, $v);
			}
		}
		return array('if'=>$bIf, 'elseif'=>$bElseIf, 'else'=>$bElse);
	}

	// Overload getValue a bit
	public function getValue($value) {
		$var = parent::getValue($value);
		if ($var == '') {
			$var = '\'\'';
		}
		return $var;
	}
}

/*

$a == $b	Equal	TRUE if $a is equal to $b.
$a === $b	Identical	TRUE if $a is equal to $b, and they are of the same type. (introduced in PHP 4)
$a != $b	Not equal	TRUE if $a is not equal to $b.
$a <> $b	Not equal	TRUE if $a is not equal to $b.
$a !== $b	Not identical	TRUE if $a is not equal to $b, or they are not of the same type. (introduced in PHP 4)
$a < $b	Less than	TRUE if $a is strictly less than $b.
$a > $b	Greater than	TRUE if $a is strictly greater than $b.
$a <= $b	Less than or equal to 	TRUE if $a is less than or equal to $b.
$a >= $b	Greater than or equal to 	TRUE if $a is greater than or equal to $b.
Array
(
    [0] => $jTE['get']['action'] != ''
)
Array
(
    [function] => if
    [args] => $jTE['get']['action'] != ''
    [block] => Array
        (
            [0] =>
            [1] => Array
                (
                    [function] => echo
                    [args] => $jTE['get']['action']|escape('htmlall')|assign('temp')
                )

            [2] => (assign('linkAction', '&amp;action=$temp')}
        )

)

*/


?>
