<?php

/** jointTemplateEngine
 *  by Jason Ellis (jellis@us.ibm.com || palamedes@rocketmail.com)
 *  by Shawn Lauriat
 *
 *  This template engine is designed to mirror all the best features of other engines
 * that we have used in the past, with none of the crap. Functionally, it needs to be
 * as close as possible to past template engines without being past template engines.
 *
 *	This template engine will use a lexer based parser as opposed to a regexp parser.
 *
 * Functional Requirements:
 *	~ {} tags
 *		{function}...{/function}	- for those that require a code block
 *		{function}					- for single action functions
 *	~ PHP like variables; {$foo}, {$foo[1]}, {$foo['bar']}, {$foo->bar}, {$foo->bar()}
 *	~ PHP like argument assignments; {foo('bar','doh')}, {$foo->bar('doh')}
 *  ~ Custom function definition - all functions treated as a plugin
 *		{$foo} is an alias for {echo $foo}
 *	~ Output modifiers - modifiers are just functions
 *		{$foo|modifer('option')}
 *		{$foo->bar()|modifer('option')|otherModifier('opt1','opt2')}
 * 
 * @package jTE
 */

define('jTE_TPL_LOAD_ERROR', translate('2148'));

class jointTemplateEngine {

	/** jTE_variables
	 *	This is the array where all template variables and objects are stored
	 */
	public $_jTE_entities		= array();

	/** jTE_functions
	 *	This is the array where all function objects are stored.
	 */
	public $_jTE_functions		= array();

	/** jTE_raw_file
	 *	This is the TPL in its raw form, no parsing done.
	 */
	protected $_jTE_raw_file		= '';

	/** jTE_parser
	 * This is the parser we are working with to allow the various template functions
	 * to reference the parser directly
	 */
	public $_jTE_parser			= '';

	public $foo = 'foobar';

	/** template_dir
	 *	This is the location of our templates
	 */
	public $template_dir	= '';

	/** compile_dir
	 *	This is the location of our cache
	 */
	public $compile_dir	= '';

	/** setEntity
	 *  This function sets a variable to the template system
	 * $varKey needs to be a string name.
	 * $varValue is a true mixed. String, integer, array, object..etc..
	 */
	public function setEntity($varKey, $varValue = '') {
		if ($varKey != '') {
			$varKey = trim($varKey,"\$()' \t\n\r\0\x0B");
			$this->_jTE_entities[$varKey] = $varValue;
			return true;
		}
		return false;
	}
	
	/**
	 * Get the associated value for a given key
	 * in the template system
	 */
	public function getEntity($varKey) {
		if (isset($this->_jTE_entities[$varKey])) {
			return $this->_jTE_entities[$varKey];
		} else {
			return false;
		}
	}

	/** unsetEntity
	 *	This function unsets a variable from the template system
	 * $varkey needs to be a string name.
	 */
	public function unsetEntity($varKey) {
		if ($varKey != '') {
			unset($this->_jTE_entities[$varKey]);
			return true;
		}
		return false;
	}

	/** setFunction
	 *  This function sets a function to the template system
	 *  ALL functions will be set this way
	 * $objKey needs to be the object/function name
	 * $objValue needs to be the function object itself
	 */
	public function setFunction($fakeFunc, $realFunc) {
		$this->_jTE_functions[$fakeFunc] = $realFunc;
		return true;
	}

	/** unsetFunction
	 *  This function unsets a function from the template system
	 * $objKey needs to be a function name.
	 */
	public function unsetFunction($objKey) {
		if ($objKey != '') {
			unset($this->_jTE_functions[$objKey]);
			return true;
		}
		return false;
	}

	### Begin: Aliases ###

	/** assign
	 * 	This function is an alias to setEntity
	 */
	public function assign($varKey, $varValue = '') {
		if ($varKey != '') {
			return $this->setEntity($varKey, $varValue);
		}
		return false;
	}

	/** register_object
	 *	This function is an alias to setEntity
	 */
	public function register_object($objKey, $objValue) {
		if ($objKey != '' && is_object($objValue)) {
			return $this->setEntity($objKey, $objValue);
		}
		return false;
	}

	/** register_function
	 *  This function is an alias to setFunction
	 * NjTE: This isn't a direct translation from previous system.
	 */
	public function register_function($realFunc, $fakeFunc) {
		if (!empty($realFunc) && !empty($fakeFunc) && function_exists($realFunc)) {
			return $this->setFunction($fakeFunc, $realFunc);
		}
		return false;
	}

	/**
	 * DEADLY SIN
	 * This will suck up memory faster than Joe can go through
	 * a bottle of scotch.
	 */
	public function fetch($template) {
		if ($template != '' && file_exists($template)) {
			ob_start();
			$this->display($template);
			return ob_get_clean();
		}
		return false;
	}
	### End: Aliases ###

	/** render
	 *	This function does all the hard work
	 * $template is the location of the template to load and use
	 */
	public function render($template) {
		if (!class_exists('jTE_parser')) {
			require_once('core/core.template_parser.php');
		}
		$this->_jTE_parser = new jTE_parser($this);
		$this->_jTE_parser->render($template);
	}

	/** display
	 * Echoes the results of render()
	 */
	public function display($template) {
		$this->render($template);
		$this->_jTE_parser->display();
	}

	protected function loadRequest() {
		global $_GET, $_POST, $_SERVER;
		$this->_jTE_entities['jTE']['now'] = time();
		$this->_jTE_entities['jTE']['get'] = $_GET;
		$this->_jTE_entities['jTE']['post'] = $_POST;
		$this->_jTE_entities['jTE']['server'] = $_SERVER;
	}

	### Constructor ###

	public function __construct() {
		$this->loadRequest();
		if (!class_exists('jTEPHP')) {
			require 'core/core.php.php';
		}
		// Load our functions
		// Load our modifiers
	}

}



?>
